Package: quantreg
Title: Quantile Regression
Description: Estimation and inference methods for models for conditional quantile functions: 
  Linear and nonlinear parametric and non-parametric (total variation penalized) models 
  for conditional quantiles of a univariate response and several methods for handling
  censored survival data.  Portfolio selection methods based on expected shortfall
  risk are also now included. See Koenker, R. (2005) Quantile Regression, Cambridge U. Press,
  <doi:10.1017/CBO9780511754098> and Koenker, R. et al. (2017) Handbook of Quantile Regression, 
  CRC Press, <doi:10.1201/9781315120256>. 
Version: 6.1
Authors@R: c(
    person("Roger", "Koenker",  role = c("cre","aut"), email =  "rkoenker@illinois.edu"),
    person("Stephen", "Portnoy",  role = c("ctb"), 
	comment = "Contributions to Censored QR code", email =  "sportnoy@illinois.edu"),
    person(c("Pin", "Tian"), "Ng",  role = c("ctb"), 
	comment = "Contributions to Sparse QR code", email =  "pin.ng@nau.edu"),
    person("Blaise", "Melly",  role = c("ctb"), 
	comment = "Contributions to preprocessing code", email =  "mellyblaise@gmail.com"),
    person("Achim", "Zeileis",  role = c("ctb"), 
	comment = "Contributions to dynrq code essentially identical to his dynlm code", 
	email =  "Achim.Zeileis@uibk.ac.at"),
    person("Philip", "Grosjean",  role = c("ctb"), 
	comment = "Contributions to nlrq code", email =  "phgrosjean@sciviews.org"),
    person("Cleve", "Moler",  role = c("ctb"), 
	comment = "author of several linpack routines"),
    person("Yousef", "Saad",  role = c("ctb"), 
	comment = "author of sparskit2"),
    person("Victor", "Chernozhukov",  role = c("ctb"), 
	comment = "contributions to extreme value inference code"),
    person("Ivan", "Fernandez-Val",  role = c("ctb"), 
	comment = "contributions to extreme value inference code"),
    person("Martin", "Maechler", role = "ctb", comment = c("tweaks (src/chlfct.f, 'tiny','Large')",
				ORCID = "0000-0002-8685-9910")),
    person(c("Brian", "D"), "Ripley",  role = c("trl","ctb"), 
	comment = "Initial (2001) R port from S (to my everlasting shame -- 
	how could I have been so slow to adopt R!) and for numerous other 
	suggestions and useful advice", email =  "ripley@stats.ox.ac.uk"))
Maintainer: Roger Koenker <rkoenker@illinois.edu>
Repository: CRAN
Depends: R (>= 3.5), stats, SparseM
Imports: methods, graphics, Matrix, MatrixModels, survival, MASS
Suggests: interp, rgl, logspline, nor1mix, Formula, zoo, R.rsp, conquer
License: GPL (>= 2)
URL: https://www.r-project.org
NeedsCompilation: yes
VignetteBuilder: R.rsp
Packaged: 2025-02-28 17:44:44 UTC; roger
Author: Roger Koenker [cre, aut],
  Stephen Portnoy [ctb] (Contributions to Censored QR code),
  Pin Tian Ng [ctb] (Contributions to Sparse QR code),
  Blaise Melly [ctb] (Contributions to preprocessing code),
  Achim Zeileis [ctb] (Contributions to dynrq code essentially identical
    to his dynlm code),
  Philip Grosjean [ctb] (Contributions to nlrq code),
  Cleve Moler [ctb] (author of several linpack routines),
  Yousef Saad [ctb] (author of sparskit2),
  Victor Chernozhukov [ctb] (contributions to extreme value inference
    code),
  Ivan Fernandez-Val [ctb] (contributions to extreme value inference
    code),
  Martin Maechler [ctb] (tweaks (src/chlfct.f, 'tiny','Large'),
    <https://orcid.org/0000-0002-8685-9910>),
  Brian D Ripley [trl, ctb] (Initial (2001) R port from S (to my
    everlasting shame -- how could I have been so slow to adopt R!) and
    for numerous other suggestions and useful advice)
Date/Publication: 2025-03-10 08:50:09 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 21:16:00 UTC; unix
Archs: quantreg.so.dSYM

useDynLib("quantreg", .registration = TRUE)
import(Matrix)
importFrom('SparseM', t, chol, backsolve, as.matrix.csr)
importFrom('MatrixModels', model.Matrix)
importFrom('stats', approxfun, contrasts)
importFrom("grDevices", "dev.new")
importClassesFrom('MatrixModels', dsparseModelMatrix)
importClassesFrom('SparseM', matrix.csr)

# Export 
export( "AIC.nlrq", "AIC.rq", "AIC.rqs", "AIC.rqss", "akj", "anova.rq", "anova.rqlist", "anova.rqs", "bandwidth.rq", "boot.crq", "boot.rq", "boot.rq.mcmb", "boot.rq.pwy", "boot.rq.spwy", "boot.rq.wxy", "boot.rq.xy", "boot.rq.pxy","rq.fit.ppro","ChangeLog", "coef.crq", "coef.nlrq", "combos", "critval", "crq", "crq.fit.pen", "crq.fit.por", "crq.fit.por2", "crq.fit.pow", "Curv", "deviance.nlrq", "dither", "dynrq", "end.dynrq", "extractAIC.nlrq", "extractAIC.rq", "FAQ", "fitted.nlrq", "fitted.rqss", "formula.nlrq", "formula.rq", "index.dynrq", "KhmaladzeTest",  "kselect", "kuantile", "kunique", "latex", "latex.summary.rqs", "latex.table", "latex.table.rq", "lm.fit.recursive", "logLik.nlrq", "logLik.rq", "logLik.rqs", "logLik.rqss", "lprq", "nlrq", "nlrq.control", "nlrqModel", "plot.qss1", "plot.qss2", "plot.rq.process", "plot.rqs", "plot.rqss", "plot.summary.crqs", "plot.summary.rqs", "plot.summary.rqss", "plot.table.rq", "predict.crq", "predict.crqs", "predict.nlrq", "predict.qss1", "predict.qss2", "predict.rq", "predict.rq.process", "predict.rqs", "predict.rqss", "print.anova.rq", "print.crq", "print.dynrq", "print.dynrqs", "print.nlrq", "print.rq", "print.rqs", "print.rqss", "print.summary.crq", "print.summary.crqs", "print.summary.dynrq", "print.summary.dynrqs", "print.summary.nlrq", "print.summary.rq", "print.summary.rqs", "print.summary.rqss", "qrisk", "qss", "qts1", "qss1", "qss2", "QTECox", "ranks", "rearrange", "resid.rqss", "residuals.nlrq", "rq", "rq.fit", "rq.fit.br", "rq.fit.fnb", "rq.fit.fnc", "rq.fit.hogg", "rq.fit.lasso", "rq.fit.pfn", "rq.fit.scad", "rq.fit.sfn", "rq.fit.sfnc", "rq.fit.conquer","rq.test.anowar", "rq.test.rank", "rq.wfit", "rqProcess", "rqs.fit", "rqss", "rqss.fit", "sfn.control", "sfnMessage", "srisk", "start.dynrq", "summary.crq", "summary.crqs", "summary.dynrqs", "summary.nlrq", "summary.rq", "summary.rqs", "summary.rqss", "table.rq", "tau.nlrq", "triogram.fidelity", "triogram.penalty", "untangle.specials", "ParetoTest", "Hill", "Hill.fit", "print.Hill", "summary.Hill", "print.summary.Hill", "Pickands.fit", "Pickands", "print.Pickands", "summary.Pickands", "print.summary.Pickands", "Pickands.fit", "Munge", "rq.fit.qfnb", "rq.fit.pfnb", "boot.rq.pwxy", "q489", "LassoLambdaHat")


importFrom("grDevices", "dev.interactive", "devAskNewPage", "gray",
     "n2mfrow", "terrain.colors")
importFrom("graphics", "abline", "contour", "lines", "matlines",
     "matplot", "par", "persp", "plot", "points", "polygon", "title")
importFrom("MASS", "Null")
importFrom("survival", "Surv")
importFrom("methods", "as", "hasArg", "new")
importFrom("utils", "flush.console", "modifyList")
importFrom("stats", ".checkMFClasses", ".getXlevels", "AIC", "approx",
     "as.formula", "as.ts", "coef", "coefficients", "cycle", 
     "model.matrix", "model.response", "rmultinom",
     "delete.response", "dnorm", "end", "fitted", "formula",
     "frequency", "getInitial", "hat", "is.empty.model",
     "is.stepfun", "knots", "lag", "lm", "lm.fit", "lm.wfit",
     "logLik", "lsfit", "median", "model.extract", "model.frame",
     "model.weights", "na.omit", "na.pass", "numericDeriv",
     "optim", "pchisq", "pf", "pnorm", "predict", "printCoefmat",
     "pt", "qnorm", "qt", "quantile", "reformulate", "relevel",
     "resid", "residuals", "rexp", "runif", "setNames", "start",
     "stepfun", "supsmu", "terms", "uniroot", "var", "napredict")

S3method(AIC, nlrq)
S3method(AIC, rq)
S3method(AIC, rqs)
S3method(AIC, rqss)
S3method(anova, rq)
S3method(anova, rqlist)
S3method(anova, rqs)
S3method(coef, crq)
S3method(coef, nlrq)
S3method(deviance, nlrq)
S3method(extractAIC, nlrq)
S3method(extractAIC, rq)
S3method(fitted, nlrq)
S3method(fitted, rqss)
S3method(formula, nlrq)
S3method(formula, rq)
S3method(latex, summary.rqs)
S3method(latex, table)
S3method(latex, table.rq)
S3method(logLik, nlrq)
S3method(logLik, rq)
S3method(logLik, rqs)
S3method(logLik, rqss)
S3method(plot, qss1)
S3method(plot, qss2)
S3method(plot, rq.process)
S3method(plot, rqs)
S3method(plot, rqss)
S3method(plot, summary.crqs)
S3method(plot, summary.rqs)
S3method(plot, summary.rqss)
S3method(plot, table.rq)
S3method(predict, crq)
S3method(predict, crqs)
S3method(predict, nlrq)
S3method(predict, qss1)
S3method(predict, qss2)
S3method(predict, rq)
S3method(predict, rq.process)
S3method(predict, rqs)
S3method(predict, rqss)
S3method(print, anova.rq)
S3method(print, crq)
S3method(print, dynrq)
S3method(print, dynrqs)
S3method(print, nlrq)
S3method(print, rq)
S3method(print, rqs)
S3method(print, rqss)
S3method(print, summary.crq)
S3method(print, summary.crqs)
S3method(print, summary.dynrq)
S3method(print, summary.dynrqs)
S3method(print, summary.nlrq)
S3method(print, summary.rq)
S3method(print, summary.rqs)
S3method(print, summary.rqss)
S3method(print, KhmaladzeTest)
S3method(plot, KhmaladzeTest)
S3method(residuals, nlrq)
S3method(end, dynrq)
S3method(start, dynrq)
S3method(summary, crq)
S3method(summary, crqs)
S3method(summary, dynrqs)
S3method(summary, nlrq)
S3method(summary, rq)
S3method(summary, rqs)
S3method(summary, rqss)
S3method(print, Hill)
S3method(summary, Hill)
S3method(print, summary.Hill)
S3method(print, Pickands)
S3method(summary, Pickands)
S3method(print, summary.Pickands)
S3method(plot, qts1)

Versions of quantreg between 3.70 and 4.75 were removed from the CRAN
archive due to uncertainties over the licensing status of the fortran
code in src/cholesky.f.  As of 9 March 2012, original authors of cholesky.f,  
Esmond Ng and Barry Peyton,  have now, very kindly,  given permission to 
use cholesky.f under an open source license.  They have requested that 
their code be credited via the following two publications:

   Esmond G. Ng and Barry W. Peyton, "Block sparse Cholesky algorithms
   on advanced uniprocessor computers". SIAM J. Sci. Stat. Comput.
   14  (1993), pp. 1034-1056.

   John R. Gilbert, Esmond G. Ng, and Barry W. Peyton, "An efficient
   algorithm to compute row and column counts for sparse Cholesky
   factorization". SIAM J. Matrix Anal. Appl. 15 (1994), pp. 1075-1091.

Use of the sparse Cholesky code in cholesky.f in quantreg is limited  to calls
to rqss() or to rq with method = "sfn".  

In the event that readers wish to access the intermediate versions of quantreg
that have been removed from CRAN, they will be available at:

	http://www.econ.uiuc.edu/~roger/research/rq/quantreg

Current versions are best accessed from CRAN and its many mirrors.

This directory contains ratfor source for the several routines of the
rq library.  These are not strictly necessary, but they may prove useful
to anyone wanting to look at the source in detail, since the fortran
translation of them by the ratfor compiler is far from readable.

Note added September 17, 2004

Apparently g77 v3.4 doesn't like double precision data statements so
these have been changed to parameter() declarations.  This turned out
to be a bug in g77, but the deed is done so I've left the parameter
statements in place of the original data statements.  However, this
entailed the dangerous step of installing the SEP ratfor77 translator.
(See http://sepwww.stanford.edu/software/ratfor.html for details.)

Past experience, and now a bug file in the SEP distro, suggests
that the ratfor switch statement is unlikely to be correctly handled.
Fortunately, none of the included routines in the directory use switch
but caution is undoubtedly indicated.  The fortran produced for rqbr.r
is somewhat different than that produced by my older sun version, and
generates two warnings from g77, but these appear to be harmless.

Note added March 16, 2005

g77 has been issuing warnings for the code generated from the
SEP distro of ratfor version of rqbr.r for a while now, but
yesterday Jan deLeeuw reported that it now generates an error
in his gfortran 4.0 on osx.  After some comparisons with old
version of the ratfor translation, in particular the rq.f in
rq/statlib.95 on ysidro, I determined that what the compiler
didn't like was the way that if-endifs were generated by
the SEP translation.  So.... I hand edited these last few
if-endifs in the .f file, and move that back into the src
directory. This is TOTALLY unsatisfactory, since it means
that this needs to be done yet again if any changes are
made to the ratfor code, but seems to be the cost of  using
the SEP translator.  A more clever guy with more time could
probably find a way to change the ratfor, but given other
constraints that ain't going to happen anytime soon.  Note
that I've for future reference put a version of the old rq.f
file generated by my old BTL ratfor translator into this
directory. as well as the new rqbr.f. 

Note added Oct 22, 2007

As part of the development of the crqfnb function I downloaded
the SEP version of ratfor77, which seems to work ok on this 
function.  It is installed in my $HOME/bin directory and the
build is in the Rinstall directory.  Note that now on yzzy
ratfor is called ratfor77.

Note added May 14, 2016

Contrary to the last note, it appears that my $HOME/bin/ratfor
command works and $HOME/bin/ratfor77 does not.

Note added Jan 22, 2020

Contrary to the last note there is no $HOME/bin/ratfor* but I've installed
ratfor90 in /usr/local/bin and this might produce something workable.  Not
yet really tested seriously, but those ugly line numbers are gone, at least.

Note added November 8 2020

I've installed a new ratfor executable in my mac mini bin directory.  It
is generated from Brian Gaeke's C distribution ratfor-1.05.  And seems to work
well; i.e. produces .f files that compile with gfortran -c with only a couple
of warnings about legacy syntax.  The good news is that these warnings are 
emitted only for the rqbr.f code produced from rqbr.r, for which my handedited
rqbr.f version still passes R CMD check.  This was reported to Brian and he
suggested flags that could be used to quell the emissions, but this didn't
seem a.) very practical, or b.) very necessary.  I've put links to Brian G's
code for ratfor and to Brian K's BTL memo describing ratfor on my
reproducibility webpage.  So for the foreseeable, we are ok on the ratfor
front, including for the new subroutines pfnb.r and qfnb.r.  To reiterate, all
the ratfor in this directory, with the exception of rqbr.r is used to generate
the fortran in the src directory above. 

library(quantreg)

rq.fit.panel <- function(X,y,s,w=c(1/3,1/3,1/3),taus=c(0.25,0.5,0.75),lambda = 0){
# prototype function for fixed effect panel data fitting of QR models
# the vector s is a strata indicator assumed (so far) to be a one-way layout
# NB:  
# 1.  The value of the shrinkage parameter lambda is an open research problem in
# 	the simplest homogeneous settings it should be the ratio of the scale parameters
# 	of the fixed effects and the idiosyncratic errors
# 2.  On return the coefficient vector has m*p + n elements where m is the number
#	quantiles being estimated, p is the number of columns of X, and n is the
#	number of distinct values of s.  The first m*p coefficients are the 
#	slope estimates, and the last n are the "fixed effects"
# 3.  Like all shrinkage (regularization) estimators, asymptotic inference is somewhat
#	problematic... so the bootstrap is the natural first resort.


	require(SparseM)
	require(quantreg)
	m <- length(w)
	if(m != length(taus))
		stop("length of w and taus must match")
	X <- as.matrix(X)
        p <- ncol(X)
        n <- length(levels(as.factor(s)))
        N <- length(y)
	if(N != length(s) || N != nrow(X))
		stop("dimensions of y,X,s must match")
        Z <- as.matrix.csr(model.matrix(~as.factor(s)-1))
        Fidelity <- cbind(as(w,"matrix.diag.csr") %x% X, cbind(w) %x% Z)
        Penalty <- cbind(as.matrix.csr(0,n,m*p),lambda*as(n,"matrix.diag.csr"))
        D <- rbind(Fidelity,Penalty)
        y <- c(w %x% y,rep(0,n))
	a <- c((w*(1-taus)) %x% (t(X)%*%rep(1,N)),
		sum(w*(1-taus)) * (t(Z) %*% rep(1,N)) + lambda * rep(1,n))
	rq.fit.sfn(D,y,rhs=a)
	}

n<-3
T<-50
nT<-n*T
u1<-rnorm(T)
u2<-rnorm(T) 
u3<-rnorm(T)
x1<-rnorm(T,1,0.85)
x2<-rnorm(T,4,1)
x3<-rnorm(T,7,1)

beta1<-1
beta2<-0.5

y1<- 0+beta1*x1+(beta2*x1)*u1
y2<- 4+beta1*x2+(beta2*x2)*u2
y3<- 8+beta1*x3+(beta2*x3)*u3


plot(c(0,9), c(0,25), type='n', xlab=expression(x[it]), ylab=expression(y[it]))
points(x1,y1,pch=15)
points(x2,y2,pch=15,col="blue")
points(x3,y3,pch=15,col="red")
legend(1,17,paste("i = ",1:3,sep = ""),pch = 15, col = c("black","blue","red"))

ya<-c(y1,y2,y3)
xa<-c(x1,x2,x3)

# Naive cross-section QR

taus <- c(.25,0.5,.75)
xx <- seq(min(xa),max(xa),0.25)
f <- coef(rq(ya~xa,tau=taus))
yy <- cbind(1,xx)%*%f
for(i in 1:3)
    lines(xx,yy[,i],col = "grey")

# Fixed effect QR

s <- rep(1:n,rep(T,n))
fp<-rq.fit.panel(xa,ya,s)$coef

bhat <- fp[1:3]
fehat <- fp[4:6]

xx1 <- seq(min(x1),max(x1),0.25)
for(i in 1:3){
    yy1 <- fehat[1] + bhat[i] * xx1
    lines(xx1,yy1,col = "black")
}

xx2 <- seq(min(x2),max(x2),0.25)
for(i in 1:3){
    yy2 <- fehat[2] + bhat[i] * xx2
    lines(xx2,yy2,col = "blue")
}

xx3 <- seq(min(x3),max(x3),0.25)
for(i in 1:3){
    yy3 <- fehat[3] + bhat[i] * xx3
    lines(xx3,yy3,col = "red")
}

require(quantreg)
data(Mammals)
     attach(Mammals)
     x <- log(weight)
     xx <- unique(x[order(x)])
     y <- log(speed)
     plot(x,y, xlab="Weight in log(Kg)", ylab="Speed in log(Km/hour)",type="n")
     points(x[hoppers],y[hoppers],pch = "h", col="red")
     points(x[specials],y[specials],pch = "s", col="blue")
     others <- (!hoppers & !specials)
     points(x[others],y[others], col="black",cex = .75)
     taus <- c(.5, .9)
     for(i in 1:length(taus)){
        fit <- rqss(y ~ qss(x, lambda = 1, constraint = "C"),tau = taus[i])
        plot(fit,title = "Running Speed of Mammals", add = TRUE, col = i, lwd = 1.5)
        }
     legend(4,2,c("Median", "0.9 Quantile"), lty = 1, col = 1:2, lwd = 1.5)
     #Now plot confidence bands for the tau = .9 fit
     plot(fit,title = "Running Speed of Mammals", band = "both", col = i, lwd = 1.5)
     #Now plot slope of the tau = .9 line
     xy <- fit$qss[[1]]$xyz
     xx <- xy[,1]
     yhat <- fit$coef[1] + xy[,2]
     g <- diff(yhat)/diff(xx)
     plot(xx[-1], g, main = "Fitted Slopes of Running Speed",
          xlab="Weight in log(Kg)", ylab="dlog(Speed) /dlog(Weight)")

## Demo of nonlinear quantile regression model based on Frank copula


vFrank <- function(x, df, delta, u)
    -log(1-(1-exp(-delta))/(1+exp(-delta*pt(x,df))*((1/u)-1)))/delta

FrankModel <- function(x, delta, mu,sigma, df, tau) {
    z <- qt(vFrank(x, df, delta, u = tau), df)
    mu + sigma*z
}

n <- 200
df <- 8
delta <- 8

set.seed(1989)

x <- sort(rt(n,df))
v <- vFrank(x, df, delta, u = runif(n))
y <- qt(v, df)
plot(x, y, pch="o", col="blue", cex = .25)
Dat <- data.frame(x = x, y = y)

us <- c(.25,.5,.75)
for(i in 1:length(us)){
    v <- vFrank(x, df, delta, u = us[i])
    lines(x, qt(v,df))
}

cfMat <- matrix(0, 3, length(us))

trace <- TRUE # a bit noisy ...
trace <- FALSE
for(i in 1:length(us)) {
    tau <- us[i]
    cat("tau = ", format(tau), ".. ")
    fit <- nlrq(y ~ FrankModel(x, delta,mu,sigma, df = 8, tau = tau),
                data = Dat, tau = tau,
                start= list(delta=5, mu = 0, sigma = 1),
                trace = trace)
    lines(x, predict(fit, newdata=x), lty=2, col="red")
    cfMat[i,] <- coef(fit)
    cat("\n")
}
colnames(cfMat) <- names(coef(fit))
cfMat

# Demo for prediction and confidence intervals 

par(ask = TRUE)
# Classical Gaussian Version
sub <- "Classical Gaussian Version"
plot(cars$speed,cars$dist,xlim=c(0,27),ylim=c(-10,130), main = "Ezekiel Data",
	sub = sub, xlab = "speed (mph)",ylab = "stopping distance (ft)")
ss = seq(0,27,len=100)
flm <- lm(dist ~ speed + I(speed^2),data = cars)
cflm <- predict(flm,list(speed=ss), data = cars, interval = "confidence")
pflm <- predict(flm,list(speed=ss), data = cars, interval = "prediction")
matlines(ss, pflm,lty = c(1,2,2),col=c("black","red","red"))
matlines(ss, cflm,lty = c(1,2,2),col=c("black","blue","blue"))

# Now try several different ways to compute alternative bands based on QR
fl=rq(dist~speed+I(speed^2),data=cars,tau= 0.25)
fu=rq(dist~speed+I(speed^2),data=cars,tau= 0.75)

# 1.  Covariance matrix estimation using "nid"
sub <- "Covariance matrix estimation using nid"
plot(cars$speed,cars$dist,xlim=c(0,27),ylim=c(-10,130), main = "Ezekiel Data",
	sub = sub, xlab = "speed (mph)",ylab = "stopping distance (ft)")
pl <- predict(fl,list(speed=ss),interval = "confidence",  se = "nid")
pu <- predict(fu,list(speed=ss),interval = "confidence",  se = "nid")
matlines(ss,pl,lty = c(1,2,0),  col=c("black","red","red"))
matlines(ss,pu,lty = c(1,0,2),  col=c("black","red","red"))
# 2.  Covariance matrix estimation using xy-bootstrap
sub <- "Covariance matrix estimation using xy-bootstrap"
plot(cars$speed,cars$dist,xlim=c(0,27),ylim=c(-10,130), main = "Ezekiel Data",
	sub = sub, xlab = "speed (mph)",ylab = "stopping distance (ft)")
pl <- predict(fl,list(speed=ss),interval = "confidence",  se = "boot",bsmethod = "xy")
pu <- predict(fu,list(speed=ss),interval = "confidence",  se = "boot",bsmethod = "xy")
matlines(ss,pl,lty = c(1,2,0),  col=c("black","red","red"))
matlines(ss,pu,lty = c(1,0,2),  col=c("black","red","red"))
# 3.  Covariance matrix estimation using weighted xy-bootstrap
sub <- "Covariance matrix estimation using weighted xy-bootstrap"
plot(cars$speed,cars$dist,xlim=c(0,27),ylim=c(-10,130), main = "Ezekiel Data",
	sub = sub, xlab = "speed (mph)",ylab = "stopping distance (ft)")
pl <- predict(fl,list(speed=ss),interval = "confidence",  se = "boot",bsmethod = "wxy")
pu <- predict(fu,list(speed=ss),interval = "confidence",  se = "boot",bsmethod = "wxy")
matlines(ss,pl,lty = c(1,2,0),  col=c("black","red","red"))
matlines(ss,pu,lty = c(1,0,2),  col=c("black","red","red"))
# 4.  Percentile method using xy-bootstrap
sub <- "Percentile method using xy-bootstrap"
plot(cars$speed,cars$dist,xlim=c(0,27),ylim=c(-10,130), main = "Ezekiel Data",
	sub = sub, xlab = "speed (mph)",ylab = "stopping distance (ft)")
pl <- predict(fl,list(speed=ss),interval = "confidence",
       type="percentile", se = "boot", bsmethod = "xy")
pu <- predict(fu,list(speed=ss),interval = "confidence",
       type="percentile", se = "boot", bsmethod = "xy")
matlines(ss,pl,lty = c(1,2,0),  col=c("black","red","red"))
matlines(ss,pu,lty = c(1,0,2),  col=c("black","red","red"))
# 5.  Direct method using "iid" covariance matrix
sub <- "Direct method using iid covariance matrix"
plot(cars$speed,cars$dist,xlim=c(0,27),ylim=c(-10,130), main = "Ezekiel Data",
	sub = sub, xlab = "speed (mph)",ylab = "stopping distance (ft)")
pl <- predict(fl,list(speed=ss),interval = "confidence", type="direct", se = "iid")
pu <- predict(fu,list(speed=ss),interval = "confidence", type="direct", se = "iid")
matlines(ss,pl,lty = c(1,2,0),  col=c("black","red","red"))
matlines(ss,pu,lty = c(1,0,2),  col=c("black","red","red"))
# 6.  Direct method using "nid" covariance matrix
sub <- "Direct method using nid covariance matrix"
plot(cars$speed,cars$dist,xlim=c(0,27),ylim=c(-10,130), main = "Ezekiel Data",
	sub = sub, xlab = "speed (mph)",ylab = "stopping distance (ft)")
pl <- predict(fl,list(speed=ss),interval = "confidence", type="direct", se = "nid")
pu <- predict(fu,list(speed=ss),interval = "confidence", type="direct", se = "nid")
matlines(ss,pl,lty = c(1,2,0),  col=c("black","red","red"))
matlines(ss,pu,lty = c(1,0,2),  col=c("black","red","red"))

par(ask = FALSE)

# Demo of combos functions
H <- combos(20,3)
if(!require("rgl",quietly=TRUE)){
      warning("The package rgl is needed for plotting")
     } else{
      if(interactive()){
         plot3d(t(H))
         lines3d(t(H),col=rep(topo.colors(57),20))
         }
     }

MCV <- function(lambdas, formula, data, tau = 0.5, k = 10){
    F <- Munge(formula, lambdas = lambdas)
    f <- rqss(F, data, tau = tau)
    n <- f$n
    m <- length(f$qss)
    y <- f$y[1:n]
    folds = sample(rep(1:k, length = n))
    U = NULL
    for(i in 1:k){
	s = which(folds != i)
	M = rqss(F, data = data[s,], tau = tau)
	nd = data[-s,]
	G = matrix(0,nrow(nd),m)
	for(j in 1:m){ #remove extrapolates, if any
	    g = f$qss[[j]]$xyz[,1]
	    G[,j] = (min(g[s]) < g[-s]) & (g[-s] < max(g[s]))
	}
	h = as.logical(apply(G,1,prod))
	u = predict(M, newdata = nd[h,]) - (y[-s])[h]
	U = c(U,(u * (tau - (u < 0))))
    }
    mean(U)
}
set.seed(1729)
n <- 200
x <- sort(runif(n, 0, 20))
g0 <- function(x, tau) 
    log(x) + 0.2*(log(x))^3 + log(x) * qnorm(tau)/4 
y <- g0(x, runif(n))
D <- data.frame(y = y, x = x)
lams <- mcvs <- seq(.02, 5, by = 0.2)
for(i in 1:length(mcvs))
    mcvs[i] <- MCV(lams[i], y ~ qss(x, lambda = lambdas[1]), D)
par(mfrow = c(1,2))
plot(lams, mcvs, cex = .5, lwd = 2, type = 'l',
    xlab = expression(lambda), ylab = expression(MCV( lambda )))
lambdastar <- lams[which.min(mcvs)]

plot(x, y, cex = .5, col = "grey")
f <- rqss(y ~ qss(x, lambda = lambdastar), data = D)
plot(f, add = TRUE, lwd = 2)
lines(x,g0(x, 0.5), col = "red", lwd = 2)
text(10, 1,bquote(lambda == ~  .(lambdastar)))

# Toy rqss example based on Figure 2 of Polson and Scott (2016, JRSSB)
# NB:  Solutions are piecewise linear, unlike those of Polson and Scott
# whose ADMM procedure does only 30 iterations.
dgp <- function(n) {
    x <- 0:n/n
    y <- rnorm(n+1, 5 * sin(2 * pi * x), 0.5 + exp(1.5 * sin(4 * pi * x))) 
    data.frame(x = x, y = y)
}
D <- dgp(1000)
plot(D$x, D$y, cex = .5)
taus <- 1:9/10
for(i in 1:length(taus))
    plot(rqss(y ~ qss(x, lambda = 1/10), tau = taus[i], data = D), 
	 rug = FALSE, add = TRUE)

# log likelihood for stackloss fit
require(quantreg)
data(stackloss)
logLik.rq.process <- function(fit){
    y <- model.response(model.frame(fit))
    fhat <- predict(fit, type = "fhat")
    fy <- mapply(function(f,y) f(y), fhat, y)
    sum(log(fy))
}
# First try with full process estimates
f0 <- rq(stack.loss ~ 1, tau=-1)
f1 <- rq(stack.loss ~ stack.x, tau=-1)
l0 <- logLik(f0)
l1 <- logLik(f1)
# Now try with discrete process estimates
f0 <- rq(stack.loss ~ 1, tau=1:19/20)
f1 <- rq(stack.loss ~ stack.x, tau=1:19/20)
l0 <- logLik(f0)
l1 <- logLik(f1)

#### Demo for an rgl Animation of Cobar Ore fitting

require(quantreg)

if(requireNamespace("interp")){
## Make sure the demo does not ``die'' when rgl is not available:
do.rgl <- interactive() && require(rgl)

data(CobarOre)

### Make an initial quite rough fit of the data
fit <- rqss(z ~ qss(cbind(x,y), lambda = .01, ndum = 100),
            data = CobarOre)
dummies <- fit$qss[[1]]$dummies
zcol <- CobarOre$z

if(do.rgl) {
    plot(fit, render = "rgl")
    cat("Now orient the plot as needed:",
	"Resize window,",
	"mouse button 1 to change viewpoint,",
	"mouse button 2 to zoom,",
	"and hit return when ready",sep="\n")
    scan()
    rgl.bg(color="8")
} else {
    if(!interactive()) pdf(file = "cobar-demo.pdf")
    plot(fit)
}

for(i in 1:20) {
    fname <- paste("cobar",i,".png",sep="")
    lam <- 2*i/100
    fit <- rqss(z ~ qss(cbind(x,y), lambda = lam, dummies = dummies),
                data = CobarOre)
    if(do.rgl) {
        rgl.clear()
        plot(fit, render = "rgl", zcol = zcol)
        rgl.snapshot(fname)
    } else {
        plot(fit, zcol = zcol)
    }
}
}

## Demo for a simple change point (broken-stick) problem

x <- runif(200,0,10)
u <- rnorm(200)/5
y <- 1 + x - .5 * (x-3) * (x > 3) + u
plot(y ~ x, cex= .5, col = "grey")


z <- rqss(y ~ qss(x,lambda = 10), tau= .50)
plot(z, col = "dark blue")

#Now plot the fitted points and jump points in derivative

eps <- 0.00001 # Zero Tolerance
Nz <- abs(z$resid[1:200]) < eps
Nj <- abs(z$resid[201:398]) > eps
xx <- z$qss[[1]]$xyz[,1]
yy <- z$coef[1] + z$qss[[1]]$xyz[,2]
xj <- xx[3:200]
yj <- yy[3:200]
points(xx[Nz],yy[Nz],col="green")
points(xj[Nj],yj[Nj],col="red")
print(paste("Number of zero residuals:  ",sum(Nz)))
print(paste("Number of jumps in slope:  ",sum(Nj)))
legend(6,3,c("Derivative Jumps", "Zero residuals"),pch = "o", col=c("red","green"))

# Example Comparison of Kaplan-Meier vs crq fitting
# The red crq estimate should overplot the black KM Survival Curve.
if (requireNamespace("survival", quietly = TRUE)){
    n <- 100
    y <- rchisq(n,3)
    c <- rchisq(n,5)
    Y <- pmin(y,c)
    d <- (y < c)
    Surv <- survival::Surv
    plot(survival::survfit(Surv(Y,d)~1))
    f <- crq(Surv(Y,d)~1, method = "Portnoy", grid = "pivot")
    x <- f$sol[2,]
    p <- 1-f$sol[1,]
    p <- c(p,p[length(p)])
    par(col = "red")
    fs <- plot(stepfun(x, p),do.points = FALSE, add = TRUE)
}

#Toy rqss example with lasso shrinkage of linear covariate effects

n <- 100
p <- 9
q <- 3
beta <- c( rep(1,q), rep(0,p-q))
w <- matrix(rnorm(n*p),n,p)
x <- runif(n,0,10)
z <- runif(n,0,10)
y <- w %*% beta + sin(x) + (z^2)/50 + rnorm(n)/5
d <- data.frame(w,x,y,z) 
f <- rqss(y ~ w + qss(x,lambda = 3) + qss(z,lambda = 2),
                method = "lasso", lambda = 3, data = d)
plot(f, bands = "both", bcol = c("lightsteelblue", "lightsteelblue4"))

#### Demo for a plot of two quantile functions of food expenditure

###-- short version of the rq *VIGNETTE* --- use that!

data(engel)
## do *NOT* attach()

## Poor is defined as at the .1 quantile of the sample distn
## Rich is defined as at the .9 quantile of the sample distn
x.poor <- quantile(engel[,"income"], .10)
x.rich <- quantile(engel[,"income"], .90)

z <- rq(foodexp ~ income, tau= -1, data = engel)

ps <- z$sol["tau",]
coefs <- z$sol[4:5,]
qs.poor <- c(c(1,x.poor) %*% coefs)
qs.rich <- c(c(1,x.rich) %*% coefs)
## now plot the two quantile functions to compare
par(mfrow = c(1,2))
plot(c(ps,ps),c(qs.poor,qs.rich),type="n",xlab=expression(tau),ylab="quantile")
plot(stepfun(ps,c(qs.poor[1],qs.poor)),do.points=FALSE,add=TRUE)
plot(stepfun(ps,c(qs.poor[1],qs.rich)),do.points=FALSE,add=TRUE,
        col.hor = "gray", col.vert = "gray")
## now plot associated conditional density estimates
## weights from ps (process)
ps.wts <- (c(0,diff(ps)) + c(diff(ps),0)) / 2
ap <- akj(qs.poor, z=qs.poor, p = ps.wts)
ar <- akj(qs.rich, z=qs.rich, p = ps.wts)
plot(c(qs.poor,qs.rich), c(ap$dens,ar$dens), type="n",
     xlab= "Food Expenditure", ylab= "Density")
lines(qs.rich, ar$dens, col="gray")
lines(qs.poor, ap$dens, col="black")
legend("topright", c("poor","rich"), lty = c(1,1), col=c("black","gray"))

# Analysis of the QAR(1) Melbourne Temperature Example
# Using new type = "fhat" option in predict.rqs
require(splines)
  if(interactive()){ 
    oldpar <- par(ask = TRUE)
    data(MelTemp)
    x <- MelTemp[-3650]
    y <- MelTemp[-1]
    s <- (x<40)  #Delete a few (influential, ridiculously hot) days
    x <- x[s]
    y <- y[s]
    z <- seq(10,36,length=100)

	fit <- rq(y~ bs(x,knots=quantile(x,c(.05,.25,.5,.75,.95))), tau =  1:19/20)
	par(cex=1,pty="s")
	xlab <- "yesterday's max temperature"
	ylab <- "today's max temperature"
	plot(x,y,pch=".",xlab=xlab,ylab=ylab)
	matlines(z,predict(fit, newdata = data.frame(x = z)), lty = 1)
	abline(c(0,1),lty=3)
	title("Melbourne QAR Model")

	taus <- 1:199/200
	xs <- c(11,16,21,25,30,35)
	fit <- rq(y~ bs(x,knots=quantile(x,c(.05,.25,.5,.75,.95))), tau = taus)
	fhats <- predict(fit,newdata = data.frame(x = xs), type = "fhat", h = 1)
	par(mfrow = c(2,3))
	for(i in 1:length(xs)){
		fhat <- fhats[[i]]
		x <- environment(fhat)$x
		xlab <- "today's max temperature"
		plot(x,fhat(x),type="l",xlab=xlab,ylab="density")
		abline(v=xs[i], col="red")
		title(paste("Yesterday's Temp", format(round(xs[i]))))
		}
	par(oldpar)
    }

# Convert H_0:  R beta = r  to an exclusion restriction a la Section 3.7.3 of QR book
# Note the typo in the definition of yt in that source!

if(require(MASS)){ # For Null
require(quantreg) # For Null
X <- cbind(1, matrix(rnorm(500),100,5))
y <- rnorm(100)
R <- matrix(rnorm(18),3,6)
r <- rep(1,3)
R <- t(R)
P <- MASS::Null(R)
Xt <- t(lsfit(P,t(X),intercept = FALSE)$coef)
Zt <- t(lsfit(R,t(X),intercept = FALSE)$coef)
yt <- y - Zt %*% r
f0 <- rq(yt ~ Xt - 1)
f1 <- rq(yt ~ Xt + Zt - 1)
T <- anova(f0,f1)
}

# UIS example for the crq Peng-Hwang  method.

#estimate the Peng and Huang model using log(TIME) AFT specification
if(requireNamespace("survival", quietly = TRUE)) {
    data(uis)
    Surv <- survival::Surv
    fit <- crq(Surv(log(TIME), CENSOR) ~  ND1 + ND2 + IV3 +
               TREAT + FRAC + RACE + AGE * SITE, method = "Portnoy", data = uis)
    Sfit <- summary(fit,1:19/20)
    PHit <- survival::coxph(Surv(TIME, CENSOR) ~  ND1 + ND2 + IV3 +
               TREAT + FRAC + RACE + AGE * SITE, data = uis)
    plot(Sfit, CoxPHit = PHit)
    formula <-  ~  ND1 + ND2 + IV3 + TREAT + FRAC + RACE + AGE * SITE -1
    X <- data.frame(model.matrix(formula,data=uis))
    newd <- as.list(apply(X,2,median))
    pred <- predict(fit, newdata=newd, stepfun = TRUE)
    plot(pred,do.points=FALSE,xlab = expression(tau), ylab = expression(Q(tau)),
	    lwd = 1.5, main= "Quantiles at Median Covariate Values")
    plot(rearrange(pred),add=TRUE,do.points=FALSE,col.vert ="red", col.hor="red")
    legend(.10,10,c("Raw","Rearranged"),lty = rep(1,2),col=c("black","red"))
}

#Analysis of the QAR(1) Melbourne Temperature Example
require(splines)
if(interactive()){ 
    oldpar <- par(ask = TRUE)
    data(MelTemp)
    x <- MelTemp[-3650]
    y <- MelTemp[-1]
    s <- (x<40)  #Delete a few (influential, ridiculously hot) days
    x <- x[s]
    y <- y[s]
    z <- seq(10,36,length=100)

	fit <- rq(y~ bs(x,knots=quantile(x,c(.05,.25,.5,.75,.95))), tau =  1:19/20)
	par(cex=1,pty="s")
	xlab <- "yesterday's max temperature"
	ylab <- "today's max temperature"
	plot(x,y,pch=".",xlab=xlab,ylab=ylab)
	matlines(z,predict(fit, newdata = data.frame(x = z)), lty = 1)
	abline(c(0,1),lty=3)
	title("Melbourne QAR Model")

	taus <- 1:199/200
	xs <- c(11,16,21,25,30,35)
	fit <- rq(y~ bs(x,knots=quantile(x,c(.05,.25,.5,.75,.95))), tau = taus)
	Qy <- predict(fit,newdata = data.frame(x = xs))
	par(mfrow = c(2,3))
	for(i in 1:length(xs)){
		Qyi <- Qy[i,-1]
		fhat <- akj(Qyi,Qyi,diff(taus), h = 1)$dens
		xlab <- "today's max temperature"
		plot(Qyi,fhat,type="l",xlab=xlab,ylab="density")
		abline(v=xs[i], col="red")
		title(paste("Yesterday's Temp", format(round(xs[i]))))
		}
	par(oldpar)

}

# A toy example to illustrate univariate smoothing with automatic lambda selection
n <- 2000
x <- 1:n/n
noise <- rgamma(n,3,1)
g0 <- function(x) sin(10*x)
y <- g0(x)+noise
arqss <- function(x,y,tau,g0 = NULL){
    g <- function(lam,y,x,tau) AIC(rqss(y ~ qss(x, lambda = lam),tau = tau),k = -1)
    lamstar <- optimize(g, interval = c(0.01, .5), x = x, y = y, tau = tau)
    f <- rqss(y ~ qss(x, lambda = lamstar$min))
    plot(f)
    lines(x,g0(x)+qgamma(tau,3,1),col = "red")
    text(.7,2,paste("lambda = ", round(lamstar$min,3)))
}
arqss(x,y,.5,g0)

# Test case for the new subset argument
require(quantreg)
n <- 200
x <- sort(rchisq(n,4))
z <- rnorm(n)
s <- sample(1:n, n/2)
y <- log(x) + rnorm(n)/5
D = data.frame(y = y, x = x, z = z, s = (1:n) %in% s)
plot(x, y, cex = .5, col = "grey")
points(x[s], y[s],col = "pink", cex = .5)
lam = 0.2
f0 <- rqss(y ~ qss(x,lambda = lam) + z, subset = s)
f1 <- rqss(y ~ qss(x, lambda = lam) + z, subset = s, data = D)
plot(f0, add = TRUE, col = 2, lwd = 3)
plot(f1, add = TRUE, col = 4, lwd = 3)

# A Demo of simple bivariate rqss fitting of a hinge function

require(quantreg)
if(requireNamespace("interp")){
## Make sure the demo does not ``die'' when rgl is not available:
do.rgl <- interactive() && require(rgl)

#generate the data

n <- 1000
x <- runif(n)
y <- runif(n)
z <- -abs(x-y)

### Make an initial quite rough fit of the data
fit <- rqss(z ~ qss(cbind(x,y),lambda = .005))
print(summary(fit)$penalty)

if(do.rgl) {
    plot(fit, render = "rgl")
    cat("Now orient the plot as desired:",
	"Resize window,",
	"mouse button 1 to change viewpoint,",
	"mouse button 2 to zoom,",
	"and hit return when ready",sep="\n")
    scan()
    rgl.bg(color="8")
    cat("To try another value of lambda:",
	"Type a positive number",
	"To quit hit return", sep="\n")
    repeat{
	cat("lambda:  ")
	lam <- scan(what = double(1))
	if(length(lam)>0){
		fit <- rqss(z ~ qss(cbind(x,y),lambda = lam))
		rgl.clear()
		plot(fit, render = "rgl")
	} else 
		break
	}
} else {
    if(!interactive()) pdf(file = "hinge-demo.pdf")
    plot(fit)
}
}

## Demo for a Plot of Engel curve in sample space

data(engel)

plot(foodexp ~ income, data = engel, cex= .5, col = "blue",
     xlab = "Household Income", ylab = "Food Expenditure")

z <- rq(foodexp ~ income, tau= .50, data = engel)# "median line": L1 - regression
abline(z, col = "dark blue")
abline(lm(foodexp ~ income, data = engel), lty=2, col="red") #the dreaded ols line

taus <- c(.05,.1,.25,.75,.90,.95)
nt <- length(taus)

for( i in 1:length(taus)) {
    abline(rq(foodexp~income, tau=taus[i], data = engel), col="gray")
}

legend("bottomright",
       c("L1 (tau = .50)", "OLS", paste("tau= ", formatC(rev(taus)))),
       col = c("dark blue", "red", rep("gray", nt)),
       lty = c(1,2, rep(1, nt)),
       inset = 0.03)

"qrisk" <-
function(x, alpha=c(.1,.3), w = c(.7,.3), mu = .07, R = NULL, r = NULL, lambda = 10000){
#
# find optimal Choquet-risk portfolios given:
#
#	x 	(n by p) matrix of asset returns
#	alphas	alphas defining a Choquet capacity risk function
#	w	w defining weights for Choquet capacity risk function
#	R	Matrix defining constraints on the parameters 
#	r	rhs defining constraints on the parameters 
#	mu	required mean rate of return	
#	lambda	Lagrange multiplier for RoR constraint 
#
	n <- nrow(x)
	p <- ncol(x)
	m <- length(alpha)
	if(length(w)!=m)stop("length of w doesn't match length of alpha")
	xbar <- apply(x,2,mean)
	y <- x[,1]
	r <- c(r,lambda*(xbar[1]-mu), -lambda*(xbar[1]-mu))
	X <- x[,1]-x[,-1]
	R <- rbind(R,lambda*(xbar[1]-xbar[-1]), -lambda*(xbar[1]-xbar[-1]))
	R <- cbind(matrix(0,nrow(R),m),R)
	f <- rq.fit.hogg(X,y,taus=alpha,weights=w,R=R,r=r)
	fit <- f$coefficients
	pihat <- c(1-sum(fit[-(1:m)]),fit[-(1:m)])
	x <- as.matrix(x)
	yhat <- x%*%pihat
	etahat <- quantile(yhat,alpha)
	muhat <- mean(yhat)
	qrisk <- 0
	for(i in 1:length(alpha))
		qrisk <- qrisk + w[i]*sum(yhat[yhat<etahat[i]])/(n*alpha[i]) 
	list(pihat = pihat, muhat = muhat, qrisk = qrisk)
}
"srisk" <-
function(x,mu=.07,lambda=100000000,alpha=.1,eps=.0001){
#
# find optimal sigma-risk (Markowitz) portfolios given:
#
#	x 	(n by p) matrix of asset returns
#	mu	required mean return
#	lambda	Lagrange multiplier for required mean return constraint
#
	n <- nrow(x)
	p <- ncol(x)
	X <- rbind(x,apply(x,2,mean))
	y <- X[,1]
	y[n+1] <- lambda*(y[n+1]-mu)
	X <- X[,1]-X[,-1]
	X <- cbind(c(rep(1,n),0),X)
	X[n+1,] <- lambda*X[n+1,]
	fit <- lm(y~X-1)
	pihat <- c(1-sum(fit$coef[-1]),fit$coef[-1])
	if(abs(fit$residual[n+1]) > eps) return("lambda too small?")
	yhat <- x%*%pihat
	muhat <- mean(x%*%pihat)
	sigma <- sqrt(var(x%*%pihat))
	list(pihat = pihat, muhat = muhat, sigma = sigma)
}

C Output from Public domain Ratfor, version 1.05
      subroutine pwxy(n,p,m,a,y,tau,qk,r,b,w,band,n0,d,u,wn,wp, aa,yy,sl
     *o,shi,rhs,glob,ghib,nit,info)
      integer n,p,m,kk(2),nn,n0,nit(5,m),info(m),sumbad,i,j,k,ir
      integer loq,hiq, slo(n),shi(n),ifix,ibad
      logical notopt
      double precision a(p,n),y(n),tau,qk(2),r(n),d(n),u(n),b(p,m),w(n)
      double precision wn(n,9), wp(p,(p+3)),band(n)
      double precision glob(p),ghib(p),aa(p,n),yy(n),rhs(p)
      double precision zero,one,beta,eps,big
      parameter(zero = 0.0d0)
      parameter(one = 1.0d0)
      parameter(beta = 0.99995d0)
      parameter(big = 1.0d+10)
      parameter(eps = 1.0d-06)
      do23000 ir = 1,m 
      notopt = .true.
      call grexp(n,w,one)
      nn = n0
      ifix = 0
      ibad = 0
23002 if(notopt)then
      ibad = ibad + 1
      loq = max0(1, int(n*tau - nn/2.))
      hiq = min0(int(n*tau + nn/2.), n)
      qk(1) = r(loq)
      qk(2) = r(hiq)
      call iphil(n,0,slo)
      call iphil(n,0,shi)
      do23004 i = 1,n
      if(r(i) .lt. qk(1))then
      slo(i) = 1
      else
      if(r(i) .gt. qk(2))then
      shi(i) = 1
      endif
      endif
23004 continue
23005 continue
23010 if(notopt)then
      ifix = ifix + 1
      call dphil(p,zero,glob)
      call dphil(p,zero,ghib)
      call dphil(n,one,d)
      call dphil(n,one,u)
      k = 0
      do23012 i = 1,n
      if(slo(i) .eq. 0 .and. shi(i) .eq. 0)then
      k = k + 1
      call dphil(p,zero,aa(1,k))
      call daxpy(p,w(i),a(1,i),1,aa(1,k),1)
      yy(k) = -y(i)*w(i)
      else
      if(slo(i) .eq. 1)then
      do23018 j = 1,p
      glob(j) = glob(j) + a(j,i) * w(i)
23018 continue
23019 continue
      else
      if(shi(i) .eq. 1)then
      do23022 j = 1,p
      ghib(j) = ghib(j) + a(j,i) * w(i)
23022 continue
23023 continue
      endif
      endif
      endif
23012 continue
23013 continue
      call dcopy(p,glob,1,aa(1,k+1),1)
      call dcopy(p,ghib,1,aa(1,k+2),1)
      yy(k+1) = big
      yy(k+2) = -big
      call dgemv('N',p,k+2,one-tau,aa,p,d,1,zero,rhs,1)
      call dscal(k+2,zero,wn,1)
      call daxpy(k+2,one-tau,u,1,wn,1)
      call rqfnb(k+2,p,aa,yy,rhs,d,u,beta,eps,wn,wp,nit(1,ir),info(ir))
      call dcopy(p,wp,1,b(1,ir),1)
      call dcopy(n,y,1,u,1)
      call dgemv('T',p,n,one,a,p,b(1,ir),1,one,u,1)
      sumbad = 0
      do23024 i = 1,n 
      if((u(i) .gt. 0) .and. slo(i) .eq. 1)then
      slo(i) = 0
      sumbad = sumbad + 1
      endif
      if((u(i) .lt. 0) .and. shi(i) .eq. 1)then
      shi(i) = 0
      sumbad = sumbad + 1
      endif
23024 continue
23025 continue
      if(sumbad .gt. 0)then
      if(sumbad .gt. 0.1 * nn)then
      nn = min(2 * nn, n)
      goto 23011
      endif
      else
      notopt = .false.
      endif
      goto 23010
      endif
23011 continue
      nit(4,ir) = ifix
      nit(5,ir) = ibad
      goto 23002
      endif
23003 continue
23000 continue
23001 continue
      return
      end

"FAQ" <- function(pkg = "quantreg")
     file.show(file.path(system.file(package = pkg),"FAQ"))
"ChangeLog" <- function(pkg = "quantreg")
     file.show(file.path(system.file(package = pkg),"ChangeLog"))

C Output from Public domain Ratfor, version 1.05
      subroutine rqs(m,n,k,m5,n2,a,b,tau,toler,ift,x,e,s,wa,wb)
      double precision b(m,k),a(m,n),x(n,k),e(m),wa(m5,n2),wb(m)
      double precision tau,toler
      integer s(m),m,n,k,m5,n2,ift(k)
      do23000 i=1,k
      call rq0(m,n,m5,n2,a,b(1,i),tau,toler,ift(i),x(1,i),e,s,wa,wb)
23000 continue
23001 continue
      return
      end

library(quantreg)

(dDIR <- system.file("demo", package = "quantreg"))
set.seed(1) # since some demos randomly generate

cat("Running demos from package 'quantreg' : \n\n")
for(ff in list.files(dDIR, pattern="\\.R$", full.names = TRUE)) {
   f <- basename(ff)
   cat("\n", f," :\n", paste(rep.int("-", nchar(f)), collapse=''),
       "\n", sep='')

   source(ff, echo = TRUE)
}

# Wrapper to compute several quantiles of a sample of n observations
# Calls  K.C. Kiwiel's version of Floyd and Rivest's select algorithm
# Caveat Emptor!!  The ks need to  be sorted.
subroutine kuantiles(k,m,n,x)
integer i,j,k(m),m,n
double precision  x(n)

j = 0
do i = 1,m{
	call dsel05(k(i)-j,n-j,x(j+1))
	j = k(i)
	}
return
end

C Output from Public domain Ratfor, version 1.05
      subroutine qselect(n,x,q)
      integer n,k,l,r
      double precision x(n),q
      k=nint(q*n)
      l=1
      r=n
      call select(n,x,l,r,k)
      q=x(k)
      return
      end
      recursive subroutine select(n,x,l,r,k)
      integer n,m,l,r,k,ll,rr,i,j,mmax
      double precision x(n),z,s,d,t,fm,cs,cd
      parameter(cs = 0.5d0)
      parameter(cd = 0.5d0)
      parameter(mmax = 600)
23000 if(r.gt.l)then
      if(r-l.gt.mmax)then
      m=r-l+1
      i=k-l+1
      fm = dfloat(m)
      z=log(fm)
      s=cs*exp(2*z/3)
      d=cd*sqrt(z*s*(m-s)/fm)*sign(1,i-m/2)
      ll=max(l,nint(k-i*s/fm + d))
      rr=min(r,nint(k+(m-i)*s/fm + d))
      call select(n,x,ll,rr,k)
      endif
      t=x(k)
      i=l
      j=r
      call dswap(1,x(l),1,x(k),1)
      if(x(r).gt.t)then
      call dswap(1,x(r),1,x(l),1)
      endif
23006 if(i.lt.j)then
      call dswap(1,x(i),1,x(j),1)
      i=i+1
      j=j-1
23008 if(x(i).lt.t)then
      i=i+1
      goto 23008
      endif
23009 continue
23010 if(x(j).gt.t)then
      j=j-1
      goto 23010
      endif
23011 continue
      goto 23006
      endif
23007 continue
      if(x(l).eq.t)then
      call dswap(1,x(l),1,x(j),1)
      else
      j=j+1
      call dswap(1,x(j),1,x(r),1)
      endif
      if(j.le.k)then
      l=j+1
      endif
      if(k.le.j)then
      r=j-1
      endif
      goto 23000
      endif
23001 continue
      return
      end

C Output from Public domain Ratfor, version 1.05
      subroutine powell(n,p,p2,a,b,c,x,tau,h,f,u,s,g,d,xh,maxit,nflag)
      integer n,p,p2
      double precision x(p),a(n,p),b(n),c(n)
      double precision f(n),u(p,p),s(n),g(p2),d(p),xh(p)
      double precision zero, one, mone, step, tau,pow
      integer h(p),hin,hout,k,it,inset,maxit,nflag
      parameter(zero = 0.0d0, one = 1.d0, mone = -1.d0)
      it = 0
23000 continue
      it = it + 1
      if(it .gt. 1)then
      call pivot(n,p,h,hin,hout,a,u,d,xh,nflag)
      endif
      if(nflag .gt. 0)then
      nflag = nflag + 2
      return
      endif
      do23007 i = 1,p
      xh(i) = b(h(i))
23007 continue
23008 continue
      call dgemv('N',p,p,one,u,p,xh,1,zero,x,1)
      call dgemv('N',n,p,one,a,n,x,1,zero,f,1)
      do23009 i = 1,n
      if(inset(p,i,h) .gt. 0 .or. f(i) .gt. c(i))then
      s(i) = zero
      else
      if(b(i) .lt. f(i))then
      s(i) = one - tau
      else
      s(i) = - tau
      endif
      endif
23009 continue
23010 continue
      call dgemv('T',n,p,one,a,n,s,1,zero,xh,1)
      call dgemv('T',p,p,one,u,p,xh,1,zero,g,1)
      do23015 i = 1,p 
      if(f(h(i)) .lt. c(h(i)))then
      if(b(h(i)) .lt. c(h(i)))then
      g(i + p) = - g(i) + one - tau
      else
      g(i + p) = - g(i) - tau
      endif
      else
      g(i + p) = - g(i) + tau
      endif
      g(i) = g(i) + one - tau
23015 continue
23016 continue
      k = idmin(p2,g,1)
      if(g(k) .ge. 0 .or. it .gt. maxit)then
      goto 23002
      endif
      call dscal(p,zero,d,1)
      if(k .le. p)then
      call daxpy(p,one,u(1,k),1,d,1)
      else
      k = k - p
      call daxpy(p,mone,u(1,k),1,d,1)
      endif
      call dgemv('N',n,p,one,a,n,d,1,zero,s,1)
      do23025 i = 1,n 
      call dcopy(p,x,1,xh,1)
      step = (b(i) - f(i))/s(i)
      call daxpy(p,step,d,1,xh,1)
      s(i) = pow(n,p,xh,a,b,c,tau)
23025 continue
23026 continue
      hin = idmin(n,s,1)
      if(inset(p,hin,h) .gt. 0)then
      nflag = 2
      goto 23002
      endif
      hout = h(k)
23001 goto 23000
23002 continue
      if(it .gt. maxit)then
      nflag = 1
      endif
      return
      end
      subroutine pivot(n,p,h,hin,hout,a,b,u,v,eflag)
      integer n,p,h(p),hin,hout,inset,k,eflag
      double precision a(n,p),b(p,p),u(p),v(p)
      double precision zero,one
      parameter(zero = 0.d0, one = 1.d0)
      eflag = 0
      k = inset(p,hout,h)
      if(k .eq. 0)then
      eflag = 1
      return
      endif
      if(inset(p,hin,h) .gt. 0)then
      eflag = 2
      return
      endif
      if(hin .lt. 1 .or. hin .gt. n)then
      eflag = 3
      return
      endif
      call dcopy(p,a(hin,1),n,v,1)
      call dgemv('T',p,p,one,b,p,v,1,zero,u,1)
      call dcopy(p,b(1,k),1,v,1)
      do23037 j = 1,p
      do23039 i = 1,p
      if(j .eq. k)then
      b(i,j) = b(i,j)/u(k)
      else
      b(i,j) = b(i,j) - (u(j)/u(k)) * v(i)
      endif
23039 continue
23040 continue
23037 continue
23038 continue
      h(k) = hin
      return
      end
      integer function inset(p,k,h)
      integer p,k,h(p)
      do23043 inset = 1,p
      if(h(inset) .eq. k)then
      return
      endif
23043 continue
23044 continue
      inset = 0
      return
      end
      double precision function pow(n,p,x,a,b,c,tau)
      integer n,p
      double precision x(p),a(n,p),b(n),c(n)
      double precision tau,u,zero,rho,fit,ddot
      parameter(zero= 0.d0)
      pow = zero
      do23047 i = 1,n
      fit = ddot(p,a(i,1),n,x,1)
      u = b(i) - min(fit,c(i))
      pow = pow + rho(u, tau)
23047 continue
23048 continue
      return
      end
      double precision function rho(u,tau)
      double precision u,tau,one
      parameter(one = 1.d0)
      if(u .lt. 0)then
      rho = u * (tau - one)
      else
      rho = u * tau
      endif
      return
      end

# Revised Version of Fitzenberger's Steepest Descent Algorithm for the Powell Estimator
#
# Roger Koenker:  last revision: 6 February 2008 
#
# Problem:  || min{Ax,c} - b ||_tau  = min!
#
subroutine powell(n,p,p2,A,b,c,x,tau,h,f,U,s,g,d,xh,maxit,nflag)

integer n,p,p2
double precision x(p),A(n,p),b(n),c(n)
double precision f(n),U(p,p),s(n),g(p2),d(p),xh(p)
double precision zero, one, mone, step, tau,pow
integer h(p),hin,hout,k,it,inset,maxit,nflag

PARAMETER(zero = 0.0d0, one = 1.d0, mone = -1.d0)

it = 0

repeat {
it = it + 1
if(it > 1) # Otherwise, assume U is A(h)^{-1}
	call pivot(n,p,h,hin,hout,A,U,d,xh,nflag)
if(nflag > 0)
	{nflag = nflag + 2; return}
do i = 1,p{
	xh(i) = b(h(i))
	}

call dgemv('N',p,p,one,U,p,xh,1,zero,x,1)
call dgemv('N',n,p,one,A,n,x,1,zero,f,1)
do i = 1,n{
	if(inset(p,i,h) > 0 | f(i) > c(i))
		s(i) = zero
	else if(b(i) < f(i))
		s(i) = one - tau
	else
		s(i) = - tau
	}
call dgemv('T',n,p,one,A,n,s,1,zero,xh,1)
call dgemv('T',p,p,one,U,p,xh,1,zero,g,1)
do i = 1,p {
	if(f(h(i)) < c(h(i))) 
		if(b(h(i)) < c(h(i)))
			g(i + p) = - g(i) + one - tau
		else
			g(i + p) = - g(i) - tau
	else
		g(i + p) = - g(i) + tau
	g(i) = g(i) + one - tau
	}
k = idmin(p2,g,1)
if(g(k) >= 0 | it > maxit)
	break
call dscal(p,zero,d,1)
if(k <= p)
	call daxpy(p,one,U(1,k),1,d,1)
else{
	k = k - p
	call daxpy(p,mone,U(1,k),1,d,1)
	}
call dgemv('N',n,p,one,A,n,d,1,zero,s,1)
do i = 1,n {
	call dcopy(p,x,1,xh,1)
	step = (b(i) - f(i))/s(i)
	call daxpy(p,step,d,1,xh,1)
	s(i) = pow(n,p,xh,A,b,c,tau)
	}
hin = idmin(n,s,1)
if(inset(p,hin,h) > 0) 
	{nflag = 2; break}
hout = h(k)
}
if(it > maxit) nflag = 1
return
end

#####################################################
subroutine pivot(n,p,h,hin,hout,A,B,u,v,eflag)

integer n,p,h(p),hin,hout,inset,k,eflag
double precision A(n,p),B(p,p),u(p),v(p)
double precision zero,one

PARAMETER(zero = 0.d0, one =  1.d0) 

eflag = 0
k = inset(p,hout,h)
if(k == 0)
	{eflag = 1; return}
if(inset(p,hin,h) > 0)
	{eflag = 2; return}
if(hin < 1 | hin > n)
	{eflag = 3; return}

call dcopy(p,A(hin,1),n,v,1)
call dgemv('T',p,p,one,B,p,v,1,zero,u,1)
call dcopy(p,B(1,k),1,v,1)
do j = 1,p{
	do i = 1,p{
		if(j == k)
			B(i,j) = B(i,j)/u(k)
		else
			B(i,j) = B(i,j) - (u(j)/u(k)) * v(i)
		}
	}
h(k) = hin
return
end


#####################################################
integer function inset(p,k,h)
integer p,k,h(p)

do inset = 1,p{
	if(h(inset) == k) return
	}
inset = 0
return
end

#####################################################
double precision function pow(n,p,x,A,b,c,tau)
integer n,p
double precision x(p),A(n,p),b(n),c(n)
double precision tau,u,zero,rho,fit,ddot

PARAMETER(zero= 0.d0) 

pow = zero
do i = 1,n{
	fit = ddot(p,A(i,1),n,x,1)
	u = b(i)  - min(fit,c(i))
	pow = pow + rho(u, tau)
	}
return
end
#####################################################
double precision function rho(u,tau)
double precision u,tau,one

PARAMETER(one = 1.d0) 

if(u < 0)
	rho = u * (tau - one)
else
	rho = u * tau
return
end

c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
      subroutine srqfn(n,m,nnza,a,ja,ia,ao,jao,iao,nnzdmax,d,jd,id,
     &                  dsub,jdsub,nnzemax,e,je,ie,nsubmax,lindx,xlindx,
     &                  nnzlmax,lnz,xlnz,iw,iwmax,iwork,xsuper,tmpmax,
     &                  tmpvec,wwm,wwn,cachsz,level,x,s,u,c,y,b,small,
     &                  ierr,maxit,timewd)
      integer nnza,m,n,nnzdmax,nnzemax,iwmax,
     &        nnzlmax,nsubmax,cachsz,level,tmpmax,ierr,maxit,
     &        ja(nnza),jao(nnza),jdsub(nnzemax+1),jd(nnzdmax),
     &        ia(n+1),iao(m+1),id(m+1),lindx(nsubmax),xlindx(m+1),
     &        iw(m,5),xlnz(m+1),iwork(iwmax),xsuper(m+1),je(nnzemax),
     &        ie(m+1)
      double precision small,
     &                 a(nnza),ao(nnza),dsub(nnzemax+1),d(nnzdmax),
     &                 lnz(nnzlmax),c(n),y(m),wwm(m,3),tmpvec(tmpmax),
     &                 wwn(n,14),x(n),s(n),u(n),e(nnzemax),b(m)
      double precision timewd(7)
      call slpfn(n,m,nnza,a,ja,ia,ao,jao,iao,nnzdmax,d,jd,id,
     &          dsub,jdsub,nsubmax,lindx,xlindx,nnzlmax,lnz,
     &          xlnz,iw(1,1),iw(1,2),iwmax,iwork,iw(1,3),iw(1,4),
     &          xsuper,iw(1,5),tmpmax,tmpvec,wwm(1,2),cachsz,
     &          level,x,s,u,c,y,b,wwn(1,1),wwn(1,2),wwn(1,3),
     &          wwn(1,4),nnzemax,e,je,ie,wwm(1,3),wwn(1,5),wwn(1,6),
     &          wwn(1,7),wwn(1,8),wwn(1,9),wwn(1,10),wwn(1,11),
     &          wwn(1,12),wwn(1,13),wwn(1,14),wwm(1,1),small,ierr,
     &          maxit,timewd)
      return
      end
      subroutine slpfn(n,m,nnza,a,ja,ia,ao,jao,iao,nnzdmax,d,jd,id,
     &                dsub,jdsub,nsubmax,lindx,xlindx,nnzlmax,lnz,
     &                xlnz,invp,perm,iwmax,iwork,colcnt,snode,xsuper,
     &                split,tmpmax,tmpvec,newrhs,cachsz,level,x,s,u,
     &                c,y,b,r,z,w,q,nnzemax,e,je,ie,dy,dx,ds,dz,dw,dxdz,
     &                dsdw,xi,xinv,sinv,ww1,ww2,small,ierr,maxit,timewd)
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c Sparse implentation of LMS's interior point method via 
c    Ng-Peyton's sparse Cholesky factorization for sparse 
c    symmetric positive definite
c INPUT:
c     n -- the number of row in the coefficient matrix A'
c     m -- the number of column in the coefficient matrix A'
c     nnza -- the number of non-zero elements in A'
c     a -- an nnza-vector of non-zero values of the design 
c          matrix (A') stored in csr format 
c     ja -- an nnza-vector of indices of the non-zero elements of
c           the coefficient matrix 
c     ia -- an (n+1)-vector of pointers to the begining of each
c           row in a and ja
c     ao -- an nnza-vector of work space for the transpose of
c           the design matrix stored in csr format or the 
c           design matrix stored in csc format
c     jao -- an nnza-vector of work space for the indices of the 
c            transpose of the design matrix 
c     iao -- an (n+1)-vector of pointers to the begining of each
c            column in ao and jao
c     nnzdmax -- upper bound of the non-zero elements in AA'
c     d -- an nnzdmax-vector of non-zero values of AQ^(-1)
c     jd -- an nnzdmax-vector of indices in d
c     id -- an (m+1)-vector of pointers to the begining of each
c           row in d and jd
c     dsub -- the values of e excluding the diagonal elements
c     jdsub -- the indices to dsub
c     nsubmax -- upper bound of the dimension of lindx
c     lindx -- an nsub-vector of interger which contains, in 
c           column major order, the row subscripts of the nonzero
c           entries in L in a compressed storage format
c     xlindx -- an (m+1)-vector of integer of pointers for lindx
c     nnzlmax -- the upper bound of the non-zero entries in 
c                L stored in lnz, including the diagonal entries
c     lnz -- First contains the non-zero entries of d; later
c            contains the entries of the Cholesky factor
c     xlnz -- column pointer for L stored in lnz
c     invp -- an n-vector of integer of inverse permutation 
c             vector
c     perm -- an n-vector of integer of permutation vector
c     iw -- integer work array of length m
c     iwmax -- upper bound of the general purpose integer 
c              working storage iwork; set at 7*m+3
c     iwork -- an iwsiz-vector of integer as work space
c     colcnt -- array of length m, containing the number of 
c               non-zeros in each column of the factor, including
c               the diagonal entries
c     snode -- array of length m for recording supernode 
c              membership
c     xsuper -- array of length m+1 containing the supernode
c               partitioning
c     split -- an m-vector with splitting of supernodes so that 
c              they fit into cache
c     tmpmax -- upper bound of the dimension of tmpvec
c     tmpvec -- a tmpmax-vector of temporary vector
c     newrhs -- extra work vector for right-hand side and 
c               solution
c     cachsz -- size of the cache (in kilobytes) on the target 
c               machine
c     level -- level of loop unrolling while performing numerical
c              factorization
c     x -- an n-vector, the initial feasible solution in the primal
c              that corresponds to the design matrix A'
c     s -- an n-vector 
c     u -- an n-vector of upper bound for x
c     c -- an n-vector, usually the "negative" of
c          the pseudo response
c     y -- an m-vector, the initial dual solution 
c     b -- an n-vector, the rhs of the equality constraint
c          usually X'a = (1-tau)X'e in the rq setting
c     r -- an n-vector of residuals
c     z -- an n-vector of the dual slack variable
c     w -- an n-vector 
c     q -- an n-vector of work array containing the diagonal 
c          elements of the Q^(-1) matrix
c     nnzemax -- upper bound of the non-zero elements in AA'
c     e -- an nnzdmax-vector containing the non-zero entries of
c          AQ^(-1)A' stored in csr format
c     je -- an nnzemax-vector of indices for e
c     ie -- an (m+1)-vector of pointers to the begining of each
c           row in e and je
c     dy -- work array
c     dx -- work array
c     ds -- work array
c     dz -- work array
c     dw -- work array
c     dxdz -- work array
c     dsdw -- work arry
c     xi -- work array
c     xinv -- work array
c     sinv -- work array
c     ww1 -- work array
c     ww2 -- work array
c     small -- convergence tolerance for inetrior algorithm
c     ierr -- error flag
c       1 -- insufficient work space in call to extract
c       2 -- nnzd > nnzdmax
c       3 -- insufficient storage in iwork when calling ordmmd;
c       4 -- insufficient storage in iwork when calling sfinit;
c       5 -- nnzl > nnzlmax when calling sfinit
c       6 -- nsub > nsubmax when calling sfinit
c       7 -- insufficient work space in iwork when calling symfct
c       8 -- inconsistancy in input when calling symfct
c       9 -- tmpsiz > tmpmax when calling bfinit; increase tmpmax
c       10 -- nonpositive diagonal encountered when calling 
c            blkfct, the matrix is not positive definite
c       11 -- insufficient work storage in tmpvec when calling 
c            blkfct
c       12 -- insufficient work storage in iwork when calling 
c            blkfct
c     maxit -- upper limit of the iteration; on return holds the
c              number of iterations
c     timew -- amount of time to execute this subroutine
c OUTPUT:
c     y -- an m-vector of primal solution
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
      integer nnza,m,n,nsuper,nnzdmax,nnzemax,iwmax,nnzd,
     &        nnzlmax,nsubmax,cachsz,level,tmpmax,ierr,maxit,it,
     &        ja(nnza),jao(nnza),jdsub(nnzemax+1),jd(nnzdmax),
     &        ia(n+1),iao(m+1),id(m+1),lindx(nsubmax),xlindx(m+1),
     &        invp(m),perm(m),xlnz(m+1),iwork(iwmax),
     &        colcnt(m),snode(m),xsuper(m+1),split(m),je(nnzemax),
     &        ie(m+1)
      double precision ddot,gap,zero,one,beta,small,deltap,deltad,mu,g,
     &                 a(nnza),ao(nnza),dsub(nnzemax+1),d(nnzdmax),
     &                 lnz(nnzlmax),c(n),b(m),newrhs(m),y(m),
     &                 tmpvec(tmpmax),r(n),z(n),w(n),x(n),s(n),
     &                 u(n),q(n),
     &                 e(nnzemax),dy(m),dx(n),ds(n),dz(n),dw(n),
     &                 dxdz(n),dsdw(n),xinv(n),sinv(n),xi(n),
     &                 ww1(n),ww2(m)
      double precision timewd(7)
      real gtimer,timbeg,timend
      external smxpy1,smxpy2,smxpy4,smxpy8
      external mmpy1,mmpy2,mmpy4,mmpy8
      parameter (beta=9.995d-1, one=1.0d0, zero=0.0d0)
      do i = 1,7
         timewd(i) = 0.0
      enddo
      it = 0
      nnzd = ie(m+1) - 1
      nnzdsub = nnzd - m
c
c  Compute the initial gap
c
       gap = ddot(n,z,1,x,1) + ddot(n,w,1,s,1)
c
c  Start iteration
c
   20 continue
      if(gap .lt. small .or. it .gt. maxit) goto 30
      it = it + 1
c
c  Create the diagonal matrix Q^(-1) stored in q as an n-vector
c  and update the residuals in r
c
      do i=1,n
         q(i) = one/(z(i)/x(i)+w(i)/s(i))
         r(i) = z(i) - w(i)
      enddo
c
c  Obtain AQ^(-1) and store in d,jd,id in csr format
c
      call amudia(m,1,ao,jao,iao,q,d,jd,id)
c
c  Obtain AQ^(-1)A' and store in e,je,ie in csr format
c
      call amub(m,m,1,d,jd,id,a,ja,ia,e,je,ie,nnzemax,iwork,ierr)
      if (ierr .ne. 0) then
         ierr = 2
         go to 100
      endif
c
c  Extract the non-diagonal structure of e,je,ie and store in dsub,jdsub
c
      call extract(e,je,ie,dsub,jdsub,m,nnzemax,nnzemax+1,ierr)
      if (ierr .ne. 0) then
         ierr = 1
         go to 100
      endif
c
c Compute b - Ax + AQ^(-1)r and store it in c in two steps
c  First: store Ax in ww2
      call amux(m,x,ww2,ao,jao,iao)
c
c  Second: save AQ^(-1)r in c temporarily
c
      call amux(m,r,c,d,jd,id)
      do i = 1,m
         c(i) = b(i) - ww2(i) + c(i)
      enddo
c
c  Compute dy = (AQ^(-1)A')^(-1)(b-Ax+AQ^(-1)r); result returned via dy
c
c Call chlfct to perform Cholesky's decomposition of e,je,ie
c
      call chlfct(m,xlindx,lindx,invp,perm,iwork,nnzdsub,jdsub,
     &            colcnt,nsuper,snode,xsuper,nnzlmax,nsubmax,xlnz,lnz,
     &            ie,je,e,cachsz,tmpmax,level,tmpvec,split,ierr,it,
     &            timewd)
      if (ierr .ne. 0) go to 100
c
c Call blkslv: Numerical solution for the new rhs stored in b
c
      do i = 1,m
         newrhs(i) = c(perm(i))
      enddo
      timbeg = gtimer()
      call blkslv(nsuper,xsuper,xlindx,lindx,xlnz,lnz,newrhs)
      timend = gtimer()
      timewd(7) = timewd(7) + timend - timbeg
      do i = 1,m
         dy(i) = newrhs(invp(i))
      enddo
c
c  Compute dx = Q^(-1)(A'dy - r), ds = -dx, dz  and dw
c
      call amux(n,dy,dx,a,ja,ia)
      do i=1,n
         dx(i) = q(i) * (dx(i) - r(i))
         ds(i) = -dx(i)
         dz(i) = -z(i) * (one + dx(i) / x(i))
         dw(i) = -w(i) * (one + ds(i) / s(i))
      enddo
c
c Compute the maximum allowable step lengths
c
      call bound(x,dx,s,ds,z,dz,w,dw,n,beta,deltap,deltad)
      if (deltap * deltad .lt. one) then
c
c Update mu
c
         mu = ddot(n,z,1,x,1) + ddot(n,w,1,s,1)
         g = ddot(n,z,1,x,1) + deltap*ddot(n,z,1,dx,1)
     &       + deltad*ddot(n,dz,1,x,1) + deltad*deltap*ddot(n,dz,1,dx,1)
     &       + ddot(n,w,1,s,1) + deltap*ddot(n,w,1,ds,1)
     &       + deltad*ddot(n,dw,1,s,1) + deltad*deltap*ddot(n,dw,1,ds,1)
         mu = mu*((g/mu)**3)/(2.d0*dfloat(n))
c
c Compute dxdz and dsdw
c
         do i = 1,n
             dxdz(i) = dx(i)*dz(i)
             dsdw(i) = ds(i)*dw(i)
             xinv(i) = one/x(i)
             sinv(i) = one/s(i)
             xi(i) = xinv(i) * dxdz(i) - sinv(i) * dsdw(i)
     &               - mu * (xinv(i) - sinv(i))
             ww1(i) = q(i) * xi(i)
         enddo
c
c Compute AQ^(-1)(dxdz - dsdw - mu(X^(-1) - S^(-1))) and
c store it in ww2 temporarily
c
         call amux(m,ww1,ww2,ao,jao,iao)
         do i = 1,m
            c(i) = c(i) + ww2(i)
         enddo
c
c
c Compute dy and return the result in dy
c
c Call blkslv: Numerical solution for the new rhs stored in b
c
      do i = 1,m
         newrhs(i) = c(perm(i))
      enddo
      timbeg = gtimer()
      call blkslv(nsuper,xsuper,xlindx,lindx,xlnz,lnz,newrhs)
      timend = gtimer()
      timewd(7) = timewd(7) + timend - timbeg
      do i = 1,m
         dy(i) = newrhs(invp(i))
      enddo
c
c  Compute dx = Q^(-1)(A'dy - r + mu(X^(-1) - S^(-1)) -dxdz + dsdw),
c  ds = -dx, dz  and dw
c
         call amux(n,dy,dx,a,ja,ia)
         do i=1,n
            dx(i) = q(i) * (dx(i) - xi(i) - r(i))
            ds(i) = -dx(i)
            dz(i) = -z(i) + xinv(i)*(mu - z(i)*dx(i) - dxdz(i))
            dw(i) = -w(i) + sinv(i)*(mu - w(i)*ds(i) - dsdw(i))
         enddo
c
c Compute the maximum allowable step lengths
c
         call bound(x,dx,s,ds,z,dz,w,dw,n,beta,deltap,deltad)
      endif
c
c Take the step
c
      call daxpy(n,deltap,dx,1,x,1)
      call daxpy(n,deltap,ds,1,s,1)
      call daxpy(n,deltad,dw,1,w,1)
      call daxpy(n,deltad,dz,1,z,1)
      call daxpy(m,deltad,dy,1,y,1)
      gap = ddot(n,z,1,x,1) + ddot(n,w,1,s,1)
      goto 20
   30 continue
  100 continue
      maxit = it
      return
      end

C Output from Public domain Ratfor, version 1.05
      subroutine rqfnb(n,p,a,y,rhs,d,u,beta,eps,wn,wp,nit,info)
      integer n,p,info,nit(3)
      double precision a(p,n),y(n),rhs(p),d(n),u(n),wn(n,9),wp(p,p+3)
      double precision one,beta,eps
      parameter( one = 1.0d0)
      call lpfnb(n,p,a,y,rhs,d,u,beta,eps,wn(1,1),wn(1,2), wp(1,1),wn(1,
     *3),wn(1,4),wn(1,5),wn(1,6), wp(1,2),wn(1,7),wn(1,8),wn(1,9),wp(1,3
     *),wp(1,4),nit,info)
      return
      end
      subroutine lpfnb(n,p,a,c,b,d,u,beta,eps,x,s,y,z,w, dx,ds,dy,dz,dw,
     *dr,rhs,ada,nit,info)
      integer n,p,pp,i,info,nit(3),maxit
      double precision a(p,n),c(n),b(p)
      double precision zero,one,mone,big,ddot,dmax1,dmin1,dxdz,dsdw
      double precision deltap,deltad,beta,eps,mu,gap,g
      double precision x(n),u(n),s(n),y(p),z(n),w(n),d(n),rhs(p),ada(p,p
     *)
      double precision dx(n),ds(n),dy(p),dz(n),dw(n),dr(n)
      parameter( zero = 0.0d0)
      parameter( one = 1.0d0)
      parameter( mone = -1.0d0)
      parameter( big = 1.0d+20)
      parameter( maxit = 500)
      nit(1)=0
      nit(2)=0
      nit(3)=n
      pp=p*p
      call dgemv('N',p,n,one,a,p,c,1,zero,y,1)
      do23000 i=1,n
      d(i)=one
23000 continue
23001 continue
      call stepy(n,p,a,d,y,ada,info)
      if(info .ne. 0)then
      return
      endif
      call dcopy(n,c,1,s,1)
      call dgemv('T',p,n,mone,a,p,y,1,one,s,1)
      do23004 i=1,n
      if(dabs(s(i)).lt.eps)then
      z(i)=dmax1(s(i), zero) + eps
      w(i)=dmax1(-s(i),zero) + eps
      else
      z(i)=dmax1(s(i), zero)
      w(i)=dmax1(-s(i),zero)
      endif
      s(i)=u(i)-x(i)
23004 continue
23005 continue
      gap = ddot(n,z,1,x,1)+ddot(n,w,1,s,1)
23008 if(gap .gt. eps .and. nit(1).lt.maxit)then
      nit(1)=nit(1)+1
      do23010 i = 1,n
      d(i) = one/(z(i)/x(i) + w(i)/s(i))
      ds(i)=z(i)-w(i)
      dz(i)=d(i)*ds(i)
23010 continue
23011 continue
      call dcopy(p,b,1,dy,1)
      call dgemv('N',p,n,mone,a,p,x,1,one,dy,1)
      call dgemv('N',p,n,one,a,p,dz,1,one,dy,1)
      call dcopy(p,dy,1,rhs,1)
      call stepy(n,p,a,d,dy,ada,info)
      if(info .ne. 0)then
      return
      endif
      call dgemv('T',p,n,one,a,p,dy,1,mone,ds,1)
      deltap=big
      deltad=big
      do23014 i=1,n
      dx(i)=d(i)*ds(i)
      ds(i)=-dx(i)
      dz(i)=-z(i)*(dx(i)/x(i) + one)
      dw(i)=-w(i)*(ds(i)/s(i) + one)
      if(dx(i).lt.0)then
      deltap=dmin1(deltap,-x(i)/dx(i))
      endif
      if(ds(i).lt.0)then
      deltap=dmin1(deltap,-s(i)/ds(i))
      endif
      if(dz(i).lt.0)then
      deltad=dmin1(deltad,-z(i)/dz(i))
      endif
      if(dw(i).lt.0)then
      deltad=dmin1(deltad,-w(i)/dw(i))
      endif
23014 continue
23015 continue
      deltap=dmin1(beta*deltap,one)
      deltad=dmin1(beta*deltad,one)
      if(min(deltap,deltad) .lt. one)then
      nit(2)=nit(2)+1
      mu = ddot(n,x,1,z,1)+ddot(n,s,1,w,1)
      g = mu + deltap*ddot(n,dx,1,z,1)+ deltad*ddot(n,dz,1,x,1) + deltap
     **deltad*ddot(n,dz,1,dx,1)+ deltap*ddot(n,ds,1,w,1)+ deltad*ddot(n,
     *dw,1,s,1) + deltap*deltad*ddot(n,ds,1,dw,1)
      mu = mu * ((g/mu)**3) /dfloat(2*n)
      do23026 i=1,n
      dr(i)=d(i)*(mu*(1/s(i)-1/x(i))+ dx(i)*dz(i)/x(i)-ds(i)*dw(i)/s(i))
23026 continue
23027 continue
      call dswap(p,rhs,1,dy,1)
      call dgemv('N',p,n,one,a,p,dr,1,one,dy,1)
      call dpotrs('U',p,1,ada,p,dy,p,info)
      call dgemv('T',p,n,one,a,p,dy,1,zero,u,1)
      deltap=big
      deltad=big
      do23028 i=1,n
      dxdz = dx(i)*dz(i)
      dsdw = ds(i)*dw(i)
      dx(i)= d(i)*(u(i)-z(i)+w(i))-dr(i)
      ds(i)= -dx(i)
      dz(i)= -z(i)+(mu - z(i)*dx(i) - dxdz)/x(i)
      dw(i)= -w(i)+(mu - w(i)*ds(i) - dsdw)/s(i)
      if(dx(i).lt.0)then
      deltap=dmin1(deltap,-x(i)/dx(i))
      endif
      if(ds(i).lt.0)then
      deltap=dmin1(deltap,-s(i)/ds(i))
      endif
      if(dz(i).lt.0)then
      deltad=dmin1(deltad,-z(i)/dz(i))
      endif
      if(dw(i).lt.0)then
      deltad=dmin1(deltad,-w(i)/dw(i))
      endif
23028 continue
23029 continue
      deltap=dmin1(beta*deltap,one)
      deltad=dmin1(beta*deltad,one)
      endif
      call daxpy(n,deltap,dx,1,x,1)
      call daxpy(n,deltap,ds,1,s,1)
      call daxpy(p,deltad,dy,1,y,1)
      call daxpy(n,deltad,dz,1,z,1)
      call daxpy(n,deltad,dw,1,w,1)
      gap = ddot(n,z,1,x,1)+ddot(n,w,1,s,1)
      goto 23008
      endif
23009 continue
      call daxpy(n,mone,w,1,z,1)
      call dswap(n,z,1,x,1)
      return
      end
      subroutine stepy(n,p,a,d,b,ada,info)
      integer n,p,pp,i,info
      double precision a(p,n),b(p),d(n),ada(p,p),zero
      parameter( zero = 0.0d0)
      pp=p*p
      do23038 j=1,p
      do23040 k=1,p
      ada(j,k)=zero
23040 continue
23041 continue
23038 continue
23039 continue
      do23042 i=1,n
      call dsyr('U',p,d(i),a(1,i),1,ada,p)
23042 continue
23043 continue
      call dposv('U',p,1,ada,p,b,p,info)
      return
      end

c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
      subroutine srqfnc(n1,m,nnza1,a1,ja1,ia1,ao1,jao1,iao1,n2,nnza2,
     &                  a2,ja2,ia2,ao2,jao2,iao2,nnzdmax,d,jd,id,
     &                  dsub,jdsub,nnzemax,e,je,ie,nnzgmax,g,jg,ig,
     &                  nnzhmax,h,jh,ih,nsubmax,lindx,xlindx,nnzlmax,
     &                  lnz,xlnz,iw,iwmax,iwork,xsuper,tmpmax,tmpvec,
     &                  maxn1n2,ww1,wwm,wwn1,wwn2,cachsz,level,x1,x2,
     &                  s,u,c1,c2,y,small,ierr,maxit,timewd)
      integer nnza1,nnza2,m,n1,n2,nnzdmax,nnzemax,nnzgmax,nnzhmax,iwmax,
     &        nnzlmax,nsubmax,cachsz,level,tmpmax,ierr,maxit,maxn1n2,
     &        ja1(nnza1),jao1(nnza1),ja2(nnza2),jao2(nnza2),
     &        jdsub(nnzhmax+1),jd(nnzdmax),ia1(n1+1),iao1(m+1),
     &        ia2(n2+1),iao2(m+1),id(m+1),lindx(nsubmax),xlindx(m+1),
     &        iw(m,5),xlnz(m+1),iwork(iwmax),xsuper(m+1),je(nnzemax),
     &        ie(m+1),jg(nnzgmax),ig(m+1),jh(nnzhmax),ih(m+1)
      double precision small,
     &                 a1(nnza1),ao1(nnza1),a2(nnza2),ao2(nnza2),
     &                 dsub(nnzhmax+1),d(nnzdmax),g(nnzgmax),
     &                 h(nnzhmax),lnz(nnzlmax),c1(n1),c2(n2),y(m),
     &                 ww1(maxn1n2),wwm(m,6),tmpvec(tmpmax),
     &                 wwn1(n1,10),wwn2(n2,7),x1(n1),x2(n2),s(n1),
     &                 u(n1),e(nnzemax)
      double precision timewd(7)
      parameter (beta=9.995d-1, one=1.0d0, zero=0.0d0)
      call slpfnc(n1,m,nnza1,a1,ja1,ia1,ao1,jao1,iao1,n2,nnza2,
     &          a2,ja2,ia2,ao2,jao2,iao2,nnzdmax,d,jd,id,dsub,
     &          jdsub,nsubmax,lindx,xlindx,nnzlmax,lnz,xlnz,iw(1,1),
     &          iw(1,2),iwmax,iwork,iw(1,3),iw(1,4),xsuper,iw(1,5),
     &          tmpmax,tmpvec,wwm(1,2),wwm(1,3),cachsz,level,x1,x2,s,u,
     &          c1,c2,y,wwm(1,1),wwn2(1,1),wwn1(1,1),
     &          wwn2(1,2),wwn1(1,2),wwn1(1,3),wwn2(1,3),nnzemax,e,je,
     &          ie,nnzgmax,g,jg,ig,nnzhmax,h,jh,ih,wwm(1,4),wwn1(1,4),
     &          wwn2(1,4),wwn1(1,5),wwn1(1,6),wwn2(1,5),wwn1(1,7),
     &          wwn1(1,8),wwn2(1,6),wwn1(1,9),wwn1(1,10),wwn2(1,7),
     &          maxn1n2,ww1,wwm(1,5),wwm(1,6),small,ierr,maxit,timewd)
      return
      end
c23456789012345678901234567890123456789012345678901234567890123456789012
      subroutine slpfnc(n1,m,nnza1,a1,ja1,ia1,ao1,jao1,iao1,n2,nnza2,
     &                a2,ja2,ia2,ao2,jao2,iao2,nnzdmax,d,jd,id,dsub,
     &                jdsub,nsubmax,lindx,xlindx,nnzlmax,lnz,xlnz,invp,
     &                perm,iwmax,iwork,colcnt,snode,xsuper,split,
     &                tmpmax,tmpvec,rhs,newrhs,cachsz,level,x1,x2,s,u,
     &                c1,c2,y,b,r2,z1,
     &                z2,w,q1,q2,nnzemax,e,je,
     &                ie,nnzgmax,g,jg,ig,nnzhmax,h,jh,ih,dy,dx1,
     &                dx2,ds,dz1,dz2,dw,
     &                dxdz1,dxdz2,dsdw,xi1,xi2,
     &                maxn1n2,ww1,ww2,ww3,small,ierr,maxit,timewd)
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c Sparse implentation of LMS's interior point method via 
c    Ng-Peyton's sparse Cholesky factorization for sparse 
c    symmetric positive definite
c INPUT:
c     n1 -- the number of row in the coefficient matrix A1'
c     m -- the number of column in the coefficient matrix A1'
c     nnza1 -- the number of non-zero elements in A'
c     a1 -- an nnza1-vector of non-zero values of the design 
c          matrix (A1') stored in csr format 
c     ja1 -- an nnza1-vector of indices of the non-zero elements of
c           the coefficient matrix 
c     ia1 -- an (n1+1)-vector of pointers to the begining of each
c           row in a1 and ja1
c     ao1 -- an nnza1-vector of work space for the transpose of
c           the design matrix stored in csr format or the 
c           design matrix stored in csc format
c     jao1 -- an nnza1-vector of work space for the indices of the 
c            transpose of the design matrix 
c     iao1 -- an (n1+1)-vector of pointers to the begining of each
c            column in ao1 and jao1
c     n2 -- the number of row in the constraint matrix A2'
c     nnza2 -- the number of non-zero elements in A2' 
c     a2 -- an nnza2-vector of non-zero values of the contraint 
c          matrix (A2') stored in csr format 
c     ja2 -- an nnza2-vector of indices of the non-zero elements of
c           the constraint matrix 
c     ia2 -- an (n2+1)-vector of pointers to the begining of each
c           row in a2 and ja2
c     ao2 -- an nnza2-vector of work space for the transpose of
c           the constraint matrix stored in csr format or the 
c           constraint matrix stored in csc format
c     jao2 -- an nnza2-vector of work space for the indices of the 
c            transpose of the constraint matrix 
c     iao2 -- an (n2+1)-vector of pointers to the begining of each
c            column in ao2 and jao2
c     nnzdmax -- upper bound of the non-zero elements in A1A1'
c     d -- an nnzdmax-vector of non-zero values used to store
c          the transpose of the design matrix multiplied by the design 
c          matrix (A1A1') stored in csr format; 
c          also used to store A1Q1^(-1) and A2Q2^(-1) later
c     jd -- an nnzdmax-vector of indices in d
c     id -- an (m+1)-vector of pointers to the begining of each
c           row in d and jd
c     dsub -- the values of d excluding the diagonal elements
c     jdsub -- the indices to dsub
c     nsubmax -- upper bound of the dimension of lindx
c     lindx -- an nsub-vector of interger which contains, in 
c           column major order, the row subscripts of the nonzero
c           entries in L in a compressed storage format
c     xlindx -- an (m+1)-vector of integer of pointers for lindx
c     nnzlmax -- the upper bound of the non-zero entries in 
c                L stored in lnz, including the diagonal entries
c     lnz -- First contains the non-zero entries of d; later
c            contains the entries of the Cholesky factor
c     xlnz -- column pointer for L stored in lnz
c     invp -- an n1-vector of integer of inverse permutation 
c             vector
c     perm -- an n1-vector of integer of permutation vector
c     iw -- integer work array of length m
c     iwmax -- upper bound of the general purpose integer 
c              working storage iwork; set at 7*m+3
c     iwork -- an iwsiz-vector of integer as work space
c     colcnt -- array of length m, containing the number of 
c               non-zeros in each column of the factor, including
c               the diagonal entries
c     snode -- array of length m for recording supernode 
c              membership
c     xsuper -- array of length m+1 containing the supernode
c               partitioning
c     split -- an m-vector with splitting of supernodes so that 
c              they fit into cache
c     tmpmax -- upper bound of the dimension of tmpvec
c     tmpvec -- a tmpmax-vector of temporary vector
c     rhs -- m-vector to store the rhs
c     newrhs -- extra work vector for right-hand side and 
c               solution
c     cachsz -- size of the cache (in kilobytes) on the target 
c               machine
c     level -- level of loop unrolling while performing numerical
c              factorization
c     x1 -- an n1-vector, the initial feasible solution for the primal 
c           solution that corresponds to the design matrix A1'
c     x2 -- an n2-vector, the initial feasible solution for the primal 
c           solution that corresponds to the constraint matrix A2'
c     s -- an n1-vector 
c     u -- an n1-vector of the upper bound for x1
c     c1 -- an n1-vector in the primal; negative response in the 
c           regression quantile setting
c     c2 -- an n2-vector, the negative rhs of the inequality constraint
c     y -- an m-vector, the initial dual solution 
c     b -- an n1-vector, usualy the rhs of the equality constraint
c          X'a = (1-tau)X'e in the rq setting
c     r2 -- an n2-vector of residuals
c     z1 -- an n1-vector of the dual slack variable
c     z2 -- an n2-vector 
c     w -- an n-vector 
c     q1 -- an n1-vector of work array containing the diagonal 
c          elements of the Q1^(-1) matrix
c     q2 -- an n2-vector of work array containing the diagonal 
c          elements of the Q2^(-1) matrix
c     e -- an nnzdmax-vector containing the non-zero entries of
c          A1Q1^(-1)A1' stored in csr format
c     je -- an nnzdmax-vector of indices for e
c     ie -- an (m+1)-vector of pointers to the begining of each
c           row in e and je
c     nnzgmax -- upper bound of the non-zero elements in g,jg
c     g -- an nnzgmax-vector containing the non-zero entries of
c          A2Q2^(-1)A2' stored in csr format
c     jg -- an nnzgmax-vector of indices for g
c     ig -- an (m+1)-vector of pointers to the begining of each
c           row in g and jg
c     nnzhmax -- upper bound of the non-zero elements in h,jh
c     h -- an nnzhmax-vector containing the non-zero entries of
c          AQ^(-1)A' stored in csr format
c     jh -- an nnzhmax-vector of indices for h
c     ih -- an (m+1)-vector of pointers to the begining of each
c           row in h and jh
c     dy -- an m-vector of work array
c     dx1 -- an n1-vector of work array
c     dx2 -- an n2-vector of work array
c     ds -- an n1-vector of work array
c     dz1 -- an n1-vector of work array
c     dz2 -- an n2-vector of work array
c     dw -- an n1-vector of work array
c     dxdz1 -- an n1-vector of work array
c     dxdz2 -- an n2-vector of work array
c     dsdw -- an n1-vector of work arry
c     xi1 -- an n1-vector of work array
c     xi2 -- an n2-vector of work array
c     xinv1 -- an n1-vector of work array
c     xinv2 -- an n2-vector of work array
c     sinv -- work array
c     maxn1n2 -- max(n1,n2)
c     ww1 -- an maxn1n2-vector of work array
c     ww2 -- an m-vector of work array
c     ww3 -- an m-vector of work array
c     small -- convergence tolerance for inetrior algorithm
c     ierr -- error flag
c       1 -- insufficient storage when calling extract;
c       3 -- insufficient storage in iwork when calling ordmmd;
c       4 -- insufficient storage in iwork when calling sfinit;
c       5 -- nnzl > nnzlmax when calling sfinit
c       6 -- nsub > nsubmax when calling sfinit
c       7 -- insufficient work space in iwork when calling symfct
c       8 -- inconsistancy in input when calling symfct
c       9 -- tmpsiz > tmpmax when calling symfct; increase tmpmax
c       10 -- nonpositive diagonal encountered when calling 
c            blkfct
c       11 -- insufficient work storage in tmpvec when calling 
c            blkfct
c       12 -- insufficient work storage in iwork when calling 
c            blkfct
c       13 -- nnzd > nnzdmax in e,je when calling amub
c       14 -- nnzd > nnzdmax in g,jg when calling amub
c       15 -- nnzd > nnzdmax in h,jh when calling aplb
c     maxit -- upper limit of the iteration; on return holds the
c              number of iterations
c     timewd -- amount of time to execute this subroutine
c OUTPUT:
c     y -- an m-vector of primal solution
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
      integer nnza1,nnza2,m,n1,n2,maxn1n2,nsuper,nnzdmax,nnzemax,
     &        nnzgmax,
     &        nnzhmax,iwmax,nnzlmax,nsubmax,cachsz,level,tmpmax,ierr,
     &        maxit,it,nnzdsub,nnzd,
     &        ja1(nnza1),jao1(nnza1),ja2(nnza2),jao2(nnza2),
     &        jdsub(nnzhmax+1),jd(nnzdmax),ia1(n1+1),iao1(m+1),
     &        ia2(n2+1),iao2(m+1),id(m+1),lindx(nsubmax),xlindx(m+1),
     &        invp(m),perm(m),xlnz(m+1),iwork(iwmax),colcnt(m),
     &        snode(m),xsuper(m+1),split(m),je(nnzemax),ie(m+1),
     &        jg(nnzgmax),ig(m+1),jh(nnzhmax),ih(m+1)
      double precision ddot,gap,zero,one,beta,small,deltap,deltad,mu,g1,
     &                 a1(nnza1),ao1(nnza1),a2(nnza2),ao2(nnza2),
     &                 dsub(nnzhmax+1),d(nnzdmax),lnz(nnzlmax),c1(n1),
     &                 c2(n2),b(m),rhs(m),newrhs(m),y(m),tmpvec(tmpmax),
     &                 r2(n2),z1(n1),z2(n2),w(n1),x1(n1),
     &                 x2(n2),s(n1),u(n1),q1(n1),q2(n2),e(nnzemax),
     &                 g(nnzgmax),h(nnzhmax),dy(m),dx1(n1),dx2(n2),
     &                 ds(n1),dz1(n1),dz2(n2),dw(n1),dxdz1(n1),
     &                 dxdz2(n2),dsdw(n1),
     &                 xi1(n1),xi2(n2),ww1(maxn1n2),ww2(m),ww3(m)
      double precision timewd(7)
      real gtimer,timbeg,timend
      external smxpy1,smxpy2,smxpy4,smxpy8
      external mmpy1,mmpy2,mmpy4,mmpy8
      parameter (beta=9.995d-1, one=1.0d0, zero=0.0d0)
      do i = 1,7
         timewd(i) = 0.0
      enddo
      it = 0
c
c  Compute the initial gap
c
       gap = ddot(n1,z1,1,x1,1) + ddot(n2,z2,1,x2,1) + ddot(n1,w,1,s,1)
c
c  Start iteration
c
   20 continue
      if(gap .lt. small .or. it .gt. maxit) goto 30
         it = it + 1
c
c  Create the diagonal matrix Q1^(-1) stored in q1 as an n1-vector,
c  the diagonal matrix Q2^(-1) stored in q2 as an n2-vector,
c  and store the residuals in r1 in ds, and r3 in dy temporarily,
c  and r2 in r2 permanently
c
c    Call amux to obtain A1x1 and store the value in ww2 
c
         call amux(m,x1,ww2,ao1,jao1,iao1)
c
c    Call amux to obtain A2x2 and store the value in ww3
c
         call amux(m,x2,ww3,ao2,jao2,iao2)
c
c    Store A2'y temporarily in r2
c
         call amux(n2,y,r2,a2,ja2,ia2)
         do i=1,n1
            q1(i) = one/(z1(i)/x1(i)+w(i)/s(i))
            ds(i) = z1(i) - w(i)
         enddo
         do i=1,n2
            q2(i) = x2(i)/z2(i)
            r2(i) = c2(i)-r2(i)
         enddo
         do i=1,m
            dy(i) = b(i) - ww2(i) - ww3(i)
         enddo
c
c  Obtain AQA = A1Q1^(-1)A1' + A2Q2^(-1)A2' in 5 steps
c
c  Step1: Obtain A1Q1^(-1) and store the values in d,jd,id in csr format
c         Also compute A1Q1^(-1)r1 and store the values in ww2 to be used 
c         to generate r3;
c  Step2: Compute A1Q1^(-1)A1' and store the values in e,je,ie 
c  Step3: Obtain A2Q2^(-1) and store the values in d,jd,id in csr format
c         Also compute A2Q2^(-1)r2 and store the values in in ww3 to 
c         be used to generate r3;
c  Step4: Compute A2Q2^(-1)A2' and store the value in g,jg,ig 
c  Step5: Compute AQA and store the values in h,jh,ih
c
c    Step 1
c
         call amudia(m,1,ao1,jao1,iao1,q1,d,jd,id)
         call amux(m,ds,ww2,d,jd,id)
c
c   Step 2
c
         call amub(m,m,1,d,jd,id,a1,ja1,ia1,e,je,ie,nnzemax,iwork,ierr)
         if (ierr .ne. 0) then
            ierr = 13
            go to 100
         endif
c
c   Step 3
c
         call amudia(m,1,ao2,jao2,iao2,q2,d,jd,id)
         call amux(m,r2,ww3,d,jd,id)
c
c   Step 4
c
         call amub(m,m,1,d,jd,id,a2,ja2,ia2,g,jg,ig,nnzgmax,iwork,ierr)
         if (ierr .ne. 0) then
            ierr = 14
            go to 100
         endif
c
c   Step 5
c
         call aplb(m,m,1,e,je,ie,g,jg,ig,h,jh,ih,nnzhmax,iwork,ierr)
         if (ierr .ne. 0) then
            ierr = 15
            go to 100
         endif
c
c  Generate rhs = r3 + A1Q1^(-1)r1 + A2Q2^(-1)r2 and store in rhs
c
         do i = 1,m
            rhs(i) = dy(i) + ww2(i) + ww3(i)
         enddo
c
c  Extract the non-diagonal structure of h,jh,ih and store in dsub,jdsub
c
         nnzd = ih(m+1) - 1
         nnzdsub = nnzd - m
         call extract(h,jh,ih,dsub,jdsub,m,nnzhmax,nnzhmax+1,ierr)
         if (ierr .ne. 0) then
            ierr = 1
            go to 100
         endif
c
c  Compute dy = (AQ^(-1)A')^(-1)rhs; result returned via dy
c
c Call chlfct to perform Cholesky's decomposition of h,jh,ih
c
         call chlfct(m,xlindx,lindx,invp,perm,iwork,nnzdsub,jdsub,
     &            colcnt,nsuper,snode,xsuper,nnzlmax,nsubmax,xlnz,lnz,
     &            ih,jh,h,cachsz,tmpmax,level,tmpvec,split,ierr,it,
     &            timewd)
         if (ierr .ne. 0) go to 100
c
c Call blkslv: Numerical solution for the new rhs stored in rhs
c
         do i = 1,m
            newrhs(i) = rhs(perm(i))
         enddo
         timbeg = gtimer()
         call blkslv(nsuper,xsuper,xlindx,lindx,xlnz,lnz,newrhs)
         timend = gtimer()
         timewd(7) = timewd(7) + timend - timbeg
         do i = 1,m
            dy(i) = newrhs(invp(i))
         enddo
c
c  Compute dx1 = Q1^(-1)(A1'dy - r1), ds = -dx1, dz1, dz2  and dw
c
         call amux(n1,dy,dx1,a1,ja1,ia1)
         call amux(n2,dy,dx2,a2,ja2,ia2)
         do i = 1,n1
            dx1(i) = q1(i) * (dx1(i) - ds(i))
            ds(i) = -dx1(i)
            dz1(i) = -z1(i) * (one + dx1(i) / x1(i))
            dw(i) = -w(i) * (one + ds(i) / s(i))
         enddo
         do i = 1,n2
            dx2(i) = q2(i) * (dx2(i) - r2(i))
            dz2(i) = -z2(i) * (one + dx2(i) / x2(i))
         enddo
c
c Compute the maximum allowable step lengths
c
         call boundc(x1,dx1,x2,dx2,s,ds,z1,dz1,z2,dz2,w,dw,n1,n2,
     &               beta,deltap,deltad)
         if (deltap * deltad .lt. one) then
c
c Update mu
c
            mu = ddot(n1,z1,1,x1,1) + ddot(n2,z2,1,x2,1) 
     &           + ddot(n1,w,1,s,1)
            g1 = mu + deltap*ddot(n1,z1,1,dx1,1)
     &          + deltad*ddot(n1,dz1,1,x1,1)  
     &          + deltad*deltap*ddot(n1,dz1,1,dx1,1)
     &          + deltap*ddot(n2,z2,1,dx2,1)
     &          + deltad*ddot(n2,dz2,1,x2,1)  
     &          + deltad*deltap*ddot(n2,dz2,1,dx2,1)
     &          + deltap*ddot(n1,w,1,ds,1)
     &          + deltad*ddot(n1,dw,1,s,1) 
     &          + deltad*deltap*ddot(n1,dw,1,ds,1)
            mu = mu*((g1/mu)**3)/(2.d0*dfloat(n1)+dfloat(n2))
c
c Compute dx1dz1, dx2dz2 and dsdw
c
            do i = 1,n1
                dxdz1(i) = dx1(i)*dz1(i)
                dsdw(i) = ds(i)*dw(i)
                xi1(i) = dxdz1(i)/x1(i) - dsdw(i)/s(i)
     &                  - mu * (one/x1(i) - one/s(i))

                ww1(i) = q1(i) * xi1(i)
            enddo
c
c Compute A1Q1^(-1)(X1^(-1)*dx1dz1 - S^(-1)*dsdw - mu(X1^(-1) - S^(-1))) and
c store it in ww2 temporarily
c
            call amux(m,ww1,ww2,ao1,jao1,iao1)
            do i = 1,n2
                dxdz2(i) = dx2(i)*dz2(i)
                xi2(i) = (dxdz2(i) - mu)/x2(i)
                ww1(i) = q2(i) * xi2(i)
            enddo
c
c Compute A2Q2^(-1)(X2^(-1)*dx2dz2 - mu X2^(-1)) and store it in ww3
c temporarily
c
            call amux(m,ww1,ww3,ao2,jao2,iao2)
            do i = 1,m
               rhs(i) = rhs(i) + ww2(i) + ww3(i)
            enddo
c
c
c Compute (AQ^(-1)A')^(-1)rhs and return the result in dy
c
c Call blkslv: Numerical solution for the new rhs stored in rhs
c
            do i = 1,m
               newrhs(i) = rhs(perm(i))
            enddo
            timbeg = gtimer()
            call blkslv(nsuper,xsuper,xlindx,lindx,xlnz,lnz,newrhs)
            timend = gtimer()
            timewd(7) = timewd(7) + timend - timbeg
            do i = 1,m
               dy(i) = newrhs(invp(i))
            enddo
c
c  Compute dx1=Q1^(-1)(A1'dy-X1^(-1)*dx1dz1-S^(-1)*dsdw
c  -mu*(X1^(-1)-S^(-1))-r1), ds = -dx1, dz1, dz2  and dw
c
            call amux(n1,dy,dx1,a1,ja1,ia1)
            call amux(n2,dy,dx2,a2,ja2,ia2)
            do i = 1,n1
               dx1(i) = q1(i) * (dx1(i) - xi1(i) - z1(i) + w(i))
               ds(i) = -dx1(i)
               dz1(i) = -z1(i) + (mu - z1(i)*dx1(i) 
     &                  - dxdz1(i))/x1(i)
               dw(i) = -w(i) + (mu - w(i)*ds(i) - dsdw(i))/s(i)
            enddo
            do i = 1,n2
               dx2(i) = q2(i) * (dx2(i) - xi2(i) - r2(i))
               dz2(i) = -z2(i) + (mu - z2(i)*dx2(i) - 
     &                  dxdz2(i))/x2(i)
            enddo
c
c Compute the maximum allowable step lengths
c
            call boundc(x1,dx1,x2,dx2,s,ds,z1,dz1,z2,dz2,w,dw,n1,n2,
     &                  beta,deltap,deltad)
         endif
c
c Take the step
c
         call daxpy(n1,deltap,dx1,1,x1,1)
         call daxpy(n2,deltap,dx2,1,x2,1)
         call daxpy(n1,deltap,ds,1,s,1)
         call daxpy(n1,deltad,dw,1,w,1)
         call daxpy(n1,deltad,dz1,1,z1,1)
         call daxpy(n2,deltad,dz2,1,z2,1)
         call daxpy(m,deltad,dy,1,y,1)
         gap = ddot(n1,z1,1,x1,1) + ddot(n2,z2,1,x2,1) + 
     &         ddot(n1,w,1,s,1)
      goto 20
   30 continue
  100 continue
      maxit = it
      return
      end

C  The following routines are from LINPACK and do not seem to be part of
C  the standard R distro.
      subroutine dgedi(a,lda,n,ipvt,det,work,job)
      integer lda,n,ipvt(*),job
      double precision a(lda,*),det(2),work(*)
c
c     dgedi computes the determinant and inverse of a matrix
c     using the factors computed by dgeco or dgefa.
c
c     on entry
c
c        a       double precision(lda, n)
c                the output from dgeco or dgefa.
c
c        lda     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c        ipvt    integer(n)
c                the pivot vector from dgeco or dgefa.
c
c        work    double precision(n)
c                work vector.  contents destroyed.
c
c        job     integer
c                = 11   both determinant and inverse.
c                = 01   inverse only.
c                = 10   determinant only.
c
c     on return
c
c        a       inverse of original matrix if requested.
c                otherwise unchanged.
c
c        det     double precision(2)
c                determinant of original matrix if requested.
c                otherwise not referenced.
c                determinant = det(1) * 10.0**det(2)
c                with  1.0 .le. dabs(det(1)) .lt. 10.0
c                or  det(1) .eq. 0.0 .
c
c     error condition
c
c        a division by zero will occur if the input factor contains
c        a zero on the diagonal and the inverse is requested.
c        it will not occur if the subroutines are called correctly
c        and if dgeco has set rcond .gt. 0.0 or dgefa has set
c        info .eq. 0 .
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     blas daxpy,dscal,dswap
c     fortran dabs,mod
c
c     internal variables
c
      double precision t
      double precision ten
      integer i,j,k,kb,kp1,l,nm1
c
c
c     compute determinant
c
      if (job/10 .eq. 0) go to 70
         det(1) = 1.0d0
         det(2) = 0.0d0
         ten = 10.0d0
         do 50 i = 1, n
            if (ipvt(i) .ne. i) det(1) = -det(1)
            det(1) = a(i,i)*det(1)
c        ...exit
            if (det(1) .eq. 0.0d0) go to 60
   10       if (dabs(det(1)) .ge. 1.0d0) go to 20
               det(1) = ten*det(1)
               det(2) = det(2) - 1.0d0
            go to 10
   20       continue
   30       if (dabs(det(1)) .lt. ten) go to 40
               det(1) = det(1)/ten
               det(2) = det(2) + 1.0d0
            go to 30
   40       continue
   50    continue
   60    continue
   70 continue
c
c     compute inverse(u)
c
      if (mod(job,10) .eq. 0) go to 150
         do 100 k = 1, n
            a(k,k) = 1.0d0/a(k,k)
            t = -a(k,k)
            call dscal(k-1,t,a(1,k),1)
            kp1 = k + 1
            if (n .lt. kp1) go to 90
            do 80 j = kp1, n
               t = a(k,j)
               a(k,j) = 0.0d0
               call daxpy(k,t,a(1,k),1,a(1,j),1)
   80       continue
   90       continue
  100    continue
c
c        form inverse(u)*inverse(l)
c
         nm1 = n - 1
         if (nm1 .lt. 1) go to 140
         do 130 kb = 1, nm1
            k = n - kb
            kp1 = k + 1
            do 110 i = kp1, n
               work(i) = a(i,k)
               a(i,k) = 0.0d0
  110       continue
            do 120 j = kp1, n
               t = work(j)
               call daxpy(n,t,a(1,j),1,a(1,k),1)
  120       continue
            l = ipvt(k)
            if (l .ne. k) call dswap(n,a(1,k),1,a(1,l),1)
  130    continue
  140    continue
  150 continue
      return
      end
      subroutine dgeco(a,lda,n,ipvt,rcond,z)
      integer lda,n,ipvt(*)
      double precision a(lda,*),z(*)
      double precision rcond
c
c     dgeco factors a double precision matrix by gaussian elimination
c     and estimates the condition of the matrix.
c
c
c     if  rcond  is not needed, dgefa is slightly faster.
c     to solve  a*x = b , follow dgeco by dgesl.
c     to compute  inverse(a)*c , follow dgeco by dgesl.
c     to compute  determinant(a) , follow dgeco by dgedi.
c     to compute  inverse(a) , follow dgeco by dgedi.
c
c     on entry
c
c        a       double precision(lda, n)
c                the matrix to be factored.
c
c        lda     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c     on return
c
c        a       an upper triangular matrix and the multipliers
c                which were used to obtain it.
c                the factorization can be written  a = l*u  where
c                l  is a product of permutation and unit lower
c                triangular matrices and  u  is upper triangular.
c
c        ipvt    integer(n)
c                an integer vector of pivot indices.
c
c        rcond   double precision
c                an estimate of the reciprocal condition of  a .
c                for the system  a*x = b , relative perturbations
c                in  a  and  b  of size  epsilon  may cause
c                relative perturbations in  x  of size  epsilon/rcond .
c                if  rcond  is so small that the logical expression
c                           1.0 + rcond .eq. 1.0
c                is true, then  a  may be singular to working
c                precision.  in particular,  rcond  is zero  if
c                exact singularity is detected or the estimate
c                underflows.
c
c        z       double precision(n)
c                a work vector whose contents are usually unimportant.
c                if  a  is close to a singular matrix, then  z  is
c                an approximate null vector in the sense that
c                norm(a*z) = rcond*norm(a)*norm(z) .
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     linpack dgefa
c     blas daxpy,ddot,dscal,dasum
c     fortran dabs,dmax1,dsign
c
c     internal variables
c
      double precision ddot,ek,t,wk,wkm
      double precision anorm,s,dasum,sm,ynorm
      integer info,j,k,kb,kp1,l
c
c
c     compute 1-norm of a
c
      anorm = 0.0d0
      do 10 j = 1, n
         anorm = dmax1(anorm,dasum(n,a(1,j),1))
   10 continue
c
c     factor
c
      call dgefa(a,lda,n,ipvt,info)
c
c     rcond = 1/(norm(a)*(estimate of norm(inverse(a)))) .
c     estimate = norm(z)/norm(y) where  a*z = y  and  trans(a)*y = e .
c     trans(a)  is the transpose of a .  the components of  e  are
c     chosen to cause maximum local growth in the elements of w  where
c     trans(u)*w = e .  the vectors are frequently rescaled to avoid
c     overflow.
c
c     solve trans(u)*w = e
c
      ek = 1.0d0
      do 20 j = 1, n
         z(j) = 0.0d0
   20 continue
      do 100 k = 1, n
         if (z(k) .ne. 0.0d0) ek = dsign(ek,-z(k))
         if (dabs(ek-z(k)) .le. dabs(a(k,k))) go to 30
            s = dabs(a(k,k))/dabs(ek-z(k))
            call dscal(n,s,z,1)
            ek = s*ek
   30    continue
         wk = ek - z(k)
         wkm = -ek - z(k)
         s = dabs(wk)
         sm = dabs(wkm)
         if (a(k,k) .eq. 0.0d0) go to 40
            wk = wk/a(k,k)
            wkm = wkm/a(k,k)
         go to 50
   40    continue
            wk = 1.0d0
            wkm = 1.0d0
   50    continue
         kp1 = k + 1
         if (kp1 .gt. n) go to 90
            do 60 j = kp1, n
               sm = sm + dabs(z(j)+wkm*a(k,j))
               z(j) = z(j) + wk*a(k,j)
               s = s + dabs(z(j))
   60       continue
            if (s .ge. sm) go to 80
               t = wkm - wk
               wk = wkm
               do 70 j = kp1, n
                  z(j) = z(j) + t*a(k,j)
   70          continue
   80       continue
   90    continue
         z(k) = wk
  100 continue
      s = 1.0d0/dasum(n,z,1)
      call dscal(n,s,z,1)
c
c     solve trans(l)*y = w
c
      do 120 kb = 1, n
         k = n + 1 - kb
         if (k .lt. n) z(k) = z(k) + ddot(n-k,a(k+1,k),1,z(k+1),1)
         if (dabs(z(k)) .le. 1.0d0) go to 110
            s = 1.0d0/dabs(z(k))
            call dscal(n,s,z,1)
  110    continue
         l = ipvt(k)
         t = z(l)
         z(l) = z(k)
         z(k) = t
  120 continue
      s = 1.0d0/dasum(n,z,1)
      call dscal(n,s,z,1)
c
      ynorm = 1.0d0
c
c     solve l*v = y
c
      do 140 k = 1, n
         l = ipvt(k)
         t = z(l)
         z(l) = z(k)
         z(k) = t
         if (k .lt. n) call daxpy(n-k,t,a(k+1,k),1,z(k+1),1)
         if (dabs(z(k)) .le. 1.0d0) go to 130
            s = 1.0d0/dabs(z(k))
            call dscal(n,s,z,1)
            ynorm = s*ynorm
  130    continue
  140 continue
      s = 1.0d0/dasum(n,z,1)
      call dscal(n,s,z,1)
      ynorm = s*ynorm
c
c     solve  u*z = v
c
      do 160 kb = 1, n
         k = n + 1 - kb
         if (dabs(z(k)) .le. dabs(a(k,k))) go to 150
            s = dabs(a(k,k))/dabs(z(k))
            call dscal(n,s,z,1)
            ynorm = s*ynorm
  150    continue
         if (a(k,k) .ne. 0.0d0) z(k) = z(k)/a(k,k)
         if (a(k,k) .eq. 0.0d0) z(k) = 1.0d0
         t = -z(k)
         call daxpy(k-1,t,a(1,k),1,z(1),1)
  160 continue
c     make znorm = 1.0
      s = 1.0d0/dasum(n,z,1)
      call dscal(n,s,z,1)
      ynorm = s*ynorm
c
      if (anorm .ne. 0.0d0) rcond = ynorm/anorm
      if (anorm .eq. 0.0d0) rcond = 0.0d0
      return
      end

      subroutine dgefa(a,lda,n,ipvt,info)
c      use numerical_libraries

      integer lda,n,ipvt(*),info
      double precision a(lda,*)
c
c     dgefa factors a double precision matrix by gaussian elimination.
c
c     dgefa is usually called by dgeco, but it can be called
c     directly with a saving in time if  rcond  is not needed.
c     (time for dgeco) = (1 + 9/n)*(time for dgefa) .
c
c     on entry
c
c        a       double precision(lda, n)
c                the matrix to be factored.
c
c        lda     integer
c                the leading dimension of the array  a .
c
c        n       integer
c                the order of the matrix  a .
c
c     on return
c
c        a       an upper triangular matrix and the multipliers
c                which were used to obtain it.
c                the factorization can be written  a = l*u  where
c                l  is a product of permutation and unit lower
c                triangular matrices and  u  is upper triangular.
c
c        ipvt    integer(n)
c                an integer vector of pivot indices.
c
c        info    integer
c                = 0  normal value.
c                = k  if  u(k,k) .eq. 0.0 .  this is not an error
c                     condition for this subroutine, but it does
c                     indicate that dgesl or dgedi will divide by zero
c                     if called.  use  rcond  in dgeco for a reliable
c                     indication of singularity.
c
c     linpack. this version dated 08/14/78 .
c     cleve moler, university of new mexico, argonne national lab.
c
c     subroutines and functions
c
c     blas daxpy,dscal,idamax
c
c     internal variables
c
      double precision t
c           integer idamax,j,k,kp1,l,nm1
      integer j,k,kp1,l,nm1
c
c
c     gaussian elimination with partial pivoting
c
      info = 0
      nm1 = n - 1
      if (nm1 .lt. 1) go to 70
      do 60 k = 1, nm1
         kp1 = k + 1
c
c        find l = pivot index
c
         l = idamax(n-k+1,a(k,k),1) + k - 1
         ipvt(k) = l
c
c        zero pivot implies this column already triangularized
c
         if (a(l,k) .eq. 0.0d0) go to 40
c
c           interchange if necessary
c
            if (l .eq. k) go to 10
               t = a(l,k)
               a(l,k) = a(k,k)
               a(k,k) = t
   10       continue
c
c           compute multipliers
c
            t = -1.0d0/a(k,k)
            call dscal(n-k,t,a(k+1,k),1)
c
c           row elimination with column indexing
c
            do 30 j = kp1, n
               t = a(l,j)
               if (l .eq. k) go to 20
                  a(l,j) = a(k,j)
                  a(k,j) = t
   20          continue
               call daxpy(n-k,t,a(k+1,k),1,a(k+1,j),1)
   30       continue
         go to 50
   40    continue
            info = k
   50    continue
   60 continue
   70 continue
      ipvt(n) = n
      if (a(n,n) .eq. 0.0d0) info = n
      return
      end

"latex" <- function(x, ...) UseMethod("latex")

"latex.summary.rqs" <-
function (x, transpose = FALSE, caption = "caption goes here.", 
	digits = 3, file = as.character(substitute(x)), ...)
{   
        taus <- function(x) x$tau
        taus <- unlist(lapply(x,taus))
        taus <- format(round(taus, digits))
        coef <- lapply(x,coefficients)
        p <- nrow(coef[[1]])
        k <- ncol(coef[[1]])
        m <- length(taus)
        rlab <- dimnames(coef[[1]])[[1]]
        clab <- taus
        a <- format(round(array(unlist(coef),c(p,k,m)),digits = digits))
        table <- matrix("", p, m)
        for (i in 1:m) {
           for (j in 1:p) {
              if (k == 3) {
                table[j, i] <- paste("$\\underset{(", a[j, 2,
                  i], ",", a[j, 3, i], ")}{", a[j, 1, i], "}$", sep="")
            }
              else if (k == 4) {
                table[j, i] <- paste("$\\underset{(", a[j,2,i] , ")}{",
                        a[j,1, i], "}$",sep="")
               }
            }
        }   
        rowlabel <- "Covariates"
        dimnames(table) <- list(rlab, clab)
        if(transpose) { 
              table <- t(table)
              rowlabel <- "Quantiles"
             }
        latex.table(table, caption = caption, rowlabel = rowlabel, file = file, ...)
        invisible()
}       

"latex.table.rq" <-  
function(x, ...) {
cat("table.rq and related methods are defunct -- see rq, which now accepts a vector of taus.")
}

"plot.table.rq" <- 
function(x, ...){
cat("table.rq() and related methods are defunct --  see ?rq, which now accepts a vector of taus.")
}

"table.rq" <- 
function(x, ...){
cat("table.rq() and related methods are defunct  -- see ?rq, which now accepts a vector of taus.")
}

plot.rqs <- function(x, parm = NULL, ols = TRUE,
  mfrow = NULL, mar = NULL, ylim = NULL, main = NULL, col = 1:2, lty = 1:2,
  cex = 0.5, pch = 20, type = "b", xlab = "", ylab = "", ...)
{
  ## extract taus
  taus <- x$tau

  ## obtain rq coefficients (in "parm")
  cf <- coef(x)
  if(is.null(parm)) parm <- rownames(cf)
  if(is.numeric(parm)) parm <- rownames(cf)[parm]
  cf <- cf[parm,,drop = FALSE]

  ## obtain OLS coefficients
  if(ols) {
    obj <- x
    mt <- terms(x)
    mf <- model.frame(x)
    y <- model.response(mf)
    X <- model.matrix(mt, mf, contrasts = x$contrasts)
    olscf <- lm.fit(X, y)$coefficients[parm]
  }

  ## plotting parameters
  mfrow_orig <- par("mfrow")
  mar_orig <- par("mar")
  if(is.null(mfrow)) mfrow <- n2mfrow(length(parm))
  if(is.null(mar)) mar <- c(3.1, 3.1, 3.1, 1.6)
  par(mfrow = mfrow, mar = mar)
  col <- rep(col, length.out = 2)
  lty <- rep(lty, length.out = 2)
  if(is.null(main)) main <- parm
  main <- rep(main, length.out = length(parm))
  xlab <- rep(xlab, length.out = length(parm))
  ylab <- rep(ylab, length.out = length(parm))

  ## actual plotting
 ylim0 <- ylim
 for (i in seq(along = parm)) {
        if(is.null(ylim)){         
           if(ols)
	        ylim <- range(c(cf[i, ], olscf[i]))
           else
		ylim <- range(cf[i,])
	   }
        plot(taus, cf[i, ], cex = cex, pch = pch, type = type, col = col[1],
            ylim = ylim, xlab = xlab[i], ylab = ylab[i], main = main[i], ...)
        if (ols)
            abline(h = olscf[i], col = col[2], lty = 2)
        abline(h = 0, col = gray(0.3))
	ylim <- ylim0
    }

  ## restore original par settings
  par(mfrow = mfrow_orig, mar = mar_orig)
  if(ols)
     invisible(cbind(cf, ols = olscf))
  else
     invisible(cf)
}  
  
plot.summary.rqs <- function(x, parm = NULL, level = 0.9, ols = TRUE,
  mfrow = NULL, mar = NULL, ylim = NULL, main = NULL,
  col = gray(c(0, 0.75)), border = NULL, lcol = 2, lty = 1:2,
  cex = 0.5, pch = 20, type = "b", xlab = "", ylab = "", ...)
{
  ## normal quantile
  zalpha <- qnorm(1 - (1-level)/2)

  ## extract taus
  taus <- sapply(x, function(x) x$tau)

  ## obtain rq coefficients
  cf <- lapply(x, coef)
  if(ncol(cf[[1]]) == 4) {
    for(i in 1:length(cf)) {
      cfi <- cf[[i]]
      cfi <- cbind(cfi[,1], cfi[,1] - cfi[,2] * zalpha, cfi[,1] + cfi[,2] * zalpha)
      colnames(cfi) <- c("coefficients", "lower bd", "upper bd")
      cf[[i]] <- cfi
    }
  }
  if(ncol(cf[[1]]) != 3) stop("summary.rqs components have wrong dimension")

  ## extract only coefficients in "parm"
  if(is.null(parm)) parm <- rownames(cf[[1]])
  if(is.numeric(parm)) parm <- rownames(cf[[1]])[parm]
  cf <- lapply(cf, function(x) x[parm,,drop=FALSE])
  names(cf) <- paste("tau=", taus)

  ## obtain OLS coefficients
  if(ols) {
    obj <- x[[1]]
    mt <- terms(obj)
    mf <- model.frame(obj)
    y <- model.response(mf)
    X <- model.matrix(mt, mf, contrasts = obj$contrasts)
    olscf <- summary(lm(y ~ X))$coefficients
    rownames(olscf) <- rownames(coef(obj))
    olscf <- cbind(olscf[parm,1,drop=FALSE],
                   olscf[parm,1,drop=FALSE] - olscf[parm,2,drop=FALSE] * zalpha,
                   olscf[parm,1,drop=FALSE] + olscf[parm,2,drop=FALSE] * zalpha)
    colnames(olscf) <- c("coefficients", "lower bd", "upper bd")
  }

  ## plotting parameters
  mfrow_orig <- par("mfrow")
  mar_orig <- par("mar")
  if(is.null(mfrow)) mfrow <- n2mfrow(length(parm))
  if(is.null(mar)) mar <- c(3.1, 3.1, 3.1, 1.6)
  par(mfrow = mfrow, mar = mar)
  col <- rep(col, length.out = 2)
  lty <- rep(lty, length.out = 2)
  if(is.null(border)) border <- col[2]
  if(is.null(main)) main <- parm
  main <- rep(main, length.out = length(parm))
  xlab <- rep(xlab, length.out = length(parm))
  ylab <- rep(ylab, length.out = length(parm))

  ## actual plotting
  ylim0 <- ylim
  Ylim <- NULL
    if(is.matrix(ylim0))
	if((nrow(ylim0) != 2) || ncol(ylim0) != length(parm))
	    stop("ylim matrix of incompatible dimensions")
  for(i in seq(along = parm)) {
  b <- t(sapply(seq(along = cf), function(tau) cf[[tau]][i, ]))
  if(is.matrix(ylim0)) ylim <- ylim0[,i]
  if(is.null(ylim0)){
           if(ols)
                ylim <- range(c(b[,2], b[,3], olscf[i]))
           else
                ylim <- range(b[,2],b[,3])
           }
    plot(rep(taus, 2), c(b[,2], b[,3]), type = "n",
      ylim = ylim, xlab = xlab[i], ylab = ylab[i], main = main[i])
    polygon(c(taus, rev(taus)), c(b[,2], rev(b[,3])), col = col[2], border = border)
    points(taus, b[,1], cex = cex, pch = pch, type = type, col = col[1], ...)
    if(ols) {
      abline(h = olscf[i, 1], col = lcol, lty = lty[1])
      abline(h = olscf[i, 2], col = lcol, lty = lty[2])
      abline(h = olscf[i, 3], col = lcol, lty = lty[2])
    }
    abline(h = 0, col = gray(0.3))
    Ylim <- cbind(Ylim, ylim)
  }

  ## restore original par settings and return
  par(mfrow = mfrow_orig, mar = mar_orig)
  if(ols)
        x <- c(cf, list(ols = olscf))
  else
        x <- cf
  invisible(list(structure(as.vector(unlist(x)), .Dim = c(dim(x[[1]]), length(x)),
    .Dimnames = list(rownames(x[[1]]), colnames(x[[1]]), names(x))),Ylim = Ylim))
}
"latex.table" <-
function (x, file = as.character(substitute(x)), rowlabel = file, 
    rowlabel.just = "l", cgroup, n.cgroup, rgroup, n.rgroup = NULL, 
    digits, dec, rdec, cdec, append = FALSE, dcolumn = FALSE, cdot = FALSE, 
    longtable = FALSE, table.env = TRUE, lines.page = 40, caption, caption.lot, 
    label = file, double.slash = FALSE, ...) 
{
    nc <- ncol(x)
    nr <- nrow(x)
    if (missing(caption) & !missing(caption.lot)) 
        warning("caption.lot is ignored unless caption is specified")
    if (!longtable & !table.env & !missing(caption)) 
        stop("you must have table.env=TRUE if caption is given")
    if (!missing(digits)) 
        .Options$digits <- digits
    sl <- if (double.slash) 
        "\\\\"
    else "\\"
    rlj <- if (rowlabel.just == "l") 
        "l"
    else "c"
    if (!missing(dec)) {
        if (length(dec) == 1) 
            x <- round(x, dec)
        else {
            if (!is.matrix(dec) || nrow(dec) != nrow(x) || ncol(dec) != 
                ncol(x)) 
                stop("dimensions of dec do not match those of x")
            for (i in 1:nr) for (j in 1:nc) x[i, j] <- round(x[i, 
                j], dec[i, j])
        }
        cx <- format(x)
    }
    else if (!missing(rdec)) {
        cx <- NULL
        for (i in 1:nr) {
            x[i, ] <- round(x[i, ], rdec[i])
            cx <- rbind(cx, format(x[i, ]))
        }
    }
    else if (!missing(cdec)) {
        cx <- NULL
        for (j in 1:nc) {
            x[, j] <- round(x[, j], cdec[j])
            cx <- cbind(cx, format(x[, j]))
        }
    }
    else cx <- format(x)
    cx[is.na(x)] <- ""
    if (dcolumn) 
        sep <- "."
    else {
        #cx <- translate(cx, " ", "~")
        cx <- matrix(chartr(" ", "~", cx), nrow=nr)
        if (cdot) {
            #cx <- translate(cx, "[.]", "\\\\cdot", multichar = TRUE)
            cx <- gsub("[.]", "\\\\cdot", cx)
            cx <- matrix(paste("$", cx, "$", sep = ""), nrow = nr)
            cx[is.na(x)] <- ""
        }
        sep <- "c"
    }
    if (is.null(n.rgroup) && !missing(rgroup)) 
        n.rgroup <- rep(nr/length(rgroup), length(rgroup))
    if (!is.null(n.rgroup) && sum(n.rgroup) != nr) 
        stop("sum of n.rgroup must equal number of rows in x")
    if (!missing(rgroup) && !is.null(n.rgroup) && (length(rgroup) != 
        length(n.rgroup))) 
        stop("lengths of rgroup and n.rgroup must match")
    fi <- paste(file, ".tex", sep = "")
    rowname <- dimnames(x)[[1]]
    if (length(rowname) == 0) {
        rowname <- NULL
        rowlabel <- NULL
        if (!missing(rgroup)) 
            stop("you must have row dimnames to use rgroup")
    }
    #start new file
    if (!append) 
        cat("", file = fi)
    cat("%", deparse(match.call()), "\n%\n", file = fi, append = TRUE)
    if (dcolumn) 
        cat(sl, "newcolumn{.}{D{.}{", sl, "cdot}{-1}}\n", file = fi, 
            append = TRUE)
    if (!is.null(rowlabel)) 
        form <- paste("|", rowlabel.just, "|", sep = "")
    else form <- ""
    f <- paste("|", sep, sep = "", collapse = "")
    if (missing(cgroup)) 
        ff <- c(rep(f, nc), "|")
    else {
        k <- length(cgroup)
        if (missing(n.cgroup)) 
            n.cgroup <- rep(nc/k, k)
        if (sum(n.cgroup) != nc) 
            stop("sum of n.cgroup must equal number of columns")
        if (length(n.cgroup) != length(cgroup)) 
            stop("cgroup and n.cgroup must have same lengths")
        ff <- NULL
        for (i in 1:k) ff <- c(ff, rep(f, n.cgroup[i]), "|")
    }
    form <- paste(form, paste(ff, collapse = ""), sep = "")
    #if(missing(cgroup)) hline <- "" else hline <- paste(sl,"hline",sep="")
    hline <- ""
    if (!missing(caption)) 
        caption <- paste(sl, "caption", if (missing(caption.lot)) 
            NULL
        else paste("[", caption.lot, "]", sep = ""), "{", caption, 
            if (longtable) 
                NULL
            else paste(sl, "label{", label, "}", sep = ""), "}", 
            sep = "")
    if (!longtable) {
        if (table.env){ 
            cat(sl, "begin{table}[hptb]\n", sep = "", file = fi, append = TRUE)
            cat(caption, "\n", sep = "", file = fi, append = TRUE)
	}
        cat(sl, "begin{center}\n", file = fi, sep = "", append = TRUE)
        cat(sl, "begin{tabular}{", form, "} ", sl, "hline", hline, 
            "\n", sep = "", file = fi, append = TRUE)
    }
    else {
        cat(paste(sl, "setlongtables", sep = ""), paste(sl, "begin{longtable}{", 
            form, "}", sep = ""), sep = "\n", file = fi, append = TRUE)
        if (!missing(caption)) 
            cat(caption, sl, sl, "\n", sep = "", file = fi, append = TRUE)
        cat(sl, "hline", hline, "\n", sep = "", file = fi, append = TRUE)
    }
    if (!missing(cgroup)) {
        cgroup <- paste(sl, "bf ", cgroup, sep = "")
        if (is.null(rowlabel)) {
            labs <- c(paste(sl, "multicolumn{", n.cgroup[1], 
                "}{|c||}{", cgroup[1], "}", sep = "", collapse = ""), 
                if (k > 2) paste(sl, "multicolumn{", n.cgroup[c(-1, 
                  -k)], "}{c||}{", cgroup[c(-1, -k)], "}", sep = "") else NULL, 
                paste(sl, "multicolumn{", n.cgroup[k], "}{c|}{", 
                  cgroup[k], "}", sep = ""))
            g <- paste(sl, "hline", sep = "")
        }
        else {
            rowlabel <- paste(sl, "bf ", rowlabel, sep = "")
            labs <- c(paste(sl, "multicolumn{1}{|", rlj, "||}{", 
                rowlabel, "}", sep = ""), paste(sl, "multicolumn{", 
                n.cgroup[-k], "}{c||}{", cgroup[-k], "}", sep = ""), 
                paste(sl, "multicolumn{", n.cgroup[k], "}{c|}{", 
                  cgroup[k], "}", sep = ""))
            g <- paste(sl, "cline{2-", nc + 1, "}", sep = "")
        }
        cat(labs, file = fi, sep = "&", append = TRUE)
        cat(sl, sl, " ", g, "\n", sep = "", file = fi, append = TRUE)
        if (!is.null(rowlabel)) 
            rowlabel <- ""
    }
    collabel <- dimnames(x)[[2]]
    if (is.null(collabel)) 
        collabel <- as.character(1:nc)
    labs <- c(rowlabel, collabel)
    if (missing(cgroup)) {
        if (is.null(rowlabel)) 
            pre <- c(paste(sl, "multicolumn{1}{|c|}{", sep = ""), 
                rep(paste(sl, "multicolumn{1}{c|}{", sep = ""), 
                  nc - 1))
        else pre <- c(paste(sl, "multicolumn{1}{|", rlj, "||}{", 
            sep = ""), rep(paste(sl, "multicolumn{1}{c|}{", sep = ""), 
            nc))
    }
    else {
        if (is.null(rowlabel)) {
            pre <- NULL
            j <- 0
            for (i in 1:k) {
                if (n.cgroup[i] > 1) {
                  g <- rep(paste(sl, "multicolumn{1}{c|}{", sep = ""), 
                    n.cgroup[i] - 1)
                  if (j == 0) 
                    g[1] <- paste(sl, "multicolumn{1}{|c|}{", 
                      sep = "")
                  pre <- c(pre, g)
                }
                j <- j + n.cgroup[i]
                if (j == 1) 
                  g <- paste(sl, "multicolumn{1}{|c||}{", sep = "")
                else if (j < nc) 
                  g <- paste(sl, "multicolumn{1}{c||}{", sep = "")
                else g <- paste(sl, "multicolumn{1}{c|}{", sep = "")
                pre <- c(pre, g)
            }
        }
        else {
            pre <- paste(sl, "multicolumn{1}{|", rlj, "||}{", 
                sep = "")
            j <- 0
            for (i in 1:k) {
                pre <- c(pre, rep(paste(sl, "multicolumn{1}{c|}{", 
                  sep = ""), n.cgroup[i] - 1))
                j <- j + n.cgroup[i]
                if (j < nc) 
                  g <- paste(sl, "multicolumn{1}{c||}{", sep = "")
                else g <- paste(sl, "multicolumn{1}{c|}{", sep = "")
                pre <- c(pre, g)
            }
        }
    }
    labs <- paste(pre, labs, "}", sep = "")
    cat(labs, file = fi, sep = "&", append = TRUE)
    cat(sl, sl, " ", sl, "hline", hline, "\n", sep = "", file = fi, 
        append = TRUE)
    if (longtable) {
        if (missing(caption)) 
            cat(sl, "endhead\n", sl, "hline", sl, "endfoot\n", 
                sep = "", file = fi, append = TRUE)
        else {
            cat(sl, "endfirsthead\n", sep = "", file = fi, append = TRUE)
            if (!missing(caption)) 
                cat(sl, "caption[]{\\em (continued)} ", sl, sl, 
                  "\n", sep = "", file = fi, append = TRUE)
            cat(sl, "hline", hline, "\n", sep = "", file = fi, 
                append = TRUE)
            cat(labs, file = fi, sep = "&", append = TRUE)
            cat(sl, sl, " ", sl, "hline", hline, "\n", sl, "endhead", 
                sl, "hline", sl, "endfoot\n", sep = "", file = fi, 
                append = TRUE)
            cat(sl, "label{", label, "}\n", sep = "", file = fi, 
                append = TRUE)
        }
    }
    if (is.null(n.rgroup)) 
        rg.end <- 0
    else {
        rg.end <- cumsum(n.rgroup)
        rg.start <- rg.end - n.rgroup + 1
        if (missing(rgroup)) 
            rgroup <- rep("", length(n.rgroup))
        else rgroup <- paste("{", sl, "bf ", rgroup, "}", sep = "")
    }
    linecnt <- 0
    for (i in 1:nr) {
        if (!missing(rgroup)) {
            k <- rg.start == i
            if (any(k)) {
                j <- (1:length(n.rgroup))[k]
                if (longtable && linecnt > 0 && (linecnt + n.rgroup[j] + 
                  (n.rgroup[j] > 1)) > lines.page) {
                  cat(sl, "newpage\n", sep = "", file = fi, append = TRUE)
                  linecnt <- 0
                }
                if (n.rgroup[j] > 1) {
                  cat(rgroup[j], rep("", nc), file = fi, sep = "&", 
                    append = TRUE)
                  linecnt <- linecnt + 1
                  cat(sl, sl, "\n", sep = "", file = fi, append = TRUE)
                }
                l <- rg.start[j]:rg.end[j]
                if (length(l) > 1) 
                  rowname[l] <- paste("~~", rowname[l], sep = "")
                else rowname[l] <- paste("{", sl, "bf ", rowname[l], 
                  "}", sep = "")
            }
        }
        else if (longtable && linecnt > 0 && (linecnt + 1 > lines.page)) {
            cat(sl, "newpage\n", sep = "", file = fi, append = TRUE)
            linecnt <- 0
        }
        cat(c(rowname[i], cx[i, ]), file = fi, sep = "&", append = TRUE)
        linecnt <- linecnt + 1
        if (i < nr && any(rg.end == i)) 
            g <- paste(sl, "hline", sep = "")
        else g <- ""
        cat(sl, sl, " ", g, "\n", sep = "", file = fi, append = TRUE)
    }
    cat(sl, "hline", hline, "\n", sep = "", file = fi, append = TRUE)
    if (longtable) 
        cat(sl, "end{longtable}\n", sep = "", file = fi, append = TRUE)
    else {
        cat(sl, "end{tabular}\n", sep = "", file = fi, append = TRUE)
        if (!missing(caption)) {
            cat(sl, "vspace{3mm}\n", sep = "", file = fi, append = TRUE)
            #cat(caption, "\n", file = fi, append = TRUE)
	    }
        #cat(caption, "\n", file = fi, append = TRUE)
        cat(sl, "end{center}\n", sep = "", file = fi, append = TRUE)
        if (table.env) 
            cat(sl, "end{table}\n", sep = "", file = fi, append = TRUE)
    }
    invisible()
}
"table.rq" <-
function (x, ...) 
stop("table.rq now defunct, rq() now accepts vector tau argument.  See ?rq.")

C Output from Public domain Ratfor, version 1.05
      subroutine combin(r,n,m,a,c,e,last)
      integer r,n,m,t,k,j,m0,mj,s
      integer a(n,m),c(r),e(r),last(r)
      logical odd
      m0 = r-n
      t = n+1
      k = 1
      j = 0
      c(1) = 0
23000 continue
      j = j + 1
      c(j) = j
      e(j) = j - 1
      if(odd(j))then
      last(j) = m0 + j
      else
      last(j) = j + 1
      endif
      if(j .eq. n)then
      goto 23002
      endif
23001 goto 23000
23002 continue
      do23007 i = 1,n
      a(i,1) = c(i)
23007 continue
23008 continue
      if(n .lt. r)then
23011 continue
      k = k + 1
      s = c(j)
      mj = m0 + j
      e(n+1) = n
      if(odd(j))then
      if(c(j) .eq. mj)then
      c(j) = c(j-1) + 1
      last(j+1) = c(j) + 1
      else
      c(j) = c(j) + 1
      endif
      else
      if(c(j) .eq. c(j-1) + 1)then
      c(j) = mj
      else
      last(j+1) = c(j)
      c(j) = c(j) - 1
      endif
      endif
      if(c(j) .eq. last(j))then
      last(j) = s
      e(j+1) = e(j)
      e(j) = j-1
      endif
      if( (j .lt. n) .and. (c(j) .eq. mj))then
      t = j
      j = e(t+1)
      e(t+1) = t
      else
      if(t .eq. j)then
      t = t + 1
      endif
      if(t .lt. e(n+1))then
      j = t
      else
      j = e(n+1)
      endif
      endif
      do23028 i = 1,n
      a(i,k) = c(i)
23028 continue
23029 continue
      if(j .eq. 0)then
      goto 23013
      endif
23012 goto 23011
23013 continue
      endif
      return
      end
      logical function odd(j)
      integer j
      odd = (mod(j,2) .eq. 1)
      return
      end

#parzen, wei and ying's bootstrap
subroutine pwy(m,n,k,m5,n2,a,c,b,t,toler,ift,x,e,s, wa,wb)
double precision b(m),a(k,n),x(n,k)
double precision wa(m5,n2),wb(m),e(m),c(m,n)
double precision t,toler
integer m,n,k,m5,n2,ift
integer s(m)
do i=1,k{
	call dcopy(n,a(i,1),k,c(m,1),m)
	call rq0(m,n,m5,n2,c,b,t,toler,ift,x(1,i),e,s,wa,wb)
	}
return
end
#ratfor outer loop for xy-pairs rq bootstrap
#notation is horrendous 
#   ratfor   R-function
#______________________
#	m -> n  number of original obs
#	n -> p  number of parameters
#	k -> R  number of BS replications
#	mofn -> m  number of BS observations
#
subroutine xys(mofn,m,n,k,mofn5,n2,a,b,tau,toler,ift,x,e,s, wa,wb,aa,bb,ss)
double precision b(m),a(m,n),x(n,k)
double precision wa(mofn5,n2),wb(mofn)
double precision aa(mofn,n),bb(mofn),e(mofn)
double precision tau,toler
integer ss(mofn,k),s(mofn),mofn,m,n,k,mofn5,n2,ift(k)
do i=1,k {
	do ii=1,mofn{
		bb(ii)=b(ss(ii,i))
		do jj=1,n{
			aa(ii,jj)=a(ss(ii,i),jj)
			}
		}
	call rq0(mofn,n,mofn5,n2,aa,bb,tau,toler,ift(i),x(1,i),e,s,wa,wb)
	}
return
end
# Weighted (Bose) Bootstrap version
subroutine wxy(m,n,k,m5,n2,a,b,tau,toler,ift,x,e,s,wa,wb,aa,bb,w)
double precision b(m),a(m,n),x(n,k)
double precision w(m,k),wa(m5,n2),wb(m)
double precision aa(m,n),bb(m),e(m)
double precision tau,toler
integer s(m),m,n,k,m5,n2,ift(k)
do i=1,k {
	do ii=1,m{
		bb(ii)=b(ii)*w(ii,i)
		do jj=1,n{
			aa(ii,jj)=a(ii,jj)*w(ii,i)
			}
		}
	call rq0(m,n,m5,n2,aa,bb,tau,toler,ift(i),x(1,i),e,s,wa,wb)
	}
return
end

#does a matrix multiply to make Y matrix for heqf bootstrap
subroutine heqfy(n,p,r,x,b,y)
integer n,p,r
double precision x(n,p),b(p,n,r),y(n,r),ddot
do i=1,r{
	do j=1,n{
		y(j,i)=ddot(p,x(j,1),n,b(1,j,i),1)
		}
	}
return
end

C Output from Public domain Ratfor, version 1.05
      subroutine rqfn(n,p,a,y,rhs,d,u,beta,eps,wn,wp,aa,nit,info)
      integer n,p,info,nit(3)
      double precision a(p,n),y(n),rhs(p),d(n),u(n),wn(n,10),wp(p,p+3),a
     *a(p,p)
      double precision one,beta,eps
      parameter( one = 1.0d0)
      call fna(n,p,a,y,rhs,d,u,beta,eps,wn(1,1),wn(1,2), wp(1,1),wn(1,3)
     *,wn(1,4),wn(1,5), wn(1,6), wp(1,2),wn(1,7),wn(1,8),wn(1,9),wn(1,10
     *),wp(1,3), wp(1,4),aa,nit,info)
      return
      end
      subroutine fna(n,p,a,c,b,d,u,beta,eps,x,s,y,z,w, dx,ds,dy,dz,dw,ds
     *dw,dxdz,rhs,ada,aa,nit,info)
      integer n,p,pp,i,info,nit(3)
      double precision a(p,n),c(n),b(p)
      double precision zero,one,mone,big,ddot,dmax1,dmin1,dasum
      double precision deltap,deltad,beta,eps,cx,by,uw,uz,mu,mua,acomp,r
     *dg,g
      double precision x(n),u(n),s(n),y(p),z(n),w(n),d(n),rhs(p),ada(p,p
     *)
      double precision aa(p,p),dx(n),ds(n),dy(p),dz(n),dw(n),dxdz(n),dsd
     *w(n)
      parameter( zero = 0.0d0)
      parameter( half = 0.5d0)
      parameter( one = 1.0d0)
      parameter( mone = -1.0d0)
      parameter( big = 1.0d+20)
      nit(1)=0
      nit(2)=0
      nit(3)=n
      pp=p*p
      call dgemv('N',p,n,one,a,p,c,1,zero,y,1)
      call stepy(n,p,a,d,y,aa,info)
      if(info .ne. 0)then
      return
      endif
      do23002 i=1,p
      do23004 j=1,p
      ada(i,j)=zero
23004 continue
23005 continue
      ada(i,i)=one
23002 continue
23003 continue
      call dtrtrs('U','T','N',p,p,aa,p,ada,p,info)
      call dcopy(pp,ada,1,aa,1)
      call dcopy(n,c,1,s,1)
      call dgemv('T',p,n,mone,a,p,y,1,one,s,1)
      do23006 i=1,n
      d(i)=one
      if(dabs(s(i)) .lt. eps)then
      z(i) = dmax1( s(i),zero) + eps
      w(i) = dmax1(-s(i),zero) + eps
      else
      z(i) = dmax1( s(i),zero)
      w(i) = dmax1(-s(i),zero)
      endif
      s(i)=u(i)-x(i)
23006 continue
23007 continue
      cx = ddot(n,c,1,x,1)
      by = ddot(p,b,1,y,1)
      uw = dasum(n,w,1)
      uz = dasum(n,z,1)
      rdg = (cx - by + uw)
23010 if(rdg .gt. eps)then
      nit(1)=nit(1)+1
      do23012 i =1,n
      d(i) = one/(z(i)/x(i) + w(i)/s(i))
      ds(i)=z(i)-w(i)
      dx(i)=d(i)*ds(i)
23012 continue
23013 continue
      call dgemv('N',p,n,one,a,p,dx,1,zero,dy,1)
      call dcopy(p,dy,1,rhs,1)
      call stepy(n,p,a,d,dy,ada,info)
      if(info .ne. 0)then
      return
      endif
      call dgemv('T',p,n,one,a,p,dy,1,mone,ds,1)
      deltap=big
      deltad=big
      do23016 i=1,n
      dx(i)=d(i)*ds(i)
      ds(i)=-dx(i)
      dz(i)=-z(i)*(dx(i)/x(i) + one)
      dw(i)=w(i)*(dx(i)/s(i) - one)
      dxdz(i)=dx(i)*dz(i)
      dsdw(i)=ds(i)*dw(i)
      if(dx(i).lt.0)then
      deltap=dmin1(deltap,-x(i)/dx(i))
      endif
      if(ds(i).lt.0)then
      deltap=dmin1(deltap,-s(i)/ds(i))
      endif
      if(dz(i).lt.0)then
      deltad=dmin1(deltad,-z(i)/dz(i))
      endif
      if(dw(i).lt.0)then
      deltad=dmin1(deltad,-w(i)/dw(i))
      endif
23016 continue
23017 continue
      deltap=dmin1(beta*deltap,one)
      deltad=dmin1(beta*deltad,one)
      if(deltap*deltad.lt.one)then
      nit(2)=nit(2)+1
      acomp=ddot(n,x,1,z,1)+ddot(n,s,1,w,1)
      g=acomp+deltap*ddot(n,dx,1,z,1)+ deltad*ddot(n,dz,1,x,1)+ deltap*d
     *eltad*ddot(n,dz,1,dx,1)+ deltap*ddot(n,ds,1,w,1)+ deltad*ddot(n,dw
     *,1,s,1)+ deltap*deltad*ddot(n,ds,1,dw,1)
      mu=acomp/dfloat(2*n)
      mua=g/dfloat(2*n)
      mu=mu*(mua/mu)**3
      do23028 i=1,n
      dz(i)=d(i)*(mu*(1/s(i)-1/x(i))+ dx(i)*dz(i)/x(i)-ds(i)*dw(i)/s(i))
23028 continue
23029 continue
      call dswap(p,rhs,1,dy,1)
      call dgemv('N',p,n,one,a,p,dz,1,one,dy,1)
      call dpotrs('U',p,1,ada,p,dy,p,info)
      call daxpy(p,mone,dy,1,rhs,1)
      call dgemv('T',p,n,one,a,p,rhs,1,zero,dw,1)
      deltap=big
      deltad=big
      do23030 i=1,n
      dx(i)=dx(i)-dz(i)-d(i)*dw(i)
      ds(i)=-dx(i)
      dz(i)=mu/x(i) - z(i)*dx(i)/x(i) - z(i) - dxdz(i)/x(i)
      dw(i)=mu/s(i) - w(i)*ds(i)/s(i) - w(i) - dsdw(i)/s(i)
      if(dx(i).lt.0)then
      deltap=dmin1(deltap,-x(i)/dx(i))
      else
      deltap=dmin1(deltap,-s(i)/ds(i))
      endif
      if(dz(i).lt.0)then
      deltad=dmin1(deltad,-z(i)/dz(i))
      endif
      if(dw(i).lt.0)then
      deltad=dmin1(deltad,-w(i)/dw(i))
      endif
23030 continue
23031 continue
      deltap=dmin1(beta*deltap,one)
      deltad=dmin1(beta*deltad,one)
      endif
      call daxpy(n,deltap,dx,1,x,1)
      call daxpy(n,deltap,ds,1,s,1)
      call daxpy(p,deltad,dy,1,y,1)
      call daxpy(n,deltad,dz,1,z,1)
      call daxpy(n,deltad,dw,1,w,1)
      cx=ddot(n,c,1,x,1)
      by=ddot(p,b,1,y,1)
      uw = dasum(n,w,1)
      uz = dasum(n,z,1)
      rdg=(cx-by+uw)
      goto 23010
      endif
23011 continue
      call daxpy(n,mone,w,1,z,1)
      call dswap(n,z,1,x,1)
      return
      end

C Output from Public domain Ratfor, version 1.05
      subroutine brutpow(n,p,m,h,a,b,c,x,tau,u,xh,d,jminz,nflag)
      integer n,p,m
      double precision x(p),a(n,p),b(n),c(n)
      double precision u(p,p),d(p),xh(p)
      double precision zero, one,tau,pow,minz,z
      integer h(p,m),k,findk,jminz,nflag
      parameter(zero = 0.0d0, one = 1.d0)
      jminz = 1
      minz = pow(n,p,x,a,b,c,tau)
      do23000 j = 2,m 
      k = findk(p,h(1,j),h(1,j-1))
      if(k .eq. 0)then
      nflag = 4
      return
      endif
      call pivot(n,p,h(1,j-1),h(k,j),h(k,j-1),a,u,d,xh,nflag)
      if(nflag .gt. 0)then
      return
      endif
      do23006 i = 1,p
      xh(i) = b(h(i,j))
23006 continue
23007 continue
      call dgemv('N',p,p,one,u,p,xh,1,zero,x,1)
      z = pow(n,p,x,a,b,c,tau)
      if(z .lt. minz)then
      minz = z
      jminz = j
      endif
23000 continue
23001 continue
      return
      end
      integer function findk(p,h,g)
      integer p,k,h(p),g(p)
      findk = 0
      do23010 k = 1,p
      if(h(k) .ne. g(k))then
      findk = k
      goto 23011
      endif
23010 continue
23011 continue
      return
      end

crq.fit.pow <- function(x, y, yc, tau=0.5, weights=NULL, start, left=TRUE,maxit = 500){

 x <- as.matrix(x)
 y <- as.vector(y)
 n <- nrow(x)
 p <- ncol(x)
 if(missing(yc)) {
     yc <- rep(0,n) 
     if(!left) warning("default right (!?) censoring at zero")
    } 
 yc <- as.vector(yc)
 if(length(weights)){
     if (any(weights < 0))
         stop("negative weights not allowed")
     contr <- attr(x, "contrasts")
     x <- x * weights
     y <- y * weights
     }
 if(left) {
	x <- -x
	y <- -y
	yc <- -yc
	tau <- 1-tau
	}
# Starting value of beta 
if(missing(start)){ 
   tol <- .Machine$double.eps^(2/3)
   r <- rq.fit.br(x,y,tau)$residuals
   h <- which(abs(r) < tol)[1:p]
   }
else if(start[1] == "global"){
# Generate H matrix 
   H <- combos(n,p)
   m <- ncol(H)

# Starting value of beta 
   U <- solve(x[H[,1],])
   bh <- U %*% y[H[,1]]

f <- .Fortran("brutpow",
        as.integer(n),
        as.integer(p),
        as.integer(m),
        as.integer(H),
        as.double(x),
        as.double(y),
        as.double(yc),
        as.double(bh),
        as.double(tau),
        as.double(U),
        double(p),
        double(p),
        kminz = integer(1),
        nflag = as.integer(0))
if(f$nflag!=0)
        warning(switch(f$nflag,
                "Error in pivot:  hout not in h",
                "Error in pivot:  hin in h",
                "Error in pivot:  hin out of bounds",
                "Error in findk:  k not found"
                ))
   k  <- f$kminz
   bh <- solve(x[H[,k],],y[H[,k]])
   residuals <- as.matrix(y - pmin(yc, x %*% bh))

   return(list(coefficients= bh,residuals = residuals))
  }
else if(is.numeric(start) && length(start) == p){
	if(all(start %in% 1:n)) h <- start
	else  h <- order(abs(y - x %*% start))[1:p]
	}
else
   stop("Invalid starting value")

xhinv <- try(solve(x[h,]))
if(class(xhinv)[1] == "try-error")
	stop("Singular basic solution generated by 'start'")

f <- .Fortran("powell", 
	as.integer(n), 
	as.integer(p), 
	as.integer(2*p), 
	as.double(x),
	as.double(y), 
	as.double(yc), 
	coef = double(p), 
	as.double(tau), 
	as.integer(h), 
	double(n), 
	as.double(xhinv), 
	double(n), 
	double(2*p),
	double(p),
	double(p),
	as.integer(maxit), 
	nflag = as.integer(0))
	
if(f$nflag!=0) 
	warning(switch(f$nflag, "Max iterations reached",
		"Solution may be nonunique",
		"Error in pivot:  hout not in h",
		"Error in pivot:  hin in h",
		"Error in pivot:  hin out of bounds"
		))
coef <- f$coef
residuals <- as.matrix(y - pmin(yc, x %*% coef))

return (list(coefficients= coef, residuals = residuals))
}


Curv <- function (y,  yc, ctype = c("left", "right")) 
{
    nn <- length(y)
    if(length(yc) != nn)
        stop("Event times and censoring times of different length")
    ctype <- match.arg(ctype)
    if(ctype == "right" && any(y > yc))
        stop("Event times can not exceed ctimes for right censoring")
    if(ctype == "left" && any(y < yc))
        stop("Event times can not be less than ctimes for left censoring")
    if(!(ctype == "left" || ctype == "right"))
        stop("Invalid ctype for method Powell")
    ss <- cbind(y, yc)
    dimnames(ss) <- list(NULL, c("time", "ctime"))
    attr(ss, "ctype") <- ctype
    attr(ss, "type") <- ctype
    class(ss) <- "Surv"
    ss
}

QTECox <- function(x, smooth = TRUE){
# compute quantile treatment effect for a Cox PengHuang fit
g <- survival::survfit(x)
if(smooth)
        g <- supsmu(1-g$surv,g$time)
taus <- (g$x[-1] + g$x[-length(g$x)])/2
Qhat <- (g$y[-1] + g$y[-length(g$y)])/2

dQ <- diff(Qhat)/diff(taus)
taus <- taus[-1]; Qhat <- Qhat[-1]
QTE <- outer(dQ * (1-taus) * log(1-taus) / Qhat, coef(x))
list(QTE = QTE , taus = taus)
}
boot.crq <- function(x, y, c, taus, method, ctype = "right", R=100,  
		mboot,  bmethod = "jack", ...)
        {
        n <- length(y)
        p <- ncol(x)
        if(missing(mboot)) {
                if(bmethod=="jack") mboot <- 2*ceiling(sqrt(n))
                else mboot <- n }
	if(length(taus) > 1)
	    A <- array(0,dim=c(p,length(taus),R))
	else
	    A <- matrix(0,p,R)
        for (i in 1:R){
                if(bmethod == "jack") { 
                        s <- sample(1:n,mboot)
                        yb <- y[-s]
                        xb <- x[-s, ]
                        cb <- c[-s]  
                        w <- rep(1,n-mboot)
                        }
                else if(bmethod == "xy-pair"){
                        w <- table(sample(1:n,mboot,replace=TRUE))
                        s <- as.numeric(names(w))
                        w <- as.numeric(w)
                        yb <- y[s]
                        xb <- x[s,]
                        cb <- c[s]
                        }
                else if(bmethod == "Bose"){
                        w <- rexp(n)
                        yb <- y
                        xb <- x
                        cb <- c
                        }
                else
                        stop("invalid bmethod for boot.crq")
		if(method == "Portnoy")
                	a <- crq.fit.por(xb,yb,cb, weights = w, ctype = ctype, ... )
		#else if(method == "Portnoy2")
                	#a <- crq.fit.por2(xb,yb,cb, weights = w, ctype = ctype, ... )
		else if(method == "PengHuang")
                	a <- crq.fit.pen(xb,yb,cb, weights = w, ctype = ctype, ... )
		else
			stop("Invalid method for boot.crq")
                if((i %% floor(R/10)) == 0 & n > 100000)
                        cat(paste("bootstrap roughly ",100*(i/R)," percent complete\n"))
		if(length(taus) > 1)
		    A[,,i] <- coef(a,taus)
		else
		    A[,i] <- coef(a,taus)
                }
        list(A = A, n = length(y), mboot = mboot, bmethod = bmethod)
        }

coef.crq <- function(object, taus = 1:4/5, ...)
        {
        # Extract coefficients from the crq solution array 
        
        if(min(taus) < 0 || max(taus) > 1) stop("taus out of range [0,1]")
	if(length(object$tau) == 1){
        	coef <- object$coefficients
        	return(coef)
        	}
        taus <- sort(taus)
        S <- object$sol
        ctype <- object$ctype
        r <- S[1, ]
        r <- c(r[1],r)
	if(is.unsorted(r)) r <- r[-length(r)] #kludge until why this happens is found
        B <- S[-1,,drop = FALSE]
	B <- t(cbind(B,B[,ncol(B),drop = FALSE]))
        ts <- taus[taus > min(r) & taus < max(r)]
        bin <- findInterval(ts,r)
        wgt <- (ts - r[bin])/(r[bin + 1] - r[bin])
        binlag <- bin - 1
        binlag[binlag == 0] <- 1
        coef <- t(wgt * B[bin, , drop = FALSE] + (1 - wgt) * B[binlag, , drop = FALSE])
        nna <- length(taus) - length(ts)
        if(nna > 0) {
           if (ctype == "left")
              coef <- cbind(matrix(NA, nrow(coef), nna), coef)
           else
              coef <- cbind(coef, matrix(NA, nrow(coef), nna))
        }
        taulabs <- paste("tau=", format(round(taus, 3)))
        dimnames(coef)[[2]] <- taulabs
        coef[-nrow(coef),]  # Delete Qhat entries
        }

crq <- function (formula, taus, data, subset, weights, na.action, 
	method = c("Powell","Portnoy","Portnoy2", "PengHuang"), contrasts = NULL, ...)
{
    if(!requireNamespace("survival", quietly = TRUE))
	stop("crq requires survival package to be installed")
    #if(method == "Portnoy2") stop("Portnoy2 method not (yet) implemented")
    Surv <- survival::Surv
    call <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset", "weights", "na.action"),
        names(mf), 0)
    mf <- mf[c(1, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1]] <- as.name("model.frame")
    mf <- eval.parent(mf)
    if (method == "model.frame")
        return(mf)
    mt <- attr(mf, "terms")
    X <- model.matrix(mt, mf, contrasts)
    weights <- as.vector(model.weights(mf))
    Y <- model.extract(mf, "response")
    eps <- .Machine$double.eps^(2/3)
    if(!inherits(Y,"Surv"))
        stop("Response must be a survival object")
    method <- match.arg(method)
    if(method == "Powell") {
	ctype <- attr(Y, "ctype")
	if(!ctype %in% c("right","left")) 
             stop("Only right or left censoring Surv objects are allowed")
	left <- (ctype == "left")
        if (any(taus < -eps) || any(taus > 1 + eps))
            stop("invalid taus:  taus should be >= 0 and <= 1")
	y <- Y[,1]
	cen <- Y[,2]
	if (length(taus) > 1) {
          coef <- matrix(0, ncol(X), length(taus))
          fitted <- resid <- matrix(0, nrow(X), length(taus))
          for (i in 1:length(taus)) {
            z <- crq.fit.pow(X, y, cen, tau = taus[i], weights, left = left, ...)
            coef[, i] <- z$coefficients
            resid[, i] <- z$residuals
            fitted[, i] <- y - z$residuals
          }
          taulabs <- paste("tau=", format(round(taus, 3)))
          dimnames(coef) <- list(dimnames(X)[[2]], taulabs)
          dimnames(resid) <- list(dimnames(X)[[1]], taulabs)
          fit <- list(coefficients = coef, residuals = resid, fitted.values = fitted)
	  fit$tau <- taus
	  class(fit) <- "crqs"
	}
	else {
          fit <- crq.fit.pow(X, y, cen, tau = taus, weights, left = left, ...)
	  fit$tau <- taus
	  class(fit) <- "crq"
	}
     }
    else if(method == "Portnoy"){
        ctype <- "right"
        if(attr(Y,"type") != "right"){
            if(attr(Y,"type") == "left") ctype <- "left"
            else stop("Only right censoring Surv objects are allowed for Portnoy method")
            }
        y <- Y[,1]
        cen <-  Y[,2]
        fit <- crq.fit.por(X, y, cen, weights, ctype = ctype,   ...)
	class(fit) <- "crq"
	}
    else if(method == "Portnoy2"){
        ctype <- "right"
        if(attr(Y,"type") != "right"){
            if(attr(Y,"type") == "left") ctype <- "left"
            else stop("Only right censoring Surv objects are allowed for Portnoy method")
            }
        y <- Y[,1]
        cen <-  Y[,2]
        fit <- crq.fit.por2(X, y, cen, weights, ctype = ctype,   ...)
	class(fit) <- "crq"
	}
    else if(method == "PengHuang"){
        ctype <- "right"
        if(attr(Y,"type") != "right"){
            if(attr(Y,"type") == "left") ctype <- "left"
            else stop("Only right censoring Surv objects are allowed for Portnoy method")
            }
        y <- Y[,1]
        cen <-  Y[,2]
        fit <- crq.fit.pen(X, y, cen, weights,  ctype = ctype, ...)
	class(fit) <- "crq"
	}
    else
	stop("Method not defined for crq")

fit$terms <- mt
fit$call <- call
fit$formula <-  formula(mt)
fit$method <-  method
fit$contrasts <- contrasts 
fit$ctype <-  ctype
attr(fit, "na.message") <- attr(m, "na.message")
fit
}
predict.crqs <-
function (object, newdata, type = NULL, ...) {
pred <-  predict.rqs(object, newdata, ...) 
	}
predict.crq <-
function (object, newdata,  ...) {
method <- object$method
if(method %in% c("Portnoy","Portnoy", "PengHuang")){ #Kludge to make crq sol matrix look like rq sol matrix 
	p2 <- nrow(object$sol)
	object$sol <- object$sol[c(1,p2,p2,2:(p2-1)),]
	}
pred <- switch(method,
	"Powell" = predict.rq(object, newdata,  ...), 
	"Portnoy"  = predict.rq.process(object, newdata,  ...), 
	"Portnoy2"  = predict.rq.process(object, newdata,  ...), 
	"PengHuang" = predict.rq.process(object, newdata,  ...) 
	)
}
crq.fit.por <- function(x, y, cen, weights = NULL, grid, ctype = "right")
{
      if(!is.matrix(x))
	  stop("x must be a matrix")
      if(!length(dimnames(x)))
	  dimnames(x)[[2]] <- paste("V",1:p)
      p <- ncol(x)
      n <- length(y) 
      cen <- 1 - cen #NB: Fortran routine wants censoring indicator flipped (!!!)
      mp <- max(n + 5 + max(1, sum(cen)),n+p)
      eps <- 1e-04
      kmax <- 10000 # perhaps this should be parameter in future
      if(length(weights)){
		if (any(weights < 0)) 
		        stop("negative weights not allowed")
    		contr <- attr(x, "contrasts")
    		x <- weights * x
    		y <- weights * y
		}
    if(ctype == "left") y <- -y
    if(missing(grid))  grid <-  seq(1/n,1-1/n,by=1/(5 + 3 * n^.4))
    if(is.numeric(grid)){
		ginit <- min(grid)
		dgrid <- diff(grid)
		gstep <- median(dgrid)
		if(any(dgrid <0)) stop("grid is not monotonic")
		if(gstep < eps) stop("grid stepsize too small")
		nsol <- 3*n
		mw = -1
		}
      else if(grid  == "pivot"){
		nsol <- 3*n
		ginit <- 1/(2*n)
		gstep <- 1/(2*n)
		mw <- min(10, mp) # Somewhat arbitrary see crq.f
		}
      else 
		stop("Invalid grid")
	z <- .Fortran("crqf",
		as.integer(n),
		as.integer(p),
                as.integer(mp), 
		as.integer(p+2),
		as.double(x),
		as.double(y),
		as.integer(cen),
		as.double(ginit),
                as.integer(mw),
                as.double(gstep),
		ift = integer(1),
		h = integer(p),
                xh = double(p*p),
		wa = double(mp*p),
		wb = double(mp),
		wc = double(mp*(p+2)),
		wd = double(mp),
		we = double(mp),
		wf = double(p),
		iflag = integer(mp),
                as.integer(nsol),
		sol = double(nsol*(p+2)),
                lsol = integer(1),
		icen = integer(n),
		tcen = double(n),
		lcen = integer(1))
	nw <- z$h[1]
	flag <- z$ift
	msg <- switch(flag,
		paste("Error in input dimensions, n,p,mw "),
		paste("Error in input dimensions, n,p,mw "),
		paste("Error in input dimensions, n,p,mw "),
		paste("Less than p=",p,"observations above tau = 0 solution"),
		paste("Possible degeneracy at",nw,"tau values.",
          		"$tau.degen: first mp =", n + 5 + sum(cen)," such tau values"), 
		paste("Number of pivots to be saved in sol > nsol.",
          		"Redefine nsol: use nsol < n to save for tau = i/(nsol-1)"),
		paste("Error with partial return: possible degeneracies",
         		"Max number of rq calls exceeded: dither x or increase mw"),
		paste("Premature stop: defective conditional distribution"),
		paste("Simplex iteration limit exceeded -- consider dithering y"))
	#if(flag > 0 && flag != 5 && flag < 10)
	if(flag %in% c(1:4,6,7))
		ifelse(flag <= 3,stop(msg),warning(msg))
	J <- z$lsol
	B <- matrix(z$sol, nrow=p+2, ncol=nsol, byrow=FALSE)[,1:J, drop = FALSE]
	if(B[1,J] < B[1,J-1]) B <- B[,-J] # SLP hack Oct 27 2014
	ic <- z$icen
	sp <- (1:n)[ic == 1]
	tsp <- z$tcen[sp]
        t1 <- z$wd[1:nw]
        if(ctype == "left") {
            B[1,] <- 1 - B[1,]
            B[-1,] <- - B[-1,]
            B <- B[,ncol(B):1]
            }
	dimnames(B) <- list(c("tau",dimnames(x)[[2]],"Qbar"),NULL)
	a <- list(sol=B, Isplit=sp, tsp = tsp, status=ic, ctype = ctype)
	fitted <- x %*% B[-c(1,dim(B)[1]),]
	dimnames(fitted) <- list(NULL, paste("tau=",round(B[1,],4))) 
	a$fitted.values <- fitted
	class(a) <- "crq"
	return(a) 
	}
crq.fit.pen <- function(x, y, cen, weights=NULL,grid, ctype = "right" ){
      p <- ncol(x)
      n <- length(y)
      if(missing(grid)) grid <-  seq(1/n,1-1/n,by=min(0.01,1/(2*length(y)^.7))) 
      if(!is.numeric(grid)) stop("Invalid grid")
      if(any(grid < 0) || any(grid > 1)) stop("Invalid grid")
      m <- length(grid)
      xbar <- apply(x,2,mean)
      if(length(weights)){
                if (any(weights < 0))
                        stop("negative weights not allowed")
                contr <- attr(x, "contrasts")
                x <- x * weights
                y <- y * weights
                }
        if(ctype == "left") y <- -y
        s <- rep(0,n)
        u <- rep(1,n)
        d <- rep(1,n)
        r <- rep(1,p)
        B <- matrix(0,p,m)
        cc <- as.logical(cen)
        y1 <- y[cc]
	n1 <- length(y1)
        x1 <- x[cc,]
	z <- .Fortran("crqfnb", as.integer(n), as.integer(p), 
		a1 = as.double(t(as.matrix(x1))), c1 = as.double(-y1), n1=as.integer(n1),
		as.double(x), as.double(y),as.double(cen),B =as.double(B),
		g = as.double(grid),m = as.integer(m), as.double(r), 
		as.double(s), as.double(d), as.double(u),
        	wn = double(n1 * 9), wp = double((p + 3) * p),
        	info = integer(1))
	J <- z$m - 1
	B <- matrix(-z$B, p, m)
	B <- B[,1:J,drop = FALSE]
        qhat <- t(xbar) %*% B
        B <- rbind(grid[1:J],B,qhat)
        dimnames(B) <- list(c("tau",dimnames(x)[[2]],"Qhat"),NULL)
        if(ctype == "left") {
            B[1,] <- 1 - B[1,]
            B[-1,] <- - B[-1,]
            B <- B[,ncol(B):1]
            }
        B  <- list(sol=B, ctype = ctype)
	class(B) <- "crq"
	B
        }
plot.summary.crqs <-
function (x, nrow = 3, ncol = 3, CoxPHit = NULL, ...) {
    taus <- function(x) x$tau
    xx <- unlist(lapply(x, taus))
    coef <- lapply(x, coefficients)
    dots <- list(...)
    p <- nrow(coef[[1]])
    k <- ncol(coef[[1]])
    if(k != 6) stop("summary.crqs object has wrong column dimension")
    m <- length(xx)
    blab <- dimnames(coef[[1]])[[1]]
    a <- array(unlist(coef), c(p, k, m))
    if(length(CoxPHit))
	CoxQTE <- QTECox(CoxPHit)
    oldpar <- par(no.readonly=TRUE)
    par(mfrow = c(nrow, ncol))
    for (i in 2:p) {
         b  <- a[i, 1, ]
         bl <- a[i, 2, ]
         bu <- a[i, 3, ]
	if(length(dots))
	    plot(rep(xx, 2), c(bl, bu), type = "n", ...)
	else
	    plot(rep(xx, 2), c(bl, bu), xlab = "", ylab = "", type = "n")
        title(paste(blab[i]), cex = 0.75)
        polygon(c(xx, rev(xx)), c(bl, rev(bu)), col = "LightSkyBlue")
        points(xx, b, cex = 0.5, pch = "o", col = "blue")
        lines(xx, b, col = "blue")
        abline(h = 0)
        if(length(CoxPHit)) {
	    lines(CoxQTE$taus,CoxQTE$QTE[,i-1],col="red")
            }
    }
par(oldpar)
}
summary.crq <-
function (object, taus = 1:4/5, alpha = .05, se = "boot", covariance = TRUE, ...) {
    mt <- terms(object)
    m <- model.frame(object)
    x <- model.matrix(mt, m, contrasts = object$contrasts)
    Y <- model.response(m)
    method <- object$method
    ctype <- object$ctype
    y <- Y[,1]
    cen  <- Y[,2]
    wt <- as.vector(model.weights(object$model))
    if (!is.null(wt)) {
        resid <- resid * wt
        x <- x * wt
        y <- y * wt
	cen <- cen * wt
    }
    if(method == "Powell"){
       coef <- coefficients(object)
       vnames <- dimnames(x)[[2]]
       resid <- object$residuals
       tau <- object$tau
       n <- length(resid)
       p <- length(coef)
       rdf <- n - p
       if(rdf < 1) stop("residual degrees of freedom nonpositive")
       if (se == "boot") {
           if(attr(Y,"type") == "left")
              s <- cen  <  x %*% coef
           else
              s <- cen  >  x %*% coef
           B <- boot.rq(x[s, ], y[s], tau, ...)
           cov <- cov(B$B)
           serr <- sqrt(diag(cov))
           }
       else stop("Only boot method is implemented for crq inference")
       coef <- array(coef, c(p, 6))
       fact <- qt(1 - alpha/2, rdf)
       coef[, 2] <- coef[,2] - fact * serr
       coef[, 3] <- coef[,3] + fact * serr
       coef[, 4] <- serr
       coef[, 5] <- coef[, 1]/coef[, 4]
       coef[, 6] <- 2 * (1 - pt(abs(coef[, 5]),rdf)) 
       cnames <- c("Value","Lower Bd","Upper Bd","Std Error","T Value","Pr(>|t|)")
       dimnames(coef) <- list(vnames, cnames)
       object <- object[c("call", "terms")]
       if (covariance == TRUE) 
           object$cov <- cov
       object$B <- B
       object$coefficients <- coef
       object$rdf <- rdf
       object$tau <- tau
       class(object) <- "summary.crq"
       return(object)
       }
    else if(method == "Portnoy" || method == "Portnoy2" || method == "PengHuang") {
       coef <- as.matrix(coef(object,taus))
       coef <- coef[,apply(coef,2,function(x) any(!is.na(x))),drop = FALSE] # Delete NA columns if any
       if(ctype == "right") taus <- taus[1:ncol(coef)]
       else  taus <- taus[(1 + length(taus)-ncol(coef)):length(taus)]
       B <- boot.crq(x, y, cen, taus, method = method, ctype = ctype, ...)
       bmethod <- B$bmethod
       nas <- apply(is.na(B$A[1,,, drop = TRUE]),1,sum)
       Brep <- dim(B$A)[3]
       p <- dim(B$A)[1]
       if(bmethod == "jack") 
	   sqmn <- sqrt((B$n - B$mboot - p + 1)/B$mboot)
       else 
	   sqmn <- sqrt(B$mboot/B$n)
       fact <-   qnorm(1 - alpha/2)/qnorm(.75)
       A <- apply(B$A, 1:2, quantile, probs = 1:3/4, na.rm = TRUE)
       #D <- .5 * fact *(A[3,,]-A[1,,]) * sqmn
       #L <- coef - D
       #U <- coef + D
       DU <- fact *(A[3,,]-A[2,,]) * sqmn
       DL <- fact *(A[2,,]-A[1,,]) * sqmn
       L <- coef - DL
       U <- coef + DU
       S <- (U - L)/(2 * qnorm(1 - alpha/2))
       T <- coef/S
       P <- 2 * (1 - pnorm(abs(T)))
       G <- list()
       cnames <- c("Value","Lower Bd","Upper Bd","Std Error","T Value","Pr(>|t|)")
       for(i in 1:length(taus)){
	   tab <- cbind(coef[,i],L[,i],U[,i],S[,i],T[,i],P[,i])
	   dimnames(tab)[[2]] <- cnames
	   if(covariance) 
	       cov <- var(t(B$A[,i,]), na.rm = TRUE) * sqmn^2
	   else
	       cov <- NULL
	   G[[i]] <- list(tau = taus[i], coefficients = tab, NAs = nas[i], 
			  cov = cov, Brep = Brep, bmethod = bmethod)
	   }
       class(G) <- "summary.crqs"
       return(G)
       }
    else
       stop("Invalid method for summary.crq")
   }
"summary.crqs" <-
function (object, ...) {
	if(!object$method == "Powell")
		stop("Invalid method")
        taus <- object$tau
        xsum <- as.list(taus)
        for(i in 1:length(taus)){
                xi <- object
                xi$coefficients <- xi$coefficients[,i]
                xi$residuals <- xi$residuals[,i]
                xi$tau <- xi$tau[i]
                class(xi) <- "crq"
                xsum[[i]] <- summary(xi, ...)
                }
        class(xsum) <- "summary.crqs"
        xsum
        }

print.crq <- function(x, ...){
	if(!is.null(cl <- x$call)) {
                cat("Call:\n")
                dput(cl)
		}
        coef <- coef(x, ...)
        cat("\nCoefficients:\n")
	print.default(coef)
        invisible(x)
	}

print.summary.crqs <- function(x, ...)
    lapply(x,print.summary.crq)
print.summary.crq <- function (x, digits = max(5, .Options$digits - 2), ...) {
    coef <- x$coefficients
    tau <- x$tau
    if(length(x$NAs)) NAs <- x$NAs
    else NAs <- 0
    cat("\ntau: ")
    print(format(round(tau, digits = digits)), quote = FALSE, ...)
    if(NAs > 0) 
       cat(paste("   Number of NA Bootstrap Replications: ", NAs, "out of", x$Brep))
    cat("\nCoefficients:\n")
    print(format(round(coef, digits = digits)), quote = FALSE, ...)
    invisible(x)
}

#  SLP: Mar 31 2015 (Revised RWK:  Aug  2015)
#  An R version of crq.fit.por without any frills on a fixed grid.
crq.fit.por2 <- function(x, y, cen, weights = NULL, grid, ctype = "right") {
    method <- "fn"
    BIG <- 1e+6
    ztol <- 1e-6
    n <- length(y)
    p <- ncol(x)
    cen <- as.logical(cen)
    uncen <- !cen
    if(sum(uncen) < p)  
	stop("too few uncensored observations") 
    sol <- matrix(NA, p+2,1)
    if (length(weights)) {
        if (any(weights < 0)) 
            stop("negative weights not allowed")
        contr <- attr(x, "contrasts")
        x <- weights * x
        y <- weights * y
    }
    w <- rep(1,n)
    if (ctype == "left")  y <- -y
    if(missing(grid)) { # Fixme:  else if ...
	g0 <- 1/(2*n)
	dg <- 1/(5 + 3*n^.4)
	grid <- seq(g0,1-g0,by=dg)
    }
    repeat { # Peel censored points below g0 fit
	z <- rq.wfit(x, y, tau = g0, weights = w, method = method)
	J <- which(abs(z$resid) < ztol & cen)
	if(length(J)){
	    y <- y[-J]
	    x <- x[-J,]
	    w <- w[-J]
	    cen <- cen[-J]
	}
	else break
    }
    n <- length(y)
    U <- which(cen)
    if(length(cen) == 0) { # No censored points left!
	z <- rq(y ~ x - 1, tau = grid, weights = w, method = method)
	return(coef(z))
    }
    else { # Data Augmentation:
	m <- sum(cen)
	x <- rbind(x, x[cen, ])
	y <- c(y, rep(BIG, m))
	w[cen] <- 0
	w <- c(w, rep(1, m))
    }
    taw <- rep(0, length(y))
    K <- NULL
    sol <- matrix(NA, p+2, length(grid))
    b <- z$coef
    xbar <- apply(x, 2, mean)
    sol[,1] <- c(0, b, sum(xbar * b))
    for(i in 2:length(grid)){
	tau <- grid[i]
	if(length(K)){ # Update w
	    w[K] <- (tau - taw[K])/(1 - taw[K])
	    idx <- which(which(cen) %in% K)[rank(K)]
	    w[n + idx] <- 1 - w[K]
	}
	z <- rq.wfit(x, y, tau = tau, weights = w, method = method)
	b0 <- b
	b <- z$coef
	sol[,i] <- c(tau, b, sum(b * xbar))
	if(any(abs(z$resid) < ztol & (y > BIG - 1))) break
	J <- U[z$resid[U] < 0] 
	if(length(J)){
	    U <- setdiff(U, J)
	    K <- union(K, J)
	    y0 <- x[J,,drop = FALSE] %*% b0
	    y1 <- y[J] - z$resid[J]
	    gstep <- grid[i] - grid[i-1]
	    taw[J] <- tau - gstep * (y1 - y[J])/(y1 - y0)
	}
    }
    if (ctype == "left") {
        sol[1, ] <- 1 - sol[1, ]
        sol[-1, ] <- -sol[-1, ]
    }
    dimnames(sol) <- list(c("tau", dimnames(x)[[2]], "Qbar"), NULL)
    return(list(sol = sol, ctype = ctype)) 
}

C Output from Public domain Ratfor, version 1.05
      subroutine sakj(x,z,p,iker,dens,psi,score,nx,nz,h,alpha,kappa,xlam
     *)
      double precision dens(nz),score(nz),psi(nz),h,kappa
      double precision z(nz),x(nx),xlam(nx),p(nx),qrange,pi
      double precision con1,sum,sqsum,xsd,a,fifth,hinv,half
      double precision xn,xker,dxker,ddxker,fact,xponen,alpha,glog,zero,
     *one,two
      parameter( zero = 0.d0)
      parameter( one = 1.d0)
      parameter( two = 2.d0)
      parameter( four = 4.d0)
      parameter( half = 0.5d0)
      parameter( fifth = 0.2d0)
      parameter( pi = 3.141593d0)
      xn=nx
      if(iker.eq.0)then
      con1=one/sqrt(2.0*pi)
      else
      if(iker.eq.1)then
      con1=one/pi
      endif
      endif
      if(h.le.0.)then
      sum=0.
      sqsum=0.
      do23006 i=1,nx
      sqsum=sqsum+x(i)*x(i)*p(i)
      sum=sum+x(i)*p(i)
23006 continue
23007 continue
      xsd=dsqrt(sqsum-sum*sum)
      sum=zero
      i=1
23008 if(.not.(i.lt.nx))goto 23010
      sum=sum+p(i)
      if(sum.lt..25)then
      goto 23009
      else
      qrange=x(i)
      goto 23010
      endif
23009 i=i+1
      goto 23008
23010 continue
      sum=one
      i=nx
23013 if(.not.(i.gt.0))goto 23015
      sum=sum-p(i)
      if(sum.gt..75)then
      goto 23014
      else
      qrange=x(i)-qrange
      goto 23015
      endif
23014 i=i-1
      goto 23013
23015 continue
      a=min(xsd,qrange/1.34)
      h=kappa*a/(xn**fifth)
      endif
      hinv=one/h
      do23018 j=1,nx 
      xker=0.
      if(iker.eq.0)then
      do23022 i=1,nx 
      xponen=(x(j)-x(i))*hinv
      xponen=half*xponen**2
      xker=xker+p(i)*exp(-xponen)*hinv
23022 continue
23023 continue
      else
      if(iker.eq.1)then
      do23026 i=1,nx 
      xponen=(x(j)-x(i))*hinv
      xker=xker+p(i)*hinv/(1+xponen**2)
23026 continue
23027 continue
      endif
      endif
      xlam(j)=con1*xker
23018 continue
23019 continue
      glog=zero
      do23028 i=1,nx
      glog=glog+p(i)*log(xlam(i))
23028 continue
23029 continue
      g=exp(glog)
      ginv=one/g
      do23030 i=1,nx 
      xlam(i)=hinv/((xlam(i)*ginv)**(-alpha))
23030 continue
23031 continue
      do23032 j=1,nz 
      xker=zero
      dxker=zero
      ddxker=zero
      if(iker.eq.0)then
      do23036 i=1,nx 
      xponen=(z(j)-x(i))*xlam(i)
      fact=exp(-half*xponen*xponen)*xlam(i)
      xker=xker+p(i)*fact
      dxker=dxker-p(i)*fact*xponen*xlam(i)
      ddxker=ddxker- p(i)*fact*(one - xponen**2)*xlam(i)**2
23036 continue
23037 continue
      else
      if(iker.eq.1)then
      do23040 i=1,nx 
      xponen=(z(j)-x(i))*xlam(i)
      fact=xlam(i)/(one+xponen**2)
      xker=xker+p(i)*fact
      dxker=dxker-p(i)*two*xponen*fact**2
      ddxker=ddxker- p(i)*two*(fact**2)*(xlam(i)- four*(xponen**2)*fact)
23040 continue
23041 continue
      endif
      endif
      dens(j)=con1*xker
      psi(j)=-(dxker/xker)
      score(j)=(dxker/xker)**2-ddxker/xker
23032 continue
23033 continue
      return
      end

        INTEGER FUNCTION IDMIN(N,X,INCX)
        INTEGER I,N,INCX
        DOUBLE PRECISION X(INCX,*),SMIN
C
C THIS FUNCTION RETURNS THE INDEX OF THE SMALLEST (ALGEBRAIC)
C COMPONENT OF X.  ADAPTED FROM PORT3 LIBRARY -- FEB 2008.
C ONLY EVERY INCXTH COMPONENT OF X IS CONSIDERED.
C
        IDMIN=0
        IF(N.EQ.0) RETURN
        SMIN=X(1,1)
        IDMIN=1
        DO 10 I=1,N
        IF(SMIN.LE.X(1,I)) GO TO 10
           SMIN=X(1,I)
           IDMIN=I
 10     CONTINUE
        RETURN
        END

C  Comments, bug reports, etc. should be sent to:  portnoy@stat.uiuc.edu
C
      SUBROUTINE CRQF(M,N,MPLUS,N2,X,Y,C,TZERO,MAXW,STEP,IFT,H,XH,
     * WA,WB,WC,WD,WE,WF,IA,NSOL,SOL,LSOL,ICEN,TCEN,LCEN)
C
C     M = number of Observations
C     N = Number of Parameters
C     MPLUS = row dim of X, WA, and WC .GE. M+1
C     N2 = N+2
C     X is the X matrix (MPLUS BY N) (includes censored obs)
C     Y is the y vector (M) (includes censored obs)
C     C is the censoring integer(M) vector, 1 if obs is censored,
C                                           0 otherwise
C     TZERO is the initial tau value
C     MAXW ( .LE. MPLUS ): Maximal number of calls to weighted RQ
C           for possible degeneracies. If exceeded (IFT = 7), dither.
C           If  MAXW .LE. 0, don't pivot -- use only weighted rq
C     STEP: Step size for weighted rq at degeneracy
C     IFT exit code:
C        0:  ok
C        1:  M < 0  or  N < 0  OR  M < N
C	 2:  MPLUS .LT. M + 1  or  N2 .NE. N+2
C        3:  MAXW > MPLUS
C        4:  less than N noncensored obs above the tau=0 soln
C        5:  possible degeneracy, rq called at tau + step, see IA
C        6:  LSOL becomes greater than NSOL
C        7:  MAXW exceeded
C        8:  weighted rq tries to include infinite basis element
C     H  is an integer work vector (N) ( = basis indices )
C     XH is a double precision work array (N by N)
C     WA is a double precision work array (MPLUS by N)
C     WB is a double precision work vector (MPLUS)
C     WC is a double precision work array (MPLUS by N+2)
C     WD is a double precision work vector (MPLUS)
C     WE is a double precision work vector (MPLUS)
C     WF is a double precision work vector (N)
C     IA is an integer work vector (MPLUS) 
C     NSOL is an (estimated) row dimension of the solution array
C          if NSOL < M, solutions returned only at tau = i/(NSOL-1)
C          if all solutions are desired, NSOL = 3*M is a good choice
C  on output:
C     SOL is the coefficient solution array (N+2 by NSOL)
C         first and last rows give tau bkpts and quantile
C         rows 2:(N+1) give the beta coefficients
C     LSOL is the actual dimension of the solution arrays
C          LSOL = NSOL  if NSOL < M, else LSOL .LE. NSOL
C     ICEN (M) indicates status of censored observations
C          = 0 if uncensored (or uncrossed censored while pivoting)
C          = 1 if crossed censored
C          = 2 if deleted as below tau = 0 solution
C          = 3 if above last (maximal tau) solution
C     TCEN (M) are the corresponding tau value where censored obs is
C          firstcrossed:  weight = (tau - TCEN(I))/(1 - TCEN(I))
C          TCEN = 1 if censored obs is never crossed (ICEN = 3)
C          TCEN = 0 if censored obs is deleted (ICEN = 2)
C     LCEN = number of censored observations
C     WD = list of first MPLUS tau values at which degeneracy
C          may have occurred (and weighted RQ was called)
C     H(1) = number of calls to weighted RQ (apparent degeneracy)
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      INTEGER H(N),ICEN(M),C(M),IA(MPLUS)
      LOGICAL APC, DEG
      DOUBLE PRECISION ONE
      DIMENSION X(M,N),Y(M),SOL(N2,NSOL),WC(MPLUS,N2),XH(N,N)
      DIMENSION WA(MPLUS,N),WB(MPLUS),WD(MPLUS),WE(MPLUS)
      DIMENSION F(2),WF(N),TCEN(M)
      DATA BIG/1.D17/
      DATA ZERO/0.00D0/
      DATA HALF/0.50D0/
      DATA ONE/1.00D0/
      DATA TWO/2.00D0/
C
C  CHECK DIMENSION PARAMETERS
C
      DEG = .FALSE.
      IFT=0
      N1 = NSOL-1
      LCEN = 0
      DO 2 I = 1,M
      IF(C(I) .EQ. 1) LCEN = LCEN + 1
   2  CONTINUE
      IF(MPLUS .LT. M+1) IFT = 2
      IF(N2 .NE. N+2) IFT = 2
      IF(M .LE. ZERO .OR. N. LE. ZERO .OR. M .LT. N) IFT = 1
      IF(MAXW .GT. MPLUS) IFT = 3
      IF(IFT .NE. 0) RETURN
C
C  INITIALIZATION
C
      TOLER = 1.D-11
      TOL1 = 10.0D0*TOLER
      IF(TZERO .LE. ZERO  .OR.  TZERO .GT. STEP) TZERO = STEP
      DO 5 I=1,M
        ICEN(I) = 0
        TCEN(I) = ONE
        WB(I) = Y(I)
        DO 5 J=1,N
   5    WA(I,J) = X(I,J)
      MA = M
C
C  GET TAU = O SOLUTION
C  DELETE CENSORED OBS BELOW TAU = 0 SOLN
C
  10  IF(MA .LE. N) THEN
        IFT = 4
        RETURN
      ENDIF
      CALL RQ1(MA,N,MPLUS,N2,WA,WB,TZERO,TOLER,IFT1,
     * WF,WE,IA,WC,WD)
      K = 0
      KL = 0
C
C  CHECK FOR CENSORED OBS BELOW TAU = 0 SOLN
C
      L = 0
      DO 20 I=1,M
      IF(ICEN(I) .EQ. 2) GO TO 20
      L = L + 1
      IF( C(I) .EQ. 1 .AND. WE(L) .LE. TOL1 ) THEN
        K = K + 1
        ICEN(I) = 2
        GO TO 20
      ENDIF
      IF( DABS(WE(L)) .GE. TOL1) GO TO 20
      KL = KL + 1
      IF(KL .LE. N) H(KL) = I
  20  CONTINUE
      IF( K .EQ. 0) GO TO 40
      MA = MA - K
      L = 0
C
C  DELETE CENSORED OBS BELOW TAU = 0 SOLN, AND RETRY
C 
      DO 30 I=1,M
        IF(ICEN(I) .EQ. 2) GO TO 30
        L = L + 1
        WB(L) = Y(I)
        DO 25 J=1,N
  25      WA(L,J) = X(I,J)
  30    CONTINUE
      GO TO 10
  40  CONTINUE
C
C  SAVE INITIAL SOLUTION, AND INITIALIZE PIVOTING LOOP
C
      SOL(1,1) = ZERO
      DO 50 J = 1,N
  50  SOL(J+1,1) = WF(J)
      NWRQ = 0
      APC = .FALSE.
      LSOL = 2
      TAU = TZERO
C
C  COMPUTE NEXT TAU
C  FIRST CHECK FOR DEGENERACY AT TZERO
C
      IF (KL .GT. N)  GO TO 500
C
C  GET X(H,)
C
      DO 70 K = 1,N
        I= H(K)
        DO 60 J = 1,N
  60    XH(K,J) = X(I,J)
  70  CONTINUE
C
C  GET X(H,) INVERSE
C
      CALL DGECO(XH,N,N,IA,V,WC)
      IF(V. LT. TOLER) GO TO 500
      JOB = 01
      CALL DGEDI(XH,N,N,IA,F,WC,JOB)
C
C  GET RESIDUALS
C
      DO 90 I=1,M
      S = Y(I)
      DO 80 J = 1,N
  80  S = S - WF(J)*X(I,J)
  90  WE(I) = S
C
C  BEGIN PIVOTING; WD = GRAD UPPER BD, IA = SIGN(GRAD DENOM)
C
 200  CONTINUE
      CALL GRAD(X,M,N,H,ICEN,TCEN,XH,WE,TOLER,IA,WC,WD)
      KL = 0
      KM = 1
      S = WD(1)
      IF(N .EQ. 1) GO TO 230
      DO 210 J=2,N
 210  S = DMIN1(WD(J),S)
      DO 220 J=1,N
      IF(WD(J) .GE. S + TOLER) GO TO 220
        KL = KL + 1
        KM = J
 220  CONTINUE
 230  CONTINUE
C     
C  IF ALL POS RESIDS CENSORED, RETURN; ELSE CHECK FOR DEGEN.
C     
      IF(APC) THEN
        SOL(1,LSOL) = DMAX1(S, TAU)
        DO 240 J=1,N
 240    SOL(J+1,LSOL) = WF(J)
        GO TO 600
      ENDIF
C
C  IF DEGENERACY, CALL WEIGHTED RQ
C
      IF (KL .GT. 1) GO TO 500
C     
C  CHECK FOR INFEASIBILITY CAUSED BY REWEIGHTING
C 
      IF(S .LT. TAU + TOL1) THEN
        LSOL = LSOL - 1
        TAU = TAU - .8*STEP
        GO TO 500
      ENDIF
      TAU = S
C
C  GET NEW OBSERVATION TO ENTER BASIS
C  FIRST DEFINE WD = BASIS INDICATOR = 1 IF I IN H(J)
C
      DO 250 I=1,M
 250  WD(I) = ZERO
      DO 260 J=1,N
 260  WD(H(J)) = ONE
      KN = 0
      D = BIG 
      KIN = 0
      DO 300 I=1,M
      IF(ICEN(I) .EQ. 2) GO TO 300
      S = ZERO
      DO 270 J=1,N
 270  S = S + X(I,J)*XH(J,KM) 
      S = IA(KM)*S
      IF(DABS(S).LT.TOL1 .OR. (C(I).EQ.1 .AND. ICEN(I).NE.1)
     *    .OR. WD(I).EQ.ONE) GO TO 300
      S = WE(I)/S
      IF(S .LT. TOL1) GO TO 300
      IF(S .LE. D - TOL1) THEN
        D = S
        KIN = I
        KN = 0
      ELSE
        IF(S .LE. D + TOL1) KN = 1
      ENDIF
 300  CONTINUE
      IF(KN .EQ. 1) GO TO 500
      H(KM) = KIN
C
C  UPDATE SOLUTION
C  GET NEW X(H,)
C
        DO 310 K = 1,N
        I = H(K)
        DO 310 J = 1,N
 310    XH(K,J) = X(I,J)
C
C  GET X(H,) INVERSE
C
 315  CALL DGECO(XH,N,N,IA,V,WA)
      IF(V. LT. TOLER) GO TO 500
      JOB = 01
      CALL DGEDI(XH,N,N,IA,F,WA,JOB)
C
C  GET BETA-HAT, RESIDS
C
      DO 340 K = 1,N
      S = ZERO
      DO 330 J = 1,N
 330  S = S + XH(K,J)*Y(H(J))
 340  WF(K) = S
 345  DO 360 I=1,M
      S = Y(I)
      DO 350 J = 1,N
 350  S = S - WF(J)*X(I,J)
 360  WE(I) = S
C
C  SAVE SOLUTION
C
 365  DO 380 I = (LSOL-1),N1
      IF(NSOL.GE.M .OR. TAU .GT. DFLOAT(I)/DFLOAT(N1) - 10*TOL1) THEN
        SOL(1,LSOL) = TAU
        DO 370 J = 1,N
 370    SOL(J+1,LSOL) = WF(J)
        LSOL = LSOL + 1
        GO TO 390
      ENDIF
 380  CONTINUE
 390  CONTINUE
C
C  CHECK FOR DIM(SOL) EXCEEDED
C
      IF(LSOL .GT. NSOL) THEN
        IFT = 6
        GO TO 600
      ENDIF
      IF(APC) THEN
        LSOL = LSOL - 1
        GO TO 600
      ENDIF
C
C  SET WT FOR CROSSED CENSORED OBSERVATIONS
C  APC = .TRUE.  IF ALL POS RESID ARE UNCROSSED CENSORED
C
      APC = .TRUE.
C  if the following are left uncommented: still get zero column
C      TAUW = TAU - STEP/TWO
C      IF(MAXW.GT.ZERO)THEN
C        TAUW=TAU
C      ENDIF
      DO 400 I=1,M
      IF(ICEN(I) .EQ. 2) GO TO 400
      IF(WE(I).GE.TOL1 .AND. (C(I).EQ.0 .OR.  ICEN(I).EQ.1))
     *   APC = .FALSE.
      IF(WE(I).LE.-TOL1 .AND. C(I).EQ.1 .AND. ICEN(I).EQ.0) THEN
C  
C at this point,  Y(I) is a crossed censored obs  (C_i)
C for the grid method  TAUW = TAU - STEP*(B2-Y(I))/(B2-B1)
C   where   B1 = x_i' beta_j   and   B2 = x_i' beta_(j+1)
C otherwise (for pivot), TAUW = TAU
C
        IF(MAXW.LT.0) THEN
         B1 = ZERO
         B2 = ZERO
         DO 396 J=1,N
          B1 = B1 + X(I,J) * SOL(J+1,LSOL - 2)
396       B2 = B2 + X(I,J) * SOL(J+1,LSOL - 1)
         A1 = (B2 - Y(I))/(B2-B1)
         TAUW = TAU - A1*STEP
        ELSE
         TAUW = TAU
        ENDIF
        ICEN(I) = 1
        TCEN(I) = TAUW
      ENDIF
 400  CONTINUE
      IF(APC) THEN
        IF(DEG) THEN
          IFT = 5
          LSOL = LSOL - 1
          GO TO 600
        ENDIF
        GO TO 200
      ENDIF
C
C  RETURN IF TAU > 1
C
      IF(TAU .GE. ONE - 10.0D0 * TOL1) GO TO 600
      IF(DEG) GO TO 500
      IF(MAXW .GT. 0) GO TO 200
C
C  POSSIBLE DEGENERACY -- USE WEIGHTED RQ1 TO RESOLVE
C
 500  NWRQ = NWRQ + 1
      IF(MAXW .GT. 0  .AND.  NWRQ .GT. MAXW) THEN
        IFT = 7
        LSOL = LSOL - 1
        GO TO 600
      ENDIF
      IF(NWRQ .LE. NSOL) SOL(N+2,NWRQ) = TAU
      TAU = TAU + STEP
      IF(TAU .GE. ONE) TAU = ONE - TOL1
      L = 0
      L1 = 0
      DO 506 J=1,N
 506  WF(J) = ZERO
      DO 530 I = 1,M
      IF(ICEN(I) .EQ. 0) THEN
        IF (C(I) .EQ. 0) THEN
          L1 = L1 + 1
          WB(L1) = Y(I)
          DO 510 J=1,N
 510      WA(L1,J) = X(I,J)
        ELSE
          DO 514 J = 1,N
 514      WF(J) = WF(J) + X(I,J)
        ENDIF
      ENDIF
      IF(ICEN(I) .EQ. 1) THEN
        L1 = L1 + 1
        L = L+1
        W = (TAU - TCEN(I))/(ONE - TCEN(I))
        WB(L1) = W * Y(I)
        DO 520 J = 1,N
          WA(L1,J) = W * X(I,J)
 520      WF(J) = WF(J) + (ONE - W) * X(I,J)
      ENDIF
 530  CONTINUE
      MAL = L1+1
      DO 534 J=1,N
 534  WA(MAL,J) = WF(J)
      WB(MAL) = BIG
      CALL RQ1(MAL,N,MPLUS,N2,WA,WB,TAU,TOLER,IFT1,WF,WE,IA,WC,WD)
      DEG = .FALSE.
      IF(DABS(WE(MAL)) .LE. .0001) THEN
        IFT = 8
        LSOL = LSOL - 1
        GO TO 600
      ENDIF 
      L = 0
      K = 0
      DO 550 I=1,M
        IF(ICEN(I) .EQ. 2) GO TO 550
        L = L+1
        IF(DABS(WE(L)) .GE. TOL1) GO TO 550
          K = K+1
          IF(K .LE. N) THEN
            H(K) = I
            DO 540 J=1,N
 540        XH(K,J) = X(I,J)
          ENDIF
 550  CONTINUE
      IF(K .LT. N) THEN
        IFT = 8
        LSOL = LSOL - 1
        GO TO 600
      ENDIF
      IF(K .GT. N) DEG = .TRUE.
      GO TO 345
C
C  DEFINE OUTPUT AND RETURN
C
 600  H(1) = NWRQ
      L = MIN0(NWRQ,MPLUS)
      DO 610 I=1,L
 610  WD(I) = SOL(N+2,I)
      DO 620 I=1,M
      IF(ICEN(I) .EQ. 2) GO TO 620
      IF(C(I) .EQ.1 .AND. TCEN(I) .EQ. ONE) ICEN(I) = 3
 620  CONTINUE
      V = DFLOAT(MA)
      DO 650 J = 1,N
      S = ZERO
        DO 640 I = 1,M
          IF(ICEN(I) .EQ. 2) GO TO 640
          S = S + X(I,J)
 640    CONTINUE
 650  WF(J) = S/V
      DO 670 I = 1,LSOL
      S = ZERO
        DO 660 J = 1,N
 660    S = S + WF(J) * SOL(J+1,I)
 670  SOL(N+2,I) = S
      RETURN
      END
      SUBROUTINE GRAD(X,M,N,H,ICEN,TCEN,XH,
     * R,TOL,IFLAG,WA,GUP)
C
C  X matrix (M BY N)
C  M = Number of Observations
C  N = Number of Parameters
C  H = basis, integer(N) vector 
C  ICEN (integer(M)) = 2 for deleted obs
C                    = 1 for crossed non-censored obs
C                    = 0 otherwise
C  TCEN(M) are the corresponding tau values where a censored obs is
C    first crossed (ICEN=1): weight = (tau - TCEN(I))/(1 - TCEN(I))
C    TCEN = 1 if censored obs is never crossed
C  XH = (N by N) X(H,) inverse matrix
C  R(M) = residuals
C  TOL = zero tolerance (1.D-10 by default)
C  IFLAG (M)  work vector
C  WA (M by N) work array
C  returns:
C    GUP(N)  = upper bounds on tau
C    IFLAG(1:N) = sign(grad denom)
C               = +1 if bound from lower grad bound
C               = -1 if bound from upper grad bound
C    
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      INTEGER H(N),ICEN(M),IFLAG(*)
      DOUBLE PRECISION ONE
      DIMENSION X(M,N),GUP(N),XH(N,N)
      DIMENSION R(M),TCEN(M),WA(M,N)
      DATA  ZERO/0.00D0/, ONE/1.0d0/
C
C GET X'(XH-INVERSE)
C
      DO 80 I = 1,M
      IF(ICEN(I) .EQ. 2) GO TO 80
      DO 70 J = 1,N
      A = ZERO
        DO 60 K = 1,N
  60    A = A + X(I,K)*XH(K,J)
  70  WA(I,J) = A
  80  CONTINUE
  
C
C  GET GRADIENT BOUNDS
C  FIRST SET IFLAG = 1 FOR BASIS INDICES (TEMPORARY)
C
      T = ZERO
      DO 90 I = 1,M
  90  IFLAG(I) = 0
      DO 95 J = 1,N
  95  IFLAG(H(J)) = 1
      DO 120 J=1,N
      A = ZERO
      B = ZERO
      C = ZERO
      D = ZERO
      DO 100 I = 1,M
        IF(ICEN(I) .EQ. 2) GO TO 100
        IF(ICEN(I) .EQ. 0) THEN
          IF(R(I) .GT. TOL)    A = A + WA(I,J)
          IF(R(I) .LT. - TOL)  B = B + WA(I,J)
          GO TO 100
        ENDIF
        IF(IFLAG(I).EQ.1) GO TO 100
C
C  IFLAG = 0: NOT BASIS  AND  ICEN = 1: CROSSED CENSORED
C
        IF(R(I) .LT. -TOL) THEN
           T = TCEN(I)/(ONE - TCEN(I))
           C = C - T*WA(I,J)
           GO TO 100
        ENDIF
        IF(R(I) .GT. TOL) THEN          
           D = D - WA(I,J)
        ENDIF
 100  CONTINUE
      D = D - C
      L = ICEN(H(J))
      IF(L .NE. 0) T = TCEN(H(J))/( ONE - TCEN(H(J)) )
      S = DFLOAT(L)*(T + ONE) - ONE
      E1 = A + B - D - S
      E2 = A + B - D + ONE
      IF(E1 .GT. ZERO) THEN
        GUP(J) = (B + C - S)/E1
        IFLAG(J+M) = 1
        GO TO 120
      ENDIF
      IF(E2 .LT. ZERO) THEN
        GUP(J) = (B + C)/E2
        IFLAG(J+M) = -1
        GO TO 120
      ENDIF
      GUP(J) = - ONE
 120  CONTINUE
      DO 130 J = 1,N
 130  IFLAG(J) = IFLAG(J+M)
      RETURN
      END

# multiple tau estimation with fnb algorithm and globbing

# Input:
    # n = full sample size
    # p = parametric dimension of model
    # m = number of quantiles to be estimated
    # a = p by n (transposed) design matrix
    # y = n-vector of responses
    # q = m-vector of quantiles to be estimated
    # r = residual vector from fit for q[1]
    # b = coefficient matrix 
    # band = confidence band (see rq.fit.pfnb)
    # m0 = initial sample size default is n^(2/3) * p^(1/2)
    # d = n-vector of ones
    # u = n-vector of ones
    # wn = 9*n work array
    # wp = p(p+3) work array
    # aa = n by p array for globbed design matrix
    # yy = n vector for globbed response matrix
    # slo = n-vector of boolean integers
    # shi = n-vector of boolean integers
    # rhs = p-vector for right hand side 
    # glob = p-vector for glob
    # ghib = p-vector for ghib
    # nit = 3-vector of iteration counts
    # info = flag for convergence
    # Maybe there should be a control object 
    # Possible control parameter m0
    # and/or work storage could be better rationalized.

subroutine pfnb(n,p,m,a,y,q,r,b,band,m0,d,u,wn,wp,
		aa,yy,slo,shi,rhs,glob,ghib,nit,info)
integer n,p,m,kk(2),mm,m0,nit(5,m),info,sumbad
integer i,j,k,slo(n),shi(n),ifix,ibad
logical notopt
double precision a(p,n),y(n),r(n),q(m),d(n),u(n),b(p,m)
double precision wn(n,9), wp(p,(p+3)),band(n),qk(2)
double precision glob(p),ghib(p),aa(p,n),yy(n),rhs(p)
double precision zero,one,tau,beta,eps,big,fm,fn

parameter(zero = 0.0d0)
parameter(one = 1.0d0)
parameter(beta = 0.99995d0)
parameter(big = 1.0d+10)
parameter(eps = 1.0d-06)

    
do iq = 1,m {
  notopt = .true.
  tau = q(iq)
  mm = m0
  ifix = 0
  ibad = 0
  while (notopt) {
    ibad = ibad + 1
    fm = mm
    fn = dble(n)
    kk(1) = int(n * dmax1(1./fn, tau - fm/(2 * fn))) + 1
    kk(2) = int(n * dmin1(tau + fm/(2. * fn), (fn - 1)/fn))
    do i = 1,n
	u(i) = r(i)/band(i)
    call kuantiles(kk,2,n,u)
    qk(1) = u(kk(1))
    qk(2) = u(kk(2))
    call iphil(n,0,slo)
    call iphil(n,0,shi)
    do i = 1,n{
	if(r(i) < (band(i) * qk(1)))
	    slo(i) = 1
	else if(r(i) > (band(i) * qk(2)))
	    shi(i) = 1
    }
    while (notopt) {
	ifix = ifix + 1
	call dphil(p,zero,glob)
	call dphil(p,zero,ghib)
	call dphil(n,one,d)
	call dphil(n,one,u)
	k = 0
	do i = 1,n{
	    if(slo(i) == 0 & shi(i) == 0){
		k = k + 1
		call dcopy(p,a(1,i),1,aa(1,k),1)
                yy(k) = -y(i)
		}
	    else if (slo(i) == 1) {
		do j = 1,p 
		    glob(j) = glob(j) + a(j,i) 
                }
	    else if (shi(i) == 1) {
		do j = 1,p 
		    ghib(j) = ghib(j) + a(j,i) 
                }
	}
	call dcopy(p,glob,1,aa(1,k+1),1)
	call dcopy(p,ghib,1,aa(1,k+2),1)
        yy(k+1) =  big
        yy(k+2) = -big
	call dgemv('N',p,k+2,one-tau,aa,p,d,1,zero,rhs,1)
	call dscal(k+2,zero,wn,1)
	call daxpy(k+2,one-tau,u,1,wn,1)
        call rqfnb(k+2,p,aa,yy,rhs,d,u,beta,eps,wn,wp,nit(1,iq),info)
	call dcopy(p,wp,1,b(1,iq),1)
	call dcopy(n,y,1,r,1)
	call dgemv('T',p,n,one,a,p,b(1,iq),1,one,r,1)
	# Check predicted signs of residuals (in r)
	sumbad = 0
	do i = 1,n {
	    if((r(i) > 0) & slo(i) == 1){
		slo(i) = 0
		sumbad = sumbad + 1
	    }
	    if((r(i) < 0) & shi(i) == 1){
		shi(i) = 0
		sumbad = sumbad + 1
	    }
	}
        if(sumbad > 0) {
            if (sumbad > 0.1 * mm) {
                mm = min(2 * mm, n)
                break
            }
        }
        else notopt = .false.
    }
    nit(4,iq) = ifix
    nit(5,iq) = ibad
  }
}
return
end
subroutine iphil(n,a,v)
integer n,i,a,v(n)
do i = 1,n
    v(i) = a
return
end
subroutine dphil(n,a,v)
integer i,n
double precision a,v(n)
do i = 1,n
    v(i) = a
return
end

# Subroutine to list the r choose n subsets of {1,2,...,r} in an order such that
# adjacent subsets have only one element swapped.  Algorithm modeled on the pascal
# algorithm of Limin Xiang and Kazuo Ushijima (2001) "On O(1) Time Algorithms for
# Combinatorial Generation," Computer Journal, 44(4), 292-302. 
# http://comjnl.oxfordjournals.org/cgi/reprint/44/4/292

# Translated into ratfor:  12 February, 2008 R. Koenker.

subroutine combin(r,n,m,a,c,e,Last)

integer r,n,m,t,k,j,M0,Mj,s
integer A(n,m),c(r),e(r),Last(r)
logical odd
	
	M0 = r-n
	t = n+1
	k = 1
	j = 0
	c(1) = 0
	repeat{
		j = j + 1
		c(j) = j
		e(j) = j - 1
		if(odd(j))
			Last(j) = M0 + j
		else
			Last(j) = j + 1
		if(j == n) break
		}
	do i = 1,n
		A(i,1) = c(i)
	if(n < r) {
	repeat {
		k = k + 1
		S = c(j)
		Mj = M0 + j
		e(n+1) = n
		if(odd(j)){
			if(c(j) == Mj) {
				c(j) = c(j-1) + 1
				Last(j+1) = c(j) + 1
				}
			else
				c(j) = c(j) + 1
			}
		else { 
			if(c(j) == c(j-1) + 1)
				c(j) = Mj
			else {
				Last(j+1) = c(j)
				c(j) = c(j) - 1
				}
			}
		if(c(j) == Last(j)) {
			Last(j) = S
			e(j+1) = e(j)
			e(j) = j-1
			}
		if( (j < n) & (c(j) == Mj)) {
			t = j
			j = e(t+1)
			e(t+1) = t
			}
		else {
			if(t == j) t = t + 1
			if(t < e(n+1)) j = t
			else j = e(n+1)
			}
		do i = 1,n
			A(i,k) = c(i)
		if(j == 0) break
		}
	}
return
end


logical function odd(j) 
integer j
odd = (mod(j,2) == 1)
return
end

c
c Extract: Subroutine to extract the non-diagonal structure and
c   entries from A stored in CSR format
c 
      subroutine extract(d,jd,id,dsub,jdsub,m,nnzd,nnzds,ierr)
      integer jd(nnzd),jdsub(nnzds),id(*),m,mp1,ierr,nnzd,nnzds
      double precision d(nnzd),dsub(nnzds)
c
c Call csrmsr in SPARSKIT2 to transform the storage format in d
c   from csr to msr
c
      call csrmsr(m,d,jd,id,dsub,jdsub,dsub,jdsub,nnzds,ierr)
      mp1 = m + 1
      do i=1,mp1
         jdsub(i) = jdsub(i) - mp1
      enddo
      return
      end

c----------------------------------------------------------------------c
       subroutine amub (nrow,ncol,job,a,ja,ia,b,jb,ib,
     *                  c,jc,ic,nzmax,iw,ierr) 
      double precision a(*), b(*), c(*) 
      integer ja(*),jb(*),jc(*),ia(nrow+1),ib(*),ic(*),iw(ncol)
c-----------------------------------------------------------------------
c performs the matrix by matrix product C = A B 
c-----------------------------------------------------------------------
c on entry:
c ---------
c nrow  = integer. The row dimension of A = row dimension of C
c ncol  = integer. The column dimension of B = column dimension of C
c job   = integer. Job indicator. When job = 0, only the structure
c                  (i.e. the arrays jc, ic) is computed and the
c                  real values are ignored.
c
c a,
c ja,
c ia   = Matrix A in compressed sparse row format.
c 
c b, 
c jb, 
c ib    =  Matrix B in compressed sparse row format.
c
c nzmax = integer. The  length of the arrays c and jc.
c         amub will stop if the result matrix C  has a number 
c         of elements that exceeds exceeds nzmax. See ierr.
c 
c on return:
c----------
c c, 
c jc, 
c ic    = resulting matrix C in compressed sparse row sparse format.
c           
c ierr  = integer. serving as error message. 
c         ierr = 0 means normal return,
c         ierr .gt. 0 means that amub stopped while computing the
c         i-th row  of C with i=ierr, because the number 
c         of elements in C exceeds nzmax.
c
c work arrays:
c------------
c iw    = integer work array of length equal to the number of
c         columns in B.
c Note: 
c-------
c   The row dimension of B is not needed. However there is no checking 
c   on the condition that ncol(A) = nrow(B). 
c
c----------------------------------------------------------------------- 
      double precision scal 
      logical values
      values = (job .ne. 0) 
      len = 0
      ic(1) = 1 
      ierr = 0
c     initialize array iw.
      do 1 j=1, ncol
         iw(j) = 0
 1    continue
c
      do 500 ii=1, nrow 
c     row i 
         do 200 ka=ia(ii), ia(ii+1)-1 
            if (values) scal = a(ka)
            jj   = ja(ka)
            do 100 kb=ib(jj),ib(jj+1)-1
               jcol = jb(kb)
               jpos = iw(jcol)
               if (jpos .eq. 0) then
                  len = len+1
                  if (len .gt. nzmax) then
                     ierr = ii
                     return
                  endif
                  jc(len) = jcol
                  iw(jcol)= len
                  if (values) c(len)  = scal*b(kb)
               else
                  if (values) c(jpos) = c(jpos) + scal*b(kb)
               endif
 100     continue
 200     continue
         do 201 k=ic(ii), len
            iw(jc(k)) = 0
 201     continue
         ic(ii+1) = len+1
 500  continue
      return
c-------------end-of-amub-----------------------------------------------
c-----------------------------------------------------------------------
      end
c-----------------------------------------------------------------------
      subroutine amudia (nrow,job, a, ja, ia, diag, b, jb, ib)
      double precision a(*), b(*), diag(*) 
      integer ja(*),jb(*), ia(nrow+1),ib(nrow+1) 
c-----------------------------------------------------------------------
c performs the matrix by matrix product B = A * Diag  (in place) 
c-----------------------------------------------------------------------
c on entry:
c ---------
c nrow	= integer. The row dimension of A
c
c job   = integer. job indicator. Job=0 means get array b only
c         job = 1 means get b, and the integer arrays ib, jb.
c
c a,
c ja,
c ia   = Matrix A in compressed sparse row format.
c 
c diag = diagonal matrix stored as a vector dig(1:n)
c
c on return:
c----------
c
c b, 
c jb, 
c ib	= resulting matrix B in compressed sparse row sparse format.
c	    
c Notes:
c-------
c 1)        The column dimension of A is not needed. 
c 2)        algorithm in place (B can take the place of A).
c-----------------------------------------------------------------
      do 1 ii=1,nrow
c     
c     scale each element 
c     
         k1 = ia(ii)
         k2 = ia(ii+1)-1
         do 2 k=k1, k2
            b(k) = a(k)*diag(ja(k)) 
 2       continue
 1    continue
c     
      if (job .eq. 0) return
c     
      do 3 ii=1, nrow+1
         ib(ii) = ia(ii)
 3    continue
      do 31 k=ia(1), ia(nrow+1) -1 
         jb(k) = ja(k)
 31   continue
      return
c-----------------------------------------------------------------------
c-----------end-of-amudiag----------------------------------------------
      end 
c----------------------------------------------------------------------c
      subroutine amux (n, x, y, a,ja,ia) 
      double precision  x(*), y(*), a(*) 
      integer n, ja(*), ia(*)
c-----------------------------------------------------------------------
c         A times a vector
c----------------------------------------------------------------------- 
c multiplies a matrix by a vector using the dot product form
c Matrix A is stored in compressed sparse row storage.
c
c on entry:
c----------
c n     = row dimension of A
c x     = real array of length equal to the column dimension of
c         the A matrix.
c a, ja,
c    ia = input matrix in compressed sparse row format.
c
c on return:
c-----------
c y     = real array of length n, containing the product y=Ax
c
c-----------------------------------------------------------------------
c local variables
c
      double precision t
      integer i, k
c-----------------------------------------------------------------------
      do 100 i = 1,n
c
c     compute the inner product of row i with vector x
c 
         t = 0.0d0
         do 99 k=ia(i), ia(i+1)-1 
            t = t + a(k)*x(ja(k))
 99      continue
c
c     store result in y(i) 
c
         y(i) = t
 100  continue
c
      return
      end
c-----------------------------------------------------------------------
      subroutine aplb (nrow,ncol,job,a,ja,ia,b,jb,ib,
     *     c,jc,ic,nzmax,iw,ierr)
      double precision a(*), b(*), c(*) 
      integer ja(*),jb(*),jc(*),ia(nrow+1),ib(nrow+1),ic(nrow+1),
     *     iw(ncol)
c-----------------------------------------------------------------------
c performs the matrix sum  C = A+B. 
c-----------------------------------------------------------------------
c on entry:
c ---------
c nrow	= integer. The row dimension of A and B
c ncol  = integer. The column dimension of A and B.
c job   = integer. Job indicator. When job = 0, only the structure
c                  (i.e. the arrays jc, ic) is computed and the
c                  real values are ignored.
c
c a,
c ja,
c ia   = Matrix A in compressed sparse row format.
c 
c b, 
c jb, 
c ib	=  Matrix B in compressed sparse row format.
c
c nzmax	= integer. The  length of the arrays c and jc.
c         amub will stop if the result matrix C  has a number 
c         of elements that exceeds exceeds nzmax. See ierr.
c 
c on return:
c----------
c c, 
c jc, 
c ic	= resulting matrix C in compressed sparse row sparse format.
c	    
c ierr	= integer. serving as error message. 
c         ierr = 0 means normal return,
c         ierr .gt. 0 means that amub stopped while computing the
c         i-th row  of C with i=ierr, because the number 
c         of elements in C exceeds nzmax.
c
c work arrays:
c------------
c iw	= integer work array of length equal to the number of
c         columns in A.
c
c-----------------------------------------------------------------------
      logical values
      values = (job .ne. 0) 
      ierr = 0
      len = 0
      ic(1) = 1 
      do 1 j=1, ncol
         iw(j) = 0
 1    continue
c     
      do 500 ii=1, nrow
c     row i 
         do 200 ka=ia(ii), ia(ii+1)-1 
            len = len+1
            jcol    = ja(ka)
            if (len .gt. nzmax) goto 999
            jc(len) = jcol 
            if (values) c(len)  = a(ka) 
            iw(jcol)= len
 200     continue
c     
         do 300 kb=ib(ii),ib(ii+1)-1
            jcol = jb(kb)
            jpos = iw(jcol)
            if (jpos .eq. 0) then
               len = len+1
               if (len .gt. nzmax) goto 999
               jc(len) = jcol
               if (values) c(len)  = b(kb)
               iw(jcol)= len
            else
               if (values) c(jpos) = c(jpos) + b(kb)
            endif
 300     continue
         do 301 k=ic(ii), len
            iw(jc(k)) = 0
 301     continue
         ic(ii+1) = len+1
 500  continue
      return
 999  ierr = ii
      return
c------------end of aplb ----------------------------------------------- 
c-----------------------------------------------------------------------
      end
      subroutine csrmsr (n,a,ja,ia,ao,jao,wk,iwk,nnzao,ierr)
      double precision a(*),ao(*),wk(n)
      integer ia(n+1),ja(*),jao(*),iwk(n+1),nnzao,ierr
c----------------------------------------------------------------------- 
c Compressed Sparse Row   to      Modified - Sparse Row 
c                                 Sparse row with separate main diagonal
c----------------------------------------------------------------------- 
c converts a general sparse matrix a, ja, ia into 
c a compressed matrix using a separated diagonal (referred to as
c the bell-labs format as it is used by bell labs semi conductor
c group. We refer to it here as the modified sparse row format.
c Note: this has been coded in such a way that one can overwrite
c the output matrix onto the input matrix if desired by a call of
c the form 
c
c     call csrmsr (n, a, ja, ia, a, ja, wk,iwk)
c
c In case ao, jao, are different from a, ja, then one can
c use ao, jao as the work arrays in the calling sequence:
c
c     call csrmsr (n, a, ja, ia, ao, jao, ao,jao)
c
c----------------------------------------------------------------------- 
c
c on entry :
c---------
c a, ja, ia = matrix in csr format. note that the 
c	     algorithm is in place: ao, jao can be the same
c            as a, ja, in which case it will be overwritten on it
c            upon return.
c nnzao = the number of non-zero entries in ao, jao
c	 
c on return :
c-----------
c
c ao, jao  = sparse matrix in modified sparse row storage format:
c	   +  ao(1:n) contains the diagonal of the matrix. 
c	   +  ao(n+2:nnz) contains the nondiagonal elements of the
c             matrix, stored rowwise.
c	   +  jao(n+2:nnz) : their column indices
c	   +  jao(1:n+1) contains the pointer array for the nondiagonal
c             elements in ao(n+2:nnz) and jao(n+2:nnz).
c             i.e., for i .le. n+1 jao(i) points to beginning of row i 
c	      in arrays ao, jao.
c	       here nnz = number of nonzero elements+1 
c ierr:
c    = -1 : length of ao, jao < itpr
c
c work arrays:
c------------
c wk	= real work array of length n
c iwk   = integer work array of length n+1
c
c notes: 
c------- 
c        Algorithm is in place.  i.e. both:
c
c          call csrmsr (n, a, ja, ia, ao, jao, ao,jao)
c          (in which  ao, jao, are different from a, ja)
c           and
c          call csrmsr (n, a, ja, ia, a, ja, wk,iwk) 
c          (in which  wk, jwk, are different from a, ja)
c        are OK.
c--------
c coded by Y. Saad Sep. 1989. Rechecked Feb 27, 1990.
c Modified by Pin Ng on June 11, 2002 to provide warning when 
c    iptr > nnzao+1
c-----------------------------------------------------------------------
      icount = 0
c
c store away diagonal elements and count nonzero diagonal elements.
c
      do 1 i=1,n
         wk(i) = 0.0d0
         iwk(i+1) = ia(i+1)-ia(i)
         do 2 k=ia(i),ia(i+1)-1
            if (ja(k) .eq. i) then
               wk(i) = a(k)
               icount = icount + 1 
               iwk(i+1) = iwk(i+1)-1
            endif
 2       continue
 1    continue
c     
c compute total length
c     
      iptr = n + ia(n+1) - icount
      if (iptr .gt. nnzao+1) then
         ierr = -1
         return
      endif
c     
c     copy backwards (to avoid collisions)
c     
      do 500 ii=n,1,-1
         do 100 k=ia(ii+1)-1,ia(ii),-1
            j = ja(k)
            if (j .ne. ii) then
               ao(iptr) = a(k)
               jao(iptr) = j 
               iptr = iptr-1
            endif
 100     continue
 500  continue
c
c compute pointer values and copy wk(*)
c
      jao(1) = n+2
      do 600 i=1,n
         ao(i) = wk(i) 
         jao(i+1) = jao(i)+iwk(i+1)
 600  continue
      return	
c------------ end of subroutine csrmsr ---------------------------------
c----------------------------------------------------------------------- 
      end

# Wrapper for rq solutions with multiple y's
subroutine rqs(m,n,k,m5,n2,a,b,tau,toler,ift,x,e,s,wa,wb)
double precision b(m,k),a(m,n),x(n,k),e(m),wa(m5,n2),wb(m)
double precision tau,toler
integer s(m),m,n,k,m5,n2,ift(k)
do i=1,k 
	call rq0(m,n,m5,n2,a,b(1,i),tau,toler,ift(i),x(1,i),e,s,wa,wb)
return
end

dynrq <- function (formula, tau = 0.5, data, subset, weights, na.action, method = "br", 
    contrasts = NULL, start = NULL, end = NULL, ...) 
{
    stopifnot(requireNamespace("zoo"))
    Zenv <- new.env(parent = environment(formula))
    assign("dynformula", function(x) structure(x, class = unique(c("dynformula", 
        oldClass(x)))), envir = Zenv)
    assign("L", function(x, k = 1) {
        if (length(k) > 1) {
            rval <- lapply(k, function(i) lag(x, k = -i))
            rval <- if (inherits(x, "ts")) 
                do.call("ts.intersect", rval)
            else do.call("zoo::merge.zoo", c(rval, list(all = FALSE)))
            colnames(rval) <- k
        }
        else {
            rval <- lag(x, k = -k)
        }
        return(rval)
    }, envir = Zenv)
    assign("d", function(x, lag = 1) diff(x, lag = lag), envir = Zenv)
    assign("season", function(x, ref = NULL) {
        freq <- frequency(x)
        stopifnot(freq > 1 && identical(all.equal(freq, round(freq)), 
            TRUE))
        freq <- ofreq <- round(freq)
        freq <- if (freq == 12) 
            month.abb
        else if (freq == 4) 
            paste("Q", 1:4, sep = "")
        else 1:freq
        rval <- factor(zoo::coredata(cycle(x)), labels = freq)
        if (!is.null(ref)) 
            rval <- relevel(rval, ref = ref)
        rval <- zoo::zoo(rval, zoo::index(x), ofreq)
        return(rval)
    }, envir = Zenv)
    assign("trend", function(x, scale = TRUE) {
        freq <- ofreq <- if (inherits(x, "ts")) 
            frequency(x)
        else attr(x, "frequency")
        if (is.null(freq) | !scale) 
            freq <- 1
        stopifnot(freq >= 1 && identical(all.equal(freq, round(freq)), 
            TRUE))
        freq <- round(freq)
        rval <- zoo::zoo(seq_along(zoo::index(x))/freq, zoo::index(x), frequency = ofreq)
        return(rval)
    }, envir = Zenv)
    assign("harmon", function(x, order = 1) {
        freq <- frequency(x)
        stopifnot(freq > 1 && identical(all.equal(freq, round(freq)), 
            TRUE))
        freq <- round(freq)
        order <- round(order)
        stopifnot(order <= freq/2)
        rval <- outer(2 * pi * zoo::index(x), 1:order)
        rval <- cbind(apply(rval, 2, cos), apply(rval, 2, sin))
        colnames(rval) <- if (order == 1) {
            c("cos", "sin")
        }
        else {
            c(paste("cos", 1:order, sep = ""), paste("sin", 1:order, 
                sep = ""))
        }
        if ((2 * order) == freq) 
            rval <- rval[, -(2 * order)]
        return(rval)
    }, envir = Zenv)
    assign("model.frame.dynformula", function(formula, data = NULL, 
        subset = NULL, na.action = na.omit, drop.unused.levels = FALSE, 
        xlev = NULL, ...) {
        if (is.null(data)) 
            data <- parent.frame()
        if (!is.list(data)) 
            data <- as.list(data)
        args <- as.list(attr(terms(formula), "variables"))[-1]
        args$retclass <- "list"
        args$all <- FALSE
        formula <- terms(formula)
        attr(formula, "predvars") <- as.call(append(zoo::merge.zoo, args))
        attr(formula, "predvars")[[1]] <- as.name("merge.zoo")
        NextMethod("model.frame", formula = formula)
    }, envir = Zenv)
    if (missing(data)) 
        data <- Zenv
    orig.class <- if (is.data.frame(data) || is.environment(data)) 
        class(eval(attr(terms(formula), "variables")[[2]], data, 
            Zenv))
    else class(data)
    cl <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset", "weights", "na.action"), names(mf), 0)
    mf <- mf[c(1, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1]] <- as.name("model.frame")
    mf[[2]] <- as.call(list(as.name("dynformula"), mf[[2]]))
    mf <- eval(mf, envir = Zenv)
    mfna <- attr(mf, "na.action")
    if (length(zoo::index(mf[, 1])) > nrow(mf)) {
        for (i in 1:NCOL(mf)) attr(mf[, i], "index") <- attr(mf[, 
            i], "index")[-as.vector(mfna)]
    }
    is.zoofactor <- function(x) !is.null(attr(x, "oclass")) && 
        attr(x, "oclass") == "factor"
    for (i in 1:NCOL(mf)) if (is.zoofactor(mf[, i])) 
        mf[, i] <- zoo::coredata(mf[, i])
    mf1 <- mf[, 1]
    start <- if (is.null(start)) 
        1
    else {
        if (length(start) > 1) 
            start <- start[1] + (start[2] - 1)/frequency(mf1)
        start <- min(which(zoo::index(mf1) >= start))
    }
    end <- if (is.null(end)) 
        length(mf1)
    else {
        if (length(end) > 1) 
            end <- end[1] + (end[2] - 1)/frequency(mf1)
        end <- max(which(zoo::index(mf1) <= end))
    }
    if (end < start) {
        warning("empty model frame specified")
        mf <- head(mf, 0)
        mf1 <- head(mf1, 0)
    }
    else {
        mf <- mf[start:end, , drop = FALSE]
        mf1 <- mf1[start:end]
        if (!is.null(mfna)) 
            attr(mf, "na.action") <- mfna[as.vector(mfna) >= 
                start & as.vector(mfna) <= end]
    }
    if ("ts" %in% orig.class && zoo::is.regular(mf1, strict = TRUE)) {
        for (i in 1:ncol(mf)) if (!is.factor(mf[, i])) 
            mf[, i] <- as.ts(mf[, i])
    }
    if (all(orig.class == "numeric")) {
        for (i in 1:ncol(mf)) if (!is.factor(mf[, i])) 
            mf[, i] <- as.vector(mf[, i])
    }
    rownames(mf) <- zoo::index2char(zoo::index(mf1), frequency(mf1))
    mt <- attr(mf, "terms")
    attr(mt, "predvars") <- NULL
    attr(mt, "dataClasses") <- NULL
    Y <- model.response(mf, "numeric")
    weights <- as.vector(model.weights(mf))
    if (is.empty.model(mt)) {
        X <- NULL
        rval <- list(coefficients = numeric(0), residuals = Y, 
            fitted.values = 0 * Y, weights = weights, rank = 0, df.residual = length(Y))
    }
    else {
        Rho <- function(u, tau) u * (tau - (u < 0))
        eps <- .Machine$double.eps^(2/3)
        X <- model.matrix(mt, mf, contrasts)
        if (length(tau) > 1) {
            if (any(tau < -eps) || any(tau > 1 + eps))
                stop("invalid tau:  taus should be >= 0 and <= 1")
            coef <- matrix(0, ncol(X), length(tau))
            rho <- rep(0, length(tau))
            fitted <- resid <- matrix(0, nrow(X), length(tau))
            for (i in 1:length(tau)) {
                z <- {
                    if (length(weights))
                      rq.wfit(X, Y, tau = tau[i], weights, method, ...)
                    else rq.fit(X, Y, tau = tau[i], method, ...)
                }
                coef[, i] <- z$coefficients
                resid[, i] <- z$residuals
                rho[i] <- sum(Rho(z$residuals, tau[i]))
                fitted[, i] <- Y - z$residuals
            }
            taulabs <- paste("tau=", format(round(tau, 3)))
            dimnames(coef) <- list(dimnames(X)[[2]], taulabs)
            dimnames(resid) <- list(dimnames(X)[[1]], taulabs)
            rval <- z
            rval$coefficients <- coef
            rval$residuals <- resid
            rval$fitted.values <- fitted
            class(rval) <- c("dynrqs", "rqs")
        }
    else {
        rval <- {
            if(length(weights))
                rq.wfit(X, Y, tau = tau, weights, method, ...)
            else rq.fit(X, Y, tau = tau, method, ...)
        }
        dimnames(rval$residuals) <- list(dimnames(X)[[1]], NULL)
        rho <- sum(Rho(rval$residuals, tau))
        class(rval) <- "rq"
        class(rval) <- c("dynrq", "rq")
        }

    }
    rval$na.action <- attr(mf, "na.action")
    rval$contrasts <- attr(X, "contrasts")
    rval$xlevels <- .getXlevels(mt, mf)
    rval$call <- cl
    rval$tau <- tau
    rval$terms <- mt
    rval$model <- mf
    rval$index <- zoo::index(mf1)
    rval$frequency <- frequency(mf1)
    rval$residuals <- drop(rval$residuals)
    rval$X <- X
    rval$y <- Y
    rval$rho <- rho
    rval$method <- method
    rval$fitted.values <- drop(rval$fitted.values)

    return(rval)
}

index.dynrq <- function(x, ...) {
  x$index
}

start.dynrq <- function(x, ...) {
  start(x$residuals)
}

end.dynrq <- function(x, ...) {
  end(x$residuals)
}

print.dynrq <- function(x, ...) {
  rx <- residuals(x)
  cat(paste("\nDynamic quantile regression \"", class(rx)[1], "\" data:\n", sep = ""))
  cat(paste("Start = ", zoo::index2char(zoo::index(rx)[1], x$frequency),
            ", End = ",   zoo::index2char(zoo::index(rx)[length(rx)], x$frequency), "\n", sep = ""))
  NextMethod()
}
print.dynrqs <- function(x, ...) {
  rx <- residuals(x)
  ix <- dimnames(rx)[[1]]
  cat(paste("\nDynamic quantile regression \"", class(rx)[1], "\" data:\n", sep = ""))
  cat(paste("Start = ", zoo::index2char(ix[1], x$frequency),
            ", End = ",   zoo::index2char(ix[length(ix)], x$frequency), "\n", sep = ""))
  NextMethod()
}

summary.dynrqs <- function(object, vcov. = NULL, df = NULL, ...) {
  rval <- NextMethod()
  #rval$frequency <- object$frequency
  #class(rval) <- c("summary.dynrqs", class(rval))
  return(rval)
}

print.summary.dynrq <- function(x, ...) {
  rx <- residuals(x)
  x$residuals <- zoo::coredata(x$residuals)
  cat(paste("\nDynamic quantile  regression \"", class(rx)[1], "\" data:\n", sep = ""))
  cat(paste("Start = ", zoo::index2char(zoo::index(rx)[1], x$frequency),
            ", End = ",   zoo::index2char(zoo::index(rx)[length(rx)], x$frequency), "\n", sep = ""))
  NextMethod()
}
print.summary.dynrqs <- function(x, ...) {
  lapply(x, print)
}

      SUBROUTINE RQ1(M,N,MDIM,N2,A,B,T,TOLER,IFT,X,E,S,WA,WB)
C
C     Modified to remove SOL and related vars -- only good for single tau
C     Number of Observations
C     Number of Parameters
C     MDIM  row dimension for arrays  A  and  WA
C     N+2
C     A is the X matrix
C     B is the y vector
C     T, the desired quantile
C     TOLER, smallest detectable |x-y|/x machine precision to the 2/3
C     IFT exit code:
C		0-ok
C		else dimensions inconsistent or T not in (0,1)
C     X the parameter estimate betahat
C     E is the residual vector
C     S is an integer work array (M)
C     WA is a real work array (M5,N2)
C     WB is another real work array (M)
C     Utilization:  If you just want a solution at a single quantile you
C     The algorithm  is a slightly modified version of Algorithm AS 229 
C     described in Koenker and D'Orey, "Computing Regression Quantiles,
C     Applied Statistics, pp. 383-393. 
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      INTEGER I,J,K,KL,KOUNT,KR,M,M1,M2,M3,M4,M5,IFT
      INTEGER MDIM,N,N1,N2,OUT,S(MDIM)
      LOGICAL STAGE,TEST,INIT,IEND
      DOUBLE PRECISION MIN,MAX
      DIMENSION B(MDIM),A(MDIM,N),X(N),WA(MDIM,N2),
     *          WB(MDIM),E(M)
      DATA BIG/1.D37/
      DATA ZERO/0.00D0/
      DATA HALF/0.50D0/
      DATA ONE/1.00D0/
      DATA TWO/2.00D0/
C
C  CHECK DIMENSION PARAMETERS
C
      IFT=0
      IF(N2 .NE. N+2)IFT = 4
      IF(M.LE.ZERO.OR.N.LE.ZERO)IFT = 5
      IF(IFT .GT. TWO)RETURN
C
C  INITIALIZATION
C
      M1 = M+1
      N1 = N+1
      M2 = M+2
      M3 = M+3
      M4 = M+4
      M5 = M+5
      DO 2 I=1,M
      WB(I)=B(I)
      DO 1 J=1,N
      WA(I,J)=A(I,J)
  1   CONTINUE
  2   CONTINUE
      WA(M2,N1)=ZERO
      DIF = ZERO
      IEND = .TRUE.
      IF(T .GE. ZERO .AND. T .LE. ONE)GOTO 3
      IFT = 6
      RETURN
  3   CONTINUE
      INIT = .FALSE.
      KOUNT = 0
      DO 9 K=1,N
      WA(M5,K) = ZERO
      DO 8 I=1,M
      WA(M5,K) = WA(M5,K) + WA(I,K)
  8   CONTINUE
      WA(M5,K) = WA(M5,K)/FLOAT(M)
  9   CONTINUE
      DO 10 J=1,N
      WA(M4,J) = J
      X(J) = ZERO
 10   CONTINUE
      DO 40 I=1,M
      WA(I,N2) = N+I
      WA(I,N1) = WB(I)
      IF(WB(I).GE.ZERO)GOTO 30
      DO 20 J=1,N2
      WA(I,J) = -WA(I,J)
 20   CONTINUE
 30   E(I) = ZERO
 40   CONTINUE
      DO 42 J=1,N
      WA(M2,J) = ZERO
      WA(M3,J) = ZERO
      DO 41 I=1,M
      AUX = SIGN(ONE,WA(M4,J)) * WA(I,J)
      WA(M2,J) = WA(M2,J) + AUX * (ONE - SIGN(ONE,WA(I,N2)))
      WA(M3,J) = WA(M3,J) + AUX * SIGN(ONE,WA(I,N2))
 41   CONTINUE
      WA(M3,J) = TWO * WA(M3,J)
 42   CONTINUE
      GOTO 48
 43   CONTINUE
      DO 44 I=1,M
      S(I) = ZERO
 44   CONTINUE
      DO 45 J=1,N
      X(J) = ZERO
 45   CONTINUE
C
C  COMPUTE NEXT T
C
      SMAX = TWO
      DO 47 J=1,N
      B1 =  WA(M3,J)
      A1 = (-TWO - WA(M2,J))/B1
      B1 = -WA(M2,J)/B1
      IF(A1 .LT. T)GOTO 46
      IF(A1 .GE. SMAX) GOTO 46
      SMAX = A1
      DIF = (B1 - A1 )/TWO
 46   IF(B1 .LE. T) GOTO 47
      IF(B1 .GE. SMAX)GOTO 47
      SMAX = B1
      DIF = (B1 - A1)/TWO
 47   CONTINUE
      TNT = SMAX + TOLER * (ONE + ABS(DIF)) 
      IF(TNT .GE. T1 + TOLER)IEND = .TRUE.
      T = TNT
      IF(IEND)T = T1
 48   CONTINUE
C
C COMPUTE NEW MARGINAL COSTS
C
      DO 49 J=1,N
      WA(M1,J) = WA(M2,J) + WA(M3,J) * T
 49   CONTINUE
      IF(INIT) GOTO 265
C
C STAGE 1
C
C DETERMINE THE VECTOR TO ENTER THE BASIS
C
      STAGE=.TRUE.
      KR=1
      KL=1
 70   MAX=-ONE
      DO 80 J=KR,N
      IF(ABS(WA(M4,J)).GT.N)GOTO 80
      D=ABS(WA(M1,J))
      IF(D.LE.MAX)GOTO 80
      MAX=D
      IN=J
 80   CONTINUE
      IF(WA(M1,IN).GE.ZERO)GOTO 100
      DO 90 I=1,M4
      WA(I,IN)=-WA(I,IN)
 90   CONTINUE
C
C DETERMINE THE VECTOR TO LEAVE THE BASIS
C
 100  K=0
      DO 110 I=KL,M
      D=WA(I,IN)
      IF(D.LE.TOLER)GOTO 110
      K=K+1
      WB(K)=WA(I,N1)/D
      S(K)=I
      TEST=.TRUE.
 110  CONTINUE
 120  IF(K.GT.0)GOTO 130
      TEST=.FALSE.
      GOTO 150
 130  MIN=BIG
      DO 140 I=1,K
      IF(WB(I).GE.MIN)GOTO 140
      J=I
      MIN=WB(I)
      OUT=S(I)
 140  CONTINUE
      WB(J)=WB(K)
      S(J)=S(K)
      K=K-1
C
C CHECK FOR LINEAR DEPENDENCE IN STAGE 1
C
 150  IF(TEST.OR..NOT.STAGE)GOTO 170
      DO 160 I=1,M4
      D=WA(I,KR)
      WA(I,KR)=WA(I,IN)
      WA(I,IN)=D
 160  CONTINUE
      KR=KR+1
      GOTO 260
 170  IF(TEST)GOTO 180
      WA(M2,N1)=TWO
      GOTO 390
 180  PIVOT=WA(OUT,IN)
      IF(WA(M1,IN)-PIVOT-PIVOT.LE.TOLER)GOTO 200
      DO 190 J=KR,N1
      D=WA(OUT,J)
      WA(M1,J)=WA(M1,J)-D-D
      WA(M2,J)=WA(M2,J)-D-D
      WA(OUT,J)=-D
 190  CONTINUE
      WA(OUT,N2)=-WA(OUT,N2)
      GOTO 120
C
C PIVOT ON WA(OUT,IN)
C
 200  DO 210 J=KR,N1
      IF(J.EQ.IN)GOTO 210
      WA(OUT,J)=WA(OUT,J)/PIVOT
 210  CONTINUE
      DO 230 I=1,M3
      IF(I.EQ.OUT)GOTO 230
      D=WA(I,IN)
      DO 220 J=KR,N1
      IF(J.EQ.IN)GOTO 220
      WA(I,J)=WA(I,J)-D*WA(OUT,J)
 220  CONTINUE
 230  CONTINUE
      DO 240 I=1,M3
 
      IF(I.EQ.OUT)GOTO 240
      WA(I,IN)=-WA(I,IN)/PIVOT
 240  CONTINUE
      WA(OUT,IN)=ONE/PIVOT
      D=WA(OUT,N2)
      WA(OUT,N2)=WA(M4,IN)
      WA(M4,IN)=D
      KOUNT=KOUNT+1
      IF(.NOT.STAGE)GOTO 270
C
C INTERCHANGE ROWS IN STAGE 1
C
      KL=KL+1
      DO 250 J=KR,N2
      D=WA(OUT,J)
      WA(OUT,J)=WA(KOUNT,J)
      WA(KOUNT,J)=D
 250  CONTINUE
 260  IF(KOUNT+KR.NE.N1)GOTO 70
C
C STAGE 2
C
 265  STAGE=.FALSE.
C
C DETERMINE THE VECTOR TO ENTER THE BASIS
C
 270  MAX=-BIG
      DO 290 J=KR,N
      D=WA(M1,J)
      IF(D.GE.ZERO)GOTO 280
      IF(D.GT.(-TWO))GOTO 290
      D=-D-TWO
 280  IF(D.LE.MAX)GOTO 290
      MAX=D
      IN=J
 290  CONTINUE
      IF(MAX.LE.TOLER)GOTO 310
      IF(WA(M1,IN).GT.ZERO)GOTO 100
      DO 300 I=1,M4
      WA(I,IN)=-WA(I,IN)
 300  CONTINUE
      WA(M1,IN)=WA(M1,IN)-TWO
      WA(M2,IN)=WA(M2,IN)-TWO
      GOTO 100
C
C COMPUTE QUANTILES
C
 310  CONTINUE
      DO 320 I=1,KL-1
      K=WA(I,N2)*SIGN(ONE,WA(I,N2))
      X(K) = WA(I,N1) * SIGN(ONE,WA(I,N2))
 320  CONTINUE
 390  SUM = ZERO
      DO 400 I=KL,M
      K = WA(I,N2) * SIGN(ONE,WA(I,N2))
      D = WA(I,N1) * SIGN(ONE,WA(I,N2))
      SUM = SUM + D * SIGN(ONE,D) * (HALF + SIGN(ONE,D)*(T-HALF))
      K=K-N
      E(K)=D
 400  CONTINUE
      RETURN
      END

subroutine rqfnb(n,p,a,y,rhs,d,u,beta,eps,wn,wp,nit,info)
integer n,p,info,nit(3)
double precision a(p,n),y(n),rhs(p),d(n),u(n),wn(n,9),wp(p,p+3)
double precision one,beta,eps
parameter( one = 1.0d0)
call lpfnb(n,p,a,y,rhs,d,u,beta,eps,wn(1,1),wn(1,2),
        wp(1,1),wn(1,3),wn(1,4),wn(1,5),wn(1,6),
        wp(1,2),wn(1,7),wn(1,8),wn(1,9),wp(1,3),wp(1,4),nit,info)
return
end
# This is a revised form of my primal-dual log barrier form of the
# interior point LP solver based on Lustig, Marsten and Shanno ORSA J Opt 1992.
# It is a projected Newton primal-dual logarithmic barrier method which uses
# the predictor-corrector approach of Mehrotra for the mu steps.
# For the sake of brevity we will call it a Frisch-Newton algorithm.
# Problem:
# 	min c'x s.t. Ax=b, 0<=x<=u
# 
# Denote dx,dy,dw,ds,dz as the steps for the respective variables x,y,w,s,z
# 
subroutine lpfnb(n,p,a,c,b,d,u,beta,eps,x,s,y,z,w,
	dx,ds,dy,dz,dw,dr,rhs,ada,nit,info)

integer n,p,pp,i,info,nit(3),maxit
double precision a(p,n),c(n),b(p)
double precision zero,one,mone,big,ddot,dmax1,dmin1,dxdz,dsdw
double precision deltap,deltad,beta,eps,mu,gap,g
double precision x(n),u(n),s(n),y(p),z(n),w(n),d(n),rhs(p),ada(p,p)
double precision dx(n),ds(n),dy(p),dz(n),dw(n),dr(n)

parameter( zero  = 0.0d0)
parameter( one   = 1.0d0)
parameter( mone  = -1.0d0)
parameter( big  = 1.0d+20)
parameter( maxit  = 500)

# Initialization:  We follow the notation of LMS
# On input we require:
# 
# 	c = n-vector of marginal costs (-y in the rq problem)
# 	a = p by n matrix of linear constraints (x' in rq)
# 	b = p-vector of rhs ((1-tau)x'e in rq)
#       u = upper bound vector ( e in rq)
# 	beta = barrier parameter, LMS recommend .99995
# 	eps = convergence tolerance, LMS recommend 10d-8
# 	
# 	the integer vector nit returns iteration counts
# 	the integer info contains an error code from the Cholesky in stepy
# 		info = 0 is fine
# 		info < 0 invalid argument to dposv 
# 		info > 0 singular matrix
nit(1)=0
nit(2)=0
nit(3)=n
pp=p*p
# Start at the OLS estimate for the dual vector y
call dgemv('N',p,n,one,a,p,c,1,zero,y,1)
do i=1,n
	d(i)=one
call stepy(n,p,a,d,y,ada,info)
if(info != 0) return
# put current residual vector in s (temporarily)
call dcopy(n,c,1,s,1)
call dgemv('T',p,n,mone,a,p,y,1,one,s,1)
# Initialize remaining variables
do i=1,n{
	if(dabs(s(i))<eps){
		z(i)=dmax1(s(i), zero) + eps
		w(i)=dmax1(-s(i),zero) + eps
		}
	else {
		z(i)=dmax1(s(i), zero) 
		w(i)=dmax1(-s(i),zero)
		}
	s(i)=u(i)-x(i)
	}
gap = ddot(n,z,1,x,1)+ddot(n,w,1,s,1)
while(gap > eps && nit(1)<maxit) {
	nit(1)=nit(1)+1
	do i = 1,n{
		d(i) = one/(z(i)/x(i) + w(i)/s(i))
		ds(i)=z(i)-w(i)
		dz(i)=d(i)*ds(i)
		}
	call dcopy(p,b,1,dy,1)#save rhs
	call dgemv('N',p,n,mone,a,p,x,1,one,dy,1) 
	call dgemv('N',p,n,one,a,p,dz,1,one,dy,1) 
	call dcopy(p,dy,1,rhs,1)#save rhs
	call stepy(n,p,a,d,dy,ada,info)
	if(info != 0) return
	call dgemv('T',p,n,one,a,p,dy,1,mone,ds,1) #ds -> A'dy - ds
	deltap=big
	deltad=big
	do i=1,n{
		dx(i)=d(i)*ds(i)
		ds(i)=-dx(i)
		dz(i)=-z(i)*(dx(i)/x(i) + one)
		dw(i)=-w(i)*(ds(i)/s(i) + one)
		if(dx(i)<0)deltap=dmin1(deltap,-x(i)/dx(i))
		if(ds(i)<0)deltap=dmin1(deltap,-s(i)/ds(i))
		if(dz(i)<0)deltad=dmin1(deltad,-z(i)/dz(i))
		if(dw(i)<0)deltad=dmin1(deltad,-w(i)/dw(i))
		}
	deltap=dmin1(beta*deltap,one)
	deltad=dmin1(beta*deltad,one)
	if(min(deltap,deltad) < one){
		nit(2)=nit(2)+1
		# Update mu
		mu = ddot(n,x,1,z,1)+ddot(n,s,1,w,1)
		g = mu + deltap*ddot(n,dx,1,z,1)+
			deltad*ddot(n,dz,1,x,1) +
			deltap*deltad*ddot(n,dz,1,dx,1)+
			deltap*ddot(n,ds,1,w,1)+
			deltad*ddot(n,dw,1,s,1) +
			deltap*deltad*ddot(n,ds,1,dw,1)
		mu = mu * ((g/mu)**3) /dfloat(2*n)
		# Compute modified step
		do i=1,n{
			dr(i)=d(i)*(mu*(1/s(i)-1/x(i))+
				dx(i)*dz(i)/x(i)-ds(i)*dw(i)/s(i))
			}
		call dswap(p,rhs,1,dy,1)
		call dgemv('N',p,n,one,a,p,dr,1,one,dy,1)# new rhs
		call dpotrs('U',p,1,ada,p,dy,p,info)# backsolve for dy
		call dgemv('T',p,n,one,a,p,dy,1,zero,u,1)#ds=A'ddy
		deltap=big
		deltad=big
		do i=1,n{
			dxdz =  dx(i)*dz(i)
			dsdw =  ds(i)*dw(i)
			dx(i)=  d(i)*(u(i)-z(i)+w(i))-dr(i)
			ds(i)= -dx(i)
			dz(i)= -z(i)+(mu - z(i)*dx(i) - dxdz)/x(i)
			dw(i)= -w(i)+(mu - w(i)*ds(i) - dsdw)/s(i)
			if(dx(i)<0)deltap=dmin1(deltap,-x(i)/dx(i))
			if(ds(i)<0)deltap=dmin1(deltap,-s(i)/ds(i))
			if(dz(i)<0)deltad=dmin1(deltad,-z(i)/dz(i))
			if(dw(i)<0)deltad=dmin1(deltad,-w(i)/dw(i))
			}
		deltap=dmin1(beta*deltap,one)
		deltad=dmin1(beta*deltad,one)
		}
	call daxpy(n,deltap,dx,1,x,1)
	call daxpy(n,deltap,ds,1,s,1)
	call daxpy(p,deltad,dy,1,y,1)
	call daxpy(n,deltad,dz,1,z,1)
	call daxpy(n,deltad,dw,1,w,1)
	gap = ddot(n,z,1,x,1)+ddot(n,w,1,s,1)
	}
# return residuals in the vector x
call daxpy(n,mone,w,1,z,1)
call dswap(n,z,1,x,1)
return
end
subroutine stepy(n,p,a,d,b,ada,info)
integer n,p,pp,i,info
double precision a(p,n),b(p),d(n),ada(p,p),zero
parameter( zero = 0.0d0)
# Solve the linear system ada'x=b by Choleski -- d is diagonal
# Note that a isn't altered, and on output ada returns the upper
# triangle Choleski factor, which can be reused, eg with blas dtrtrs
pp=p*p
do j=1,p
	do k=1,p
		ada(j,k)=zero
do i=1,n
	call dsyr('U',p,d(i),a(1,i),1,ada,p)
call dposv('U',p,1,ada,p,b,p,info)
return
end

# This is a special version of K-dO rq modified to compute the LR process
# Note that the sol array is now p+3 by J and the third row contains the
# value of the objective function at the tau specified in row one.
#
#     This software is in the public domain  and may be freely
#     used and redistributed for non-commercial purposes.  No guarantees
#     are offered or implied.  comments, bug reports, etc are welcome
#     and should be sent to roger@ysidro.econ.uiuc.edu or to
#
#        roger koenker
#        department of economics
#        university of illinois
#        champaign, illinois, 61820
#
#
subroutine rqbr(m,nn,m5,n3,n4,a,b,t,toler,ift,x,e,s,wa,wb,nsol,ndsol,sol,dsol,lsol,h,qn,cutoff,ci,tnmat,big,lci1)
#
#     m = number of observations
#     n = number of parameters
#     m5 = m+5
#     n3 = n+3
#     n4 = n+4
#     a is the x matrix
#     b is the y vector
#     t, the desired quantile
#     toler, smallest detectable |x-y|/x machine precision to the 2/3
#     ift exit code:
#     		0-ok
#    		else dimensions inconsistent see below
#     x the parameter estimate betahat
#     e is the residual vector
#     s is an integer work array
#     wa is a real work array
#     wb is another real work array
#     nsol is an estimated (row) dimension of the primal solution array say 3*m; minimum value = 2
#     ndsol is an estimated (row) dimension of the dual solution array say 3*m; minimum value = 2
#     sol is the primal solution array
#     dsol is the dual solution arry
#     lsol is the actual dimension of the solution arrays
#     h is the matrix of basic observations indices
#     qn is the vector of residuals variances from the projection of each column of the x matrix on the remaining columns
#     cutoff, the critical point for N(0,1)
#     ci is the matrix of confidence intervals
#     tnmat is the matrix of the JGPK rank test statistics
#     big, large positive finite floating-point number
#     utilization:  if you just want a solution at a single quantile you
#     neednt bother with sol, nsol, etc, if you want all the solutions
#     then set theta to something <0 and sol and dsol will return all the
#     estimated quantile solutions.
#     the algorithm  is a slightly modified version of algorithm as 229
#     described in koenker and dorey, computing regression quantiles,
#     applied statistics, pp. 383-393.
#
integer i,j,k,kl,kount,kr,l,lsol,m,m1,m2,m3,m4,m5,ift
integer n,n1,n2,n3,nsol,ndsol,out,s(m),h(nn,nsol)
integer nn,n4,idxcf
logical stage,test,init,iend,lup
logical lci1,lci2,skip
double precision a1,aux,b1,big,d,dif,pivot,smax,t,t0,t1,tnt,dk
double precision min,max,toler,zero,half,one,two
double precision b(m),sol(n3,nsol),a(m,nn),x(nn),wa(m5,n4),wb(m)
double precision sum,e(m),dsol(m,ndsol)
double precision qn(nn),cutoff,ci(4,nn),tnmat(4,nn),tnew,told,tn
parameter( zero = 0.d0)
parameter( one = 1.d0)
parameter( two = 2.d0)
#
#  check dimension parameters
#
n=nn
ift = 0
wa(m+2,nn+1) = one
if (m5!=m+5)
  ift = 3
if (n3!=n+3)
  ift = 4
if (n4!=n+4)
  ift = 5
if (m<=zero||n<=zero)
  ift = 6
if (ift<=two) {
#
#  initialization
#
  half = one/two
  iend = .true.
  lci2 = .false.
  lup = .true.
  skip = .false.
  idxcf = 0
  tnew = zero
  tn = zero
  m1 = m+1
  n1 = n+1
  n2 = n+2
  m2 = m+2
  m3 = m+3
  m4 = m+4
  do j = 1,n{
    x(j) = zero
    }
  do i = 1,m
    e(i) = zero
  if (t<zero||t>one) {
    t0 = one/(two*float(m))
    t1 = one-t0
    t = t0
    iend = .false.
    lci1 = .false.
    }
  repeat { #0
    do i = 1,m {
      k = 1
      do j = 1,nn
        if (k<=nn){
          if(j==idxcf)
            skip = .true.
          else
            skip = .false.
          if(!skip){
            wa(i,k) = a(i,j)
            k = k+1
            }
          }  
      wa(i,n4) = n+i
      wa(i,n2) = b(i)
      if (idxcf != 0)
        wa(i,n3) = tnew*a(i,idxcf)
      else
        wa(i,n3) = zero
      wa(i,n1) = wa(i,n2)-wa(i,n3)
      if (wa(i,n1)<zero)
        do j = 1,n4
          wa(i,j) = -wa(i,j)
      }
    do j = 1,n {
      wa(m4,j) = j
      wa(m2,j) = zero
      wa(m3,j) = zero
      do i = 1,m {
        aux = sign(one,wa(m4,j))*wa(i,j)
        wa(m2,j) = wa(m2,j)+aux*(one-sign(one,wa(i,n4)))
        wa(m3,j) = wa(m3,j)+aux*sign(one,wa(i,n4))
        }
      wa(m3,j) = two*wa(m3,j)
      }
    dif = zero
    init = .false.
    if(!lci2){
#compute the columns means
      do k = 1,n {
        wa(m5,k) = zero
        do i = 1,m{
          wa(m5,k) = wa(m5,k)+a(i,k)
          }
        wa(m5,k) = wa(m5,k)/float(m)
        }
      }
    lsol = 1
    kount = 0
    repeat { #1
#
# compute new marginal costs
#
      do j = 1,n{
        wa(m1,j) = wa(m2,j)+wa(m3,j)*t
        }
      if (!init) {
#
# stage 1
#
# determine the vector to enter the basis
#
        stage = .true.
        kr = 1
        kl = 1
        go to 30
        }
      repeat { #2
#
# stage 2
#
        stage = .false.
        repeat { #3
#
# determine the vector to enter the basis
#
          max = -big
          do j = kr,n {
            d = wa(m1,j)
            if (d<zero) {
              if (d>(-two))
                next 1
              d = -d-two
              }
            if (d>max) {
              max = d
              in = j
              }
            }
          if (max<=toler)
            break 2
          if (wa(m1,in)<=zero) {
            do i = 1,m4
              wa(i,in) = -wa(i,in)
            wa(m1,in) = wa(m1,in)-two
            wa(m2,in) = wa(m2,in)-two
            }
          repeat { #4
#
# determine the vector to leave the basis
#
            k = 0
            do i = kl,m {
              d = wa(i,in)
              if (d>toler) {
                k = k+1
                wb(k) = wa(i,n1)/d
                s(k) = i
                test = .true.
                }
              }
            repeat { #5
              if (k<=0)
                test = .false.
              else {
                min = big
                do i = 1,k
                  if (wb(i)<min) {
                    j = i
                    min = wb(i)
                    out = s(i)
                    }
                wb(j) = wb(k)
                s(j) = s(k)
                k = k-1
                }
#
# check for linear dependence in stage 1
#
              if (!test&&stage)
                break 1
              if (!test)
                break 5
              pivot = wa(out,in)
              if (wa(m1,in)-pivot-pivot<=toler){
                go to 10
                }
              do j = kr,n3 {
                d = wa(out,j)
                wa(m1,j) = wa(m1,j)-d-d
                wa(m2,j) = wa(m2,j)-d-d
                wa(out,j) = -d
                }
              wa(out,n4) = -wa(out,n4)
              } #5
            do i = 1,m4 {
              d = wa(i,kr)
              wa(i,kr) = wa(i,in)
              wa(i,in) = d
              }
            kr = kr+1
            go to 20
#
# pivot on wa(out,in)
#
            10  do j = kr,n3
              if (j!=in)
                wa(out,j) = wa(out,j)/pivot
            do i = 1,m3
              if (i!=out) {
                d = wa(i,in)
                do j = kr,n3
                  if (j!=in)
                    wa(i,j) = wa(i,j)-d*wa(out,j)
                }
            do i = 1,m3
              if (i!=out)
                wa(i,in) = -wa(i,in)/pivot
            wa(out,in) = one/pivot
            d = wa(out,n4)
            wa(out,n4) = wa(m4,in)
            wa(m4,in) = d
            kount = kount+1
            if (!stage)
              break 1
#
# interchange rows in stage 1
#
            kl = kl+1
            do j = kr,n4 {
              d = wa(out,j)
              wa(out,j) = wa(kount,j)
              wa(kount,j) = d
              }
            20  if (kount+kr==n1)
              break 2
            30  max = -one
            do j = kr,n
              if (abs(wa(m4,j))<=n) {
                d = abs(wa(m1,j))
                if (d>max) {
                  max = d
                  in = j
                  }
                }
            if (wa(m1,in)<zero)
              do i = 1,m4
                wa(i,in) = -wa(i,in)
            } #4
          } #3
        } #2
      if (kr==1) {
        do j = 1,n {
          d = abs(wa(m1,j))
          if (d<=toler||two-d<=toler){
            ift = 1
            wa(m2,nn+1) = zero
            go to 80
            }
          }
        }
      80 kount = 0
      sum = zero
      if (!lci2){
        do i = 1,kl-1 {
          k = wa(i,n4)*sign(one,wa(i,n4))
          x(k) = wa(i,n1)*sign(one,wa(i,n4))
          }
        }
      do i = 1,n {
        kd = abs(wa(m4,i))-n
        dsol(kd,lsol) = one+wa(m1,i)/two
        if (wa(m4,i)<zero)
          dsol(kd,lsol) = one-dsol(kd,lsol)
        if (!lci2){
          sum = sum + x(i)*wa(m5,i)
          sol(i+3,lsol) = x(i)
          h(i,lsol) = kd
          }
        }
      do i = kl,m {
        kd = abs(wa(i,n4))-n
        if (wa(i,n4)<zero)
          dsol(kd,lsol) = zero
        if (wa(i,n4)>zero)
          dsol(kd,lsol) = one
        }
      if (!lci2){
        sol(1,lsol) = smax
        sol(2,lsol) = sum
        sum = zero
        do j=kl,m{
          d = wa(j,n1)*sign(one,wa(j,n4))
          sum = sum + d*(smax + half*(sign(one,d) - one))
	 }
        sol(3,lsol) = sum
        do i=1,m
          dsol(i,lsol+1) = dsol(i,lsol)
        }
      if (lci2){
# compute next theta
        a1 = zero
        do i = 1,m {
          a1 = a1+a(i,idxcf)*(dsol(i,lsol)+t-one)
          }
        tn = a1/sqrt(qn(idxcf)*t*(one-t))
        if (abs(tn)<cutoff){
          if (lup)
            smax = big
          else
            smax = -big
          do i =1,kl-1 {
            k = wa(i,n4)*sign(one,wa(i,n4))
            sol(k,1) = wa(i,n2)*sign(one,wa(i,n4))
            sol(k,2) = wa(i,n3)*sign(one,wa(i,n4))/tnew
            }
          do i = kl,m {
            a1 = zero
            b1 = zero
            k = wa(i,n4)*sign(one,wa(i,n4))-n
            dk = wa(i,n4)*sign(one,wa(i,n4))
            l = 1
            do j = 1,n{
              if (j==idxcf)
                l = l+1
              a1 = a1 + a(k,l)*sol(j,1)
              b1 = b1 + a(k,l)*sol(j,2)
              l = l+1
              }
            tnt = (b(k)-a1)/(a(k,idxcf)-b1)
            if (lup){
              if (tnt>tnew)
                if (tnt<smax){
                  smax = tnt
                  out = i
                  }
              }
            else {
              if (tnt<tnew)
                if (tnt>smax){
                  smax = tnt
                  out = i
                  }
              }
            }
          if (lup){
            told = tnew 
            tnew = smax + toler
            ci(3,idxcf) = told - toler
            tnmat(3,idxcf) = tn
            if (!(tnew < big-toler)){
              ci(3,idxcf) = big
              ci(4,idxcf) = big
              tnmat(3,idxcf) = tn
              tnmat(4,idxcf) = tn
              lup = .false.
              go to 70
              }
            }
          else{
            told = tnew 
            tnew = smax - toler
            ci(2,idxcf) = told + toler
            tnmat(2,idxcf) = tn
            if (!(tnew > -big+toler)){
              ci(2,idxcf) = -big
              ci(1,idxcf) = -big
              tnmat(2,idxcf) = tn
              tnmat(1,idxcf) = tn
              lup = .true.
              go to 60
              }
            }
#update the new marginal cost
          do i = 1,m{
            wa(i,n3) = wa(i,n3)/told*tnew
            wa(i,n1) = wa(i,n2) - wa(i,n3)
            }
          do j = kr,n3{
            d = wa(out,j)
            wa(m1,j) = wa(m1,j) -d -d
            wa(m2,j) = wa(m2,j) -d -d
            wa(out,j) = -d
            }
          wa(out,n4) = -wa(out,n4)
          init = .true.
          }
        else{
          if (lup){
            ci(4,idxcf) = tnew - toler
            tnmat(4,idxcf) = tn
            lup = .false.
            go to 70
            }
          else{
            ci(1,idxcf) = tnew + toler
            tnmat(1,idxcf) = tn
            lup = .true.
            go to 60
            }
          }
        }
      if ((iend)&&(!lci2))
        go to 40
      if (!lci2){
        init = .true.
        lsol = lsol+1
        do i = 1,m
          s(i) = zero
        do j = 1,n
          x(j) = zero
#
#  compute next t
#
        smax = two
        do j = 1,n {
          b1 = wa(m3,j)
          a1 = (-two-wa(m2,j))/b1
          b1 = -wa(m2,j)/b1
          if (a1>=t)
            if (a1<smax) {
              smax = a1
              dif = (b1-a1)/two
              }
          if (b1>t)
            if (b1<smax) {
              smax = b1
              dif = (b1-a1)/two
              }
          }
        tnt = smax+toler*(one+abs(dif))
        if (tnt>=t1+toler)
          iend = .true.
        t = tnt
        if (iend)
          t = t1
        }
      } #1
    wa(m2,nn+1) = two
    ift = 2
    go to 50
    40  if (lsol>2) {
      sol(1,1) = zero
      #sol(2,1) = zero
      sol(3,1) = zero
      sol(1,lsol) = one
      #sol(2,lsol) = zero
      sol(3,lsol) = zero
      do i = 1,m {
        dsol(i,1) = one
        dsol(i,lsol) = zero
        dsol(i,lsol+1) = zero
        }
      }
    l = kl-1
    do i = 1,l
      if (wa(i,n1)<zero)
        do j = kr,n4
          wa(i,j) = -wa(i,j)
    50  sum = zero
    if(!lci2){
      do i = kl,m {
        k = wa(i,n4)*sign(one,wa(i,n4))
        d = wa(i,n1)*sign(one,wa(i,n4))
        sum = sum+d*sign(one,d)*(half+sign(one,d)*(t-half))
        k = k-n
        e(k) = d
        }
      wa(m2,n2) = kount
      wa(m1,n2) = n1-kr
      wa(m1,n1) = sum
      }
    if (wa(m2,nn+1)==two)
      break 1
    if (!lci1)
      break 1
    if (!lci2){
      lci2 = .true.
      n = nn-1
      n1 = n+1
      n2 = n+2
      n3 = n+3
      n4 = n+4
    60 idxcf = idxcf+1
      if (idxcf>nn){
        break 1
	}
    70  if (lup){
        tnew = x(idxcf)+toler
        told = tnew
        ci(3,idxcf) = x(idxcf)
        tnmat(3,idxcf) = zero
        }
      else{
        tnew = x(idxcf)-toler
        told = tnew
        ci(2,idxcf) = x(idxcf)
        tnmat(2,idxcf) = zero
        }
      }
    } #0
  # restore the original value of dual when ci is true
  do i=1,m
    dsol(i,lsol) = dsol(i,lsol+1)
  }
return
end

C Output from Public domain Ratfor, version 1.05
      subroutine akj(x,z,p,iker,dens,psi,score,nx,nz,h,alpha,kappa,xlam
     *)
      double precision dens(nz),score(nz),psi(nz),h,kappa
      double precision z(nz),x(nx),xlam(nx),p(nx),qrange,pi
      double precision con1,sum,sqsum,xsd,a,fifth,hinv,half
      double precision xn,xker,dxker,ddxker,fact,xponen,alpha,glog,zero,
     *one,two
      parameter( zero = 0.d0)
      parameter( one = 1.d0)
      parameter( two = 2.d0)
      parameter( four = 4.d0)
      parameter( half = 0.5d0)
      parameter( fifth = 0.2d0)
      parameter( pi = 3.141593d0)
      xn=nx
      if(iker.eq.0)then
      con1=one/sqrt(2.0*pi)
      else
      if(iker.eq.1)then
      con1=one/pi
      endif
      endif
      if(h.le.0.)then
      sum=0.
      sqsum=0.
      do23006 i=1,nx
      sqsum=sqsum+x(i)*x(i)*p(i)
      sum=sum+x(i)*p(i)
23006 continue
23007 continue
      xsd=dsqrt(sqsum-sum*sum)
      sum=zero
      i=1
23008 if(.not.(i.lt.nx))goto 23010
      sum=sum+p(i)
      if(sum.lt..25)then
      goto 23009
      else
      qrange=x(i)
      goto 23010
      endif
23009 i=i+1
      goto 23008
23010 continue
      sum=one
      i=nx
23013 if(.not.(i.gt.0))goto 23015
      sum=sum-p(i)
      if(sum.gt..75)then
      goto 23014
      else
      qrange=x(i)-qrange
      goto 23015
      endif
23014 i=i-1
      goto 23013
23015 continue
      a=min(xsd,qrange/1.34)
      h=kappa*a/(xn**fifth)
      endif
      hinv=one/h
      do23018 j=1,nx 
      xker=0.
      if(iker.eq.0)then
      do23022 i=1,nx 
      xponen=(x(j)-x(i))*hinv
      xponen=half*xponen**2
      xker=xker+p(i)*exp(-xponen)*hinv
23022 continue
23023 continue
      else
      if(iker.eq.1)then
      do23026 i=1,nx 
      xponen=(x(j)-x(i))*hinv
      xker=xker+p(i)*hinv/(1+xponen**2)
23026 continue
23027 continue
      endif
      endif
      xlam(j)=con1*xker
23018 continue
23019 continue
      glog=zero
      do23028 i=1,nx
      glog=glog+p(i)*log(xlam(i))
23028 continue
23029 continue
      g=exp(glog)
      ginv=one/g
      do23030 i=1,nx 
      xlam(i)=hinv/((xlam(i)*ginv)**(-alpha))
23030 continue
23031 continue
      do23032 j=1,nz 
      xker=zero
      dxker=zero
      ddxker=zero
      if(iker.eq.0)then
      do23036 i=1,nx 
      xponen=(z(j)-x(i))*xlam(i)
      fact=exp(-half*xponen*xponen)*xlam(i)
      xker=xker+p(i)*fact
      dxker=dxker-p(i)*fact*xponen*xlam(i)
      ddxker=ddxker- p(i)*fact*(one - xponen**2)*xlam(i)**2
23036 continue
23037 continue
      else
      if(iker.eq.1)then
      do23040 i=1,nx 
      xponen=(z(j)-x(i))*xlam(i)
      fact=xlam(i)/(one+xponen**2)
      xker=xker+p(i)*fact
      dxker=dxker-p(i)*two*xponen*fact**2
      ddxker=ddxker- p(i)*two*(fact**2)*(xlam(i)- four*(xponen**2)*fact)
23040 continue
23041 continue
      endif
      endif
      dens(j)=con1*xker
      psi(j)=-(dxker/xker)
      score(j)=(dxker/xker)**2-ddxker/xker
23032 continue
23033 continue
      return
      end

# Peng-Huang Censored Quantile Regression
subroutine crqfnb(n,p,a1,c1,n1,X,y,c,B,g,m,r,s,d,u,wn,wp,info)

# Input
#	a1 = p by n1 design matrix (X[c,] transposed)
#	c1 = n1 response vector -y[c] 
#	X  = n by p design matrix
#	y  = n response vector
#	c  = n censoring indicator
#	g  = m grid vector of taus
#	d  = residual n-vector, initialized == 1 
#	s  = initialized cumhaz vector (rep(0,n))
#	u  = upper bound n-vector, initalized == 1
# Workspace
#	r  = rhs p-vector
#	wn = work array n1 by 9
#	wp = work array p by p+3
# Output
#	B  = p by m matrix of estimated coefficients
#	m  = final column dimension of of B
# Note
#	d  is used both for residuals and as a work vector for rqfnb.
#	u  needs to be reinitialized to ones at each iteration
	
integer n,p,n1,m,info,nit(3)
double precision a1(p,n1),c1(n),X(n,p),y(n),c(n),B(p,m),g(m)
double precision wn(n1,9),wp(p,p+3),r(p),s(n),d(n),u(n)
double precision zero,half,one,beta,eps,dH

parameter( zero  =  0.0d0)
parameter( half  =  0.5d0)
parameter( one   =  1.0d0)
parameter( beta  =  0.99995d0)
parameter( eps   =  1.0d-8)

do k = 2,m {
	dH = -log(one - g(k)) + log(one - g(k-1))
	do i = 1,n {
		u(i) = one
		wn(i,1) = half # initialize dual vector
		if(d(i) >= zero) s(i) = s(i) + dH
		d(i) = c(i) - s(i)
		}
	call dgemv('T',n,p,one,X,n,d,1,zero,r,1)
	call rqfnb(n1,p,a1,c1,r,d,u,beta,eps,wn,wp,nit,info)
	if(info != 0) break
	call dcopy(p,wp,1,B(1,k-1),1)
	call dcopy(n,y,1,d,1)
	call dgemv('N',n,p,one,X,n,B(1,k-1),1,one,d,1)
	}
m = k-1
return
end

      subroutine dsel05( k, n, x)
      integer            k, n
      double precision   x(n)
c
c     Selects the smallest k elements of the array x[1:n].
c     The input array is permuted so that the smallest k elements of
c     x are x(i), i = 1,...,k, (in arbitrary order) and x(k) is the
c     kth smallest element.
c
c     This is a Fortran 77 version of the Algol 68 procedure from
c
c        R.W. Floyd and R.L. Rivest: "Algorithm 489: The Algorithm
c        SELECT---for Finding the $i$th Smallest of $n$ Elements",
c        Comm. ACM 18, 3 (1975) 173,
c
c     including some modifications suggested in
c
c        T. Brown: "Remark on Algorithm 489", ACM Trans. Math.
c        Software 3, 2 (1976), 301-304.
c
c     Array stack(2,nstack) permits up to nstack levels of recursion.
c     For standard parameters cs <= 1 and cutoff >= 600,
c     nstack = 5 suffices for n up to 2**31-1 (maximum integer*4).
      integer            nstack
      parameter         (nstack=10)
      integer            stack(2,nstack)
c
c     Parameters cutoff, cs and csd are as originally proposed.
      integer            cutoff
      parameter         (cutoff=600)
      double precision   cs, csd
      parameter         (cs=0.5d0, csd=0.5d0)
c     Brown's version
c     parameter         (cs=0.5d0, csd=0.1d0)
c
c     Subprograms called:
      intrinsic          dble, exp, log, max, min, sign
c
c     Written by K.C. Kiwiel, 8 March 2006, kiwiel@ibspan.waw.pl
c
c     Local variables:
      integer            i, j, jstack, l, m, r, s, sd
      double precision   dm, swap, v, z
      l=1
      r=n
      jstack=0
c     entry to SELECT( x, n, l, r, k)
c     SELECT will rearrange the values of the array segment x[l:r] so
c     that x(k) (for some given k; l <= k <= r) will contain the
c     (k-l+1)-th smallest value, l <= i <= k will imply x(i) <= x(k),
c     and k <= i <= r will imply x(k) <= x(i).
c     while r > l do
    1 continue
      if (l.ge.r) goto 6
c        The additional test below prevents stack overflow.
         if (r-l.gt.cutoff .and. jstack.lt.nstack) then
c           Use SELECT recursively on a sample of size s to get an
c           estimate for the (k-l+1)-th smallest element into x(k),
c           biased slightly so that the (k-l+1)-th element is
c           expected to lie in the smaller set after partitioning.
            m=r-l+1
            i=k-l+1
            dm=m
            z=log(dm)
            s=cs*exp(2*z/3)+0.5d0
            sd=csd*sqrt(z*s*(1-s/dm))*sign(1d0,i-dm/2)+0.5d0
            if (i.eq.m/2) sd=0
c           Brown's modification
c           sd=csd*sqrt(z*s*(1-s/dm))*(2*i/dm-1)+0.5d0
c           Push the current l and r on the stack.
            jstack=jstack+1
            stack(1,jstack)=l
            stack(2,jstack)=r
c           Find new l and r for the next recursion.
            l=max(dble(l),k-i*(s/dm)+sd)+0.5d0
            r=min(dble(r),k-i*(s/dm)+sd+s)+0.5d0
c           call SELECT( x, n, l, r, k)
            goto 1
         endif
    2    continue
c        Partition x[l:r] about the pivot v := x(k).
         v=x(k)
c        Initialize pointers for partitioning.
         i=l
         j=r
c        Swap x(l) and x(k).
         x(k)=x(l)
         x(l)=v
         if (v.lt.x(r)) then
c           Swap x(l) and x(r).
            x(l)=x(r)
            x(r)=v
         endif
c        while i < j do
    3    continue
         if (i.lt.j) then
c           Swap x(i) and x(j).
            swap=x(j)
            x(j)=x(i)
            x(i)=swap
            i=i+1
            j=j-1
c           Scan up to find element >= v.
    4       continue
            if (x(i).lt.v) then
               i=i+1
               goto 4
            endif
c           Scan down to find element <= v.
    5       continue
            if (x(j).gt.v) then
               j=j-1
               goto 5
            endif
            goto 3
c           end of while i < j do
         endif
         if (x(l).eq.v) then
c           Swap x(l) and x(j).
            swap=x(j)
            x(j)=v
            x(l)=swap
         else
            j=j+1
c           Swap x(j) and x(r).
            swap=x(j)
            x(j)=x(r)
            x(r)=swap
         endif
c        Now adjust l, r so that they surround the subset containing
c        the (k-l+1)-th smallest element.
         if (j.le.k) l=j+1
         if (k.le.j) r=j-1
         goto 1
c        end of while r > l do
    6 continue
c     Exit if the stack is empty.
      if (jstack.eq.0) return
c     Pop l and r from the stack.
      l=stack(1,jstack)
      r=stack(2,jstack)
      jstack=jstack-1
c     Continue as if after a return from a recursive call.
      goto 2
      end

# bootstrap estimation with fnb algorithm and globbing

# Input:
    # n = full sample size
    # p = parametric dimension of model
    # m = number of bootstrap replications
    # a = p by n (transposed) design matrix
    # y = n-vector of responses
    # w = n-vector of weights for bootstrap
    # r = residual n-vector from fit 
    # b = coefficient matrix 
    # band = confidence band (see rq.fit.pfnb)
    # n0 = initial sample size 
    # d = n-vector of ones
    # u = n-vector of ones
    # wn = 9*n work array
    # wp = p(p+3) work array
    # aa = n by p array for globbed design matrix
    # yy = n vector for globbed response matrix
    # slo = n-vector of boolean integers
    # shi = n-vector of boolean integers
    # rhs = p-vector for right hand side 
    # glob = p-vector for glob
    # ghib = p-vector for ghimb
    # nit = 3-vector of iteration counts x R
    # info = flag for convergence x R

subroutine pwxy(n,p,m,a,y,tau,qk,r,b,w,band,n0,d,u,wn,wp,
		aa,yy,slo,shi,rhs,glob,ghib,nit,info)
integer n,p,m,kk(2),nn,n0,nit(5,m),info(m),sumbad,i,j,k,ir
integer loq,hiq, slo(n),shi(n),ifix,ibad
logical notopt
double precision a(p,n),y(n),tau,qk(2),r(n),d(n),u(n),b(p,m),w(n)
double precision wn(n,9), wp(p,(p+3)),band(n)
double precision glob(p),ghib(p),aa(p,n),yy(n),rhs(p)
double precision zero,one,beta,eps,big

parameter(zero = 0.0d0)
parameter(one = 1.0d0)
parameter(beta = 0.99995d0)
parameter(big = 1.0d+10)
parameter(eps = 1.0d-06)

    
do ir = 1,m {
  notopt = .true.
  call grexp(n,w,one)
  nn = n0
  ifix = 0
  ibad = 0
  while (notopt) {
    ibad = ibad + 1
    loq = max0(1, int(n*tau - nn/2.))
    hiq = min0(int(n*tau + nn/2.), n)
    qk(1) = r(loq)
    qk(2) = r(hiq)
    call iphil(n,0,slo)
    call iphil(n,0,shi)
    do i = 1,n{
	if(r(i) < qk(1))
	    slo(i) = 1
	else if(r(i) > qk(2))
	    shi(i) = 1
    }
    while (notopt) {
	ifix = ifix + 1
	call dphil(p,zero,glob)
	call dphil(p,zero,ghib)
	call dphil(n,one,d)
	call dphil(n,one,u)
	k = 0
	do i = 1,n{
	    if(slo(i) == 0 & shi(i) == 0){
		k = k + 1
		call dphil(p,zero,aa(1,k))
		call daxpy(p,w(i),a(1,i),1,aa(1,k),1)
                yy(k) = -y(i)*w(i)
		}
	    else if (slo(i) == 1) {
		do j = 1,p 
		    glob(j) = glob(j) + a(j,i) * w(i)
                }
	    else if (shi(i) == 1) {
		do j = 1,p 
		    ghib(j) = ghib(j) + a(j,i) * w(i)
                }
	}
	call dcopy(p,glob,1,aa(1,k+1),1)
	call dcopy(p,ghib,1,aa(1,k+2),1)
        yy(k+1) =  big
        yy(k+2) = -big
	call dgemv('N',p,k+2,one-tau,aa,p,d,1,zero,rhs,1)
	call dscal(k+2,zero,wn,1)
	call daxpy(k+2,one-tau,u,1,wn,1)
        call rqfnb(k+2,p,aa,yy,rhs,d,u,beta,eps,wn,wp,nit(1,ir),info(ir))
	call dcopy(p,wp,1,b(1,ir),1)
	call dcopy(n,y,1,u,1)
	call dgemv('T',p,n,one,a,p,b(1,ir),1,one,u,1)
	# Check predicted signs of residuals (in u)
	sumbad = 0
	do i = 1,n {
	    if((u(i) > 0) & slo(i) == 1){
		slo(i) = 0
		sumbad = sumbad + 1
	    }
	    if((u(i) < 0) & shi(i) == 1){
		shi(i) = 0
		sumbad = sumbad + 1
	    }
	}
        if(sumbad > 0) {
            if (sumbad > 0.1 * nn) {
                nn = min(2 * nn, n)
                break
            }
        }
        else notopt = .false.
    }
    nit(4,ir) = ifix
    nit(5,ir) = ibad
  }
}
return
end

C Output from Public domain Ratfor, version 1.05
      subroutine rqfnc(n1,n2,p,a1,y,a2,r,rhs,d1,d2,u,beta,eps,wn1,wn2,wp
     *,nit,info)
      integer n1,n2,p,info,nit(3)
      double precision a1(p,n1),a2(p,n2),y(n1),r(n2),rhs(p),d1(n1),d2(n2
     *),u(n1)
      double precision wn1(n1,9),wn2(n2,6),wp(p,p+3)
      double precision one,beta,eps
      parameter(one = 1.0d0)
      call lpfnc(n1,n2,p,a1,y,a2,r,rhs,d1,d2,u,beta,eps,wn1(1,1),wn2(1,1
     *),wn1(1,2), wp(1,1),wn1(1,3),wn2(1,2),wn1(1,4),wn1(1,5),wn2(1,3),w
     *n1(1,6), wp(1,2),wn1(1,7),wn2(1,4),wn1(1,8),wn1(1,9),wn2(1,5),wn2(
     *1,6), wp(1,3),wp(1,4),nit,info)
      return
      end
      subroutine lpfnc(n1,n2,p,a1,c1,a2,c2,b,d1,d2,u,beta,eps,x1,x2,s, y
     *,z1,z2,w,dx1,dx2,ds,dy,dz1,dz2,dw,dr1,dr2,r2, rhs,ada,nit,info)
      integer n1,p,i,info,nit(3),maxit
      double precision a1(p,n1),a2(p,n2),c1(n1),c2(n2),b(p)
      double precision zero,one,mone,big,ddot,dmax1,dmin1,dxdz1,dxdz2,ds
     *dw
      double precision deltap,deltad,beta,eps,mu,gap,g
      double precision x1(n1),x2(n2),u(n1),s(n1),y(p),z1(n1),z2(n2),w(n1
     *)
      double precision d1(n1),d2(n2),rhs(p),ada(p,p)
      double precision dx1(n1),dx2(n2),ds(n1),dy(p),dz1(n1),dz2(n2),dw(n
     *1)
      double precision dr1(n1),dr2(n2),r2(n2)
      parameter(zero = 0.0d0)
      parameter(one = 1.0d0)
      parameter(mone = -1.0d0)
      parameter(big = 1.0d+20)
      parameter(maxit = 500)
      nit(1)=0
      nit(2)=0
      nit(3)=n1
      call dgemv('N',p,n1,one,a1,p,c1,1,zero,y,1)
      do23000 i=1,n1
      d1(i)=one
23000 continue
23001 continue
      do23002 i=1,n2
      d2(i)=zero
      z2(i)=one
23002 continue
23003 continue
      call stepy2(n1,n2,p,a1,d1,a2,d2,y,ada,info)
      if(info .ne. 0)then
      return
      endif
      call dcopy(n1,c1,1,s,1)
      call dgemv('T',p,n1,mone,a1,p,y,1,one,s,1)
      do23006 i=1,n1
      if(dabs(s(i)) .lt. eps)then
      z1(i)=dmax1(s(i),zero)+eps
      w(i)=dmax1(-s(i),zero)+eps
      else
      z1(i)=dmax1(s(i),zero)
      w(i)=dmax1(-s(i),zero)
      endif
      s(i)=u(i)-x1(i)
23006 continue
23007 continue
      gap = ddot(n1,z1,1,x1,1)+ddot(n2,z2,1,x2,1)+ddot(n1,w,1,s,1)
23010 if(gap .gt. eps .and. nit(1).lt.maxit)then
      nit(1)=nit(1)+1
      call dcopy(n2,c2,1,r2,1)
      call dgemv('T',p,n2,mone,a2,p,y,1,one,r2,1)
      call dcopy(p,b,1,dy,1)
      call dgemv('N',p,n1,mone,a1,p,x1,1,one,dy,1)
      call dgemv('N',p,n2,mone,a2,p,x2,1,one,dy,1)
      do23012 i = 1,n1
      d1(i)=one/(z1(i)/x1(i) + w(i)/s(i))
      ds(i)=z1(i)-w(i)
      dz1(i)=d1(i)*ds(i)
23012 continue
23013 continue
      do23014 i = 1,n2
      d2(i)=x2(i)/z2(i)
      dz2(i)=d2(i)*r2(i)
23014 continue
23015 continue
      call dgemv('N',p,n1,one,a1,p,dz1,1,one,dy,1)
      call dgemv('N',p,n2,one,a2,p,dz2,1,one,dy,1)
      call dcopy(p,dy,1,rhs,1)
      call stepy2(n1,n2,p,a1,d1,a2,d2,dy,ada,info)
      if(info .ne. 0)then
      return
      endif
      call dgemv('T',p,n1,one,a1,p,dy,1,mone,ds,1)
      deltap=big
      deltad=big
      do23018 i=1,n1
      dx1(i)=d1(i)*ds(i)
      ds(i)=-dx1(i)
      dz1(i)=-z1(i)*(dx1(i)/x1(i) + one)
      dw(i)=-w(i)*(ds(i)/s(i) + one)
      if(dx1(i).lt.0)then
      deltap=dmin1(deltap,-x1(i)/dx1(i))
      endif
      if(ds(i).lt.0)then
      deltap=dmin1(deltap,-s(i)/ds(i))
      endif
      if(dz1(i).lt.0)then
      deltad=dmin1(deltad,-z1(i)/dz1(i))
      endif
      if(dw(i).lt.0)then
      deltad=dmin1(deltad,-w(i)/dw(i))
      endif
23018 continue
23019 continue
      call dcopy(n2,r2,1,dx2,1)
      call dgemv('T',p,n2,one,a2,p,dy,1,mone,dx2,1)
      do23028 i=1,n2
      dx2(i)=d2(i)*dx2(i)
      dz2(i)=-z2(i)*(dx2(i)/x2(i) + one)
      if(dx2(i).lt.0)then
      deltap=dmin1(deltap,-x2(i)/dx2(i))
      endif
      if(dz2(i).lt.0)then
      deltad=dmin1(deltad,-z2(i)/dz2(i))
      endif
23028 continue
23029 continue
      deltap=dmin1(beta*deltap,one)
      deltad=dmin1(beta*deltad,one)
      if(min(deltap,deltad) .lt. one)then
      nit(2)=nit(2)+1
      mu = ddot(n1,x1,1,z1,1)+ddot(n2,x2,1,z2,1)+ddot(n1,s,1,w,1)
      g = mu + deltap*ddot(n1,dx1,1,z1,1)+ deltad*ddot(n1,dz1,1,x1,1)+ d
     *eltap*deltad*ddot(n1,dz1,1,dx1,1)+ deltap*ddot(n2,dx2,1,z2,1)+ del
     *tad*ddot(n2,dz2,1,x2,1)+ deltap*deltad*ddot(n2,dz2,1,dx2,1)+ delta
     *p*ddot(n1,ds,1,w,1)+ deltad*ddot(n1,dw,1,s,1) + deltap*deltad*ddot
     *(n1,ds,1,dw,1)
      mu = mu * ((g/mu)**3) /(dfloat(2*n1)+dfloat(n2))
      do23036 i=1,n1
      dsdw = ds(i)*dw(i)
      dr1(i)=d1(i)*(mu*(one/s(i)-one/x1(i))+ dx1(i)*dz1(i)/x1(i)-dsdw/s(
     *i))
23036 continue
23037 continue
      do23038 i=1,n2
      dr2(i)=d2(i)*(dx2(i)*dz2(i)/x2(i)-mu/x2(i))
23038 continue
23039 continue
      call dswap(p,rhs,1,dy,1)
      call dgemv('N',p,n1,one,a1,p,dr1,1,one,dy,1)
      call dgemv('N',p,n2,one,a2,p,dr2,1,one,dy,1)
      call dpotrs('U',p,1,ada,p,dy,p,info)
      call dgemv('T',p,n1,one,a1,p,dy,1,zero,u,1)
      deltap=big
      deltad=big
      do23040 i=1,n1
      dsdw = ds(i)*dw(i)
      dxdz1 = dx1(i)*dz1(i)
      dx1(i) = d1(i)*(u(i)-z1(i)+w(i))-dr1(i)
      ds(i) = -dx1(i)
      dz1(i) = -z1(i)+(mu - z1(i)*dx1(i) - dxdz1)/x1(i)
      dw(i) = -w(i)+(mu - w(i)*ds(i) - dsdw)/s(i)
      if(dx1(i).lt.0)then
      deltap=dmin1(deltap,-x1(i)/dx1(i))
      endif
      if(ds(i).lt.0)then
      deltap=dmin1(deltap,-s(i)/ds(i))
      endif
      if(dz1(i).lt.0)then
      deltad=dmin1(deltad,-z1(i)/dz1(i))
      endif
      if(dw(i).lt.0)then
      deltad=dmin1(deltad,-w(i)/dw(i))
      endif
23040 continue
23041 continue
      call dgemv('T',p,n2,one,a2,p,dy,1,zero,u,1)
      do23050 i=1,n2
      dxdz2 = dx2(i)*dz2(i)
      dx2(i) = d2(i)*(u(i)-r2(i))-dr2(i)
      dz2(i) = -z2(i)+(mu - z2(i)*dx2(i) - dxdz2)/x2(i)
      if(dx2(i).lt.0)then
      deltap=dmin1(deltap,-x2(i)/dx2(i))
      endif
      if(dz2(i).lt.0)then
      deltad=dmin1(deltad,-z2(i)/dz2(i))
      endif
23050 continue
23051 continue
      deltap=dmin1(beta*deltap,one)
      deltad=dmin1(beta*deltad,one)
      endif
      call daxpy(n1,deltap,dx1,1,x1,1)
      call daxpy(n2,deltap,dx2,1,x2,1)
      call daxpy(n1,deltap,ds,1,s,1)
      call daxpy(p,deltad,dy,1,y,1)
      call daxpy(n1,deltad,dz1,1,z1,1)
      call daxpy(n2,deltad,dz2,1,z2,1)
      call daxpy(n1,deltad,dw,1,w,1)
      gap = ddot(n1,z1,1,x1,1)+ddot(n2,z2,1,x2,1)+ddot(n1,w,1,s,1)
      goto 23010
      endif
23011 continue
      call daxpy(n1,mone,w,1,z1,1)
      call dswap(n1,z1,1,x1,1)
      return
      end
      subroutine stepy2(n1,n2,p,a1,d1,a2,d2,b,ada,info)
      integer n1,n2,p,i,j,k,info
      double precision a1(p,n1),a2(p,n2),b(p),d1(n1),d2(n2),ada(p,p),zer
     *o
      parameter(zero = 0.0d0)
      do23056 j=1,p
      do23058 k=1,p
      ada(j,k)=zero
23058 continue
23059 continue
23056 continue
23057 continue
      do23060 i=1,n1
      call dsyr('U',p,d1(i),a1(1,i),1,ada,p)
23060 continue
23061 continue
      do23062 i=1,n2
      call dsyr('U',p,d2(i),a2(1,i),1,ada,p)
23062 continue
23063 continue
      call dposv('U',p,1,ada,p,b,p,info)
      return
      end

      SUBROUTINE SRTPAI ( A, SA, P, SP, N )
C
C     SRTPAI SETS P(1) = 1, P(SP+1) = 2, ..., P((N-1)*SP+1) = N
C     AND THEN REARRANGES P(1), P(SP+1), ..., P((N-1)*SP+1) SO THAT
C     A( (P(I)-1)*SA+1 ) .LE. A( (P(J)-1)*SA+1 ) IF AND ONLY IF
C     I .LT. J, WHERE I AND J SUBSCRIPT PROPER ELEMENTS OF P
C
      INTEGER SP, P(SP, *), SA, H, PIH, PI
      INTEGER A(SA, *)
C
C     CHECK INPUT PARAMETERS AND INITIALIZE H
C
      CALL I1SRT( SA, SP, N )
      IF ( I0SRT( 1, N, H ) .LT. 1 ) RETURN
C
C     INITIALIZE P
C
      DO 10 I = 1, N
 10     P(1, I) = I
C
C       CHECK IF DONE WITH SORT
C
 20     IF ( H .LT. 1 ) RETURN
        K = N - H
C
C       COMPARE
C
        DO 40 J = 1, K
          I = J
 30         IH = I + H
            PI = P(1, I)
            PIH = P(1, IH)
            IF ( A(1, PI) .LE. A(1, PIH) ) GOTO 40
C
C           EXCHANGE
C
            P(1, I) = PIH
            P(1, IH) = PI
C
C           PERCOLATE EXCHANGED LIST ELEMENT UP TO PROPER PLACE
C
            I = I - H
            IF ( I .GE. 1 ) GOTO 30
 40       CONTINUE
C
        H = ( H - 1 ) / 3
        GOTO 20
C
      END    
      SUBROUTINE I1SRT ( SA, SP, N )
C
C     I1SRT CHECKS LEGALITY OF VALUES OF SA, SP, N
C
      INTEGER SA, SP
C
C/6S
C     IF ( N .LT. 0 )
C    1   CALL SETERR( 27HSRTXXX - ILLEGAL VALUE OF N, 27, 1, 2 )
C     IF ( SA .LE. 0 )
C    1   CALL SETERR( 28HSRTXXX - ILLEGAL VALUE OF SA, 28, 2, 2 )
C     IF ( SP .LE. 0 )
C    1   CALL SETERR( 28HSRTXXX - ILLEGAL VALUE OF SP, 28, 3, 2 )
C/7S
C     IF ( N .LT. 0 )
C    1   CALL SETERR( 'SRTXXX - ILLEGAL VALUE OF N', 27, 1, 2 )
C     IF ( SA .LE. 0 )
C    1   CALL SETERR( 'SRTXXX - ILLEGAL VALUE OF SA', 28, 2, 2 )
C     IF ( SP .LE. 0 )
C    1   CALL SETERR( 'SRTXXX - ILLEGAL VALUE OF SP', 28, 3, 2 )
C/
C
      RETURN
C
      END


      INTEGER FUNCTION I0SRT ( SA, N, H )
C
C     I0SRT CHECKS INPUT PARAMETERS N, SA AND CALCULATES H
C     RETURNS H = 0 IF NO SORTING NECESSARY, ELSE
C     RETURNS SPACING, H, FOR FIRST INSERTION SORT.
C     I0SRT RETURNS TOTAL NUMBER OF ELEMENTS IN ARRAY = N * SA
C
      INTEGER SA, H
C
C/6S
C     IF ( N .LT. 0 )
C    1   CALL SETERR( 27HSRTXXX - ILLEGAL VALUE OF N, 27, 1, 2 )
C     IF ( SA .LE. 0 )
C    1   CALL SETERR( 28HSRTXXX - ILLEGAL VALUE OF SA, 28, 2, 2 )
C/7S  
C     IF ( N .LT. 0 )
C    1   CALL SETERR( 'SRTXXX - ILLEGAL VALUE OF N', 27, 1, 2 )
C     IF ( SA .LE. 0 )
C    1   CALL SETERR( 'SRTXXX - ILLEGAL VALUE OF SA', 28, 2, 2 )
C/
C
C     CHECK IF SORTING IS NECESSARY
C
      H = 0
      I0SRT = N * SA
      IF ( N .LE. 1 ) RETURN
C
C     CALCULATE H USING H NEW = 3 * H OLD + SA
C
      H = 4 * SA
C
 10     H = 3 * H + SA
        IF ( H .LT. I0SRT ) GOTO 10
C
      H = ( H - 4 * SA ) / 9
C
      RETURN
C
      END

subroutine rqfnc(n1,n2,p,a1,y,a2,r,rhs,d1,d2,u,beta,eps,wn1,wn2,wp,nit,info)
integer n1,n2,p,info,nit(3)
double precision a1(p,n1),a2(p,n2),y(n1),r(n2),rhs(p),d1(n1),d2(n2),u(n1)
double precision wn1(n1,9),wn2(n2,6),wp(p,p+3)
double precision one,beta,eps
parameter(one = 1.0d0)
call lpfnc(n1,n2,p,a1,y,a2,r,rhs,d1,d2,u,beta,eps,wn1(1,1),wn2(1,1),wn1(1,2),
	wp(1,1),wn1(1,3),wn2(1,2),wn1(1,4),wn1(1,5),wn2(1,3),wn1(1,6),
        wp(1,2),wn1(1,7),wn2(1,4),wn1(1,8),wn1(1,9),wn2(1,5),wn2(1,6),
	wp(1,3),wp(1,4),nit,info)
return
end
# This is a revised form of the primal-dual log barrier form of the
# interior point LP solver based on Lustig, Marsten and Shanno ORSA J Opt 1992.
# It generalizes rqfnb and lpfnb to accomodate inequality constraints 
# It is a projected Newton primal-dual logarithmic barrier method which uses
# the predictor-corrector approach of Mehrotra for the mu steps.
# For the sake of brevity we will call it a Frisch-Newton algorithm.
# Problem:
# 	min c1'x1 + c2'x2  s.t. A1x1 + A2x2 = b, 0<=x1<=u, 0<=x2
# 
# Denote dx1,dx2,dy,dw,ds,dz1,dz2 as the steps for x1,x2,y,w,s,z1,z2
# 
subroutine lpfnc(n1,n2,p,a1,c1,a2,c2,b,d1,d2,u,beta,eps,x1,x2,s,
	y,z1,z2,w,dx1,dx2,ds,dy,dz1,dz2,dw,dr1,dr2,r2, rhs,ada,nit,info)

integer n1,p,i,info,nit(3),maxit
double precision a1(p,n1),a2(p,n2),c1(n1),c2(n2),b(p)
double precision zero,one,mone,big,ddot,dmax1,dmin1,dxdz1,dxdz2,dsdw
double precision deltap,deltad,beta,eps,mu,gap,g
double precision x1(n1),x2(n2),u(n1),s(n1),y(p),z1(n1),z2(n2),w(n1)
double precision d1(n1),d2(n2),rhs(p),ada(p,p)
double precision dx1(n1),dx2(n2),ds(n1),dy(p),dz1(n1),dz2(n2),dw(n1)
double precision dr1(n1),dr2(n2),r2(n2)

parameter(zero  = 0.0d0)
parameter(one   = 1.0d0)
parameter(mone   = -1.0d0)
parameter(big  = 1.0d+20)
parameter(maxit  = 500)

# Initialization:  We try to follow the notation of LMS
# On input we require:
# 
# 	c1 = n1-vector of marginal costs (-y in the rq problem)
# 	a1 = p by n1 matrix of equality constraints (x' in rq)
# 	c2 = n2-vector of marginal costs (-r in the rq problem)
# 	a2 = p by n2 matrix of inequality constraints (R' in rq)
# 	b = p-vector of rhs ((1-tau)x'e in rq)
#       u = upper bound vector ( 1_p in rq)
# 	beta = barrier parameter, LMS recommend .99995
# 	eps = convergence tolerance, LMS recommend 10d-8
# 	
# 	the integer vector nit returns iteration counts
# 	the integer info contains an error code from the Cholesky in stepy
# 		info = 0 is fine
# 		info < 0 invalid argument to dposv 
# 		info > 0 singular matrix
nit(1)=0
nit(2)=0
nit(3)=n1
# Start at the OLS estimate for the dual vector y
call dgemv('N',p,n1,one,a1,p,c1,1,zero,y,1) 
do i=1,n1
	d1(i)=one  
do i=1,n2{
	d2(i)=zero
	z2(i)=one
	}
call stepy2(n1,n2,p,a1,d1,a2,d2,y,ada,info)
if(info != 0) return
# put current residual vector r1 in s (temporarily)
call dcopy(n1,c1,1,s,1)
call dgemv('T',p,n1,mone,a1,p,y,1,one,s,1)
# Initialize remaining variables
do i=1,n1{
	if(dabs(s(i)) < eps){
		z1(i)=dmax1(s(i),zero)+eps
		w(i)=dmax1(-s(i),zero)+eps
		}
	else {
		z1(i)=dmax1(s(i),zero)
		w(i)=dmax1(-s(i),zero)
		}
	s(i)=u(i)-x1(i)
	}
gap = ddot(n1,z1,1,x1,1)+ddot(n2,z2,1,x2,1)+ddot(n1,w,1,s,1)
# 
# Main Loop
#
while(gap > eps && nit(1)<maxit) {
	nit(1)=nit(1)+1
	call dcopy(n2,c2,1,r2,1)
	call dgemv('T',p,n2,mone,a2,p,y,1,one,r2,1) 
	call dcopy(p,b,1,dy,1)
	call dgemv('N',p,n1,mone,a1,p,x1,1,one,dy,1) 
	call dgemv('N',p,n2,mone,a2,p,x2,1,one,dy,1) 
	do i = 1,n1{
		d1(i)=one/(z1(i)/x1(i) + w(i)/s(i)) 
		ds(i)=z1(i)-w(i) 
		dz1(i)=d1(i)*ds(i) 
		}
	do i = 1,n2{
		d2(i)=x2(i)/z2(i) 
		dz2(i)=d2(i)*r2(i) 
		}
	call dgemv('N',p,n1,one,a1,p,dz1,1,one,dy,1) 
	call dgemv('N',p,n2,one,a2,p,dz2,1,one,dy,1) 
	call dcopy(p,dy,1,rhs,1) #save rhs
	call stepy2(n1,n2,p,a1,d1,a2,d2,dy,ada,info)
	if(info != 0) return
	call dgemv('T',p,n1,one,a1,p,dy,1,mone,ds,1) 
	deltap=big
	deltad=big
	do i=1,n1{
		dx1(i)=d1(i)*ds(i)
		ds(i)=-dx1(i)
		dz1(i)=-z1(i)*(dx1(i)/x1(i) + one)
		dw(i)=-w(i)*(ds(i)/s(i) + one)
		if(dx1(i)<0)deltap=dmin1(deltap,-x1(i)/dx1(i))
		if(ds(i)<0)deltap=dmin1(deltap,-s(i)/ds(i))
		if(dz1(i)<0)deltad=dmin1(deltad,-z1(i)/dz1(i))
		if(dw(i)<0)deltad=dmin1(deltad,-w(i)/dw(i))
		}
	call dcopy(n2,r2,1,dx2,1)
	call dgemv('T',p,n2,one,a2,p,dy,1,mone,dx2,1) #dx2 = A2'dy - r2
	do i=1,n2{
		dx2(i)=d2(i)*dx2(i)
		dz2(i)=-z2(i)*(dx2(i)/x2(i) + one)
		if(dx2(i)<0)deltap=dmin1(deltap,-x2(i)/dx2(i))
		if(dz2(i)<0)deltad=dmin1(deltad,-z2(i)/dz2(i))
		}
	deltap=dmin1(beta*deltap,one)
	deltad=dmin1(beta*deltad,one)
	if(min(deltap,deltad) < one){
		nit(2)=nit(2)+1
		# Update mu
		mu = ddot(n1,x1,1,z1,1)+ddot(n2,x2,1,z2,1)+ddot(n1,s,1,w,1)
		g = mu + deltap*ddot(n1,dx1,1,z1,1)+
			deltad*ddot(n1,dz1,1,x1,1)+
			deltap*deltad*ddot(n1,dz1,1,dx1,1)+
			deltap*ddot(n2,dx2,1,z2,1)+
			deltad*ddot(n2,dz2,1,x2,1)+
			deltap*deltad*ddot(n2,dz2,1,dx2,1)+
			deltap*ddot(n1,ds,1,w,1)+
			deltad*ddot(n1,dw,1,s,1) +
			deltap*deltad*ddot(n1,ds,1,dw,1)
		mu = mu * ((g/mu)**3) /(dfloat(2*n1)+dfloat(n2))
		# Compute modified step
		do i=1,n1{
			dsdw =  ds(i)*dw(i)
			dr1(i)=d1(i)*(mu*(one/s(i)-one/x1(i))+
				dx1(i)*dz1(i)/x1(i)-dsdw/s(i))
			} 
		do i=1,n2{
			dr2(i)=d2(i)*(dx2(i)*dz2(i)/x2(i)-mu/x2(i))
			} 
		call dswap(p,rhs,1,dy,1)
		call dgemv('N',p,n1,one,a1,p,dr1,1,one,dy,1)
		call dgemv('N',p,n2,one,a2,p,dr2,1,one,dy,1)
		call dpotrs('U',p,1,ada,p,dy,p,info)
		call dgemv('T',p,n1,one,a1,p,dy,1,zero,u,1)
		deltap=big
		deltad=big
		do i=1,n1{
			dsdw =  ds(i)*dw(i)
			dxdz1 =  dx1(i)*dz1(i)
			dx1(i) =  d1(i)*(u(i)-z1(i)+w(i))-dr1(i)
			ds(i) = -dx1(i)
			dz1(i) = -z1(i)+(mu - z1(i)*dx1(i) - dxdz1)/x1(i)
			dw(i) = -w(i)+(mu - w(i)*ds(i) - dsdw)/s(i)
			if(dx1(i)<0)deltap=dmin1(deltap,-x1(i)/dx1(i))
			if(ds(i)<0)deltap=dmin1(deltap,-s(i)/ds(i))
			if(dz1(i)<0)deltad=dmin1(deltad,-z1(i)/dz1(i))
			if(dw(i)<0)deltad=dmin1(deltad,-w(i)/dw(i))
			}
		call dgemv('T',p,n2,one,a2,p,dy,1,zero,u,1)#u=A2'dy
		do i=1,n2{
			dxdz2 =  dx2(i)*dz2(i)
			dx2(i) =  d2(i)*(u(i)-r2(i))-dr2(i)
			dz2(i) = -z2(i)+(mu - z2(i)*dx2(i) - dxdz2)/x2(i)
			if(dx2(i)<0)deltap=dmin1(deltap,-x2(i)/dx2(i))
			if(dz2(i)<0)deltad=dmin1(deltad,-z2(i)/dz2(i))
			}
		deltap=dmin1(beta*deltap,one)
		deltad=dmin1(beta*deltad,one)
		}
	call daxpy(n1,deltap,dx1,1,x1,1)
	call daxpy(n2,deltap,dx2,1,x2,1)
	call daxpy(n1,deltap,ds,1,s,1)
	call daxpy(p,deltad,dy,1,y,1)
	call daxpy(n1,deltad,dz1,1,z1,1)
	call daxpy(n2,deltad,dz2,1,z2,1)
	call daxpy(n1,deltad,dw,1,w,1)
	gap = ddot(n1,z1,1,x1,1)+ddot(n2,z2,1,x2,1)+ddot(n1,w,1,s,1)
	}
# return residuals in the vector x1
call daxpy(n1,mone,w,1,z1,1)
call dswap(n1,z1,1,x1,1)
return
end
subroutine stepy2(n1,n2,p,a1,d1,a2,d2,b,ada,info)
integer n1,n2,p,i,j,k,info
double precision a1(p,n1),a2(p,n2),b(p),d1(n1),d2(n2),ada(p,p),zero
parameter(zero = 0.0d0)
# Solve the linear system ada'x=b by Choleski -- d is diagonal
# Note that a isn't altered, and on output ada returns the upper
# triangle Choleski factor, which can be reused, eg with blas dtrtrs
do j=1,p
	do k=1,p
		ada(j,k)=zero
do i=1,n1
	call dsyr('U',p,d1(i),a1(1,i),1,ada,p)
do i=1,n2
	call dsyr('U',p,d2(i),a2(1,i),1,ada,p)
call dposv('U',p,1,ada,p,b,p,info)
return
end

"kuantile"<-
function(x, probs = seq(0,1, 0.25),na.rm = FALSE, 
	names = TRUE, type = 7, ...) {
    if(na.rm)
	x <- x[!is.na(x)]
    else if(any(is.na(x)))
	stop("NA's and NaN's not allowed in 'x' if 'na.rm' is FALSE")
    if(any(is.na(probs)))
	stop("NA's and NaN's in 'probs' not allowed")
    if(any(probs < 0 | probs > 1))
        stop("probs outside [0,1]")
    p <- probs
    op <- order(p)
    p <- p[op]
    n <- length(x)
    m <- length(p)
    g <- rep(.5,m)
    if(type == 1) #Hyndman-Fan Typology
	k <- j <- pmax(1,ceiling(p*n))
    else if(type == 2){
	j <- pmax(1,floor(p*n))
	k <- sort(c(pmax(1,j),pmin(j+1,n)))
	g <- ifelse(p*n > j, 1, 0.5)
	}
    else if(type == 3)
	k <- j <- pmax(1,round(p*n))
    else{
	switch(type - 3, 
		{a <- 0; b <- 1},#Type 4
		{a <- b <- 0.5}, #Type 5
		{a <- b <- 0},   #Type 6
		{a <- b <- 1},   #Type 7
		{a <- b <- 1/3}, #Type 8
		{a <- b <- 3/8}) #Type 9
	d <- a + p * (1 - a - b)
        j <- floor(p*n + d)
	g <- p*n + d - j
    	k <- sort(c(pmax(1,j),pmin(j+1,n)))
	}
    uk <- kunique(k)
    uz <- kselect(x,uk$xU)
    z <- uz[uk$ix]
    if(type %in% c(1,3))
	A <- matrix(z,m,2)
    else
	A <- t(matrix(z,2,m)) 
    G <- cbind(1-g,g)
    y <- (A * G) %*% c(1,1) # <=> diag(crossprod(A,G))
    y <- y[rank(probs)]
    if(names && m > 0){
	dig <- max(2,getOption("digits"))
	names(y) <- paste(format(100*probs, trim = TRUE, digits = dig), "%", sep="")
	}
    class(y) <- "kuantile"
    return(y)
}
"kselect" <-
function(x,k){
    n <- length(x)
    m <- length(k)
    z <- .Fortran("kuantiles",
        	k = as.integer(k),
        	m = as.integer(m),
        	n = as.integer(n),
        	x = as.double(x))
    return(z$x[z$k])
    }
"kunique" <-
function (x, isuniq = !duplicated(x))
{ # shamelessly plagarized from Martin Maechler's sfsmisc package
    need.sort <- is.unsorted(x)
    if (need.sort) {
        xs <- sort(x, index.return = TRUE)
        ixS <- xs$ix
        isuniq <- isuniq[ixS]
        x <- xs$x
    }
    ix <- as.integer(cumsum(isuniq))
    if (need.sort)
        ix <- ix[sort.list(ixS)]
    list(ix = ix, xU = x[isuniq])
}
q489 <- function(x, tau = .5){
    n <- length(x)
    z <- .Fortran("qselect", 
		  as.integer(n), 
		  as.double(x),
		  q = as.double(tau))
    z$q
}

"rqss.fit" <-
function (x, y, tau = 0.5, method = "sfn", rhs = NULL, control, ...)
{
    if(is.null(rhs)) rhs <- (1 - tau) * t(x) %*% rep(1,length(y))
    else tau <- 0.5
    fit <- switch(method,
	sfn = rq.fit.sfn(x, y, tau = tau,  rhs = rhs, control = control, ...),
	lasso = rq.fit.sfn(x, y, tau = tau,  rhs = rhs, control = control, ...),
        sfnc = rq.fit.sfnc(x, y, tau = tau,  rhs = rhs, control = control, ...), {
            what <- paste("rq.fit.", method, sep = "")
            if (exists(what, mode = "function"))
                (get(what, mode = "function"))(x, y, ...)
            else stop(paste("unimplemented method:", method))
        })
    fit$contrasts <- attr(x, "contrasts")
    fit$resid <- c(y - x %*% fit$coef)

    fit
}
"untangle.specials" <-
function (tt, special, order = 1)
{
    spc <- attr(tt, "specials")[[special]]
    if (length(spc) == 0)
        return(list(vars = character(0), terms = numeric(0)))
    facs <- attr(tt, "factor")
    fname <- dimnames(facs)
    ff <- apply(facs[spc, , drop = FALSE], 2, sum)
    list(vars = (fname[[1]])[spc], terms = seq(ff)[ff & match(attr(tt,
        "order"), order, nomatch = 0)])
}

"qss" <-
function (x, constraint = "N", lambda = 1, ndum = 0, dummies = NULL, 
    Dorder = 1, w = rep(1, length(x))) 
{
    if (is.matrix(x)) {
        if (ncol(x) == 2) 
            qss <- qss2(x, constraint = constraint, dummies = dummies, 
                lambda = lambda, ndum = ndum, w = w)
        else if (ncol(x) == 1) 
            x <- as.vector(x)
        else stop("qss objects must have dimension 1 or 2")
    }
    if (is.vector(x)) {
	if(Dorder == 1)
	    qss <- qss1(x, constraint = constraint, lambda = lambda, 
		dummies = dummies, ndum = ndum, w = w)
	else if(Dorder == 0){
	    if(!(constraint %in% c("N","I","D")))
		stop("This constraint not implemented for Dorder = 0")
	    else
		qss <- qts1(x, constraint = constraint, lambda = lambda, 
		    dummies = dummies, ndum = ndum, w = w)
	}
	else
	    stop("Dorder must be either 0 or 1")
    }
    qss
}

"qss2" <-
function(x, y, constraint = "N", lambda = 1, ndum= 0, dummies = NULL, w=rep(1,length(x))){
#
# Sparse Additive Quantile Smoothing Spline Models - Bivariate (Triogram) Module
#
# This function returns a structure intended to make model.matrix for a bivariate
# nonparametric component of a model formula specified by a call to rqss().  A sparse form
# of the Frisch Newton algorithm is eventually called to compute the estimator.
# An optional convexity/concavity constraint can be specified.  If
# the formula consists of a single qss component then the estimator solves the
# following variational problem:
#
#       min sum rho_tau (y_i - g(x_i)) + lambda V(grad(g))
#
# where V(f) denotes the total variation of the function f.  The solution is a piecewise
# linear function on the Delaunay triangulation formed by the observed (x_i,y_i) points.
# Additive models can consist
# of several components of this form plus partial linear and univariate qss components.
# To resolve the identifiability problem we delete the first column of the qss design
# components.  On return F contains the fidelity portion of the design, A the penalty
# contribution of the design, R the constraint portion, and r the rhs of the constraints.
#
# Constraints are specified by the constraint argument:
#
#	N	none
#	V	convex
#	C	concave
#
# Author:  Roger Koenker   April 2, 2003
#
# For a prototype see triogram in ~roger/projects/tv/cobar/.RData on ysidro.
#
# Warning:   Under development...todo:
#
#       o  weights
#       o  dummy x's
#       o  tau's
#       o  lambda's
#       o  ...
#
   stopifnot(requireNamespace("tripack"))
#
    y <- x[,2]
    x <- x[,1]
    n <- length(x)
    if (n != length(y))
        stop("xy lengths do not match")
    f <- triogram.fidelity(x, y, ndum = ndum, dummies = dummies)
    F <- f$F
    A <- triogram.penalty(f$x, f$y)
    switch(constraint, V = {
        R <- A
        r <- rep(0, nrow(R))
    }, C = {
        R <- -A
        r <- rep(0, nrow(R))
    }, N = {
        R = NULL
        r = NULL
    })
    list(x = list(x = f$x, y = f$y), F = F[, -1], dummies = f$dummies,
        lambda = lambda, A = A[, -1], R = R[, -1], r = r)
}

qts1 <- function (x, constraint = "N", lambda = 1, dummies = dummies, 
    ndum = 0, w = rep(1, length(x))) 
{ # This is the taut string TV(g) not TV(g') penalty option
    xun <- unique(x[order(x)])
    h <- diff(xun)
    nh <- length(h)
    nx <- length(x)
    p <- nh + 1
    makeD <- function(p) {
        new("matrix.csr", ra = c(rbind(rep(-1, (p - 1)), rep(1,
            (p - 1)))), ja = as.integer(c(rbind(1:(p - 1), 2:p))),
            ia = as.integer(2 * (1:p) - 1), dimension = as.integer(c(p -
                1, p)))
    }
    A <- new("matrix.csr", ra = c(rbind(-1/h, 1/h)), ja = as.integer(c(rbind(1:nh, 
        2:(nh + 1)))), ia = as.integer(2 * (1:(nh + 1)) - 1), 
        dimension = as.integer(c(nh, nh + 1)))
    if (length(xun) == length(x)) {
        F <- new("matrix.csr", ra = rep(1, nx), ja = as.integer(rank(x)), 
            ia = 1:(nx + 1), dimension = as.integer(c(nx, nx)))
    }
    else {
        F <- new("matrix.csr", ra = rep(1, nx), ja = as.integer(factor(x)), 
            ia = 1:(nx + 1), dimension = as.integer(c(nx, length(xun))))
    }
    switch(constraint, 
	   I = {R <- makeD(p); r <- rep(0, p-1)},
	   D = {R <- -makeD(p); r <- rep(0, p-1)},
	   N = {R <- NULL; r <- NULL})
    list(x = list(x = xun), F = F[, -1], lambda = lambda, A = A[,
        -1], Dorder = 0, R = R[,-1], r = r)
}

"qss1" <-
function (x, constraint = "N", lambda = 1, dummies = dummies,
        ndum = 0, w = rep(1, length(x))){
#
# Sparse Additive Quantile Smoothing Spline Models - Univariate Module
#
# This function returns a structure intended to make model.matrix for a univariate
# nonparametric component of a model formula specified by a call to rqss().  A sparse form
# of the Frisch Newton algorithm is eventually called to compute the estimator.
# Optional monotonicity and/or convexity/concavity constraints can be specified.  If
# the formula consists of a single qss component then the estimator solves the
# following variational problem:
#
#       min sum rho_tau (y_i - g(x_i)) + lambda V(g')
#
# where V(f) denotes the total variation of the function f.  The solution is a piecewise
# linear function with "knots" at the observed x_i points.  Additive models can consist
# of several components of this form plus partial linear and triogram components.
# To resolve the identifiability problem we delete the first column of the qss design
# components.  On return F contains the fidelity portion of the design, A is the penalty
# contribution of the design, R the constraint portion, and r the rhs of the constraints.
#
# Constraints are specified by the constraint argument:
#
#	N	none
#	I	monotone increasing
#	D	monotone decreasing
#	V	convex
#	C	concave
#	CI	concave and monotone increasing
#	...	etc
#
# Author:  Roger Koenker   February 27, 2003
#
# Warning:   Under development...todo:
#
#       o  weights
#       o  dummy x's
#       o  tau's
#       o  lambda's
#       o  ...
#
#
    xun <- unique(x[order(x)])
    h <- diff(xun)
    nh <- length(h)
    nx <- length(x)
    p <- nh + 1
    B <- new("matrix.csr", ra = c(rbind(-1/h, 1/h)), ja = as.integer(c(rbind(1:nh,
        2:(nh + 1)))), ia = as.integer(2 * (1:(nh + 1)) - 1),
        dimension = as.integer(c(nh, nh + 1)))
    makeD <- function(p) {
        new("matrix.csr", ra = c(rbind(rep(-1, (p - 1)), rep(1,
            (p - 1)))), ja = as.integer(c(rbind(1:(p - 1), 2:p))),
            ia = as.integer(2 * (1:p) - 1), dimension = as.integer(c(p -
                1, p)))
    }
    D <- makeD(nh)
    A <- D %*% B
    if (length(xun) == length(x)){
        F <- new("matrix.csr", ra = rep(1, nx), ja = as.integer(rank(x)),
            ia = 1:(nx + 1), dimension = as.integer(c(nx, nx)))
	}
    else {
        F <- new("matrix.csr", ra = rep(1, nx), ja = as.integer(factor(x)),
            ia = 1:(nx + 1), dimension = as.integer(c(nx, length(xun))))
	}
   switch(constraint,
        V = {   R <- A;
                r <- rep(0,nrow(R))
                },
        C = {   R <- -A;
                r <- rep(0,nrow(R))
                },
        I = {   R <- makeD(p)
                r <- rep(0,p-1)
                },
        D = {   R <- -makeD(p)
                r <- rep(0,p-1)
                },
        VI = {  R <- makeD(p)
		R <- rbind(R,A)
                r <- rep(0,nrow(R))
		},
        VD = {  R <- -makeD(p)
		R <- rbind(R,A)
                r <- rep(0,nrow(R))
		},
        CI = {  R <- makeD(p)
		R <- rbind(R,-A)
                r <- rep(0,nrow(R))
		},
        CD = {  R <- -makeD(p)
		R <- rbind(R,-A)
                r <- rep(0,nrow(R))
		},
	N = { R=NULL; r=NULL}
	)
   list(x = list(x=xun), F=F[,-1], lambda = lambda, A=A[,-1], 
	Dorder = 1, R=R[,-1], r=r)
}
"plot.qss1" <-
function(x, rug = TRUE, jit = TRUE, add = FALSE, ...)
{
    if(!add) plot(x,type = "n", ...)
	lines(x, ...)
    if(rug) {
	if(jit)
	    rug(jitter(x[,1]))
	else rug(x[,1])
    }
}
"plot.qts1" <-
function (x, rug = TRUE, jit = TRUE, add = FALSE, ...) 
{
    if (!add) 
        plot(x, type = "S", ...)
    lines(x, type = "S", ...)
    if (rug) {
        if (jit) 
            rug(jitter(x[, 1]))
        else rug(x[, 1])
    }
}
"plot.qss2" <-
function (x, render = "contour", ncol = 100, zcol = NULL, ...)
{
    stopifnot(requireNamespace("tripack"))
    y <- x[, 2]
    z <- x[, 3]
    x <- x[, 1]
    tri <- tripack::tri.mesh(x, y)
    if (render == "rgl") {
        if(!requireNamespace("rgl",quietly=TRUE))
		stop("The package rgl is required")
        collut <- terrain.colors(ncol)
        if (!length(zcol))
            zcol <- z
        if (max(z) > max(zcol) || min(z) < min(zcol))
            warning("fitted z values out of range of zcol vector")
        zlim <- range(zcol)
        colz <- ncol * (z - zlim[1])/(zlim[2] - zlim[1]) + 1
        colz <- collut[colz]
        s <- c(t(tripack::triangles(tri)[, 1:3]))
        rgl::rgl.triangles(x[s], y[s], z[s], col = colz[s])
    }
    else {
        stopifnot(requireNamespace("akima"))
        if(render == "contour"){
                plot(x, y, type = "n", ...)
                contour(akima::interp(x, y, z), add = TRUE, frame.plot = TRUE, ...)
                tripack::convex.hull(tri, plot.it = TRUE, add = TRUE)
                }
        else if(render == "persp")
                persp(akima::interp(x, y, z, ), theta = -40, phi = 20, xlab = "x",
                        ylab = "y", zlab = "z", ...)
        else stop(paste("Unable to render: ",render))
    }
}
plot.rqss <-
function (x, rug = TRUE, jit = TRUE, bands = NULL, coverage = 0.95, add = FALSE, 
	  shade = TRUE, select = NULL, pages = 0, titles = NULL, bcol = NULL, ...) 
{
    SetLayout <- function(m, p) {
            # Shamelessly cribbed from mgcv
            # m is the number of plots
            # p is the number of pages
        if (p > m) 
            p <- m
        if (p < 0) 
            p <- 0
        if (p != 0) {
            q <- m%/%p
            if ((m%%p) != 0) {
                q <- q + 1
                while (q * (p - 1) >= m) p <- p - 1
            }
            c <- trunc(sqrt(q))
            if (c < 1) 
                c <- 1
            r <- q%/%c
            if (r < 1) 
                r <- 1
            while (r * c < q) r <- r + 1
            while (r * c - q > c && r > 1) r <- r - 1
            while (r * c - q > r && c > 1) c <- c - 1
            oldpar <- par(mfrow = c(r, c))
        }
        else oldpar <- par()
        return(oldpar)
    }
    m <- length(x$qss)
    if (m == 0) 
        stop("No qss object to plot")
    if(length(select)) {
        if(all(select %in% 1:m))
            oldpar <- SetLayout(length(select), pages)
        else
            stop(paste("select must be in 1:",m,sep="")) 
        }
    else
        oldpar <- SetLayout(m, pages)
    if ((pages == 0 && prod(par("mfrow")) < m && dev.interactive()) || 
        pages > 1 && dev.interactive()) 
        ask <- TRUE
    else ask <- FALSE
    if (ask) {
        oask <- devAskNewPage(TRUE)
        on.exit(devAskNewPage(oask))
    }
    qssnames <- names(x$qss)
    if(length(titles)){
        if(length(titles) != length(qssnames))
            stop("Length of titles doesn't match length of qssnames")
	}
    else{
        titles <- paste("Effect of ", qssnames)
	}
    if (length(bands)) {
        rdf <- x$n - x$edf
        if (any(unlist(lapply(x$qss, function(x) ncol(x$xyz) == 3)))) 
            warning("Can't plot confidence bands in 3D (yet)")
        band <- as.list(rep(NA, m))
        V <- summary(x, cov = TRUE, ...)$Vqss
        "summary.qss1" <- function(object, V, ngrid = 400, ...) {
            x <- object$xyz[, 1]
            eps <- 0.01
            newd <- data.frame(x = seq(min(x) + eps, max(x) - 
                eps, length = ngrid))
            G <- predict(object, newd, ...)
            ones <- as.matrix.csr(matrix(1, nrow(G$D), 1))
            D <- cbind(ones, G$D)
            S <- as.matrix(D %*% V %*% t(D))
            se <- sqrt(diag(S))
            cv <- qt(1 - (1 - coverage)/2, rdf)
            if (bands %in% c("uniform","both")) {
                E <- eigen(as.matrix(V))
                B <- E$vectors %*% diag(sqrt(pmax(0,E$values))) %*% t(E$vectors)
                D <- as.matrix(D)
                BX1 <- B %*% t(D[-1, ])
                BX1 <- BX1/sqrt(apply(BX1^2, 2, sum))
                BX0 <- B %*% t(D[-nrow(D), ])
                BX0 <- BX0/sqrt(apply(BX0^2, 2, sum))
                kappa <- sum(sqrt(apply((BX1 - BX0)^2, 2, sum)))
                cvu <- critval(kappa, alpha = 1 - coverage, rdf = rdf)
                if(bands == "both") cv <- c(cvu,cv)
                else cv <- cvu
                }
            list(pred = data.frame(x = G$x, y = G$y, se = se), 
                cv = cv)
        }
    }
    if(!length(select)) 
        select <- 1:m
    for (i in select) {
	if(!is.null(x$qss[[i]]$Dorder)){ 
	    if(x$qss[[i]]$Dorder == 0){
		qts <- x$qss[[i]]$xyz
		qts[, 2] <- x$coef[1] + qts[, 2]
		plot.qts1(qts, add = add, ...)
		}
	    else { 
		qss <- x$qss[[i]]$xyz
		if (length(bands)) {
		    if (is.na(x$coef["(Intercept)"])) 
			stop("rqss confidence bands require an intercept parameter")
		    B <- summary(x$qss[[i]], V[[i]], ...)
		    cv <- B$cv
		    B <- B$pred
		    B$y <- B$y + x$coef["(Intercept)"]
		    if(!length(bcol)) bcol <- c("grey85","grey65")
		    for(k in 1:length(cv)){
			if (add || k > 1) 
			    if(shade){
				polygon(c(B$x,rev(B$x)),
				c(B$y - cv[k] * B$se,rev(B$y + cv[k] * B$se)),
				col = bcol[k], border = FALSE)
			    }
			    else
				matlines(B$x, cbind(B$y, B$y + cv[k] * cbind(-B$se, 
				B$se)), lty = c(1, 2, 2), col = c("black", "blue", "blue"))
			else {
			    matplot(B$x, B$y + cv[k] * cbind(-B$se, B$se), 
				xlab = paste(qssnames[i]), ylab = "Effect", type = "n", ...)
			    if(shade){
				polygon(c(B$x,rev(B$x)),
				    c(B$y - cv[k] * B$se,rev(B$y + cv[k] * B$se)),
				    col = bcol[k], border = FALSE)
			    } 
			    else{
				lines(B$x, B$y + cv * B$se, lty = 2, ...)
				lines(B$x, B$y - cv * B$se, lty = 2, ...)
			    }
			}
			lines(B$x, B$y, ...)
		    }
		    band[[i]] <- list(x = B$x, blo = B$y - B$se %o% cv, bhi = B$y + B$se %o% cv)
		    if (rug) {
			if (jit) 
			    rug(jitter(qss[, 1]))
			else rug(qss[, 1])
                    }
		}
		else {
		    qss[, 2] <- x$coef[1] + qss[, 2]
		    plot.qss1(qss, xlab = paste(qssnames[i]), ylab = "Effect", 
		    rug = rug, jit = jit, add = add, ...)
		}
	    }
        }
	qss <- x$qss[[i]]$xyz
        if (ncol(qss) == 3) {
             qss[, 3] <- x$coef[1] + qss[, 3]
             plot.qss2(qss, ...)
        }
	title(titles[i])
    }
    if (pages > 0) 
        par(oldpar)
    if (length(bands)) 
        class(band) <- "rqssband"
    else
        band <- NULL
    invisible(band)
}

"triogram.fidelity" <- function (x, y, ndum=0, dummies = NULL)
{
#Make fidelity block of the triogram design in sparse matrix.csr form
#The rather esoteric match call identifies and handles duplicated xy points
n <- length(x)
A <- as.data.frame(cbind(x,y))
dupA <- duplicated(A)
if(any(dupA)){
  x <- x[!dupA]
  y <- y[!dupA]
  J <- match(do.call("paste",c(A,"\r")),do.call("paste",c(A[!dupA,],"\r")))
  z <- new("matrix.csr",ra=rep(1,n), ja=J, ia=1:(n+1),dimension=as.integer(c(n,max(J))))
  }
else{
  z <- as(n,"matrix.diag.csr")
  z <- as(z,"matrix.csr")
 }
#Augment with dummy vertices, if any...
if(length(dummies)){
	if (is.list(dummies)){
        	if (all(!is.na(match(c("x", "y"), names(dummies))))){
        		ndum <- length(dummies$x)
			if(length(dummies$y) == ndum){
        			x <- c(x,dummies$x)
        			y <- c(y,dummies$y)
        			zdum <- as.matrix.csr(0,n,ndum)
        			z <- cbind(z,zdum)
				}
        		else stop("dummies x and y components differ in length")
			}
        	else stop("dummies list lacking x and y elements")
		}
    	else stop("dummies argument invalid (not a list) in triogram.fidelity")
	}
else if(ndum > 0){
        u <- runif(ndum); v <- runif(ndum)
        xd <- min(x) + u * (max(x)-min(x))
        yd <- min(y) + v * (max(y)-min(y))
        T <- tripack::tri.mesh(x,y)
        s <- tripack::in.convex.hull(T,xd,yd)
        x <- c(x,xd[s])
        y <- c(y,yd[s])
        ndum <- sum(s)
        zdum <- as.matrix.csr(0,n,ndum)
        z <- cbind(z,zdum)
	dummies <- list(x = xd[s],y = yd[s])
        }
list(x=x,y=y,F=z, dummies = dummies)
}
"triogram.penalty" <- function (x, y, eps = .Machine$double.eps)
{
    n <- length(x)
    tri <- tripack::tri.mesh(x, y)
    bnd <- tripack::on.convex.hull(tri,x,y)
    q <- length(tri$tlist)
    m <- 13 * n
    z <- .Fortran("penalty", as.integer(n), as.integer(m), as.integer(q),
        as.double(x), as.double(y), as.integer(bnd),as.integer(tri$tlist),
        as.integer(tri$tlptr), as.integer(tri$tlend), rax = double(m),
	jax = integer(m), ned = integer(1), as.double(eps), 
	ierr = integer(1))[c("rax", "jax", "iax", "ned", "ierr")]
    if (z$ierr == 1)
        stop("collinearity in ggap")
    nnz <- 4 * z$ned
    ra <- z$rax[1:nnz]
    ja <- z$jax[1:nnz]
    ia <- as.integer(1 + 4 * (0:z$ned))
    dim <- as.integer(c(z$ned, n))
    new("matrix.csr",ra=ra,ja=ja,ia=ia,dimension=dim)
}

predict.rqss <-
function (object, newdata, interval = "none",  level = 0.95, ...) 
{
    ff <- object$fake.formula
    Terms <- delete.response(terms(object$formula, "qss"))
    Names <- all.vars(parse(text = ff))
    if (any(!(Names %in% names(newdata)))) 
        stop("newdata doesn't include some model variables")
    #ff <- reformulate(ff)
    nd <- eval(model.frame(ff, data = newdata), parent.frame())
    qssterms <- attr(Terms, "specials")$qss
    if (length(qssterms)) {
        tmp <- untangle.specials(Terms, "qss")
        dropv <- tmp$terms
        m <- length(dropv)
        if (length(dropv)) 
            PLTerms <- Terms[-dropv]
        attr(PLTerms, "specials") <- tmp$vars
    }
    else {
        PLTerms <- Terms
        m <- 0
    }
    if(requireNamespace("MatrixModels") && requireNamespace("Matrix"))
        X <- as(MatrixModels::model.Matrix(PLTerms, data = nd, 
		  contrasts = contrasts, sparse = TRUE),"matrix.csr")
    else
        X <- model.matrix(PLTerms, data = nd)
    p <- ncol(X)
    y <- X %*% object$coef[1:p]
    X <- as.matrix.csr(X)
    if (m > 0) {
        for (i in 1:m) {
            qss <- object$qss[[i]]
            names <- all.vars(Terms[dropv[i]])
            names <- names[names %in% Names]
            dimnames(qss$xyz)[[2]] <- c(names, "zfit")
            newd <- nd[names]
            if (ncol(qss$xyz) == 3) {
                g <- predict.qss2(qss$xyz, newdata = newd, ...)
                y <- y + g$z
                if(interval == "confidence")
                    X <- cbind(X,g$D)
            }
            else if (ncol(qss$xyz) == 2) {
                g <- predict(qss, newdata = newd, ...)
                y <- y + g$y
                if(interval == "confidence")
                    X <- cbind(X,g$D)
            }
            else stop("invalid fitted qss object")
        }
    }
    if(interval == "confidence"){
        v <- sqrt(diag(X %*% summary(object, cov = TRUE)$V %*% t(X)))
        calpha <- qnorm(1 - (1-level)/2)
        y <- cbind(y,y - v*calpha,y + v*calpha)
        dimnames(y)[[2]] <- c("yhat","ylower","yupper")
	}
    y
}

"predict.qss1" <-
function (object, newdata, ...)
{
    x <- object$xyz[, 1]
    y <- object$xyz[, 2] 
    if(ncol(newdata)==1)
        newdata <- newdata[,1]
    else
        stop("newdata should have only one column for predict.qss1")
    if (any(diff(x) < 0))
        stop("x coordinates in qss1 object not monotone")
    if (max(newdata) > max(x) || min(newdata) < min(x))
        stop("no extrapolation allowed in predict.qss")
    bin <- cut(newdata, unique(x), label = FALSE, include.lowest = TRUE)
    p <- length(x)
    m <- length(newdata)
    V <- cbind(bin, bin + 1)
    B <- cbind(x[bin + 1] - newdata, newdata - x[bin])/(x[bin +
        1] - x[bin])
    ra <- c(t(B))
    ja <- as.integer(c(t(V)))
    ia <- as.integer(c(2 * (1:m) - 1, 2 * m + 1))
    dim <- c(m, p)
    D <- new("matrix.csr", ra = ra, ja = ja, ia = ia, dimension = dim)
    list(x = newdata, y = D %*% y, D = D[, -1])
}

predict.qss2 <- function (object, newdata, ...) 
{
    x <- object[, 1]
    y <- object[, 2]
    z <- object[, 3]
    tri.area <- function(v) {
        0.5 * ((v[2, 1] - v[1, 1]) * (v[3, 2] - v[1, 2]) - (v[3, 
            1] - v[1, 1]) * (v[2, 2] - v[1, 2]))
    }
    barycentric <- function(v) {
        b <- rep(0, 3)
        Area <- tri.area(v[1:3, ])
        b[1] <- tri.area(v[c(4, 2, 3), ])/Area
        b[2] <- tri.area(v[c(1, 4, 3), ])/Area
        b[3] <- tri.area(v[c(1, 2, 4), ])/Area
        if (any(b < 0) || any(b > 1)) 
            stop("barycentric snafu")
        b
    }
    if (is.list(newdata)) {
	fnames <- (dimnames(object)[[2]])[1:2]
        if (all(!is.na(match(fnames, names(newdata))))) {
            newx <- newdata[[fnames[1]]]
            newy <- newdata[[fnames[2]]]
	   }
        else (stop("qss object and newdata frame names conflict"))
        }
    else if (is.matrix(newdata)) 
        if (ncol(newdata) == 2) {
            newx <- newdata[, 1]
            newy <- newdata[, 2]
        }
        else (stop("newdata matrix must have 2 columns"))
    tri <- tripack::tri.mesh(x, y)
    if (!all(tripack::in.convex.hull(tri, newx, newy))) 
        stop("some newdata points outside convex hull")
    p <- length(x)
    m <- length(newx)
    V <- matrix(0, m, 3)
    B <- matrix(0, m, 3)
    for (i in 1:m) {
        V[i, ] <- unlist(tripack::tri.find(tri, newx[i], newy[i]))
        v <- rbind(cbind(x[V[i, ]], y[V[i, ]]), c(newx[i], newy[i]))
        B[i, ] <- barycentric(v)
    }
    ra <- c(t(B))
    ja <- as.integer(c(t(V)))
    ia <- as.integer(3 * (0:m) + 1)
    D <- new("matrix.csr", ra = ra, ja = ja, ia = ia, dimension = c(m, 
        p))
    list(x = newx, y = newy, z = c(D %*% z), D = D[,-1])
}

fitted.rqss  <-  function(object, ...) (object$X %*% object$coef)[1:object$n]
resid.rqss <- function(object, ...) object$resid[1:object$n]

rqss <- function (formula, tau = 0.5, data = parent.frame(), weights, 
    subset, na.action, method = "sfn", lambda = NULL, contrasts = NULL, 
    ztol = 1e-05, control = sfn.control(), ...) 
{
    call <- match.call()
    m <- match.call(expand.dots = FALSE)
    temp <- c("", "formula", "data", "weights", "na.action", "subset")
    m <- m[match(temp, names(m), nomatch = 0)]
    m[[1]] <- as.name("model.frame")
    special <- "qss"
    Terms <- if (missing(data)) 
        terms(formula, special)
    else terms(formula, special, data = data)
    qssterms <- attr(Terms, "specials")$qss
    dropx <- NULL
    if (length(tau) > 1) {
        tau <- tau[1]
        warning("multiple taus not supported, using first element")
    }
    if (length(qssterms)) {
        tmpc <- untangle.specials(Terms, "qss")
        ord <- attr(Terms, "order")[tmpc$terms]
        if (any(ord > 1)) 
            stop("qss can not be used in an interaction")
        dropx <- tmpc$terms
        if (length(dropx)) 
            Terms <- Terms[-dropx]
        attr(Terms, "specials") <- tmpc$vars
        fnames <- function(x) {
            fy <- all.names(x[[2]])
            if (fy[1] == "cbind") 
                fy <- fy[-1]
            fy
        }
        fqssnames <- unlist(lapply(parse(text = tmpc$vars), fnames))
        qssnames <- unlist(lapply(parse(text = tmpc$vars), function(x) deparse(x[[2]])))
    }
    ff <- delete.response(terms(formula(Terms)))
    if (exists("fqssnames")) {
	mff <- m
	mff$formula <- Terms
	#mff <- delete.response(terms(formula(Terms)))
        ffqss <- paste(fqssnames, collapse = "+")
	#ff <- paste(deparse(formula(mff)), "+", ffqss)
	mff$formula <- as.formula(paste(deparse(mff$formula), "+", ffqss))
	ff <- delete.response(terms(formula(mff)))
	ff <- deparse(formula(ff))
	mff <- eval(mff, parent.frame())
    }
    m$formula <- Terms
    m <- eval(m, parent.frame())
    weights <- model.extract(m, weights)
    process <- (tau < 0 || tau > 1)
    Y <- model.extract(m, "response")
    if (requireNamespace("MatrixModels") && requireNamespace("Matrix")) {
        X <- MatrixModels::model.Matrix(Terms, m, contrasts.arg = contrasts, 
            sparse = TRUE)
        vnames <- dimnames(X)[[2]]
        X <- as(X, "matrix.csr")
    }
    else {
        X <- model.matrix(Terms, m, contrasts)
        vnames <- dimnames(X)[[2]]
    }
    p <- ncol(X)
    pf <- environment(formula)
    nrL <- 0
    if (method == "lasso") {
        if (!length(lambda)) 
            stop("No lambda specified for lasso constraint")
        if (length(lambda) == 1) 
            lambda <- c(0, rep(lambda, p - 1))
        if (length(lambda) != p) 
            stop("lambda must be either of length p, or length 1")
        if (any(lambda < 0)) 
            stop("negative lambdas disallowed")
        L <- diag(lambda, nrow = length(lambda))
        L <- L[which(lambda != 0), , drop = FALSE]
        L <- as.matrix.csr(L)
        nrL <- nrow(L)
        ncL <- ncol(L)
    }
    if (length(qssterms) > 0) {
        F <- as.matrix.csr(X)
	pf <- environment(formula)
        qss <- lapply(tmpc$vars, function(u) eval(parse(text = u), mff, pf))
        mqss <- length(qss)
        ncA <- rep(0, mqss + 1)
        nrA <- rep(0, mqss + 1)
        nrR <- rep(0, mqss + 1)
        for (i in 1:mqss) {
            F <- cbind(F, qss[[i]]$F)
            ncA[i + 1] <- ncol(qss[[i]]$A)
            nrA[i + 1] <- nrow(qss[[i]]$A)
            nrR[i + 1] <- ifelse(is.null(nrow(qss[[i]]$R)), 0, 
                nrow(qss[[i]]$R))
            vnames <- c(vnames, paste(qssnames[i], 1:ncA[i + 
                1], sep = ""))
        }
        A <- as.matrix.csr(0, sum(nrA), sum(ncA))
        if (sum(nrR) > 0) {
            R <- as.matrix.csr(0, sum(nrR), sum(ncA))
            nrR <- cumsum(nrR)
        }
        ncA <- cumsum(ncA)
        nrA <- cumsum(nrA)
        lambdas <- rep(0, mqss)
        for (i in 1:mqss) {
            lambdas[i] <- qss[[i]]$lambda
            Arows <- (1 + nrA[i]):nrA[i + 1]
            Acols <- (1 + ncA[i]):ncA[i + 1]
            A[Arows, Acols] <- qss[[i]]$lambda * qss[[i]]$A
            if (nrR[i] < nrR[i + 1]) 
                R[(1 + nrR[i]):nrR[i + 1], (1 + ncA[i]):ncA[i + 
                  1]] <- qss[[i]]$R
        }
        A <- cbind(as.matrix.csr(0, nrA[mqss + 1], p), A)
        if (nrR[mqss + 1] > 0) {
            R <- cbind(as.matrix.csr(0, nrR[mqss + 1], p), R)
            r <- rep(0, nrR[mqss + 1])
        }
        else {
            R <- NULL
            r <- NULL
        }
        if (method == "lasso") 
            A <- rbind(cbind(L, as.matrix.csr(0, nrL, ncol(F) - ncL)), A)
	if(length(weights)){
	   F <- F * weights
	   Y <- Y * weights
	}
        X <- rbind(F, A)
        Y <- c(Y, rep(0, nrow(A)))
        rhs <- t(rbind((1 - tau) * F, 0.5 * A)) %*% rep(1, nrow(X))
        XpX <- t(X) %*% X
        nnzdmax <- XpX@ia[length(XpX@ia)] - 1
        if (is.null(control[["nsubmax"]])) 
            control[["nsubmax"]] <- max(nnzdmax, floor(1000 + 
                exp(-1.6) * nnzdmax^1.2))
        if (is.null(control[["nnzlmax"]])) 
            control[["nnzlmax"]] <- floor(2e+05 - 2.8 * nnzdmax + 
                7e-04 * nnzdmax^2)
        if (is.null(control[["tmpmax"]])) 
            control[["tmpmax"]] <- floor(1e+05 + exp(-12.1) * 
                nnzdmax^2.35)
        fit <- if (length(r) > 0) 
            rqss.fit(X, Y, tau = tau, rhs = rhs, method = "sfnc", 
                R = R, r = r, control = control, ...)
        else rqss.fit(X, Y, tau = tau, rhs = rhs, method = "sfn", 
            control = control, ...)
        for (i in 1:mqss) {
            ML <- p + 1 + ncA[i]
            MU <- p + ncA[i + 1]
            qss[[i]] <- list(xyz = cbind(qss[[i]]$x$x, qss[[i]]$x$y, 
                c(0, fit$coef[ML:MU])), dummies = qss[[i]]$dummies, 
			     Dorder = qss[[i]]$Dorder)
            if (ncol(qss[[i]]$xyz) == 2) 
                class(qss[[i]]) <- ifelse(qss[[i]]$Dorder == 1,"qss1", "qts1")
            else class(qss[[i]]) <- "qss2"
        }
        names(qss) <- qssnames
        fit$qss <- qss
    }
    else {
        X <- as.matrix.csr(X)
        nrA <- 0
        if (method == "lasso") {
            rhs <- t(rbind((1 - tau) * X, 0.5 * L)) %*% rep(1, 
                nrow(X) + nrow(L))
            X <- rbind(X, L)
            Y <- c(Y, rep(0, nrL))
        }
        else rhs <- NULL
        if (length(weights)) {
            if (any(weights < 0)) 
                stop("negative weights not allowed")
            X <- X * weights
            Y <- Y * weights
        }
        XpX <- t(X) %*% X
        nnzdmax <- XpX@ia[length(XpX@ia)] - 1
        if (is.null(control[["nsubmax"]])) 
            control[["nsubmax"]] <- max(nnzdmax, floor(1000 + 
                exp(-1.6) * nnzdmax^1.2))
        if (is.null(control[["nnzlmax"]])) 
            control[["nnzlmax"]] <- floor(2e+05 - 2.8 * nnzdmax + 
                7e-04 * nnzdmax^2)
        if (is.null(control[["tmpmax"]])) 
            control[["tmpmax"]] <- floor(1e+05 + exp(-12.1) * 
                nnzdmax^2.35)
        fit <- rqss.fit(X, Y, tau = tau, rhs = rhs, control = control, 
            method = method, ...)
        fit$nrA <- nrA
    }
    names(fit$coef) <- vnames
    n <- length(fit$resid) - nrL - nrA[length(nrA)]
    if(length(weights)) fit$resid[1:n] <- fit$resid[1:n]/weights
    uhat <- fit$resid[1:n]
    Rho <- function(u, tau) sum(u * (tau - (u < 0)))
    fit$fidelity <- Rho(uhat, tau)
    fit$edf <- sum(abs(uhat) < ztol)
    fit$X <- X
    fit$y <- Y
    fit$n <- n
    fit$nrL <- nrL
    fit$terms <- Terms
    fit$fake.formula <- ff
    fit$formula <- formula
    fit$method <- method
    fit$call <- call
    fit$tau <- tau
    if (length(qssterms)) {
        fit$lambdas <- lambdas
        fit$qssnames <- qssnames
        fit$nrA <- nrA
        fit$ncA <- cumsum(c(p, diff(ncA)))
    }
    else fit$ncA <- p
    attr(fit, "na.message") <- attr(m, "na.message")
    class(fit) <- "rqss"
    fit
}
"Munge" <- function(formula, ...) { 
    # Recursive substitution {per Gabor Grothendieck}
    if (length(formula) > 1) {
	if (identical(formula[[2]], as.name(names(list(...))))) 
	    formula <- eval(formula, list(...))
	if (length(formula) > 1) 
	    for (i in 1:length(formula)) 
		formula[[i]] <- Recall(formula[[i]], ...)
	}
   formula
}

"summary.rqss" <- function(object, cov = FALSE, ztol = 1e-5, ...){
    resid <- object$resid
    coef <- object$coef
    lambdas <- object$lambdas
    formula <- object$formula
    fidelity <- object$fidelity
    edf <- object$edf
    nrA <- object$nrA
    ncA <- object$ncA
    nrL <- object$nrL
    tau <- object$tau
    cntl <- object$control
    X <- object$X
    p <- ncol(object$X)
    m <- length(ncA)
    n <- length(resid) - nrL - nrA[m]
    uhat <- resid[1:n]
    h <- bandwidth.rq(tau, n, hs = TRUE)
    while((tau - h < 0) || (tau + h > 1)) h <- h/2
    h <- (qnorm(tau + h) - qnorm(tau - h)) * max(ztol,min(sqrt(var(uhat)),
        (quantile(uhat, 0.75) - quantile(uhat, 0.25))/1.34))
    f <- c(dnorm(uhat/h)/h,rep(1, length(resid) - n))
    D <- t(X) %*% (f * X)
    D <- chol(.5 * (D + t(D)), nsubmax = cntl$nsubmax,
             nnzlmax = cntl$nnzlmax, tmpmax = cntl$tmpmax)
    D <- backsolve(D,diag(p))
    D0 <- tau * (1 - tau) * t(X) %*% X
    V <- D %*% D0 %*% D
    scale <- mean(f)
    serr <- sqrt(diag(V))
    ptab <- array(coef[1:ncA[1]], c(ncA[1],4))
    pnames <- names(coef[1:ncA[1]])
    dimnames(ptab) <- list(pnames, c("Estimate", "Std. Error", "t value", "Pr(>|t|)"))
    ptab[, 2] <- serr[1:ncA[1]]
    ptab[, 3] <- ptab[, 1]/ptab[, 2]
    ptab[, 4] <- 2 * (1 - pt(abs(ptab[, 3]), n - edf))
    if(cov) {
        Vcov <- V[1:ncA[1],1:ncA[1]]
        Vqss <- as.list(1:(m-1))
        }
    if(m > 1) {
        penalty <- rep(NA, m - 1)
        qssedfs <- rep(NA, m - 1)
        ntab <- matrix(0, m - 1, 5)
        vnames <- sub(".$","",names(coef[ncA[-length(ncA)]+1]))
        dimnames(ntab) <- list(vnames, c("EDF", "Lambda", "Penalty", "F value", "Pr(>F)"))
        for(i in 2:m) {
           ntab[i - 1, 3] <- sum(abs(resid[n + nrL + ((nrA[i - 1] + 1):nrA[i])]))
           ntab[i - 1, 1] <- sum(abs(resid[n + nrL + ((nrA[i - 1] + 1):nrA[i])]) > ztol)
           v <- V[c(1, (ncA[i-1] + 1):ncA[i]), c(1, (ncA[i-1] + 1):ncA[i])]
           v <- .5 * (v + t(v))
           if(cov) Vqss[[i-1]] <- v
           b <- coef[(ncA[i-1] + 1):ncA[i]]
           ntab[i-1, 4]  <- t(b) %*% solve(v[-1,-1],b)/ntab[i-1,1]
           ntab[i-1, 5]  <- 1 - pf(ntab[i-1,4],ntab[i-1,1], n - edf)
           }
        ntab[,2] <- lambdas
        ntab[,3] <- ntab[,3]/lambdas
        if(cov)
           z <- list(coef = ptab, qsstab = ntab, fidelity = fidelity, tau = tau,
		formula = formula, edf = edf, n = n, Vcov = Vcov, Vqss = Vqss, V = V)
        else
           z <- list(coef = ptab, qsstab = ntab, fidelity = fidelity, tau = tau,
		formula = formula, edf = edf, n = n)
        }
    else
        if(cov)
           z <- list(coef = ptab, fidelity = fidelity, formula = formula, tau = tau,
		edf = edf, n = n, Vcov = Vcov, V = V)
        else
           z <- list(coef = ptab, fidelity = fidelity, formula = formula, tau = tau,
		edf = edf, n = n)

    class(z) <- "summary.rqss"
    return(z)
}
"plot.summary.rqss" <- function(x, ...)
    warning("No plot method for summary.rqss objects:   plot the rqss object instead")

"print.rqss" <- function(x, ...) {
    cat("Formula:\n")
    print(x$formula)
    sx <- summary(x)
    cat("Quantile fidelity at tau = ", x$tau, "is", sx$fidelity, "\n")
    cat("Estimated Model Dimension is", sx$edf, "\n")
}
 
"logLik.rqss" <- function(object, ...){
	n <- object$n
	tau <- object$tau
	val <- n * (log(tau * (1-tau)) - 1 - log(object$fidelity/n))
	attr(val,"n") <- n
	attr(val,"df") <- object$edf
	class(val) <- "logLik"
	val
	}
"AIC.rqss" <- function(object, ... , k = 2){
	v <- logLik(object)
	if(k < 0) 
		k <- log(attr(v,"n"))
	val <- AIC(logLik(object), k = k)
	attr(val,"edf") <- attr(v,"df")
	val
	}
"dither"  <- function(x, type = "symmetric", value = NULL) {
	if(length(x) == 0)
		return(x)
	if(!is.numeric(x))
		stop("'x' must be numeric")
	if(!length(value))
		value <- min(diff(sort(unique(x))))
	if(type == "symmetric")
		v <- x + runif(length(x), -value/2, value/2)
	else if(type == "right")
		v <- x + runif(length(x), 0, value)
	else
		stop("invalid type")
	v
}
	
print.summary.rqss <-
function (x, digits = max(3, getOption("digits") - 3), signif.stars = getOption("show.signif.stars"), 
    ...) 
{
    cat("Formula:\n")
    print(x$formula)
    if (length(x$coef) > 0) {
        cat("\nParametric coefficients:\n")
        printCoefmat(x$coef, digits = digits, signif.stars = signif.stars, 
            na.print = "NA", ...)
    }
    cat("\n")
    if (length(x$qsstab) > 0) {
        cat("Approximate significance of qss terms:\n")
        printCoefmat(x$qsstab, digits = digits, signif.stars = signif.stars, 
            has.Pvalue = TRUE, na.print = "NA", cs.ind = 1, ...)
    }
    cat("\n")
    if (length(x$fidelity) > 0) 
        cat("  Quantile Fidelity at tau = ", x$tau, "  is  ", formatC(x$fidelity, 
		digits = 6, width = 11), "\n", sep = "")
    cat("  Effective Degrees of Freedom = ", formatC(x$edf, digits = 5, width = 8, 
        flag = "-"), "  Sample Size = ", x$n, "\n", sep = "")
    invisible(x)
}
critval <- function(kappa, alpha = 0.05, rdf = 0){
# Hotelling tube critical value for uniform confidence bands
# This should agree to about 6 digits with the following call to locfit:
# crit(const = c(kappa,1), cov = 1-alpha,rdf = rdf)$crit.val
   tube <- function(x,alpha,rdf){
      if(rdf <= 0)
         kappa * exp(-x^2/2)/pi + 2 * (1 - pnorm(x)) -  alpha
      else
         kappa * (1+x^2/rdf)^(-rdf/2)/pi + 2 * (1 - pt(x,rdf)) -  alpha
      }
   uniroot(tube,c(1,5),alpha = alpha, rdf = rdf)$root
   }

bandwidth.rq <- function(p, n, hs = TRUE, alpha = 0.05)
{
	# Bandwidth selection for sparsity estimation two flavors:
	#	Hall and Sheather(1988, JRSS(B)) rate = O(n^{-1/3})
	#	Bofinger (1975, Aus. J. Stat)  -- rate = O(n^{-1/5})
	# Generally speaking, default method, hs=TRUE is preferred.

	x0 <- qnorm(p)
	f0 <- dnorm(x0)
	if(hs)
            n^(-1/3) * qnorm(1 - alpha/2)^(2/3) *
                ((1.5 * f0^2)/(2 * x0^2 + 1))^(1/3)
	else n^-0.2 * ((4.5 * f0^4)/(2 * x0^2 + 1)^2)^ 0.2
}


plot.rq.process <- function(x, nrow = 3, ncol = 2, ...)
{
    ## Function to plot estimated quantile regression  process
	tdim <- dim(x$sol)
	p <- tdim[1] - 3
	m <- tdim[2]
	oldpar <- par(no.readonly=TRUE)
	par(mfrow = c(nrow, ncol))
	ylab <- dimnames(x$sol)[[1]]
	for(i in 1:p) {
		plot(x$sol[1,], x$sol[3 + i,  ], xlab = "tau",
                     ylab = ylab[3 + i], type = "l")
	}
par(oldpar)
}

print.rqs <- function (x, ...)
{
    if (!is.null(cl <- x$call)) {
        cat("Call:\n")
        dput(cl)
    }
    coef <- coef(x)
    cat("\nCoefficients:\n")
    print(coef, ...)
    rank <- x$rank
    nobs <- nrow(residuals(x))
    p <- nrow(coef)
    rdf <- nobs - p
    cat("\nDegrees of freedom:", nobs, "total;", rdf, "residual\n")
    if (!is.null(attr(x, "na.message")))
        cat(attr(x, "na.message"), "\n")
    invisible(x)
}

"print.rq" <-
function(x, ...)
{
	if(!is.null(cl <- x$call)) {
		cat("Call:\n")
		dput(cl)
	}
	coef <- coef(x)
	cat("\nCoefficients:\n")
	print(coef, ...)
	rank <- x$rank
	nobs <- length(residuals(x))
	if(is.matrix(coef))
		p <- dim(coef)[1]
	else p <- length(coef)
	rdf <- nobs - p
	cat("\nDegrees of freedom:", nobs, "total;", rdf, "residual\n")
	if(!is.null(attr(x, "na.message")))
		cat(attr(x, "na.message"), "\n")
	invisible(x)
}

print.summary.rqs <- function(x, ...) {
    lapply(x, print.summary.rq)
    invisible(x)
}

"print.summary.rq" <-
function(x, digits = max(5, .Options$digits - 2), ...)
{
	cat("\nCall: ")
	dput(x$call)
	coef <- x$coef
	## df <- x$df
	## rdf <- x$rdf
	tau <- x$tau
	cat("\ntau: ")
	print(format(round(tau,digits = digits)), quote = FALSE, ...)
	cat("\nCoefficients:\n")
	print(format(round(coef, digits = digits)), quote = FALSE, ...)
	invisible(x)
}

"rq" <-
function (formula, tau = 0.5, data, subset, weights, na.action, method = "br",
    model = TRUE, contrasts = NULL, ...)
{
    call <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset", "weights", "na.action"), names(mf), 0)
    mf <- mf[c(1,m)]
    mf$drop.unused.levels <- TRUE
    mf[[1]] <- as.name("model.frame")
    mf <- eval.parent(mf)
    if(method == "model.frame")return(mf)
    mt <- attr(mf, "terms")
    weights <- as.vector(model.weights(mf))
    tau <- sort(unique(tau))
    eps <- .Machine$double.eps^(2/3)
    if (any(tau == 0)) tau[tau == 0] <- eps
    if (any(tau == 1)) tau[tau == 1] <- 1 - eps
    Y <- model.response(mf)
    if(method == "sfn"){
	if(requireNamespace("MatrixModels") && requireNamespace("Matrix")){
	    X <- MatrixModels::model.Matrix(mt, data, sparse = TRUE)
	    vnames <- dimnames(X)[[2]]
	    X <- as(X ,"matrix.csr")
	    mf$x <- X
	    }
    }	
    else{
	X <- model.matrix(mt, mf, contrasts)
	vnames <- dimnames(X)[[2]]
    }
    Rho <- function(u,tau) u * (tau - (u < 0))
    if (length(tau) > 1) {
      if (any(tau < 0) || any(tau > 1)) 
        stop("invalid tau:  taus should be >= 0 and <= 1")
      coef <- matrix(0, ncol(X), length(tau))
      rho <- rep(0, length(tau))
      if(!(method %in% c("ppro","qfnb","pfnb"))){
        fitted <- resid <- matrix(0, nrow(X), length(tau))
	for(i in 1:length(tau)){
	    z <- {if (length(weights))
                 	rq.wfit(X, Y, tau = tau[i], weights, method, ...)
                  else rq.fit(X, Y, tau = tau[i], method, ...)
    		 }
	    coef[,i] <- z$coefficients
	    resid[,i] <- z$residuals
            rho[i] <- sum(Rho(z$residuals,tau[i]))
	    fitted[,i] <- Y - z$residuals
	   }
	taulabs <- paste("tau=",format(round(tau,3)))
	dimnames(coef) <- list(vnames, taulabs)
	dimnames(resid)[[2]] <- taulabs
	fit <- z
        fit$coefficients <-  coef
        fit$residuals <- resid
        fit$fitted.values <- fitted
	if(method == "lasso") class(fit) <- c("lassorqs","rqs")
	else if(method == "scad") class(fit) <- c("scadrqs","rqs")
	else class(fit) <- "rqs"
	}
      else if(method == "pfnb"){ # Preprocessing in fortran loop
	  fit <- rq.fit.pfnb(X, Y, tau)
          class(fit) = "rqs" 
          }
      else if(method == "qfnb"){ # simple fortran loop method 
	  fit <- rq.fit.qfnb(X, Y, tau)
          class(fit) = ifelse(length(tau) == 1,"rq","rqs") 
          }
      else if(method == "ppro"){ # Preprocessing method in R
	  fit <- rq.fit.ppro(X, Y, tau, ...)
          class(fit) = ifelse(length(tau) == 1,"rq","rqs") 
          }
    }
    else{
      process <- (tau < 0 || tau > 1)
      if(process && method != "br") 
        stop("when tau not in [0,1] method br must be used")
      fit <- {
        if(length(weights))
          rq.wfit(X, Y, tau = tau, weights, method, ...)
        else rq.fit(X, Y, tau = tau, method, ...)
        }
      if(process)
	rho <- list(tau = fit$sol[1,], rho = fit$sol[3,])
      else {
	if(length(dim(fit$residuals)))
          dimnames(fit$residuals) <- list(dimnames(X)[[1]],NULL)
          rho <-  sum(Rho(fit$residuals,tau))
          } 
	if(method == "lasso") class(fit) <- c("lassorq","rq")
	else if(method == "scad") class(fit) <- c("scadrq","rq")
        else class(fit) <- ifelse(process, "rq.process", "rq")
	}
    fit$na.action <- attr(mf, "na.action")
    fit$formula <- formula
    fit$terms <- mt
    fit$xlevels <- .getXlevels(mt,mf)
    fit$call <- call
    fit$tau <- tau
    fit$weights <- weights
    fit$residuals <- drop(fit$residuals)
    fit$rho <- rho
    fit$method <- method
    fit$fitted.values <- drop(fit$fitted.values)

    attr(fit, "na.message") <- attr(m, "na.message")
    if(model) fit$model <- mf
    fit
}
"rq.fit" <-
function(x, y, tau = 0.5, method = "br", ...)
{
    if(length(tau) > 1 && method != "ppro") {
	tau <- tau[1]
        warning("Multiple taus not allowed in rq.fit: solution restricted to first element")
        }

    fit <- switch(method,
		fn = rq.fit.fnb(x, y, tau = tau, ...),
		fnb = rq.fit.fnb(x, y, tau = tau, ...),
		fnc = rq.fit.fnc(x, y, tau = tau, ...),
		sfn = rq.fit.sfn(x, y, tau = tau, ...),
		conquer = rq.fit.conquer(x, y, tau = tau, ...),
		pfn = rq.fit.pfn(x, y, tau = tau, ...),
		pfnb = rq.fit.pfnb(x, y, tau = tau, ...),
		ppro= rq.fit.ppro(x, y, tau = tau, ...),
		br = rq.fit.br(x, y, tau = tau, ...),
		lasso = rq.fit.lasso(x, y, tau = tau, ...),
		scad = rq.fit.scad(x, y, tau = tau, ...),
		{
			what <- paste("rq.fit.", method, sep = "")
			if(exists(what, mode = "function"))
				(get(what, mode = "function"))(x, y, ...)
			else stop(paste("unimplemented method:", method))
		}
		)
	fit$fitted.values <- y - fit$residuals
	fit$contrasts <- attr(x, "contrasts")
	fit
}
"rqs.fit"<-
function(x, y, tau = 0.5, tol = 0.0001)
{
# function to compute rq fits for multiple y's
        x <- as.matrix(x)
        p <- ncol(x)
        n <- nrow(x)
        m <- ncol(y)
        z <- .Fortran("rqs",
                as.integer(n),
                as.integer(p),
                as.integer(m),
                as.integer(n + 5),
                as.integer(p + 2),
                as.double(x),
                as.double(y),
                as.double(tau),
                as.double(tol),
                flag = integer(m),
                coef = double(p * m),
                resid = double(n),
                integer(n),
                double((n + 5) * (p + 2)),
                double(n))
        if(sum(z$flag)>0){
                if(any(z$flag)==2)
                        warning(paste(sum(z$flag==2),"out of",m,
                                "BS replications have near singular design"))
                if(any(z$flag)==1)
                        warning(paste(sum(z$flag==1),"out of",m,"may be nonunique"))
                }
        return(t(matrix(z$coef, p, m)))
}
"formula.rq" <-
function (x, ...)
{
    form <- x$formula
    if (!is.null(form)) {
        form <- formula(x$terms)
        environment(form) <- environment(x$formula)
        form
    }
    else formula(x$terms)
}

"predict.rq" <-
function (object, newdata, type = "none", interval = c("none",
    "confidence"), level = 0.95, na.action = na.pass, ...)
{
    if (missing(newdata))
        return(object$fitted)
    else {
        tt <- terms(object)
        Terms <- delete.response(tt)
        m <- model.frame(Terms, newdata, na.action = na.action,
            xlev = object$xlevels)
        if (!is.null(cl <- attr(Terms, "dataClasses")))
            .checkMFClasses(cl, m)
        X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
    }
    pred <- drop(X %*% object$coefficients)
    dots <- list(...)
    if (length(dots$se))
        boot <- (dots$se == "boot")
    else boot <- FALSE
    if (length(dots$mofn))
         mofn <- dots$mofn
    interval <- match.arg(interval)
    if (!interval == "none") {
        if (interval == "confidence") {
            if (type == "percentile") {
                if (boot) {
		    if(exists("mofn")) {# Rescale and recenter!!
		      n <- length(object$fitted)
                      factor <- ifelse(mofn < n, sqrt(mofn/n), 1)
                      XB <- X %*% t(summary(object, cov = TRUE, ...)$B)/factor
                      pl <- apply(XB, 1, function(x) quantile(x, (1 - level)/2))
                      pu <- apply(XB, 1, function(x) quantile(x, 1 - (1 - level)/2))
                      pl <- pred + factor * (pl - pred)
                      pu <- pred + factor * (pu - pred)
                  }
                  else {
                      XB <- X %*% t(summary(object, cov = TRUE, ...)$B)
                      pl <- apply(XB, 1, function(x) quantile(x, (1 - level)/2))
                      pu <- apply(XB, 1, function(x) quantile(x, 1 - (1 - level)/2))
                  }
                  pred <- cbind(pred, pl, pu)
                  colnames(pred) <- c("fit", "lower", "higher")
                }
                else stop("Percentile method requires se = \"boot\".")
            }
            else if (type == "direct") {
                if (boot)
                  stop("Direct method incompatible with bootstrap covariance matrix estimation")
                Z <- rq.fit(object$x, object$y, tau = -1)$sol
                V <- summary(object, cov = TRUE, ...)
                df <- V$rdf
                tfrac <- qt(1 - (1 - level)/2, df)
                Vun <- V$cov * V$scale^2
                tau <- object$tau
                bn <- tfrac * sqrt(diag(X %*% Vun %*% t(X)))
                tauU <- pmin(tau + bn, 1 - 1/df)
                tauL <- pmax(tau - bn, 1/df)
                tauhat <- Z[1, ]
                yhat <- X %*% Z[-(1:3), ]
                n <- nrow(X)
                pl <- yhat[cbind(1:n, cut(tauL, tauhat, label = FALSE))]
                pu <- yhat[cbind(1:n, cut(tauU, tauhat, label = FALSE))]
                pred <- cbind(pred, pl, pu)
                colnames(pred) <- c("fit", "lower", "higher")
            }
            else {
                V <- summary(object, cov = TRUE, ...)
                df <- V$rdf
                tfrac <- qt((1 - level)/2, df)
                sdpred <- sqrt(diag(X %*% V$cov %*% t(X)))
                pred <- cbind(pred, pred + tfrac * sdpred %o%
                  c(1, -1))
                colnames(pred) <- c("fit", "lower", "higher")
            }
        }
        else stop(paste("No interval method for", interval))
    }
    pred
}

"predict.rqs" <-
function (object, newdata, type = "Qhat", stepfun = FALSE, na.action = na.pass, ...)
{
   ## with all defaults
   if(missing(newdata) && !stepfun && (type == "Qhat")) return(object$fitted)

   ## otherwise
   tt <- delete.response(terms(object))
   m <- if(missing(newdata)) model.frame(object) else model.frame(tt, newdata,
       na.action = na.action, xlev = object$xlevels)
   if(!is.null(cl <- attr(tt, "dataClasses"))) .checkMFClasses(cl, m)
   X <- model.matrix(tt, m, contrasts = object$contrasts)
   pred <- t(X %*% object$coefficients)
   taus <- object$tau
   M <- NCOL(pred)

   ## return stepfun or matrix
   if(stepfun) {
       if(type == "Qhat"){
	   pred <- rbind(pred[1,],pred)
          if(M > 1) 
	      f <- apply(pred, 2, function(y) stepfun(taus, y))
          else
              f <- stepfun(taus, c(pred[1,1], pred[,1]))
          }
       else if(type == "Fhat"){
	   taus <- c(taus[1], taus)
           if(M > 1) 
	       f <- apply(pred, 2, function(y) {
                        o <- order(y)
                        stepfun(y[o], taus[c(1,o)])})
          else
              f <- stepfun(pred[,1],taus)
       }
       else stop("Stepfuns must be either 'Qhat' or 'Fhat'\n")
       return(f)
   } 
   else if(type == "fhat"){
	akjfun <- function(z, p, d = 10, g = 300, ...) {
            mz <- sum(z * p)
            sz <- sqrt(sum((z - mz)^2 * p))
            hz <- seq(mz - d * sz, mz + d * sz, length = g)
            fz <- akj(z, hz, p = p, ...)$dens
            approxfun(hz, fz)
        }
        p <- diff(taus)
        if (M > 1) 
            f <- apply(pred[-1, ], 2, function(z) akjfun(z, p, ...))
        else akjfun(pred[, 1], p, ...)
        return(f)
    }
  else return(t(pred))
}


"predict.rq.process" <-
function (object, newdata, type = "Qhat", stepfun = FALSE, na.action = na.pass, ...)
{
    if(missing(newdata) && !stepfun && (type == "Qhat")) return(object$fitted)
    tt <- terms(object)
    Terms <- delete.response(tt)
    m <- model.frame(Terms, newdata, na.action = na.action,
        xlev = object$xlevels)
    if (!is.null(cl <- attr(Terms, "dataClasses"))) .checkMFClasses(cl, m)
    X <- model.matrix(Terms, m, contrasts = object$contrasts)
    if(!length(X)) X <- rep(1, NROW(object$dsol)) # intercept only hack
    pred <- t(X %*% object$sol[-(1:3),, drop = FALSE])
    taus <- object$sol[1,]
    M <- NCOL(pred)
    if(stepfun){
       if(type == "Qhat"){
	  pred <- rbind(pred[1,], pred)
          if(M > 1)
	      f <- apply(pred,2,function(y) stepfun(taus, y))
          else
              f <- stepfun(taus, pred[,1])
          }
       else if(type == "Fhat"){
	  taus <- c(taus[1],taus)
          if(M > 1) 
	      f <- apply(pred,2,function(y) stepfun(y,taus))
          else
              f <- stepfun(pred[,1],taus)
          }
       else stop("Stepfuns must be either 'Qhat' or 'Fhat'")
       return(f)
    }
    else if(type == "fhat"){
	akjfun <- function(z, p, d = 10, g = 300, ...){
	   mz <- sum(z * p)
	   sz <- sqrt(sum((z - mz)^2 * p))
           hz <- seq(mz -  d * sz, mz+ d * sz, length = g)
           fz <- akj(z, hz, p = p, ...)$dens
           approxfun(hz,fz)
	}
	p <- diff(taus)
        if(M > 1) 
	    f <- apply(pred[-1,], 2, function(z) akjfun(z, p, ...))
        else
	   f = akjfun(pred[,1], p, ...)
       return(f)
       }
    else return(t(pred))
}
"rearrange"  <- function (f, xmin, xmax)
# Revised Version September 11 2007.
{
    if (is.list(f))
        lapply(f, rearrange)
    else {
        if (!is.stepfun(f))
            stop("Only stepfuns can be rearranged.\n")
    	call	<- attributes(f)$call;
    	right <- call[match("right",names(call))]=="TRUE()"
        x 	<- knots(f)
	n	<- length(x)
	if(missing(xmin)) xmin = x[1]
	if(missing(xmax)) xmax = x[n]
	x	<- x[(x >= xmin) & (x <= xmax)]
	x	<- c(xmin, x, xmax)
	n	<- length(x)
	y	<- f(x)
        o 	<- ifelse(rep(right,n-1), order(y[-1])+1, order(y[-n]))
        x 	<- cumsum(c(x[1], diff(x)[o - right]))
        y 	<- y[o]
        y 	<- c(y[1], y, max(y))
        stepfun(x, y, right = right)
    }

}



# Function to compute regression quantiles using original simplex approach
# of Barrodale-Roberts/Koenker-d'Orey.  There are several options.
# The options are somewhat different than those available for the Frisch-
# Newton version of the algorithm, reflecting the different natures of the
# problems typically solved.  Succintly BR for "small" problems, FN for
# "large" ones.  Obviously, these terms are conditioned by available hardware.
#
# Basically there are two modes of use:
# 1.  For Single Quantiles:
#
#       if tau is between 0 and 1 then only one quantile solution is computed.
#
#       if ci = FALSE  then just the point estimate and residuals are returned
#		If the column dimension of x is 1 then ci is set to FALSE since
#		since the rank inversion method has no proper null model.
#       if ci = TRUE  then there are two options for confidence intervals:
#
#               1.  if iid = TRUE we get the original version of the rank
#                       inversion intervals as in Koenker (1994)
#               2.  if iid = FALSE we get the new version of the rank inversion
#                       intervals which accounts for heterogeneity across
#                       observations in the conditional density of the response.
#                       The theory of this is described in Koenker-Machado(1999)
#               Both approaches involve solving a parametric linear programming
#               problem, the difference is only in the factor qn which
#               determines how far the PP goes.  In either case one can
#               specify two other options:
#                       1. interp = FALSE returns two intervals an upper and a
#                               lower corresponding to a level slightly
#                               above and slightly below the one specified
#                               by the parameter alpha and dictated by the
#                               essential discreteness in the test statistic.
#				interp = TRUE  returns a single interval based on
#                               linear interpolation of the two intervals
#                               returned:  c.values and p.values which give
#                               the critical values and p.values of the
#                               upper and lower intervals. Default: interp = TRUE.
#                       2.  tcrit = TRUE uses Student t critical values while
#                               tcrit = FALSE uses normal theory ones.
# 2. For Multiple Quantiles:
#
#       if tau < 0 or tau >1 then it is presumed that the user wants to find
#       all of the rq solutions in tau, and the program computes the whole
#	quantile regression solution as a process in tau, the resulting arrays
#	containing the primal and dual solutions, betahat(tau), ahat(tau)
#       are called sol and dsol.  These arrays aren't printed by the default
#       print function but they are available as attributes.
#       It should be emphasized that this form of the solution can be
#	both memory and cpu quite intensive.  On typical machines it is
#	not recommended for problems with n > 10,000.
#	In large problems a grid of solutions is probably sufficient.
#
rq.fit.br <-
function (x, y, tau = 0.5, alpha = 0.1, ci = FALSE, iid = TRUE,
	interp = TRUE, tcrit = TRUE)
{
    tol <- .Machine$double.eps^(2/3)
    eps <- tol
    big <- .Machine$double.xmax
    x <- as.matrix(x)
    p <- ncol(x)
    n <- nrow(x)
    ny <- NCOL(y)
    nsol <- 2
    ndsol <- 2
    # Check for Singularity of X since br fortran isn't very reliable about this
    if (qr(x)$rank < p)
        stop("Singular design matrix")
    if (tau < 0 || tau > 1) {
        nsol <- 3 * n
        ndsol <- 3 * n
        lci1 <- FALSE
        qn <- rep(0, p)
        cutoff <- 0
        tau <- -1
    }
    else {
        if (p == 1)
            ci <- FALSE
        if (ci) {
            lci1 <- TRUE
            if (tcrit)
                cutoff <- qt(1 - alpha/2, n - p)
            else cutoff <- qnorm(1 - alpha/2)
            if (!iid) {
                h <- bandwidth.rq(tau, n, hs = TRUE)
                bhi <- rq.fit.br(x, y, tau + h, ci = FALSE)
                bhi <- coefficients(bhi)
                blo <- rq.fit.br(x, y, tau - h, ci = FALSE)
                blo <- coefficients(blo)
                dyhat <- x %*% (bhi - blo)
                if (any(dyhat <= 0)) {
                  pfis <- (100 * sum(dyhat <= 0))/n
                  warning(paste(pfis, "percent fis <=0"))
                }
                f <- pmax(eps, (2 * h)/(dyhat - eps))
                qn <- rep(0, p)
                for (j in 1:p) {
                  qnj <- lm(x[, j] ~ x[, -j] - 1, weights = f)$resid
                  qn[j] <- sum(qnj * qnj)
                }
            }
            else qn <- 1/diag(solve(crossprod(x)))
        }
        else {
            lci1 <- FALSE
            qn <- rep(0, p)
            cutoff <- 0
        }
    }
    z <- .Fortran("rqbr", as.integer(n), as.integer(p), as.integer(n +
        5), as.integer(p + 3), as.integer(p + 4), as.double(x),
        as.double(y), as.double(tau), as.double(tol), flag = as.integer(1),
        coef = double(p), resid = double(n), integer(n), double((n +
            5) * (p + 4)), double(n), as.integer(nsol), as.integer(ndsol),
        sol = double((p + 3) * nsol), dsol = double(n * ndsol),
        lsol = as.integer(0), h = integer(p * nsol), qn = as.double(qn),
        cutoff = as.double(cutoff), ci = double(4 * p), tnmat = double(4 *
            p), as.double(big), as.logical(lci1))
    if (z$flag != 0)
        warning(switch(z$flag, "Solution may be nonunique", "Premature end - possible conditioning problem in x"))
    if (tau < 0 || tau > 1) {
        sol <- matrix(z$sol[1:((p + 3) * z$lsol)], p + 3)
        dsol <- matrix(z$dsol[1:(n * z$lsol)], n)
        vnames <- dimnames(x)[[2]]
        dimnames(sol) <- list(c("tau", "Qbar", "Obj.Fun", vnames),
            NULL)
        return(list(sol = sol, dsol = dsol))
    }
    if (!ci) {
        coef <- z$coef
        dual <- z$dsol[1:n]
        names(coef) <- dimnames(x)[[2]]
        return(list(coefficients = coef, x = x, y = y, residuals = y - x %*% z$coef,
		dual = dual))
    }
    if (interp) {
        Tn <- matrix(z$tnmat, nrow = 4)
        Tci <- matrix(z$ci, nrow = 4)
        Tci[3, ] <- Tci[3, ] + (abs(Tci[4, ] - Tci[3, ]) * (cutoff -
            abs(Tn[3, ])))/abs(Tn[4, ] - Tn[3, ])
        Tci[2, ] <- Tci[2, ] - (abs(Tci[1, ] - Tci[2, ]) * (cutoff -
            abs(Tn[2, ])))/abs(Tn[1, ] - Tn[2, ])
        Tci[2, ][is.na(Tci[2, ])] <- -big
        Tci[3, ][is.na(Tci[3, ])] <- big
        coefficients <- cbind(z$coef, t(Tci[2:3, ]))
        vnames <- dimnames(x)[[2]]
        cnames <- c("coefficients", "lower bd", "upper bd")
        dimnames(coefficients) <- list(vnames, cnames)
        residuals <- y - drop(x %*% z$coef)
        return(list(coefficients = coefficients, residuals = residuals))
    }
    else {
        Tci <- matrix(z$ci, nrow = 4)
        coefficients <- cbind(z$coef, t(Tci))
        residuals <- y - drop(x %*% z$coef)
        vnames <- dimnames(x)[[2]]
        cnames <- c("coefficients", "lower bound", "Lower Bound",
            "upper bd", "Upper Bound")
        dimnames(coefficients) <- list(vnames, cnames)
        c.values <- t(matrix(z$tnmat, nrow = 4))
        c.values <- c.values[, 4:1]
        dimnames(c.values) <- list(vnames, cnames[-1])
        p.values <- if (tcrit)
            matrix(pt(c.values, n - p), ncol = 4)
        else matrix(pnorm(c.values), ncol = 4)
        dimnames(p.values) <- list(vnames, cnames[-1])
        list(coefficients = coefficients, residuals = residuals,
            c.values = c.values, p.values = p.values)
    }
}

"rq.fit.conquer" <- function(x, y, tau = 0.5, kernel = c("Gaussian", "uniform", 
    "parabolic", "triangular"), h = 0,  tol = 1e-04, 
    iteMax = 5000, ci = FALSE, alpha = 0.05, B = 200)
{
    if(!requireNamespace("conquer", quietly = TRUE))
            stop("method conquer requires package conquer")
    fit = conquer::conquer(x[,-1], y, tau = tau, kernel = kernel, h = h, 
	tol = tol, iteMax = iteMax, ci = FALSE, alpha = alpha, B = 1000)
    coefficients = fit$coeff 
    names(coefficients) = dimnames(x)[[2]]
    residuals = fit$residual
    list(coefficients = coefficients, tau = tau, residuals = residuals)
}
"rq.fit.fnb" <-
function (x, y, tau = 0.5, rhs = (1-tau)*apply(x,2,sum), beta = 0.99995, eps = 1e-06)
{
    n <- length(y)
    p <- ncol(x)
    if (n != nrow(x))
        stop("x and y don't match n")
    if (tau < eps || tau > 1 - eps)
        stop("No parametric Frisch-Newton method.  Set tau in (0,1)")
    d   <- rep(1,n)
    u   <- rep(1,n)
    wn <- rep(0,10*n)
    wn[1:n] <- (1-tau) #initial value of dual solution
    z <- .Fortran("rqfnb", as.integer(n), as.integer(p), a = as.double(t(as.matrix(x))),
        c = as.double(-y), rhs = as.double(rhs), d = as.double(d),as.double(u),
        beta = as.double(beta), eps = as.double(eps),
        wn = as.double(wn), wp = double((p + 3) * p),
        nit = integer(3), info = integer(1))
    if (z$info != 0)
        warning(paste("Error info = ", z$info, "in stepy: possibly singular design"))
    coefficients <- -z$wp[1:p]
    names(coefficients) <- dimnames(x)[[2]]
    residuals <- y - x %*% coefficients
    list(coefficients=coefficients, tau=tau, residuals=residuals, nit = z$nit)
}

"rq.fit.fnc" <-
function (x, y, R, r, tau = 0.5, beta = 0.9995, eps = 1e-06)
{
    n1 <- length(y)
    n2 <- length(r)
    p <- ncol(x)
    if (n1 != nrow(x))
        stop("x and y don't match n1")
    if (n2 != nrow(R))
        stop("R and r don't match n2")
    if (p != ncol(R))
        stop("R and x don't match p")
    if (tau < eps || tau > 1 - eps)
        stop("No parametric Frisch-Newton method.  Set tau in (0,1)")
    rhs <- (1 - tau) * apply(x, 2, sum)
    u <- rep(1, max(n1,n2)) #upper bound vector and scratch vector
    wn1 <- rep(0, 9 * n1)
    wn1[1:n1] <- (1 - tau) #store the values of x1
    wn2 <- rep(0, 6 * n2)
    wn2[1:n2] <- 1 #store the values of x2
    z <- .Fortran("rqfnc", as.integer(n1), as.integer(n2), as.integer(p),
	a1 = as.double(t(as.matrix(x))), c1 = as.double(-y),
	a2 = as.double(t(as.matrix(R))), c2 = as.double(-r),
	rhs = as.double(rhs), d1 = double(n1), d2 = double(n2),
	as.double(u), beta = as.double(beta), eps = as.double(eps),
        wn1 = as.double(wn1), wn2 = as.double(wn2), wp = double((p + 3) * p),
        it.count = integer(3), info = integer(1))
    if (z$info != 0)
        stop(paste("Error info = ", z$info, "in stepy2: singular design"))
    coefficients <- -z$wp[1:p]
    names(coefficients) <- dimnames(x)[[2]]
    residuals <- y - x %*% coefficients
    it.count <- z$it.count
    list(coefficients=coefficients, tau=tau, residuals=residuals, it = it.count)
}
"rq.fit.scad" <-
function (x, y, tau = 0.5, alpha = 3.2, lambda = 1, start = "rq", beta = 0.9995, eps = 1e-06)
{
    n <- length(y)
    p <- ncol(x)
    if (n != nrow(x))
        stop("x and y don't match n")
    if (tau < eps || tau > 1 - eps)
        stop("No parametric Frisch-Newton method.  Set tau in (0,1)")
    if(length(lambda) == 1)
         lambda <- c(0,rep(lambda,p-1))
    if(length(lambda) != p)
          stop(paste("lambda must be either of length ",p," or length one"))
    if(any(lambda < 0))
          stop("negative lambdas disallowed")
    R <- diag(lambda,nrow = length(lambda))
    R <- R[which(lambda != 0),, drop = FALSE]
    r <- rep(0,nrow(R))
    X <- rbind(x,  R)
    Y <- c(y, r)
    N <- length(Y)
    rhs <- (1 - tau) * apply(x, 2, sum) + apply(R,2,sum)
    dscad <- function(x, a = 3.7, lambda = 2){
        lambda *  sign(x) *  (abs(x) <= lambda) +
        sign(x) * (a * lambda -  abs(x)) / (a - 1) *
                (abs(x) <= a * lambda) * (abs(x) > lambda)
        }
    binit <- switch(start,
    	rq =   rq.fit.fnb(x, y, tau = tau)$coef[-1],
    	lasso = rq.fit.lasso(x, y, tau = tau, lambda = lambda)$coef[-1]
	)
    coef <- rep(.Machine$double.xmax,p)
    vscad <- rhs - c(0,dscad(binit) * sign(binit))
    it <- 0
    while(sum(abs(binit - coef[-1])) > eps){
	it <- it + 1
    	d <- rep(1, N)
    	u <- rep(1, N)
    	wn <- rep(0, 10 * N)
    	wn[1:N] <- c(rep((1 - tau),n),rep(.5,nrow(R)))
	vrhs <- rhs - vscad
	binit <- coef[-1]
    	z <- .Fortran("rqfnb", as.integer(N), as.integer(p), a = as.double(t(as.matrix(X))),
        	c = as.double(-Y), vrhs = as.double(vrhs), d = as.double(d),
        	as.double(u), beta = as.double(beta), eps = as.double(eps),
        	wn = as.double(wn), wp = double((p + 3) * p), 
            	it.count = integer(3), info = integer(1))
	coef <- -z$wp[1:p]
	vscad <- c(0,dscad(coef[2:p]) * sign(coef[2:p]))
	}
    if (z$info != 0)
        stop(paste("Error info = ", z$info, "in stepy2: singular design"))
    coefficients <- -z$wp[1:p]
    names(coefficients) <- dimnames(x)[[2]]
    residuals <- y - x %*% coefficients
    it.count <- z$it.count
    list(coefficients=coefficients, residuals=residuals, tau = tau,
		lambda = lambda, it = it.count)
}


"rq.fit.lasso" <-
function (x, y, tau = 0.5, lambda = 1, beta = 0.99995, eps = 1e-06)
{
    n <- length(y)
    p <- ncol(x)
    if (n != nrow(x))
        stop("x and y don't match n")
    if(length(lambda) == 1)
         lambda <- c(0,rep(lambda,p-1))
    if(length(lambda) != p)
          stop(paste("lambda must be either of length ",p," or length one"))
    if(any(lambda < 0))
          stop("negative lambdas disallowed")
    R <- diag(lambda,nrow = length(lambda))
    R <- R[which(lambda != 0),, drop = FALSE]
    r <- rep(0,nrow(R))
    if (tau < eps || tau > 1 - eps)
        stop("No parametric Frisch-Newton method.  Set tau in (0,1)")
    X <- rbind(x, R)
    Y <- c(y, r)
    N <- length(Y)
    rhs <- (1 - tau) * apply(x, 2, sum) + 0.5 * apply(R,2,sum)
    d <- rep(1, N)
    u <- rep(1, N)
    wn <- rep(0, 10 * N)
    wn[1:N] <- 0.5
    z <- .Fortran("rqfnb", as.integer(N), as.integer(p), a = as.double(t(as.matrix(X))),
        c = as.double(-Y), rhs = as.double(rhs), d = as.double(d),
        as.double(u), beta = as.double(beta), eps = as.double(eps),
        wn = as.double(wn), wp = double((p + 3) * p), 
            it.count = integer(3), info = integer(1))
    if (z$info != 0)
        stop(paste("Error info = ", z$info, "in stepy2: singular design"))
    coefficients <- -z$wp[1:p]
    names(coefficients) <- dimnames(x)[[2]]
    residuals <- y - x %*% coefficients
    it.count <- z$it.count
    list(coefficients=coefficients, residuals=residuals, tau = tau,
	lambda = lambda, it = it.count)
}

"rq.fit.pfn" <-
# This is an implementation (purely in R) of the preprocessing phase
# of the rq algorithm described in Portnoy and Koenker, Statistical
# Science, (1997) 279-300.  In this implementation it can be used
# as an alternative method for rq() by specifying method="pfn"
# It should probably be used only on very large problems and then
# only with some caution.  Very large in this context means roughly
# n > 100,000.  The options are described in the paper, and more
# explicitly in the code.  Again, it would be nice perhaps to have
# this recoded in a lower level language, but in fact this doesn't
# seem to make a huge difference in this case since most of the work
# is already done in the rq.fit.fnb calls.
#
function(x, y, tau = 0.5,  Mm.factor = 0.8,
	max.bad.fixups = 3, eps = 1e-6)
{
	#rq function for n large --
	n <- length(y)
	if(nrow(x) != n)
		stop("x and y don't match n")
	if(tau < 0 | tau > 1)
		stop("tau outside (0,1)")
	p <- ncol(x)
	m <- round(sqrt(p) * n^(2/3))
	not.optimal <- TRUE
	ifix = 0
	ibad = 0
	while(not.optimal) {
		ibad = ibad + 1
		if(m < n)
			s <- sample(n, m)
		else {
			z <- rq.fit.fnb(x, y, tau = tau,  eps = eps)
			break
		}
		xx <- x[s,  ]
		yy <- y[s]
		z <- rq.fit.fnb(xx, yy, tau = tau,  eps = eps)
		xxinv <- solve(chol(crossprod(xx)))
		band <- sqrt(((x %*% xxinv)^2) %*% rep(1, p))
		#sqrt(h<-ii)
		r <- y - x %*% z$coef
		M <- Mm.factor * m
		lo.q <- max(1/n, tau - M/(2 * n))
		hi.q <- min(tau + M/(2 * n), (n - 1)/n)
		kappa <- quantile(r/pmax(eps, band), c(lo.q, hi.q))
		sl <- r < band * kappa[1]
		su <- r > band * kappa[2]
		bad.fixups <- 0
		while(not.optimal & (bad.fixups < max.bad.fixups)) {
			ifix = ifix + 1
			xx <- x[!su & !sl,  ]
			yy <- y[!su & !sl]
			if(any(sl)) {
				glob.x <- c(t(x[sl,  , drop = FALSE]) %*% rep(
					1, sum(sl)))
				glob.y <- sum(y[sl])
				xx <- rbind(xx, glob.x)
				yy <- c(yy, glob.y)
			}
			if(any(su)) {
				ghib.x <- c(t(x[su,  , drop = FALSE]) %*% rep(
					1, sum(su)))
				ghib.y <- sum(y[su])
				xx <- rbind(xx, ghib.x)
				yy <- c(yy, ghib.y)
			}
			z <- rq.fit.fnb(xx, yy, tau = tau,  eps = eps)
			b <- z$coef
			r <- y - x %*% b
			su.bad <- (r < 0) & su
			sl.bad <- (r > 0) & sl
			if(any(c(su.bad, sl.bad))) {
				if(sum(su.bad | sl.bad) > 0.1 * M) {
				    # This warning may get annoying?
				    warning("Too many fixups:  doubling m")
				    bad.fixups <- bad.fixups + 1
				    m <- 2 * m
				    break
				}
				su <- su & !su.bad
				sl <- sl & !sl.bad
			}
			else not.optimal <- FALSE
		}
	}
	nit <- c(z$nit,ifix,ibad)
	coefficients <- z$coef
	names(coefficients) <- dimnames(x)[[2]]
	list(coefficients=coefficients, tau=tau, nit = nit)
}

"rq.wfit" <-
function(x, y, tau = 0.5, weights, method = "br",  ...)
{
	if(any(weights < 0))
		stop("negative weights not allowed")
	if(length(tau) > 1) {
	    tau <- tau[1]
	    warning("Multiple taus not allowed in rq.wfit: solution restricted to first element")
	}
	contr <- attr(x, "contrasts")
	wx <- x * weights
	wy <- y * weights
	fit <- switch(method,
		fn = rq.fit.fnb(wx, wy, tau = tau, ...),
		fnb = rq.fit.fnb(wx, wy, tau = tau, ...),
		br = rq.fit.br(wx, wy, tau = tau, ...),
		fnc = rq.fit.fnc(wx, wy, tau = tau, ...),
		sfn = rq.fit.sfn(wx, wy, tau = tau, ...),
		conquer = rq.fit.conquer(wx, wy, tau = tau, ...),
		ppro = rq.fit.ppro(wx, wy, tau = tau, ...),
                pfn = rq.fit.pfn(wx, wy, tau = tau, ...), 
                pfnb = rq.fit.pfnb(wx, wy, tau = tau, ...), {
			what <- paste("rq.fit.", method, sep = "")
			if(exists(what, mode = "function"))
				(get(what, mode = "function"))(x, y, ...)
			else stop(paste("unimplemented method:", method))
		}
		)
        if(length(fit$sol))
            fit$fitted.values <- x %*% fit$sol[-(1:3),]
        else
            fit$fitted.values <- x %*% fit$coef
	fit$residuals <- y - fit$fitted.values
	fit$contrasts <- attr(x, "contrasts")
	fit$weights <- weights
	fit
}

"summary.rqs" <-
function (object, ...) {
        taus <- object$tau
        xsum <- as.list(taus)
	dots <- list(...)
	U <- NULL # Reuse bootstrap randomization
        for(i in 1:length(taus)){
                xi <- object
                xi$coefficients <- xi$coefficients[,i]
                xi$residuals <- xi$residuals[,i]
                xi$tau <- xi$tau[i]
                class(xi) <- "rq"
                xsum[[i]] <- summary(xi, U = U, ...)
                if(i == 1 && length(xsum[[i]]$U)) U <- xsum[[i]]$U 
		if(class(object)[1] == "dynrqs"){
		    class(xsum[[1]]) <- c("summary.dynrq", "summary.rq")
	            if(i == 1) xsum[[1]]$model <- object$model
		    }
                }
        class(xsum) <- "summary.rqs"
	if(class(object)[1] == "dynrqs") 
	    class(xsum) <- c("summary.dynrqs", "summary.rqs")
        xsum
        }
"logLik.rq" <- function(object,  ...){
        n <- length(object$residuals)
        p <- length(object$coefficients)
	pen <- (length(object$lambda) > 0)
	tau <- object$tau
        fid <- object$rho
        val <- n * (log(tau * (1-tau)) - 1 - log(fid/n))
        attr(val,"n") <- n
	if(pen){
	   if(!hasArg(edfThresh)) edfThresh <- 0.0001
           attr(val,"df") <- sum(abs(object$coefficients) > edfThresh)
	  }
	else  attr(val,"df") <- p
        class(val) <- "logLik"
        val
        }
"logLik.rqs" <- function(object, ...){
        n <- nrow(object$residuals)
        p <- nrow(object$coefficients)
	pen <- (length(object$lambda) > 0)
	tau <- object$tau
        fid <- object$rho
        val <- n * (log(tau * (1-tau)) - 1 - log(fid/n))
        attr(val,"n") <- n
	if(pen){
	   if(!hasArg(edfThresh)) edfThresh <- 0.0001
           attr(val,"df") <- apply(abs(object$coefficients) > edfThresh,2,sum)
	  }
	else  attr(val,"df") <- p
        class(val) <- "logLik"
        val
        }
"AIC.rq" <- function(object, ... , k = 2){
        v <- logLik(object)
        if(k <= 0)
                k <- log(attr(v,"n"))
        val <- AIC(v, k = k)
        attr(val,"edf") <- attr(v,"df")
        val
        }
"extractAIC.rq"  <- function(fit, scale, k=2, ...){
aic <- AIC(fit,k)
edf <- attr(aic, "edf")
c(edf, aic)
}

"AIC.rqs" <- function(object, ... , k = 2){
        v <- logLik(object)
        if(k <= 0)
                k <- log(attr(v,"n"))
        val <- AIC(v, k = k)
        attr(val,"edf") <- attr(v,"df")
        val
        }


"summary.rq" <-
# This function provides  methods for summarizing the output of the
# rq command. In this instance, "summarizing" means essentially provision
# of either standard errors, or confidence intervals for the rq coefficents.
# Since the preferred method for confidence intervals is currently the
# rank inversion method available directly from rq() by setting ci=TRUE, with br=TRUE.
# these summary methods are intended primarily for comparison purposes
# and for use on large problems where the parametric programming methods
# of rank inversion are prohibitively memory/time consuming.  Eventually
# iterative versions of rank inversion should be developed that would
# employ the Frisch-Newton approach.
#
# Object is the result of a call to rq(), and the function returns a
# table of coefficients, standard errors, "t-statistics", and p-values, and, if
# covariance=TRUE a structure describing the covariance matrix of the coefficients,
# i.e. the components of the Huber sandwich.
#
# There are five options for "se":
#
#	1.  "rank" strictly speaking this doesn't produce a "standard error"
#		at all instead it produces a coefficient table with confidence
#		intervals for the coefficients based on inversion of the
#		rank test described in GJKP and Koenker (1994).
#	2.  "iid" which presumes that the errors are iid and computes
#		an estimate of the asymptotic covariance matrix as in KB(1978).
#	3.  "nid" which presumes local (in tau) linearity (in x) of the
#		the conditional quantile functions and computes a Huber
#		sandwich estimate using a local estimate of the sparsity.
#	4.  "ker" which uses a kernel estimate of the sandwich as proposed
#		by Powell.
#	5.  "boot" which uses one of several flavors of  bootstrap methods:
#		"xy"	uses xy-pair method
#		"wxy"	uses weighted (generalized) method
#		"pwy"	uses the parzen-wei-ying method
#		"pxy"	uses the preprocessing method
#		"mcmb"	uses the Markov chain marginal bootstrap method
#		"cluster"  uses the Hagemann clustered wild gradient method
#		"conquer"  uses the He et al multiplier bootstrap
#		"BLB"	uses the Bag of Little Bootstraps method
#
#
function (object, se = NULL, covariance = FALSE, hs = TRUE, U = NULL, gamma = 0.7, ...)
{
    if(object$method == "lasso")
         stop("no inference for lasso'd rq fitting: try rqss (if brave, or credulous)")
    if(object$method == "conquer") se = "conquer"
    mt <- terms(object)
    m <- model.frame(object)
    y <- model.response(m)
    dots <- list(...)
    method <- object$method
    if(object$method == "sfn"){
	x <- object$model$x
	vnames <- names(object$coef)
	ctrl <- object$control
    }
    else{
	x <- model.matrix(mt, m, contrasts = object$contrasts)
	vnames <- dimnames(x)[[2]]
    }
    wt <- as.vector(model.weights(object$model))
    tau <- object$tau
    eps <- .Machine$double.eps^(1/2)
    coef <- coefficients(object)
    if (is.matrix(coef))
        coef <- coef[, 1]
    resid <- object$residuals
    n <- length(y)
    p <- length(coef)
    rdf <- n - p
    if (!is.null(wt)) {
        resid <- resid * wt
        x <- x * wt
        y <- y * wt
    }
    if (is.null(se)) {
        if (n < 1001 & covariance == FALSE)
            se <- "rank"
        else se <- "nid"
    }
    if (se == "rank") {
        f <- rq.fit.br(x, y, tau = tau, ci = TRUE, ...)
    }
    if (se == "iid") {
        xxinv <- diag(p)
        xxinv <- backsolve(qr(x)$qr[1:p, 1:p,drop=FALSE], xxinv)
        xxinv <- xxinv %*% t(xxinv)
        pz <- sum(abs(resid) < eps)
        h <- max(p + 1, ceiling(n * bandwidth.rq(tau, n, hs = hs)))
        ir <- (pz + 1):(h + pz + 1)
        ord.resid <- sort(resid[order(abs(resid))][ir])
        xt <- ir/(n - p)
        sparsity <- rq(ord.resid ~ xt)$coef[2]
        cov <- sparsity^2 * xxinv * tau * (1 - tau)
        scale <- 1/sparsity
        serr <- sqrt(diag(cov))
    }
    else if (se == "nid") {
        h <- bandwidth.rq(tau, n, hs = hs)
	while((tau - h < 0) || (tau + h > 1)) h <- h/2
        bhi <- rq.fit(x, y, tau = tau + h, method = method)$coef
        blo <- rq.fit(x, y, tau = tau - h, method = method)$coef
        dyhat <- x %*% (bhi - blo)
        if (any(dyhat <= 0))
            warning(paste(sum(dyhat <= 0), "non-positive fis"))
        f <- pmax(0, (2 * h)/(dyhat - eps))
        fxxinv <- diag(p)
	if(method == "sfn"){
	    D <- t(x) %*% (f * x)
            D <- chol(0.5 * (D + t(D)), nsubmax = ctrl$nsubmax,
                      nnzlmax = ctrl$nnzlmax, tmpmax = ctrl$tmpmax)
            fxxinv <- backsolve(D, fxxinv)
        }
	else{
	    fxxinv <- backsolve(qr(sqrt(f) * x)$qr[1:p, 1:p,drop=FALSE], fxxinv)
	    fxxinv <- fxxinv %*% t(fxxinv)
	}
	xx <- t(x) %*% x
        cov <- tau * (1 - tau) * fxxinv %*% xx %*% fxxinv
        scale <- mean(f)
        serr <- sqrt(diag(cov))
    }
    else if (se == "ker") {
        h <- bandwidth.rq(tau, n, hs = hs)
	while((tau - h < 0) || (tau + h > 1)) h <- h/2
        uhat <- c(y - x %*% coef)
        h <- (qnorm(tau + h) - qnorm(tau - h))*
		min(sqrt(var(uhat)), ( quantile(uhat,.75)- quantile(uhat, .25))/1.34 )
        f <- dnorm(uhat/h)/h
        fxxinv <- diag(p)
        fxxinv <- backsolve(qr(sqrt(f) * x)$qr[1:p, 1:p,drop=FALSE], fxxinv)
        fxxinv <- fxxinv %*% t(fxxinv)
        cov <- tau * (1 - tau) * fxxinv %*% crossprod(x) %*%
            fxxinv
        scale <- mean(f)
        serr <- sqrt(diag(cov))
    }
    else if (se == "boot") {
	if("cluster" %in% names(dots)) {
	    bargs <- modifyList(list(x = x, y = y, tau = tau), dots)
	    if(length(object$na.action)) {
	         cluster <- dots$cluster[-object$na.action]
	         bargs <- modifyList(bargs, list(cluster = cluster))
	    }
	    if(class(bargs$x)[1] == "matrix.csr")
	         bargs <- modifyList(bargs, list(control = ctrl))
	B <- do.call(boot.rq, bargs)
	}
	else
	    B <- boot.rq(x, y, tau, coef = coef, ...)
        cov <- cov(B$B)
        serr <- sqrt(diag(cov))
        }
   else if (se == "BLB"){ # Bag of Little Bootstraps
        n <- length(y)
        b <- ceiling(n^gamma)
        S <- n %/% b
        V <- matrix(sample(1:n, b * S), b, S)
        Z <- matrix(0, NCOL(x), S)
        for(i in 1:S){
            v <- V[,i]
	    B <- boot.rq(x[v,], y[v], tau, bsmethod = "BLB", blbn = n, ...)
            Z[,i] <- sqrt(diag(cov(B$B)))
        }
	cov <- cov(B$B)
        serr <- apply(Z, 1, mean)
    }
   else if(se == "extreme"){
    tau0 <- tau
    if(tau > 0.5) {
	y <- -y
	tau <- 1 - tau
    }
    if(length(dots$mofn)) mofn = dots$mofn
    else mofn = floor(n/5)
    if(length(dots$mofn)) kex = dots$kex
    else kex = 20
    if(length(dots$alpha)) alpha = dots$alpha
    else alpha = 0.1
    if(length(dots$R)) R = dots$R
    else R = 200
    m <- (tau * n + kex)/(tau * n)
    taub <- min(tau * n/mofn, tau + (.5 - tau)/3)
    # This warning is a bit different than Section 1.3.4 of the Handbook chapter
    #if (tau.b.e == tau.e + (.5 - tau.e) / 3 && b >= min(n / 3, 1000))
    #	    warning("tau may be non-extremal results are not likely to differ from central inference");
    xbar <- apply(x,2, mean)
    b0 <- rq.fit(x, y, tau, method = method)$coef
    bm <- rq.fit(x, y, tau = m*tau, method = method)$coef
    An <- (m-1)*tau * sqrt(n/(tau*(1-tau)))/c(crossprod(xbar,bm - b0))
    bt <- rq.fit(x, y, tau=taub, method = method)$coef
    s <- matrix(sample(1:n, mofn * R, replace = T), mofn, R)
    mbe <- (taub * mofn + kex)/(taub * mofn)
    bmbeb <- rq.fit(x, y, tau = mbe*taub, method = method)$coef
    # Accelerated xy bootstrap
    B0 <- boot.rq.pxy(x, y, s, taub, bt, method = method)
    Bm <- boot.rq.pxy(x, y, s, tau = mbe * taub, bmbeb, method = method)
    B <- (mbe - 1) * taub * sqrt(mofn/(taub * (1-taub))) *
	(B0 - b0)/c((Bm - B0) %*% xbar) 
    if (tau0 <= 0.5) {
        bbc <- b0 - apply(B, 2, quantile, .5, na.rm = TRUE)/An
        ciL <- b0 - apply(B, 2, quantile, 1 - alpha/2, na.rm = TRUE)/An
        ciU <- b0 - apply(B, 2, quantile, alpha/2, na.rm = TRUE)/An
    } else {
        bbc <- -(b0 - apply(B, 2, quantile, .5, na.rm = TRUE)/An)
        ciL <- -(b0 - apply(B, 2, quantile, alpha/2, na.rm = TRUE)/An)
        ciU <- -(b0 - apply(B, 2, quantile, 1 - alpha/2, na.rm = TRUE)/An)
    }
    B <- R-sum(is.na(B[,1]))
    coef <- cbind(b0, bbc, ciL, ciU)
    if(tau0 > 0.5) {coef <-  -coef; tau <- tau0}
    dimnames(coef) = list(dimnames(x)[[2]], c("coef", "BCcoef","ciL", "ciU"))
}
   else if(se == "conquer"){
       if(length(dots$R)) R = dots$R
       else R = 200
       Z <- conquer(x[,-1], y, tau, ci = TRUE, B = R)
       #Fixme:  should have option to choose another bsmethod
       coef <- cbind(Z$coef, Z$perCI)
       cnames <- c("coefficients", "lower bd", "upper bd")
       dimnames(coef) <- list(vnames, cnames)
       resid <- y - x %*% Z$coef
   }

    if( se == "rank"){
	coef <- f$coef
	}
    else if(!(se %in% c("conquer", "extreme"))){
    	coef <- array(coef, c(p, 4))
    	dimnames(coef) <- list(vnames, c("Value", "Std. Error", "t value",
             "Pr(>|t|)"))
    	coef[, 2] <- serr
    	coef[, 3] <- coef[, 1]/coef[, 2]
    	coef[, 4] <- if (rdf > 0)
			2 * (1 - pt(abs(coef[, 3]), rdf))
    		     else NA
	}
    object <- object[c("call", "terms")]
    if (covariance == TRUE) {
        if(se != "rank") object$cov <- cov
        if(se == "iid") object$scale <- scale
        if(se %in% c("nid", "ker")) {
            object$Hinv <- fxxinv
            object$J <- crossprod(x)
            object$scale <- scale
        }
        else if (se == "boot") {
            object$B <- B$B
	    object$U <- B$U
        }
    }
    object$coefficients <- coef
    object$residuals <- resid
    object$rdf <- rdf
    object$tau <- tau
    class(object) <- "summary.rq"
    object
}

akj <- function(x,
                z = seq(min(x), max(x), length = 2 * length(x)),
                p = rep(1/length(x), length(x)),
                h = -1, alpha = 0.5, kappa = 0.9, iker1 = 0)
{
    nx <- length(x)
    stopifnot(is.numeric(x),
              length(p) == nx,
	      any((iker1 <- as.integer(iker1)) == 0:1))
    nz <- length(z)
    if(is.unsorted(x))
	x <- sort(x)
    .Fortran("sakj",
	     as.double(x),
	     as.double(z),
	     as.double(p),
	     iker1,
	     dens  = double(nz),
	     psi   = double(nz),
	     score = double(nz),
	     as.integer(nx),
	     as.integer(nz),
	     h = as.double(h),
	     as.double(alpha),
	     as.double(kappa),
	     double(nx))[c("dens", "psi", "score", "h")]
}

"lm.fit.recursive" <-
function(X, y, int = TRUE)
{
	if(int)
		X <- cbind(1, X)
	p <- ncol(X)
	n <- nrow(X)
	D <- qr(X[1:p,  ])
	A <- qr.coef(D, diag(p))
	A[is.na(A)] <- 0
	A <- crossprod(t(A))
	Ax <- rep(0, p)
	b <- matrix(0, p, n)
	b[, p] <- qr.coef(D, y[1:p])
	b[is.na(b)] <- 0
	z <- .Fortran( "rls",
		as.integer(n),
		as.integer(p),
		as.double(t(X)),
		as.double(y),
		b = as.double(b),
		as.double(A),
		as.double(Ax))
	bhat <- matrix(z$b, p, n)
	return(bhat)
}

"rq.fit.hogg" <-
function (x, y, taus = c(.1,.3,.5), weights = c(.7,.2,.1),  
	R= NULL, r = NULL, beta = 0.99995, eps = 1e-06) 
{
    n <- length(y)
    n2 <- NROW(R)
    m <- length(taus)
    p <- ncol(x)+m
    if (n != nrow(x)) 
        stop("x and y don't match n")
    if (m != length(weights)) 
        stop("taus and weights differ in length")
    if (any(taus < eps) || any(taus > 1 - eps)) 
        stop("taus outside (0,1)")
    W <- diag(weights)
    if(m == 1) W <- weights
    x <- as.matrix(x)
    X <- cbind(kronecker(W,rep(1,n)),kronecker(weights,x))
    y <- kronecker(weights,y)
    rhs <- c(weights*(1 - taus)*n, sum(weights*(1-taus)) * apply(x, 2, sum))
    if(n2!=length(r))
	stop("R and r of incompatible dimension")
    if(!is.null(R))
	if(ncol(R)!=p)
	    stop("R and X of incompatible dimension")
    d <- rep(1, m*n)
    u <- rep(1, m*n)
    if(length(r)){
       wn1 <- rep(0, 10 * m*n)
       wn1[1:(m*n)] <- .5
       wn2 <- rep(0,6*n2)
       wn2[1:n2] <- 1 
       z <- .Fortran("rqfnc", as.integer(m*n), as.integer(n2), as.integer(p), 
           a1 = as.double(t(as.matrix(X))), c1 = as.double(-y), 
           a2 = as.double(t(as.matrix(R))), c2 = as.double(-r), 
           rhs = as.double(rhs), d1 = double(m*n), d2 = double(n2), 
           as.double(u), beta = as.double(beta), eps = as.double(eps), 
           wn1 = as.double(wn1), wn2 = as.double(wn2), wp = double((p + 3) * p), 
	   it.count = integer(3), info = integer(1))
	}
    else{
	wn <- rep(0, 10 * m*n)
    	wn[1:(m*n)] <- .5
    	z <- .Fortran("rqfnb", as.integer(m*n), as.integer(p), a = as.double(t(as.matrix(X))), 
		c = as.double(-y), rhs = as.double(rhs), d = as.double(d), as.double(u), 
		beta = as.double(beta), eps = as.double(eps), wn = as.double(wn), 
		wp = double((p + 3) * p), it.count = integer(2), info = integer(1))
	}
    if (z$info != 0) 
        warning(paste("Info = ", z$info, "in stepy: singular design: iterations ", z$it.count[1]))
    coefficients <- -z$wp[1:p]
    if(any(is.na(coefficients)))stop("NA coefs:  infeasible problem?")
    list(coefficients = coefficients, nit = z$it.count, flag = z$info)
}
#preprocessing for the QR process
"rq.fit.ppro" <- function (x, y, tau, weights=NULL, Mm.factor = 0.8, eps = 1e-06, ...) 
{
  ntau <- length(tau)
  n <- length(y)
  if (nrow(x) != n) stop("x and y don't match n")
  p <- ncol(x)
  m <- n * sqrt(p) * max(diff(tau)) # check this length(tau) == 1 case?
  dots <- list(...)
  method <- ifelse(length(dots$pmethod), dots$pmethod, "fn")
  if(length(weights)){
      if(any(weights < 0))
	 stop("negative weights not allowed")
      if(length(weights) != n)
	 stop("weights not of length(y)")
      else {
         x <- x * weights
         y <- y * weights
      }
  }
  coef <- matrix(NA, p, ntau)
  resid <- matrix(NA, n, ntau)
  rho <- rep(0, ntau)
  Rho <- function(u, tau) u * (tau - (u < 0))
  z <- rq.fit(x, y, tau=tau[1], method = method)
  r <- z$resid
  coef[,1] <- z$coef
  rho[1] <- sum(Rho(z$residuals, tau[1]))
  xxinv <- solve(chol(crossprod(x)))
  # Is pmax really necessary here?
  band <- pmax(eps, sqrt(((x %*% xxinv)^2) %*% rep(1, p)))
  for(i in 2:ntau){
    not.optimal <- TRUE
    mm <- m
    while (not.optimal) {
      M <- Mm.factor * mm
      lo.q <- max(1/n, tau[i] - M/(2 * n))
      hi.q <- min(tau[i] + M/(2 * n), (n - 1)/n)
      kappa <- quantile(r/band, c(lo.q, hi.q))
      sl <- r < band * kappa[1]
      su <- r > band * kappa[2]
      while (not.optimal) {
        xx <- x[!su & !sl, ]
        yy <- y[!su & !sl]
        if (any(sl)) {
          glob.x <- c(t(x[sl, , drop = FALSE]) %*% rep(1, sum(sl)))
          glob.y <- sum(y[sl])
          xx <- rbind(xx, glob.x)
          yy <- c(yy, glob.y)
        }
        if (any(su)) {
          ghib.x <- c(t(x[su, , drop = FALSE]) %*% rep(1, sum(su)))
          ghib.y <- sum(y[su])
          xx <- rbind(xx, ghib.x)
          yy <- c(yy, ghib.y)
        }
        z <- rq.fit(xx, yy, tau = tau[i], method = method)
        b <- z$coef
        r <- y - x %*% b
        su.bad <- (r < 0) & su
        sl.bad <- (r > 0) & sl
        bad.signs <- sum(su.bad | sl.bad)
        if (bad.signs > 0) {
          if (bad.signs > 0.1 * M) {
            mm <- 2*mm
            warning("Too many fixups:  doubling m")
            break
          }
          su <- su & !su.bad
          sl <- sl & !sl.bad
        }
        else not.optimal <- FALSE
      }
    }
    coef[,i] <- b
    resid[,i] <- y - x %*% b
    rho[i] <- sum(Rho(resid, tau[i]))
  }
  dimnames(coef) <- list(dimnames(x)[[2]], tau)
  list(coefficients=coef, residuals = resid, rho=rho, weights = weights)
}

# R function for fnb call for multiple taus
rq.fit.qfnb <- function(x,y,tau){
    n <- nrow(x)
    p <- ncol(x)
    m <- length(tau)
    d <- rep(1, n)
    u <- rep(1, n)
    z <- .Fortran("qfnb",
		  n = as.integer(n),
		  p = as.integer(p),
		  m = as.integer(m),
		  a = as.double(t(x)),
		  y = as.double(-y),
		  t = as.double(tau),
		  r = double(p),
		  d = as.double(d),
		  u = as.double(u),
		  wn = double(n*9),
		  wp = double(p*(p+3)),
		  B = double(p*m),
		  nit = integer(3),
		  info = integer(1))
    if(z$info != 0)
	warning(paste("Info = ", z$info, "in stepy: singular design: nit = ", z$nit[1]))
    coefficients <- matrix(-z$B, p, m)
    dimnames(coefficients) <- list(dimnames(x)[[2]],paste("tau = ",tau))
    list(coefficients = coefficients, nit = z$nit, flag = z$info)
}
rq.fit.pfnb <- function (x, y, tau, m0 = NULL, eps = 1e-06) {
    m <- length(tau)
    n <- length(y)
    if(!is.matrix(x)) dim(x) <- c(n,1) 
    if (nrow(x) != n) 
        stop("x and y don't match n")
    p <- ncol(x)
    if(!length(m0))
	m0 <- floor(n^(2/3) * sqrt(p)) # Needs testing!
    s <- sample(n,m0)
    xs <- x[s,,drop = FALSE]
    ys <- y[s]
    z <- rq.fit(xs, ys, tau = tau[1], method = "fn")
    r <- y - x %*% z$coef
    b <- matrix(0,p,m)
    nit <- matrix(0,5,m)
    xxinv <- solve(chol(crossprod(xs)))
    band <- pmax(eps, sqrt(((x %*% xxinv)^2) %*% rep(1, p)))
    z <- .Fortran("pfnb",
		  as.integer(n),
		  as.integer(p),
		  as.integer(m),
		  as.double(t(x)),
		  as.double(y),
		  as.double(tau),
		  as.double(r),
		  b = as.double(-b),
		  as.double(band),
		  as.integer(m0),
		  double(n),
		  double(n),
		  double(n*9),
		  double(p*(p+3)),
		  double(p*n),
		  double(n),
		  integer(n),
		  integer(n),
		  double(p),
		  double(p),
		  double(p),
		  nit = as.integer(nit),
		  info = integer(m))
    coefficients <- matrix(-z$b,p,m)
    nit <- matrix(z$nit,5,m)
    dimnames(coefficients) <- list(dimnames(x)[[2]],paste("tau = ",tau))
    list(coefficients = coefficients, nit = nit, flag = z$info)
}

subroutine sakj(x,z,p,iker,dens,psi,score,nx,nz,h,alpha,kappa,xlam)
# univariate kernel density-score estimator
# the algorithm is basically from Silverman as adapted for Portnoy and Koenker
# Annals paper on adaptive L-estimation in regression.

# x--pts used for centers of kernel assumed to be sorted!!!!
# z--pts at which density is calculated
# p--probability associated with x's
# dens--f(z), the densities at z
# psi--f'(z)/f(z) the score at z
# score--(log(f(z)))'', the J fn at z
# nx--number of pts in x
# nz--number of pts in z
# iker--kernel
#          0=gaussian
#	   1=cauchy
# h--initial window size (overall)--choose zero for default
# kappa--constant determining initial (default) window width
# xlam--Silverman's lambda, window adjustment for each x

double precision dens(nz),score(nz),psi(nz),h,kappa,con1
double precision z(nz),x(nx),xlam(nx),p(nx),qrange,pi,four
double precision con1,sum,sqsum,xsd,a,fifth,hinv,half,g,ginv
double precision xn,xker,dxker,ddxker,fact,xponen,alpha,glog,zero,one,two
parameter( zero = 0.d0)
parameter( one = 1.d0)
parameter( two = 2.d0)
parameter( four = 4.d0)
parameter( half = 0.5d0)
parameter( fifth = 0.2d0)
parameter( pi = 3.141593d0)
xn=nx

# call srtad(x,1,nx) # port sort routine now done in S interface.
if(iker==0) con1=one/sqrt(2.0*pi)
else if(iker==1) con1=one/pi

# if no h is provided, calculate a default
if(h<=0.) {
	sum=0.
	sqsum=0.
	do i=1,nx
		{ sqsum=sqsum+x(i)*x(i)*p(i)
		sum=sum+x(i)*p(i)
		}
	xsd=dsqrt(sqsum-sum*sum)
	sum=zero
	for(i=1;i<nx;i=i+1) {
		sum=sum+p(i)
		if(sum<.25) next
		else { qrange=x(i);break }
		}
	sum=one
	for(i=nx;i>0;i=i-1) {
		sum=sum-p(i)
		if(sum>.75) next
		else { qrange=x(i)-qrange;break }
		}
	a=min(xsd,qrange/1.34)
	h=kappa*a/(xn**fifth) # see Silverman p 48
	}
hinv=one/h
# Stage one:  compute pilot estimate of density
	do j=1,nx {
		xker=0.
		if(iker==0) {
			do i=1,nx {
				xponen=(x(j)-x(i))*hinv
				xponen=half*xponen**2
				xker=xker+p(i)*exp(-xponen)*hinv
				}
			}
		else if(iker==1) {
			do i=1,nx {
				xponen=(x(j)-x(i))*hinv
				xker=xker+p(i)*hinv/(1+xponen**2)
				}
			}
		xlam(j)=con1*xker
		}
# Stage two:  Automatic window widths (Silverman p101)
	glog=zero
	do i=1,nx
		{ glog=glog+p(i)*log(xlam(i))}
	g=exp(glog)
	ginv=one/g
	do i=1,nx {
            xlam(i)=hinv/((xlam(i)*ginv)**(-alpha))
        }
# notice xlam no longer its own self at this pt! xlam is 1/(h*lambda(i))
# substitution of * for / thus achieved speeds things up

# Stage two:  new density-score estimates
	do j=1,nz {
		xker=zero
		dxker=zero
		ddxker=zero
		if(iker==0) {  # gaussian kernel
                    do i=1,nx {
                        xponen=(z(j)-x(i))*xlam(i)
                        fact=exp(-half*xponen*xponen)*xlam(i)
                        xker=xker+p(i)*fact
                        dxker=dxker-p(i)*fact*xponen*xlam(i)
                        ddxker=ddxker-
                            p(i)*fact*(one - xponen**2)*xlam(i)**2
                    }
                }
		else if(iker==1) {  # cauchy kernel
                    do i=1,nx {
                        xponen=(z(j)-x(i))*xlam(i)
                        fact=xlam(i)/(one+xponen**2)
                        xker=xker+p(i)*fact
                        dxker=dxker-p(i)*two*xponen*fact**2
                        ddxker=ddxker-
                            p(i)*two*(fact**2)*(xlam(i)-
                                                four*(xponen**2)*fact)
                    }
                }
		dens(j)=con1*xker
		psi(j)=-(dxker/xker)
		score(j)=(dxker/xker)**2-ddxker/xker
		}
return
end

C Output from Public domain Ratfor, version 1.0
      subroutine spwy(n,m,k,nnza,a,ja,ia,ao,jao,iao,nnzdmax,d,jd,id, dsu
     *b,jdsub,nnzemax,e,je,ie,nsubmax,lindx,xlindx, nnzlmax,lnz,xlnz,iw,
     *iwmax,iwork,xsuper,tmpmax, tmpvec, wwm,wwn,cachsz,level,x,s,u,c,y,
     *b,small, ierr,maxit,time)
      integer nnza,m,n,nnzdmax,nnzemax,iwmax, nnzlmax,nsubmax,cachsz,lev
     *el,tmpmax,ierr,maxit,time(7)
      ja(nnza),jao(nnza),jdsub(nnzemax+1),jd(nnzdmax), ia(n+1),iao(m+1),
     *id(m+1),lindx(nsubmax),xlindx(m+1), iw(m,5),xlnz(m+1),iwork(iwmax)
     *,xsuper(m+1),je(nnzemax),ie(m+1)
      double precision small, a(nnza),ao(nnza),dsub(nnzemax+1),d(nnzdmax
     *), lnz(nnzlmax),c(n),y(m),wwm(m,3),tmpvec(tmpmax), wwn(n,14),x(n),
     *s(n),u(n),e(nnzemax),b(m,k)
      do23000 i=1,k
      call dcopy(m,b(1,i),1,a(nnza - m),1)
      call slpfn(n,m,nnza,a,ja,ia,ao,jao,iao,nnzdmax,d,jd,id, dsub,jdsub
     *,nsubmax,lindx,xlindx,nnzlmax,lnz, xlnz,iw(1,1),iw(1,2),iwmax,iwor
     *k,iw(1,3),iw(1,4), xsuper,iw(1,5),tmpmax,tmpvec,wwm(1,2),cachsz, l
     *evel,x,s,u,c,y,b,wwn(1,1),wwn(1,2),wwn(1,3), wwn(1,4),nnzemax,e,je
     *,ie,wwm(1,3),wwn(1,5),wwn(1,6), wwn(1,7),wwn(1,8),wwn(1,9),wwn(1,1
     *0),wwn(1,11), wwn(1,12),wwn(1,13),wwn(1,14),wwm(1,1),small,ierr,ma
     *xit, time)
      call dcopy(m,y,1,b(1,i),1)
23000 continue
23001 continue
      return
      end

C Output from Public domain Ratfor, version 1.05
      subroutine grexp(n, x, a)
      integer i,n
      double precision x(n),a
      call fseedi()
      do23000 i = 1,n
      call frexp(x(i), a)
23000 continue
23001 continue
      call fseedo()
      return
      end

"anova.rq" <-
function (object, ..., test = "Wald", joint = TRUE,
       score = "tau", se = "nid", iid = TRUE, R = 200, trim = NULL)

{
    if (length(list(object, ...)) > 1) {
	objects <- list(object, ...)
        return(anova.rqlist(objects, ..., test = test, joint = joint,
             score = score, se = se, iid = iid, R = R, trim = trim))
    }
    stop("Anova is only defined (yet) for lists of rq objects")
}
anova.rqs <- function(object, ..., se = "nid", iid = TRUE, joint = TRUE){
    class(object) <- "rq"
    m <- length(object$tau)
    z <- rep(list(object), m)
    for(i in 1:m){
        z[[i]]$coefficients <- object$coefficients[,i]
        z[[i]]$tau <- object$tau[i]
        z[[i]]$rho <- object$rho[i]
    }
    return(anova.rqlist(z, ..., se = se, iid = iid, joint = joint))
}

"anova.rqlist" <-
function (object, ..., test = "Wald", joint = TRUE, 
		score = "tau", se = "nid", iid = TRUE, R = 200, trim = NULL) 
{
    objects <- object
    responses <- as.character(lapply(objects, function(x) formula(x)[[2]]))
    sameresp <- responses == responses[1]
    if (!all(sameresp)) 
        stop("Models don't all have the same response variable")
    n <- length(objects[[1]]$resid)
    models <- as.character(lapply(objects, function(x) formula(x)))
    nobjects <- length(objects)
    dimp <- lapply(objects, function(x) length(coef(x)))
    objects <- objects[order(-unlist(dimp))]
    mf <- model.frame(objects[[1]])
    models <- as.character(lapply(objects, function(x) formula(x)))
    taus <- unlist(lapply(objects, function(x) x$tau))
    if(is.matrix(coef(objects[[1]])))
    	names <- lapply(objects, function(x) dimnames(coef(x))[[1]])
    else
    	names <- lapply(objects, function(x) names(coef(x)))
    if (test == "Wald") 
        objects <- lapply(objects, function(x) summary(x,se=se,R=R,covariance = TRUE))
    sametaus <- taus == taus[[1]]
    if (all(sametaus)) {
        Tn <- rep(0, nobjects - 1)
        ndf <- Tn
        ddf <- Tn
        pvalue <- Tn
        topnote <- paste("Model ", format(1:nobjects), ": ", 
            models, sep = "", collapse = "\n")
        if (test == "anowar") {
            X1 <- model.matrix(objects[[1]],mf,contrasts=objects[[1]]$contrasts)
            y <- model.response(mf)
	    weights <- as.vector(model.weights(mf))
            tau <- taus[[1]]
            for (i in 2:nobjects) {
                if (!all(names[[i]] %in% names[[1]])) 
                  stop("Models aren't nested")
                mf <- model.frame(objects[[i]])
                X0 <- model.matrix(objects[[i]], mf,contrasts=objects[[i]]$contrasts)
                Htest <- rq.test.anowar(X0, X1, y, tau = tau, R = R)
                ndf[i - 1] <- Htest$ndf
                Tn[i - 1] <- Htest$Tn
                ddf[i - 1] <- Htest$ddf
                pvalue[i - 1] <- Htest$pvalue
            }
    	table <- data.frame(ndf, ddf, Tn, pvalue)
        }
        else if (test == "rank") {
            x1 <- model.matrix(objects[[1]],mf,contrasts=objects[[1]]$contrasts)
            y <- model.response(mf)
	    weights <- as.vector(model.weights(mf))
            for (i in 2:nobjects) {
                if (!all(names[[i]] %in% names[[1]])) 
                  stop("Models aren't nested")
                nullH <- is.na(match(names[[1]], names[[i]]))
		X1 <- as.matrix(x1[, nullH])
                mf <- model.frame(objects[[i]])
                X0 <- model.matrix(objects[[i]], mf,contrasts=objects[[i]]$contrasts)
		if(score == "tau") tau <- taus[[1]]
                Htest <- rq.test.rank(X0, X1, y, score = score, weights = weights, 
			iid = iid, tau = tau, trim = trim)
                ndf[i - 1] <- Htest$ndf
                Tn[i - 1] <- Htest$Tn
                ddf[i - 1] <- Htest$ddf
                pvalue[i - 1] <- Htest$pvalue
            }
    	table <- data.frame(ndf, ddf, Tn, pvalue)
        }
        else if (test == "Wald") {
            V <- lapply(objects, function(x) x$cov)
            coef <- lapply(objects, function(x) coef(x)[,1])
            for (i in 2:nobjects) {
                if (!all(names[[i]] %in% names[[1]])) 
                  stop("Models aren't nested")
                nullH <- is.na(match(names[[1]], names[[i]]))
                ndf[i - 1] <- sum(nullH)
                Tn[i - 1] <- t((coef[[1]])[nullH]) %*% solve((V[[1]])[nullH, 
                  nullH], (coef[[1]])[nullH])/ndf[i - 1]
                ddf[i - 1] <- n - length(names[[1]])
                pvalue[i - 1] <- 1 - pf(Tn[i - 1], ndf[i - 1], 
                  ddf[i - 1])
            }
    	table <- data.frame(ndf, ddf, Tn, pvalue)
        }
        else stop("test only defined for anowar, Wald and rank")
    }
    else {
        m <- length(taus)
	n <- NROW(objects[[1]]$residuals)
        for (i in 2:m) {
            if (!setequal(names[[i]], names[[1]])) 
                stop("Models with common tau don't have same X")
        }
        if (names[[1]][1] != "(Intercept)") 
            stop("Intercept required in common tau testing")
        Omega <- outer(taus, taus, pmin) - outer(taus, taus)
        J <- objects[[1]]$J
        p <- dim(J)[1]
        H <- array(unlist(lapply(objects, function(x) x$Hinv)), 
            c(p, p, m))
        H <- matrix(aperm(H, c(1, 3, 2)), p * m, p) %*% t(chol(J))
        W <- (H %*% t(H)) * (kronecker(Omega, outer(rep(1, p), 
            rep(1, p))))
        coef <- unlist(lapply(objects, function(x) coef(x)[,1]))
	if(joint){
        	D <- kronecker(diff(diag(m)), cbind(0, diag(p - 1)))
        	ndf <- (p - 1) * (m - 1)
        	Tn <- t(D %*% coef) %*% solve(D %*% W %*% t(D), D %*% coef)/ndf
        	ddf <- n * m - (p-1) * (m - 1)
        	pvalue <- 1 - pf(Tn, ndf, ddf)
        	nobjects <- 1
        	tnote1 <- paste("Model: ", models[[1]], "\n", sep = "")
        	tnote2 <- paste("Joint Test of Equality of Slopes: tau in { ", 
        	    	paste(taus, collapse = " "), " }\n")
        	topnote <- paste(tnote1, tnote2, sep = "")
    		table <- data.frame(ndf, ddf, Tn, pvalue)
		}
	else{
	   Tn <- pvalue <- rep(0,p-1)
	   ndf <- m-1
	   ddf <- n*m - (m-1)
	   for(i in 2:p){
		E <- matrix(0, 1, p)
		E[1,i] <- 1
		D <- kronecker(diff(diag(m)),E)
		Tn[i-1] <-  t(D %*% coef) %*% solve(D %*% W %*% t(D), D %*% coef)/ndf
        	pvalue[i-1] <- 1 - pf(Tn[i-1], ndf, ddf)
		}
      	   tnote1 <- paste("Model: ", models[[1]], "\n", sep = "")
       	   tnote2 <- paste("Tests of Equality of Distinct Slopes: tau in { ", 
            	paste(taus, collapse = " "), " }\n")
       	   topnote <- paste(tnote1, tnote2, sep = "")
    	   table <- data.frame(ndf, ddf, Tn, pvalue)
	   dimnames(table)[[1]] <- names[[1]][2:p]
          }
    }
    x <- list(table=table,topnote=topnote)
    class(x) <- "anova.rq"
    return(x)
}
"print.anova.rq" <-
function(x,...){
    table <- x$table
    topnote <- x$topnote
    dimnames(table)[[2]] <- c("Df", "Resid Df", "F value", "Pr(>F)")
    title <- "Quantile Regression Analysis of Deviance Table\n"
    a <- structure(table, heading = c(title, topnote), class = c("anova", 
        "data.frame"))
    print(a)
}
rq.test.anowar <- function(x0,x1,y,tau,R){
        if(!requireNamespace("logspline", quietly = TRUE))
	    stop("anowar test requires logspline package")
        n <- length(y)
        f0 <- rq(y ~ x0 - 1, tau = tau)
        f1 <- rq(y ~ x1 - 1, tau = tau)
        Rho <- function(u,tau) u * (tau - (u < 0))
        Mn <- sum(Rho(f0$resid,tau)) - sum(Rho(f1$resid,tau))
	if(!hasArg(R)) R <- 200
        W  <- matrix(rexp(n*R,1),n,R)
        B1 <- t(boot.rq.wxy(x1,y,W,tau))
        B0 <- t(boot.rq.wxy(x0,y,W,tau))
        R0 <- Rho(y - x0 %*% B0,tau)
        R1 <- Rho(y - x1 %*% B1,tau)
        W0 <- apply(R0*W, 2, sum) - apply(W * Rho(f0$resid,tau), 2, sum)
        W1 <- apply(R1*W, 2, sum) - apply(W * Rho(f1$resid,tau), 2, sum)
        RefDistn <- logspline::logspline(W0 - W1)
        pvalue <- 1 - logspline::plogspline(Mn,RefDistn)
        ndf <- ncol(x1) - ncol(x0)
        ddf <- n - ncol(x1)
        list(Tn = Mn, ndf=ndf, ddf=ddf, pvalue=pvalue)
        }
"rq.test.rank" <-
function (x0, x1, y, v = NULL, score = "wilcoxon", weights = NULL, 
    tau = 0.5, iid = TRUE, delta0 = rep(0, NCOL(x1)), omega = 1, trim = NULL, 
    pvalue = "F") 
{
    if (length(weights) > 0) {
        y <- weights * y
        x0 <- weights * x0
        x1 <- weights * x1
    }
    n <- length(y)
    if (!length(v) > 0) 
        if(score == "tau"){
           r <- rq.fit.br(x0,y,tau = tau)$dual - (1 - tau)
           r <- list(ranks = r, A2 =  tau * (1 - tau))
           }
        else{
           v <- rq(y ~ x0 - 1, tau = -1)
           r <- ranks(v, score, tau, trim)
           }
    else 
        r <- ranks(v, score, tau, trim)
    if(iid == FALSE && score == "tau"){
         h <- bandwidth.rq(tau, n, hs = TRUE)
         bhi <- rq.fit.br(x0, y, tau + h, ci = FALSE)
         bhi <- coefficients(bhi)
         blo <- rq.fit.br(x0, y, tau - h, ci = FALSE)
         blo <- coefficients(blo)
         dyhat <- x0 %*% (bhi - blo)
         if(any(dyhat <= 0)) {
              pfis <- (100 * sum(dyhat <= 0))/n
              warning(paste(pfis, "percent fis <=0"))
              }
         eps <- .Machine$double.eps^(2/3)
         f <- pmax(eps, (2 * h)/(dyhat - eps))
         x1hat <- resid(lm.wfit(x0, x1, w = f))
        }
    else
        x1hat <- as.matrix(qr.resid(qr(x0), x1))
    Tn <- as.matrix(t(x1hat) %*% r$ranks)
    Tn <- t(Tn) %*% solve(crossprod(x1hat)) %*% Tn/r$A2
    ncp <- (omega^2) * t(delta0) %*% (crossprod(x1hat)/n) %*% 
        delta0/r$A2
    ndf <- NCOL(x1)
    Tn <- Tn/ndf
    ddf <- length(y) - NCOL(x0) - NCOL(x1)
    if (pvalue == "F") 
        pvalue <- 1 - pf(Tn, ndf, ddf, ncp)
    else 
        pvalue <- 1 - pchisq(Tn*ndf, ndf, ncp)
    list(Tn = Tn, ndf = ndf, ddf = ddf, pvalue = pvalue)
}
"ranks" <- 
function (v, score = "wilcoxon", tau = 0.5, trim = NULL) 
{
    A2 <- 1
    if(length(trim) & !(score == "wilcoxon"))
        stop("trimming is only permitted with the wilcoxon score functions") 
    if (score == "wilcoxon") {
        phibar <- 1/2
        A2 <- 1/12
        u <- v$sol[1,]
        J <- length(u)
        d <- v$dsol
        if(length(trim)){
             if(length(trim) == 1) trim <- c(trim, 1 - trim)
             else if(length(trim) > 2) stop("Only 2 trimming proportions allowed")
             else {
                if(any(trim < 0) || any(trim > 1)) stop("trim must lie in [0,1]")
                trim <- sort(trim)
                klo <- findInterval(trim[1],u)
                khi <- findInterval(trim[2],u)
                wlo <- (trim[1] - u[klo])/(u[klo+1] - u[klo])
                dlo <- wlo * d[,klo] + (1-wlo) * d[,klo + 1]
                s <-  (klo+1):khi
                s <-  (klo+1):khi
                if(khi < J) {
                   whi <- (trim[2] - u[khi])/(u[khi+1] - u[khi])
                   dhi <- whi * d[,khi] + (1-whi) * d[,khi + 1]
                   u <- c(trim[1], u[s], trim[2])
                   d <- cbind(dlo, d[,s], dhi)
                   }
                else {
                   u <- c(trim[1],u[s])
                   d <- cbind(dlo,  d[,s])
                   }
                a <- trim[1]; b <- trim[2]
                phibar <- c <- ( a^2 + 2*b - b^2)/2
                A2 <- (2/3) *(a^3 - b^3) - c * a^2 - 2 * b * c + c^2 + (1+c) * b^2 
                }
            }
        D <- v$dsol
        J <- length(u)
        ranks <- c((0.5 * (d[, 2:J] + d[, 1:(J - 1)]) %*% diff(u)) - phibar)
        return(list(ranks = ranks, A2 = A2))
    }
    else if (score == "normal") {
        J <- ncol(v$sol)
        dt <- v$sol[1, 2:J] - v$sol[1, 1:(J - 1)]
        dphi <- c(0, dnorm(qnorm(v$sol[1, 2:(J - 1)])), 0)
        dphi <- diff(dphi)
        ranks <- as.vector((((v$dsol[, 2:J] - v$dsol[, 1:(J - 
            1)]))) %*% (dphi/dt))
        return(list(ranks = ranks, A2 = A2))
    }
    else if (score == "sign") {
        j.5 <- sum(v$sol[1, ] < 0.5)
        w <- (0.5 - v$sol[1, j.5])/(v$sol[1, j.5 + 1] - v$sol[1, 
            j.5])
        r <- w * v$dsol[, j.5 + 1] + (1 - w) * v$dsol[, j.5]
        ranks <- 2 * r - 1
        return(list(ranks = ranks, A2 = A2))
    }
    else if (score == "tau") {
        j.tau <- sum(v$sol[1, ] < tau)
        w <- (tau - v$sol[1, j.tau])/(v$sol[1, j.tau + 1] - v$sol[1, 
            j.tau])
        r <- w * v$dsol[, j.tau + 1] + (1 - w) * v$dsol[, j.tau]
        ranks <- r - (1 - tau)
        A2 <- tau * (1 - tau)
        return(list(ranks = ranks, A2 = A2))
    }
    else if (score == "normalscale") {
	J <- ncol(v$sol)
	taus <- v$sol[1,]
        dt <- taus[2:J] - taus[1:(J - 1)]
	Qt <- qnorm(taus[2:(J-1)])
        phi <- c(0, -dnorm(Qt)*Qt, 0)
        dphi <- diff(phi)
        ranks <- as.vector((((v$dsol[, 2:J] - v$dsol[, 1:(J -
            1)]))) %*% (dphi/dt))
        return(list(ranks = ranks, A2 = 2))
	}
    else if (score == "halfnormalscale") {
	J <- ncol(v$sol)
	taus <- v$sol[1,]
        dt <- taus[2:J] - taus[1:(J - 1)]
	Qt <- qnorm(taus[2:(J-1)])
        phi <- c(0, (taus[2:(J-1)] - dnorm(Qt)*Qt)*(taus[2:(J-1)] > .5), 1)
        dphi <- diff(phi)
	dphi[dphi > .5] <- 0 #Kludge to zap jump point.
        ranks <- (((v$dsol[, 2:J] - v$dsol[, 1:(J-1)]))) 
	ranks <- as.vector(ranks %*% (dphi/dt))
        return(list(ranks = ranks, A2 = 1.25))
	}
    else if(score == "lehmann"){
	J <- ncol(v$sol)
        taus <- v$sol[1,]
        dt <- taus[2:J] - taus[1:(J - 1)]
	taus <- taus[2:(J-1)]
	phi <- c(0, -(taus -1)*log(1-taus),0)
	dphi <- diff(phi)
        ranks <- (((v$dsol[, 2:J] - v$dsol[, 1:(J-1)])))
        ranks <- as.vector(ranks %*% (dphi/dt))
        return(list(ranks = ranks, A2 = 1))
	}
    else if (score == "interquartile") {
        j.25 <- sum(v$sol[1, ] < 0.25)
        w <- (0.25 - v$sol[1, j.25])/(v$sol[1, j.25 + 1] - v$sol[1, 
            j.25])
        r.25 <- w * v$dsol[, j.25 + 1] + (1 - w) * v$dsol[, j.25]
        j.75 <- sum(v$sol[1, ] < 0.75)
        w <- (0.75 - v$sol[1, j.75])/(v$sol[1, j.75 + 1] - v$sol[1, 
            j.75])
        r.75 <- w * v$dsol[, j.75 + 1] + (1 - w) * v$dsol[, j.75]
        ranks <- 0.5 + r.75 - r.25
        A2 <- 1/4
        return(list(ranks = ranks, A2 = A2))
    }
    else stop("invalid score function")
}

C Output from Public domain Ratfor, version 1.05
      subroutine rls(n,p,x,y,b,a,ax)
      integer n,p
      double precision x(p,n),y(n),b(p,n),a(p,p),ax(p)
      double precision zero,one,mone,f,r,ddot
      data one/1.d0/
      data mone/-1.d0/
      data zero/0.d0/
      do23000 i = (p+1),n 
      call dgemv('N',p,p,one,a,p,x(1,i),1,zero,ax,1)
      f = one + ddot(p,x(1,i),1,ax,1)
      r = (y(i)-ddot(p,x(1,i),1,b(1,i-1),1))/f
      call daxpy(p,one,b(1,i-1),1,b(1,i),1)
      call daxpy(p,r,ax,1,b(1,i),1)
      call dger(p,p,mone/f,ax,1,ax,1,a,p)
23000 continue
23001 continue
      return
      end

C Output from Public domain Ratfor, version 1.05
      subroutine crqfnb(n,p,a1,c1,n1,x,y,c,b,g,m,r,s,d,u,wn,wp,info)
      integer n,p,n1,m,info,nit(3)
      double precision a1(p,n1),c1(n),x(n,p),y(n),c(n),b(p,m),g(m)
      double precision wn(n1,9),wp(p,p+3),r(p),s(n),d(n),u(n)
      double precision zero,half,one,beta,eps,dh
      parameter( zero = 0.0d0)
      parameter( half = 0.5d0)
      parameter( one = 1.0d0)
      parameter( beta = 0.99995d0)
      parameter( eps = 1.0d-8)
      do23000 k = 2,m 
      dh = -log(one - g(k)) + log(one - g(k-1))
      do23002 i = 1,n 
      u(i) = one
      wn(i,1) = half
      if(d(i) .ge. zero)then
      s(i) = s(i) + dh
      endif
      d(i) = c(i) - s(i)
23002 continue
23003 continue
      call dgemv('T',n,p,one,x,n,d,1,zero,r,1)
      call rqfnb(n1,p,a1,c1,r,d,u,beta,eps,wn,wp,nit,info)
      if(info .ne. 0)then
      goto 23001
      endif
      call dcopy(p,wp,1,b(1,k-1),1)
      call dcopy(n,y,1,d,1)
      call dgemv('N',n,p,one,x,n,b(1,k-1),1,one,d,1)
23000 continue
23001 continue
      m = k-1
      return
      end

c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c
c  Function to obtain the step length
c
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c
      subroutine boundc(x1,dx1,x2,dx2,s,ds,z1,dz1,z2,dz2,w,dw,n1,n2,
     &                 beta,deltap,deltad)
c
      integer n1,n2
      double precision x1(n1),dx1(n1),x2(n2),dx2(n2),s(n1),ds(n1),
     &                 z1(n1),dz1(n1),z2(n2),dz2(n2),w(n1),dw(n1)
      double precision deltap,deltad,dmin1,big,one,beta
      parameter (big = 1.0d20, one = 1.0d0)
      deltap = big
      deltad = big
      do i = 1,n1
         if(dx1(i) .lt. 0) deltap = dmin1(deltap, -x1(i)/dx1(i))
         if(ds(i) .lt. 0) deltap = dmin1(deltap, -s(i)/ds(i))
         if(dz1(i) .lt. 0) deltad = dmin1(deltad, -z1(i)/dz1(i))
         if(dw(i) .lt. 0) deltad = dmin1(deltad, -w(i)/dw(i))
      enddo
      do i = 1,n2
         if(dx2(i) .lt. 0) deltap = dmin1(deltap, -x2(i)/dx2(i))
         if(dz2(i) .lt. 0) deltad = dmin1(deltad, -z2(i)/dz2(i))
      enddo
      deltap = dmin1(beta*deltap,one)
      deltad = dmin1(beta*deltad,one)
      return
      end

"boot.rq"<-
function (x, y, tau = 0.5, R = 200, bsmethod = "xy", mofn = length(y), 
	  coef = NULL, blbn = NULL, cluster = NULL, U = NULL, ...)
{
    n <- length(y)
    if(class(x)[1] != "matrix.csr") x <- as.matrix(x)
    p <- ncol(x) 
    B <- matrix(0, R, p)
    if(tau < 0 || tau > 1) stop("tau outside (0,1) not allowed")
    if(length(cluster)) bsmethod <- "cluster"
    if (bsmethod == "xy") {
	if(!length(U)){
	    if(mofn < p || mofn > n) stop("mofn is out of range")
	    U <- matrix(sample(n, mofn * R, replace = TRUE), mofn, R)
	}
        B <- sqrt(mofn/n)*boot.rq.xy(x, y, U, tau)
    }   
    else if (bsmethod == "wxy") {
        if(!length(U)) U <- matrix(rexp(n * R,1), n, R)
        B <- boot.rq.wxy(x, y, U, tau)
    }   
    else if (bsmethod == "pxy") {
        if (!length(U)) {
            if (mofn < p || mofn > n) stop("mofn is out of range")
            U <- matrix(sample(n, mofn * R, replace = TRUE), mofn, R)
            }
        B <- sqrt(mofn/n) * boot.rq.pxy(x, y, U, tau, coef, ...)
    }
    else if (bsmethod == "pwxy") {
        B <- sqrt(mofn/n) * boot.rq.pwxy(x, y, tau, coef, ...)$B
    }
    else if (bsmethod == "BLB") {
        B <- boot.rq.pwxy(x, y, tau, coef, ...)$B
    }   
    else if (bsmethod == "cluster"){ # Hagemann wild gradient bootstrap
	if(!length(U)){
	    if(length(cluster) != n) stop("cluster is wrong length")
	    u <- unique(cluster)
	    m <- length(u)
	    h <- c(-(sqrt(5) - 1)/2, (sqrt(5) + 1)/2)
	    hp <- c((sqrt(5) + 1)/sqrt(20), (sqrt(5) - 1)/sqrt(20))
	    U <- matrix(sample(h, R * m, prob = hp, replace = TRUE), m, R)
	    U <- apply(U, 2, function(v) v[match(cluster,u)])
	}
	if(class(x)[1] == "matrix.csr") 
	    r <- c(rq.fit.sfn(x, y, tau)$resid)
	else
	    r <- c(rq.fit.fnb(x, y, tau)$resid)
	psi <- (r < 0) - tau
	W <- as.matrix(t(x) %*% (U * psi))
	if(class(x)[1] == "matrix.csr") 
	    B <- boot.rq.spwy(W, x, y, tau)
	else
	    B <- boot.rq.pwy(W, x, y, tau)
    }
    else if (bsmethod == "jack"){ # Portnoy proposal
	if(!length(U)){
	    if(missing(mofn)) mofn <- n - ceiling(2*sqrt(n))
	    if(mofn < p || mofn > n) stop("mofn is out of range")
	    U <- matrix(0, mofn, R)
	    U <- apply(U, 2, function(x) sample(n, mofn))
	}
	B <- (mofn - p + 1)/sqrt(n * (n - mofn)) * boot.rq.xy(x, y, U, tau)
    }
    else if (bsmethod == "pwy") {
	if(!length(U)){
	    U <- matrix(runif(n * R), n, R)
	}
	W <- t(x) %*% ((U < tau) - tau)
        B <- boot.rq.pwy(W, x, y, tau)
    }
    else if (bsmethod == "mcmb") {
        B <- boot.rq.mcmb(x, y, tau = tau, R = R)
    }
    else if (bsmethod == "wild") {
        n <- length(y)
        fit <- rq.fit(x, y, tau = tau)
        S <- sample(c(-2*tau,2*(1-tau)),prob = c(tau,1-tau),size = n * R, replace = TRUE)
        W <- matrix(S,n,R)
        r <- c(fit$resid)
        f0 <- akj(r,z=0)$dens 
        r <- r + hat(x) * (tau - I(r < 0))/f0 
        Y <- c(fitted(fit)) + W * abs(r)
        B <- rqs.fit(x,Y,tau = tau)
    }
    else stop("your chosen bootstrap method is not allowed")
    #cat(paste("Bootstrap standard errors based on ",R," replications"))
    list(B = B, U = U)
}
"boot.rq.mcmb" <-
function (x, y, tau = 0.5, R = 200) 
{
	n <- length(y)
	p <- ncol(x)
	if(n < 2000)
		fit <- rq(y~x - 1, tau = tau, ci = FALSE)
	else
		fit <- rq(y~x - 1, tau = tau, method = "fn")
	coef <- fit$coef
	svdx <- svd(x)
	condx <- svdx$d[1]/svdx$d[p]
	Ainv <- svdx$v %*% diag(svdx$d) %*% t(svdx$v) 
	coefTilda <-  Ainv %*% coef
	A <- svdx$v %*% diag(1/svdx$d) %*% t(svdx$v) 
	r <- fit$resid
    	psi <- signr <- sign(r)
    	psi[signr > 0] <- tau
    	psi[signr < 0] <- tau - 1
	psimat <- matrix(psi, nrow = n, ncol = p, byrow = FALSE)
	x <- x %*% A
	ZTilda <- x * psimat
	sumxij <- apply(x, 2, sum)
	sumabsxij <- apply(abs(x), 2, sum)
	zstar <- .C("bootnp", as.double(t(x)), as.double(y), 
		as.double(tau), as.double(coefTilda), 
		as.double(t(A)), as.double(ZTilda), 
		as.double(sumxij), as.double(sumabsxij), 
		as.integer(n), as.integer(p), success = as.integer(1), 
		theta = as.double(rep(0, R * p + p), as.integer(c(p, R + 1))), 
		as.integer(R))
	if (zstar$success == 0) 
		return(list(success = 0))
	else{
		B <- matrix(zstar$theta,p,R+1)[,-1]
		B <- t(A %*% B)
		}
	B
}

"boot.rq.xy"<-
function(x, y, s, tau = 0.5, tol = 0.0001)
{
#function to compute xypairs bootstrap for regression quantiles 
#stripped down for monte-carlo purposes
	x <- as.matrix(x)
	p <- ncol(x)
	n <- nrow(x)
	R <- ncol(s)
	m <- nrow(s)
	z <- .Fortran("xys",
		as.integer(m),
		as.integer(n),
		as.integer(p),
		as.integer(R),
		as.integer(m + 5),
		as.integer(p + 2),
		as.double(x),
		as.double(y),
		as.double(tau),
		as.double(tol),
		flag = integer(R),
		coef = double(p * R),
		resid = double(m),
		integer(m),
		double((m + 5) * (p + 2)),
		double(m),
		xx = double(m * p),
		yy = double(m),
		as.integer(s))
	if(sum(z$flag)>0){
		if(any(z$flag)==2) 
			warning(paste(sum(z$flag==2),"out of",R, 
				"BS replications have near singular design"))
		if(any(z$flag)==1) 
			warning(paste(sum(z$flag==1),"out of",R,"may be nonuniqu")) 
		}
	return(t(matrix(z$coef, p, R)))
}
"boot.rq.wxy"<-
function(x, y, w, tau = 0.5, tol = 0.0001)
{
#function to compute weighted bootstrap a la Bose for regression quantiles 
        x <- as.matrix(x)
        p <- ncol(x)
        n <- nrow(x)
        R <- ncol(w)
        m <- nrow(w)
        z <- .Fortran("wxy",
                as.integer(n),
                as.integer(p),
                as.integer(R),
                as.integer(m + 5),
                as.integer(p + 2),
                as.double(x),
                as.double(y),
                as.double(tau),
                as.double(tol),
                flag = integer(R),
                coef = double(p * R),
                resid = double(m),
                integer(m),
                double((m + 5) * (p + 2)),
                double(m),
                xx = double(m * p),
                yy = double(m),
                as.double(w))
        if(sum(z$flag)>0){
                if(any(z$flag)==2)
                        warning(paste(sum(z$flag==2),"out of",R,
                                "BS replications have near singular design"))
                if(any(z$flag)==1)
                        warning(paste(sum(z$flag==1),"out of",R,"may be nonunique"))
                }
        return(t(matrix(z$coef, p, R)))
}

"boot.rq.pwy"<-
function(U,X, y, tau = 0.5, tol=1e-4)
{
#resampling method of parzen,wei,ying for quantile regression
#NB. x should be full design matrix including intercept
	n <- length(y)
	p <- ncol(X)
	R <- ncol(U)
	Y <- c(y,length(y)*max(abs(y)))
	x <- rbind(X,0)
	xu <- t(U)/tau
	n <- n+1
	z<-.Fortran("pwy",
		as.integer(n),
		as.integer(p),
		as.integer(R),
		as.integer(n + 5),
		as.integer(p + 2),
		as.double(xu),
		as.double(x),
		as.double(Y),
		as.double(tau),
		as.double(tol),
		flag = as.integer(1),
		coef = double(p * R),
		resid = double(n),
		integer(n),
		double((n + 5) * (p + 2)),
		double(n))
	return(t(matrix(z$coef, p, R)))
}

#################################################################
#	NB:  In an ideal world the loop below would be in fortran, in a
#		a really ideal world all of this resampling code
#		would be able to adapt to the "fn" or "sfn" etc
#		method used by the original fitting scheme.
#		My initial attempt to allow control to be passed
#		from the initial fit of the coefficients was a failure.
#		There was a "memory not mapped" segfault presumably
#		due to the fact that the pwy scheme has one extra
#		observation.  Who would have guessed things were so
#		delicately tuned.  This should be fixed when I'm in
#		a more delicate frame of mind.  Meanwhile, on Andreas
#		test problem using sfn is substantially faster 26s
#		vs 148s for fnb, and even more for br as I used earlier.
#################################################################


boot.rq.spwy <- function(W, a, y, tau=.5)
{
	y <- c(y, length(y) * max(abs(y)))
	n <- length(y)
	m <- a@dimension[2]
	a <- rbind(a, as.matrix.csr(t(rep(1,m))))
	nra <- length(a@ra)
	W <- W/tau
	k <- ncol(W) # Number of resampling realizations
	if(m != nrow(W)) 
	    stop("W row dimension not compatible with design matrix")
	if(n != a@dimension[1])
	     stop("Dimensions of design matrix and the response vector not compatible")
	for(i in 1:k){
	    a@ra[(nra - m + 1):nra] <- W[,i]
	    W[,i] <- rq.fit.sfn(a,y,tau = tau)$coef
	}
	B <- t(W)
	B
}
# Preprocessing version of the xy bootstrap
#	globs as in "pfn" method
#	matrix s contains multinomial resampling draws (n by R)  
boot.rq.pxy <- function (x, y, s, tau = 0.5, coef, method="fn", Mm.factor = 3) {
  x <- as.matrix(x)
  p <- ncol(x)
  n <- nrow(x)
  R <- ncol(s)
  mofn <- nrow(s)
  #I have implemented the bootstrap with the count vector instead of the permutation vector
  counts <- apply(s, 2, tabulate, nbins=n)
  m <- round((p*n)^(1/2)) # Is there any theory for this?
  xxinv <- solve(chol(crossprod(x)))
  band <- sqrt(rowSums((x %*% xxinv)^2)) # H[i,i] vector
  r <- (y - x %*% coef)/band
  #To avoid computing quantiles of r in each replication I calculate once the cdf
  rs <- sort(r)
  B <- matrix(NA, R, p)
  #add two observations for the globs 
  x <- rbind(0,0,x)
  y <- c(0,0,y)
  counts <- rbind(1,1,counts)
  for(i in 1:R){
    mm <- m
    not.optimal <- TRUE
    #keep only the observations with positive weights
    bs <- (counts[,i] > 0)
    xb <- x[bs,]
    yb <- y[bs]
    rb <- r[bs[3:length(bs)]]
    wb <- counts[bs, i]
    while (not.optimal) {
      M <- Mm.factor * mm
      lo.q <- max(1, ceiling(n*tau - M/2))
      hi.q <- min(floor(n*tau + M/2), n)
      kappa <- rs[c(lo.q, hi.q)]
      sl <- c(FALSE, FALSE, rb < kappa[1])
      su <- c(FALSE, FALSE, rb > kappa[2])
      while (not.optimal) {
        if (any(sl)) {
          xb[1,] <- colSums(xb[sl, , drop = FALSE] * wb[sl])
          yb[1] <- sum(yb[sl] * wb[sl])
	} else sl[1] <- TRUE
        if (any(su)) {
          xb[2,] <- colSums(xb[su, , drop = FALSE] * wb[su])
          yb[2] <- sum(yb[su] * wb[su])
        } else su[2] <- TRUE
        sel <- c(!su & !sl)
        #use weighted QR
        z <- tryCatch(z <- rq.wfit(xb[sel, ], yb[sel], weights=wb[sel], 
		  tau = tau, method = method), error = function(e) e)
        if(inherits(z, "error")){
          mm <- 2*mm
          warning("Singular design: doubling m")
          break
          }        
        b <- z$coef
        rb <- yb[3:sum(bs)] - xb[3:sum(bs), ] %*% b
        su.bad <- c(FALSE, FALSE, rb < 0) & su
        sl.bad <- c(FALSE, FALSE, rb > 0) & sl
        bad.signs <- sum(su.bad | sl.bad)
        if (bad.signs > 0) {
          if (bad.signs > 0.1 * M) {
            mm <- 2*mm
            #warning("Too many fixups:  doubling m")
            break
          }
          su <- su & !su.bad
          sl <- sl & !sl.bad
        }
        else not.optimal <- FALSE
      }
    }
    B[i,] <- b
  }
  B
}

boot.rq.pwxy <- function (x, y, tau, coef, R = 200, m0 = NULL, eps = 1e-06, ...) 
{
    n <- length(y)
    if (nrow(x) != n) 
        stop("x and y don't match n")
    p <- ncol(x)
    if (!length(m0)) 
        m0 <- round(sqrt(n*p))
    r <- y - x %*% coef
    b <- matrix(0, p, R)
    nit <- matrix(0, 5, R) # This version keeps track of fixups!
    info <- rep(0, R)
    xxinv <- solve(chol(crossprod(x)))
    band <- pmax(eps, sqrt(((x %*% xxinv)^2) %*% rep(1, p)))
    r <- r/band
    o <- order(r)
    r <- r[o]
    y <- y[o]
    x <- x[o,]
    loq <- max(1, ceiling(n*tau - m0/2))
    hiq <- min(floor(n*tau + m0/2), n)
    qk <- r[c(loq,hiq)]
    z <- .Fortran("pwxy", as.integer(n), as.integer(p), as.integer(R), 
        as.double(t(x)), as.double(y), as.double(tau), as.double(qk),as.double(r), 
        b = as.double(-b), double(n), as.double(band), as.integer(m0), double(n), 
        double(n), double(n * 9), double(p * (p + 3)), double(p * 
            n), double(n), integer(n), integer(n), double(p), 
        double(p), double(p), nit = as.integer(nit), info = as.integer(info))
    if(any(z$info != 0)) warnings("Some bootstrap replications have abnormal convergence")
    B <- t(matrix(-z$b, p, R))
    nit <- matrix(z$nit,5,R)
    list(B = B, nit = nit)
}

# Toy fnb routine for multiple taus
subroutine qfnb(n,p,m,a,y,t,r,d,u,wn,wp,B,nit, info)

# Input:
#	n = sample size
#	p = parametric dimension of model
#	m = dimension of tau vector
#	a = p by n design matrix (transposed)
#	y = n dimensional response vector
#	t = m dimensional tau vector
#	r = p dimensional rhs vector
#	d = n dimensional vector of ones
#	u = n dimensional vector of ones
#	wn = n by 9 work array
#	wp = p by p+3 work array
#
# Output:	
#	B = p by m matrix of coefficients

integer n,p,m,nit(3),info
double precision a(p,n), y(n), t(m), B(p,m), r(p)
double precision d(n), u(n), wn(n,9), wp(p,p+3)
double precision zero, one, eps, beta

parameter( zero  =  0.0d0)
parameter( one   =  1.0d0)
parameter( beta  =  0.99995d0)
parameter( eps   =  1.0d-6)
    
do i = 1,m{
	call dgemv('N',p,n,one-t(i),a,p,d,1,zero,r,1)
	call dscal(n,zero,wn,1)
	call daxpy(n,one-t(i),u,1,wn,1)
	call rqfnb(n,p,a,y,r,d,u,beta,eps,wn,wp,nit,info)
	if(info != 0) break
	do j = 1,n{
	    u(j) = one
	    d(j) = one
	}
	call dcopy(p,wp,1,B(1,i),1)
    }
return
end

# This is a primal-dual log barrier form of the
# interior point LP solver of Lustig, Marsten and Shanno ORSA J Opt 1992.
# It is a projected Newton primal-dual logarithmic barrier method which uses
# the predictor-corrector approach of Mehrotra for the mu steps.
# For the sake of brevity we will call it a Frisch-Newton algorithm.
# The primary difference between this code and the previous version fna.r is
# that we assume a feasible starting point so p,d,b feasibility gaps = 0.
# Problem:
# 	min c'x s.t. Ax=b, 0<=x<=u
# 
# The linear system we are trying to solve at each interation is:
# 		 Adx = 0
# 	     dx + ds = 0
# 	A'dy -dw +dz = 0
# 	   Xdz + Zdx = me - XZe - DXDZe
# 	   Sdw + Wds = me - SWe - DSDWe 
# But the algorithm proceeds in two steps the first of which is to solve:
# 		 Adx = 0
# 	     dx + ds = 0
# 	A'dy -dw +dz = 0
# 	   Xdz + Zdx =  - XZe 
# 	   Sdw + Wds =  - SWe
# and then to make some refinement of mu and modify the implied Newton step.
# Denote dx,dy,dw,ds,dz as the steps for the respective variables x,y,w,s,z, and
# the corresponding upper case letters are the diagonal matrices respectively.
# 
# To illustrate the use of the function we include a calling routine to
# compute solutions to the linear quantile regression estimation problem.
# See the associated S function rqfn for details on the calling sequence.
# On input:
# 	a is the p by n matrix X'
# 	y is the n-vector of responses
# 	u is the n-vector of upper bounds 
#       d is an n-vector of ones
#       wn is an n-vector of ones in the first n elements
# 	beta is a scaling constant, conventionally .99995
# 	eps is a convergence tolerance, conventionally 1d-07
# On output:
# 	a,y are unaltered
# 	wp contains the solution  coefficient vector in the first p elements
# 	wn contains the residual vector in the first n elements
# 
# 
subroutine rqfn(n,p,a,y,rhs,d,u,beta,eps,wn,wp,aa,nit,info)
integer n,p,info,nit(3)
double precision a(p,n),y(n),rhs(p),d(n),u(n),wn(n,10),wp(p,p+3),aa(p,p)
double precision one,beta,eps
parameter( one = 1.0d0)
call fna(n,p,a,y,rhs,d,u,beta,eps,wn(1,1),wn(1,2),
	wp(1,1),wn(1,3),wn(1,4),wn(1,5), wn(1,6),
	wp(1,2),wn(1,7),wn(1,8),wn(1,9),wn(1,10),wp(1,3), wp(1,4),aa,nit,info)
return
end
subroutine fna(n,p,a,c,b,d,u,beta,eps,x,s,y,z,w,
		dx,ds,dy,dz,dw,dsdw,dxdz,rhs,ada,aa,nit,info)

integer n,p,pp,i,info,nit(3)
double precision a(p,n),c(n),b(p)
double precision zero,one,mone,big,ddot,dmax1,dmin1,dasum
double precision deltap,deltad,beta,eps,cx,by,uw,uz,mu,mua,acomp,rdg,g
double precision x(n),u(n),s(n),y(p),z(n),w(n),d(n),rhs(p),ada(p,p)
double precision aa(p,p),dx(n),ds(n),dy(p),dz(n),dw(n),dxdz(n),dsdw(n)

parameter( zero  = 0.0d0)
parameter( half  = 0.5d0)
parameter( one   = 1.0d0)
parameter( mone   = -1.0d0)
parameter( big  = 1.0d+20)

# Initialization:  We try to follow the notation of LMS
# On input we require:
# 
# 	c = n-vector of marginal costs (-y in the rq problem)
# 	a = p by n matrix of linear constraints (x' in rq)
# 	b = p-vector of rhs ((1-tau)x'e in rq)
#       u = upper bound vector ( e in rq)
# 	beta = barrier parameter, LMS recommend .99995
# 	eps = convergence tolerance, LMS recommend 10d-8
# 	
# 	the integer vector nit returns iteration counts
# 	the integer info contains an error code from the Cholesky in stepy
# 		info = 0 is fine
# 		info < 0 invalid argument to dposv 
# 		info > 0 singular matrix
nit(1)=0
nit(2)=0
nit(3)=n
pp=p*p
# Start at the OLS estimate for the parameters
call dgemv('N',p,n,one,a,p,c,1,zero,y,1)
call stepy(n,p,a,d,y,aa,info)
if(info != 0) return
# Save sqrt of aa' for future use for confidence band
do i=1,p{
	do j=1,p
		ada(i,j)=zero
        ada(i,i)=one
	}
call dtrtrs('U','T','N',p,p,aa,p,ada,p,info)
call dcopy(pp,ada,1,aa,1)
# put current residual vector in s (temporarily)
call dcopy(n,c,1,s,1)
call dgemv('T',p,n,mone,a,p,y,1,one,s,1)
# Initialize remaining variables
# N.B. x must be initialized on input: for rq as (one-tau) in call coordinates
do i=1,n{
	d(i)=one
	if(dabs(s(i)) < eps){
		z(i) = dmax1( s(i),zero) + eps
		w(i) = dmax1(-s(i),zero) + eps
		}
	else {
		z(i) = dmax1( s(i),zero)
		w(i) = dmax1(-s(i),zero) 
		}
	s(i)=u(i)-x(i)
	}
cx = ddot(n,c,1,x,1)
by = ddot(p,b,1,y,1)
uw = dasum(n,w,1)
uz = dasum(n,z,1)
# rdg =  (cx - by + uw)/(one + dabs( by - uw))
# rdg =  (cx - by + uw)/(one + uz + uw)
rdg =  (cx - by + uw)
while(rdg > eps) {
	nit(1)=nit(1)+1
	do i =1,n{
		d(i) = one/(z(i)/x(i) + w(i)/s(i))
		ds(i)=z(i)-w(i)
		dx(i)=d(i)*ds(i)
		}
	call dgemv('N',p,n,one,a,p,dx,1,zero,dy,1)#rhs
	call dcopy(p,dy,1,rhs,1)#save rhs
	call stepy(n,p,a,d,dy,ada,info)
	if(info != 0) return
	call dgemv('T',p,n,one,a,p,dy,1,mone,ds,1)
	deltap=big
	deltad=big
	do i=1,n{
		dx(i)=d(i)*ds(i)
		ds(i)=-dx(i)
		dz(i)=-z(i)*(dx(i)/x(i) + one)
		dw(i)=w(i)*(dx(i)/s(i) - one)
		dxdz(i)=dx(i)*dz(i)
		dsdw(i)=ds(i)*dw(i)
		if(dx(i)<0)deltap=dmin1(deltap,-x(i)/dx(i))
		if(ds(i)<0)deltap=dmin1(deltap,-s(i)/ds(i))
		if(dz(i)<0)deltad=dmin1(deltad,-z(i)/dz(i))
		if(dw(i)<0)deltad=dmin1(deltad,-w(i)/dw(i))
		}
	deltap=dmin1(beta*deltap,one)
	deltad=dmin1(beta*deltad,one)
	if(deltap*deltad<one){
		nit(2)=nit(2)+1
		acomp=ddot(n,x,1,z,1)+ddot(n,s,1,w,1)
		g=acomp+deltap*ddot(n,dx,1,z,1)+
			deltad*ddot(n,dz,1,x,1)+ 
			deltap*deltad*ddot(n,dz,1,dx,1)+
			deltap*ddot(n,ds,1,w,1)+
			deltad*ddot(n,dw,1,s,1)+ 
			deltap*deltad*ddot(n,ds,1,dw,1)
		mu=acomp/dfloat(2*n)
		mua=g/dfloat(2*n)
		mu=mu*(mua/mu)**3
		#if(acomp>1) mu=(g/dfloat(n))*(g/acomp)**2
		#else mu=acomp/(dfloat(n)**2)
		do i=1,n{
			dz(i)=d(i)*(mu*(1/s(i)-1/x(i))+
				dx(i)*dz(i)/x(i)-ds(i)*dw(i)/s(i))
			}
		call dswap(p,rhs,1,dy,1)
		call dgemv('N',p,n,one,a,p,dz,1,one,dy,1)#new rhs
		call dpotrs('U',p,1,ada,p,dy,p,info)
		call daxpy(p,mone,dy,1,rhs,1)#rhs=ddy
		call dgemv('T',p,n,one,a,p,rhs,1,zero,dw,1)#dw=A'ddy
		deltap=big
		deltad=big
		do i=1,n{
			dx(i)=dx(i)-dz(i)-d(i)*dw(i)
			ds(i)=-dx(i)
			dz(i)=mu/x(i) - z(i)*dx(i)/x(i) - z(i) - dxdz(i)/x(i)
			dw(i)=mu/s(i) - w(i)*ds(i)/s(i) - w(i) - dsdw(i)/s(i)
			if(dx(i)<0)deltap=dmin1(deltap,-x(i)/dx(i))
			else deltap=dmin1(deltap,-s(i)/ds(i))
			if(dz(i)<0)deltad=dmin1(deltad,-z(i)/dz(i))
			if(dw(i)<0)deltad=dmin1(deltad,-w(i)/dw(i))
			}
		deltap=dmin1(beta*deltap,one)
		deltad=dmin1(beta*deltad,one)
		}
	call daxpy(n,deltap,dx,1,x,1)
	call daxpy(n,deltap,ds,1,s,1)
	call daxpy(p,deltad,dy,1,y,1)
	call daxpy(n,deltad,dz,1,z,1)
	call daxpy(n,deltad,dw,1,w,1)
	cx=ddot(n,c,1,x,1)
	by=ddot(p,b,1,y,1)
	uw = dasum(n,w,1)
	uz = dasum(n,z,1)
	#rdg=(cx-by+uw)/(one+dabs(by-uw))
	#rdg=(cx-by+uw)/(one+uz+uw)
	rdg=(cx-by+uw)
	}
# return residuals in the vector x
call daxpy(n,mone,w,1,z,1)
call dswap(n,z,1,x,1)
return
end

# Routines to compute the X contribution of the total variation penalty term
# Given a Delaunay triangulation structure as provided by tripack 

# Written in the nearly extinct ratfor language of the lost tribe of Murray Hill
# With luck it can be translated into the dreaded fortran understood by g77.

# Roger Koenker:  Last Modified October 29, 2002.

# The subroutines employ the Renka tripack library and strongly rely on
# the counterclockwise ordering of the elements in the adjacency list.
# the vector bnd is used to identify and neglect edges on the boundary
# In future versions one might want to have the option of making some other
# provisions for these edges.


subroutine penalty(n,m,q,x,y,bnd,tlist,tlptr,tlend,rax,jax,ned,eps,ierr)

integer n,m,q,lp,lpl,ned,ierr
integer bnd(n),tlist(q),tlptr(q),tlend(n),n4(4),p4(4),jax(m)

double precision x(n),y(n),rax(m),eps
double precision x4(4),y4(4),g4(4)

logical orient


#loop over the vertices: i is index of one end of the edge j is the other end
ned = 0
do i=1,n{
	lpl = tlend(i)
	lp  = lpl
	repeat{
		lp = tlptr(lp)
		j  = iabs(tlist(lp))
		if(j > i){
			n4(1) = i
			n4(2) = j
			call fadjs(n4,n,q,tlist,tlptr,tlend)
			if(bnd(i)*bnd(j) == 0){
				ned = ned + 1
				do k = 1,4{
					x4(k) = x(n4(k))
					y4(k) = y(n4(k))
					}
				if(orient(x4,y4)){
					call iswap(1,n4(3),1,n4(4),1)
					call dswap(1,x4(3),1,x4(4),1)
					call dswap(1,y4(3),1,y4(4),1)
					}
				call ggap(x4,y4,g4,eps,ierr)
				if(ierr == 1) return
				call srtpai(n4,1,p4,1,4)
				do k = 1,4{
					rax((ned - 1)*4 + k) = g4(p4(k))
					jax((ned - 1)*4 + k) = n4(p4(k))
					}
				if(ned*4 > m) return
				}
			}
		if(lp == lpl) break
		}
	}
return
end
logical function orient(x,y)
double precision x(4), y(4)
orient = (y(2) -y(1))*(x(3)-x(4))+(x(1)-x(2))*(y(3)-y(4)) > 0
return
end 
subroutine fadjs(n4,n,q,tlist,tlptr,tlend)
# Subroutine to find matching adjacent vertices for the triogram edges
# On input:
#       n4[1:2]  contain the indices of the edge of interest
#       tlist,tlptr,tlendd  is the (tripack) structure describing the triangulation
# On output:
#       n4[3:4]  contains indices of the two adjacent vertices 
#
# Adjacency tlist is in counter-clockwise order so we want to find the two
# vertices that are immediately above and below  n1 in the n0 tlist
#
# Roger Koenker June 4, 2002
#

integer n,q,vp,vpl,v,v0,match
integer n4(4),tlist(q),tlptr(q),tlend(n)

# Check whether edge is on the boundary

match = 0
vpl = tlend(n4(1))
vp = vpl
k = 0
repeat{
       k = k+1
       vp = tlptr(vp)
       v  = tlist(vp)
       if(k>1 & iabs(v) == n4(2)){
               n4(3) = iabs(v0)
               match = 1
               next
               }
       if(match > 0){
               n4(4) = iabs(v)
               break
               }
       v0 = v
       }
return
end

subroutine ggap(x,y,g,eps,ierr)
double precision x(4),y(4),g(4),w(2,4),h(2),D1,D2,eps
# Triogram package:  Roger Koenker  June 4, 2002
# given four (x,y) pairs for an edge compute contribution to the penalty
# ierr returns 1 if the edge is degenerate, ie either determinant is 0.
	D1 = -x(2) * y(1) + x(3) * y(1) + x(1) * y(2) -
	      x(3) * y(2) - x(1) * y(3) + x(2) * y(3)
	D2 = -x(2) * y(1) + x(4) * y(1) + x(1) * y(2) -
	     x(4) * y(2) - x(1) * y(4) + x(2) * y(4)
	if(dabs(D1) < eps | dabs(D2) < eps) {
		ierr = 1
		return
		}
        h(1) =  -(y(1) - y(2)) 
	h(2) =   (x(1) - x(2))
        w(1, 1) = (y(2) - y(3))/D1 - (y(2) - y(4))/D2
        w(2, 1) = (x(3) - x(2))/D1 - (x(4) - x(2))/D2
        w(1, 2) = (y(3) - y(1))/D1 - (y(4) - y(1))/D2
        w(2, 2) = (x(1) - x(3))/D1 - (x(1) - x(4))/D2
        w(1, 3) = (y(1) - y(2))/D1
        w(2, 3) = (x(2) - x(1))/D1
        w(1, 4) = (y(2) - y(1))/D2
        w(2, 4) = (x(1) - x(2))/D2
	do i = 1,4{
		g(i) = h(1)*w(1,i)+h(2)*w(2,i)
		}
ierr = 0
return
end

      SUBROUTINE ISWAP (N, IX, INCX, IY, INCY)
C***AUTHOR  Vandevender, W. H., (SNLA)
C***DESCRIPTION
C
C                Extended B L A S  Subprogram
C    Description of Parameters
C
C     --Input--
C        N  number of elements in input vector(s)
C       IX  integer vector with N elements
C     INCX  storage spacing between elements of IX
C       IY  integer vector with N elements
C     INCY  storage spacing between elements of IY
C
C     --Output--
C       IX  input vector IY (unchanged if N .LE. 0)
C       IY  input vector IX (unchanged if N .LE. 0)
C
C     Interchange integer IX and integer IY.
C     For I = 0 to N-1, interchange  IX(LX+I*INCX) and IY(LY+I*INCY),
C     where LX = 1 if INCX .GE. 0, else LX = 1+(1-N)*INCX, and LY is
C     defined in a similar way using INCY.
C
C***REFERENCES  C. L. Lawson, R. J. Hanson, D. R. Kincaid and F. T.
C                 Krogh, Basic linear algebra subprograms for Fortran
C                 usage, Algorithm No. 539, Transactions on Mathematical
C                 Software 5, 3 (September 1979), pp. 308-323.
C***ROUTINES CALLED  (NONE)
C***REVISION HISTORY  (YYMMDD)
C   850601  DATE WRITTEN
C   861211  REVISION DATE from Version 3.2
C   891214  Prologue converted to Version 4.0 format.  (BAB)
C   920310  Corrected definition of LX in DESCRIPTION.  (WRB)
C   920501  Reformatted the REFERENCES section.  (WRB)
C***END PROLOGUE  ISWAP
      INTEGER IX(*), IY(*), ITEMP1, ITEMP2, ITEMP3
C***FIRST EXECUTABLE STATEMENT  ISWAP
      IF (N .LE. 0) RETURN
      IF (INCX .NE. INCY) GO TO 5
      IF (INCX-1) 5,20,60
C
C     Code for unequal or nonpositive increments.
C
    5 IIX = 1
      IIY = 1
      IF (INCX .LT. 0) IIX = (1-N)*INCX + 1
      IF (INCY .LT. 0) IIY = (1-N)*INCY + 1
      DO 10 I = 1,N
        ITEMP1 = IX(IIX)
        IX(IIX) = IY(IIY)
        IY(IIY) = ITEMP1
        IIX = IIX + INCX
        IIY = IIY + INCY
   10 CONTINUE
      RETURN
C
C     Code for both increments equal to 1.
C
C     Clean-up loop so remaining vector length is a multiple of 3.
C
   20 M = MOD(N,3)
      IF (M .EQ. 0) GO TO 40
      DO 30 I = 1,M
        ITEMP1 = IX(I)
        IX(I) = IY(I)
        IY(I) = ITEMP1
   30 CONTINUE
      IF (N .LT. 3) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,3
        ITEMP1 = IX(I)
        ITEMP2 = IX(I+1)
        ITEMP3 = IX(I+2)
        IX(I) = IY(I)
        IX(I+1) = IY(I+1)
        IX(I+2) = IY(I+2)
        IY(I) = ITEMP1
        IY(I+1) = ITEMP2
        IY(I+2) = ITEMP3
   50 CONTINUE
      RETURN
C
C     Code for equal, positive, non-unit increments.
C
   60 NS = N*INCX
      DO 70 I = 1,NS,INCX
        ITEMP1 = IX(I)
        IX(I) = IY(I)
        IY(I) = ITEMP1
   70 CONTINUE
      RETURN
      END

# THIS IS A FAILED ATTEMPT TO MAKE THE BOOTSTRAP LOOP OF PWY IN FORTRAR
# Parzen, Wei and Ying's bootstrap with sparse fn fitting
# subroutine pwy(m,n,k,m5,n2,a,c,b,t,toler,ift,x,e,s, wa,wb)
# Calling sequence is just like that of srqfn.f except that
# 
#	a is now a (n+1) by p design matrix
#	c is now a n+1 vector
#	wu on input is the matrix of simulated gradient vectors
#		on output is the p by k solution matrix
#	y as in srqfn on input is the initial condition of dual
#	k is the column dimension of y, this is only added argument
#
subroutine spwy(n,m,k,b1,wu,nnza,a,ja,ia,ao,jao,iao,nnzdmax,d,jd,id,
  dsub,jdsub,nnzemax,e,je,ie,nsubmax,lindx,xlindx,
  nnzlmax,lnz,xlnz,iw,iwmax,iwork,xsuper,tmpmax, tmpvec,
  wwm,wwn,cachsz,level,x,s,u,c,y,b,small,ierr,maxit,time)
integer nnza,m,n,nnzdmax,nnzemax,iwmax,
  nnzlmax,nsubmax,cachsz,level,tmpmax,ierr,maxit,time(7),
  ja(nnza),jao(nnza),jdsub(nnzemax+1),jd(nnzdmax),
  ia(n+1),iao(m+1),id(m+1),lindx(nsubmax),xlindx(m+1),
  iw(m,5),xlnz(m+1),iwork(iwmax),xsuper(m+1),je(nnzemax),ie(m+1)
double precision small, a(nnza),ao(nnza),dsub(nnzemax+1),d(nnzdmax),
  lnz(nnzlmax),c(n),y(m),wwm(m,3),tmpvec(tmpmax),
  wwn(n,14),x(n),s(n),u(n),e(nnzemax),b(m),b1(m),wu(m,k)

do i=1,k{
    call dcopy(m,wu(1,i),1,a(nnza - m),1)
    call dcopy(m,b1,1,y,1)
    call slpfn(n,m,nnza,a,ja,ia,ao,jao,iao,nnzdmax,d,jd,id,
        dsub,jdsub,nsubmax,lindx,xlindx,nnzlmax,lnz,
        xlnz,iw(1,1),iw(1,2),iwmax,iwork,iw(1,3),iw(1,4),
        xsuper,iw(1,5),tmpmax,tmpvec,wwm(1,2),cachsz,
        level,x,s,u,c,y,b,wwn(1,1),wwn(1,2),wwn(1,3),
        wwn(1,4),nnzemax,e,je,ie,wwm(1,3),wwn(1,5),wwn(1,6),
        wwn(1,7),wwn(1,8),wwn(1,9),wwn(1,10),wwn(1,11),
        wwn(1,12),wwn(1,13),wwn(1,14),wwm(1,1),small,ierr,maxit, time)
    call dcopy(m,y,1,wu(1,i),1)
}
return
end

#WARNING  --  needs considerable work to finish implementing sfn.control fix.
#-----------------------------------------------------------------------------
# Storage parameters and other controls for sfn fitting
#	nsubmax = upper bound of dimension of lindx
#	tmpmax = upper bound for the dimension of tmpvec
#	nnzlmax = maximum number of non-zero elements in L
#	cachsz = size of the cache memory; it's machine dependent
#	small = the tolerance used to check for convergence
#	maxiter = allowed limit for sfn iterations
#	warn.mesg = flag to control printing of warnings
#


sfn.control <- function( nsubmax = NULL, tmpmax = NULL, nnzlmax = NULL, 
    cachsz = 64, small = 1e-6, maxiter=100, warn.mesg=TRUE)
    list(nsubmax = nsubmax, tmpmax = tmpmax, nnzlmax = nnzlmax, cachsz = cachsz,
        small = small, maxiter = maxiter, warn.mesg = warn.mesg)

#################################################################
# Interface for a sparse implementation of LMS interior point method
#	a = structure of the design matrix A' stored in csr format
#	y = pseudo response vector
#	tau = the desired regression quantile
#       rhs = the rhs of the dual problem -- specify at your own risk
#################################################################


rq.fit.sfn <- function(a,y,tau=.5, rhs = (1-tau)*c(t(a) %*% rep(1,length(y))), control)
{
	y <- -y
	n <- length(y)
	m <- a@dimension[2]
	if(n != a@dimension[1])
	     stop("Dimensions of design matrix and the response vector not compatible")
	u <- rep(1,length=n)
	x <- rep((1-tau),length=n)
	nnzdmax <- nnza <- a@ia[n+1]-1
	iwmax <- 7*m+3
	ao <- t(a)
	e <- ao %*% a
	nnzemax <- e@ia[m+1]-1
        ctrl <- sfn.control()
        if (!missing(control)) {
             control <- as.list(control)
             ctrl[names(control)] <- control
             }
	nsubmax <- ctrl$nsubmax
	tmpmax <- ctrl$tmpmax
	nnzlmax <- ctrl$nnzlmax
        if (is.null(ctrl$nsubmax)) ctrl$nsubmax <- nsubmax <- nnzemax
        if (is.null(ctrl$tmpmax)) ctrl$tmpmax <- tmpmax <- 6 * m
        if (is.null(ctrl$nnzlmax)) ctrl$nnzlmax <- nnzlmax <- 4 * nnzdmax
	wwm <- vector("numeric",3*m)
	s <- u - x
	b1 <- solve(e, ao %*% y, tmpmax=tmpmax,nnzlmax=nnzlmax,nsubmax=nsubmax)
	r <- y - a %*% b1
	z <- ifelse(abs(r)<ctrl$small,(r*(r>0)+ctrl$small),r*(r>0))
	w <- z - r
	wwn <- matrix(0,n,14)
	wwn[,1] <- r
	wwn[,2] <- z
	wwn[,3] <- w
	fit <- .Fortran("srqfn",
		n = as.integer(n),
		m = as.integer(m),
		nnza = as.integer(nnza),
		a = as.double(a@ra),
		ja = as.integer(a@ja),
		ia = as.integer(a@ia),
		ao = as.double(ao@ra),
		jao = as.integer(ao@ja),
		iao = as.integer(ao@ia),
		nnzdmax = as.integer(nnzdmax),
		d = double(nnzdmax),
		jd = integer(nnzdmax),
		id = integer(m+1),
		dsub = double(nnzemax+1),
		jdsub = integer(nnzemax+1),
		nnzemax = as.integer(nnzemax),
		e = as.double(e@ra),
		je = as.integer(e@ja),
		ie = as.integer(e@ia),
		nsubmax = as.integer(nsubmax),
		lindx = integer(nsubmax),
		xlindx = integer(m+1),
		nnzlmax = as.integer(nnzlmax),
		lnz = double(nnzlmax),
		xlnz = integer(m+1),
		iw = integer(m*5),
		iwmax = as.integer(iwmax),
		iwork = integer(iwmax),
		xsuper = integer(m+1),
		tmpmax = as.integer(tmpmax),
		tmpvec = double(tmpmax),
		wwm = as.double(wwm),
		wwn = as.double(wwn),
		cachsz = as.integer(ctrl$cachsz),
		level = as.integer( 8 ),
		x = as.double(x),
		s = as.double(s),
		u = as.double(u),
		c = as.double(y),
		sol = as.double(b1),
		rhs = as.double(rhs),
		small = as.double(ctrl$small),
		ierr = integer(1),
		maxiter = as.integer(ctrl$maxiter),
		time = double(7))[c("sol","ierr","maxiter","time")]
        ierr <- fit$ierr
	if(!(ierr==0) && ctrl$warn.mesg)
            warning(sfnMessage(ierr))
	coefficients <- -fit$sol
	residuals <- -y - a %*% coefficients
	list(coefficients = coefficients,
	     residuals = residuals,
	     control = ctrl,
             ierr = ierr,
             it = fit$maxiter)

}
#------------------------------------------------------------------------------
#################################################################
# Interface for a sparse implementation of LMS interior point method
#	x = structure of A1' stored in csr format
#	y = pseudo response vector
#	m = column dimension of A1' in full mode
#       R = structure of the constraint matrix A2' stored in csr format
#	r = rhs of the inequality constraints
#	tau = the desired regression quantile
#       rhs = the rhs of the dual problem -- specify at your own risk
#################################################################


rq.fit.sfnc <- function(x, y, R, r, tau = 0.5,
                        rhs = (1-tau)*c(t(x) %*% rep(1,length(y))),control)
{
	y <- -y
	r <- -r
	n1 <- length(y)
	m <- x@dimension[2]
	if(n1 != x@dimension[1])
            stop("The design matrix A1' and response vector y are not compatible")
	n2 <- length(r)
	if(n2 != R@dimension[1])
            stop("The constraint matrix A2' and constraint rhs are not compatible")
	maxn1n2 <- max(n1,n2)
	u <- rep(1,length=n1)
	x1 <- rep(1-tau,length=n1)
	x2 <- rep(1,length=n2)
	wwm <- vector("numeric",6*m)
	wwm[1:m] <- rhs
	nnzx <- x@ia[x@dimension[1]+1]-1
	nnzR <- R@ia[R@dimension[1]+1]-1
	nnzdmax <- max(nnzx,nnzR)
	iwmax <- 7*m+3
	ao1 <- t(x)
	ao2 <- t(R)
	e <- ao1 %*% x
	g <- ao2 %*% R
	h <- e + g
	nnzemax <- e@ia[e@dimension[1]+1]-1
	nnzgmax <- g@ia[g@dimension[1]+1]-1
	nnzhmax <- h@ia[h@dimension[1]+1]-1
        ctrl <- sfn.control()
        if (!missing(control)) {
             control <- as.list(control)
             ctrl[names(control)] <- control
             }
	nsubmax <- ctrl$nsubmax
	tmpmax <- ctrl$tmpmax
	nnzlmax <- ctrl$nnzlmax
        if (is.null(ctrl$nsubmax)) nsubmax <- nnzhmax
        if (is.null(ctrl$tmpmax)) tmpmax <- 6 * m
        if (is.null(ctrl$nnzlmax)) nnzlmax <- 4 * nnzdmax
	s <- u - x1
	chol.o <- chol(e, tmpmax=tmpmax, nsubmax=nsubmax, nnzlmax=nnzlmax)
	b <- backsolve(chol.o, ao1 %*% y)
	r1 <- y - x %*% b
	z1 <- ifelse(abs(r1) < ctrl$small, (r1*(r1>0)+ctrl$small), r1*(r1>0))
	w <- z1 - r1
	z2 <- rep(1,n2)
	wwn1 <- matrix(0,n1,10)
	wwn1[,1] <- z1
	wwn1[,2] <- w
	wwn2 <- matrix(0,n2,7)
	wwn2[,2] <- z2
	fit <- .Fortran("srqfnc",
		n1 = as.integer(n1),
		m = as.integer(m),
		nnzx = as.integer(nnzx),
		x = as.double(x@ra),
		jx = as.integer(x@ja),
		ix = as.integer(x@ia),
		ao1 = as.double(ao1@ra),
		jao1 = as.integer(ao1@ja),
		iao1 = as.integer(ao1@ia),
		n2 = as.integer(n2),
		nnzR = as.integer(nnzR),
		R = as.double(R@ra),
		jR = as.integer(R@ja),
		iR = as.integer(R@ia),
		ao2 = as.double(ao2@ra),
		jao2 = as.integer(ao2@ja),
		iao2 = as.integer(ao2@ia),
		nnzdmax = as.integer(nnzdmax),
		d = double(nnzdmax),
		jd = integer(nnzdmax),
		id = integer(m+1),
		dsub = double(nnzhmax+1),
		jdsub = integer(nnzhmax+1),
		nnzemax = as.integer(nnzemax),
		e = as.double(e@ra),
		je = as.integer(e@ja),
		ie = as.integer(e@ia),
		nnzgmax = as.integer(nnzgmax),
		g = double(nnzgmax),
		jg = integer(nnzgmax),
		ig = integer(m+1),
		nnzhmax = as.integer(nnzhmax),
		h = double(nnzhmax),
		jh = integer(nnzhmax),
		ih = integer(m+1),
		nsubmax = as.integer(nsubmax),
		lindx = integer(nsubmax),
		xlindx = integer(m+1),
		nnzlmax = as.integer(nnzlmax),
		lnz = double(nnzlmax),
		xlnz = integer(m+1),
		iw = integer(m*5),
		iwmax = as.integer(iwmax),
		iwork = integer(iwmax),
		xsuper = integer(m+1),
		tmpmax = as.integer(tmpmax),
		tmpvec = double(tmpmax),
		maxn1n2 = as.integer(maxn1n2),
		ww1 = double(maxn1n2),
		wwm = as.double(wwm),
		wwn1 = as.double(wwn1),
		wwn2 = as.double(wwn2),
		cachsz = as.integer(ctrl$cachsz),
		level = as.integer( 8 ),
		x1 = as.double(x1),
		x2 = as.double(x2),
		s = as.double(s),
		u = as.double(u),
		c1 = as.double(y),
		c2 = as.double(r),
		sol = as.double(b),
		small = as.double(ctrl$small),
		ierr = integer(1),
		maxiter = as.integer(ctrl$maxiter),
		time = double(7))[c("sol","ierr","maxiter","time")]
        ierr <- fit$ierr
	if(ierr == 13)# stop()
            stop("Increase nnzh.factor")
	if(!(ierr==0) && ctrl$warn.mesg)
            warning(sfnMessage(ierr))
	coefficients <- -fit$sol
	residuals <- -y - x %*% coefficients
	list(coefficients = coefficients,
	     residuals = residuals,
	     control = ctrl,
             ierr = ierr,
             it = fit$maxiter)
}
"sfnMessage" <- function(ierr){
   switch(ierr,
       "insufficient storage (work space) when calling extract\n",
       "nnzd > nnzdmax\n",
       "insufficient storage in iwork when calling ordmmd\n",
       "insufficient storage in iwork when calling sfinit\n",
       "nnzl > nnzlmax when calling sfinit\n",
       "nsub > nsubmax when calling sfinit\n",
       "insufficient work space in iwork when calling symfct\n",
       "inconsistancy in input when calling symfct\n",
       "tmpsiz > tmpmax when calling bfinit; increase tmpmax\n",
       "nonpositive diagonal encountered blkfct() matrix is not positive definite\n",
       "insufficient work storage in tmpvec when calling blkfct\n",
       "insufficient work storage in iwork  when calling blkfct\n",
	"impossible error condition",
	"impossible error condition",
	"impossible error condition",
	"impossible error condition",
       "tiny diagonals replaced with Inf when calling blkfct\n")
}

#This is a simple recursive least squares routine Reference:  Harvey TSM p. 100
#
subroutine rls(n,p,x,y,b,A,Ax)
integer n,p
double precision x(p,n),y(n),b(p,n),A(p,p),Ax(p)
double precision zero,one,mone,f,r,ddot
data one/1.d0/
data mone/-1.d0/
data zero/0.d0/
#
#On input:
#
#	A = crossprod(x[1:p,1:p))^{-1} 
#	b(,p) = A crossprod(x[1:p,1:p],y[1:p])  
#
do i = (p+1),n {
	call dgemv('N',p,p,one,A,p,x(1,i),1,zero,Ax,1)
	f = one + ddot(p,x(1,i),1,Ax,1)
	r = (y(i)-ddot(p,x(1,i),1,b(1,i-1),1))/f
	call daxpy(p,one,b(1,i-1),1,b(1,i),1)
	call daxpy(p,r,Ax,1,b(1,i),1)
	call dger(p,p,mone/f,Ax,1,Ax,1,A,p)
	}
return
end
		

"combos" <- function(n,p){
    if(length(n) != 1){ 
        n <- n[1]
        warning("Using first element as n")
        }
    if(length(p) != 1){ 
        p <- p[1]
        warning("Using first element as p")
        }
    if(n != as.integer(n)){
        warning("Coercing n to integer")
        n <- as.integer(n)
        }
    if(p != as.integer(p)){
        warning("Coercing p to integer")
        p <- as.integer(p)
        }
    if(p > n) stop("p is greater than n")
    m <- choose(n,p)
    z <- .Fortran("combin",
		as.integer(n),
		as.integer(p),
		as.integer(m),
		a = integer(p*m),
		integer(n),
		integer(n),
		integer(n))
	matrix(z$a,p)
	}

# New Version of Brute Force Algorithm for the Powell Estimator
#
# Roger Koenker:  last revision: 12 February 2008 
#
# Problem:  || min{Ax,c} - b ||_tau  = min!
#
# The matrix H is p by m matrix of the n choose p basis indices h on input.
# When called it is assumed (!!) that U = A[H[,1],]^{-1} and xh = U b[H[,1]].
subroutine brutpow(n,p,m,H,A,b,c,x,tau,U,xh,d,jminz,nflag)

integer n,p,m
double precision x(p),A(n,p),b(n),c(n)
double precision U(p,p),d(p),xh(p)
double precision zero, one,tau,pow,minz,z
integer H(p,m),k,findk,jminz,nflag

PARAMETER(zero = 0.0d0, one = 1.d0)

jminz = 1
minz  = pow(n,p,x,A,b,c,tau)
do j = 2,m {
	k = findk(p,H(1,j),H(1,j-1))
	if(k == 0)
		{nflag = 4; return}
	call pivot(n,p,H(1,j-1),H(k,j),H(k,j-1),A,U,d,xh,nflag)
	if(nflag > 0)
		return
	do i = 1,p{
		xh(i) = b(H(i,j))
		}
	call dgemv('N',p,p,one,U,p,xh,1,zero,x,1)
	z = pow(n,p,x,A,b,c,tau)
	if(z < minz) {
		minz  = z
		jminz = j
		}
	}
return
end

#####################################################

integer function findk(p,h,g)
integer p,k,h(p),g(p)
findk = 0
do k = 1,p{
	if(h(k) != g(k)) 
		{findk = k; break}
	}
return
end
	

C Output from Public domain Ratfor, version 1.05
      subroutine penalty(n,m,q,x,y,bnd,tlist,tlptr,tlend,rax,jax,ned,eps
     *,ierr)
      integer n,m,q,lp,lpl,ned,ierr
      integer bnd(n),tlist(q),tlptr(q),tlend(n),n4(4),p4(4),jax(m)
      double precision x(n),y(n),rax(m),eps
      double precision x4(4),y4(4),g4(4)
      logical orient
      ned = 0
      do23000 i=1,n
      lpl = tlend(i)
      lp = lpl
23002 continue
      lp = tlptr(lp)
      j = iabs(tlist(lp))
      if(j .gt. i)then
      n4(1) = i
      n4(2) = j
      call fadjs(n4,n,q,tlist,tlptr,tlend)
      if(bnd(i)*bnd(j) .eq. 0)then
      ned = ned + 1
      do23009 k = 1,4
      x4(k) = x(n4(k))
      y4(k) = y(n4(k))
23009 continue
23010 continue
      if(orient(x4,y4))then
      call iswap(1,n4(3),1,n4(4),1)
      call dswap(1,x4(3),1,x4(4),1)
      call dswap(1,y4(3),1,y4(4),1)
      endif
      call ggap(x4,y4,g4,eps,ierr)
      if(ierr .eq. 1)then
      return
      endif
      call srtpai(n4,1,p4,1,4)
      do23015 k = 1,4
      rax((ned - 1)*4 + k) = g4(p4(k))
      jax((ned - 1)*4 + k) = n4(p4(k))
23015 continue
23016 continue
      if(ned*4 .gt. m)then
      return
      endif
      endif
      endif
      if(lp .eq. lpl)then
      goto 23004
      endif
23003 goto 23002
23004 continue
23000 continue
23001 continue
      return
      end
      logical function orient(x,y)
      double precision x(4), y(4)
      orient = (y(2) -y(1))*(x(3)-x(4))+(x(1)-x(2))*(y(3)-y(4)) .gt. 0
      return
      end
      subroutine fadjs(n4,n,q,tlist,tlptr,tlend)
      integer n,q,vp,vpl,v,v0,match
      integer n4(4),tlist(q),tlptr(q),tlend(n)
      match = 0
      vpl = tlend(n4(1))
      vp = vpl
      k = 0
23021 continue
      k = k+1
      vp = tlptr(vp)
      v = tlist(vp)
      if(k.gt.1 .and. iabs(v) .eq. n4(2))then
      n4(3) = iabs(v0)
      match = 1
      goto 23022
      endif
      if(match .gt. 0)then
      n4(4) = iabs(v)
      goto 23023
      endif
      v0 = v
23022 goto 23021
23023 continue
      return
      end
      subroutine ggap(x,y,g,eps,ierr)
      double precision x(4),y(4),g(4),w(2,4),h(2),d1,d2,eps
      d1 = -x(2) * y(1) + x(3) * y(1) + x(1) * y(2) - x(3) * y(2) - x(1)
     * * y(3) + x(2) * y(3)
      d2 = -x(2) * y(1) + x(4) * y(1) + x(1) * y(2) - x(4) * y(2) - x(1)
     * * y(4) + x(2) * y(4)
      if(dabs(d1) .lt. eps .or. dabs(d2) .lt. eps)then
      ierr = 1
      return
      endif
      h(1) = -(y(1) - y(2))
      h(2) = (x(1) - x(2))
      w(1, 1) = (y(2) - y(3))/d1 - (y(2) - y(4))/d2
      w(2, 1) = (x(3) - x(2))/d1 - (x(4) - x(2))/d2
      w(1, 2) = (y(3) - y(1))/d1 - (y(4) - y(1))/d2
      w(2, 2) = (x(1) - x(3))/d1 - (x(1) - x(4))/d2
      w(1, 3) = (y(1) - y(2))/d1
      w(2, 3) = (x(2) - x(1))/d1
      w(1, 4) = (y(2) - y(1))/d2
      w(2, 4) = (x(1) - x(2))/d2
      do23030 i = 1,4
      g(i) = h(1)*w(1,i)+h(2)*w(2,i)
23030 continue
23031 continue
      ierr = 0
      return
      end

C Output from Public domain Ratfor, version 1.05
      subroutine pfnb(n,p,m,a,y,q,r,b,band,m0,d,u,wn,wp, aa,yy,slo,shi,r
     *hs,glob,ghib,nit,info)
      integer n,p,m,kk(2),mm,m0,nit(5,m),info,sumbad
      integer i,j,k,slo(n),shi(n),ifix,ibad
      logical notopt
      double precision a(p,n),y(n),r(n),q(m),d(n),u(n),b(p,m)
      double precision wn(n,9), wp(p,(p+3)),band(n),qk(2)
      double precision glob(p),ghib(p),aa(p,n),yy(n),rhs(p)
      double precision zero,one,tau,beta,eps,big,fm,fn
      parameter(zero = 0.0d0)
      parameter(one = 1.0d0)
      parameter(beta = 0.99995d0)
      parameter(big = 1.0d+10)
      parameter(eps = 1.0d-06)
      do23000 iq = 1,m 
      notopt = .true.
      tau = q(iq)
      mm = m0
      ifix = 0
      ibad = 0
23002 if(notopt)then
      ibad = ibad + 1
      fm = mm
      fn = dble(n)
      kk(1) = int(n * dmax1(1./fn, tau - fm/(2 * fn))) + 1
      kk(2) = int(n * dmin1(tau + fm/(2. * fn), (fn - 1)/fn))
      do23004 i = 1,n
      u(i) = r(i)/band(i)
23004 continue
23005 continue
      call kuantiles(kk,2,n,u)
      qk(1) = u(kk(1))
      qk(2) = u(kk(2))
      call iphil(n,0,slo)
      call iphil(n,0,shi)
      do23006 i = 1,n
      if(r(i) .lt. (band(i) * qk(1)))then
      slo(i) = 1
      else
      if(r(i) .gt. (band(i) * qk(2)))then
      shi(i) = 1
      endif
      endif
23006 continue
23007 continue
23012 if(notopt)then
      ifix = ifix + 1
      call dphil(p,zero,glob)
      call dphil(p,zero,ghib)
      call dphil(n,one,d)
      call dphil(n,one,u)
      k = 0
      do23014 i = 1,n
      if(slo(i) .eq. 0 .and. shi(i) .eq. 0)then
      k = k + 1
      call dcopy(p,a(1,i),1,aa(1,k),1)
      yy(k) = -y(i)
      else
      if(slo(i) .eq. 1)then
      do23020 j = 1,p
      glob(j) = glob(j) + a(j,i)
23020 continue
23021 continue
      else
      if(shi(i) .eq. 1)then
      do23024 j = 1,p
      ghib(j) = ghib(j) + a(j,i)
23024 continue
23025 continue
      endif
      endif
      endif
23014 continue
23015 continue
      call dcopy(p,glob,1,aa(1,k+1),1)
      call dcopy(p,ghib,1,aa(1,k+2),1)
      yy(k+1) = big
      yy(k+2) = -big
      call dgemv('N',p,k+2,one-tau,aa,p,d,1,zero,rhs,1)
      call dscal(k+2,zero,wn,1)
      call daxpy(k+2,one-tau,u,1,wn,1)
      call rqfnb(k+2,p,aa,yy,rhs,d,u,beta,eps,wn,wp,nit(1,iq),info)
      call dcopy(p,wp,1,b(1,iq),1)
      call dcopy(n,y,1,r,1)
      call dgemv('T',p,n,one,a,p,b(1,iq),1,one,r,1)
      sumbad = 0
      do23026 i = 1,n 
      if((r(i) .gt. 0) .and. slo(i) .eq. 1)then
      slo(i) = 0
      sumbad = sumbad + 1
      endif
      if((r(i) .lt. 0) .and. shi(i) .eq. 1)then
      shi(i) = 0
      sumbad = sumbad + 1
      endif
23026 continue
23027 continue
      if(sumbad .gt. 0)then
      if(sumbad .gt. 0.1 * mm)then
      mm = min(2 * mm, n)
      goto 23013
      endif
      else
      notopt = .false.
      endif
      goto 23012
      endif
23013 continue
      nit(4,iq) = ifix
      nit(5,iq) = ibad
      goto 23002
      endif
23003 continue
23000 continue
23001 continue
      return
      end
      subroutine iphil(n,a,v)
      integer n,i,a,v(n)
      do23036 i = 1,n
      v(i) = a
23036 continue
23037 continue
      return
      end
      subroutine dphil(n,a,v)
      integer i,n
      double precision a,v(n)
      do23038 i = 1,n
      v(i) = a
23038 continue
23039 continue
      return
      end

ParetoTest <- function(formula, tau = 0.1, data = NULL, flavor = "Hill", 
		       m = 2, cicov = 0.9, ...) {
    if(flavor == "Hill") 
	z <-  Hill(formula, tau, data, ...)
    else if(flavor == "Pickands") 
	z <- Pickands(formula, tau, data, m = m, ...)
    else
	stop(paste("flavor", flavor, "not (yet) implemented")) 
    summary(z, ci = cicov, ...)
}

# Pickands Estimation and Inference for the Pareto Tail Exponent

Pickands <- function(formula, tau = 0.1, data = NULL, m = 2, ...){
    y <- model.response(model.frame(formula, data))
    X <- model.matrix(formula, data)
    Pickands.fit(X, y, tau, m, ...)
}
Pickands.fit <- function(X, y, tau, m, ...){
    if(tau > 0.5){ # Right tail
	X <- -X 
	y <- -y 
	tau = 1-tau
    }
    taus = c(tau, m * tau, 2 * m * tau)
    F <- rq(y ~ X - 1, tau = taus, ...)
    Xbar = colMeans(X)
    num = c(crossprod(Xbar, (F$coef[,3] - F$coef[,2])))
    denom = c(crossprod(Xbar, (F$coef[,2] - F$coef[,1])))
    xi <- (-1/log(m)) * log(num/denom)
    z <- list(xi = xi, tau = tau, m = m, F = F)
    class(z) <- "Pickands"
    z
}

summary.Pickands <- function(object, se = "boot", B = 200,  ci = 0.9, ...){
    tau <- object$tau
    xi <- object$xi
    m <- object$m
    X <- object$F$x
    y <- object$F$y
    n <- length(y)
    b1 <- object$F$coef[,1]
    bm <- object$F$coef[,2]
    Xbar <- colMeans(X)
    Xg <- X %*% (bm - b1)/c(crossprod(Xbar, bm - b1)) #aargh humbug!
    R <- rep(NA, B)
    for(i in 1:B){ # Parametric Bootstrap Default Method
	ys <- -(rexp(n)^(-xi) - 1)/xi * Xg
	R[i] <- Pickands.fit(X, ys, tau, m = m, ...)$xi
    }
    z <- rep(NA,6)
    z[1] <- xi
    z[2] <- xi - quantile(R - xi, .5, na.rm=TRUE)
    z[3] <- xi - quantile(R - xi, (1+ci)/2, na.rm=TRUE)
    z[4] <- xi - quantile(R - xi, (1-ci)/2, na.rm=TRUE)
    z[5] <- sqrt(var(R))
    z[6] <- B - sum(is.na(R))
    names(z) <- c("Estimate", "Bias-Corrected", paste("Lower ", 50*(1-ci),"%",sep=""),
	paste("Upper ", 50*(1+ci),"%",sep=""), "StdErr", "Sample Size")
    z <- list(z = z, tau = tau)
    class(z) = "summary.Pickands"
    z
}
print.Pickands <- function(x, ...){
    print(x$xi)
}
print.summary.Pickands <- function(x, ...){
    cat(paste("Pickands Estimation and Inference: tau = ", x$tau, 
	      "Effective Bootstrap Sample Size = ",x$z[6],"\n\n"))
    print(x$z[1:5])
}

if(FALSE){
# Test problem
require(quantreg)
n = 500
x = rnorm(n)
y = x + rt(n,2)
P = Pickands(y ~ x, .9)
S = summary(P)
}
# Hill Estimation and Inference for the Pareto Tail Exponent

Hill <- function(formula, tau = 0.1, data = NULL, ...){
    y <- model.response(model.frame(formula, data))
    X <- model.matrix(formula, data)
    Hill.fit(X, y, tau)
}
Hill.fit <- function(X, y, tau, ...){
    if(tau > 0.5){ # Right tail
	X <- -X 
	y <- -y 
	tau = 1-tau
    }
    F <- rq.fit(X, y, tau = tau, ...)
    yhat <- F$fitted.values
    z <- log(ifelse(y <= yhat, abs(y/yhat), 1))
    z <- z[is.finite(z)]
    z <- list(xi = sum(z)/(length(z)*tau), tau = tau, F = F)
    class(z) <- "Hill"
    z
}

summary.Hill <- function(object, se = "boot", B = 200, m = 2, ci = 0.9, ...){
    tau <- object$tau
    xi <- object$xi
    X <- object$F$x
    y <- object$F$y
    n <- length(y)
    b0 <- object$F$coef
    bm <- rq.fit(X, y, m * tau, ...)$coef 
    Xbar <- colMeans(X)
    Xg <- X %*% (bm - b0)/c(crossprod(Xbar, bm - b0)) #aargh humbug!
    R <- rep(NA, B)
    for(i in 1:B){ # Parametric Bootstrap Default Method
	ys <- -(rexp(n)^(-xi) - 1)/xi * Xg
	R[i] <- Hill.fit(X, ys, tau, ...)$xi
    }
    z <- rep(NA,6)
    z[1] <- xi
    z[2] <- xi - quantile(R - xi, .5, na.rm=TRUE)
    z[3] <- xi - quantile(R - xi, (1+ci)/2, na.rm=TRUE)
    z[4] <- xi - quantile(R - xi, (1-ci)/2, na.rm=TRUE)
    z[5] <- sqrt(var(R))
    z[6] <- B - sum(is.na(R))
    names(z) <- c("Estimate", "Bias-Corrected", paste("Lower ", 50*(1-ci),"%",sep=""),
	paste("Upper ", 50*(1+ci),"%",sep=""), "BS_StdErr", "Sample Size")
    z <- list(z = z, tau = tau)
    class(z) = "summary.Hill"
    z
}
print.Hill <- function(x, ...){
    print(x$xi)
}
print.summary.Hill <- function(x, ...){
    cat(paste("Hill Estimation and Inference: tau = ", x$tau, 
	      "Effective Bootstrap Sample Size = ",x$z[6],"\n\n"))
    print(x$z[1:5])
}

C Output from Public domain Ratfor, version 1.05
      subroutine profnb(n,p,m,a,y,t,r,d,u,wn,wp,b,nit, info)
      integer n,p,m,nit(3),info
      double precision a(p,n), y(n), t(m), b(p,m), r(p)
      double precision d(n), u(n), wn(n,9), wp(p,p+3)
      double precision zero, one, eps, beta
      parameter( zero = 0.0d0)
      parameter( one = 1.0d0)
      parameter( beta = 0.99995d0)
      parameter( eps = 1.0d-6)
      do23000 i = 1,m
      call dgemv('N',p,n,one-t(i),a,p,d,1,zero,r,1)
      call dscal(n,zero,wn,1)
      call daxpy(n,one-t(i),u,1,wn,1)
      call rqfnb(n,p,a,y,r,d,u,beta,eps,wn,wp,nit,info)
      if(info .ne. 0)then
      goto 23001
      endif
      do23004 j = 1,n
      u(j) = one
      d(j) = one
23004 continue
23005 continue
      call dcopy(p,wp,1,b(1,i),1)
23000 continue
23001 continue
      return
      end

C Output from Public domain Ratfor, version 1.0
      subroutine rqbr(m,nn,m5,n3,n4,a,b,t,toler,ift,x,e,s,wa,wb,nsol,nds
     *ol,sol,dsol,lsol,h,qn,cutoff,ci,tnmat,big,lci1)
      integer i,j,k,kl,kount,kr,l,lsol,m,m1,m2,m3,m4,m5,ift
      integer kd,in,nn,n4,idxcf
      integer n,n1,n2,n3,nsol,ndsol,out,s(m),h(nn,nsol)
      logical stage,test,init,iend,lup
      logical lci1,lci2,skip
      double precision a1,aux,b1,big,d,dif,pivot,smax,t,t0,t1,tnt,dk
      double precision min,max,toler,zero,half,one,two
      double precision b(m),sol(n3,nsol),a(m,nn),x(nn),wa(m5,n4),wb(m)
      double precision sum,e(m),dsol(m,ndsol)
      double precision qn(nn),cutoff,ci(4,nn),tnmat(4,nn),tnew,told,tn
      parameter( zero = 0.d0)
      parameter( one = 1.d0)
      parameter( two = 2.d0)
      n=nn
      ift = 0
      wa(m+2,nn+1) = one
      if(m5.ne.m+5)then
      ift = 3
      endif
      if(n3.ne.n+3)then
      ift = 4
      endif
      if(n4.ne.n+4)then
      ift = 5
      endif
      if(m.le.zero.or.n.le.zero)then
      ift = 6
      endif
      if(ift.le.two)then
      half = one/two
      iend = .true.
      lci2 = .false.
      lup = .true.
      skip = .false.
      idxcf = 0
      tnew = zero
      tn = zero
      m1 = m+1
      n1 = n+1
      n2 = n+2
      m2 = m+2
      m3 = m+3
      m4 = m+4
      do23010 j = 1,n
      x(j) = zero
23010 continue
23011 continue
      do23012 i = 1,m
      e(i) = zero
23012 continue
23013 continue
      if(t.lt.zero.or.t.gt.one)then
      t0 = one/(two*float(m))
      t1 = one-t0
      t = t0
      iend = .false.
      lci1 = .false.
      endif
23016 continue
      do23019 i = 1,m 
      k = 1
      do23021 j = 1,nn
      if(k.le.nn)then
      if(j.eq.idxcf)then
      skip = .true.
      else
      skip = .false.
      endif
      if(.not.skip)then
      wa(i,k) = a(i,j)
      k = k+1
      endif
      endif
23021 continue
23022 continue
      wa(i,n4) = n+i
      wa(i,n2) = b(i)
      if(idxcf .ne. 0)then
      wa(i,n3) = tnew*a(i,idxcf)
      else
      wa(i,n3) = zero
      endif
      wa(i,n1) = wa(i,n2)-wa(i,n3)
      if(wa(i,n1).lt.zero)then
      do23033 j = 1,n4
      wa(i,j) = -wa(i,j)
23033 continue
23034 continue
      endif
23019 continue
23020 continue
      do23035 j = 1,n 
      wa(m4,j) = j
      wa(m2,j) = zero
      wa(m3,j) = zero
      do23037 i = 1,m 
      aux = sign(one,wa(m4,j))*wa(i,j)
      wa(m2,j) = wa(m2,j)+aux*(one-sign(one,wa(i,n4)))
      wa(m3,j) = wa(m3,j)+aux*sign(one,wa(i,n4))
23037 continue
23038 continue
      wa(m3,j) = two*wa(m3,j)
23035 continue
23036 continue
      dif = zero
      init = .false.
      if(.not.lci2)then
      do23041 k = 1,n 
      wa(m5,k) = zero
      do23043 i = 1,m
      wa(m5,k) = wa(m5,k)+a(i,k)
23043 continue
23044 continue
      wa(m5,k) = wa(m5,k)/float(m)
23041 continue
23042 continue
      endif
      lsol = 1
      kount = 0
23045 continue
      do23048 j = 1,n
      wa(m1,j) = wa(m2,j)+wa(m3,j)*t
23048 continue
23049 continue
      if(.not.init)then
      stage = .true.
      kr = 1
      kl = 1
      go to 30
      endif
23052 continue
      stage = .false.
23055 continue
      max = -big
      do23058 j = kr,n 
      d = wa(m1,j)
      if(d.lt.zero)then
      if(d.gt.(-two))then
      goto 23058
      endif
      d = -d-two
      endif
      if(d.gt.max)then
      max = d
      in = j
      endif
23058 continue
23059 continue
      if(max.le.toler)then
      goto 23054
      endif
      if(wa(m1,in).le.zero)then
      do23070 i = 1,m4
      wa(i,in) = -wa(i,in)
23070 continue
23071 continue
      wa(m1,in) = wa(m1,in)-two
      wa(m2,in) = wa(m2,in)-two
      endif
23072 continue
      k = 0
      do23075 i = kl,m 
      d = wa(i,in)
      if(d.gt.toler)then
      k = k+1
      wb(k) = wa(i,n1)/d
      s(k) = i
      test = .true.
      endif
23075 continue
23076 continue
23079 continue
      if(k.le.0)then
      test = .false.
      else
      min = big
      do23084 i = 1,k
      if(wb(i).lt.min)then
      j = i
      min = wb(i)
      out = s(i)
      endif
23084 continue
23085 continue
      wb(j) = wb(k)
      s(j) = s(k)
      k = k-1
      endif
      if(.not.test.and.stage)then
      goto 23081
      endif
      if(.not.test)then
      goto 23047
      endif
      pivot = wa(out,in)
      if(wa(m1,in)-pivot-pivot.le.toler)then
      go to 10
      endif
      do23094 j = kr,n3 
      d = wa(out,j)
      wa(m1,j) = wa(m1,j)-d-d
      wa(m2,j) = wa(m2,j)-d-d
      wa(out,j) = -d
23094 continue
23095 continue
      wa(out,n4) = -wa(out,n4)
23080 goto 23079
23081 continue
      do23096 i = 1,m4 
      d = wa(i,kr)
      wa(i,kr) = wa(i,in)
      wa(i,in) = d
23096 continue
23097 continue
      kr = kr+1
      go to 20
10    do23098 j = kr,n3
      if(j.ne.in)then
      wa(out,j) = wa(out,j)/pivot
      endif
23098 continue
23099 continue
      do23102 i = 1,m3
      if(i.ne.out)then
      d = wa(i,in)
      do23106 j = kr,n3
      if(j.ne.in)then
      wa(i,j) = wa(i,j)-d*wa(out,j)
      endif
23106 continue
23107 continue
      endif
23102 continue
23103 continue
      do23110 i = 1,m3
      if(i.ne.out)then
      wa(i,in) = -wa(i,in)/pivot
      endif
23110 continue
23111 continue
      wa(out,in) = one/pivot
      d = wa(out,n4)
      wa(out,n4) = wa(m4,in)
      wa(m4,in) = d
      kount = kount+1
      if(.not.stage)then
      goto 23074
      endif
      kl = kl+1
      do23116 j = kr,n4 
      d = wa(out,j)
      wa(out,j) = wa(kount,j)
      wa(kount,j) = d
23116 continue
23117 continue
20    if(kount+kr.eq.n1)then
      goto 23057
      endif
30    max = -one
      do23120 j = kr,n
      if(abs(wa(m4,j)).le.n)then
      d = abs(wa(m1,j))
      if(d.gt.max)then
      max = d
      in = j
      endif
      endif
23120 continue
23121 continue
      if(wa(m1,in).lt.zero)then
      do23128 i = 1,m4
      wa(i,in) = -wa(i,in)
23128 continue
23129 continue
      endif
23073 goto 23072
23074 continue
23056 goto 23055
23057 continue
23053 goto 23052
23054 continue
      if(kr.eq.1)then
      do23132 j = 1,n 
      d = abs(wa(m1,j))
      if(d.le.toler.or.two-d.le.toler)then
      ift = 1
      wa(m2,nn+1) = zero
      go to 80
      endif
23132 continue
23133 continue
      endif
80    kount = 0
      sum = zero
      if(.not.lci2)then
      do23138 i = 1,kl-1 
      k = abs(wa(i,n4))
      x(k) = wa(i,n1)*sign(one,wa(i,n4))
23138 continue
23139 continue
      endif
      do23140 i = 1,n 
      kd = abs(wa(m4,i))-n
      dsol(kd,lsol) = one+wa(m1,i)/two
      if(wa(m4,i).lt.zero)then
      dsol(kd,lsol) = one-dsol(kd,lsol)
      endif
      if(.not.lci2)then
      sum = sum + x(i)*wa(m5,i)
      sol(i+3,lsol) = x(i)
      h(i,lsol) = kd
      endif
23140 continue
23141 continue
      do23146 i = kl,m 
      kd = abs(wa(i,n4))-n
      if(wa(i,n4).lt.zero)then
      dsol(kd,lsol) = zero
      endif
      if(wa(i,n4).gt.zero)then
      dsol(kd,lsol) = one
      endif
23146 continue
23147 continue
      if(.not.lci2)then
      sol(1,lsol) = smax
      sol(2,lsol) = sum
      sum = zero
      do23154 j=kl,m
      d = wa(j,n1)*sign(one,wa(j,n4))
      sum = sum + d*(smax + half*(sign(one,d) - one))
23154 continue
23155 continue
      sol(3,lsol) = sum
      do23156 i=1,m
      dsol(i,lsol+1) = dsol(i,lsol)
23156 continue
23157 continue
      endif
      if(lci2)then
      a1 = zero
      do23160 i = 1,m 
      a1 = a1+a(i,idxcf)*(dsol(i,lsol)+t-one)
23160 continue
23161 continue
      tn = a1/dsqrt(qn(idxcf)*t*(one-t))
      if(abs(tn).lt.cutoff)then
      if(lup)then
      smax = big
      else
      smax = -big
      endif
      do23166 i =1,kl-1 
      k = abs(wa(i,n4))
      sol(k,1) = wa(i,n2)*sign(one,wa(i,n4))
      sol(k,2) = wa(i,n3)*sign(one,wa(i,n4))/tnew
23166 continue
23167 continue
      do23168 i = kl,m 
      a1 = zero
      b1 = zero
      k = abs(wa(i,n4))-n
      l = 1
      do23170 j = 1,n
      if(j.eq.idxcf)then
      l = l+1
      endif
      a1 = a1 + a(k,l)*sol(j,1)
      b1 = b1 + a(k,l)*sol(j,2)
      l = l+1
23170 continue
23171 continue
      tnt = (b(k)-a1)/(a(k,idxcf)-b1)
      if(lup)then
      if(tnt.gt.tnew)then
      if(tnt.lt.smax)then
      smax = tnt
      out = i
      endif
      endif
      else
      if(tnt.lt.tnew)then
      if(tnt.gt.smax)then
      smax = tnt
      out = i
      endif
      endif
      endif
23168 continue
23169 continue
      if(lup)then
      told = tnew
      tnew = smax + toler
      ci(3,idxcf) = told - toler
      tnmat(3,idxcf) = tn
      if(.not.(tnew .lt. big-toler))then
      ci(3,idxcf) = big
      ci(4,idxcf) = big
      tnmat(3,idxcf) = tn
      tnmat(4,idxcf) = tn
      lup = .false.
      go to 70
      endif
      else
      told = tnew
      tnew = smax - toler
      ci(2,idxcf) = told + toler
      tnmat(2,idxcf) = tn
      if(.not.(tnew .gt. -big+toler))then
      ci(2,idxcf) = -big
      ci(1,idxcf) = -big
      tnmat(2,idxcf) = tn
      tnmat(1,idxcf) = tn
      lup = .true.
      go to 60
      endif
      endif
      do23190 i = 1,m
      wa(i,n3) = wa(i,n3)/told*tnew
      wa(i,n1) = wa(i,n2) - wa(i,n3)
23190 continue
23191 continue
      do23192 j = kr,n3
      d = wa(out,j)
      wa(m1,j) = wa(m1,j) -d -d
      wa(m2,j) = wa(m2,j) -d -d
      wa(out,j) = -d
23192 continue
23193 continue
      wa(out,n4) = -wa(out,n4)
      init = .true.
      else
      if(lup)then
      ci(4,idxcf) = tnew - toler
      tnmat(4,idxcf) = tn
      lup = .false.
      go to 70
      else
      ci(1,idxcf) = tnew + toler
      tnmat(1,idxcf) = tn
      lup = .true.
      go to 60
      endif
      endif
      endif
      if((iend).and.(.not.lci2))then
      go to 40
      endif
      if(.not.lci2)then
      init = .true.
      lsol = lsol+1
      do23200 i = 1,m
      s(i) = zero
23200 continue
23201 continue
      do23202 j = 1,n
      x(j) = zero
23202 continue
23203 continue
      smax = two
      do23204 j = 1,n 
      b1 = wa(m3,j)
      a1 = (-two-wa(m2,j))/b1
      b1 = -wa(m2,j)/b1
      if(a1.ge.t)then
      if(a1.lt.smax)then
      smax = a1
      dif = (b1-a1)/two
      endif
      endif
      if(b1.gt.t)then
      if(b1.lt.smax)then
      smax = b1
      dif = (b1-a1)/two
      endif
      endif
23204 continue
23205 continue
      tnt = smax+toler*(one+abs(dif))
      if(tnt.ge.t1+toler)then
      iend = .true.
      endif
      t = tnt
      if(iend)then
      t = t1
      endif
      endif
23046 goto 23045
23047 continue
      wa(m2,nn+1) = two
      ift = 2
      go to 50
40    if(lsol.gt.2)then
      sol(1,1) = zero
      sol(3,1) = zero
      sol(1,lsol) = one
      sol(3,lsol) = zero
      do23220 i = 1,m 
      dsol(i,1) = one
      dsol(i,lsol) = zero
      dsol(i,lsol+1) = zero
23220 continue
23221 continue
      endif
      l = kl-1
      do23222 i = 1,l
      if(wa(i,n1).lt.zero)then
      do23226 j = kr,n4
      wa(i,j) = -wa(i,j)
23226 continue
23227 continue
      endif
23222 continue
23223 continue
50    sum = zero
      if(.not.lci2)then
      do23230 i = kl,m 
      k = abs(wa(i,n4))
      d = wa(i,n1)*sign(one,wa(i,n4))
      sum = sum+d*sign(one,d)*(half+sign(one,d)*(t-half))
      k = k-n
      e(k) = d
23230 continue
23231 continue
      wa(m2,n2) = kount
      wa(m1,n2) = n1-kr
      wa(m1,n1) = sum
      endif
      if(wa(m2,nn+1).eq.two)then
      goto 23018
      endif
      if(.not.lci1)then
      goto 23018
      endif
      if(.not.(.not.lci2))goto 23234
      lci2 = .true.
      n = nn-1
      n1 = n+1
      n2 = n+2
      n3 = n+3
      n4 = n+4
60    idxcf = idxcf+1
      if(.not.(idxcf.gt.nn))goto 23236
      goto 23018
23236 continue 
70    if(.not.(lup))goto 23238
      tnew = x(idxcf)+toler
      told = tnew
      ci(3,idxcf) = x(idxcf)
      tnmat(3,idxcf) = zero
      goto 23239
23238 continue
      tnew = x(idxcf)-toler
      told = tnew
      ci(2,idxcf) = x(idxcf)
      tnmat(2,idxcf) = zero
23239 continue
23234 continue
23017 goto 23016
23018 continue
      do23242 i=1,m
      dsol(i,lsol) = dsol(i,lsol+1)
23242 continue
23243 continue
      endif
      return
      end

c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c
c  Function to obtain the step length
c
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c
      subroutine bound(x,dx,s,ds,z,dz,w,dw,n,beta,deltap,deltad)
c
      integer n
      double precision x(n),dx(n),s(n),ds(n),z(n),dz(n),w(n),dw(n)
      double precision deltap,deltad,dmin1,big,one,beta
      parameter (big = 1.0d20, one = 1.0d0)
      deltap = big
      deltad = big
      do i=1,n
         if(dx(i) .lt. 0) deltap = dmin1(deltap, -x(i)/dx(i))
         if(ds(i) .lt. 0) deltap = dmin1(deltap, -s(i)/ds(i))
         if(dz(i) .lt. 0) deltad = dmin1(deltad, -z(i)/dz(i))
         if(dw(i) .lt. 0) deltad = dmin1(deltad, -w(i)/dw(i))
      enddo
      deltap = dmin1(beta*deltap,one)
      deltad = dmin1(beta*deltad,one)
      return
      end

C Output from Public domain Ratfor, version 1.05
      subroutine kuantile(k,m,n,x)
      integer i,j,k(m),m,n
      double precision x(n)
      j = 0
      do23000 i = 1,m
      call dsel05(k(i)-j,n-j,x(j+1))
      j = k(i)
23000 continue
23001 continue
      return
      end

"rqProcess" <-
function (formula, data, taus, nullH = "location", ...) 
{
    z <- summary(f <- lm(formula, data = data, x = TRUE))
    xbar <- apply(f$x,2,mean)
    vars <- names(z$coef[-1, 1])
    p <- length(z$coef[, 1])
    n <- length(z$resid)
    Jinv <- z$cov.unscaled
    pivot <- any(taus < 0) || any(taus > 1)
    if(!pivot){ #grid method
	if(abs(diff(range(diff(taus)))) > sqrt(.Machine$double.eps))
		stop("rqProcess must be evaluated on equally spaced taus")
	ntaus <- length(taus)
	z <- summary(rq(formula, data = data, tau = taus, method = "fn"), covariance = TRUE, ...)
	coefs = t(sapply(z, coefficients)[1:p,])
	Cov = array(sapply(z, function(x) x$cov), c(p,p,ntaus))
	qtaus <- coefs %*% xbar
	Vhat <- t(coefs)[-1,,drop=FALSE]
	vhat <- t(coefs)[-1,,drop=FALSE]
	J <- solve(Jinv)
	p <- nrow(J)
	if (nullH == "location-scale") {
	    f <- lm(coefs[,-1] ~ coefs[,1]) 
	    b <- matrix(f$coef,2,p-1)[2,]
	    R <- matrix(f$resid,ntaus,p-1)
	    for (j in 1:length(taus)) {
		V <- Cov[, , j]
		v <- V[-1, -1] + V[1, 1] * outer(b,b) - 
		    outer(V[-1, 1], b) - t(outer(V[-1, 1], b))
		v <- solve(v)
		v <- chol(0.5 * (v + t(v)))
		Vhat[,j] <- v %*% R[j,]
		for (i in 2:p) {
		    v <- V[i, i] + V[1, 1] * b[i-1]^2 - 2 * V[i, 1] * b[i-1]
		    vhat[i-1,j] <- R[j, i-1]/sqrt(v)
		    }
		}
	    }
	else if (nullH == "location") {
	    b <- apply(coefs, 2, mean)
	    R <- t(coefs) - b
	    for (j in 1:length(taus)) {
		V <- Cov[, , j] 
		A <- solve(V[-1, -1,drop=FALSE])
		B <- chol(0.5 * (A + t(A)))
		Vhat[,j] <- B %*% R[-1, j,drop=FALSE]
		vhat[,j] <- R[-1, j,drop=FALSE] / (sqrt(diag(V))[-1])
		}
	    }
	}
   else{
       stop("pivot method is now deprecated for Ktest")
	}
    dimnames(Vhat) <- list(vars, NULL)
    dimnames(vhat) <- list(vars, NULL)
    x <- list(taus = taus, qtaus = qtaus, Vhat = Vhat, vhat = vhat, n = n)
    class(x) <- "rqProcess"
    x
}
"KhmaladzeTest"  <-
function (formula, data = NULL, taus = 1:99/100, nullH = "location", 
	trim = c(0.05, 0.95), h = 1, ...) 
{
    gdot = function(x, taus = 1:999/1000, h = -1){
	Qt = quantile(x, taus)
	z = akj(x, Qt, h = h)
	Qf = approxfun(taus, Qt)
	den = approxfun(Qt, z$den)
	psi = approxfun(Qt, z$psi)
	gdot0 = den(Qf(taus))
	gdot1 = psi(Qf(taus))
	gdot2 = Qf(taus) * gdot1 + 1
	cbind(gdot0, gdot1, gdot2)
    }
    f <- rqProcess(formula, data = data, taus=taus, nullH = nullH, ...)
    G = gdot(f$qtaus, taus, h = 1)
    ntaus = length(taus)
    Vtil = Vhat = G[,1] * f$Vhat
    vtil = vhat = G[,1] * f$vhat
    if(nullH == "location") X = G[-ntaus,2] 
    else X = G[-ntaus,2:3]
    for(i in 1:nrow(Vhat)){
	Y = Vhat[i,-1]
	y = vhat[i,-1]
	B = lm.fit.recursive(X,Y)
	b = lm.fit.recursive(X,y)
	Vtil[i,-1] = Y - diag(cbind(1,X) %*% B)
	vtil[i,-1] = y - diag(cbind(1,X) %*% b)
    }
    trim <- (f$taus >= trim[1] & f$taus <= trim[2])
    Tvtil <- (vtil - vtil[, 2])/sqrt(max(f$taus) - min(f$taus))
    TVtil <- apply(abs(Vtil - Vtil[, 2])/
	sqrt(max(f$taus) - min(f$taus)), 2, "sum")[trim]
    Tn <-  max(TVtil)
    THn <- apply(abs(Tvtil[, trim,drop = FALSE]), 1, max)
    x <- list(nullH = nullH, Tn = Tn, THn = THn, taus = taus, 
	      Vhat = t(Vhat), Vtil = t(Vtil))
    class(x) <- "KhmaladzeTest"
    x
}
print.KhmaladzeTest = function(x, ...) {
    cat("\nTest of H_0:", x$nullH, "\n")
    cat("\nJoint Test Statistic:", x$Tn, "\n")
    cat("\nComponent Test Statistics:", x$THn, "\n")
}
plot.KhmaladzeTest = function(x, ...) {
    dev.new(height = 6, width = 10)
    par(mfrow = c(1,2))
    matplot(x$taus, x$Vhat, xlab = expression(tau), ylab = expression(hat(v)[n](tau)), type = "l")
    title("Parametric QR Process")
    matplot(x$taus, x$Vtil, xlab = expression(tau), ylab = expression(tilde(v)[n](tau)), type = "l")
    title("Transformed Parametric QR Process")
    par(mfrow = c(1,1))
}

c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c
c Subroutine to perform Cholesky factorization
c
c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
      subroutine chlfct(m,xlindx,lindx,invp,perm,iwork,nnzdsub,jdsub,
     &                  colcnt,nsuper,snode,xsuper,nnzlmax,nsubmax,
     &                  xlnz,lnz,id,jd,d,cachsz,tmpmax,level,
     &                  tmpvec,split,ierr,it,timewd)
      integer m,ierr,nsub,iwsiz,nnzdsub,nnzl,nsuper,nnzlmax,nsubmax,
     &        tmpsiz,cachsz,tmpmax,level,it,
     &        xlindx(*),lindx(*),invp(*),perm(*),iwork(*),jdsub(*),
     &        colcnt(*),snode(*),xsuper(*),xlnz(*),id(*),jd(*),split(*)
      double precision d(*),lnz(*),tmpvec(*),timewd(7)
      real gtimer,timbeg,timend
      external smxpy1,smxpy2,smxpy4,smxpy8
      external mmpy1,mmpy2,mmpy4,mmpy8
c
c Save the matrix structure from jdsub(m+2:nzzd+1),jdsub(1:m+1)
c   to lindx and xlindx because the matrix structure is destroyed by the 
c   minimum degree ordering routine
c
      do i = 1,m+1
         xlindx(i) = jdsub(i)
      enddo
      do i = 1,nnzdsub
         lindx(i) = jdsub(m+1+i)
      enddo
c
c Reorder the matrix using minimum degree ordering routine
c
      iwsiz = 4*m
      if(it .le. 1) then
         timbeg = gtimer()
         call ordmmd(m,xlindx,lindx,invp,perm,iwsiz,iwork,nsub,ierr)
         if (ierr .eq. -1) then
            ierr = 3
            go to 100
         endif
         timend = gtimer()
         timewd(1) = timewd(1) + timend - timbeg
c
c Call sfinit: Symbolic factorization initialization
c   to compute supernode partition and storage requirements
c   for symbolic factorization. New ordering is a postordering 
c   of the nodal elimination tree
c
         iwsiz = 7 * m + 3
         timbeg = gtimer()
         call sfinit(m,nnzdsub,jdsub(1),jdsub(m+2),perm,
     &            invp,colcnt,nnzl,nsub,nsuper,snode,xsuper,iwsiz,
     &            iwork,ierr)
         if (ierr .eq. -1) then
            ierr = 4
            go to 100
         endif
         if (nnzl .gt. nnzlmax) then
            ierr = 5
            go to 100
         endif
         if (nsub .gt. nsubmax) then
            ierr = 6
            go to 100
         endif
         timend = gtimer()
         timewd(2) = timewd(2) + timend - timbeg
      endif
c
c Call symfct: Perform supernodal symbolic factorization
c
c      iwsiz = nsuper + 2 * m + 1
      timbeg = gtimer()
      call symfct(m,nnzdsub,jdsub(1),jdsub(m+2),perm,invp,
     &            colcnt,nsuper,xsuper,snode,nsub,xlindx,lindx,
     &            xlnz,iwsiz,iwork,ierr)
      if (ierr .eq. -1) then
         ierr = 7
         go to 100
      endif
      if (ierr .eq. -2) then
         ierr = 8
         go to 100
      endif
      timend = gtimer()
      timewd(3) = timewd(3) + timend - timbeg
c
c Call inpnv: Input numerical values into data structures of L
c
      iwsiz = m
      timbeg = gtimer()
      call inpnv(m,id,jd,d,perm,invp,nsuper,xsuper,xlindx,lindx,
     &           xlnz,lnz,iwork)
      timend = gtimer()
      timewd(4) = timewd(4) + timend - timbeg
c
c Call bfinit: Initialization for block factorization
c
      timbeg = gtimer()
      call bfinit(m,nsuper,xsuper,snode,xlindx,lindx,cachsz,tmpsiz,
     &            split)
      if (tmpsiz .gt. tmpmax) then
         ierr = 9
         go to 100
      endif
      timend = gtimer()
      timewd(5) = timewd(5) + timend - timbeg
c
c Call blkfct: Numerical factorization
c
      iwsiz = 2 * m + 2 * nsuper
      timbeg = gtimer()
      if (level .eq. 1) then
         call blkfct(m,nsuper,xsuper,snode,split,xlindx,lindx,xlnz,
     &               lnz,iwsiz,iwork,tmpsiz,tmpvec,ierr,mmpy1,smxpy1)
      elseif (level .eq. 2) then
         call blkfct(m,nsuper,xsuper,snode,split,xlindx,lindx,xlnz,
     &               lnz,iwsiz,iwork,tmpsiz,tmpvec,ierr,mmpy2,smxpy2)
      elseif (level .eq. 4) then
         call blkfct(m,nsuper,xsuper,snode,split,xlindx,lindx,xlnz,
     &               lnz,iwsiz,iwork,tmpsiz,tmpvec,ierr,mmpy4,smxpy4)
      elseif (level .eq. 8) then
         call blkfct(m,nsuper,xsuper,snode,split,xlindx,lindx,xlnz,
     &               lnz,iwsiz,iwork,tmpsiz,tmpvec,ierr,mmpy8,smxpy8)
      endif
      if (ierr .eq. -1) then
         ierr = 10
         go to 100
      elseif (ierr .eq. -2) then
         ierr = 11
         go to 100
      elseif (ierr .eq. -3) then
         ierr = 12
         go to 100
      endif
  100 continue
      timend = gtimer()
      timewd(6) = timewd(6) + timend - timbeg
      return
      end
c

###====== Nonlinear quantile regression with an interior point algorithm ======
# see: Koenker, R. & B.J. Park, 1996. An interior point algorithm
# for nonlinear quantile regression. J. Econom., 71(1-2): 265-283.
# adapted from nlrq routine of Koenker, R. to be compatible with  R nls models
# by Ph. Grosjean, 2001 (phgrosjean@sciviews.org)
# large parts of code are reused from the nls package of R v. 1.2.3

# TO DO:
# - nlrq should return a code 0 = convergence, 1 = lambda -> 0, etc..
# - Extensive diagnostic for summary() (Roger, what would you propose?)
# - Calculate with a list of tau values at once (currently accept only 1 value)
# - When providing several tau values, allow also to calculate a single value
#   for one or several parameters across all models fitted to all tau values...
#   ...but I have another idea for doing that more efficiently.

"nlrq.control" <- function (maxiter=100, k=2, InitialStepSize = 1, 
	big=1e+20, eps=1.0e-07, beta=0.97)
{
    list(maxiter=maxiter, k=k, InitialStepSize = InitialStepSize, 
	big=big, eps=eps, beta=beta)
}

# Still needs to be cleaned up: several parts of the code not used here (QR,...)
"nlrqModel" <- function (form, tau, data, start) 
{
    thisEnv <- environment()
    env <- new.env(parent = environment(form))
    for (i in names(data)) {
        assign(i, data[[i]], envir = env)
    }
    ind <- as.list(start)
    parLength <- 0
    for (i in names(ind)) {
        temp <- start[[i]]
        storage.mode(temp) <- "double"
        assign(i, temp, envir = env)
        ind[[i]] <- parLength + seq(along = start[[i]])
        parLength <- parLength + length(start[[i]])
    }
    useParams <- rep(TRUE, parLength)
    lhs <- eval(form[[2]], envir = env)
    rhs <- eval(form[[3]], envir = env)
    resid <- lhs - rhs
    tau <- tau
    dev <- sum(tau * pmax(resid, 0) + (tau - 1) * pmin(resid, 0))
    if (is.null(attr(rhs, "gradient"))) {
        getRHS.noVarying <- function() numericDeriv(form[[3]], 
            names(ind), env)
        getRHS <- getRHS.noVarying
        rhs <- getRHS()
    }
    else {
        getRHS.noVarying <- function() eval(form[[3]], envir = env)
        getRHS <- getRHS.noVarying
    }
    dimGrad <- dim(attr(rhs, "gradient"))
    marg <- length(dimGrad)
    if (marg > 0) {
        gradSetArgs <- vector("list", marg + 1)
        for (i in 2:marg) gradSetArgs[[i]] <- rep(TRUE, dimGrad[i - 
            1])
        useParams <- rep(TRUE, dimGrad[marg])
    }
    else {
        gradSetArgs <- vector("list", 2)
        useParams <- rep(TRUE, length(attr(rhs, "gradient")))
    }
    npar <- length(useParams)
    gradSetArgs[[1]] <- (~attr(ans, "gradient"))[[2]]
    gradCall <- switch(length(gradSetArgs) - 1, call("[", gradSetArgs[[1]], 
        gradSetArgs[[2]]), call("[", gradSetArgs[[1]], gradSetArgs[[2]], 
        gradSetArgs[[2]]), call("[", gradSetArgs[[1]], gradSetArgs[[2]], 
        gradSetArgs[[2]], gradSetArgs[[3]]), call("[", gradSetArgs[[1]], 
        gradSetArgs[[2]], gradSetArgs[[2]], gradSetArgs[[3]], 
        gradSetArgs[[4]]))
    getRHS.varying <- function() {
        ans <- getRHS.noVarying()
        attr(ans, "gradient") <- eval(gradCall)
        ans
    }
    QR <- qr(attr(rhs, "gradient"))
    qrDim <- min(dim(QR$qr))
    if (QR$rank < qrDim) 
        stop("singular gradient matrix at initial parameter estimates")
    getPars.noVarying <- function() unlist(setNames(lapply(names(ind), 
        get, envir = env), names(ind)))
    getPars.varying <- function() unlist(setNames(lapply(names(ind), 
        get, envir = env), names(ind)))[useParams]
    getPars <- getPars.noVarying
    internalPars <- getPars()
    setPars.noVarying <- function(newPars) {
        assign("internalPars", newPars, envir = thisEnv)
        for (i in names(ind)) {
            assign(i, unname(newPars[ind[[i]]]), envir = env)
        }
    }
    setPars.varying <- function(newPars) {
        internalPars[useParams] <- newPars
        for (i in names(ind)) {
            assign(i, unname(internalPars[ind[[i]]]), envir = env)
        }
    }
    setPars <- setPars.noVarying
    on.exit(remove(i, data, parLength, start, temp))
    m <- list(resid = function() resid, fitted = function() rhs, 
        formula = function() form, tau = function() tau, deviance = function() dev, 
        gradient = function() attr(rhs, "gradient"), incr = function() qr.coef(QR, resid), setVarying = function(vary = rep(TRUE, 
            length(useParams))) {
            assign("useParams", if (is.character(vary)) {
                temp <- logical(length(useParams))
                temp[unlist(ind[vary])] <- TRUE
                temp
            } else if (is.logical(vary) && length(vary) != length(useParams)) stop("setVarying : vary length must match length of parameters") else {
                vary
            }, envir = thisEnv)
            gradCall[[length(gradCall)]] <<- useParams
            if (all(useParams)) {
                assign("setPars", setPars.noVarying, envir = thisEnv)
                assign("getPars", getPars.noVarying, envir = thisEnv)
                assign("getRHS", getRHS.noVarying, envir = thisEnv)
                assign("npar", length(useParams), envir = thisEnv)
            } else {
                assign("setPars", setPars.varying, envir = thisEnv)
                assign("getPars", getPars.varying, envir = thisEnv)
                assign("getRHS", getRHS.varying, envir = thisEnv)
                assign("npar", length((1:length(useParams))[useParams]), 
                  envir = thisEnv)
            }
        }, changeTau = function(newTau) {
            assign("tau", newTau, envir = thisEnv)
            assign("dev", sum(tau * pmax(resid, 0) + (tau - 1) * pmin(resid, 0)), envir = thisEnv)
            return(dev)
        }, setPars = function(newPars) {
            setPars(newPars)
            assign("resid", lhs - assign("rhs", getRHS(), envir = thisEnv), 
                envir = thisEnv)
            assign("dev", sum(tau * pmax(resid, 0) + (tau - 1) * pmin(resid, 0)), envir = thisEnv)
            assign("QR", qr(attr(rhs, "gradient")), envir = thisEnv)
            return(QR$rank < min(dim(QR$qr)))
        }, getPars = function() getPars(), getAllPars = function() getPars(), 
        getEnv = function() env, trace = function() cat(format(dev), 
            ": ", format(getPars()), "\n"), Rmat = function() qr.R(QR), 
        predict = function(newdata = list(), qr = FALSE) {
            Env <- new.env()
            for (i in objects(envir = env)) {
                assign(i, get(i, envir = env), envir = Env)
            }
            newdata <- as.list(newdata)
            for (i in names(newdata)) {
                assign(i, newdata[[i]], envir = Env)
            }
            eval(form[[3]], envir = Env)
        })
    class(m) <- "nlrqModel"
    m
}

"nlrq" <- function (formula, data=parent.frame(), start, tau=0.5, 
	control, trace=FALSE, method = "L-BFGS-B")
{
    mf <- match.call()
    formula <- as.formula(formula)
    varNames <- all.vars(formula)
    if (length(formula) == 2) {
        formula[[3]] <- formula[[2]]
        formula[[2]] <- 0
    }
    if (missing(start)) {
        if (!is.null(attr(data, "parameters"))) {
            pnames <- names(attr(data, "parameters"))
        }
        else {
            cll <- formula[[length(formula)]]
            func <- get(as.character(cll[[1]]))
            pnames <- as.character(as.list(match.call(func, call = cll))[-1][attr(func, "pnames")])
        }
    }
    else {
        pnames <- names(start)
    }
    varNames <- varNames[is.na(match(varNames, pnames, nomatch = NA))]
    varIndex <- sapply(varNames, function(varName, data, respLength) {
        length(eval(as.name(varName), data))%%respLength == 0
    }, data, length(eval(formula[[2]], data)))
    mf$formula <- parse(text = paste("~", paste(varNames[varIndex], collapse = "+")))[[1]]
    mf$start <- mf$tau <- mf$control <- mf$algorithm <- mf$trace <- mf$method <- NULL
    mf[[1]] <- as.name("model.frame")
    mf <- as.list(eval(mf, parent.frame()))
    if (missing(start)) {
        start <- getInitial(formula, mf)
    }
    for (var in varNames[!varIndex]) mf[[var]] <- eval(as.name(var), data)
    ctrl <- nlrq.control()
    if (!missing(control)) {
        control <- as.list(control)
        ctrl[names(control)] <- control
    }
    m <- nlrqModel(formula, tau, mf, start)
    nlrq.calc <- function (model, ctrl, trace) {
        meketon <- function(x, y, w, tau, ctrl) {
            yw <- ctrl$big
            k <- 1
            while(k <= ctrl$k & yw - crossprod(y, w) > ctrl$eps) {
                d <- pmin(tau - w, 1 - tau + w)
                z <- lsfit(x, y, d^2, intercept=FALSE)
                yw <- sum(tau * pmax(z$resid, 0) + (tau - 1) * pmin(z$resid, 0))
                k <- k + 1
                s <- z$resid * d^2
                alpha <- max(ctrl$eps, pmax(s/(tau - w), -s/(1 - tau + w)))
                w <- w + (ctrl$beta/alpha) * s
            }
            coef <- z$coef
            return(list(coef=coef, w=w))
        }
        model.step <- function(lambda, Step, model, pars) {
            model$setPars(pars + lambda * Step)
            model$deviance()
        }
        w <- rep(0, length(model$resid()))
        snew <- model$deviance()
        sold <- ctrl$big
        nit <- 0
        if (trace) {
            model$trace()
            optim.ctrl <- list(trace=1)
        } else {
            optim.ctrl <- list(trace=0)
        }
        lam0 <- ctrl$InitialStepSize
        while(sold - snew > ctrl$eps & nit < ctrl$maxiter) {
            z <- meketon(model$gradient(),as.vector(model$resid()), w, tau=tau, ctrl=ctrl)
            Step <- z$coef
            Pars <- model$getPars()
            lam <- try(optim(par=lam0, fn=model.step, method=method, lower=0, upper=1, 
	        Step=Step, model=model, pars=Pars, control=optim.ctrl)$par)
            if(inherits(lam,"try.error") || !is.finite(lam)) 
                stop("optim unable to find valid step size")
            if (trace) {cat("lambda =", lam, "\n")}
            model$setPars(Pars + lam * Step)
            sold <- snew
            snew <- model$deviance()
            w <- qr.resid(qr(model$gradient()), z$w)
            w1 <- max(pmax(w, 0))
            if(w1 > tau) {w <- (w * tau)/(w1 + ctrl$eps)}
            w0 <- max(pmax( - w, 0))
            if(w0 > 1 - tau) {w <- (w * (1 - tau))/(w0 + ctrl$eps)}
            if (trace) {model$trace()}
            if (R.Version()$os == "Win32") {flush.console()}
            nit <- nit + 1
        }
        Rho <- function(u,tau) u * (tau - (u < 0))
    	model$rho <- sum(Rho(model$resid(),tau))
        model
    }
    nlrq.out <- list(m=nlrq.calc(m, ctrl, trace), data=substitute(data), 
	call=match.call())
    nlrq.out$call$control <- ctrl
    nlrq.out$call$trace <- trace
    class(nlrq.out) <- "nlrq"
    nlrq.out
}
"logLik.nlrq" <- function(object,  ...){
        n <- length(object$m$resid())
        p <- length(object$m$getPars())
        tau <- object$m$tau()
        fid <- object$m$rho
        val <- n * (log(tau * (1-tau)) - 1 - log(fid/n))
        attr(val,"n") <- n
        attr(val,"df") <- p
        class(val) <- "logLik"
        val
        }
"AIC.nlrq" <- function(object, ... , k = 2){
        v <- logLik(object)
        if(k <= 0)
                k <- log(attr(v,"n"))
        val <- AIC(v, k = k)
        attr(val,"edf") <- attr(v,"df")
        val
        }
"extractAIC.nlrq"  <- function(fit, scale, k=2, ...){
	aic <- AIC(fit,k)
	edf <- attr(aic, "edf")
	c(edf, aic)
	}




"print.nlrq" <- function (x, ...)
{
    cat("Nonlinear quantile regression\n")
    cat("   model: ", deparse(formula(x)), "\n")
    cat("    data: ", as.character(x$data), "\n")
    cat("     tau: ", as.character(x$m$tau()), "\n")
    cat("deviance: ", format(x$m$deviance()), "\n")
    print(x$m$getAllPars())
    invisible(x)
}

# For the moment, print.summary is the same as print
# However, some extra diagnostic should be done here
"summary.nlrq" <- function (object, ...)
    structure(object, class=c("summary.nlrq", class(object)))

"print.summary.nlrq" <- function (x, ...)
{
    cat("Nonlinear quantile regression\n")
    cat("   model: ", deparse(formula(x)), "\n")
    cat("    data: ", as.character(x$data), "\n")
    cat("     tau: ", as.character(x$m$tau()), "\n")
    cat("deviance: ", format(x$m$deviance()), "\n")
    print(x$m$getAllPars())
    invisible(x)
}

"coef.nlrq" <- function (object, ...) 
    object$m$getAllPars()

"deviance.nlrq" <- function (object, ...) 
    object$m$deviance()


"tau.nlrq" <- function (object, ...) 
    object$m$tau()

"fitted.nlrq" <- function (object, ...) 
{
    val <- as.vector(object$m$fitted())
    lab <- "Fitted values"
    if (!is.null(aux <- attr(object, "units")$y)) {
        lab <- paste(lab, aux)
    }
    attr(val, "label") <- lab
    val
}

"formula.nlrq" <- function (x, ...) 
    x$m$formula()

"predict.nlrq" <- function (object, newdata, ...) 
{
    if (missing(newdata)) 
        return(as.vector(fitted(object)))
    object$m$predict(newdata)
}

"residuals.nlrq" <- function (object, type = c("response", "rho"), ...) 
{
    type <- match.arg(type)
    val <- as.vector(object$m$resid())
    if (type == "rho") {
        tau <- object$m$tau()
        val <- tau * pmax(val, 0) + (1 - tau) * pmin(val, 0)
        attr(val, "label") <- paste("quantile residuals rho(", tau ,")", sep="")
    }
    else {
        lab <- "Residuals"
        if (!is.null(aux <- attr(object, "units")$y)) {
            lab <- paste(lab, aux)
        }
        attr(val, "label") <- lab
    }
    val
}
"summary.nlrq" <- function(object, ...)
{
    y <- as.vector(object$m$resid())
    X <- object$m$gradient()
    tau <- object$m$tau()
    pnames <- names(object$m$getPars())
    f <- summary(rq(y ~X-1,tau), se = "boot", covariance = TRUE, ...)
    f$coefficients[,1] <- object$m$getPars()
    f$coefficients[,3] <- f$coefficients[, 1]/f$coefficients[, 2]
    f$coefficients[, 4] <- if (f$rdf > 0)
            2 * (1 - pt(abs(f$coef[, 3]), f$rdf))
    dimnames(f$coefficients)[[1]] <- pnames
    f$call <- object$call
    f$tau <- tau
    class(f) <- "summary.nlrq"
    return(f)
}
"print.summary.nlrq" <- function (x, digits = max(5, .Options$digits - 2), ...) 
{
    cat("\nCall: ")
    dput(x$call)
    coef <- x$coef
    tau <- x$tau
    cat("\ntau: ")
    print(format(round(tau, digits = digits)), quote = FALSE, 
        ...)
    cat("\nCoefficients:\n")
    print(format(round(coef, digits = digits)), quote = FALSE, 
        ...)
    invisible(x)
}

C Output from Public domain Ratfor, version 1.05
      subroutine pwy(m,n,k,m5,n2,a,c,b,t,toler,ift,x,e,s, wa,wb)
      double precision b(m),a(k,n),x(n,k)
      double precision wa(m5,n2),wb(m),e(m),c(m,n)
      double precision t,toler
      integer m,n,k,m5,n2,ift
      integer s(m)
      do23000 i=1,k
      call dcopy(n,a(i,1),k,c(m,1),m)
      call rq0(m,n,m5,n2,c,b,t,toler,ift,x(1,i),e,s,wa,wb)
23000 continue
23001 continue
      return
      end
      subroutine xys(mofn,m,n,k,mofn5,n2,a,b,tau,toler,ift,x,e,s, wa,wb,
     *aa,bb,ss)
      double precision b(m),a(m,n),x(n,k)
      double precision wa(mofn5,n2),wb(mofn)
      double precision aa(mofn,n),bb(mofn),e(mofn)
      double precision tau,toler
      integer ss(mofn,k),s(mofn),mofn,m,n,k,mofn5,n2,ift(k)
      do23002 i=1,k 
      do23004 ii=1,mofn
      bb(ii)=b(ss(ii,i))
      do23006 jj=1,n
      aa(ii,jj)=a(ss(ii,i),jj)
23006 continue
23007 continue
23004 continue
23005 continue
      call rq0(mofn,n,mofn5,n2,aa,bb,tau,toler,ift(i),x(1,i),e,s,wa,wb)
23002 continue
23003 continue
      return
      end
      subroutine wxy(m,n,k,m5,n2,a,b,tau,toler,ift,x,e,s,wa,wb,aa,bb,w)
      double precision b(m),a(m,n),x(n,k)
      double precision w(m,k),wa(m5,n2),wb(m)
      double precision aa(m,n),bb(m),e(m)
      double precision tau,toler
      integer s(m),m,n,k,m5,n2,ift(k)
      do23008 i=1,k 
      do23010 ii=1,m
      bb(ii)=b(ii)*w(ii,i)
      do23012 jj=1,n
      aa(ii,jj)=a(ii,jj)*w(ii,i)
23012 continue
23013 continue
23010 continue
23011 continue
      call rq0(m,n,m5,n2,aa,bb,tau,toler,ift(i),x(1,i),e,s,wa,wb)
23008 continue
23009 continue
      return
      end
      subroutine heqfy(n,p,r,x,b,y)
      integer n,p,r
      double precision x(n,p),b(p,n,r),y(n,r),ddot
      do23014 i=1,r
      do23016 j=1,n
      y(j,i)=ddot(p,x(j,1),n,b(1,j,i),1)
23016 continue
23017 continue
23014 continue
23015 continue
      return
      end

lprq <- function(x, y, h, tau = .5, m = 50)
{
	## A toy routine to do locally polynomial quantile regression
        xx <- seq(min(x),max(x),length=m)
        fv <- xx
        dv <- xx
        for(i in 1:length(xx)) {
                z <- x - xx[i]
                wx <- dnorm(z/h)
                r <- rq(y~z, weights=wx, tau=tau, ci=FALSE)
                fv[i] <- r$coef[1.]
                dv[i] <- r$coef[2.]
        }
        list(xx = xx, fv = fv, dv = dv)
}

rq.fit.panel <- function(X,y,s,w=c(.25,.5,.25),taus=(1:3)/4,lambda = 1){
# prototype function for panel data fitting of QR models
# the matrix X is assumed to contain an intercept
# the vector s is a strata indicator assumed (so far) to be a one-way layout
# NB:  
# 0.  This is an altered version from that originally posted -- the definition
#     of the rhs vector now incorporates a factor of 1/2 for the penalty.
# 1.  The value of the shrinkage parameter lambda is an open research problem in
#   the simplest homogneous settings it should be the ratio of the scale parameters
#   of the fixed effects and the idiocyncratic errors
# 2.  On return the coefficient vector has m*p + n elements where m is the number
# quantiles being estimated, p is the number of colums of X, and n is the
# number of distinct values of s.  The first m*p coefficients are the 
# slope estimates, and the last n are the "fixed effects"
# 3.  Like all shrinkage (regularization) estimators, asymptotic inference is somewhat
# problematic... so the bootstrap is the natural first resort.
  require(SparseM)
  require(quantreg)
  K <- length(w)
  if(K != length(taus))
    stop("length of w and taus must match")
  X <- as.matrix(X)
  p <- ncol(X)
  n <- length(levels(as.factor(s)))
  N <- length(y)
  if(N != length(s) || N != nrow(X))
    stop("dimensions of y,X,s must match")
  Z <- as.matrix.csr(model.matrix(~as.factor(s)-1))
  Fidelity <- cbind(as(w,"matrix.diag.csr") %x% X,w %x% Z)
  Penalty <- cbind(as.matrix.csr(0,n,K*p),lambda*as(n,"matrix.diag.csr"))
  D <- rbind(Fidelity,Penalty)
  y <- c(w %x% y,rep(0,n))
  a <- c((w*(1-taus)) %x% (t(X)%*%rep(1,N)),
    sum(w*(1-taus)) * (t(Z) %*% rep(1,N)) + lambda * rep(1/2,n))
  rq.fit.sfn(D,y,rhs=a)
}

#function to compute qth quantile of a sample of n observations
subroutine qselect(n,x,q)
integer n,k,l,r
double precision x(n),q
k=nint(q*n)
l=1
r=n
call select(n,x,l,r,k)
q=x(k)
return
end
#This is a ratfor implementation of the floyd-rivest algorithm--SELECT
#Reference:  CACM 1975, alg #489, p173, algol-68 version
#Translation by Roger Koenker August, 1996.
#As originally proposed mmax=600, and cs=cd=.5
#Calls blas routine dswap
recursive subroutine select(n,x,l,r,k)
integer n,m,l,r,k,ll,rr,i,j,mmax
double precision x(n),z,s,d,t,fm,cs,cd
parameter(cs = 0.5d0)
parameter(cd = 0.5d0)
parameter(mmax = 600)
while(r>l){
	if(r-l>mmax){
		m=r-l+1
		i=k-l+1
		fm = dfloat(m)
		z=log(fm)
		s=cs*exp(2*z/3)
		d=cd*sqrt(z*s*(m-s)/fm)*sign(1,i-m/2)
		ll=max(l,nint(k-i*s/fm + d))
		rr=min(r,nint(k+(m-i)*s/fm + d))
		call select(n,x,ll,rr,k)
		}
	t=x(k)
	i=l
	j=r
	call dswap(1,x(l),1,x(k),1)
	if(x(r)>t)call dswap(1,x(r),1,x(l),1)
	while(i<j){
		call dswap(1,x(i),1,x(j),1)
		i=i+1
		j=j-1
		while(x(i)<t)i=i+1
		while(x(j)>t)j=j-1
		}
	if(x(l)==t)
		call dswap(1,x(l),1,x(j),1)
	else{
		j=j+1
		call dswap(1,x(j),1,x(r),1)
		}
	if(j<=k)l=j+1
	if(k<=j)r=j-1
	}
return
end

# Example rqss vs rq.fit.panel vs rq.fit.lasso  
# Slightly modified version of a test problem of Stefan Bache and co.
require(quantreg)
source("rq.fit.panel.R")
set.seed(1917)

tt <- 3   # time periods
nn <- 3  # individuals

#  Generate some data:
some.data <- data.frame(
    id=rep(1:nn, each=tt)             #ids
   ,ic=1                              #intercept
   ,x1=rnorm(nn*tt)                   #regressors
   ,x2=runif(nn*tt)
   ,alpha=rep(runif(nn)*2 ,each=tt)   #fixed effects
)

# response:
some.data$y <- some.data$x1 + some.data$x2 + some.data$alpha + rnorm(nn*tt)

lambda <- .2
tau <- 0.25


# Fit with rq.fit.panel
fit1 <- rq.fit.panel(cbind(1, some.data$x1, some.data$x2), some.data$y
            ,rep(1:nn, each=tt)
            ,tau=tau
            ,w=1
            ,lambda=lambda)

# fit with rqss (using the global debug variable from the panel function
# so remember to run that too! :-) :
fit2 <- rqss(y ~ ic + x1 + x2 + as.factor(id) - 1 
            ,data=some.data
            ,method="lasso"
            ,tau=tau
            ,lambda=c(0, 0, 0, rep(lambda, nn)))

# fit with rq.lasso
fit3 <- rq(y ~ ic + x1 + x2 + as.factor(id) - 1 
            ,data=some.data
            ,tau=tau
            ,method="lasso"
            ,lambda=c(0, 0, 0, rep(lambda, nn)))

# Print coefficients for comaparison.
comparefit <- function()
{
  compmat <- cbind(fit1$coef,fit2$coef, fit3$coef)
  colnames(compmat) <- c("rq.fit.panel", "rqss", "rq.fit.lasso")
  noquote(formatC(compmat, format="f", digits=8, width=12))
}

cat("all(rq.fit.panel$coef==rq$coef): ", all.equal(fit1$coef,fit3$coef), "\n")
cat("all(rq.fit.$coef==rqss$coef): ", all.equal(fit2$coef,fit3$coef), "\n")
cat("all(rqss$coef==rq.fit.panel$coef): ", all.equal(fit2$coef,fit1$coef), "\n")

%\VignetteIndexEntry{Quantile Regression}
%\VignetteDepends{quantreg,MASS,SparseM,tripack}
%\VignettePackage{quantreg}
\documentclass{amsart}
\usepackage{natbib}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\texttt{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}
\newcommand{\Rmethod}[1]{{\textit{#1}}}
\newcommand{\Rfunarg}[1]{{\textit{#1}}}
\newcommand{\R}{{\normalfont\textsf{R }}{}}
\renewcommand{\S}{{\normalfont\textsf{S }}{}}

\title{Quantile Regression in R: A Vignette}
\thanks{Version:  \today}
\author{Roger Koenker}

\begin{document}
\bibliographystyle{plainnat}
\begin{abstract}
Quantile regression is an evolving body of statistical
methods for estimating and drawing inferences about conditional
quantile functions.  An implementation of these methods in the
\R language is available in the package \Rpackage{quantreg}.
This vignette offers a brief tutorial introduction to
the package.  \R and the package \Rpackage{quantreg} are open-source
software projects and can be freely downloaded from CRAN:
\texttt{http://cran.r-project.org}.
\end{abstract}

\maketitle

\pagestyle{myheadings}
\markboth{\sc Quantile Regression in R}{\sc Roger Koenker}

<<label=R options,echo=FALSE>>=
options(width = 60)
options(SweaveHooks = list(fig = function() par(mar=c(3,3,1,0.5),mgp = c(2,1,0))))
@


\section{Introduction}




Beran's (2003) \nocite{Beran.03} provocative definition
of statistics as ``the study of algorithms for data analysis'' elevates
computational considerations to the  forefront of the field.  It is
apparent that the evolutionary success of statistical methods is to a significant degree
determined by considerations of computational convenience.
As a result,
design and dissemination of statistical software has become an integral
part of statistical research.
Algorithms are no longer the exclusive purview of
the numerical analyst, or the proto-industrial software firm;
they are an essential part of the artisanal research process.
Fortunately, modern computing has also transformed the software development
process and greatly facilitated collaborative research;
the massive collective international effort represented by the \R project
exceeds the most idealistic Marxist imagination.

Algorithms have been a crucial part of the research challenge of quantile regression
methods from their inception in the 18th century.  \citet*{Stigler.84}  describes an
amusing episode in 1760 in which the itinerant Croatian Jesuit Rudjer Boscovich sought
computational advice in London  regarding his nascent method for  median regression.
Ironically, a fully satisfactory answer to Boscovich's
questions only emerged with dawn of modern computing.  The discovery of the simplex method
and subsequent developments in linear programming have made quantile regression methods
competitive with traditional least squares methods in terms of their
computational effort.  These computational developments have also
played a critical role in encouraging a deeper appreciation of the
statistical advantages of these methods.

Since the early 1980's I have been developing software for quantile regression:
initially for the S language of Chambers and Becker (1984), later for its
commercial manifestation Splus, and since 1999 for the splendid open source
dialect \R, initiated by \citet*{IG} and sustained by the \citet*{Rcore}.
Although there is now some functionality for quantile regression in most of the
major commercial statistical packages, I have a natural predilection
for the \R
environment and the software that I have developed for \R.  In what follows, I
have tried to provide a brief tutorial introduction to this environment for
quantile regression.

\section{What is a vignette? }

This document was written in the Sweave format of \citet*{Leisch}.
Sweave is an implementation designed for \R of the literate programming style
advocated by \citet*{Knuth.LP}.  The format permits a natural interplay between code
written in {\R}, the output of that code, and commentary on the code.
Sweave documents are preprocessed by \R to produce a \LaTeX~
document that may then be processed by conventional methods.  Many
\R packages now have Sweave vignettes describing their basic
functionality.  Examples of vignettes can be found for many of the
\R packages including this one for the \texttt{quantreg} packages in
the source distribution directory \texttt{inst/doc}.

\section{Getting Started}

I will not attempt to provide another  introduction to \R.  There are already
several excellent resources intended to accomplish this task.  The
books of \citet*{Dalgaard.02} and \citet*{VR} are particularly recommended.
The CRAN website link to contributed documentation
also offers excellent  introductions in several languages.

\R is a open source software project and can be freely downloaded from the CRAN
website along with its associated documentation.  For unix-based operating systems
it is usual to download and build \R from source, but binary versions are available
for most computing platforms and can be easily installed.  Once \R is running the
installation of additional packages is quite straightward.
To install the quantile regression package from \R one simply types,

\vspace{2mm}
\noindent
\texttt{> install.packages("quantreg")}

\vspace{2mm}
\noindent
Provided that your machine has a proper internet connection and you
have write permission in the appropriate system directories,
the installation of the package should proceed automatically.
Once the \texttt{quantreg} package is installed, it needs
to be made accessible to the current \R session by the command,
<<results = hide>>=
library(quantreg)
@
These procedures provide access to an enormous variety
of specialized \texttt{packages} for statistical  analysis.
As we proceed a variety of other packages will be called upon.

Online help facilities are available in two modalities.  If you know
precisely what you are
looking for, and would simply  like to check the details
of a particular command you can, for example, try:
<<eval=FALSE>>=
help(package="quantreg")
help(rq)
@
The former command gives a brief summary of the available commands in the package, and
the latter requests more detailed information about a specific command.
An convenient shorthand for the latter command is to type simply
\texttt{?rq}.
More generally one can initiate a web-browser help session with the command,

\vspace{2mm}
\noindent
\texttt{> help.start()}

\vspace{2mm}
\noindent
and navigate as desired.  The browser approach is better adapted to exploratory
inquiries, while the command line approach is better suited to confirmatory ones.

A valuable feature of \R help files is that the examples used to illustrate commands
are executable, so they can be pasted into an \R session, or run as a group with
a command like,

\vspace{2mm}
\noindent
\texttt{> example(rq)}

\vspace{2mm}
\noindent
The examples for the basic \texttt{rq} command include an analysis of the
Brownlee stackloss data:  first the median regression, then the first quantile
regression is computed, then the full quantile regression process.  A curious
feature of this often analysed data set, but one that is very
difficult to find without quantile regresion fitting,  is the fact the 8 of the 21
points fall exactly on a hyperplane in 4-space.

The second example in the \texttt{rq} helpfile computes
a weighted univariate median using randomly generated data.  The original
\citet*{engel.1857} data on the relationship between food expenditure and household
income is considered in the third example.  The data is plotted and then six
fitted quantile regression lines are superimposed on the scatterplot.
The final example illustrates the imposition of inequality constraints on the quantile
regression coefficients using a simulated data set.

Let's consider the median regression results for the Engel example in somewhat more
detail.  Executing,
<<>>=
data(engel)
fit1 <- rq(foodexp ~ income, tau = .5, data = engel)
@
assigns the output of the median regression computation to the object \texttt{fit1}.
In the command \texttt{rq()} there are also
many options.  The first argument is a ``formula'' that specifies the model that
is desired.  In this case we wanted to fit a simple bivariate linear model
so the formula is just \texttt{y $\sim$ x}, if we had two covariates we could say,
e.g. \texttt{y $\sim$ x+z}.  Factor variables, that is variables
taking only a few discrete values, are treated specially by the formula
processing and result in a group of indicator (dummy) variables.

If we would like to see a concise summary of the result we can simply type,
<<>>=
fit1
@
By convention for all the \R linear model fitting routines,
we see only the estimated coefficients and some
information about the model being estimated.  To obtain a more
detailed evaluation of the fitted model, we can use,
<<>>=
summary(fit1)
@
The resulting table gives the estimated intercept and slope in the first column
and confidence intervals for these parameters in the second and third columns.
By default, these confidence intervals are computed by the rank inversion method
described in \citet*{K.05}, Section 3.4.5.
To extract the residuals or the coefficients of the
fitted relationship we can write,
<<>>=
r1 <- resid(fit1)
c1 <- coef(fit1)
@
They can then be easily used in subsequent calculations.

\section{Object Orientation}

A brief digression on the role of object orientation in \R is perhaps
worthwhile at this juncture.  Expressions in \R manipulate objects,
objects may be data in the form of vectors, matrices  or higher order
arrays, but objects may also be functions, or more complex collections
of objects.  Objects have a class and this clsss identifier helps to
recognize their special features and enables functions to act on them
appropriately.
Thus, for example, the function \texttt{summary} when operating on an
object of class \texttt{rq} as produced by the function \texttt{rq}
can act quite differently on the object than it would if the object
were of another class, say \texttt{lm}  indicating that it was the
product of least squares fitting.  Summary of a data structure like
a matrix or data.frame would have yet another intent and outcome.  In
the earlier dialects of \S and \R methods for various classes were
distinguished by appending the class name to the method separated by
a ``.''.  Thus, the function \texttt{summary.rq} would summarize an \texttt{rq}
object, and \texttt{summary.lm} would summarize an \texttt{lm}
object.  In either case the main objective of the summary is
to produce some inferential  evidence to accompany the point estimates
of parameters.  Likewise, plotting of various classes of \R objects
can be carried out by the expression \texttt{plot(x)} with the
expectation that the plot command will recognize the class of the
object \texttt{x} and proceed accordingly.  More recently,
\citet*{C.98} has introduced an elegant elaboration of the
class, method-dispatch framework for \S and \R.

Assignment of objects
is usually accomplished by the operator \texttt{<-}, and once assigned
these new objects are available for the duration of the \R session, or
until they are explicitly removed from the session.
\R is an open source language so {\it all} of the source files describing
the functionality of the language are ultimately accessible to the
individual user, and users are free to modify and extend the
functionality of the language in any way they see fit.  To accomplish
this one needs to be able to find functions and modify them.  This
takes us somewhat beyond the intended tutorial scope of this vignette,
however suffice it to say that most of the functions of the
\texttt{quantreg} package you will find used below, can be viewed
by simple typing the name of the function perhaps concatenated with
a class name.

\section{Formal Inference}

There are several alternative methods of  conducting inference about quantile
regression coefficients.  As an alternative to the rank-inversion
confidence intervals, one can obtain a more conventional looking
table of coefficients, standard errors, t-statistics, and p-values
using the \texttt{summary} function:
<<>>=
summary(fit1,se = "nid")
@
The standard errors reported in this table are computed as described in Section 3.2.3
for the quantile regression sandwich formula,
and using the Hall-Sheather bandwidth rule.  To obtain
the Powell kernel version of the covariance matrix estimate, one specifies the option
\texttt{se="ker"} in the \texttt{summary} command.  It is also possible to control the
bandwidths employed with the \texttt{bandwidth} option.
Another option available in \texttt{summary.rq} is
to compute bootstrapped standard errors.  This is accomplished by
specifying the option \texttt{se="boot"}.  There are currently three
flavors of the bootstrap available:  the standard $xy$-pair bootstrap,
the \citet*{PWY.94} version, and the Markov chain marginal bootstrap
of \citet*{HeHu} and \citet*{KHM}.  There is also the ability
to specify  $m$ out $n$ versions of the bootstrap in which the sample
size of the bootstrap samples is different from (typically smaller
than) the original sample size.  This ``subsampling'' approach has a
number of advantages, not the least of which is that it can be
considerably faster than the full $n$ out of $n$ version.
By default \texttt{summary}
also produces components estimating the full covariance matrix of the estimated
parameters and its constituent pieces.  For further details,
see the documentation for \texttt{summary.rq}.  In the case of the
bootstrap methods the full matrix of bootstrap replications is also
available.

There are several options to the basic fitting routine \texttt{rq}.  An important option
that controls the choice of the algorithm used in the fitting is \texttt{method}.
The default is \texttt{method = "br"} which invokes a variant of the
\citet*{BR.74} simplex algorithm
described in \citet*{KO.87}.  For problems with more than a few thousand observations it
is worthwhile considering \texttt{method = "fn"}  which invokes the Frisch-Newton algorithm
described in \citet*{PK.1997}.   Rather than traversing around the
exterior of the constraint set like the simplex method, the interior
point approach embodied in the Frisch-Newton algorithm burrows from
within the constraint set toward the exterior.  Instead of taking
steepest descent steps at each intersection of exterior edges, it
takes Newton steps based on a log-barrier Lagrangian form of the
objective function.  Special forms of Frisch-Newton are available for
problems that include linear inequality constraints and for problems
with sparse design matrices.
For extremely large problems with plausibly exchangeable
observations  \texttt{method = "pfn"} implements a version of
the Frisch-Newton algorithm with
a preprocessing step that can further speed things up considerably.

In problems of moderate size where the default simplex option is quite practical,
the parametric programming approach to finding the rank inversion
confidence intervals can be rather slow.  In such cases it may be
advantageous to try one of the other inference methods based on
estimation of the asymptotic covariance matrix, or to consider
the bootstrap.  Both approaches are described in more detail below.

To provide a somewhat more elaborate visualization of the Engel  example consider
an example that superimposes several estimated conditional quantile
functions on the Engel data  scatterplot.
In the resulting figure the median regression line appears as a solid (blue)
line, and the least squares line as a dashed (red) line.  The other quantile
regression lines appear in grey.  Note that the plotting of the  fitted
lines is easily accomplished by the convention that the command \texttt{abline}
looks for a pair of coefficients, which if found are treated as the slope and
intercept of the plotted line.  There are many options that can be used to
further fine tune the plot.  Looping over the quantiles is also conveniently
handled by {\R}'s \texttt{for} syntax.

\begin{figure}
\begin{center}
<<label=engelplot,fig=TRUE>>=
library(quantreg)
data(engel)
attach(engel)
plot(income,foodexp,cex=.25,type="n",xlab="Household Income", ylab="Food Expenditure")
points(income,foodexp,cex=.5,col="blue")
abline(rq(foodexp~income,tau=.5),col="blue")
abline(lm(foodexp~income),lty=2,col="red") #the dreaded ols line
taus <- c(.05,.1,.25,.75,.90,.95)
for( i in 1:length(taus)){
        abline(rq(foodexp~income,tau=taus[i]),col="gray")
        }
@
\end{center}
\caption{Scatterplot and Quantile Regression Fit of the Engel Food
Expenditure Data:  The plot shows a scatterplot of the Engel data
on food expenditure vs household income for a sample of 235 19th
century working class Belgian households.  Superimposed on the
plot are the $\{ .05 , .1, .25, .75, .90, .95\}$ quantile regression
lines in gray, the median fit in solid black, and the least squares
estimate of the conditional mean function as the dashed (red) line.}
\end{figure}



Often it is useful to compute quantile regressions on a discrete set
of $\tau$'s; this can be accomplished by specifying  \texttt{tau}
as a vector in \texttt{rq}:
<<label=engelcoef>>=
xx <- income - mean(income)
fit1 <- summary(rq(foodexp~xx,tau=2:98/100))
fit2 <- summary(rq(foodexp~xx,tau=c(.05, .25, .5, .75, .95)))
@
The results can be summarized as a plot.
<<label=engelcoefplot, results = hide>>=
pdf("engelcoef.pdf",width=6.5,height=3.5)
plot(fit1,mfrow = c(1,2))
dev.off()
@
or by producing a latex-formatted table.
<<label=engeltable>>=
latex(fit2, caption="Engel's Law", transpose=TRUE)
@

The \texttt{pdf} command preceding the plot tells \R that
instructions for the plotting should be written in encapsulated
pdf format and placed in  the file
\texttt{engelcoef.pdf}.  Such files are then conveniently
included in \LaTeX~  documents, for example.  The \texttt{dev.off()}
command closes the current pdf device and concludes the figure.
The horizontal lines in the coefficient plots represent the least
squares fit and its associated confidence interval.

In the one-sample
setting we know that integrating the quantile function over the
entire domain [0,1] yields the mean of the (sample) distribution,
\[
\mu = \int_{-\infty}^\infty x dF(x) = \int_0^1 F^{-1} (t)dt.
\]
Similarly, in the coefficient plots we may expect to see that integrating
individual coefficients yields roughly mean
effect as estimated by the associated least squares coefficient.
One should be cautious, however,  about this interpretation in very
heterogeneous situations.  For the Engel data, note that the least
squares intercept is significantly above any of the fitted quantile
regression curves in our initial scatter plot.  The least squares
fit is strongly affected by the two outlying observations with
relatively low food expenditure; their attraction tilts the fitted
line so its intercept drawn upward.  In fact, the intercept for the
Engel model is difficult to interpret since it asks us to consider
food expenditure for households with zero income.  Centering the
covariate observations so they have mean zero, as we have done prior to
computing \texttt{fit1} for the coefficient plot restores a reasonable
interpretation of the intercept parameter.   After centering the
least squares estimate of the intercept is a prediction of mean
food expenditure for a household with mean income, and the quantile
regression intercept, $\hat \alpha (\tau)$ is a prediction  of the
$\tau$th quantile of food expenditure for households with mean income.
In the terminology of Tukey, the ``intercept'' has become a ``centercept.''

\begin{figure}
\begin{center}
\resizebox{\textwidth}{!}
{\includegraphics{engelcoef.pdf}}
\end{center}
\caption{Engel Coefficient Plots:    the slope and intercept of the
estimated linear quantile regression linear for the Engel food
expenditure data are plotted as a function of $\tau$.  Note that
the household income variable has been centered at its mean value
for this plot, so the intercept is really a centercept and  estimates
the quantile function of food expenditure conditional on mean income.}
\end{figure}

\input{fit2.tex}

The \texttt{latex} command produces a \LaTeX~ formatted table that can be easily
included in  documents.  In many instances the plotted form of
the results will provide a more economical and informative display.
It should again be stressed that since the quantile regression functions
and indeed all of \R is open source, users can always modify the available
functions to achieve special effects required for a particular
application.  When such modifications appear to be of general
applicability, it is desirable to communicate them to the package
author, so they could be shared with the larger community.


If we want to see {\it all} the distinct quantile regression solutions
for a particular model application we can specify a tau outside the range [0,1], e.g.
<<label=rqProcess>>=
z <- rq(foodexp~income,tau=-1)
@
This form of the function carries out the parametric programming steps
required to find the entire sample path of the quantile regression
process.  The returned object is of class \texttt{rq.process}
and has several components:
the primal solution in \texttt{z\$sol}, and the dual solution
in \texttt{ z\$dsol}.   In interactive mode typing the name of an \R object
causes the program to print the object in some reasonably intelligible
manner determined by the print method designated for the object's class.
Again, plotting is often a more informative means of display and
so there is a special \texttt{plot} method for objects of class
\texttt{rq.process}.

Estimating the conditional quantile functions of
\texttt{y} at a specific values of \texttt{x} is also quite easy.
In the following code we plot the estimated empirical quantile
functions  of food expenditure for households that are at the
10th percentile of the sample income distribution,
and the 90th percentile.
In the right panel we plot corresponding density
estimates for the two groups.  The density estimates employ the
adaptive kernel method proposed by \citet*{Silverman} and implemented
in the \texttt{quantreg} function  \texttt{akj}.  This function
is particularly convenient since it permits unequal mass to be
associated with the observations such as those produced by the quantile
regression process.
\begin{figure}
\begin{center}
<<label=eqfs,fig=TRUE,height = 3 >>=
x.poor <- quantile(income,.1) #Poor is defined as at the .1 quantile of the sample distn
x.rich <- quantile(income,.9) #Rich is defined as at the .9 quantile of the sample distn
ps <- z$sol[1,]
qs.poor <- c(c(1,x.poor)%*%z$sol[4:5,])
qs.rich <- c(c(1,x.rich)%*%z$sol[4:5,])
#now plot the two quantile functions to compare
par(mfrow = c(1,2))
plot(c(ps,ps),c(qs.poor,qs.rich), type="n",
     xlab = expression(tau), ylab = "quantile")
plot(stepfun(ps,c(qs.poor[1],qs.poor)), do.points=FALSE, add=TRUE)
plot(stepfun(ps,c(qs.poor[1],qs.rich)), do.points=FALSE, add=TRUE,
     col.hor = "gray", col.vert = "gray")
## now plot associated conditional density estimates
## weights from ps (process)
ps.wts <- (c(0,diff(ps)) + c(diff(ps),0)) / 2
ap <- akj(qs.poor, z=qs.poor, p = ps.wts)
ar <- akj(qs.rich, z=qs.rich, p = ps.wts)
plot(c(qs.poor,qs.rich),c(ap$dens,ar$dens),type="n",
     xlab= "Food Expenditure", ylab= "Density")
lines(qs.rich, ar$dens, col="gray")
lines(qs.poor, ap$dens, col="black")
legend("topright", c("poor","rich"), lty = c(1,1), col=c("black","gray"))
@
\end{center}
\caption{Estimated conditional quantile and density functions for food
expenditure based on the Engel data:  Two estimates are presented one
for relatively poor households with income of
\Sexpr{format(round(x.poor,2))} Belgian francs, and the other for
relatively affluent households with
\Sexpr{format(round(x.rich,2))} Belgian francs.}
\end{figure}

Thus far we have only  considered  Engel functions that are
linear in form, and the scatterplot as well as the formal testing has revealed a strong
tendency for the dispersion of food expenditure to increase with household income.
This is a particularly common form of heteroscedasticity.  If one looks more carefully
at the fitting, one sees interesting departures from symmetry that would not be likely
to be revealed by the typical textbook testing for heteroscedasticity.
One common remedy for symptoms like these would be to reformulate the model
in log linear terms.  It is interesting to compare what happens after
the log transformation with what we have already seen.
\begin{figure}
\begin{center}
<<label=engellogplot,fig=TRUE>>=
plot(income,foodexp,log="xy",xlab="Household Income", ylab="Food Expenditure")
taus <- c(.05,.1,.25,.75,.90,.95)
abline(rq(log10(foodexp)~log10(income),tau=.5),col="blue")
abline(lm(log10(foodexp)~log10(income)),lty = 3,col="red")
for( i in 1:length(taus)){
       abline(rq(log10(foodexp)~log10(income),tau=taus[i]),col="gray")
       }
@
\end{center}
\caption{Quantile regression estimates for a log-linear
version of the Engel food expenditure model.}
\end{figure}

Note that the flag \texttt{log="xy"} produces a plot with log-log axes, and for
convenience of axis labeling these logarithms are base 10, so the subsequent fitting
is also specified as base 10 logs for plotting purposes, even though base 10 logarithms
are {\it unnatural} and would never be used in reporting numerical results.
This looks much more like a classical iid error regression model, although again some
departure from symmetry is visible.  An interesting exercise would be  to conduct some
formal testing for departures from the iid assumption of the type already considered
above.
\section{More on Testing}

Now let's consider some other forms of formal testing.
A natural first question is:   do the estimated quantile regression
relationships conform to the location shift hypothesis that assumes
that all of the conditional quantile functions have the same slope
parameters.  To begin, suppose we just estimate
the quartile fits for the Engel data and look at the default output:

<<>>=
fit1 <- rq(foodexp~income,tau=.25)
fit2 <- rq(foodexp~income,tau=.50)
fit3 <- rq(foodexp~income,tau=.75)
@

Recall that \texttt{rq} just produces coefficient estimates and
\texttt{summary} is needed to evaluate the precision of the estimates.
This is fine for judging whether covariates are significant at particular quantiles
but suppose that we wanted to test that the slopes were the same at
the  three quartiles?
This is done with the \texttt{anova} command as follows:

<<>>=
anova(fit1, fit2, fit3)
@
This is an example of a general class of tests proposed in \citet*{KB.82b}.
It may be instructive to look at the code for the command \texttt{anova.rq} to see how this
test is carried out.  The Wald approach is used and the asymptotic covariance matrix
is estimated using the approach of \citet*{HK.92}.
It also illustrates a  general syntax for testing in \R adapted to
the present situation.

If you have estimated two models with different covariate
specifications, but the same $\tau$
then \texttt{anova(f0,f1)} tests whether the more restricted model is preferred.
Note that this requires that the models with fits say \texttt{f0} and \texttt{f1}
are nested.  The procedure \texttt{anova.rq} attempts to check whether
the fitted models are nested, but this is not foolproof.
These tests can be carried out either as Wald tests based on the
estimated joint asymptotic covariance matrix of the coefficients, or
using the rank test approach described in \citet*{GJKP.94}.
A variety of other options are described in the documentation of the
function \texttt{anova.rq}.


\section{Inference on the Quantile Regression Process}
In least squares estimation of linear models it is implicitly assumed
that we are able to model the effects of the covariates as a pure
location shift, or somewhat more generally as a location and scale
shift of the response distribution.  In the simplest case of a single
binary treatment effect this amounts to assuming that the treatment
and control distributions differ by a location shift, or a
location-scale shift.  Tests of these hypotheses in the two sample
model can be conducted using the conventional two-sample Kolmogorov
Smirnov statistic, but the appearance of unknown nuisance parameters
greatly complicates the limiting distribution theory.  Similar
problems persist in the extension of these tests to the general
quantile regression setting.  Using an approach introduced by
\citet*{khma.81}, \citet*{KX.02} consider general forms
of such tests.  The tests can be viewed as a generalization of
the simple tests of equality of slopes across quantiles described
in the previous section.

In this section we briefly describe how to implement these tests in \R.
The application considered is the quantile autoregression (QAR) model
for weekly U.S. gasoline prices considered in \citet*{KX.06}.  The
data consists of 699 weekly  observations running from August 1990 to
February, 2004.  The model considered is a QAR(4) model utilizing four
lags.  We are interested in testing whether the classical location-shift
AR(4) is a plausible alternative to the QAR(4) specification, that is
whether the four QAR lag coefficients are constant with respect to $tau$.

To carry out the test we can either compute the test using the full
quantile regression process or on a moderately fine grid of taus:

<<label=gastest>>=
data(gasprice)
x <- gasprice
n <- length(x)
p <- 5 # lag length
X <- cbind(x[(p-1):(n-1)], x[(p-2):(n-2)], x[(p-3):(n-3)], x[(p-4):(n-4)])
y <- x[p:n]
T1 <- KhmaladzeTest(y ~ X,taus = -1, nullH="location")
T2 <- KhmaladzeTest(y ~ X,taus = 10:290/300, nullH="location",se="ker")
@
When \texttt{taus} contains elements outside of the the
interval $(0,1)$ then the process is standardized by a
simple form of the covariance matrix that assumes iid error.
In the second version of the test Powell's kernel form of
the sandwich formula estimate is used, see \texttt{summary.rq}.
The function \texttt{KhmaladzeTest} computes both a joint
test that {\it all} the covariate effects satisfy the null
hypothesis, and a coefficient by coefficient version of the
test.  In this example the former component, \texttt{T1\$Tn}
is \Sexpr{format(round(T1$Tn,2))}.  This test has a 1
percent critical value of 5.56,  so the test weakly
rejects the null.  For the Powell form the standardization
the corresponding test statistic is more decisive, taking
the value \Sexpr{format(round(T2$Tn,2))}.

Tests of the location-scale shift form of the null hypothesis
can be easily  done by making the appropriate change in the
\texttt{nullH} argument of the function.

\section{Nonlinear Quantile Regression}

Quantile regression models with response functions that are nonlinear
in parameters can be estimated with the function \texttt{nlrq}.  For
such models the specification of the model formula is somewhat more
esoteric than for ordinary linear models, but follows the conventions
of the \R command \texttt{nls} for nonlinear least squares estimation.

To illustrate the use of \texttt{nlrq} consider the problem of
estimating the quantile functions of the Frank copula model introduced
in \citet*{K.05}, Section 8.4.  We begin by setting some parameters
and generating data from the Frank model:

<<label = Frank>>=
	n <- 200
	df <- 8
	delta <- 8
	set.seed(4003)
	x <- sort(rt(n,df))
	u <- runif(n)
	v <- -log(1-(1-exp(-delta))/(1+exp(-delta*pt(x,df))*((1/u)-1)))/delta
	y <- qt(v,df)
@
We plot the observations, superimpose three conditional quantile
functions, and then estimate the same three quantile functions and
plot their estimated curves as the dashed curves.
\begin{figure}
\begin{center}
<<label = Frankplot, results = hide, height = 4, eval = TRUE, fig=TRUE>>=
	plot(x,y,col="blue",cex = .25)
	us <- c(.25,.5,.75)
	for(i in 1:length(us)){
		u <- us[i]
		v <- -log(1-(1-exp(-delta))/
			(1+exp(-delta*pt(x,df))*((1/u)-1)))/delta
		lines(x,qt(v,df))
		}
	Dat <- NULL
	Dat$x <- x
	Dat$y <- y
	deltas <- matrix(0,3,length(us))
	FrankModel <- function(x,delta,mu,sigma,df,tau){
        	z <- qt(-log(1-(1-exp(-delta))/
			(1+exp(-delta*pt(x,df))*((1/tau)-1)))/delta,df)
		mu + sigma*z
		}
	for(i in 1:length(us)){
		tau = us[i]
		fit <- nlrq(y~FrankModel(x,delta,mu,sigma,df=8,tau=tau),
			data=Dat,tau= tau, start=list(delta=5,
			mu = 0, sigma = 1),trace=TRUE)
		lines(x, predict(fit, newdata=x), lty=2, col="green")
		deltas[i,] <- coef(fit)
		}
@
\end{center}
\caption{Nonlinear Conditional Quantile Estimation of the Frank Copula
Model:  The solid curves are the true conditional quantile functions
and the corresponding estimated curves are indicated by the dashed
curves.}
\end{figure}

\section{Nonparametric Quantile Regression}


Nonparametric quantile regression is initially
most easily considered within a locally polynomial framework.
Locally linear fitting is carried out by the following function:
<<label=lprq>>=
"lprq" <-
function(x, y, h, m=50 , tau=.5)
{
        xx <- seq(min(x),max(x),length=m)
        fv <- xx
        dv <- xx
        for(i in 1:length(xx)) {
                z <- x - xx[i]
                wx <- dnorm(z/h)
                r <- rq(y~z, weights=wx, tau=tau, ci=FALSE)
                fv[i] <- r$coef[1.]
                dv[i] <- r$coef[2.]
        }
        list(xx = xx, fv = fv, dv = dv)
}
@
If you study the function a bit you will see that it is simply a matter
of computing a quantile regression fit at each of $m$ equally spaced $x$-values
distributed over the support of the observed $x$ points.  The function value estimates
are returned as \texttt{fv} and the first derivative estimates at the $m$ points
are returned as \texttt{dv}.  As usual you can specify $\tau$, but now you also
need to specify a bandwidth \texttt{h}.

Let's begin by exploring the effect of the \texttt{h} argument for
fitting the motorcycle data.
\begin{figure}
\begin{center}
<<label=mcycle1,fig=TRUE>>=
library(MASS)
data(mcycle)
attach(mcycle)
plot(times,accel,xlab = "milliseconds", ylab = "acceleration")
hs <- c(1,2,3,4)
for(i in 1:length(hs)){
        h = hs[i]
        fit <- lprq(times,accel,h=h,tau=.5)
        lines(fit$xx,fit$fv,lty=i)
        }
legend(45,-70,c("h=1","h=2","h=3","h=4"),lty=1:length(hs))
@
\end{center}
\caption{Estimation of a locally linear median regression
model for the motorcycle data:  four distinct bandwidths}
\end{figure}

Fitting derivatives, of course,  requires somewhat larger bandwidth
and larger sample size to achieve the same precision as function fitting.
It is a straightforward exercise to adapt the function \texttt{lprq}
so that it does locally quadratic rather than locally linear fitting.

Another simple, yet quite general, strategy for nonparametric quantile regression
uses regression splines.  The function \texttt{bs()} in the
package \texttt{splines} gives a very flexible way to construct B-spline basis
expansions.  For example you can fit a new motorcycle model like this:

\begin{figure}
\begin{center}
<<label = regspline, fig=TRUE>>=
library(splines)
plot(times,accel,xlab = "milliseconds", ylab = "acceleration",type="n")
points(times,accel,cex = .75)
X <- model.matrix(accel ~ bs(times, df=15))
for(tau in 1:3/4){
	fit <- rq(accel ~ bs(times, df=15), tau=tau, data=mcycle)
	accel.fit <- X %*% fit$coef
	lines(times,accel.fit)
	}
@
\end{center}
\caption{B-spline estimates of the three conditional quartile
functions of the motorcycle data}
\end{figure}

The fitted conditional quantile functions do  reasonably well except
in the region beyond 50 milliseconds where the data is so sparse that
all the quantile curve want to coalesce.

This procedure fits a piecewise cubic polynomial with 15 knots
(breakpoints in the third derivative) arranged at  quantiles of the $x$'s.
(You can also explicitly specify the knot sequence and the order
of the spline using the optional arguments to \texttt{bs}.)
In this instance we have estimated three quartile curves for the
B-spline model;  a salient feature of this example is the quite
dramatic variable in variability over the time scale.  There is
essentially no variability for the first few milliseconds, but quite
substantial variability after the crash.
One advantage of the B-spline approach is that it is very easy to add a partially
linear model component.  If there were another covariate, say z,
it could be added as a parametric component using the usual \texttt{formula} syntax,

\vspace{2mm}
\noindent
\texttt{> fit <- rq(y~bs(x,df=5)+z,tau=.33)}

\vspace{2mm}
\noindent
Another appealing approach to nonparametric smoothing involves penalty
methods. \citet*{KNP.94}  describe total variation penalty methods
for fitting univariate functions;  \citet*{KM.04},
extend to approach to bivariate function estimation.
Again, partially linear models are easily adapted, and there are also easy ways to impose
monotonicity and convexity on the fitted functions.
In some applications it is desirable to consider models that involve
nonparametric fitting with several covariates.  A tractable approach
that has been explored by many authors is to build additive models.
This approach is available in \R for least squares fitting of smoothing
spline components in the \texttt{mgcv} and \texttt{gss} packages.
A prototype package for quantile regression models of this type is
available using the \texttt{rqss} function of the \texttt{quantreg}
package.

The function \texttt{rqss} offers a formula interface to
nonparametric quantile regression fitting with total variation
roughness penalties.  Consider the running speed of mammals example
from Section 7.  The objective is to estimate a model for the
upper envelope of the scatterplot, a model that would reflect
best evolutionary practice in mammalian ambulatory efficiency.
In contrast to the least squares analysis
of \citet*{Chappell.89} where they are omitted,
no special allowance is made for the ``specials'' indicated by
the plotting character \texttt{s} or ``hoppers'' indicated by
\texttt{h}.
The data is plotted on a (natural) log scale, the model is fit using
$\lambda = 1$ as the penalty parameter, and the fitted curve
is plotted using a special plotting function  that understands
the structure of the objects returned from \texttt{rqss}.
The estimated turning point of the piecewise linear
fitted function occurs at a weight of about 40 Kg.

<<label = nprq, results = hide>>=
data(Mammals)
attach(Mammals)
@
\begin{figure}
\begin{center}
<<label = mammals, fig=TRUE>>=
library(MatrixModels)
library(Matrix)
x <- log(weight)
y <- log(speed)
plot(x,y, xlab="Weight in log(Kg)", ylab="Speed in log(Km/hour)",type="n")
points(x[hoppers],y[hoppers],pch = "h", col="red")
points(x[specials],y[specials],pch = "s", col="blue")
others <- (!hoppers & !specials)
points(x[others],y[others], col="black",cex = .75)
fit <- rqss(y ~ qss(x, lambda = 1),tau = .9)
plot(fit)
@
\end{center}
\caption{Maximal Running Speed of Terrestial Mammals:  The figure
illustrates the relationship between adult body mass and maximal
running speed for 107 species of terrestial mammals. The piecewise
linear curve is an estimate of the .90 conditional quantile function
estimated subject to a constraint on the total variation of the
function gradient.}
\end{figure}

Bivariate nonparametric fitting using the triogram methods described
in Section 7 can be handled in a similar manner.
If we consider the Cobar mining data from \citet*{gree:silv.94}
\begin{figure}
\begin{center}
<<label = cobar, fig = TRUE >>=
data(CobarOre)
fit <- rqss(z ~ qss(cbind(x,y), lambda = .01, ndum=100),data = CobarOre)
plot(fit, axes = FALSE, xlab = "", ylab = "")
rm(list=ls())
@
\end{center}
\caption{Contour plot of a triogram fit of the Cobar mining data.}
\end{figure}

The \texttt{qss} term in this case case requires both $x$ and $y$
components.  In addition one needs to specify a smoothing parameter
$\lambda$, and  the parameter \texttt{ndum} may be used to specify
the number of artificial vertices introduced into the fitting
procedure in addition to the actual observations.  These artificial
vertices contribute to the penalty term, but not to the fidelity.

By default the fit is rendered as a contour plot, but there are also
two forms of perspective plots.  A conventional \R \texttt{persp} plot
can be obtained by passing option \texttt{render = "persp"} to the
plot command.  More adventurous \R~gonauts are encouraged to explore
the option \texttt{render = "rgl"}, which produces a perspective
plot in the dynamic graphics interface to the open GL library provided
by the package \texttt{rgl}.  Of course this package must be
installed.  A demonstration of how to create animations of
\texttt{rqss} triogram output using \texttt{rgl} is available by
running the command \texttt{demo(cobar)}.

Another advantage of the penalty approach embodied in \texttt{rqss}
is that it is straightforward to impose additional qualitative
constraints on the fitted functions.  Univariate functions can be
constrained to be monotone and/or convex or concave.  Bivariate
functions can be constrained to be either convex or concave.
This functionality is implemented by simply imposing nonnegativity
constraints on certain linear combinations of model parameters
and illustrates one of many possible applications for such constraints.
An interesting open research problem involves formal inference on such
constraints.

\section{Conclusion}

A few of the capabilities of the \R \texttt{quantreg}
package have been described.  Inevitably, new applications will
demand new features and reveal unforseen bugs.  In either case I hope
that users will send me their comments and suggestions.
This document will be periodically updated and the current version
will be made available in the \R distribution of \texttt{quantreg}.
See the \R command \texttt{vignette()} for details on how to find
and view vignettes from within \R.




\bibliography{abbrev,book}
\end{document}

\name{print.rq}
\alias{print.rq}
\alias{print.rqs}
\title{ Print an rq object}
\description{
Print an object generated by rq
}
\usage{
\method{print}{rq}(x, ...)
\method{print}{rqs}(x, ...)
}
\arguments{
\item{x}{
  Object returned from rq representing the fit of the model.
 }
\item{...}{
  Optional arguments. 
 }
}
\seealso{  \code{\link{rq}}}

\keyword{ regression}

\name{rq.fit.pfnb}
\alias{rq.fit.pfnb}
\title{
Quantile Regression Fitting via Interior Point Methods
}
\description{
This is a lower level routine called by \code{rq()} to compute quantile
regression parameters using the Frisch-Newton algorithm.  It uses a form
of preprocessing to accelerate the computations for situations in which
several taus are required for the same model specification.
}
\usage{
rq.fit.pfnb(x, y, tau, m0 = NULL, eps = 1e-06)
}
\arguments{
\item{x}{
The design matrix
}
\item{y}{
The response vector
}
\item{tau}{
The quantiles of interest, must lie in (0,1), be sorted and preferably equally
spaced.
}
\item{m0}{
    An initial reduced sample size by default is set to be 
    \code{round((n * (log(p) + 1) )^(2/3)} this could be explored further
    to aid performance in extreme cases.
}
\item{eps}{A tolerance parameter intended to bound the confidence band entries
    away from zero.}
}
\value{
  returns a list with elements consisting of 
  \enumerate{
      \item{coefficients}{a matrix of dimension ncol(x) by length(taus)
      }
      \item{nit} {a 5 by m matrix of iteration counts: first two coordinates
	  of each column are the number of interior point iterations, the third is the 
	  number of observations in the final globbed sample size, and the last two 
	  are the number of fixups and bad-fixups respectively.  This is intended to
	  aid fine tuning of the initial sample size, m0.}
      \item{info} {an m-vector of convergence flags}
  }
}
\details{
  The details of the Frisch-Newton algorithm are explained in Koenker and Portnoy (1997),
  as is the preprocessing idea which is related to partial sorting and the algorithms
  such as \code{kuantile} for univariate quantiles that operate in time O(n).
  The preprocessing idea of exploiting nearby quantile solutions to accelerate
  estimation of adjacent quantiles is proposed in Chernozhukov et al (2020).
  This version calls a fortran version of the preprocessing algorithm that accepts
  multiple taus.  The preprocessing approach is also implemented for a single tau
  in \code{rq.fit.pfn} which may be regarded as a prototype for this function since
  it is written entirely in R and therefore is easier to experiment with.
}
\references{
Koenker, R. and S. Portnoy (1997).
The Gaussian Hare and the Laplacian Tortoise:
Computability of squared-error vs. absolute-error estimators, with discussion,
\emph{Statistical Science}, \bold{12}, 279-300.

Chernozhukov, V., I., Fernandez-Val, and Melly, B. (2020), `Fast algorithms for 
the quantile regression process', Empirical Economics, forthcoming.
}
\seealso{
  \code{\link{rq}}, \code{\link{rq.fit.br}},
  \code{\link{rq.fit.pfn}}
}
\keyword{regression}

\name{q489}
\alias{q489}
\title{Even Quicker Sample Quantiles }
\description{
The function \code{q489} computes a single sample quantile using a 
fortran implementation of the Floyd and Rivest (1975) algorithm.
In contrast to the more elaborate function \code{kuantile} that uses
the Kiweil (2005) implementation it does not attempt to replicate the
nine varieties of quantiles as documented in the base function.
\code{quantile}
}
\usage{
q489(x, tau = .5) 
}
\arguments{
  \item{x}{numeric vector}
  \item{tau}{the quantile of intereste.}
}
\details{ This is a direct translation of the Algol 68 implementation of
Floyd and Rivest (1975), implemented in Ratfor.  For the median, average 
case behavior requires \eqn{1.5 n + O((n log n)^{1/2})} comparisons. 
In preliminary experiments it seems to be somewhat faster in large samples
than the implementation \code{kuantile} of Kiwiel (2005). See  Knuth (1998)
for further details. No provision is made for non-uniqueness of the quantile.
so, when \eqn{\tau n} is an integer there may be some discrepancy.}  
\value{
  A scalar quantile of the same length as the vector p.
}
\references{ 
R.W. Floyd and R.L. Rivest: "Algorithm 489: The Algorithm
        SELECT---for Finding the $i$th Smallest of $n$ Elements",
        Comm. ACM 18, 3 (1975) 173,

K.C. Kiwiel: On Floyd and Rivest's SELECT Algorithm, Theoretical
      Computer Sci. 347 (2005) 214-238.

D. Knuth, The Art of Computer Programming, Volume 3, Sorting and 
	Searching, 2nd Ed., (1998), Addison-Wesley.
}
\author{ R.W.Floyd and R.L.Rivest, R implementation:  Roger Koenker }
\seealso{\code{\link{quantile}}}
\examples{
     medx <- q489(rnorm(1001))
}
\keyword{univar}

\name{rq.process.object}
\alias{rq.process.object}
\title{
Linear Quantile Regression Process Object 
}
\description{
These are objects of class \code{rq.process.}
They represent the fit of a linear conditional quantile function model. 
}
\section{Generation}{
This class of objects is returned from the \code{rq}
 function 
to represent a fitted linear quantile regression model. 
}
\section{Methods}{
The \code{"rq.process"} class of objects has 
methods for the following generic 
functions: 
\code{effects}, \code{formula}
, \code{labels}
, \code{model.frame}
, \code{model.matrix}
, \code{plot}
, \code{predict}
, \code{print}
, \code{print.summary}
, \code{summary}
}
\section{Structure}{
  The following components must be included in a legitimate \code{rq.process}
  object. 

  \describe{
    \item{\code{sol}}{
      The primal solution array.  This is a (p+3)  by  J  matrix  whose  
      first  row  contains  the 'breakpoints'
      \eqn{tau_1, tau_2, \dots, tau_J},   
      of  the  quantile function, i.e. the values in [0,1] at which  the  
      solution changes,  row  two  contains  the  corresponding quantiles 
      evaluated at the mean design point, i.e. the inner product of  
      xbar  and  \eqn{b(tau_i)}, the third row contains the value of the objective
      function evaluated at the corresponding \eqn{tau_j}, and the last p rows 
      of the matrix give \eqn{b(tau_i)}.  The solution \eqn{b(tau_i)} prevails from  
      \eqn{tau_i} to \eqn{tau_i+1}.  Portnoy (1991) shows that
      \eqn{J=O_p(n \log n)}{J=O_p(n log n)}.
    }
    \item{\code{dsol}}{
      The dual solution array.  This is a
      n by J matrix  containing the  dual  solution  corresponding to sol,
      the ij-th entry is 1 if \eqn{y_i > x_i b(tau_j)}, is 0 if \eqn{y_i <  x_i
      b(tau_j)},  and is between 0 and 1 otherwise, i.e. if the
      residual is zero. See  Gutenbrunner and Jureckova(1991)
      for a detailed discussion of the statistical
      interpretation of dsol.  The use of dsol in inference is described
      in Gutenbrunner, Jureckova, Koenker, and Portnoy (1994).
    }
  }
}
\details{
  These arrays are computed by parametric linear programming methods
  using using the exterior point (simplex-type) methods of the 
  Koenker--d'Orey algorithm based on Barrodale and Roberts median
  regression algorithm.
}
\references{
  [1] Koenker, R. W. and Bassett, G. W. (1978). Regression quantiles,
  \emph{Econometrica}, \bold{46}, 33--50.

  [2] Koenker, R. W. and d'Orey (1987, 1994).
  Computing Regression Quantiles.
  \emph{Applied Statistics}, \bold{36}, 383--393, and \bold{43}, 410--414.

  [3] Gutenbrunner, C. Jureckova, J. (1991).
  Regression quantile and regression rank score process in the
  linear model and derived statistics, \emph{Annals of Statistics},
  \bold{20}, 305--330.

  [4] Gutenbrunner, C., Jureckova, J., Koenker, R. and
  Portnoy, S. (1994)  Tests of linear hypotheses based on regression  
  rank scores.   \emph{Journal of Nonparametric Statistics}, 
  (2), 307--331.

  [5]  Portnoy, S. (1991).  Asymptotic behavior of the number of regression
  quantile breakpoints, \emph{SIAM Journal of Scientific
    and  Statistical Computing}, \bold{12}, 867--883.
}
\seealso{
\code{\link{rq}}.
}
\keyword{regression}

\name{Bosco}
\alias{Bosco}
\title{Boscovich Data}
\description{
  Boscovich data used to estimate the ellipticity of the earth.
  There are five measurements of the arc length of one degree of 
  latitude taken at 5 different latitudes.  See Koenker (2005) for
  further details and references.
}
\usage{data(Bosco)}
\format{A data frame containing 5 observations on 2 variables
  \describe{
    \item{x}{sine squared of latitude measured in degrees}
    \item{y}{arc length of one degree of latitude measured in toise - 56,700,
    one toise approximately equals 1.95 meters. }
  }
}
\references{
  Koenker, R. (2005), "Quantile Regression", Cambridge.
}
\examples{
data(Bosco)
plot(0:10/10,0:10*100,xlab="sin^2(latitude)",
        ylab="arc-length of 1 degree of latitude",type="n")
points(Bosco)
text(Bosco, pos = 3, rownames(Bosco))
z <- rq(y ~ x, tau = -1, data = Bosco)
title("Boscovitch Ellipticity of the Earth Example")
xb <- c(.85,.9,.6,.6)
yb <- c(400,600,450,600)
for(i in 1:4){
        abline(c(z$sol[4:5,i]))
        interval <- paste("t=(",format(round(z$sol[1,i],2)),",",
                format(round(z$sol[1,i+1],2)),")",delim="")
        text(xb[i],yb[i],interval)
        }
}
\keyword{datasets}

\name{rq.fit.ppro}
\alias{rq.fit.ppro}
\title{
    Preprocessing fitting method for QR
}
\description{
    Preprocessing method for fitting quantile regression models that
    exploits the fact that adjacent tau's should have nearly the same
    sign vectors for residuals.
}
\usage{
rq.fit.ppro(x, y, tau, weights = NULL, Mm.factor = 0.8, eps = 1e-06, ...)
}
\arguments{
  \item{x}{
      Design matrix
}
  \item{y}{
      Response vector
}
  \item{tau}{
      quantile vector of interest 
}
  \item{weights}{
      case weights
}
  \item{Mm.factor}{
      constant determining initial sample size
}
  \item{eps}{
      Convergence tolerance
}
  \item{\dots}{
      Other arguments
}
}
\details{
    See references for further details.
}
\value{
    Returns a list with components:
    \item{coefficients}{Matrix of coefficient estimates}
    \item{residuals}{Matrix of residual estimates}
    \item{rho}{vector of objective function values}
    \item{weights}{vector of case weights}
}
\references{
     Chernozhukov, V.  I. Fernandez-Val and B. Melly,
	Fast Algorithms for the Quantile Regression Process, 2020,
	Empirical Economics.,

     Portnoy, S.  and R. Koenker, The Gaussian Hare and the Laplacian
	Tortoise, Statistical Science, (1997) 279-300
}
\author{
    Blaise Melly and Roger Koenker
}
\seealso{
\code{\link{rq.fit.pfn}}, \code{\link{boot.rq.pxy}} 
}
\keyword{regression}

\name{rq.fit.br}
\alias{rq.fit.br}
\title{
Quantile Regression Fitting by Exterior Point Methods
}
\description{
  This function controls the details of QR fitting by the simplex approach
  embodied in the algorithm of Koenker and d'Orey based on the median
  regression algorithm of Barrodale and Roberts.  Typically, options
  controlling the construction of the confidence intervals would be passed
  via the \code{\dots{}} argument of \code{rq()}.
}
\usage{
rq.fit.br(x, y, tau=0.5, alpha=0.1, ci=FALSE, iid=TRUE, interp=TRUE, tcrit=TRUE)
}
\arguments{
\item{x}{
  the design matrix
}
\item{y}{
  the response variable
}
\item{tau}{
  the quantile desired, if tau lies outside (0,1) the whole process
  is estimated.
}
\item{alpha}{
  the nominal noncoverage probability for the confidence intervals, i.e. 1-alpha
  is the nominal coverage probability of the intervals. 
}
\item{ci}{
  logical flag if T then compute confidence intervals for the parameters
  using the rank inversion method of Koenker (1994).  See \code{rq()} for more
  details.  If F then return only the estimated coefficients.  Note that
  for large problems the default option ci = TRUE can be rather slow.
  Note also that rank inversion only works for p>1, an error message is
  printed in the case that ci=T and p=1.
}
\item{iid}{
  logical flag if T then the rank inversion is based on an assumption of
  iid error model, if F then it is based on an nid error assumption.
  See Koenker and Machado (1999) for further details on this distinction.
}
\item{interp}{
  As with typical order statistic type confidence intervals the test
  statistic is discrete, so it is reasonable to consider intervals that
  interpolate between values of the parameter just below the specified
  cutoff and values just above the specified cutoff.  If \code{interp =
    F} then
  the 2 ``exact'' values above and below on which the interpolation would
  be based are returned.
}
\item{tcrit}{
Logical flag if T -  Student t critical values are used, if F then normal
values are used.
}
}
\value{
  Returns an object of class \code{"rq"}
  for tau in (0,1), or else of class \code{"rq.process"}.
  Note that \code{rq.fit.br} when called for a single tau value
  will return the vector of optimal dual variables.
  See \code{\link{rq.object}} and \code{\link{rq.process.object}}
  for further details.
}
\details{
  If tau lies in (0,1) then an object of class \code{"rq"} is
  returned with various
  related inference apparatus.  If tau lies outside [0,1] then an object
  of class \code{rq.process} is returned.  In this case parametric programming
  methods are used to find all of the solutions to the QR problem for
  tau in (0,1), the p-variate resulting process is then returned as the
  array sol containing the primal solution and dsol containing the dual
  solution.  There are roughly \eqn{O(n \log n))}{O(n log n)} distinct
  solutions, so users should
  be aware that these arrays may be large and somewhat time consuming to
  compute for large problems.
}
\references{
Koenker, R. and J.A.F. Machado, (1999) Goodness of fit and related inference 
processes for quantile regression,
\emph{J. of Am Stat. Assoc.}, 94, 1296-1310. 
}
\seealso{
\code{\link{rq}}, \code{\link{rq.fit.fnb}}
}
\examples{
data(stackloss)
rq.fit.br(stack.x, stack.loss, tau=.73 ,interp=FALSE)
}
\keyword{regression}

\name{srisk}
\alias{srisk}
\title{ Markowitz (Mean-Variance) Portfolio Optimization}
\description{
This function estimates optimal mean-variance portfolio weights from a matrix
of historical or simulated asset returns.  
}
\usage{
srisk(x, mu = 0.07, lambda = 1e+08, alpha = 0.1, eps = 1e-04)
}
\arguments{
  \item{x}{ Matrix of asset returns }
  \item{mu}{Required mean rate of return for the portfolio }
  \item{lambda}{Lagrange multiplier associated with mean return constraint}
  \item{alpha}{Choquet risk parameter, unimplemented }
  \item{eps}{ tolerance parameter for mean return constraint}
}
\details{
The portfolio weights are estimated by solving a constrained least squares problem.
}
\value{
  \item{pihat}{Optimal portfolio weights}
  \item{muhat }{Mean return in sample}
  \item{sighat }{Standard deviation of returns in sample}
}
\author{ R. Koenker }

\seealso{ \code{\link{qrisk}}}

\keyword{ regression }

\name{nlrq}
\alias{nlrq}
\alias{nlrqModel}
\alias{print.nlrq} 
\alias{summary.nlrq} 
\alias{deviance.nlrq}
\alias{formula.nlrq} 
\alias{coef.nlrq} 
\alias{fitted.nlrq} 
\alias{logLik.nlrq} 
\alias{AIC.nlrq} 
\alias{extractAIC.nlrq} 
\alias{predict.nlrq} 
\alias{print.summary.nlrq} 
\alias{tau.nlrq}
\title{ Function to compute nonlinear quantile regression estimates}
\description{
This function implements an R version of an interior point method
for computing the solution to quantile regression problems which
are nonlinear in the parameters.  The algorithm is based on interior
point ideas described in Koenker and Park (1994).
}
\usage{
nlrq(formula, data=parent.frame(), start, tau=0.5, 
	control, trace=FALSE,method="L-BFGS-B")
\method{summary}{nlrq}(object, ...)
\method{print}{summary.nlrq}(x, digits = max(5, .Options$digits - 2), ...)
}
\arguments{
  \item{formula}{ formula for model in nls format; accept self-starting models }
  \item{data}{ an optional data frame in which to evaluate the variables in
	 `formula' }
  \item{start}{a named list or named numeric vector of starting estimates }
  \item{tau}{ a vector of quantiles to be estimated}
  \item{control}{ an optional list of control settings.  See `nlrq.control' for
          the names of the settable control values and their effect.}
  \item{trace}{ logical value indicating if a trace of the iteration progress
          should be printed.  Default is `FALSE'.  If `TRUE' intermediary results
          are printed at the end of each iteration. } 
  \item{method}{ method passed to optim for line search, default is "L-BFGS-B"
	but for some problems "BFGS" may be preferable.  See \code{\link{optim}} for
	further details.  Note that the algorithm wants to pass
	upper and lower bounds for the line search to optim, which is fine for
	the L-BFGS-B method.  Use of other methods will produce warnings about
	these arguments -- so users should proceed at their own risk.}
  \item{object}{an object of class nlrq needing summary.}
  \item{x}{an object of class summary.nlrq needing printing.}
  \item{digits}{Significant digits reported in the printed table.}
  \item{...}{Optional arguments passed to printing function.}
}
\details{An `nlrq' object is a type of fitted model object.  It has methods
     for the generic functions `coef' (parameters estimation at best solution),
     `formula' (model used), `deviance' (value of the objective function at best 
     solution), `print', `summary', `fitted' (vector of fitted variable according
     to the model), `predict' (vector of data points predicted by the model, using
     a different matrix for the independent variables) and also for the function
     `tau' (quantile used for fitting the model, as the tau argument of the
     function). Further help is also available for the method `residuals'.
     The summary method for nlrq uses a bootstrap approach based on the final
     linearization of the model evaluated at the estimated parameters.  
}


\value{
  A list consisting of: 
  \item{m}{an `nlrqModel' object similar to an `nlsModel' in package nls}
  \item{data }{the expression that was passed to `nlrq' as the data argument.
           The actual data values are present in the environment of the
          `m' component. }
}

\author{Based on S code by Roger Koenker modified for R and to accept models
as specified by nls by Philippe Grosjean.}

\references{ Koenker, R. and Park, B.J. (1994). An Interior Point Algorithm for
Nonlinear Quantile Regression, Journal of Econometrics, 71(1-2): 265-283.
}

\seealso{  \code{\link{nlrq.control}} , \code{\link{residuals.nlrq}} }

\examples{
# build artificial data with multiplicative error
Dat <- NULL; Dat$x <- rep(1:25, 20)
set.seed(1)
Dat$y <- SSlogis(Dat$x, 10, 12, 2)*rnorm(500, 1, 0.1)
plot(Dat)
# fit first a nonlinear least-square regression
Dat.nls <- nls(y ~ SSlogis(x, Asym, mid, scal), data=Dat); Dat.nls
lines(1:25, predict(Dat.nls, newdata=list(x=1:25)), col=1)
# then fit the median using nlrq
Dat.nlrq <- nlrq(y ~ SSlogis(x, Asym, mid, scal), data=Dat, tau=0.5, trace=TRUE)
lines(1:25, predict(Dat.nlrq, newdata=list(x=1:25)), col=2)
# the 1st and 3rd quartiles regressions
Dat.nlrq <- nlrq(y ~ SSlogis(x, Asym, mid, scal), data=Dat, tau=0.25, trace=TRUE)
lines(1:25, predict(Dat.nlrq, newdata=list(x=1:25)), col=3)
Dat.nlrq <- nlrq(y ~ SSlogis(x, Asym, mid, scal), data=Dat, tau=0.75, trace=TRUE)
lines(1:25, predict(Dat.nlrq, newdata=list(x=1:25)), col=3)
# and finally "external envelopes" holding 95 percent of the data
Dat.nlrq <- nlrq(y ~ SSlogis(x, Asym, mid, scal), data=Dat, tau=0.025, trace=TRUE)
lines(1:25, predict(Dat.nlrq, newdata=list(x=1:25)), col=4)
Dat.nlrq <- nlrq(y ~ SSlogis(x, Asym, mid, scal), data=Dat, tau=0.975, trace=TRUE)
lines(1:25, predict(Dat.nlrq, newdata=list(x=1:25)), col=4)
leg <- c("least squares","median (0.5)","quartiles (0.25/0.75)",".95 band (0.025/0.975)")
legend(1, 12.5, legend=leg, lty=1, col=1:4)
}
\keyword{models}  
\keyword{regression}
\keyword{nonlinear}
\keyword{robust}

\name{rq.fit}
\alias{rq.fit}
\title{Function to choose method for Quantile Regression  }
\usage{
rq.fit(x, y, tau=0.5, method="br", ...)
}
\arguments{
\item{x}{
  the design matrix
}
\item{y}{
  the response variable
}
\item{tau}{
  the quantile desired, if tau lies outside (0,1) the whole process
  is estimated.
}
\item{method}{
  method of computation:  "br" is Barrodale and Roberts exterior point
  "fn" is the Frisch-Newton interior point method.
 }
\item{...}{
  Optional arguments passed to fitting routine.
 }
}

\description{Function to choose method for quantile regression}
\seealso{  \code{\link{rq}} \code{\link{rq.fit.br}} \code{\link{rq.fit.fnb}}}

\keyword{ regression }

\name{plot.summary.rqs}
\alias{plot.summary.rqs}
\alias{plot.summary.rq}

\title{Visualizing sequences of quantile regression summaries}

\description{A sequence of coefficient estimates for quantile
  regressions with varying \code{tau} parameters is visualized
  along with associated confidence bands.}

\usage{
\method{plot}{summary.rqs}(x, parm = NULL, level = 0.9, ols = TRUE,
  mfrow = NULL, mar = NULL, ylim = NULL, main = NULL,
  col = gray(c(0, 0.75)), border = NULL, lcol = 2, lty = 1:2,
  cex = 0.5, pch = 20, type = "b", xlab = "", ylab = "", \dots)
}

\arguments{
  \item{x}{an object of class \code{"summary.rqs"} as produce by 
    applying the \code{summary} method to a \code{\link{rq}} object
    (with a vector of \code{tau} values).}
  \item{parm}{a specification of which parameters are to be plotted,
    either a vector of numbers or a vector of names.  By default, all
    parameters are considered.}
  \item{level}{Confidence level of bands.  When using
    the rank based confidence intervals in summary, which is the default
    method for sample sizes under 1000, you will need to control the level
    of the intervals by passing the parameter alpha to 
    \code{\link{summary.rq}}, prior to calling 
    \code{\link{plot.summary.rqs}}.  Note also that alpha = 1 - level.}
  \item{ols}{logical. Should a line for the OLS coefficient and their confidence
    bands (as estimated by \code{\link[stats]{lm}}) be added?}
  \item{mfrow, mar, ylim, main}{graphical parameters. Suitable defaults are chosen
    based on the coefficients to be visualized.  It can be useful to use a common
    vertical scale when plotting as a way of comparing confidence bands constructed
    by different methods. For this purpose one can specify a \code{ylim} as a 
     2 by \code{length(parm)} matrix.}
  \item{col}{vector of color specification for \code{rq} coefficients
    and the associated confidence polygon.}
  \item{border}{color specification for the confidence polygon. By default,
    the second element of \code{col} is used.}
  \item{lcol, lty}{color and line type specification for OLS coefficients
    and their confidence bounds.}
  \item{cex, pch, type, xlab, ylab, \dots}{further graphical parameters
    passed to \code{\link[graphics]{points}}.}
}

\details{The \code{plot} method for \code{"summary.rqs"} objects visualizes
  the coefficients along with their confidence bands. The bands can be
  omitted by using the \code{plot} method for \code{"rqs"} objects directly.}

\seealso{\code{\link{rq}}, \code{\link{plot.rqs}}}

\value{A list with components \code{z}, an array with all coefficients visualized 
    (and associated confidence bands), and \code{Ylim}, a 2 by p matrix containing  
    the y plotting limits.  The latter component may be useful for establishing a
    common scale for two or more similar plots.  The list is returned invisibly.}

\examples{
## fit Engel models (in levels) for tau = 0.1, ..., 0.9
data("engel")
fm <- rq(foodexp ~ income, data = engel, tau = 1:9/10)
sfm <- summary(fm)

## visualizations
plot(sfm)
plot(sfm, parm = 2, mar = c(5.1, 4.1, 2.1, 2.1), main = "", xlab = "tau", 
  ylab = "income coefficient", cex = 1, pch = 19)
}

\keyword{hplot}

\name{table.rq}
\alias{table.rq}
\alias{latex.table.rq}
\alias{plot.table.rq}
\title{
  Table of Quantile Regression Results
}
\description{
  Defunct Function to produce a table of quantile regression results for a group
  of specified quantiles.  See \code{rq} which now permits multiple taus.
}
\usage{
table.rq(x, \dots)
}
\arguments{
  \item{x}{
	input
  }
  \item{...}{
    other optional arguments 
  }
}
\value{
	None.
}
\seealso{ \code{\link{rq}}, }
\keyword{regression}

\name{plot.rqss}
\alias{plot.rqss}
\alias{plot.qss1}
\alias{plot.qts1}
\alias{plot.qss2}
\alias{plot.summary.rqss}
\title{Plot Method for rqss Objects}
\description{
Takes a fitted \code{rqss} object produced by \code{rqss()} and plots
the component smooth functions that make up the ANOVA decomposition.
Since the components "omit the intercept" the estimated intercept is added back
in -- this facilitates the comparison of quantile fits particularly.
For models with a partial linear component or several \code{qss} components
it may be preferable to plot the output of \code{predict.rqss}.
Note that these functions are intended to plot \code{rqss} objects only, attempting
to plot {summary.rqss}  objects just generates a warning message.
}
\usage{
\method{plot}{rqss}(x, rug = TRUE, jit = TRUE, bands = NULL, coverage = 0.95,
	add = FALSE, shade = TRUE, select = NULL, pages = 0, titles = NULL, 
	bcol = NULL, \dots)
\method{plot}{qss1}(x, rug = TRUE, jit = TRUE, add = FALSE, \dots)
\method{plot}{qts1}(x, rug = TRUE, jit = TRUE, add = FALSE, \dots)
\method{plot}{qss2}(x, render = "contour", ncol = 100, zcol = NULL, \dots)
\method{plot}{summary.rqss}(x, \dots)
}
\arguments{
  \item{x}{a fitted \code{rqss} object produced by \code{\link{rqss}()}.}
  \item{...}{additional arguments for the plotting algorithm}
  \item{rug}{if TRUE, a rugplot for the x-coordinate is plotted}
  \item{jit}{if TRUE, the x-values of the rug plot are jittered}
  \item{bands}{if TRUE, confidence bands for the smoothed effects are plotted, if
    "uniform" then uniform bands are plotted, if "both" then both the uniform
    and the pointwise bands are plotted.}
  \item{coverage}{desired coverage probability of confidence bands, if requested}
  \item{select}{vector of indices of qss objects to be plotted, by default all}
  \item{pages}{number of pages desired for the plots}
  \item{render}{a character specifying the rendering for bivariate fits;
    either \code{"contour"} (default) or \code{"rgl"}.  The latter
    requires package \pkg{rgl}.}
  \item{add}{if TRUE then add qss curve to existing (usually) scatterplot,
    otherwise initiate a new plot}
  \item{shade}{if TRUE then shade the confidence band}
  \item{titles}{title(s) as vector of character strings, by default titles are chosen for
	each plot as "Effect of CovariateName"}
  \item{bcol}{vector of two colors for confidence bands}
  \item{ncol, zcol}{Only for \code{render = "rgl"}: number of colors and
    z values for color construction.}%% FIXME: be more specific
}
\value{
    The function produces plots for the ANOVA components as a side effect. For
\code{"qss1"} the \code{"add = TRUE"} can be used to overplot the fit on a 
scatterplot.  When there are multiple pages required \code{"par(ask = TRUE)"}
is turned on so that the plots may be examined sequentially.  If \code{bands != NULL}
then a list with three components for each qss component is returned (invisibly):
	\item{x}{The x coordinates of the confidence bands}
	\item{blo}{The y coordinates of the lower confidence curve, if 
		\code{bands = "both"} then this is a  matrix with two columns}
	\item{bhi}{The y coordinates of the upper confidence curve, if
	 	\code{bands = "both"} then this is a  matrix with two columns}	
}
\details{
    For univariate \code{qss} components with \code{Dorder = 0} the fitted
    function is piecewise constant, not piecewise linear.  In this case the constraints 
    are limited to increasing, decreasing or none.  
If \code{bands == "uniform"} then the bands are uniform bands based on the 
Hotelling (1939) tube approach.  See also Naiman (1986), 
Johansen and Johnstone (1990), Sun and Loader (1994), 
and Krivobokova, Kneib, and Claeskens (2009), in particular the computation of
the "tube length" is based on the last of these references.  If \code{bands}
is non null, and not "uniform" then pointwise bands are returned.
Since bands for bivariate components are not (yet) supported, if requested
such components will be returned as \code{NULL}.
}
\author{ Roger Koenker }
\seealso{ \code{\link{rqss}}}
\references{
[1] Hotelling, H.  (1939): ``Tubes and Spheres in $n$-spaces, and a class
  of statistical problems,'' \emph{Am J. Math}, 61, 440--460.

[2] Johansen, S.,   and I.M. Johnstone  (1990): ``Hotelling's
  Theorem on the Volume of Tubes: Some Illustrations in Simultaneous
  Inference and Data Analysis,'' \emph{The Annals of Statistics}, 18, 652--684.

[3] Naiman, D. (1986)  Conservative confidence bands in curvilinear regression,  
  \emph{The Annals of Statistics}, 14, 896--906.

[4] Sun, J. and C.R. Loader, (1994) Simultaneous confidence bands for linear
regression and smoothing, \emph{The Annals of Statistics}, 22, 1328--1345.

[5] Krivobokova, T., T. Kneib, and G. Claeskens (2009) Simultaneous Confidence
Bands for Penalized Spline Estimators, preprint.

[6] Koenker, R. (2010) Additive Models for Quantile Regression:  Model Selection
and Confidence Bandaids, preprint.
}
\examples{
n <- 200
x <- sort(rchisq(n,4))
z <- x + rnorm(n)
y <- log(x)+ .1*(log(x))^2 + log(x)*rnorm(n)/4 + z
plot(x,y-z)
fN <- rqss(y~qss(x,constraint="N")+z)
plot(fN)
fI <- rqss(y~qss(x,constraint="I")+z)
plot(fI,  col="blue")
fCI <- rqss(y~qss(x,constraint="CI")+z)
plot(fCI, col="red")
%%keep objects for inspection : do not rm(n,x,y,z,fit)

## A bivariate example
data(CobarOre)
fCO <- rqss(z~qss(cbind(x,y),lambda=.08), data = CobarOre)
plot(fCO)
}
\keyword{regression}
\keyword{smooth}
\keyword{iplot}

\name{critval}
\alias{critval}
\title{
Hotelling Critical Values
}
\description{
Critical values for uniform confidence bands for rqss fitting
}
\usage{
critval(kappa, alpha = 0.05, rdf = 0)
}
\arguments{
\item{kappa}{
  length of the tube
}
\item{alpha}{
  desired non-coverage of the band, intended coverage is 1 - alpha
}
\item{rdf}{
  "residual" degrees of freedom of the fitted object.  If \code{rdf=0}
  then the Gaussian version of the critical value is computed, otherwise
  the value is based on standard Student t theory.
}
}
\value{
  A scalar critical value that acts as a multiplier for the uniform
  confidence band construction.
}
\details{
  The Hotelling tube approach to inference has a long and illustrious
  history.  See Johansen and Johnstone (1989) for an overview.  The implementation
  here is based on Sun and Loader (1994) and Loader's \pkg{locfit} package, although
  a simpler root finding approach is substituted for the iterative method used
  there.  At this stage, only univariate bands may be constructed.
}
\references{
Hotelling, H.  (1939): ``Tubes and Spheres in $n$-spaces, and a class
  of statistical problems,'' \emph{Am J. Math}, 61, 440--460.

Johansen, S.,   I.M. Johnstone  (1990): ``Hotelling's
  Theorem on the Volume of Tubes: Some Illustrations in Simultaneous
  Inference and Data Analysis,'' \emph{The Annals of Statistics}, 18, 652--684.

Sun, J. and C.V. Loader:  (1994) ``Simultaneous Confidence Bands for Linear Regression
and smoothing,'' \emph{The Annals of Statistics}, 22, 1328--1345.
}
\seealso{
  \code{\link{plot.rqss}}
}
\keyword{regression}

\name{boot.rq.pwxy}
\alias{boot.rq.pwxy}
\title{
    Preprocessing weighted bootstrap method
}
\description{
    Bootstrap method exploiting preprocessing strategy to reduce 
    computation time for large problem.  In contrast to 
    \code{\link{boot.rq.pxy}}  which uses the classical multinomial 
    sampling scheme and is coded in R, this uses the exponentially
    weighted bootstrap scheme and is coded in fortran and consequently
    is considerably faster in larger problems.
}
\usage{
boot.rq.pwxy(x, y, tau, coef, R = 200, m0 = NULL, eps = 1e-06, ...)
}
\arguments{
  \item{x}{
      Design matrix
}
  \item{y}{
      response vector
}
  \item{tau}{
      quantile of interest
}
  \item{coef}{
      point estimate of fitted object
}
  \item{R}{
      the number of bootstrap replications desired.
}
  \item{m0}{
      constant to determine initial sample size, defaults to sqrt(n*p)
      but could use some further tuning...
}
  \item{eps}{
      tolerance for convergence of fitting algorithm
}
  \item{...}{
      other parameters not yet envisaged.
}
}
\details{
    The fortran implementation is quite similar to the R code for 
    \code{\link{boot.rq.pxy}} except that there is no multinomial sampling.
    Instead \code{rexp(n)} weights are used.
}
\value{
  returns a list with elements: 
  \enumerate{
      \item{coefficients}{a matrix of dimension ncol(x) by R}
      \item{nit} {a 5 by m matrix of iteration counts}
      \item{info} {an m-vector of convergence flags}
  }
}
\references{
    Chernozhukov, V.  I. Fernandez-Val and B. Melly,
	Fast Algorithms for the Quantile Regression Process, 2019,
	arXiv, 1909.05782,

     Portnoy, S.  and R. Koenker, The Gaussian Hare and the Laplacian
	Tortoise, Statistical Science, (1997) 279-300
}
\author{
    Blaise Melly and Roger Koenker
}
\seealso{
\code{\link{boot.rq.pxy}}
}
\keyword{bootstrap} 

\name{rq.object}
\alias{rq.object}
\alias{formula.rq}
\alias{logLik.rq}
\alias{logLik.rqs}
\alias{AIC.rq}
\alias{AIC.rqs}
\alias{extractAIC.rq}
\title{
Linear Quantile Regression Object 
}
\description{
  These are objects of class \code{"rq"}.
  They represent the fit of a linear conditional quantile function model. 
}
\section{Generation}{
  This class of objects is returned from the \code{rq} function 
  to represent a fitted linear quantile regression model. 
}
\section{Methods}{
  The \code{"rq"} class of objects has methods for the following generic 
functions: 
\code{coef}, \code{effects}
, \code{formula}
, \code{labels}
, \code{model.frame}
, \code{model.matrix}
, \code{plot}
, \code{logLik}
, \code{AIC}
, \code{extractAIC}
, \code{predict}
, \code{print}
, \code{print.summary}
, \code{residuals}
, \code{summary}
}
\section{Structure}{
  The following components must be included in a legitimate \code{rq} object. 
  \describe{
    \item{\code{coefficients}}{
      the coefficients of the quantile regression fit. 
      The names of the coefficients are the names of the 
      single-degree-of-freedom effects (the columns of the 
      model matrix). 
      If the model was fitted by method \code{"br"} with \code{ci=TRUE}, then
      the coefficient component consists of a matrix whose
      first column consists of the vector of estimated coefficients
      and the second and third columns are the lower and upper
      limits of a confidence interval for the respective coefficients.
    }
    \item{\code{residuals}}{
      the residuals from the fit. 
    }
    \item{\code{dual}}{
      the vector dual variables from the fit. 
    }
    \item{\code{rho}}{
      The value(s) of objective function at the solution.
    }
    \item{\code{contrasts}}{
      a list containing sufficient information to construct the contrasts 
      used to fit any factors occurring in the model. 
      The list contains entries that are either matrices or character vectors. 
      When a factor is coded by contrasts, the corresponding contrast matrix 
      is stored in this list. 
      Factors that appear only as dummy variables and variables in the model 
      that are matrices correspond to character vectors in the list. 
      The character vector has the level names for a factor or the column 
      labels for a matrix. 
    }
    \item{\code{model}}{
      optionally the model frame, if \code{model=TRUE}. 
    }
    \item{\code{x}}{
      optionally the model matrix, if \code{x=TRUE}. 
    }
    \item{\code{y}}{
      optionally the response, if \code{y=TRUE}. 
    }
  }
}
\details{
  The coefficients, residuals, and effects may be extracted 
  by the generic functions of the same name, rather than 
  by the \code{$} operator.   For pure \code{rq} objects this is less critical 
  than for some of the inheritor classes.  In particular, for fitted rq objects
  using "lasso" and "scad" penalties, \code{logLik} and \code{AIC} functions
  compute degrees of freedom of the fitted model as the number of estimated
  parameters whose absolute value exceeds a threshold \code{edfThresh}.  By
  default this threshold is 0.0001, but this can be passed via the \code{AIC}
  function if this value is deemed unsatisfactory.  The function \code{AIC}
  is a generic function in R, with parameter \code{k} that controls the form
  of the penalty:  the default value of \code{k} is 2 which yields the classical
  Akaike form of the penalty, while \code{k <= 0} yields the Schwarz (BIC)
  form of the penalty.
  Note that the extractor function \code{coef} returns a vector with missing values 
  omitted.  
}
\seealso{
  \code{\link{rq}},  \code{\link{coefficients}}.   
}
\keyword{regression}

\name{rq.fit.fnb}
\alias{rq.fit.fnb}
\title{
Quantile Regression Fitting via Interior Point Methods
}
\description{
This is a lower level routine called by \code{rq()} to compute quantile
regression methods using the Frisch-Newton algorithm.
}
\usage{
rq.fit.fnb(x, y, tau=0.5, rhs = (1-tau)*apply(x,2,sum), beta=0.99995, eps=1e-06)
}
\arguments{
\item{x}{
The design matrix
}
\item{y}{
The response vector
}
\item{tau}{
The quantile of interest, must lie in (0,1)
}
\item{rhs}{
The right hand size of the dual equality constraint, modify at your own risk.
}
\item{beta}{
technical step length parameter -- alter at your own risk!
}
\item{eps}{
  tolerance parameter for convergence.  In cases of multiple optimal solutions
  there may be some discrepancy between solutions produced by method
  \code{"fn"} and method \code{"br"}.  This is due to the fact that
  \code{"fn"} tends to converge to a point near the centroid of the
  solution set, while \code{"br"} stops at a vertex of the set.  
}
}
\value{
  returns an object of class \code{"rq"}, which can be passed to
  \code{\link{summary.rq}} to obtain standard errors, etc.
}
\details{
  The details of the algorithm are explained in Koenker and Portnoy (1997).
  The basic idea can be traced back to the log-barrier methods proposed by
  Frisch in the 1950's for constrained optimization.  But the current
  implementation is based on proposals by Mehrotra and others in the
  recent (explosive) literature on interior point methods for solving linear 
  programming problems.  This function replaces an earlier one \code{rq.fit.fn},
  which required the initial dual values to be feasible.  This version allows the
  user to specify an infeasible starting point for the dual problem, that
  is one that may not satisfy the dual equality constraints.  It still 
  assumes that the starting value satisfies the upper and lower bounds.
}
\references{
Koenker, R. and S. Portnoy (1997).
The Gaussian Hare and the Laplacian Tortoise:
Computability of squared-error vs. absolute-error estimators, with discussion,
\emph{Statistical Science}, \bold{12}, 279-300.
}
\seealso{
  \code{\link{rq}}, \code{\link{rq.fit.br}},
  \code{\link{rq.fit.pfn}}
}
\keyword{regression}

\name{MelTemp}
\alias{MelTemp}
\docType{data}
\title{Daily maximum temperatures in Melbourne, Australia}
\description{
     Daily maximum temperatures in Melbourne, Australia, from
     1981-1990. Leap days have been omitted.
}
\usage{data(MelTemp)}
\format{Time series of frequency 365}

\source{
Hyndman, R.J., Bashtannyk, D.M. and Grunwald, G.K. (1996)
     "Estimating and visualizing conditional densities". _Journal of
     Computational and Graphical Statistics_, *5*, 315-336.
}
\examples{
data(MelTemp)
demo(Mel)
}
\keyword{datasets}

\name{predict.rqss}
\alias{predict.rqss}
\alias{predict.qss1}
\alias{predict.qss2}
\title{Predict from fitted nonparametric quantile regression smoothing spline models}
\description{
  Additive models for nonparametric quantile regression using total
  variation penalty methods can be fit with the \code{\link{rqss}}
  function.  Univarariate and bivariate components can be predicted
  using these functions.
}
\usage{
\method{predict}{rqss}(object, newdata, interval = "none", level = 0.95, ...)
\method{predict}{qss1}(object, newdata, ...)
\method{predict}{qss2}(object, newdata, ...)
}
\arguments{
\item{object}{ is a fitted object produced by \code{\link{rqss}} }
\item{newdata}{ a data frame describing the observations at which
  prediction is to be made.  For qss components, newdata should
  lie in strictly within the convex hull of the fitting data.  Newdata
  corresponding to the partially linear component of the model
  may require caution concerning the treatment of factor levels, if any.}
\item{interval}{If set to \code{confidence} then a \code{level} confidence interval
for the predictions is returned.}
\item{level}{intended coverage probability for the confidence intervals}
\item{\dots}{ optional arguments }
}
\details{
For both univariate and bivariate prediction linear interpolation is
done.  In the bivariate case, this involves computing barycentric
coordinates of the new points relative to their enclosing triangles.
It may be of interest to plot individual components of fitted rqss
models:  this is usually best done by fixing the values of other
covariates at reference values typical of the sample data and
predicting the response at varying values of one qss term at a
time.   Direct use of the \code{predict.qss1} and \code{predict.qss2} functions
is discouraged since it usually corresponds to predicted values
at absurd  reference values of the other covariates, i.e. zero.
}
\value{
A vector of predictions, or in the case that \code{interval = "confidence")}
a matrix whose first column is the vector of predictions and whose second and
third columns are the lower and upper confidence limits for each prediction.
}
\author{ R. Koenker }
\seealso{ \code{\link{rqss}}
}
\examples{
n <- 200
lam <- 2
x <- sort(rchisq(n,4))
z <- exp(rnorm(n)) + x
y <- log(x)+ .1*(log(x))^2 + z/4 +  log(x)*rnorm(n)/4
plot(x,y - z/4 + mean(z)/4)
Ifit <- rqss(y ~ qss(x,constraint="I") + z)
sfit <- rqss(y ~ qss(x,lambda = lam) + z)
xz <- data.frame(z = mean(z),
                 x = seq(min(x)+.01,max(x)-.01,by=.25))
lines(xz[["x"]], predict(Ifit, xz), col=2)
lines(xz[["x"]], predict(sfit, xz), col=3)
legend(10,2,c("Increasing","Smooth"),lty = 1, col = c(2,3))
title("Predicted Median Response at Mean Value of z")
%%keep objects for inspection : do not rm(x,y,z,xz,fit)

## Bivariate example -- loads pkg "tripack"
require(tripack)
require(akima)
data(CobarOre)
fit <- rqss(z ~ qss(cbind(x,y), lambda=.08),
            data= CobarOre)
plot(fit, col="grey",
     main = "CobarOre data -- rqss(z ~ qss(cbind(x,y)))")
T <- with(CobarOre, tri.mesh(x, y))
set.seed(77)
ndum <- 100
xd <- with(CobarOre, runif(ndum, min(x), max(x)))
yd <- with(CobarOre, runif(ndum, min(y), max(y)))
table(s <- in.convex.hull(T, xd, yd))
pred <- predict(fit, data.frame(x = xd[s], y = yd[s]))
contour(interp(xd[s],yd[s], pred),
        col="red", add = TRUE)
}
\keyword{regression}
\keyword{smooth}
\keyword{robust}

\name{boot.rq}
\alias{boot.rq}
\alias{boot.rq.xy}
\alias{boot.rq.wxy}
\alias{boot.rq.pwy}
\alias{boot.rq.spwy}
\alias{boot.rq.mcmb}
\title{ Bootstrapping Quantile Regression}
\description{
These functions can be used to construct standard errors, confidence
intervals and tests of hypotheses regarding quantile regression models.
}
\usage{
boot.rq(x, y, tau = 0.5, R = 200, bsmethod = "xy", mofn = length(y), 
	coef = NULL, blbn = NULL, cluster = NULL, U = NULL,  ...)
}
\arguments{
  \item{x}{ The regression design matrix}
  \item{y}{ The regression response vector}
  \item{tau}{ The quantile of interest}
  \item{R}{ The number of bootstrap replications}
  \item{bsmethod}{ The method to be employed.  There are (as yet) five
	options:  method = "xy" uses the xy-pair method, and
	method = "pwy" uses the method of Parzen, Wei and Ying (1994)
	method = "mcmb" uses the Markov chain marginal bootstrap
	of He and Hu (2002) and Kocherginsky, He and Mu (2003).
	The fourth method = "wxy" uses the generalized bootstrap
	of Bose and Chatterjee (2003) with unit exponential weights,
	see also Chamberlain and Imbens (2003).  The fifth method
	"wild" uses the wild bootstrap method proposed by Feng, He and Hu (2011). }
  \item{mofn}{ optional argument for the bootstrap method "xy" that
	permits subsampling (m out of n) bootstrap.  Obviously mofn
	should be substantially larger than the column dimension of x,
	and should be less than the sample size.}
  \item{coef}{coefficients from initial fitted object}
  \item{blbn}{orginal sample size for the BLB model}
  \item{cluster}{If non-NULL this argument should specify cluster id
      numbers for each observation, in which case the clustered version of 
      the bootstrap based on the proposal of Hagemann (2017). If present
      \code{bsmethod} is set to set to "cluster". If this option is used
      and the fitting method for the original call was "sfn" then the
      bootstrapping will be carried out with the "sfn" as well.  This
      is usually substantially quicker than the older version which
      employed the "br" variant of the simplex method.  Use of "sfn"
      also applies to the "pwy" method when the original fitting 
      was done with "sfn".  Finally, if \code{na.action = "omit"} and
      \code{length(object$na.action) > 0} then these elements are also
      removed from the \code{cluster} variable.  Consequently, the 
      length of the \code{cluster} variable should always be the same
      as the length of the original response variable before any 
      \code{na.action} takes place.  }
  \item{U}{If non-NULL this argument should specify an array of indices
      or gradient evaluations to be used by the corresponding bootstrap
      method as specified by \code{bsmethod}.  This is NOT intended as
      a user specified input, instead it is specified in \code{summary.rqs}
      to ensure that bootstrap samples for multiple taus use the same
      realizations of the random sampling.}
  \item{...}{ Optional arguments to control bootstrapping}
}
\details{
Their are several refinements that are still unimplemented.  Percentile
methods should be incorporated, and extensions of the methods to be used 
in anova.rq should be made.  And more flexibility about what algorithm is
used would also be good. 
}
\value{
A list consisting of two elements:
  A matrix \code{B} of dimension R by p is returned with the R resampled
	estimates of the vector of quantile regression parameters. When
	mofn < n for the "xy" method this matrix has been deflated by
	the factor sqrt(m/n).
  A matrix \code{U} of sampled indices (for \code{bsmethod in c("xy", "wxy")}) 
	or gradient evaluations (for \code{bsmethod in c("pwy", "cluster")})
	used to generate the bootstrapped realization, and potentially reused
	for other \code{taus} when invoked from \code{summary.rqs}.
}
\references{ 
[1] Koenker, R. W. (1994). Confidence Intervals for regression quantiles, in
P. Mandl and M. Huskova (eds.), \emph{Asymptotic Statistics}, 349--359,
Springer-Verlag, New York.

[2] Kocherginsky, M., He, X. and Mu, Y. (2005).
Practical Confidence Intervals for Regression Quantiles,
Journal of Computational and Graphical Statistics, 14, 41-55.

[3] Hagemann, A. (2017) Cluster Robust Bootstrap inference in 
quantile regression models, Journal of the American Statistical Association , 
112, 446--456.

[4] He, X. and Hu, F. (2002). Markov Chain Marginal Bootstrap.
Journal of the American Statistical Association , Vol. 97, no. 459,
783-795. 

[5] Parzen, M. I., L. Wei,  and Z. Ying  (1994): A resampling
method based on pivotal estimating functions,'' Biometrika, 81, 341--350.

[6] Bose, A. and S. Chatterjee, (2003) Generalized bootstrap for estimators
of minimizers of convex functions, \emph{J. Stat. Planning and Inf}, 117, 225-239.

[7]  Chamberlain G.  and Imbens G.W.  (2003) Nonparametric Applications of 
Bayesian Inference, Journal of Business & Economic Statistics, 21, pp. 12-18.

[8]  Feng, Xingdong, Xuming He, and Jianhua Hu (2011) Wild Bootstrap for
Quantile Regression, Biometrika, 98, 995--999. 
}

\author{ Roger Koenker (and Xuming He and M. Kocherginsky for the mcmb code)}
\seealso{  \code{\link{summary.rq}}}
\examples{
y <- rnorm(50)
x <- matrix(rnorm(100),50)
fit <- rq(y~x,tau = .4)
summary(fit,se = "boot", bsmethod= "xy")
summary(fit,se = "boot", bsmethod= "pwy")
#summary(fit,se = "boot", bsmethod= "mcmb")
}
\keyword{ regression}

\name{LassoLambdaHat}
\alias{LassoLambdaHat}
\title{Lambda selection for QR lasso problems}
\description{
    Default procedure for selection of lambda in lasso constrained
    quantile regression as proposed by Belloni and Chernozhukov (2011)
}
\usage{
LassoLambdaHat(X, R = 1000, tau = 0.5, C = 1, alpha = 0.95)
}
\arguments{
  \item{X}{Design matrix}
  \item{R}{Number of replications}
  \item{tau}{quantile of interest}
  \item{C}{Cosmological constant}
  \item{alpha}{Interval threshold}
}
\value{
    vector of default lambda values of length p, the column dimension of X.
}
\details{
  As proposed by Belloni and Chernozhukov, a reasonable default lambda
  would be the upper quantile of the simulated values.  The procedure is based 
  on idea that a simulated gradient can be used as a pivotal statistic.
  Elements of the default vector are standardized by the respective standard deviations
  of the covariates. Note that the sqrt(tau(1-tau)) factor cancels in their (2.4) (2.6).
  In this formulation even the intercept is penalized.  If the lower limit of the
  simulated interval is desired one can specify \code{alpha = 0.05}.

} 
\references{
    Belloni, A. and  V. Chernozhukov. (2011) l1-penalized quantile regression 
    in high-dimensional sparse models. \emph{Annals of Statistics}, 39 82 - 130.
}
\examples{
n <- 200
p <- 10
x <- matrix(rnorm(n*p), n, p)
b <- c(1,1, rep(0, p-2))
y <- x \%*\% b + rnorm(n)
f <- rq(y ~ x, tau = 0.8, method = "lasso")
# See f$lambda to see the default lambda selection
}

\name{rq}
\alias{rq}
\title{
Quantile Regression 
}
\description{
Returns an object of class \code{"rq"} \code{"rqs"} 
or \code{"rq.process"} that represents a quantile regression fit. 
}
\usage{
rq(formula, tau=.5, data, subset, weights, na.action,
   method="br", model = TRUE, contrasts, \dots) 
}
\arguments{
  \item{formula}{
    a formula object, with the response on the left of a \code{~} operator, 
    and the terms, separated by \code{+} operators, on the right. 
  }
  \item{tau}{
    the quantile(s) to be estimated, this is generally a number strictly between 0 and 1, 
    but if specified strictly outside this range, it is presumed that the solutions 
    for all values of \code{tau} in (0,1) are desired.  In the former case an
    object of class \code{"rq"} is returned, in the latter,
    an object of class \code{"rq.process"} is returned.  As of version 3.50,
    tau can also be a vector of values between 0 and 1; in this case an
    object of class \code{"rqs"} is returned containing among other things
    a matrix of coefficient estimates at the specified quantiles.
  }
  \item{data}{
    a data.frame in which to interpret the variables 
    named in the formula, or in the subset and the weights argument. 
    If this is missing, then the variables in the formula should be on the 
    search list.  This may also be a single number to handle some special  
    cases -- see below for details.   
  }
  \item{subset}{
    an optional vector specifying a subset of observations to be
    used in the fitting process.}
  \item{weights}{
    vector of observation weights; if supplied, the algorithm fits
    to minimize the sum of the weights multiplied into the
    absolute residuals. The length of weights must be the same as
    the number of observations.  The weights must be nonnegative
    and it is strongly recommended that they be strictly positive,
    since zero weights are ambiguous. 
  }
  \item{na.action}{
    a function to filter missing data. 
    This is applied to the model.frame after any subset argument has been used. 
    The default (with \code{na.fail}) is to create an error if any missing values are  
    found.  A possible alternative is \code{na.omit}, which 
    deletes observations that contain one or more missing values. 
  }
  \item{model}{if TRUE then the model frame is returned.  This is
    essential if one wants to call summary subsequently.
  }
  \item{method}{
    the algorithmic method used to compute the fit.  There are several
    options:   
    \enumerate{
	\item \code{"br"} The default method is the modified  version of the
    Barrodale and Roberts algorithm for \eqn{l_1}{l1}-regression,
    used by \code{l1fit} in S, and is described in detail in 
    Koenker and d'Orey(1987, 1994),  default = \code{"br"}. 
    This is quite efficient for problems up to several thousand observations, 
    and may be used to compute the full quantile regression process.  It 
    also implements a scheme for computing confidence intervals for 
    the estimated parameters, based on inversion of a rank test described 
    in Koenker(1994).  

    \item \code{"fn"} For larger problems it is advantageous to use 
    the Frisch--Newton interior point method \code{"fn"}. 
    This is described in detail in Portnoy and Koenker(1997).

    \item \code{"pfn"} For even larger problems one can use the Frisch--Newton approach after 
    preprocessing \code{"pfn"}.  Also described in detail in Portnoy and Koenker(1997), 
    this method is primarily well-suited for large n, small p problems, that is when the 
    parametric dimension of the model is modest.  

    \item \code{"sfn"}  For large problems with  large
    parametric dimension it is often advantageous to use method \code{"sfn"}
    which also uses the Frisch-Newton algorithm, but exploits sparse algebra
    to compute iterates.  This is especially helpful when the model includes
    factor variables that, when expanded, generate design matrices that
    are very sparse.  At present options for inference, i.e. summary methods
    are somewhat limited when using the \code{"sfn"} method.  Only the option
    \code{se = "nid"} is currently available, but I hope to implement some
    bootstrap options in the near future.

    \item \code{"fnc"}  Another option enables the user to specify
    linear inequality constraints on the fitted coefficients; in this
    case one needs to specify the matrix \code{R} and the vector \code{r}
    representing the constraints in the form \eqn{Rb \geq r}.  See the
    examples below.  

    \item \code{"conquer"}  For very large problems especially those with
    large parametric dimension, this option provides a link to the \pkg{conquer}
    of He, Pan, Tan, and Zhou (2020).  Calls to \code{summary} when the fitted
    object is computed with this option invoke the multiplier bootstrap percentile
    method of the \pkg{conquer} package and can be considerably quicker than other
    options when the problem size is large.  Further options for this fitting
    method are described in the \pkg{conquer} package.  Note that this option
    employs a smoothing form of the usual QR objective function so solutions
    may be expected to differ somewhat from those produced with the other options.

    \item \code{"pfnb"} This option is intended for applications with large
    sample sizes and/or moderately fine tau grids.  It uses a form of preprocessing
    to accelerate the solution process.  The loop over taus occurs inside the Fortran
    call and there should be more efficient than other methods in large problems.

    \item \code{"qfnb"} This option is like the preceeding one except that it doesn't
    use the preprocessing option.

    \item \code{"ppro"} This option is an R prototype of the \code{pfnb} and is 
    offered for historical/interpretative purposes, but probably should be considered
    deprecated.
    
    \item \code{"lasso"} There are two penalized methods:  \code{"lasso"} 
    and \code{"scad"} that implement the lasso penalty and Fan and Li
    smoothly clipped absolute deviation penalty, respectively.  These
    methods should probably be regarded as experimental.
    }
  }
  \item{contrasts}{
    a list giving contrasts for some or all of the factors 
    default = \code{NULL} appearing in the model formula. 
    The elements of the list should have the same name as the variable 
    and should be either a contrast matrix (specifically, any full-rank 
    matrix with as many rows as there are levels in the factor), 
    or else a function to compute such a matrix given the number of levels. 
  }
  \item{...}{
    additional arguments for the fitting routines 
    (see \code{\link{rq.fit.br}} and \code{\link{rq.fit.fnb}}, etc.
    and the functions they call). 
  }
}
\value{
  See \code{\link{rq.object}} and \code{\link{rq.process.object}} for details. 
  Inferential matters are handled with \code{\link{summary}}.  There are
  extractor methods \code{logLik} and \code{AIC} that are potentially
  relevant for model selection.
}
\details{For further details see the vignette available from \R with
    \code{ vignette("rq",package="quantreg")} and/or the Koenker (2005).
    For estimation of nonlinear (in parameters) quantile regression models
    there is the function \code{nlrq} and for nonparametric additive 
    quantile regression there is the function \code{rqss}.
    Fitting of quantile regression models with censored data is handled by the
    \code{crq} function.} 
\examples{
data(stackloss)
rq(stack.loss ~ stack.x,.5)  #median (l1) regression  fit for the stackloss data. 
rq(stack.loss ~ stack.x,.25)  #the 1st quartile, 
        #note that 8 of the 21 points lie exactly on this plane in 4-space! 
rq(stack.loss ~ stack.x, tau=-1)   #this returns the full rq process
rq(rnorm(50) ~ 1, ci=FALSE)    #ordinary sample median --no rank inversion ci
rq(rnorm(50) ~ 1, weights=runif(50),ci=FALSE)  #weighted sample median 
#plot of engel data and some rq lines see KB(1982) for references to data
data(engel)
attach(engel)
plot(income,foodexp,xlab="Household Income",ylab="Food Expenditure",type = "n", cex=.5)
points(income,foodexp,cex=.5,col="blue")
taus <- c(.05,.1,.25,.75,.9,.95)
xx <- seq(min(income),max(income),100)
f <- coef(rq((foodexp)~(income),tau=taus))
yy <- cbind(1,xx)\%*\%f
for(i in 1:length(taus)){
        lines(xx,yy[,i],col = "gray")
        }
abline(lm(foodexp ~ income),col="red",lty = 2)
abline(rq(foodexp ~ income), col="blue")
legend(3000,500,c("mean (LSE) fit", "median (LAE) fit"),
	col = c("red","blue"),lty = c(2,1))
#Example of plotting of coefficients and their confidence bands
plot(summary(rq(foodexp~income,tau = 1:49/50,data=engel)))
#Example to illustrate inequality constrained fitting
n <- 100
p <- 5
X <- matrix(rnorm(n*p),n,p)
y <- .95*apply(X,1,sum)+rnorm(n)
#constrain slope coefficients to lie between zero and one
R <- cbind(0,rbind(diag(p),-diag(p)))
r <- c(rep(0,p),-rep(1,p))
rq(y~X,R=R,r=r,method="fnc")
}
\section{Method}{
The function computes an estimate on the tau-th conditional quantile
function of the response, given the covariates, as specified by the
formula argument.  Like \code{lm()}, the function presumes a linear
specification for the quantile regression model, i.e. that the formula
defines a model that is linear in parameters.  For non-linear (in parameters)
quantile regression see the package \code{nlrq()}.  
The function minimizes a weighted sum of absolute
residuals that can be formulated as a linear programming problem.  As
noted above, there are several different algorithms that can be chosen
depending on problem size and other characteristics.  For moderate sized
problems (\eqn{n \ll 5,000, p \ll 20}{n << 5,000, p << 20}) it is recommended 
that the default \code{"br"} method be used. There are several choices of methods for
computing confidence intervals and associated test statistics.  
See the documentation for \code{\link{summary.rq}} for further details
and options.  
}

\keyword{regression}
\references{
[1] Koenker, R. W. and Bassett, G. W. (1978). Regression quantiles, 
\emph{Econometrica}, \bold{46}, 33--50. 

[2] Koenker, R.W. and d'Orey (1987, 1994). Computing regression quantiles. 
\emph{Applied Statistics}, \bold{36}, 383--393, and \bold{43}, 410--414. 

[3] Gutenbrunner, C. Jureckova, J. (1991). 
Regression quantile and regression rank score process in the 
linear model and derived statistics, \emph{Annals of Statistics},
\bold{20}, 305--330.

[4] Xuming He and Xiaoou Pan and Kean Ming Tan and Wen-Xin Zhou, (2020)
conquer: Convolution-Type Smoothed Quantile Regression,
\url{https://CRAN.R-project.org/package=conquer}

[4] Koenker, R. W. (1994). Confidence Intervals for regression quantiles, in 
P. Mandl and M. Huskova (eds.), \emph{Asymptotic Statistics}, 349--359,  
Springer-Verlag, New York.   

[5] Koenker, R. and S. Portnoy (1997) The Gaussian Hare and the Laplacean 
Tortoise:  Computability of Squared-error vs Absolute Error Estimators, 
(with discussion).  \emph{Statistical Science,} \bold{12}, 279-300.

[6] Koenker, R. W. (2005). \emph{Quantile Regression},  Cambridge U. Press.

There is also recent information available at the URL:
\url{http://www.econ.uiuc.edu/~roger/}.
}
\seealso{
  \code{\link{FAQ}}, 
  \code{\link{summary.rq}}, 
  \code{\link{nlrq}},
  \code{\link{rq.fit}},
  \code{\link{rq.wfit}},
  \code{\link{rqss}},
  \code{\link{rq.object}},
  \code{\link{rq.process.object}}
}

\name{boot.rq.pxy}
\alias{boot.rq.pxy}
\title{
    Preprocessing bootstrap method
}
\description{
    Bootstrap method exploiting preprocessing strategy to reduce 
    computation time for large problem.
}
\usage{
boot.rq.pxy(x, y, s, tau = 0.5, coef, method = "fn", Mm.factor = 3)
}
\arguments{
  \item{x}{
      Design matrix
}
  \item{y}{
      response vector
}
  \item{s}{
      matrix of multinomial draws for xy bootstrap
}
  \item{tau}{
      quantile of interest
}
  \item{coef}{
      point estimate of fitted object
}
  \item{method}{
      fitting method for bootstrap
}
  \item{Mm.factor}{
      constant to determine initial sample size
}
}
\details{
    See references for further details.
}
\value{
    Returns matrix of bootstrap estimates.
}
\references{
    Chernozhukov, V.  I. Fernandez-Val and B. Melly,
	Fast Algorithms for the Quantile Regression Process, 2019,
	arXiv, 1909.05782,

     Portnoy, S.  and R. Koenker, The Gaussian Hare and the Laplacian
	Tortoise, Statistical Science, (1997) 279-300
}
\author{
    Blaise Melly and Roger Koenker
}
\seealso{
\code{\link{rq.fit.ppro}}
}
\keyword{bootstrap} 

\name{boot.crq}
\alias{boot.crq}
\title{ Bootstrapping Censored Quantile Regression}
\description{
Functions used to estimated standard errors, confidence
intervals and tests of hypotheses for censored quantile regression models
using the Portnoy and Peng-Huang methods.  }
\usage{
boot.crq(x, y, c, taus, method, ctype = "right", R = 100, mboot, bmethod = "jack", ...)
}
\arguments{
  \item{x}{ The regression design matrix}
  \item{y}{ The regression response vector}
  \item{c}{ The censoring indicator}
  \item{taus}{ The quantiles of interest}
  \item{method}{ The fitting method: either "P" for Portnoy or "PH" for Peng and Huang.}
  \item{ctype}{ Either "right" or "left"}
  \item{R}{ The number of bootstrap replications}
  \item{bmethod}{ The bootstrap method to be employed.  There are (as yet) three
	options:  method  = "jack" uses the delete-d jackknife method
	described by Portnoy (2013), method = "xy-pair" uses the xy-pair method, 
	that is the usual multinomial resampling of xy-pairs, while  method
	= "Bose" uses the Bose and Chatterjee (2003) weighted resampling
	method with exponential weights.  The "jack" method is now the default.}
  \item{mboot}{ optional argument for the bootstrap method:  for bmethod = "jack"
	 it specifies the number, d, of the delete-d jackknife, for 
	 method = "xy-pair" it specifies the size of the bootstrap samples,
	 that permits subsampling (m out of n) bootstrap.  By default in the
	 former case it is set to 2 [sqrt(n)], for the latter the default is
	 n.  Obviously mboot should be substantially larger than the column dimension 
	 of x, and should be less than the sample size in both cases.}
  \item{...}{ Optional further arguments to control bootstrapping}
}
\details{
There are several refinements that are still unimplemented.  Percentile
methods should be incorporated, and extensions of the methods to be used 
in anova.rq should be made.  Note that bootstrapping for the Powell 
method "Powell" is done via \code{\link{boot.rq}}.  For problems with
\code{n > 3000} a message is printed indicated progress in the resampling.
}
\value{
  A matrix of dimension R by p is returned with the R resampled
  estimates of the vector of quantile regression parameters. When
  mofn < n for the "xy" method this matrix has been deflated by
  the factor sqrt(m/n)
}
\references{ 
 Bose, A. and S. Chatterjee, (2003) Generalized bootstrap for estimators
        of minimizers of convex functions, \emph{J. Stat. Planning and Inf}, 117,
        225-239.
 Portnoy, S. (2013) The Jackknife's Edge:  Inference for Censored Quantile Regression,
	\emph{CSDA}, forthcoming.

}

\author{ Roger Koenker }
\seealso{  \code{\link{summary.crq}}}
\keyword{ regression}

\name{qss}
\alias{qss}
\alias{qss1}
\alias{qts1}
\alias{qss2}
\alias{triogram.fidelity}
\alias{triogram.penalty}
\title{Additive Nonparametric Terms for rqss Fitting}
\description{
In the formula specification of \code{rqss} nonparametric terms
are specified with \code{qss}.  Both univariate and bivariate
specifications are possible, and qualitative constraints may also be specified
for the qss terms.
}
\usage{
qss(x, constraint = "N", lambda = 1, ndum = 0, dummies = NULL, 
    Dorder = 1, w = rep(1, length(x)))
}
\arguments{
  \item{x}{The covariate determining the nonparametric component, if
	x is a matrix with two columns then  the qss
	function will construct a penalized triogram term.}
  \item{lambda}{The smoothing parameter governing the tradeoff between
	fidelity and the penalty component for this term.  Larger lambdas
	produce smoother fits. In future
	versions there should be an automatic mechanism for default
	choice of the lambdas.  For now, this is the responsibility
	of the user.}
  \item{constraint}{Optional specification of qualitative constraints
	on the fitted univariate qss functions, take the values: "N","I","D","V","C"
	"VI","VD","CI","CD" for none, increasing, decreasing, convex,
	concave, convex and increasing, etc.  And for bivariate qss
	components can take the values "N","V","C" for none, convex, and concave.
	Note that confidence bands for constrained fits of this sort, while
	available from \code{plot.rqss} as of yet lack a formal justification.}
  \item{ndum}{number of dummy vertices: this is only relevant for qss2
	terms.  In addition to vertices at the observed (x,y) points
	ndum dummy vertices are generated -- distributed uniformly over
	the rectangle given by the Cartesian product of the ranges of
	x and y -- observations that fall in the convex hull of the
	observations are retained. So the actual number of dummy
	vertices used is smaller than ndum.  The values of these
	vertices are returned in the list dummies, so that they can
	be reused.}
  \item{Dorder}{Order of the total variation penalty, the default of 1
      implies a penalty on the first derivative of the fitted function,
      a value of 0 implies total variation of the fitted function
      itself will be penalized.  Note that only monotonicity constraints,
      "I" and "D" are allowed when \code{Dorder = 0}, and result in estimates
      that are equivalent to a form of isotonic regression when lambda is
      sufficiently near zero.  Results in this case from the package \pkg{isotone}
      may differ slightly when plotted due to multiple solutions so it is prudent
      to evaluate the objective function for both solutions.}
  \item{dummies}{list of dummy vertices as generated, for example by
	triogram.fidelity when ndum > 0.  Should be a list with x
	and y components.  These points should lie inside the convex
	hull of the real xy points, but no explicit checking of this
	assertion is currently done.}
  \item{w}{ weights not yet unimplemented }
}
\details{
The various pieces returned are stored in sparse matrix.csr form.
See \code{\link{rqss}} for details on how they are assembled.  To preserve the
sparsity of the design matrix the first column of each qss term is dropped.
This differs from the usual convention that would have forced qss terms
to have mean zero.  This convention has implications for prediction that need to be
recognized.  The penalty components for qss terms are based on total
variation penalization of the first derivative (and gradient, for bivariate x)
as described in the references appearing in the help for \code{rqss}.
When Dorder = 0, fitting is like the taut string methods of Davies (2014), except
for the fact that fidelity is quantilesque rather than quadratic,
and that no provision is made for automatic selection of the smoothing 
parameter.

For the bivariate case, package \pkg{tripack} (and for plotting also
\pkg{akima}) are required (automatically, by the \R code).
}
\value{
  \item{F}{Fidelity component of the design matrix}
  \item{dummies}{List of dummy vertices}
  \item{A}{Penalty component of the design matrix}
  \item{R}{Constraint component of the design matrix}
  \item{r}{Constraint component of the rhs}
}
\author{Roger Koenker}

\references{
    Davies, Laurie (2014) \emph{Data Analysis and Approximate Models}, CRC Press.
}
\seealso{ \code{\link{rqss}}}
\keyword{smooth}
\keyword{robust}

\name{Peirce}
\alias{Peirce}
\docType{data}
\title{C.S. Peirce's Auditory Response Data}
\description{
Data from sequence experiments conducted by C.S. Pierce in 1872 to
determine the distribution of response times to an auditory stimulus.
}
\usage{data(Peirce)}
\format{
A \code{link{list}} of 24 objects each representing one day of the
experiment.  Each element of the list consists of three components:
the date the measurements were made, an \code{x} component
recording the response time in milliseconds, and an associated  \code{y}
component recording a count of the number of times that the response
was recorded to be equal to be equal to the  corresponding \code{x} entry.
There  are roughly 500 observations (counts) on each of the 24 days.
}
\details{
A detailed description of the experiment can be found in Peirce (1873).  A
young man of about 18 with no prior experience was  employed to respond to
a signal  ``consisting of a sharp sound like a rap, the answer being made
upon a telegraph-operator's key nicely adjusted.''  The response times,
made with the aid of a Hipp cronoscope were recorded to the nearest 
millisecond.  The data was analyzed by Peirce who concluded that after
the first day, when the the observer was entirely inexperienced, the
curves representing the densities of the response times ``differed very
little from that derived from the theory of least squares,'' i.e. from
the Gaussian density.

The data was subsequently analysed by Samama, in a diploma thesis supervised
by Maurice Frechet, who reported briefly the findings in Frechet (1924),
and by Wilson and Hilferty (1929).  In both instances the reanalysis showed
that Laplace's first law of error, the double exponential distribution, was
a better representation for the data than was the Gaussian law.  Koenker (2009)
constains further discussion and an attempt to reproduce the Wilson and
Hilferty analysis.

The data is available in two formats:  The first in a "raw" form as 24 text
files as scanned from the reprinted Peirce source, the second as an R
dataset \code{Peirce.rda} containing the list.  Only the latter
is provided here, for the raw data and how to read see the more complete
archive at:  \url{ http://www.econ.uiuc.edu/~roger/research/frechet/frechet.html}
See the examples section below for some details on 
provisional  attempt to reproduce part of the Wilson and Hilferty
analysis.  An open question regarding the dataset is:  How did Wilson
and Hilferty compute standard deviations for the median as they appear
in their table?  The standard textbook suggestion of Yule (1917) yields
far too small a bandwidth.  The methods employed in the example section
below, which rely on relatively recent proposals, are somewhat closer, 
but still deviate somewhat from the results reported by Wilson and Hilferty.
}
\source{
Peirce, C.~S.  (1873): ``On the Theory of Errors of Observation,''
  \emph{Report of the Superintendent of the U.S. Coast Survey}, pp. 200--224,
  Reprinted in \emph{The New Elements of Mathematics}, (1976) collected papers
  of C.S. Peirce, ed. by C. Eisele, Humanities Press: Atlantic Highlands, N.J.,
  vol. 3, part 1, 639--676.
}
\references{
Fr\'echet, M.  (1924): ``Sur la loi des erreurs d'observation,''
  \emph{Matematichiskii Sbornik}, 32, 5--8.
Koenker, R. (2009): ``The Median is the Message:  Wilson and Hilferty's 
  Reanalysis of C.S. Peirce's Experiments on the Law of Errors,'' 
  \emph{American Statistician}, 63, 20-25.
Wilson, E.~B.,  and M.~M. Hilferty  (1929): ``Note on C.S.
  Peirces Experimental Discussion of the Law of Errors,'' \emph{Proceedings
  of the National Academy of Sciences of the U.S.A.}, 15, 120--125.
Yule, G.~U.  (1917): \emph{An Introduction to the Theory of
  Statistics}. Charles Griffen: London, 4 edn.
}
\examples{

# Make table like Wilson and Hilferty

data("Peirce")
set.seed(10) #Dither the counts
tab <- matrix(0,24,11)
for(i in 1:24){
	y <- rep(Peirce[[i]]$x, Peirce[[i]]$y) + runif(sum(Peirce[[i]]$y), -.5, .5)
	f1 <- summary(rq(y~1),se="iid")$coef[1:2]
	n <- length(y)
	f0 <- 1/(2 * sum(abs(y-f1[1])/n)) #Laplace proposal
	f0 <- (1/(2 * f0))/ sqrt(n)
	f2 <- summary(lm(y~1))$coef[1:2]
	outm <- sum(y < (f1[1] - 3.1 * sqrt(n) * f2[2]))
	outp <- sum(y > (f1[1] + 3.1 * sqrt(n) * f2[2]))
	outt <- outm + outp
	inm <- y > (f1[1] - 0.25 * sqrt(n) * f2[2])
	inp <- y < (f1[1] + 0.25 * sqrt(n) * f2[2])
	int <- sum(inm * inp)
	Eint <- round(n * (pnorm(.25) - pnorm(-.25)))
	excess <- round(100*(int - Eint)/Eint)
	tab[i,] <- c(f1, f0, f2, outm, outp, outt,int,Eint,excess)
	cnames <- c("med","sdmed1","sdmed0","mean","sdmean","below","above","outliers",
		"inliers","Einliers","ExcessIns")
	dimnames(tab) <- list(paste("Day",1:24),cnames)
	}
}
\keyword{datasets}

\name{plot.rqs}
\alias{plot.rqs}

\title{Visualizing sequences of quantile regressions}

\description{A sequence of coefficient estimates for quantile
  regressions with varying \code{tau} parameters is visualized.}

\usage{
\method{plot}{rqs}(x, parm = NULL, ols = TRUE,
  mfrow = NULL, mar = NULL, ylim = NULL, main = NULL, col = 1:2, lty = 1:2,
  cex = 0.5, pch = 20, type = "b", xlab = "", ylab = "", \dots)
}

\arguments{
  \item{x}{an object of class \code{"rqs"} as produce by \code{\link{rq}}
    (with a vector of \code{tau} values).}
  \item{parm}{a specification of which parameters are to be plotted,
    either a vector of numbers or a vector of names.  By default, all
    parameters are considered.}
  \item{ols}{logical. Should a line for the OLS coefficient (as estimated
    by \code{\link[stats]{lm}}) be added?}
  \item{mfrow, mar, ylim, main}{graphical parameters. Suitable defaults are chosen
    based on the coefficients to be visualized.}
  \item{col, lty}{graphical parameters. For each parameter, the first
    element corresponds to the \code{rq} coefficients and the second to
    the \code{lm} coefficients.}
  \item{cex, pch, type, xlab, ylab, \dots}{further graphical parameters
    passed.}
}

\details{The \code{plot} method for \code{"rqs"} objects visualizes the
  coefficients only, confidence bands can be added by using the \code{plot}
  method for the associated \code{"summary.rqs"} object.}

\seealso{\code{\link{rq}}, \code{\link{plot.summary.rqs}}}

\value{A matrix with all coefficients visualized is returned invisibly.}

\examples{
## fit Engel models (in levels) for tau = 0.1, ..., 0.9
data("engel")
fm <- rq(foodexp ~ income, data = engel, tau = 1:9/10)

## visualizations
plot(fm)
plot(fm, parm = 2, mar = c(5.1, 4.1, 2.1, 2.1), main = "", xlab = "tau", 
  ylab = "income coefficient", cex = 1, pch = 19)
}

\keyword{hplot}

\name{residuals.nlrq}
\alias{residuals.nlrq}
\title{ Return residuals of an nlrq object }
\description{
Set algorithmic parameters for nlrq (nonlinear quantile regression function)
}
\usage{
\method{residuals}{nlrq}(object, type = c("response", "rho"), ...) 
}
\arguments{
  \item{object}{an `nlrq' object as returned by function `nlrq'}
  \item{type}{the type of residuals to return: "response" is the distance
  between observed and predicted values; "rho" is the weighted distance used
  to calculate the objective function in the minimisation algorithm as 
  tau * pmax(resid, 0) + (1 - tau) * pmin(resid, 0), where resid are the
  simple residuals as above (with type="response").
  }
  \item{...}{further arguments passed to or from other methods.}
}
\seealso{  \code{\link{nlrq}} }

\keyword{models}  
\keyword{regression}
\keyword{nonlinear}

\name{summary.rq}
\alias{summary.rq}
\alias{summary.rqs}
\alias{summary.rcrqs}
\title{
Summary methods for Quantile Regression
}
\description{
Returns a summary list for a quantile regression fit.  A null value
will be returned if printing is invoked.
}
\usage{
\method{summary}{rq}(object, se = NULL, covariance=FALSE, hs = TRUE,  U = NULL, gamma = 0.7, ...)
\method{summary}{rqs}(object, ...)
}
\arguments{
  \item{object}{
    This is an object of class \code{"rq"} or \code{"rqs"} produced by 
    a call to \code{rq()}, depending on whether one or more taus are
    specified.
  }
  \item{se}{
    specifies the method used to compute standard standard errors.  There
    are currently seven available methods:  
    \enumerate{
      \item \code{"rank"} which produces confidence intervals for the
      estimated parameters by inverting a rank test as described in
      Koenker (1994).  This method involves solving a parametric linear
      programming problem, and for large sample sizes can be extremely
      slow, so by default it is only used when the sample size is less
      than 1000, see below.  The default option assumes that the errors are
      iid, while the option iid = FALSE implements a proposal of Koenker
      Machado (1999).  See the documentation for \code{rq.fit.br} for additional arguments.
      \item \code{"iid"} which presumes that the errors are iid and computes
      an estimate of the asymptotic covariance matrix as in KB(1978).
      
      \item \code{"nid"} which presumes local (in \code{tau})
      linearity (in \code{x}) of the
      the conditional quantile functions and computes a Huber
      sandwich estimate using a local estimate of the sparsity.
      If the initial fitting was done with method "sfn" then use
      of \code{se = "nid"} is recommended.  However, if the cluster
      option is also desired then \code{se = "boot"} can be used
      and bootstrapping will also employ the "sfn" method.
      
      \item \code{"ker"} which uses a kernel estimate of the sandwich
      as proposed by Powell(1991).

      \item \code{"boot"} which implements one of several possible bootstrapping
      alternatives for estimating standard errors including a variate of the wild
      bootstrap for clustered response.  See \code{\link{boot.rq}} for
      further details.  

      \item \code{"BLB"} which implements the bag of little bootstraps method
      proposed in Kleiner, et al (2014).  The sample size of the little bootstraps
      is controlled by the parameter \code{gamma}, see below.  At present only
      \code{bsmethod = "xy"} is sanction, and even that is experimental.  This
      option is intended for applications with very large n where other flavors
      of the bootstrap can be slow.

      \item \code{"conquer"} which is invoked automatically if the fitted 
      object was created with \code{method = "conquer"}, and returns the
      multiplier bootstrap percentile confidence intervals described in
      He et al (2020).

      \item \code{"extreme"} which uses the subsampling method of Chernozhukov
      Fernandez-Val, and Kaji (2018) designed for inference on extreme quantiles.
    }
    If \code{se = NULL} (the default)  and \code{covariance = FALSE}, and
    the sample size is less than 1001, then the "rank" method is used, 
    otherwise the "nid" method is used.
  }
  \item{covariance}{
    logical flag to indicate whether the full covariance matrix of the 
    estimated parameters should be returned. 
  }
  \item{hs}{
    Use Hall Sheather bandwidth for sparsity estimation
    If false revert to Bofinger bandwidth.
   }
   \item{U}{Resampling indices or gradient evaluations used for bootstrap,
       see \code{\link{boot.rq}}.}
   \item{gamma}{parameter controlling the effective sample size of the'bag
       of little bootstrap samples that will be \code{b = n^gamma} where
       \code{n} is the sample size of the original model.}
  \item{...}{
    Optional arguments to summary, e.g. bsmethod to use bootstrapping.
    see \code{\link{boot.rq}}.  When using the "rank" method for confidence
    intervals, which is the default method for sample sizes less than 1000,
    the type I error probability of the intervals can be controlled with the
    alpha parameter passed via "...",  thereby controlling the width of the
    intervals plotted by \code{plot.summary.rqs}. Similarly, the arguments
    alpha, mofn and kex can be passed when invoking the \code{"extreme"} option
    for  "se" to control the percentile interval reported, given by estimated
    quantiles [alpha/2, 1 - alpha/2]; \code{kex} is a tuning parameter for the
    extreme value confidence interval construction. The size of the bootstrap
    subsamples for the "extreme" option can also be controlled by passing
    the argument \code{mofm} via "...".  Default values for kex, mofn and
    alpha are 20, \code{floor(n/5)} and 0.1, respectively.
   }
}
\value{
  a list is returned with the following components, when \code{object}
  is of class \code{"rqs"} then there is a list of such lists.

\item{coefficients}{
  a p by 4 matrix consisting of the coefficients, their estimated standard
  errors, their t-statistics, and their associated p-values, in the case of
  most "se" methods.  For methods "rank" and "extreme" potentially asymetric
  confidence intervals are return in lieu of standard errors and p-values.
}
\item{cov}{
  the estimated covariance matrix for the coefficients in the model,
  provided that \code{cov=TRUE} in the called sequence.  This option
  is not available when se = "rank".
}
\item{Hinv}{
  inverse of the estimated Hessian matrix returned if \code{cov=TRUE} and
  \code{se \%in\% c("nid","ker") }, note that for \code{se = "boot"} there
  is no way to split the estimated covariance matrix into its sandwich
  constituent parts.
}
\item{J}{
  Unscaled Outer product of gradient matrix returned if \code{cov=TRUE} and \code{se
    != "iid"}. The Huber sandwich is \code{cov = tau (1-tau) Hinv \%*\% J \%*\% Hinv}.
    as for the \code{Hinv} component, there is no \code{J} component when
    \code{se == "boot"}.  (Note that to make the Huber sandwich you need to add the 
    tau (1-tau) mayonnaise yourself.)
    }
\item{B}{Matrix of bootstrap realizations.}
\item{U}{Matrix of bootstrap randomization draws.}
}

\details{
When the default summary method is used, it tries to estimate a sandwich
form of the asymptotic covariance matrix and this involves estimating
the conditional density at each of the sample observations, negative
estimates can occur if there is crossing of the neighboring quantile
surfaces used to compute the difference quotient estimate.  
A warning message is issued when such negative estimates exist indicating
the number of occurrences -- if this number constitutes a large proportion
of the sample size, then it would be prudent to consider an alternative 
inference method like the bootstrap.
If the number of these is large relative to the sample size it is sometimes
an indication that some additional nonlinearity in the covariates
would be helpful, for instance quadratic effects.
Note that the default \code{se} method is rank, unless the sample size exceeds
1001, in which case the \code{rank} method is used.
There are several options for alternative resampling methods.  When
\code{summary.rqs} is invoked, that is when \code{summary} is called
for a \code{rqs} object consisting of several \code{taus}, the \code{B}
components of the returned object can be used to construct a joint covariance
matrix for the full object.}
 

\references{
  Chernozhukov, Victor, Ivan Fernandez-Val, and Tetsuya Kaji, (2018)
    Extremal Quantile Regression, in Handbook of Quantile Regression,
    Eds. Roger Koenker, Victor Chernozhukov, Xuming He, Limin Peng,
    CRC Press.

  Koenker, R. (2004) \emph{Quantile Regression}.

  Bilias, Y. Chen, S. and Z. Ying, Simple resampling methods for censored
	quantile regression, \emph{J. of Econometrics}, 99, 373-386.

  Kleiner, A., Talwalkar, A., Sarkar, P. and Jordan, M.I. (2014) A Scalable
	bootstrap for massive data, \emph{JRSS(B)}, 76, 795-816.

  Powell, J. (1991) Estimation of Monotonic Regression Models under
	Quantile Restrictions, in Nonparametric and Semiparametric Methods
	in Econometrics, W. Barnett, J. Powell, and G Tauchen (eds.),
	Cambridge U. Press.

}
\seealso{
  \code{\link{rq}}
  \code{\link{bandwidth.rq}}
}
\examples{
data(stackloss)
y <- stack.loss
x <- stack.x
summary(rq(y ~ x, method="fn")) # Compute se's for fit using "nid" method.
summary(rq(y ~ x, ci=FALSE),se="ker")
# default "br" alg, and compute kernel method se's
}
\keyword{regression}

\name{KhmaladzeTest}
\alias{KhmaladzeTest}
\title{ Tests of Location and Location Scale Shift Hypotheses for Linear Models}
\description{Tests of the hypothesis that a linear model specification
is of the location shift or location-scale shift form.  The tests are based 
on the Doob-Meyer Martingale transformation approach proposed by Khmaladze(1981)
for general goodness of fit problems as adapted to quantile regression by 
Koenker and Xiao (2002).}

\usage{
KhmaladzeTest(formula, data = NULL, taus = 1:99/100, nullH = "location" ,  
	trim = c(0.05, 0.95), h = 1, ...) 
}
\arguments{
\item{formula}{a formula specifying the model to fit by \code{\link{rqProcess}}}
\item{data}{a data frame within which to interpret the formula}
\item{taus}{An equally spaced grid of points on which to evaluate the 
quantile regression process, if any taus fall outside (0,1) then the full
process is computed.}
\item{nullH}{a character vector indicating whether the "location" shift hypothesis
(default) or the "location-scale" shift hypothesis  should be tested.  }
\item{trim}{ a vector indicating the lower and upper bound of the quantiles to
included in the computation of the test statistics (only, not
estimates).  }
\item{h}{an initial bandwidth for the call to \code{\link{akj}}.} 
\item{...}{other arguments to be passed to \code{\link{summary.rq}.}}
}
\value{
an object of class KhmaladzeTest  is returned containing:

\item{nullH}{ The form of the null hypothesis.}

\item{Tn}{
Joint test statistic of the hypothesis that all the slope
parameters of the model satisfy the hypothesis.
}
\item{THn}{
Vector of test statistics testing whether individual slope
parameters satisfy the null hypothesis.
}
}

\examples{
data(barro)
T = KhmaladzeTest( y.net ~ lgdp2 + fse2 + gedy2 + Iy2 + gcony2, 
		data = barro, taus = seq(.05,.95,by = .01))
plot(T)
}
\keyword{htest}
\references{
Khmaladze, E. (1981) ``Martingale Approach in the Theory of
Goodness-of-fit Tests,'' \emph{Theory of Prob. and its Apps}, 26,
240--257.

Koenker, Roger and Zhijie Xiao (2002), ``Inference on the Quantile
Regression Process'',  \emph{Econometrica},  81, 1583--1612.
\url{http://www.econ.uiuc.edu/~roger/research/inference/inference.html}
}

\name{plot.KhmaladzeTest}
\alias{plot.KhmaladzeTest}
\title{ Plot a KhmaladzeTest object}
\description{
Plot an object generated by KhmaladzeTest
}
\usage{
\method{plot}{KhmaladzeTest}(x, ...)
}
\arguments{
\item{x}{
  Object returned from KhmaladzeTest representing the fit of the model.
 }
\item{...}{
  Optional arguments. 
 }
}
\seealso{  \code{\link{KhmaladzeTest}}}

\keyword{ regression}

\name{nlrq.control}
\alias{nlrq.control}
\title{ Set control parameters for nlrq }
\description{
Set algorithmic parameters for nlrq (nonlinear quantile regression function)
}
\usage{
nlrq.control(maxiter=100, k=2, InitialStepSize = 1, big=1e+20, eps=1e-07, beta=0.97)
}
\arguments{
  \item{maxiter}{maximum number of allowed iterations}
  \item{k}{the number of iterations of the Meketon algorithm to be calculated 
	in each step, usually 2 is reasonable, occasionally it may be helpful 
	to set k=1 }
  \item{InitialStepSize}{ Starting value in \code{optim} to determine the step
	length of iterations.  The default value of 1 is sometimes too optimistic.
	In such cases, the value 0 forces optim to just barely stick its toe in
	the water.}
  \item{big}{ a large scalar}
  \item{eps}{ tolerance for convergence of the algorithm }
  \item{beta}{ a shrinkage parameter which controls the recentering process 
	in the interior point algorithm. }
}
\seealso{  \code{\link{nlrq}} }

\keyword{ environment}

\name{lm.fit.recursive}
\alias{lm.fit.recursive}
\title{ Recursive Least Squares }
\description{
This function fits a linear model by recursive least squares.  It is
a utility routine for the \code{\link{KhmaladzeTest}} function of the quantile regression
package.
}
\usage{
lm.fit.recursive(X, y, int=TRUE)
}
\arguments{
  \item{X}{ Design Matrix }
  \item{y}{ Response Variable}
  \item{int}{ if TRUE then append intercept to X}
}
\value{
return p by n matrix of fitted parameters, where p. The
ith column gives the solution up to "time" i. 
}
\references{ A. Harvey, (1993) Time Series Models, MIT }
\author{ R. Koenker }

\keyword{methods}

\name{crq}
\alias{crq}
\alias{crq.fit.por}
\alias{crq.fit.por2}
\alias{crq.fit.pow}
\alias{crq.fit.pen}
\alias{print.crq}
\alias{print.crq}
\alias{coef.crq}
\alias{predict.crq}
\alias{predict.crqs}
\alias{Curv}
\title{Functions to fit censored quantile regression models} 
\description{Fits a conditional quantile regression model for censored data. There
are three distinct methods:  the first is the fixed censoring method
of Powell (1986) as implemented by Fitzenberger (1996), the second is the random
censoring method of Portnoy (2003).  The third method is based on Peng and Huang (2008).} 
\usage{
crq(formula, taus, data, subset, weights, na.action, 
	method = c("Powell", "Portnoy", "Portnoy2", "PengHuang"), contrasts = NULL, ...)
crq.fit.pow(x, y, yc, tau=0.5, weights=NULL, start, left=TRUE, maxit = 500)
crq.fit.pen(x, y, cen, weights=NULL, grid, ctype = "right") 
crq.fit.por(x, y, cen, weights=NULL, grid, ctype = "right") 
crq.fit.por2(x, y, cen, weights=NULL, grid, ctype = "right") 
Curv(y, yc, ctype=c("left","right"))
\method{print}{crq}(x, ...)
\method{print}{crq}(x, ...)
\method{predict}{crq}(object, newdata,  ...)
\method{predict}{crqs}(object, newdata, type = NULL, ...)
\method{coef}{crq}(object,taus = 1:4/5,...)
}
\arguments{
  \item{formula}{A formula object, with the response on the left of the `~'
        operator, and the terms on the right.  The response must be a
        \code{Surv} object as returned by either the \code{Curv} or \code{Surv} 
	function. For the Powell method, the Surv object should
	be created by \code{Curv} and have arguments (event time, censoring time,type), 
	where "type" can take values either "left" or "right". 
	The default (for historical reasons) for type in this case is "left".
	For the Portnoy and Peng and Huang  methods the \code{Surv}  should be created 
	with the usual \code{Surv} function and have (event time, censoring indicator).}  
  \item{y}{The event time.}
  \item{newdata}{An optional data frame in which to look for variables with which 
	to predict. If omitted, the fitted values are used.}
  \item{grid}{A vector of taus on which the quantile process should be evaluated. This
	should be monotonic, and take values in (0,1).  For the "Portnoy"  method,
	grid = "pivot" computes the full solution for all distinct taus.  The "Portnoy"
	method also enforces an equally spaced grid, see the code for details.}
  \item{x}{An object of class \code{crq} or \code{crq}.}
  \item{object}{An object of class \code{crq} or \code{crq}.}
  \item{yc}{The censoring times for the "Powell" method.}
  \item{ctype}{Censoring type: for the "Powell" method, used in \code{Curv}, by 
	default "left".  If you don't like "left", maybe you will like "right".
	Note that for fixed censoring assumed in the "Powell" method, censoring
	times \code{yc} must be provided for all observations and the event
	times \code{y} must satisfy the (respective) inequality constraints.
	For the Portnoy and Peng-Huang methods ctype is determined by the
	specification of the response as specified in \code{Surv}.
        }
  \item{type}{specifies either "left" or "right" as the form of censoring 
	 in the \code{Surv} function for the "Portnoy" and "PengHuang"  methods.}
  \item{cen}{The censoring indicator for the "Portnoy" and "PengHuang"  methods.}
  \item{maxit}{Maximum number of iterations allowed for the "Powell" methods.}
  \item{start}{The starting value for the coefs for the "Powell" method.  Because
	the Fitzenberger algorithm stops when it achieves a local minimum
	of the Powell objective function, the starting value acts as an
	a priori "preferred point".  This is advantageous in some instances
	since the global Powell solution can be quite extreme. By default the
	starting value is the "naive rq" solution that  treats all the censored
	observations as uncensored.  If \code{start} is equal to "global"
	then an attempt is made to compute to global optimum of the Powell
	objective.  This entails an exhaustive evaluation of all n choose p
	distinct basic solution so is rather impractical for moderately large
	problems. Otherwise, the starting value can specify a set of p indices
	from 1:n defining an initial basic solution, or it may specify a p-vector
	of initial regression coefficients.  In the latter case the initial basic
	solution is the one closest to the specified parameter vector.}
  \item{left}{A logical indicator for left censoring for the "Powell" method.}
  \item{taus}{The quantile(s) at which the model is to be estimated.}
  \item{tau}{The quantile at which the model is to be estimated.}
  \item{data}{A data.frame in which to interpret the variables named in the
          `formula',  in the `subset', and the `weights' argument.}
  \item{subset}{an optional vector specifying a subset of observations to be
          used in the fitting process.}
  \item{weights}{vector of observation weights; if supplied, the algorithm
          fits to minimize the sum of the weights multiplied into the
          absolute residuals. The length of weights vector must be the same as
          the number of observations.  The weights must be nonnegative
          and it is strongly recommended that they be strictly
          positive, since zero weights are ambiguous.} 
  \item{na.action}{a function to filter missing data.  This is applied to the
          model.frame after any subset argument has been used.  The
          default (with 'na.fail') is to create an error if any missing
          values are   found.  A possible alternative is 'na.omit',
          which  deletes observations that contain one or more missing
          values. } 
  \item{method}{The method used for fitting.  There are currently
	two options: method "Powell" computes the Powell estimator using
	the algorithm of Fitzenberger (1996), method "Portnoy" computes the
	Portnoy (2003) estimator.  The  method is "PengHuang" uses the method
	of Peng and Huang (2007), in this case the variable "grid"
	can be passed to specify the vector of quantiles at which the solution
	is desired.}
  \item{contrasts}{a list giving contrasts for some or all of the factors 
          default = 'NULL' appearing in the model formula.  The
          elements of the list should have the same name as the
          variable  and should be either a contrast matrix
          (specifically, any full-rank  matrix with as many rows as
          there are levels in the factor),  or else a function to
          compute such a matrix given the number of levels.} 
  \item{...}{additional arguments for the fitting routine, for method "Powell"
	it may be useful to pass starting values of the regression parameter
	via the argument "start", while for methods "Portnoy" or "PengHuang"
	one may wish to specify an alternative to the default grid for evaluating 
	the fit.}
}
\details{The Fitzenberger algorithm uses a variant of the Barrodale and Roberts
	simplex method.  Exploiting the fact that the solution must be characterized
	by an exact fit to p points when there are p parameters to be estimated,
	at any trial basic solution it computes the directional derivatives in the 
	2p distinct directions
	and choses the direction that (locally) gives steepest descent.  It then
	performs a one-dimensional line search to choose the new basic observation
	and continues until it reaches a local mimumum.  By default it starts at
	the naive \code{rq} solution ignoring the censoring;  this has the (slight)
	advantage that the estimator is consequently equivariant to canonical
	transformations of the data.  Since the objective function is no longer convex
	there can be no guarantee that this  produces a global minimum estimate.
	In small problems exhaustive search over solutions defined by p-element
	subsets of the n observations can be used, but this quickly becomes
	impractical for large p and n.   This global version of the Powell
	estimator can be invoked by specifying \code{start = "global"}. Users
	interested in this option would be well advised to compute \code{choose(n,p)}
	for their problems before trying it.  The method operates by pivoting
	through this many distinct solutions and choosing the one that gives the
	minimal Powell objective.  The algorithm used for the Portnoy 
	method is described in considerable detail in Portnoy (2003). 
	There is a somewhat simplified version of the Portnoy method that is
	written in R and iterates over a discrete grid.  This version should
	be considered somewhat experimental at this stage, but it is known to
	avoid some difficulties with the more complicated fortran version of
	the algorithm that can occur in degenerate problems.
	Both the Portnoy and Peng-Huang estimators may be unable to compute
	estimates of the conditional quantile parameters in the upper tail of
	distribution.  Like the Kaplan-Meier estimator, when censoring is heavy
	in the upper tail the estimated distribution is defective  and quantiles
	are only estimable on a sub-interval of (0,1).
	The Peng and Huang estimator can be
	viewed as a generalization of the Nelson Aalen estimator of the cumulative
	hazard function,  and can be formulated as a variant of the conventional
	quantile regression dual problem.  See Koenker (2008) for further details.
	This paper is available from the package with \code{vignette("crq")}.}
\value{An object of class \code{crq}.}

\references{

Fitzenberger, B.  (1996): ``A Guide to Censored Quantile
Regressions,'' in \emph{Handbook of Statistics}, ed. by C.~Rao,   and
G.~Maddala. North-Holland: New York.

Fitzenberger, B.  and P. Winker (2007): ``Improving the Computation of
Censored Quantile Regression Estimators,'' CSDA, 52,  88-108.

Koenker, R. (2008): ``Censored Quantile Regression Redux,'' \emph{J. 
Statistical Software}, 27, \url{https://www.jstatsoft.org/v27/i06}.

Peng, L and Y Huang, (2008) Survival Analysis with Quantile Regression Models,
\emph{J. Am. Stat. Assoc.}, 103, 637-649. 

Portnoy, S. (2003) ``Censored Quantile Regression,'' \emph{JASA},
98,1001-1012.

Powell, J. (1986) ``Censored Regression Quantiles,'' \emph{J.
Econometrics}, 32, 143--155.
}


\author{Steve Portnoy and  Roger Koenker}

\seealso{\code{\link{summary.crq}}}

\examples{
# An artificial Powell example
set.seed(2345)
x <- sqrt(rnorm(100)^2)
y <-  -0.5 + x +(.25 + .25*x)*rnorm(100)
plot(x,y, type="n")
s <- (y > 0)
points(x[s],y[s],cex=.9,pch=16)
points(x[!s],y[!s],cex=.9,pch=1)
yLatent <- y
y <- pmax(0,y)
yc <- rep(0,100)
for(tau in (1:4)/5){
        f <- crq(Curv(y,yc) ~ x, tau = tau, method = "Pow")
        xs <- sort(x)
        lines(xs,pmax(0,cbind(1,xs)\%*\%f$coef),col="red")
        abline(rq(y ~ x, tau = tau), col="blue")
        abline(rq(yLatent ~ x, tau = tau), col="green")
        }
legend(.15,2.5,c("Naive QR","Censored QR","Omniscient QR"),
        lty=rep(1,3),col=c("blue","red","green"))

# crq example with left censoring
set.seed(1968)
n <- 200
x <-rnorm(n)
y <- 5 + x + rnorm(n)
plot(x,y,cex = .5)
c <- 4 + x + rnorm(n)
d <- (y > c)
points(x[!d],y[!d],cex = .5, col = 2)
f <- crq(survival::Surv(pmax(y,c), d, type = "left") ~ x, method = "Portnoy")
g <- summary(f)
for(i in 1:4) abline(coef(g[[i]])[,1])
}

\keyword{survival}
\keyword{regression}

\name{dynrq}
\alias{dynrq}
\alias{print.dynrq}
\alias{print.dynrqs}
\alias{summary.dynrq}
\alias{summary.dynrqs}
\alias{print.summary.dynrq}
\alias{print.summary.dynrqs}
\alias{time.dynrq}
\alias{index.dynrq}
\alias{start.dynrq}
\alias{end.dynrq}

\title{Dynamic Linear Quantile Regression}

\description{
Interface to \code{\link{rq.fit}} and \code{\link{rq.wfit}} for fitting dynamic linear 
quantile regression models.  The interface is based very closely
on Achim Zeileis's dynlm package.  In effect, this is  mainly
``syntactic sugar'' for formula processing, but one should never underestimate
the value of good, natural sweeteners.
}

\usage{dynrq(formula, tau = 0.5, data, subset, weights, na.action, method = "br",
  contrasts = NULL, start = NULL, end = NULL, ...)}

\arguments{
  \item{formula}{a \code{"formula"} describing the linear model to be fit.
    For details see below and \code{\link{rq}}.}
  \item{tau}{the quantile(s) to be estimated, may be vector valued, but all
    all values must be in (0,1).} 
  \item{data}{an optional \code{"data.frame"} or time series object (e.g.,
    \code{"ts"} or \code{"zoo"}), containing the variables
    in the model.  If not found in \code{data}, the variables are taken
    from \code{environment(formula)}, typically the environment from which
    \code{rq} is called.}
  \item{subset}{an optional vector specifying a subset of observations
    to be used in the fitting process.}
  \item{weights}{an optional vector of weights to be used
    in the fitting process. If specified, weighted least squares is used
    with weights \code{weights} (that is, minimizing \code{sum(w*e^2)});
    otherwise ordinary least squares is used.}
  \item{na.action}{a function which indicates what should happen
    when the data contain \code{NA}s.  The default is set by
    the \code{na.action} setting of \code{\link{options}}, and is
    \code{\link{na.fail}} if that is unset.  The \dQuote{factory-fresh}
    default is \code{\link{na.omit}}. Another possible value is
    \code{NULL}, no action. Note, that for time series regression
    special methods like \code{\link{na.contiguous}}, \code{\link[zoo]{na.locf}}
    and \code{\link[zoo]{na.approx}} are available.}
  \item{method}{the method to be used; for fitting, by default
    \code{method = "br"} is used; \code{method = "fn"} employs
    the interior point (Frisch-Newton) algorithm.  The latter is advantageous
    for problems with sample sizes larger than about 5,000.}
  \item{contrasts}{an optional list. See the \code{contrasts.arg}
    of \code{model.matrix.default}.}
  \item{start}{start of the time period which should be used for fitting the model.}
  \item{end}{end of the time period which should be used for fitting the model.}
  \item{\dots}{additional arguments to be passed to the low level
    regression fitting functions.}
}

\details{
The interface and internals of \code{dynrq} are very similar to \code{\link{rq}},
but currently \code{dynrq} offers two advantages over the direct use of
\code{rq} for time series applications of quantile regression: 
extended formula processing, and preservation of time series attributes.  
Both features have been shamelessly lifted from Achim Zeileis's
package dynlm.

For specifying the \code{formula} of the model to be fitted, there are several
functions available which allow for convenient specification
of dynamics (via \code{d()} and \code{L()}) or linear/cyclical patterns
(via \code{trend()}, \code{season()}, and \code{harmon()}).
These new formula functions require that their arguments are time
series objects (i.e., \code{"ts"} or \code{"zoo"}).

Dynamic models: An example would be \code{d(y) ~ L(y, 2)}, where
\code{d(x, k)} is \code{diff(x, lag = k)} and \code{L(x, k)} is
\code{lag(x, lag = -k)}, note the difference in sign. The default
for \code{k} is in both cases \code{1}. For \code{L()}, it
can also be vector-valued, e.g., \code{y ~ L(y, 1:4)}. 

Trends: \code{y ~ trend(y)} specifies a linear time trend where
\code{(1:n)/freq} is used by default as the covariate, \code{n} is the 
number of observations and \code{freq} is the frequency of the series
(if any, otherwise \code{freq = 1}). Alternatively, \code{trend(y, scale = FALSE)}
would employ \code{1:n} and \code{time(y)} would employ the original time index.

Seasonal/cyclical patterns: Seasonal patterns can be specified
via \code{season(x, ref = NULL)} and harmonic patterns via
\code{harmon(x, order = 1)}.  \code{season(x, ref = NULL)} creates a factor 
with levels for each cycle of the season. Using
the \code{ref} argument, the reference level can be changed from the default
first level to any other. \code{harmon(x, order = 1)} creates a matrix of
regressors corresponding to \code{cos(2 * o * pi * time(x))} and 
\code{sin(2 * o * pi * time(x))} where \code{o} is chosen from \code{1:order}.

See below for examples. 

Another aim of \code{dynrq} is to preserve 
time series properties of the data. Explicit support is currently available 
for \code{"ts"} and \code{"zoo"} series. Internally, the data is kept as a \code{"zoo"}
series and coerced back to \code{"ts"} if the original dependent variable was of
that class (and no internal \code{NA}s were created by the \code{na.action}).

}

\seealso{\code{\link[zoo]{zoo}}, 
\code{\link[zoo]{merge.zoo}}}

\examples{
###########################
## Dynamic Linear Quantile Regression Models ##
###########################

require(zoo)
## multiplicative median SARIMA(1,0,0)(1,0,0)_12 model fitted to UK seatbelt data
     uk <- log10(UKDriverDeaths)
     dfm <- dynrq(uk ~ L(uk, 1) + L(uk, 12))
     dfm

     dfm3 <- dynrq(uk ~ L(uk, 1) + L(uk, 12),tau = 1:3/4)
     summary(dfm3)
 ## explicitly set start and end
     dfm1 <- dynrq(uk ~ L(uk, 1) + L(uk, 12), start = c(1975, 1), end = c(1982, 12))
 ## remove lag 12
     dfm0 <- update(dfm1, . ~ . - L(uk, 12))
     tuk1  <- anova(dfm0, dfm1)
 ## add seasonal term
     dfm1 <- dynrq(uk ~ 1, start = c(1975, 1), end = c(1982, 12))
     dfm2 <- dynrq(uk ~ season(uk), start = c(1975, 1), end = c(1982, 12))
     tuk2 <- anova(dfm1, dfm2)
 ## regression on multiple lags in a single L() call
     dfm3 <- dynrq(uk ~ L(uk, c(1, 11, 12)), start = c(1975, 1), end = c(1982, 12))
     anova(dfm1, dfm3)

###############################
## Time Series Decomposition ##
###############################

## airline data
\dontrun{
ap <- log(AirPassengers)
fm <- dynrq(ap ~ trend(ap) + season(ap), tau = 1:4/5)
sfm <- summary(fm)
plot(sfm)
}

## Alternative time trend specifications:
##   time(ap)                  1949 + (0, 1, ..., 143)/12
##   trend(ap)                 (1, 2, ..., 144)/12
##   trend(ap, scale = FALSE)  (1, 2, ..., 144)

###############################
## An Edgeworth (1886) Problem##
###############################
# DGP
fye <- function(n, m = 20){
    a <- rep(0,n)
    s <- sample(0:9, m, replace = TRUE)
    a[1] <- sum(s)
    for(i in 2:n){
       s[sample(1:20,1)] <- sample(0:9,1)
       a[i] <- sum(s)
    }
    zoo::zoo(a)
}
x <- fye(1000)
f <- dynrq(x ~ L(x,1))
plot(x,cex = .5, col = "red")
lines(fitted(f), col = "blue")

}

\keyword{regression}

\name{rq.fit.sfn}
\alias{rq.fit.sfn}
\alias{sfnMessage}
\title{Sparse Regression Quantile Fitting}
\description{
  Fit a quantile regression model using a sparse implementation of the
  Frisch-Newton interior-point algorithm.
}
\usage{
rq.fit.sfn(a, y, tau = 0.5, rhs = (1-tau)*c(t(a) \%*\% rep(1,length(y))), control)
}
\arguments{
  \item{a}{structure of the design matrix X stored in csr format}
  \item{y}{response vector}
  \item{tau}{desired quantile}
  \item{rhs}{the right-hand-side of the dual problem; regular users
    shouldn't need to specify this, but in special cases can be quite
    usefully altered to meet special needs.  See e.g. Section 6.8 of
    Koenker (2005).}
  \item{control}{control parameters for fitting routines:  see \code{sfn.control}}
}
\details{
  This is a sparse implementation of the Frisch-Newton algorithm for quantile
  regression described in Portnoy and Koenker (1997). The sparse matrix
  linear algebra is implemented through the functions available in the R
  package \pkg{SparseM}.
}
\value{
  \item{coef}{Regression quantile coefficients}
  \item{ierr}{Error code for the internal Fortran routine \code{srqfnc}:
    \describe{
      \item{1:}{ insufficient work space in call to \code{extract}}
      \item{2:}{ nnzd > nnzdmax}
      \item{3:}{ insufficient storage in iwork when calling ordmmd}
      \item{4:}{ insufficient storage in iwork when calling sfinit}
      \item{5:}{ nnzl > nnzlmax when calling sfinit}
      \item{6:}{ nsub > nsubmax when calling sfinit}
      \item{7:}{ insufficient work space in iwork when calling symfct}
      \item{8:}{ inconsistancy in input when calling symfct}
      \item{9:}{ tmpsiz > tmpmax when calling bfinit; increase tmpmax}
      \item{10:}{ nonpositive diagonal encountered, not positive definite}
      \item{11:}{ insufficient work storage in tmpvec when calling blkfct}
      \item{12:}{ insufficient work storage in iwork when calling blkfct}
      \item{17:}{ tiny diagonals replaced with Inf when calling blkfct}
    }
  }
  \item{it}{Iteration count}
  \item{time}{Amount of time used in the computation}
}
\references{
  Portnoy, S. and R. Koenker (1997) The Gaussian Hare and the Laplacean Tortoise: 
  Computability of Squared-error vs Absolute Error Estimators, (with discussion). 
  \emph{Statistical Science}, 12, 279-300.

  Koenker, R and Ng, P. (2003).  SparseM:  A Sparse Matrix Package for \R, 
  \emph{J. of Stat. Software}, 8, 1--9.

  Koenker, R. (2005) \emph{Quantile Regression}, Cambridge U. Press.
}
\author{Pin Ng}
\seealso{\code{rq.fit.sfnc} for the constrained version,
  \code{SparseM} for a sparse matrix package for \R
}
\examples{
## An artificial example :
n <- 200
p <- 50
set.seed(101)
X <- rnorm(n*p)
X[abs(X) < 2.0] <- 0
X <- cbind(1, matrix(X, n, p))
y <- 0.5 * apply(X,1,sum) + rnorm(n) ## true beta = (0.5, 0.5, ...)

sX <- as.matrix.csr(X)
try(rq.o <- rq.fit.sfn(sX, y)) #-> not enough tmp memory
(tmpmax <- floor(1e5 + exp(-12.1)*(sX@ia[p+1]-1)^2.35))
## now ok:
rq.o <- rq.fit.sfn(sX, y, control = list(tmpmax= tmpmax))
}
\keyword{regression}

\name{print.KhmaladzeTest}
\alias{print.KhmaladzeTest}
\title{ Print a KhmaladzeTest object}
\description{
Print an object generated by KhmaladzeTest
}
\usage{
\method{print}{KhmaladzeTest}(x, ...)
}
\arguments{
\item{x}{
  Object returned from KhmaladzeTest representing the fit of the model.
 }
\item{...}{
  Optional arguments. 
 }
}
\seealso{  \code{\link{KhmaladzeTest}}}

\keyword{ regression}

\name{latex.table}
\alias{latex.table}
\title{ Writes a latex formatted table to a file}
\description{
	Automatically generates a latex formatted table from the matrix x
	Controls rounding, alignment, etc, etc
}
\usage{\method{latex}{table}(x, file=as.character(substitute(x)), 
	rowlabel=file, rowlabel.just="l", cgroup, n.cgroup, rgroup, n.rgroup=NULL, 
	digits, dec, rdec, cdec, append=FALSE, dcolumn=FALSE, cdot=FALSE, 
	longtable=FALSE, table.env=TRUE, lines.page=40, caption, caption.lot, 
	label=file, double.slash=FALSE,\dots)
}
\arguments{
  \item{x}{ A matrix \code{x} with dimnames}
  \item{file}{ Name of output \code{file} (.tex will be added) }
  \item{rowlabel}{ If `x' has row dimnames, rowlabel is a character 
	string containing the column heading for the row dimnames. 
	The default is the name of the argument for x. }
  \item{rowlabel.just}{ If `x' has row dimnames, specifies the 
	justification for printing them.  Possible values are `
	"l", "r", "c"'. The heading (`rowlabel') itself
	is left justified if `rowlabel.just="l"', otherwise it is centered. }
  \item{cgroup}{ a vector of character strings defining major column headings. 
	The default is to have none. }
  \item{n.cgroup}{ a vector containing the number of columns for which each 
	element in cgroup is a heading.  For example, specify `cgroup=
	c("Major 1","Major 2")', `n.cgroup=c(3,3)' if "Major 1" is to 
	span columns 1-3 and "Major 2" is to span columns 4-6.  
	`rowlabel' does not count in the column numbers.  You can omit 
	`n.cgroup' if all groups have the same number of columns.  }
  \item{rgroup}{ a vector of character strings containing headings for row 
	groups.  `n.rgroup' must be present when `rgroup' is given. The 
	first `n.rgroup[1]' rows are sectioned off and `rgroup[1]' is 
	used as a bold heading for them. The usual row dimnames (which must 
	be present if `rgroup' is) are indented. The next `n.rgroup[2]' 
	rows are treated likewise, etc.  }
  \item{n.rgroup}{integer vector giving the number of rows in each grouping. 
	If `rgroup' is not specified, `n.rgroup' is just used to divide off 
	blocks of rows by horizontal lines. If `rgroup' is given but 
	`n.rgroup' is omitted, `n.rgroup' will default so that each row 
	group contains the same number of rows.  }
  \item{digits}{ causes all values in the table to be formatted to `digits' 
	significant digits.  `dec' is usually preferred.  }
  \item{dec}{ If `dec' is a scalar, all elements of the matrix will be 
	rounded to `dec' decimal places to the right of the decimal. 
	`dec' can also be a matrix whose elements correspond to `x', for 
	customized rounding of each element.}
  \item{rdec}{ a vector specifying the number of decimal places to the right 
	for each row (`cdec' is more commonly used than `rdec') }
  \item{cdec}{ a vector specifying the number of decimal places for each 
	column }
  \item{append}{  defaults to `F'. Set to `T' to append output to an 
	existing file.}
  \item{dcolumn}{ Set to `T' to use David Carlisles `dcolumn' style for 
	decimal alignment.
	Default is `F', which aligns columns of numbers by changing leading
	blanks to "~", the LaTeX space-holder. You will probably want to
	use `dcolumn' if you use `rdec', as a column may then contain varying
	number of places to the right of the decimal. `dcolumn' can line up
	all such numbers on the decimal point, with integer values right-
	justified at the decimal point location of numbers that actually
	contain decimal places.  }
  \item{cdot}{ Set to `T' to use centered dots rather than ordinary periods 
	in numbers.}
  \item{longtable}{ Set to `T' to use David Carlisles LaTeX `longtable' style, 
	allowing long tables to be split over multiple pages with headers 
	repeated on each page.}
  \item{table.env}{Set `table.env=FALSE' to suppress enclosing the table
	in a LaTeX `table' environment.  `table.env' only applies when 
	`longtable=FALSE'.  You may not specify a `caption' if `table.env=FALSE'.  }
  \item{lines.page}{ Applies if `longtable=TRUE'. No more than `lines.page' 
	lines in the body of a table will be placed on a single page. 
	Page breaks will only occur at `rgroup' boundaries. }
  \item{caption}{ a text string to use as a caption to print at the top of the 
	first page of the table. Default is no caption.  }
  \item{caption.lot}{ a text string representing a short caption to be used 
	in the "List of Tables".  By default, LaTeX will use `caption'.  }
  \item{label}{ a text string representing a symbolic label for the table 
	for referencing with the LaTex `\\ref\{label\}' command. The default 
	is `file'.  `label' is only used if `caption' is given.  }
  \item{double.slash}{set to `T' to output `\\' as `\\\\' in LaTeX commands. 
	Useful when you are reading the output file back into an S vector 
	for later output.  }
  \item{\dots}{other optional arguments}
}
\value{ returns invisibly }
\references{ Minor modification of Frank Harrell's Splus code }
\author{ Roger Koenker }
\keyword{ utilities }

\name{barro}
\alias{barro}
\title{Barro Data}
\description{
Version of the Barro Growth Data used in Koenker and Machado(1999).
This is a regression data set consisting of 161 observations on determinants
of cross country GDP growth rates. There are 13 covariates with dimnames
corresponding to the original Barro and Lee source.  See
https://www.nber.org/pub/barro.lee/.  The first 71 observations are on
the period 1965-75, remainder on 1987-85.  
}
\usage{data(barro)}
\format{A data frame containing 161 observations on 14 variables:
\tabular{rl}{
        [,1] \tab "Annual Change Per Capita GDP"\cr
        [,2] \tab "Initial Per Capita GDP"\cr
        [,3] \tab "Male Secondary Education"\cr
        [,4] \tab "Female Secondary Education"\cr
        [,5] \tab "Female Higher Education"\cr
        [,6] \tab "Male Higher Education"\cr
        [,7] \tab "Life Expectancy"\cr
        [,8] \tab "Human Capital"\cr
        [,9] \tab "Education/GDP"\cr
        [,10] \tab "Investment/GDP"\cr
        [,11] \tab "Public Consumption/GDP"\cr
        [,12] \tab "Black Market Premium"\cr
        [,13] \tab "Political Instability"\cr
        [,14] \tab "Growth Rate Terms Trade"}

}
\references{
Koenker, R. and J.A.F. Machado (1999) Goodness of Fit and Related Inference Processes for Quantile Regression, JASA, 1296-1310.}
\keyword{datasets}

\name{engel}
\alias{engel}
\title{Engel Data}
\description{
  Engel food expenditure data used in Koenker and Bassett(1982).
  This is a regression data set consisting of 235 observations on
  income and expenditure on food for Belgian working class households.
}
\usage{data(engel)}
\format{A data frame containing 235 observations on 2 variables
  \describe{
    \item{income}{annual household income in Belgian francs}
    \item{foodexp}{annual household food expenditure in Belgian francs}
  }
}
\references{
  Koenker, R. and Bassett, G (1982)
  Robust Tests of Heteroscedasticity based on Regression Quantiles;
  \emph{Econometrica} \bold{50}, 43--61.
}
\examples{
## See also    demo("engel1")
##             --------------

data(engel)
plot(engel, log = "xy",
     main = "'engel' data  (log - log scale)")
plot(log10(foodexp) ~ log10(income), data = engel,
     main = "'engel' data  (log10 - transformed)")
taus <- c(.15, .25, .50, .75, .95, .99)
rqs <- as.list(taus)
for(i in seq(along = taus)) {
  rqs[[i]] <- rq(log10(foodexp) ~ log10(income), tau = taus[i], data = engel)
  lines(log10(engel$income), fitted(rqs[[i]]), col = i+1)
}
legend("bottomright", paste("tau = ", taus), inset = .04,
       col = 2:(length(taus)+1), lty=1)
}
\keyword{datasets}

\name{qrisk}
\alias{qrisk}
\title{ Function to compute Choquet portfolio weights}
\description{
This function solves a weighted quantile regression problem to find the
optimal portfolio weights minimizing a Choquet risk criterion described
in Bassett, Koenker, and Kordas (2002).
}
\usage{
qrisk(x, alpha = c(0.1, 0.3), w = c(0.7, 0.3), mu = 0.07, 
      R = NULL, r = NULL, lambda = 10000)
}
\arguments{
  \item{x}{n by q matrix of historical or simulated asset returns }
  \item{alpha}{vector of alphas receiving positive weights in the Choquet criterion}
  \item{w}{weights associated with alpha in the Choquet criterion  }
  \item{mu}{targeted rate of return for the portfolio}
  \item{R}{matrix of constraints on the parameters of the quantile regression, see below}
  \item{r}{rhs vector of the constraints described by R}
  \item{lambda}{Lagrange multiplier associated with the constraints}
}
\details{
The function calls \code{rq.fit.hogg} which in turn calls the constrained Frisch
Newton algorithm.  The constraints Rb=r are intended to apply only to the slope
parameters, not the intercept parameters.  The user is completely responsible to
specify constraints that are consistent, ie that have at least one feasible point.
See examples for imposing non-negative portfolio weights.
}
\value{
  \item{pihat}{the optimal portfolio weights}
  \item{muhat }{the in-sample mean return of the optimal portfolio}
  \item{qrisk}{the in-sample Choquet risk of the optimal portfolio}
}
\references{
\url{http://www.econ.uiuc.edu/~roger/research/risk/risk.html}

Bassett, G., R. Koenker, G Kordas, (2004) Pessimistic Portfolio Allocation and 
Choquet Expected Utility, J. of Financial Econometrics,  2, 477-492.
}
\author{ R. Koenker }
\examples{
#Fig 1:  ... of Choquet paper
        mu1 <- .05; sig1 <- .02; mu2 <- .09; sig2 <- .07
        x <- -10:40/100
        u <- seq(min(c(x)),max(c(x)),length=100)
        f1 <- dnorm(u,mu1,sig1)
        F1 <- pnorm(u,mu1,sig1)
        f2 <- dchisq(3-sqrt(6)*(u-mu1)/sig1,3)*(sqrt(6)/sig1)
        F2 <- pchisq(3-sqrt(6)*(u-mu1)/sig1,3)
        f3 <- dnorm(u,mu2,sig2)
        F3 <- pnorm(u,mu2,sig2)
        f4 <- dchisq(3+sqrt(6)*(u-mu2)/sig2,3)*(sqrt(6)/sig2)
        F4 <- pchisq(3+sqrt(6)*(u-mu2)/sig2,3)
        plot(rep(u,4),c(f1,f2,f3,f4),type="n",xlab="return",ylab="density")
        lines(u,f1,lty=1,col="blue")
        lines(u,f2,lty=2,col="red")
        lines(u,f3,lty=3,col="green")
        lines(u,f4,lty=4,col="brown")
        legend(.25,25,paste("Asset ",1:4),lty=1:4,col=c("blue","red","green","brown"))
#Now generate random sample of returns from these four densities.
n <- 1000
if(TRUE){ #generate a new returns sample if TRUE
	x1 <- rnorm(n)
	x1 <- (x1-mean(x1))/sqrt(var(x1))
	x1 <- x1*sig1 + mu1
	x2 <- -rchisq(n,3)
	x2 <- (x2-mean(x2))/sqrt(var(x2))
	x2 <- x2*sig1 +mu1
	x3 <- rnorm(n)
	x3 <- (x3-mean(x3))/sqrt(var(x3))
	x3 <- x3*sig2 +mu2
	x4 <- rchisq(n,3)
	x4 <- (x4-mean(x4))/sqrt(var(x4))
	x4 <- x4*sig2 +mu2
	}
library(quantreg)
x <- cbind(x1,x2,x3,x4)
qfit <- qrisk(x)
sfit <- srisk(x)
# Try new distortion function
qfit1 <- qrisk(x,alpha = c(.05,.1), w = c(.9,.1),mu = 0.09)
# Constrain portfolio weights to be non-negative
qfit2 <- qrisk(x,alpha = c(.05,.1), w = c(.9,.1),mu = 0.09,
	       R = rbind(rep(-1,3), diag(3)), r = c(-1, rep(0,3)))
}
\keyword{regression}
\keyword{robust}
\seealso{\code{\link{rq.fit.hogg}}, \code{\link{srisk}}}

\name{CobarOre}
\alias{CobarOre}
\docType{data}
\title{ Cobar Ore data }
\description{
Cobar Ore data from Green and Silverman (1994). 
The data consists of measurements on the "true width"
of an ore-bearing rock layer from a mine in Cobar, Australia.
}
\usage{data(CobarOre)}
\format{
  A data frame with 38 observations on the following 3 variables.
  \describe{
    \item{x}{x-coordinate of location of mine site}
    \item{y}{y-coordinate of location of mine site}
    \item{z}{ore thickness}
  }
}
\source{
Green, P.J. and B.W. Silverman (1994) Nonparametric Regression Generalized Linear Models:
A roughness penalty approach, Chapman Hall.
}
\examples{
data(CobarOre)
plot(CobarOre)
}
\keyword{datasets}

\name{uis}
\alias{uis}
\title{UIS Drug Treatment study data}
\description{There are 628 data points in the original data, 575 of which have no missing values.

Variable descriptions:

\tabular{lll}{
Variable  \tab Description                     \tab Codes/Values                   \cr
ID        \tab Identification Code             \tab 1 - 628                        \cr
AGE       \tab Age at Enrollment               \tab Years                          \cr
BECK      \tab Beck DepressionScore            \tab 0.000 - 54.000                 \cr
HC        \tab Heroin/Cocaine Use During       \tab 1 = Heroin & Cocaine           \cr
          \tab 3 Months Prior to Admission     \tab 2 = Heroin Only                \cr 
          \tab                                 \tab 3 = Cocaine Only               \cr
          \tab                                 \tab 4 = Neither Heroin nor Cocaine \cr
IV        \tab History of IV Drug Use          \tab 1 = Never                      \cr 
          \tab                                 \tab 2 = Previous                   \cr
          \tab                                 \tab 3 = Recent                     \cr
NDT       \tab Number of Prior Drug Treatments \tab 0 - 40                         \cr
RACE      \tab Subject's Race                  \tab 0 = White                      \cr
          \tab                                 \tab 1 = Non-White                  \cr
TREAT     \tab Treatment Randomization         \tab 0 = Short                      \cr
          \tab Assignment                      \tab 1 = Long                       \cr
SITE      \tab Treatment Site                  \tab 0 = A                          \cr
          \tab                                 \tab 1 = B                          \cr
LEN.T     \tab Length of Stay in Treatment     \tab Days                           \cr
          \tab (Admission Date to Exit Date)   \tab                                \cr 
TIME      \tab Time to Drug Relapse            \tab Days                           \cr
          \tab (Measured from Admission Date)  \tab                                \cr
CENSOR    \tab Event for Treating Lost to      \tab 1 = Returned to Drugs          \cr
          \tab Follow-Up as Returned to Drugs  \tab     or Lost to Follow-Up       \cr
          \tab                                 \tab 0 = Otherwise                  \cr
Y         \tab log of TIME                     \tab                                \cr  
ND1       \tab Component of NDT                \tab                                \cr  
ND2       \tab Component of NDT                \tab                                \cr
LNDT      \tab                                 \tab                                \cr 
FRAC      \tab Compliance fraction             \tab LEN.T/90 for short trt         \cr
          \tab                                 \tab LEN.T/180 for long trt         \cr
IV3       \tab Recent IV use                   \tab 1 = Yes                        \cr
          \tab                                 \tab 0 = No 

}
}
\usage{data(uis)}
\format{A data frame with dimension 575 by 18.}
\source{Table 1.3 of Hosmer,D.W. and Lemeshow, S. (1998) }
\references{Hosmer,D.W. and Lemeshow, S. (1998) Applied Survival
Analysis: Regression Modeling of Time to Event Data, John Wiley and Sons Inc.,
New York, NY}

\keyword{datasets}

\name{predict.rq}
\alias{predict.rq}
\alias{predict.rqs}
\alias{predict.rq.process}
\title{Quantile Regression Prediction}
\description{
Prediction based on fitted quantile regression model
}
\usage{
\method{predict}{rq}(object, newdata, type = "none", interval = c("none", "confidence"), 
	level = .95, na.action = na.pass, ...)
\method{predict}{rqs}(object, newdata, type = "Qhat", stepfun = FALSE, na.action = na.pass, ...)
\method{predict}{rq.process}(object, newdata, type = "Qhat", stepfun = FALSE, na.action = na.pass, ...)
}
\arguments{
  \item{object}{ object of class rq or rqs or rq.process produced by \code{rq} }
  \item{newdata}{An optional data frame in which to look for variables with
    which to predict.  If omitted, the fitted values are used.}
  \item{interval}{type of interval desired:  default is  'none', when set to
    'confidence' the function returns a matrix predictions with point predictions
    for each of the 'newdata' points as well as lower and upper confidence limits.}
  \item{level}{converage probability for the 'confidence' intervals.}
  \item{type}{For \code{predict.rq}, the  method for 'confidence' intervals, if desired. 
    If 'percentile' then one of the bootstrap methods is used to generate percentile 
    intervals for each prediction, if 'direct' then a version of the Portnoy and Zhou 
    (1998) method is used, and otherwise an estimated covariance matrix for the parameter
    estimates is used.  Further arguments to determine the choice of bootstrap
    method or covariance matrix estimate can be passed via the \dots argument.
    For \code{predict.rqs} and \code{predict.rq.process} when \code{stepfun = TRUE},
    \code{type} is "Qhat", "Fhat" or "fhat" depending  on whether the user would
    like to have estimates of the conditional quantile, distribution or density  functions
    respectively.  As noted below the two former estimates can be monotonized with the 
    function \code{rearrange}.  When the "fhat" option is invoked, a list of conditional
    density functions is returned based on Silverman's adaptive kernel method as
    implemented in \code{akj} and \code{approxfun}.}
  \item{stepfun}{If 'TRUE' return stepfunctions otherwise return matrix of predictions.
    these functions can be estimates of either the conditional quantile or distribution
    functions depending upon the \code{type} argument.  When \code{stepfun = FALSE}
    a matrix of point estimates of the conditional quantile function at the points
    specified by the \code{newdata} argument. }
  \item{na.action}{ function determining what should be done with missing values
    in 'newdata'.  The default is to predict 'NA'.}
  \item{\dots}{ Further arguments passed to or from other methods.}
}
\details{
     Produces predicted values, obtained by evaluating the quantile
     regression function in the frame 'newdata' (which defaults to
     'model.frame(object)'.  These predictions purport to estimate
    the conditional quantile function of the response variable of
    the fitted model evaluated at the covariate values specified
    in "newdata" and the quantile(s) specified by the "tau" argument.
    Several methods are provided to compute confidence intervals for
    these predictions.  
}
\value{
     A vector or matrix of predictions, depending upon the setting of 
     'interval'.  In the case that there are multiple taus in \code{object}
     when object is of class 'rqs' setting 'stepfun = TRUE'  will produce a
     \code{stepfun} object or a list of \code{stepfun} objects.
     The function \code{rearrange} can be used to monotonize these
     step-functions, if desired.
}
\references{ 
Zhou, Kenneth Q. and Portnoy, Stephen L. (1998) 
Statistical inference on heteroscedastic models based on regression quantiles 
Journal of Nonparametric Statistics, 9, 239-260 
}
\author{R. Koenker}
\seealso{ \code{\link{rq}} \code{\link{rearrange}}}
\examples{
data(airquality)
airq <- airquality[143:145,]
f <- rq(Ozone ~ ., data=airquality)
predict(f,newdata=airq)
f <- rq(Ozone ~ ., data=airquality, tau=1:19/20)
fp <- predict(f, newdata=airq, stepfun = TRUE)
fpr <- rearrange(fp)
plot(fp[[2]],main = "Conditional Ozone Quantile Prediction")
lines(fpr[[2]], col="red")
legend(.2,20,c("raw","cooked"),lty = c(1,1),col=c("black","red"))
fp <- predict(f, newdata=airq, type = "Fhat", stepfun = TRUE)
fpr <- rearrange(fp)
plot(fp[[2]],main = "Conditional Ozone Distribution Prediction")
lines(fpr[[2]], col="red")
legend(20,.4,c("raw","cooked"),lty = c(1,1),col=c("black","red"))
  }
\keyword{regression}

\name{bandwidth.rq}
\alias{bandwidth.rq}
\title{ bandwidth selection for rq functions }
\description{
 function to compute bandwidth for sparsity estimation 
}
\usage{
bandwidth.rq(p, n, hs=TRUE, alpha=0.05)
}
\arguments{
  \item{p}{ quantile(s) of interest }
  \item{n}{ sample size }
  \item{hs}{ flag for hall-sheather method }
  \item{alpha}{ alpha level for intended confidence intervals }
}
\details{ If hs=TRUE (default) then the Hall-Sheather(1988) rule \eqn{O(n^{-1/3})}
is used, if hs=FALSE then the Bofinger \eqn{O(n^{-1/5})} is used.
}
\value{
returns a vector of bandwidths corresponding to the argument p.
}
\references{ Hall and Sheather(1988, JRSS(B)),Bofinger (1975, Aus. J. Stat)}
\author{ Roger Koenker rkoenker@uiuc.edu}
\keyword{ regression }

\name{QTECox}
\alias{QTECox}
\title{Function to obtain QTE from a Cox model}
\description{Computes quantile treatment effects comparable to those of 
crq model from a coxph object.}
\usage{
QTECox(x, smooth = TRUE)
}
\arguments{
  \item{x}{An object of class coxph produced by \code{coxph}.}
  \item{smooth}{Logical indicator if TRUE (default) 
	then Cox survival function is smoothed.}
}
\details{ Estimates of the Cox QTE, \eqn{\frac{dQ(t|x)}{dx_{j}}}{(d/dx_j) Q( t | x ) } 
          at \eqn{x=\bar{x}}{x=xbar}, can be expressed as a function of t as follows:

\deqn{\frac{dQ(t|x)}{dx_{j}}=\frac{dt}{dx_{j}}\frac{dQ(t|x)}{dt}}{
(d/dx_j) Q( t | x ) = (d/dx_j)t * (d/dt) Q(t | x)}

The Cox survival function, \eqn{S(y|x)=\exp \{-H_{0}(y)\exp (b^{\prime
}x)\}}{S( y | x ) = exp{ - H_o(y) exp(b'x) }}

\deqn{\frac{dS(y|x)}{dx_{j}}=S(y|x)log \{S(y|x)\}b_{j}}{(d/dx_j) 
S( y | x ) = S( y | x ) log(S( y | x )) b_j}


where \eqn{\frac{dQ(t|x)}{dx_{j}}}{ (d/dt) Q(t | x) }
can be estimated by \eqn{\frac{\Delta (t)}{\Delta (S)}
(1-t)}{- (diff(t)/diff(S) (1-t)}
where $S$  and $t$ denote the \code{surv} and \code{time} components
of the  \code{survfit} object.
Note that since \eqn{t=1-S(y|x)}{t = 1 - S( y | x )}, the above is the 
value corresponding to the argument $(1-t)$; and furthermore

\deqn{\frac{dt}{dx_{j}}=-\frac{dS(y|x)}{dx_{j}}=-(1-t) log (1-t)b_{j}}{
   (d/dx_j)t = - (d/dx_j) S( y | x ) =  - (1-t) log(1-t) b_j}

Thus the QTE at the mean of x's is:

\deqn{(1-S)= \frac{\Delta (t)}{\Delta (S)}S ~log
(S)b_{j}}{(1 - S) = (diff(t)/diff(S) S log(S) b_j}


Since \eqn{\Delta S}{diff(S)} is negative and $log (S)$ is also negative
this has the same sign as \eqn{b_{j}}
The crq  model fits the usual AFT form  Surv(log(Time),Status), then 

\deqn{\frac{d log (Q(t|x))}{dx_{j}}=\frac{dQ(t|x)}{dx_{j}}/
Q(t|x)}{(d/dx_j) log(Q( t | x )) = (d/dx_j) Q( t | x ) / Q( t | x )}

This is the matrix form returned.
}

\value{
  \item{taus }{points of evaluation of the QTE.}
  \item{QTE}{matrix of QTEs, the ith column contains the QTE for the
	ith covariate effect.  Note that there is no intercept effect.
	see \code{plot.summary.crqs} for usage.}
}

\references{Koenker, R. and Geling, O. (2001). Reappraising Medfly 
longevity: a quantile regression survival analysis, J. Amer. Statist. 
Assoc., 96, 458-468} 

\author{Roger Koenker Stephen Portnoy & Tereza Neocleous}
\seealso{\code{\link{crq}}}

\keyword{survival}

\name{gasprice}
\alias{gasprice}
\docType{data}
\title{Time Series of US Gasoline Prices
}
\description{ Time Series of Weekly US Gasoline Prices: 1990:8 -- 2003:26
}
\usage{data("gasprice")}
\examples{
data(gasprice)
}
\keyword{datasets}

\name{latex}
\alias{latex}
\title{ Make a latex version of an R object }
\description{
Generic function for converting an  \R object into a latex file.
}
\usage{
latex(x, ...)
}
\arguments{
  \item{x}{ \code{x} is an \R object }
  \item{\dots}{ \code{\dots} optional arguments }
}
\seealso{  \code{\link{latex.table}}, \code{\link{latex.summary.rqs}}
}
\keyword{IO}

\name{anova.rq}
\alias{anova.rq}
\alias{anova.rqs}
\alias{anova.rqlist}
\alias{print.anova.rq}
\alias{rq.test.rank}
\alias{rq.test.anowar}
\title{ Anova function for quantile regression fits }
\description{
	Compute test statistics for two or more quantile regression fits.
}
\usage{
\method{anova}{rq}(object, ..., test = "Wald", joint = TRUE, score =
                       "tau", se = "nid", iid = TRUE, R = 200, trim = NULL)
\method{anova}{rqs}(object, ..., se = "nid", iid = TRUE, joint = TRUE)
\method{anova}{rqlist}(object, ...,  test = "Wald", joint = TRUE, 
	score = "tau", se = "nid", iid = TRUE, R = 200, trim = NULL)
rq.test.rank(x0, x1, y, v = NULL, score = "wilcoxon", weights = NULL, tau=.5,
        iid = TRUE, delta0 = rep(0,NCOL(x1)), omega = 1, trim = NULL, pvalue = "F")
rq.test.anowar(x0,x1,y,tau,R)
\method{print}{anova.rq}(x, ...)
}
\arguments{
  \item{object, ...}{objects of class `rq', originating from a call to `rq'. 
	or a single object of class rqs, originating from a call to 'rq' with
	multiple taus specified.}
  \item{test}{ A character string specifying the test statistic to use.
	Can be either `Wald' or `rank'.}
  \item{joint}{A logical flag indicating whether tests of equality of slopes
	should be done as joint tests on all slope parameters, or whether
	(when joint = FALSE) separate tests on each of the slope parameters
	should be reported.  This option applies only to the tests of 
	equality of slopes in the case that estimated models correspond
	to distinct taus.}
  \item{score}{ A character string specifying the score function to use,
	only needed or applicable for the `rank' form of the test.}

  \item{trim}{optional trimming proportion parameter(s)  -- only applicable for the
	Wilcoxon score function --  when one value is provided there is symmetric
	trimming of the score integral to the interval \code{(trim, 1-trim)}, when
	there are two values provided, then the trimming restricts the integration
	to \code{(trim[1], trim[2])}.}
  \item{x}{ objects of class `summary.rq', originating from a call to `summary'. }
  \item{x0}{design matrix for the null component of the rank and anowar tests. }
  \item{x1}{design matrix for the alternative component of the rank and anowar tests. }
  \item{y}{response vector for the alternative component of the rank and anowar tests. }
  \item{v}{optional rq process fit}
  \item{se}{method for computing standard errors, either "nid" or "ker", note 
	that "boot" cannot be used for testing homogeneity of slopes.}
  \item{tau}{quantile of interest for quantile specific forms of testing.}
  \item{iid}{logical flag for quantile specific forms of testing, if TRUE the
	test presumes that the conditional densities take identical values,
	if it is FALSE then local densities are estimated and used, see Koenker(2005)
	p. 90.}
  \item{delta0}{vector of hypothetical parameter values under test, typically zeros
	but can be specified to be nonzero in cases where simulations are being used
	to evaluate the validity of the non-central chisquare theory of the test.} 
  \item{omega}{value to be used for the score and F dependent constant appearing
	in the non-centrality parameter,  this is only needed/useful when delta0
	is specified to be non-zero.  In the usual Wilcoxon (untrimmed) case this
	value is the integral the squared density.}
  \item{pvalue}{type of p-value to be used, by default a pseudo F-statistic is
	produced and the corresponding F p-value is computed, otherwise the
	more conventional chisquared p-values are reported.}
  \item{weights}{optional weight vector to be used for fitting.}
  \item{R}{ The number of resampling replications for the anowar form of the test, 
	used to estimate the reference distribution for the test statistic.}
}
\details{
There are two (as yet) distinct forms of the test.  In the first the
fitted objects all have the same specified quantile (tau) and the intent
is to test the hypothesis that smaller models are adequate relative to
the largest specified model.  In the second form of the test the linear predictor 
of the fits are all the same, but the specified quantiles (taus) are different.  

In the former case there are three options for
the argument `test', by default a Wald test is computed as in 
Bassett and Koenker (1982).  If \code{test = 'anowar'} is specified
then the test is based on the procedure suggested in Chen, Ying, Zhang
and Zhao (2008); the test is based on the difference in the QR objective
functions at the restricted and unrestricted models with a reference
distribution computed by simulation.  The p-value of this form of the
test is produced by fitting a density to the simulation values forming
the reference distribution using the \code{logspline} function from
the \pkg{logspline} package.  The acronym anowar stands for  analysis
of weighted absolute residuals.  If \code{test='rank'} is specified, then a rank
test statistic is computed as described in Gutenbrunner, Jureckova,
Koenker and Portnoy (1993).  In the latter case one can also specify
a form for the score function of the rank test, by default the Wilcoxon
score is used, the other options are score=`sign' for median (sign) scores,
or score=`normal' for normal (van der Waerden) scores.  A fourth option
is score=`tau' which is a generalization of median scores to an arbitrary
quantile, in this case the quantile is assumed to be the one associated
with the fitting of the specified objects.  The computing of
the rank form of the test is carried out in the \code{\link{rq.test.rank}}
function, see \code{\link{ranks}} for further details on the score function
options.  The Wald form of the test is local in sense that the null hypothesis
asserts only that a subset of the covariates are ``insignificant'' at
the specified quantile of interest.  The rank form of the test can also be
used to test the global hypothesis that a subset is ``insignificant''
over an entire range of quantiles.  The use of the score function
score = "tau" restricts the rank test to the local hypothesis of
the Wald test.  

In the latter case the hypothesis of interest is that the slope coefficients of
the models are identical.  The test statistic is a variant of the Wald 
test described in Koenker and Bassett (1982).

By default, both forms of the tests return an F-like statistic in the sense that the
an asymptotically Chi-squared statistic is divided by its degrees of
freedom and the reported p-value is computed for an F statistic based on 
the numerator degrees of freedom equal to the rank of the null hypothesis and
the denominator degrees of freedom is taken to be the sample size
minus the number of parameters of the maintained model.
}
\value{
 An object of class `"anova"' inheriting from class `"data.frame"'.
}
\references{
[1] Bassett, G. and R. Koenker  (1982). Tests of Linear Hypotheses
and L1 Estimation,
\emph{Econometrica}, \bold{50}, 1577--83.

[2] Koenker, R. W. and Bassett, G. W. (1982). Robust Tests for
Heteroscedasticity based on Regression Quantiles,
\emph{Econometrica}, \bold{50}, 43--61.
 
[3] Gutenbrunner, C., Jureckova, J., Koenker, R, and S. Portnoy  (1993).
Tests of Linear Hypotheses based on Regression Rank Scores,
\emph{J. of Nonparametric Statistics}, \bold{2}, 307--331.

[4] Chen, K. Z. Ying, H. Zhang, and L Zhao, (2008) Analysis of least absolute
deviations, Biometrika, 95, 107-122.

[5] Koenker, R. W. (2005).  Quantile Regression, Cambridge U. Press. 
}
\author{ Roger Koenker }

\section{WARNING }{
An attempt to verify that the models are nested in the first form
of the test is made, but this relies on checking set inclusion of
the list of variable names and is subject to obvious ambiguities
when variable names are generic.
The comparison between two or more models will only be valid if
they are fitted to the same dataset. This may be a problem if
there are missing values and R's default of `na.action = na.omit' is used. 
The rank version of the nested model tests involves computing the entire
regression quantile process using parametric linear programming and thus
can be rather slow and memory intensive on problems with more than 
several thousand observations.} 
\seealso{The model fitting function  \code{\link{rq}}, 
and the functions for testing hypothesis on the entire quantile
regression process \code{\link{KhmaladzeTest}}.  For further details
on the rank tests see \code{\link{ranks}}.}
\examples{
data(barro)
fit0 <- rq(y.net ~  lgdp2 + fse2 + gedy2 , data = barro)
fit1 <- rq(y.net ~  lgdp2 + fse2 + gedy2 + Iy2 + gcony2, data = barro)
fit2 <- rq(y.net ~  lgdp2 + fse2 + gedy2 + Iy2 + gcony2, data = barro,tau=.75)
fit3 <- rq(y.net ~  lgdp2 + fse2 + gedy2 + Iy2 + gcony2, data = barro,tau=.25)
anova(fit1,fit0)
anova(fit1,fit2,fit3)
anova(fit1,fit2,fit3,joint=FALSE)
# Alternatively fitting can be done in one call:
fit <- rq(y.net ~  lgdp2 + fse2 + gedy2 + Iy2 + gcony2, 
	  method = "fn", tau = 1:4/5, data = barro)
}
\keyword{ htest }
\keyword{ regression}
\keyword{ robust}

\name{lprq}
\alias{lprq}
\title{ locally polynomial quantile regression }
\description{
This is a toy function to illustrate how to do locally polynomial
quantile regression univariate smoothing.  
}
\usage{
lprq(x, y, h, tau = .5, m = 50)
}
\arguments{
  \item{x}{ The conditioning covariate}
  \item{y}{ The response variable  }
  \item{h}{ The bandwidth parameter }
  \item{tau}{ The quantile to be estimated }
  \item{m}{ The number of points at which the function is to be estimated }
}
\details{
The function obviously only does locally linear fitting but can be easily
adapted to locally polynomial fitting of higher order.  The author doesn't
really approve of this sort of smoothing, being more of a spline person,
so the code is left is its (almost) most trivial form. 
}
\value{
The function compute a locally weighted linear quantile regression fit
at each of the m design points, and returns:
  \item{xx}{The design points at which the evaluation occurs}
  \item{fv}{The estimated function values at these design points}
  \item{dev}{The estimated first derivative values at the design points}
}
\references{ Koenker, R. (2004) Quantile Regression }
\author{R. Koenker }
\note{One can also consider using B-spline expansions see \code{bs}.}

\seealso{ \code{rqss} for a general approach to oonparametric QR fitting.  }
\examples{
require(MASS)
data(mcycle)
attach(mcycle)
plot(times,accel,xlab = "milliseconds", ylab = "acceleration (in g)")
hs <- c(1,2,3,4)
for(i in hs){
        h = hs[i]
        fit <- lprq(times,accel,h=h,tau=.5)
        lines(fit$xx,fit$fv,lty=i)
        }
legend(50,-70,c("h=1","h=2","h=3","h=4"),lty=1:length(hs))
}
\keyword{smooth}
\keyword{robust}

\name{summary.crq}
\alias{summary.crqs}
\alias{summary.crq}
\alias{print.summary.crq}
\alias{print.summary.crqs}
\alias{plot.summary.crqs}
\title{
Summary methods for Censored Quantile Regression
}
\description{
Returns a summary object for a censored quantile regression fit.  A null value
will be returned if printing is invoked.
}
\usage{
\method{summary}{crq}(object, taus = 1:4/5, alpha = .05, se="boot", covariance=TRUE,  ...)
\method{print}{summary.crq}(x, digits = max(5, .Options$digits - 2), ...)
\method{print}{summary.crqs}(x,  ...)
\method{plot}{summary.crqs}(x, nrow = 3, ncol = 3, CoxPHit = NULL,  ...)
}
\arguments{
  \item{object}{
    An object of class \code{"crq"}  produced by a call to \code{crq()}.
  }
  \item{taus}{Quantiles to be summarized. Specifying only one value can
    produce annoying error messages harmful to your mental health.}
  \item{x}{
    An object of class \code{"crq"}  produced by a call to \code{crq()}.
  }
  \item{se}{
    specifies the method used to compute standard standard errors. but
    the only available method (so far) is "boot".  Further arguments to
    \code{\link{boot.crq}} and \code{\link{boot.rq}}  can be passed with 
    the \dots argument. 
  }
  \item{covariance}{
    logical flag to indicate whether the full covariance matrix of the 
    estimated parameters should be returned. 
  }
  \item{nrow}{Number of rows of the plot layout.}
  \item{ncol}{Number of columns of the plot layout.}
  \item{alpha}{Confidence level for summary intervals.}
  \item{digits}{Number of digits to be printed in summary display.}
  \item{CoxPHit}{An object of class coxph produced by \code{coxph}.}
  \item{...}{
    Optional arguments to summary, e.g. to specify bootstrap methods
    sample sizes, etc.  see \code{\link{boot.rq}} and \code{\link{boot.crq}}
   }
}
\value{
  For method "Powell" an object of class \code{summary.crq}  is returned 
  with the following components:
\item{coefficients}{
  a p by 4 matrix consisting of the coefficients, their estimated standard
  errors, their t-statistics, and their associated p-values.
}
\item{cov}{
  the estimated covariance matrix for the coefficients in the model,
  provided that \code{covariance = TRUE} appears in the calling sequence.
}
\item{rdf}{
  the residual degrees of freedom
}
\item{tau}{
  the quantile estimated
}
For the other methods an object of class \code{summary.crq}  is returned 
  with the following components:
\item{coefficients}{
  a list of  p by 6 matrix consisting of the coefficients, upper and lower bounds
  for a (1-alpha) level confidence interval, their estimated standard
  errors, their t-statistics, and their associated p-values, one component for each
  element of the specified \code{taus} vector.
}
\item{cov}{
  the estimated covariance matrix for the coefficients in the model,
  provided that \code{covariance = TRUE} in the called sequence.
}
}
\details{
For the Powell method the resampling strategy used by the
\code{se = "boot"} method is based on the Bilias, Chen and Ying (2000) 
proposal.  For the Portnoy and Peng-Huang methods the bootstrapping 
is by default actually based on  a delete-d jackknife, as described in
Portnoy (2013), but resampling xy pairs using either conventional multinomial 
resampling or using exponential weighting as in Bose and Chatterjee (2003)
can be used by specifying the \code{bmethod} argument.  Note that the default
number of replications is set at \eqn{R = 100} a value that is obviously too small for
most applications.  This is done merely to speed up the examples in the 
documentation and facilitate testing. Larger, more appropriate values of \eqn{R}
can be passed to the bootstrapping functions via the \code{\ldots} argument
of the \code{summary} method.  It is important to recognize that when some
of the bootstrap replications are NA they are simply ignored in the computation
of the confidence bands and standard errors as currently reported.  The number
of these NAs is returned as part of the \code{summary.crq} object, and 
when printed is also reported. 
}

\references{
Bose, A. and S. Chatterjee, (2003) Generalized bootstrap for estimators 
	of minimizers of convex functions, \emph{J. Stat. Planning and Inf}, 117,
	225-239.

Bilias, Y. Chen, S. and Z. Ying, (2000) Simple resampling methods for censored
	quantile regression, \emph{J. of Econometrics}, 99, 373-386.

Portnoy, S. (2013) The Jackknife's Edge:  Inference for Censored Quantile Regression,
        \emph{CSDA}, forthcoming.

}
\seealso{
  \code{\link{crq}}, \code{\link{QTECox}}
}
\keyword{regression}

\name{rqss}
\alias{rqss}
\alias{rqss.fit}
\alias{[.terms}
\alias{untangle.specials}
\title{Additive Quantile Regression Smoothing}
\description{
Fitting function for additive quantile regression models with possible univariate
and/or bivariate nonparametric terms estimated by total variation regularization.
}
\usage{
rqss(formula, tau = 0.5, data = parent.frame(), weights, subset, na.action,
	method = "sfn", lambda = NULL, contrasts = NULL, ztol = 1e-5, control, ...)
}
\arguments{
  \item{formula}{  a formula object, with the response on the left of a `~'
        operator,  and terms, separated by `+' operators, on the right.
	The terms may include \code{qss} terms that represent additive
	nonparametric components.  These terms can be univariate or
	bivariate.  See \code{\link{qss}} for details on how to
	specify these terms.}
  \item{tau}{
    the quantile to be estimated, this must be a number between 0 and 1,
  }
  \item{data}{
    a data.frame in which to interpret the variables
    named in the formula, or in the subset and the weights argument.
  }
 \item{weights}{
    vector of observation weights; if supplied, the algorithm fits
    to minimize the sum of the weights multiplied into the
    absolute residuals. The length of weights must be the same as
    the number of observations.  The weights must be nonnegative
    and it is strongly recommended that they be strictly positive,
    since zero weights are ambiguous.
  }
  \item{subset}{an optional vector specifying a subset of observations to be
          used in the fitting.  This can be a vector of indices of observations
	 to be included, or a logical vector.} 
  \item{na.action}{
    a function to filter missing data.
    This is applied to the model.frame after any subset argument has been used.
    The default (with \code{na.fail}) is to create an error if any missing values are
    found.  A possible alternative is \code{na.omit}, which
    deletes observations that contain one or more missing values.
  }
  \item{method}{
    the algorithmic method used to compute the fit.  There are currently
    two options.   Both are implementations of the Frisch--Newton interior
    point method described in detail in Portnoy and Koenker(1997).   Both
    are implemented using sparse Cholesky decomposition as described in
    Koenker and Ng (2003).

    Option \code{"sfnc"} is used if the user specifies inequality constraints.
    Option \code{"sfn"} is used if there are no inequality constraints.
    Linear inequality constraints on the fitted coefficients are specified
    by a matrix \code{R} and a vector \code{r}, specified inside the \code{qss}
    terms, representing the constraints in the form \eqn{Rb \ge r}{Rb >= r}.
    
    The option \code{method = "lasso"} allows one to penalize the coefficients
    of the covariates that have been entered linearly as in \code{\link{rq.fit.lasso}};
    when this is specified then there should be an additional \code{lambda}
    argument specified that determines the amount of shrinkage.  
    }
  \item{lambda}{ can be either a scalar, in which case all the slope coefficients 
    are assigned this value, or alternatively, the user can specify a vector of length 
    equal to the number of linear covariates plus one (for the intercept) and these
    values will be used as coordinate dependent shrinkage factors.
  }
  \item{contrasts}{
    a list giving contrasts for some or all of the factors
    default = \code{NULL} appearing in the model formula.
    The elements of the list should have the same name as the variable
    and should be either a contrast matrix (specifically, any full-rank
    matrix with as many rows as there are levels in the factor),
    or else a function to compute such a matrix given the number of levels.
  }
  \item{ztol}{A zero tolerance parameter used to determine the number of
    zero residuals in the fitted object which in turn determines the effective
    dimensionality of the fit.}
  \item{control}{ control argument for the fitting routines
    (see \code{\link{sfn.control}}} 
  \item{...}{Other arguments passed to fitting routines}
}
\details{ Total variation regularization for univariate and
    bivariate nonparametric quantile smoothing is described
    in Koenker, Ng and Portnoy (1994) and Koenker and Mizera(2003)
    respectively.  The additive model extension of this approach
    depends crucially on the sparse linear algebra implementation
    for R described in Koenker and Ng (2003).  There are extractor
    methods \code{\link{logLik}} and \code{\link{AIC}} that is
    relevant to lambda selection.  A more detailed description of
    some recent developments of these methods is available from
    within the package with \code{vignette("rqss")}.  Since this
    function uses sparse versions of the interior point algorithm
    it may also prove to be useful for fitting linear models
    without \code{\link{qss}} terms when the design has a sparse
    structure, as for example when there is a complicated factor 
    structure.  

    If the \pkg{MatrixModels} and \pkg{Matrix} packages are both  loadable then the 
    linear-in-parameters portion of the design matrix is made in sparse matrix form;
    this is helpful in large applications with many factor variables for which dense 
    formation of the design matrix would take too much space.

    Although modeling with \code{rqss} typically imposes smoothing penalties on
    the total variation of the first derivative, or gradient, of the fitted functions,
    for univariate smoothing, it is also possible to penalize total variation of
    the function itself using the option \code{Dorder = 0} inside \code{qss} terms.
    In such cases, estimated functions are piecewise constant rather than piecewise
    linear.  See the documentation for \code{qss} for further details.
}
\value{
    The function returns a fitted object representing the estimated
    model specified in the formula.  See \code{\link{rqss.object}}
    for further details on this object, and references to methods
    to look at it.
}
\note{
    If you intend to embed calls to \code{rqss} inside another function, then
    it is advisable to pass a data frame explicitly as the \code{data} argument
    of the \code{rqss} call, rather than relying on the magic of R scoping rules.
}
\references{
  [1] Koenker, R. and S. Portnoy (1997)
  The Gaussian Hare and the Laplacean
  Tortoise:  Computability of Squared-error vs Absolute Error Estimators,
  (with discussion).
  \emph{Statistical Science} \bold{12}, 279--300.

  [2] Koenker, R., P. Ng and S. Portnoy, (1994)
  Quantile Smoothing Splines;
  \emph{Biometrika} \bold{81}, 673--680.

  [3] Koenker, R. and I. Mizera, (2003)
  Penalized Triograms: Total Variation Regularization for Bivariate Smoothing;
  \emph{JRSS(B)} \bold{66}, 145--163.

  [4] Koenker, R. and P. Ng (2003)
  SparseM:  A Sparse Linear Algebra Package for R,
  \emph{J. Stat. Software}.
}
\author{ Roger Koenker }
\seealso{ \code{\link{qss}}
}
\examples{
n <- 200
x <- sort(rchisq(n,4))
z <- x + rnorm(n)
y <- log(x)+ .1*(log(x))^2 + log(x)*rnorm(n)/4 + z
plot(x, y-z)
f.N  <- rqss(y ~ qss(x, constraint= "N") + z)
f.I  <- rqss(y ~ qss(x, constraint= "I") + z)
f.CI <- rqss(y ~ qss(x, constraint= "CI") + z)

lines(x[-1], f.N $coef[1] + f.N $coef[-(1:2)])
lines(x[-1], f.I $coef[1] + f.I $coef[-(1:2)], col="blue")
lines(x[-1], f.CI$coef[1] + f.CI$coef[-(1:2)], col="red")

## A bivariate example
data(CobarOre)
fCO <- rqss(z ~ qss(cbind(x,y), lambda= .08), data=CobarOre)
plot(fCO)
}
\keyword{regression}
\keyword{smooth}
\keyword{robust}

\name{ParetoTest}
\alias{ParetoTest}
\alias{Hill}
\alias{Hill.fit}
\alias{print.Hill}
\alias{summary.Hill}
\alias{print.summary.Hill}
\alias{Pickands.fit}
\alias{Pickands}
\alias{print.Pickands}
\alias{summary.Pickands}
\alias{print.summary.Pickands}
\alias{Pickands.fit}
\title{Estimation and Inference on the Pareto Tail Exponent for Linear Models}
\description{Estimation and inference about the tail behavior of the response in
    linear models are based on the adaptation of the univariate Hill (1975)
    and Pickands (1975) estimators for quantile regression by Chernozhukov,
    Fernandez-Val and Kaji (2018).}

\usage{
ParetoTest(formula, tau = 0.1, data = NULL, flavor = "Hill", m = 2, cicov = .9, ...) 
}
\arguments{
\item{formula}{a formula specifying the model to fit by \code{\link{rq}}}
\item{tau}{A threshold on which to base the estimation}
\item{data}{a data frame within which to interpret the formula}
\item{flavor}{Currently limited to either "Hill" or "Pickands"}
\item{m}{a tuning parameter for the Pickands method .} 
\item{cicov}{Desired coverage probability of confidence interval.} 
\item{...}{other arguments to be passed to \code{\link{summary.rq}}.
    by default the summary method is the usual xy bootstrap, with
    \code{B = 200} replications.}
}
\value{
an object of class ParetoTest  is returned containing:

\item{z}{ A named vector with components: the estimate, a bias
corrected estimate, a lower bound of the confidence interval,
an upper bound of the confidence interval, and a Bootstrap
Standard Error estimate.}

\item{tau}{
    The tau threshold used to compute the estimate
}
}

\examples{
n = 500
x = rnorm(n)
y = x + rt(n,2)
Z = ParetoTest(y ~ x, .9, flavor = "Pickands")
}

\keyword{htest}
\references{
    Chernozhukov, Victor, Ivan Fernandez-Val, and Tetsuya Kaji, (2018)
    Extremal Quantile Regression, in Handbook of Quantile Regression, 
    Eds. Roger Koenker, Victor Chernozhukov, Xuming He, Limin Peng,
    CRC Press.

    Hill, B. M. (1975). A simple general approach to inference about the tail of a distribution. 
    The Annals of Statistics 3(5), 1163-1174.

    Pickands, J. (1975). Statistical inference using extreme order statistics. 
    The Annals of Statistics 3(1), 119-131.
}
    

\name{dither}
\alias{dither}
\title{ Function to randomly perturb a vector}
\description{
With malice aforethought, dither adds a specified random perturbation to each element
of the input vector, usually employed as a device to mitigate the effect of ties.
}

\usage{
dither(x, type = "symmetric", value = NULL)
}
\arguments{
  \item{x}{\code{x} a numeric vector }
  \item{type}{\code{type} is either 'symmetric' or 'right' }
  \item{value}{\code{value} scale of dequantization }
}
\details{
The function \code{dither} operates slightly differently than the function
\code{jitter} in base R, permitting strictly positive perturbations with
the option \code{type = "right"} and using somewhat different default schemes
for the scale of the perturbation.  Dithering the response variable is
frequently a useful option in quantile regression fitting to avoid deleterious
effects of degenerate solutions.  See, e.g. Machado  and Santos Silva (2005).
For a general introduction and some etymology see the Wikipedia article on "dither".
For integer data it is usually advisable to use \code{value = 1}.
When 'x' is a matrix or array dither treats all elements as a vector but returns
an object of the original class.
}
\value{
A dithered version of the input vector 'x'.
}

\references{ 
Machado, J.A.F. and Santos Silva, J.M.C. (2005), Quantiles for Counts,  Journal of the American Statistical Association, vol. 100, no. 472, pp. 1226-1237. 
}
\author{ R. Koenker }
\note{ Some further generality might be nice, for example something other than
uniform noise would be desirable in some circumstances.  Note that when dithering
you are entering into the "state of sin" that John von Neumann famously attributed
to anyone considering "arithmetical methods of producing random digits."  If you 
need to preserve reproducibility, then \code{set.seed} is your friend.
}
\seealso{ \code{\link{jitter}} }
\examples{
x <- rlnorm(40)
y <- rpois(40, exp(.5 + log(x)))
f <- rq(dither(y, type = "right", value = 1) ~ x)
}
\keyword{ manip }

\name{rq.fit.sfnc}
\alias{rq.fit.sfnc}
\title{Sparse Constrained Regression Quantile Fitting}
\description{
  Fit constrained regression quantiles using a sparse implementation of
  the Frisch-Newton Interior-point algorithm.
}
\usage{
rq.fit.sfnc(x, y, R, r, tau = 0.5,
            rhs = (1-tau)*c(t(x) \%*\% rep(1,length(y))),control)
}
\arguments{
  \item{x}{structure of the design matrix X stored in csr format}
  \item{y}{response vector}
  \item{R}{constraint matrix stored in csr format}
  \item{r}{right-hand-side of the constraint}
  \item{tau}{desired quantile}
  \item{rhs}{the right-hand-side of the dual problem; regular users
    shouldn't need to specify this.}
  \item{control}{control paramters for fitting see \code{sfn.control}}
}
\details{
  This is a sparse implementation of the Frisch-Newton algorithm for
  constrained quantile regression described in Koenker and Portnoy (1996).
  The sparse matrix linear algebra is implemented through the functions
  available in the R package \pkg{SparseM}.
}
\value{
  \item{coef}{Regression quantile coefficients}
  \item{ierr}{Error code for the internal Fortran routine \code{srqfn}:
    \describe{
      \item{1:}{ insufficient work space in call to \code{extract}}
      \item{3:}{ insufficient storage in iwork when calling ordmmd}
      \item{4:}{ insufficient storage in iwork when calling sfinit}
      \item{5:}{ nnzl > nnzlmax when calling sfinit}
      \item{6:}{ nsub > nsubmax when calling sfinit}
      \item{7:}{ insufficient work space in iwork when calling symfct}
      \item{8:}{ inconsistancy in input when calling symfct}
      \item{9:}{ tmpsiz > tmpmax when calling symfct; increase tmpmax}
      \item{10:}{ nonpositive diagonal encountered when calling blkfct}
      \item{11:}{ insufficient work storage in tmpvec when calling blkfct}
      \item{12:}{ insufficient work storage in iwork when calling blkfct}
      \item{13:}{ nnzd > nnzdmax in e,je when calling amub}
      \item{14:}{ nnzd > nnzdmax in g,jg when calling amub}
      \item{15:}{ nnzd > nnzdmax in h,jh when calling aplb}
      \item{15:}{ tiny diagonals replaced with Inf when calling blkfct}
    }
  }
  \item{it}{Iteration count}
  \item{time}{Amount of time used in the computation}
}
\references{
  Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R; 
  \url{https://CRAN.R-project.org/package=SparseM}

  Koenker, R. and P. Ng(2005).
  Inequality Constrained Quantile Regression, \emph{Sankya}, 418-440.
}
\author{Pin Ng}
\seealso{
  \code{\link{rq.fit.sfn}} for the unconstrained version,
  \pkg{SparseM} for the underlying sparse matrix \R package.
}
\examples{
## An artificial example :
n <- 200
p <- 50
set.seed(17)
X <- rnorm(n*p)
X[abs(X) < 2.0] <- 0
X <- cbind(1,matrix(X,n,p))
y <- 0.5 * apply(X,1,sum) + rnorm(n) ## true beta = (0.5, 0.5, ...)
R <- rbind(diag(p+1), -diag(p+1))
r <- c(rep( 0, p+1), rep(-1, p+1))

sX <- as.matrix.csr(X)
sR <- as.matrix.csr(R)
try(rq.o <- rq.fit.sfnc(sX, y, sR, r)) #-> not enough tmp memory

(tmpmax <- floor(1e5 + exp(-12.1)*(sX@ia[p+1]-1)^2.35))
## now ok:
rq.o <- rq.fit.sfnc(sX, y, sR, r, control = list(tmpmax = tmpmax))
}
\keyword{regression}

\name{print.summary.rq}
\alias{print.summary.rq}
\alias{print.summary.rqs}
\title{ Print Quantile Regression Summary Object }
\usage{
\method{print}{summary.rq}(x, digits=max(5, .Options$digits - 2), ...)
\method{print}{summary.rqs}(x, ...)
}
\arguments{
  \item{x}{
    This is an object of class \code{"summary.rq"} produced by a call to 
    \code{summary.rq()}.
  }
  \item{digits}{
    Significant digits reported in the printed table.
  }
  \item{...}{
    Optional arguments passed to printing function
  }
}
\description{Print summary of quantile regression object}
\seealso{  \code{\link{summary.rq}} }

\keyword{ regression }

\name{ranks}
\alias{ranks}
\title{
Quantile Regression Ranks
}
\description{
Function to compute ranks from the dual (regression rankscore) process.
}
\usage{
ranks(v, score="wilcoxon", tau=0.5, trim = NULL)
}
\arguments{
\item{v}{
  object of class \code{"rq.process"} generated by \code{rq()}
}
\item{score}{
  The score function desired.  Currently  implemented score  functions  
  are \code{"wilcoxon"}, \code{"normal"}, and \code{"sign"}
  which are asymptotically optimal  for  
  the  logistic,  Gaussian  and Laplace location shift models respectively.
  The "normal" score  function is also sometimes called van der Waerden scores.
  Also implemented are the \code{"tau"} which generalizes sign scores to an
  arbitrary quantile, \code{"interquartile"} which is appropriate
  for tests of scale shift, \code{normalscale} for Gaussian scale shift,
  \code{halfnormalscale} for Gaussian scale shift only to the right of the median,
  and \code{lehmann} for Lehmann local alternatives. See Koenker (2010) for
  further details on the last three of these scores.
}
\item{tau}{
  the optional value of \code{tau} if the \code{"tau"} score function is used.
}
\item{trim}{optional trimming proportion parameter(s)  -- only applicable for the
Wilcoxon score function --  when one value is provided there is symmetric 
trimming of the score integral to the interval \code{(trim, 1-trim)}, when
there are two values provided, then the trimming restricts the integration
to \code{(trim[1], trim[2])}.}
}
\value{
The function returns two components. One is the ranks,  the
other is a scale factor which is the \eqn{L_2} norm of the score
function.  All score functions should be normalized to have mean zero.
}
\details{
  See GJKP(1993) for further details.
}
\references{
  Gutenbrunner, C., J. Jureckova,  Koenker, R. and  Portnoy,
  S. (1993)  Tests  of linear hypotheses  based on regression
  rank scores, \emph{Journal of  Nonparametric  Statistics},  (2), 307--331.

  Koenker, R. Rank Tests for Heterogeneous Treatment Effects with Covariates, preprint.
}
\seealso{
  \code{\link{rq}}, \code{\link{rq.test.rank}} \code{\link{anova}}
}
\examples{
data(stackloss)
ranks(rq(stack.loss ~ stack.x, tau=-1))
}
\keyword{regression}

\name{Munge}
\alias{Munge}
\title{
    Munge rqss formula
}
\description{
    function to recursively substitute arguments into rqss formula 
}
\usage{
Munge(formula, ...)
}
\arguments{
  \item{formula}{
      A rqss formula
}
  \item{\dots}{
      Arguments to be substituted into formula
}
}
\details{
    Intended (originally) for use with \code{demo(MCV)}.
    Based on an R-help suggestion of Gabor Grothendieck.
}
\value{
    A new formula after substitution
}
\seealso{
\code{demo(MCV)}
}
\examples{
lams <- c(1.3, 3.3)
f <- y ~ qss(x, lambda = lams[1]) + qss(z, lambda = lams[2]) + s
ff <- Munge(f, lams = lams)
}
\keyword{~manip}

\name{plot.rq}
\alias{plot.rq.process}
\title{ plot the coordinates of the quantile regression process}
\description{ Function to plot quantile regression  process.  }
\usage{
\method{plot}{rq.process}(x, nrow=3, ncol=2, ...) }
\arguments{
  \item{x}{ an object produced by rq() fitting  }
  \item{nrow}{ rows in mfrow }
  \item{ncol}{ columns in mfrow}
  \item{...}{ optional arguments to plot}
}
\author{ Roger Koenker rkoenker@uiuc.edu}
\seealso{ \code{\link{rq}}}
\keyword{hplot}

\name{latex.summary.rqs}
\alias{latex.summary.rqs}
\title{ Make a latex table from a table of rq results}
\description{
  Produces a file with latex commands for a table of rq results.
}
\usage{\method{latex}{summary.rqs}(x, transpose = FALSE, caption = "caption goes here.", 
		digits = 3, file = as.character(substitute(x)), ...)
}
\arguments{
  \item{x}{\code{x} is an object of class \code{summary.rqs}}
  \item{transpose}{if \code{TRUE} transpose table so that 
        rows are quantiles and columns are covariates. }
  \item{caption}{ caption for the table}
  \item{digits}{ decimal precision of table entries.}
  \item{file}{ name of file }
  \item{\dots}{ optional arguments for \code{latex.table}  }
}
\details{
  Calls  \code{latex.table}. 
}
\value{
  Returns invisibly after writing the file.
}
\author{ R. Koenker}
\seealso{ \code{\link{summary.rqs}}, \code{\link{latex.table}}}
\keyword{IO}

\name{rq.fit.pfn}
\alias{rq.fit.pfn}
\title{ Preprocessing Algorithm for Quantile Regression}
\description{
A preprocessing algorithm for the Frisch Newton algorithm 
for quantile regression. This is one possible method for rq().}
\usage{
rq.fit.pfn(x, y, tau=0.5, Mm.factor=0.8, max.bad.fixups=3, eps=1e-06)
}
\arguments{
  \item{x}{design matrix usually supplied via rq() }
  \item{y}{ response vector usually supplied via rq() }
  \item{tau}{ quantile of interest }
  \item{Mm.factor}{  constant to determine sub sample size m}
  \item{max.bad.fixups}{ number of allowed mispredicted signs of residuals }
  \item{eps}{ convergence tolerance }
}
\details{
Preprocessing algorithm to reduce the effective sample size for QR
problems with (plausibly) iid samples.  The preprocessing relies
on subsampling of the original data, so situations in which the
observations are not plausibly iid, are likely to cause problems.
The tolerance eps may be relaxed somewhat.
}
\value{
Returns an object of type rq
}
\references{ Portnoy and Koenker, Statistical Science, (1997) 279-300}
\author{ Roger Koenker <rkoenker@uiuc.edu>}
\seealso{ \code{\link{rq}}}

\keyword{ regression }

\name{summary.rqss}
\alias{summary.rqss}
\alias{print.summary.rqss}
\title{Summary of rqss fit}
\description{ Summary Method for a fitted rqss model.  }
\usage{\method{summary}{rqss}(object, cov = FALSE, ztol = 1e-5, ...) }
\arguments{
  \item{object}{an object returned from \code{rqss} fitting, describing
	an additive model estimating a conditional quantile function. 
	See \code{\link{qss}} for details on how to specify these terms.}
  \item{cov}{if TRUE return covariance matrix for the parametric components
	as \code{Vcov} and a list of covariance matrices for the nonparametric
	components as \code{Vqss}}
  \item{ztol}{Zero tolerance parameter used to determine the number of zero
	residuals indicating the estimated parametric dimension of the model,
	the so-called effective degrees of freedom.}
  \item{...}{additional arguments}
}
\details{ This function is intended to explore
inferential methods for rqss fitting.  The function is modeled after
\code{summary.gam} in Simon Wood's (2006)  \pkg{mgcv} package.  (Of course,
Simon should not be blamed for any deficiencies in the current implementation.
The basic idea is to condition on the lambda selection and construct
quasi-Bayesian credibility intervals based on normal approximation of
the "posterior," as computed using the Powell kernel estimate of the
usual quantile regression sandwich.  See \code{\link{summary.rq}} for
further details and references.
The function produces a conventional coefficient table with standard errors
t-statistics and p-values for the coefficients on the parametric part of the
model, and another table for additive nonparametric effects.  The latter
reports F statistics intended to evaluate the significance of these components
individually.  In addition the fidelity (value of the QR objective function
evaluated at the fitted model), the effective degrees of freedom, and the
sample size are reported.
}
\value{
    \item{coef}{Table of estimated coefficients and their standard errors,
	t-statistics, and p-values for the parametric components of the model}
    \item{qsstab}{Table of approximate F statistics, effective degrees of freedom
	and values of the penalty terms for each of the additive nonparametric 
	components of the model, and the lambda values assigned to each.} 
    \item{fidelity}{Value of the quantile regression objective function.}
    \item{tau}{Quantile of the estimated model}
    \item{formula}{formula of the estimated model}
    \item{edf}{Effective degrees of freedom of the fitted model, defined
	as the number of zero residuals of the fitted model,  see Koenker
	Mizera (2003) for details.}
    \item{n}{The sample size used to fit the model.}
    \item{Vcov}{Estimated covariance matrix of the fitted parametric component}
    \item{Vqss}{List of estimated covariance matrices of the fitted 
	nonparametric component}

}
\references{
  [1] Koenker, R., P. Ng and S. Portnoy, (1994)
  Quantile Smoothing Splines;
  \emph{Biometrika} \bold{81}, 673--680.

  [2] Koenker, R. and I. Mizera, (2003)
  Penalized Triograms: Total Variation Regularization for Bivariate Smoothing;
  \emph{JRSS(B)} \bold{66}, 145--163.

  [3] Wood, S. (2006) \emph{Generalized Additive Models}, Chapman-Hall.
}
\author{ Roger Koenker }
\seealso{ \code{\link{plot.rqss}} }
\examples{
n <- 200
x <- sort(rchisq(n,4))
z <- x + rnorm(n)
y <- log(x)+ .1*(log(x))^2 + log(x)*rnorm(n)/4 + z
f  <- rqss(y ~ qss(x) + z)
summary(f)
}
\keyword{regression}
\keyword{smooth}
\keyword{robust}

\name{sfn.control}
\alias{sfn.control}
\title{Set Control Parameters for Sparse Fitting}
\description{
  Auxiliary function for setting storage dimensions and other parameters
  for \code{\link{rq.fit.sfn}()}, \code{\link{rq.fit.sfnc}()} and \code{\link{rqss}()}.
}
\usage{
sfn.control(nsubmax = NULL, tmpmax = NULL, nnzlmax = NULL, cachsz = 64,
            small = 1e-06, maxiter = 100,
            tiny = 1e-30, Large = 1e128,
            warn.mesg = TRUE)
}
\arguments{
  \item{nsubmax}{upper bound for dimension of lindx}
  \item{tmpmax}{upper bound for dimension of tmpvec}
  \item{nnzlmax}{upper bound for non-zero entries of L stored in lnz, including diagonal}
  \item{cachsz}{size of cache in kbytes on target machine}
  \item{small}{convergence tolerance for interior point algorithm}
  \item{maxiter}{maximal number of interior point iterations}
  \item{tiny}{a tiny positive number; values below \code{tiny * max(diag)} are replaced by \code{Large};
    originally was \eqn{10^{-30}} hardcoded in Fortran code.}
  \item{Large}{a large number, practically \dQuote{Infinite} to replace
    \code{tiny} diagonal entries in Cholesky; was \eqn{10^{128}}, hardcoded in
    compiled code.}
  \item{warn.mesg}{logical flag controlling printing of warnings.}
}
\details{
Sparse fitting requires a number of temporary storage arrays whose size depends
on problem specific features in somewhat mysterious ways, parameters controlling
these sizes and some other fitting aspects can be controlled by specifying elements
of this control object.
}
\value{
  A \code{\link{list}} with components named as the arguments given above.
}

\author{
Roger Koenker
}

\seealso{
  \code{\link{rq.fit.sfn}},
  \code{\link{rq.fit.sfnc}},
  and \code{\link{rqss}} from which \code{sfn.control()} is called.
}
\keyword{ utilities }

\name{rq.fit.fnc}
\alias{rq.fit.fnc}
\title{
Quantile Regression Fitting via Interior Point Methods
}
\description{
This is a lower level routine called by \code{rq()} to compute quantile
regression methods using the Frisch-Newton algorithm.  It allows the
call to specify linear inequality constraints to which the fitted
coefficients will be subjected.  The constraints are assumed to be 
formulated as Rb >= r.
}
\usage{
rq.fit.fnc(x, y, R, r, tau=0.5, beta=0.9995, eps=1e-06)
}
\arguments{
\item{x}{
The design matrix
}
\item{y}{
The response vector
}
\item{R}{
The matrix describing the inequality constraints
}
\item{r}{
The right hand side vector of inequality constraints
}
\item{tau}{
The quantile of interest, must lie in (0,1)
}
\item{beta}{
technical step length parameter -- alter at your own risk!
}
\item{eps}{
  tolerance parameter for convergence.  In cases of multiple optimal solutions
  there may be some discrepancy between solutions produced by method
  \code{"fn"} and method \code{"br"}.  This is due to the fact that
  \code{"fn"} tends to converge to a point near the centroid of the
  solution set, while \code{"br"} stops at a vertex of the set.  
}
}
\value{
  returns an object of class \code{"rq"}, which can be passed to
  \code{\link{summary.rq}} to obtain standard errors, etc.  
}
\details{
  The details of the algorithm are explained in Koenker and Ng (2002).
  The basic idea can be traced back to the log-barrier methods proposed by
  Frisch in the 1950's for constrained optimization.  But the current
  implementation is based on proposals by Mehrotra and others in the
  recent (explosive) literature on interior point methods for solving linear 
  programming problems.  See \code{"rq"} helpfile for an example.
  It is an open research problem to provide an inference apparatus for
  inequality constrained quantile regression.
}
\references{
Koenker, R. and S. Portnoy (1997).
The Gaussian Hare and the Laplacian Tortoise:
Computability of squared-error vs. absolute-error estimators, with discussion,
\emph{Statistical Science}, \bold{12}, 279-300.

Koenker, R. and P. Ng(2005).
Inequality Constrained Quantile Regression, \emph{Sankya}, 418-440.
}
\seealso{
  \code{\link{rq}}, \code{\link{rq.fit.br}},
  \code{\link{rq.fit.pfn}}
}
\keyword{regression}

\name{rq.wfit}
\alias{rq.wfit}
\title{Function to choose method for Weighted Quantile Regression  }
\description{ Weight the data and then call the chosen fitting algorithm.  }
\usage{
rq.wfit(x, y, tau=0.5, weights, method="br", ...)
}
\arguments{ 
\item{x}{ 
  the design matrix 
} 
\item{y}{ 
  the response variable 
} 
\item{tau}{ 
  the quantile desired, if tau lies outside (0,1) the whole process 
  is estimated. 
} 
\item{weights}{ 
  weights used in the fitting
} 
\item{method}{ 
  method of computation:  "br" is Barrodale and Roberts exterior point 
  "fn" is the Frisch-Newton interior point method. 
} 
\item{...}{ 
  Optional arguments passed to fitting routine.
} 
}
 
\seealso{  \code{\link{rq}} \code{\link{rq.fit.br}} \code{\link{rq.fit.fnb}}}

\keyword{ regression }

\name{rq.fit.scad}
\alias{rq.fit.scad}
\title{
SCADPenalized Quantile Regression 
}
\description{
  The fitting method implements the smoothly clipped absolute deviation
  penalty of Fan and Li for fitting quantile regression models.  
  When  the argument \code{lambda}
  is a scalar the penalty function is the scad modified l1
  norm of the last (p-1) coefficients, under the presumption that the
  first coefficient is an intercept parameter that should not be subject
  to the penalty.  When \code{lambda} is a vector it should have length
  equal the column dimension of the matrix \code{x} and then defines a
  coordinatewise specific vector of scad penalty parameters.  In this
  case \code{lambda} entries of zero indicate covariates that are not
  penalized.  There should be a sparse version of this, but isn't (yet).
}
\usage{
rq.fit.scad(x, y, tau = 0.5, alpha = 3.2, lambda = 1, start="rq", 
	beta = .9995, eps = 1e-06)
}
\arguments{
\item{x}{
  the design matrix
}
\item{y}{
  the response variable
}
\item{tau}{
  the quantile desired, defaults to 0.5.
}
\item{alpha}{
  tuning parameter of the scad penalty.
}
\item{lambda}{
  the value of the penalty parameter that determines how much shrinkage is done.
  This should be either a scalar, or a vector of length equal to the column dimension
  of the \code{x} matrix.

}
\item{start}{
  starting method, default method 'rq' uses the unconstrained rq estimate, while
  method 'lasso' uses the corresponding lasso estimate with the specified lambda.
}
\item{beta}{
  step length parameter for Frisch-Newton method.
}
\item{eps}{
 tolerance parameter for convergence. 
}
}
\value{
  Returns a list with a coefficient, residual, tau and lambda components.  
  When called from \code{"rq"} as intended the returned object
  has class "scadrqs".  
}
\details{The algorithm is an adaptation of the "difference convex algorithm"
described in Wu and Liu (2008).  It solves a sequence of (convex) QR problems 
to approximate solutions of the (non-convex) scad problem.} 
\references{
Wu, Y. and Y. Liu (2008) Variable  Selection in Quantile Regression, \emph{Statistica
Sinica}, to appear.
}
\author{R. Koenker}
\seealso{
\code{\link{rq}}}
\examples{
n <- 60
p <- 7
rho <- .5
beta <- c(3,1.5,0,2,0,0,0)
R <- matrix(0,p,p)
for(i in 1:p){
        for(j in 1:p){
                R[i,j] <- rho^abs(i-j)
                }
        }
set.seed(1234)
x <- matrix(rnorm(n*p),n,p) \%*\% t(chol(R))
y <- x \%*\% beta + rnorm(n)

f <- rq(y ~ x, method="scad",lambda = 30)
g <- rq(y ~ x, method="scad", start = "lasso", lambda = 30)
}
\keyword{regression}

\name{akj}
\alias{akj}
\title{Density Estimation using Adaptive Kernel method}
\description{
  Univariate \emph{adaptive} kernel density estimation a la Silverman.
  As used by Portnoy and Koenker (1989).
}
\usage{
akj(x, z =, p =, h = -1, alpha = 0.5, kappa = 0.9, iker1 = 0)
}
\arguments{
  \item{x}{points used for centers of kernel assumed to be sorted.}
  \item{z}{points at which density is calculated; defaults to an
    equispaced sequence covering the range of x.}
  \item{p}{vector of probabilities associated with \code{x}s; defaults
    to 1/n for each x.}
  \item{h}{initial window size (overall); defaults to Silverman's normal
    reference.}
  \item{alpha}{a sensitivity parameter that determines the sensitivity of
    the local bandwidth to variations in the pilot density; defaults to .5.}
  \item{kappa}{constant multiplier for initial (default) window width}
  \item{iker1}{integer kernel indicator: 0 for normal kernel (default)
    while 1 for Cauchy kernel (\code{\link{dcauchy}}).}
}
\value{
  a \code{\link{list}} structure is with components
  \item{dens}{the vector of estimated density values \eqn{f(z)}}
  \item{psi}{a vector of \eqn{\psi=-f'/f} function values.}
  \item{score}{a vector of score \eqn{\psi' = (f'/f)^2 - f''/f} function
    values.}
  \item{h}{same as the input argument h}
}
\note{
  if the \code{score} function values are of interest, the Cauchy kernel
  may be preferable.
}
\references{
  Portnoy, S and R Koenker, (1989)
  Adaptive L Estimation of Linear Models;
  \emph{Annals of Statistics} \bold{17}, 362--81.

  Silverman, B. (1986)
  \emph{Density Estimation}, pp 100--104.
}
\examples{
 set.seed(1)
 x <- c(rnorm(600), 2 + 2*rnorm(400))
 xx <- seq(-5, 8, length=200)
 z <- akj(x, xx)
 plot(xx, z$dens, ylim=range(0,z$dens), type ="l", col=2)
 abline(h=0, col="gray", lty=3)
 plot(xx, z$psi, type ="l", col=2, main = expression(hat(psi(x))))
 plot(xx, z$score, type ="l", col=2,
      main = expression("score " * hat(psi) * "'" * (x)))

 if(require("nor1mix")) {
  m3 <- norMix(mu= c(-4, 0, 3), sig2 = c(1/3^2, 1, 2^2),
               w = c(.1,.5,.4))
  plot(m3, p.norm = FALSE)
  set.seed(11)
  x <- rnorMix(1000, m3)
  z2 <- akj(x, xx)
  lines(xx, z2$dens, col=2)
  z3 <- akj(x, xx, kappa = 0.5, alpha = 0.88)
  lines(xx, z3$dens, col=3)
 }
}
\keyword{smooth}

\name{rq.fit.lasso}
\alias{rq.fit.lasso}
\title{
Lasso Penalized Quantile Regression 
}
\description{
  The fitting method implements the lasso penalty of Tibshirani for
  fitting quantile regression models.  When  the argument \code{lambda}
  is a scalar the penalty function is the l1
  norm of the last (p-1) coefficients, under the presumption that the
  first coefficient is an intercept parameter that should not be subject
  to the penalty.  When \code{lambda} is a vector it should have length
  equal the column dimension of the matrix \code{x} and then defines a
  coordinatewise specific vector of lasso penalty parameters.  In this
  case \code{lambda} entries of zero indicate covariates that are not
  penalized.  There should be a sparse version of this, but isn't (yet).
}
\usage{
rq.fit.lasso(x, y, tau = 0.5, lambda = 1, beta = .99995, eps = 1e-06)
}
\arguments{
\item{x}{
  the design matrix
}
\item{y}{
  the response variable
}
\item{tau}{
  the quantile desired, defaults to 0.5.
}
\item{lambda}{
  the value of the penalty parameter(s) that determine how much shrinkage is done.
  This should be either a scalar, or a vector of length equal to the column dimension
  of the \code{x} matrix.
}
\item{beta}{
  step length parameter for Frisch-Newton method.
}
\item{eps}{
 tolerance parameter for convergence. 
}
}
\value{
  Returns a list with a coefficient, residual, tau and lambda components.  
  When called from \code{"rq"} (as intended) the returned object
  has class "lassorqs".  
}
\references{
Koenker, R. (2005 \emph{Quantile Regression}, CUP.
}
\author{R. Koenker}
\seealso{
\code{\link{rq}}}
\examples{
n <- 60
p <- 7
rho <- .5
beta <- c(3,1.5,0,2,0,0,0)
R <- matrix(0,p,p)
for(i in 1:p){
        for(j in 1:p){
                R[i,j] <- rho^abs(i-j)
                }
        }
set.seed(1234)
x <- matrix(rnorm(n*p),n,p) \%*\% t(chol(R))
y <- x \%*\% beta + rnorm(n)

f <- rq(y ~ x, method="lasso",lambda = 30)
g <- rq(y ~ x, method="lasso",lambda = c(rep(0,4),rep(30,4)))
}
\keyword{regression}

\name{rqss.object}
\alias{rqss.object}
\alias{logLik.rqss}
\alias{AIC.rqss}
\alias{fitted.rqss}
\alias{resid.rqss}
\alias{print.rqss}
\title{RQSS Objects and Summarization Thereof}
\description{
Functions to reveal the inner meaning of objects created by \code{rqss} fitting.
}
\usage{
\method{logLik}{rqss}(object, ...)
\method{AIC}{rqss}(object, ..., k=2)
\method{print}{rqss}(x, ...)
\method{resid}{rqss}(object, ...)
\method{fitted}{rqss}(object, ...)
}
\arguments{
  \item{object}{an object returned from \code{rqss} fitting, describing
	an additive model estimating a conditional quantile function. 
	See \code{\link{qss}} for details on how to specify these terms.}
  \item{x}{an rqss object, as above.}
  \item{k}{a constant factor governing the weight attached to the penalty
	term on effective degrees of freedom of the fit.  By default 
	k =2 corresponding to the Akaike version of the penalty, negative
	values indicate that the k should be set to log(n) as proposed
	by Schwarz (1978).} 
  \item{...}{additional arguments}
}
\details{ Total variation regularization for univariate and
    bivariate nonparametric quantile smoothing is described
    in Koenker, Ng and Portnoy (1994) and Koenker and Mizera(2003)
    respectively.  The additive model extension of this approach
    depends crucially on the sparse linear algebra implementation
    for R described in Koenker and Ng (2003).  Eventually, these
    functions should be expanded to provide an automated lambda
    selection procedure.}
\value{
    The function \code{summary.rqss} returns a list consisting of
    the following components:
    \item{fidelity}{Value of the quantile regression objective function.}
    \item{penalty}{A list consisting of the values of the total variation 
	smoothing penalty for each of additive components.}
    \item{edf}{Effective degrees of freedom of the fitted model, defined
	as the number of zero residuals of the fitted model,  Koenker
	Mizera (2003) for details.}
    \item{qssedfs}{A list of effective degrees of freedom for each of
	the additive components of the fitted model, defined as the
	number of non-zero elements of each penalty component of the
	residual vector.}
    \item{lamdas}{A list of the lambdas specified for each of the additive
	components of the model.}

}
\references{
  [1] Koenker, R. and S. Portnoy (1997)
  The Gaussian Hare and the Laplacean
  Tortoise:  Computability of Squared-error vs Absolute Error Estimators,
  (with discussion).
  \emph{Statistical Science} \bold{12}, 279--300.

  [2] Koenker, R., P. Ng and S. Portnoy, (1994)
  Quantile Smoothing Splines;
  \emph{Biometrika} \bold{81}, 673--680.

  [3] Koenker, R. and I. Mizera, (2003)
  Penalized Triograms: Total Variation Regularization for Bivariate Smoothing;
  \emph{JRSS(B)} \bold{66}, 145--163.

  [4] Koenker, R. and P. Ng (2003)
  SparseM:  A Sparse Linear Algebra Package for R,
  \emph{J. Stat. Software}.
}
\author{ Roger Koenker }
\seealso{ \code{\link{plot.rqss}}
}
\examples{
require(MatrixModels)
n <- 200
x <- sort(rchisq(n,4))
z <- x + rnorm(n)
y <- log(x)+ .1*(log(x))^2 + log(x)*rnorm(n)/4 + z
plot(x, y-z)
f.N  <- rqss(y ~ qss(x, constraint= "N") + z)
f.I  <- rqss(y ~ qss(x, constraint= "I") + z)
f.CI <- rqss(y ~ qss(x, constraint= "CI") + z)

lines(x[-1], f.N $coef[1] + f.N $coef[-(1:2)])
lines(x[-1], f.I $coef[1] + f.I $coef[-(1:2)], col="blue")
lines(x[-1], f.CI$coef[1] + f.CI$coef[-(1:2)], col="red")

## A bivariate example
data(CobarOre)
fCO <- rqss(z ~ qss(cbind(x,y), lambda= .08), data=CobarOre)
plot(fCO)
}
\keyword{regression}
\keyword{smooth}
\keyword{robust}

\name{rqProcess}
\alias{rqProcess}
\title{ Compute Standardized Quantile Regression Process }
\description{
Computes a standardize quantile regression process for the model
specified by the formula, on the partition of [0,1] specified by the
taus argument, and standardized according to the argument nullH. 
Intended for use in \code{\link{KhmaladzeTest}}.}
\usage{
rqProcess(formula, data, taus, nullH = "location", ...)
}
\arguments{
  \item{formula}{model formula  }
  \item{data}{data frame to be used to interpret formula }
  \item{taus}{ quantiles at which the process is to be evaluated, if any
	of the taus lie outside (0,1)  then the full process is computed 
	for all distinct solutions.}
  \item{nullH}{Null hypothesis to be used for standardization}
  \item{...}{optional arguments passed to \code{\link{summary.rq}}}
}
\details{
The process computes standardized estimates based on the 
hypothesis specified in the \code{nullH} argument.  
The Vhat component is rescaled by the Cholesky 
decomposition of the tau specific covariance matrix, the vhat component is
rescaled by the marginal standard errors.  The nature of the covariance
matrix used for the standardization is controlled arguments passed via
the \code{...} argument to \code{\link{summary.rq}}.  If the full
process is estimated then these covariance options aren't available
and only a simple iid-error form of the covariance matrix is used.
}
\value{
  \item{taus}{The points of evaluation of the process}
  \item{qtaus}{Values of xbar'betahat(taus)}
  \item{Vhat}{Joint parametric QR process}
  \item{vhat}{Marginal parametric QR processes}
}
\author{R. Koenker}
\seealso{\code{\link{KhmaladzeTest}}}
\keyword{regression}

\name{rqs.fit}
\alias{rqs.fit}
\title{Function to fit multiple response quantile regression models}
\usage{
rqs.fit(x, y, tau=0.5, tol = 0.0001)
}
\arguments{
\item{x}{
  the design matrix an n by p matrix.
}
\item{y}{
  the response variable as a n by m matrix
}
\item{tau}{
  the quantile desired, if tau lies outside (0,1) 
}
\item{tol}{
  tolerance parameter for Barrodale and Roberts exterior point method.
 }
}

\description{Function intended for multiple response quantile regression
called from \code{boot.rq} for wild bootstrap option.}
\seealso{  \code{\link{boot.rq}} }

\keyword{ regression }

\name{rearrange}
\alias{rearrange}
\title{Rearrangement}
\description{ Monotonize a step function by rearrangement }
\usage{ rearrange(f,xmin,xmax) }
\arguments{ 
   \item{f}{ object of class stepfun } 
   \item{xmin}{minimum of the support of the rearranged f} 
   \item{xmax}{maximum of the support of the rearranged f} 

}
\details{
   Given a stepfunction \eqn{Q(u)}, not necessarily monotone, let 
   \eqn{F(y) = \int \{ Q(u) \le y \} du} denote the associated cdf
   obtained by randomly evaluating \eqn{Q} at \eqn{U \sim U[0,1]}.  The
   rearranged version of \eqn{Q} is \eqn{\tilde Q (u) = \inf \{
   u: F(y) \ge u \}.  The rearranged function inherits the right
   or left continuity of original stepfunction.}
}
\value{ Produces transformed stepfunction that is monotonic increasing.  }
\references{ 
Chernozhukov, V., I. Fernandez-Val, and A. Galichon, (2006)  Quantile and Probability
Curves without Crossing, Econometrica, forthcoming.

Chernozhukov, V., I. Fernandez-Val, and A. Galichon, (2009)  Improving Estimates of
Monotone Functions by Rearrangement, Biometrika, 96, 559--575. 

Hardy, G.H., J.E. Littlewood, and G. Polya (1934)  Inequalities,  Cambridge U. Press.
}
\author{R. Koenker}
\seealso{ \code{\link{rq}} \code{\link{rearrange}}}
\examples{
data(engel)
z <- rq(foodexp ~ income, tau = -1,data =engel)
zp <- predict(z,newdata=list(income=quantile(engel$income,.03)),stepfun = TRUE)
plot(zp,do.points = FALSE, xlab = expression(tau),
        ylab = expression(Q ( tau )), main="Engel Food Expenditure Quantiles")
plot(rearrange(zp),do.points = FALSE, add=TRUE,col.h="red",col.v="red")
legend(.6,300,c("Before Rearrangement","After Rearrangement"),lty=1,col=c("black","red"))
}
\keyword{regression}

\name{rq.fit.conquer}
\alias{rq.fit.conquer}
\title{Optional Fitting Method for Quantile Regression}
\description{
    This fitting method provides a link to the gradient descent 
    for convolution smoothed quantile regression problem implemented
    in the \pkg{conquer} package of He et al (2020).}
\usage{
rq.fit.conquer (x, y, tau=0.5, kernel = c("Gaussian", "uniform",
    "parabolic", "triangular"), h = 0, tol = 1e-04,
    iteMax = 5000, ci = FALSE, alpha = 0.05, B = 200)
}
\arguments{
  \item{x}{design matrix usually supplied via rq(), expected to
 have a intercept as the first column }
  \item{y}{ response vector usually supplied via rq() }
  \item{tau}{ quantile of interest }
  \item{kernel}{A character string specifying the choice of
          kernel function. Default is "Gaussian". Other choices are
          "uniform", "parabolic" or "triangular".} 
  \item{h}{The bandwidth parameter for kernel smoothing of the QR
      objective function.  Default is max{((log(n) + p) / n)^0.4, 0.05}. 
      The default is used if the input value is less than 0.05.} 
  \item{tol}{Tolerance level of the gradient descent
          algorithm. The gradient descent algorithm terminates when the
          maximal entry of the gradient is less than "tol". Default is
          1e-05.}
  \item{iteMax}{Maximum number of iterations. Default is 5000.} 
  \item{ci}{A logical flag. Default is FALSE. If "ci =
          TRUE", then three types of confidence intervals (percentile,
          pivotal and normal) will be constructed via multiplier
          bootstrap.  This option is subsumed in normal use by the
	  \code{summary.rq} functionality.}
  \item{alpha}{Nominal level for confidence intervals, may be passed
	via the call to \code{summary}}
  \item{B}{Number of bootstrap replications.  May be passed via summary.}
}
\details{
    See documentation in the \pkg{conquer} package.
}
\value{
Returns an object of class "rq".
}
\references{ 
Xuming He and Xiaoou Pan and Kean Ming Tan and Wen-Xin Zhou, (2020)
conquer: Convolution-Type Smoothed Quantile Regression,
\url{https://CRAN.R-project.org/package=conquer}}

\seealso{\code{\link{rq}}}

\keyword{regression}

\name{FAQ}
\alias{FAQ}
\alias{ChangeLog}
\title{FAQ and ChangeLog  of a package}
\description{ Show the FAQ or ChangeLog of a specified package  }
\usage{
FAQ(pkg = "quantreg")
ChangeLog(pkg = "quantreg")
}
\arguments{
  \item{pkg}{ Package Name }
}
\details{
Assumes that the FAQ and/or ChangeLog files exist in the proper "inst" directory.
}
\value{
Has only the side effect of showing the files on the screen.  
}
\keyword{ documentation }

\name{rq.fit.qfnb}
\alias{rq.fit.qfnb}
\title{
Quantile Regression Fitting via Interior Point Methods
}
\description{
This is a lower level routine called by \code{rq()} to compute quantile
regression parameters using the Frisch-Newton algorithm.  In contrast to
method "fn" it computes solutions for all the specified taus inside a
fortran loop.  See \code{\link{rq.fit.pfnb}} for further details on a more
efficient preprocessing method.
}
\usage{
rq.fit.qfnb(x, y, tau)
}
\arguments{
\item{x}{
The design matrix
}
\item{y}{
The response vector
}
\item{tau}{
The quantiles of interest, must lie in (0,1), be sorted and preferably equally
spaced.
}
}
\value{
  returns a list with elements consisting of 
  \enumerate{
      \item{coefficients}{a matrix of dimension ncol(x) by length(taus)
      }
      \item{nit} {a 3-vector of iteration counts}
      \item{info} {a convergence flag}
  }
}
\details{
  The details of the Frisch-Newton algorithm are explained in Koenker and Portnoy (1997).
  The basic idea can be traced back to the log-barrier methods proposed by
  Frisch in the 1950's for linear programming.  But the current
  implementation is based on proposals by Mehrotra and others in the
  recent (explosive) literature on interior point methods for solving linear 
  programming problems.  This function replaces an earlier one \code{rq.fit.fn},
  which required the initial dual values to be feasible.  The current version allows the
  user to specify an infeasible starting point for the dual problem, that
  is one that may not satisfy the dual equality constraints.  It still 
  assumes that the starting value satisfies the upper and lower bounds.
}
\references{
Koenker, R. and S. Portnoy (1997).
The Gaussian Hare and the Laplacian Tortoise:
Computability of squared-error vs. absolute-error estimators, with discussion,
\emph{Statistical Science}, \bold{12}, 279-300.
}
\seealso{
  \code{\link{rq}}, \code{\link{rq.fit.br}},
  \code{\link{rq.fit.pfn}}
}
\keyword{regression}

\name{kuantile}
\alias{kuantile}
\alias{kselect}
\alias{kunique}
\title{Quicker Sample Quantiles }
\description{
The function 'kuantile' computes sample quantiles corresponding
to the specified probabilities. The intent is to mimic the generic
(base) function 'quantile' but using a variant of the Floyd and
Rivest (1975) algorithm which is somewhat quicker, especially for
large sample sizes.
}
\usage{
kuantile(x, probs = seq(0, 1, .25), na.rm = FALSE, names = TRUE, type = 7, ...) 
}
\arguments{
  \item{x}{numeric vector whose sample quantiles are wanted.}
  \item{probs}{numeric vector of probabilities with values in [0,1].}
  \item{type}{ an integer between 1 and 9 selecting one of the nine quantile
          algorithms detailed below to be used.}
  \item{na.rm}{logical: if true, any 'NA' and 'NaN''s are removed from 'x'
          before the quantiles are computed.}
  \item{names}{logical: if true, the result has a 'names' attribute. } 
  \item{...}{further arguments passed to or from other methods.}
}
\details{ A vector of length 'length(p)' is returned.  See the documentation
for 'quantile' for further details on the types.  The algorithm was written
by K.C. Kiwiel.  It is a modified version of the (algol 68) SELECT procedure of
Floyd and Rivest (1975), incorporating modifications of Brown(1976).
The algorithm has linear growth in the number of comparisons required as
sample size grows.  For the median, average case behavior requires
\eqn{1.5 n + O((n log n)^{1/2})} comparisons. See Kiwiel (2005) and Knuth (1998)
for further details.  When the number of required elements of p is large, it
may be preferable to revert to a full  sort.}
\value{
  A vector of quantiles of the same length as the vector p.
}
\references{ 
R.W. Floyd and R.L. Rivest: "Algorithm 489: The Algorithm
        SELECT---for Finding the $i$th Smallest of $n$ Elements",
        Comm. ACM 18, 3 (1975) 173,

T. Brown: "Remark on Algorithm 489", ACM Trans. Math.
        Software 3, 2 (1976), 301-304.

K.C. Kiwiel: On Floyd and Rivest's SELECT Algorithm, Theoretical
      Computer Sci. 347 (2005) 214-238.

D. Knuth, The Art of Computer Programming, Volume 3, Sorting and 
	Searching, 2nd Ed., (1998), Addison-Wesley.
}
\author{ K.C. Kiwiel, R interface:  Roger Koenker }
\seealso{\code{\link{quantile}}}
\examples{
     kuantile(x <- rnorm(1001))# Extremes & Quartiles by default

     ### Compare different types
     p <- c(0.1,0.5,1,2,5,10,50)/100
     res <- matrix(as.numeric(NA), 9, 7)
     for(type in 1:9) res[type, ] <- y <- kuantile(x,  p, type=type)
     dimnames(res) <- list(1:9, names(y))
     ktiles <- res

     ### Compare different types
     p <- c(0.1,0.5,1,2,5,10,50)/100
     res <- matrix(as.numeric(NA), 9, 7)
     for(type in 1:9) res[type, ] <- y <- quantile(x,  p, type=type)
     dimnames(res) <- list(1:9, names(y))
     qtiles <- res

     max(abs(ktiles - qtiles))


}
\keyword{univar}

\name{rq.fit.hogg}
\alias{rq.fit.hogg}
\title{weighted quantile regression fitting}
\description{
Function to estimate a regression mmodel by minimizing the weighted sum of several
quantile regression functions.  See Koenker(1984) for an asymptotic look at these
estimators.  This is a slightly generalized version of what Zou and Yuan (2008) call
composite quantile regression in that it permits weighting of the components of the
objective function and also allows further linear inequality constraints on the coefficients.
}
\usage{
rq.fit.hogg(x, y, taus = c(0.1, 0.3, 0.5), weights = c(0.7, 0.2, 0.1), 
	    R = NULL, r = NULL, beta = 0.99995, eps = 1e-06)
}
\arguments{
  \item{x}{design matrix}
  \item{y}{response vector }
  \item{taus}{quantiles getting positive weight}
  \item{weights}{weights assigned to the quantiles }
  \item{R}{optional matrix describing linear inequality constraints}
  \item{r}{optional vector describing linear inequality constraints}
  \item{beta}{step length parameter of the Frisch Newton Algorithm}
  \item{eps}{tolerance parameter for the Frisch Newton Algorithm}
}
\details{
Mimimizes a weighted sum of quantile regression objective functions using
the specified taus.  The model permits distinct intercept parameters at
each of the specified taus, but the slope parameters are constrained to
be the same for all taus.  This estimator was originally suggested to
the author by Bob Hogg in one of his famous blue notes of 1979.
The algorithm used to solve the resulting linear programming problems
is either the Frisch Newton algorithm described in Portnoy and Koenker (1997),
or the closely related algorithm described in Koenker and Ng(2002) that
handles linear inequality constraints.  See \code{\link{qrisk}} for illustration
of its use in portfolio allocation.

Linear inequality constraints of the form \eqn{Rb \geq r} can be imposed  with
the convention that \eqn{b} is a \eqn{m+p} where \eqn{m} is the \code{length(taus)}
and \eqn{p} is the column dimension of \code{x} without the intercept.
}
\value{
  \item{coefficients}{estimated coefficients of the model}
}
\references{
Zou, Hui and and Ming Yuan (2008)  Composite quantile regression and the
Oracle model selection theory, Annals of Statistics, 36, 1108--11120.

Koenker, R. (1984) A note on L-estimates for linear models, 
Stat. and Prob Letters, 2, 323-5.

Portnoy, S. and Koenker, R. (1997) The Gaussian Hare and the 
Laplacean Tortoise:  Computability of Squared-error vs Absolute Error Estimators, 
(with discussion).  Statistical Science, (1997) 12, 279-300.

Koenker, R. and Ng, P (2003) Inequality Constrained Quantile Regression, preprint.
 }
\author{ Roger Koenker }


\seealso{ \code{\link{qrisk}}}

\keyword{regression}
\keyword{ robust }

\name{combos}
\alias{combos}
\title{Ordered Combinations}
\description{
All m combinations of the first n integers taken p at a time
are computed and return as an p by m matrix.  The columns
of the matrix are ordered so that adjacent columns differ
by only one element.  This is just a reordered version of
\code{combn} in base R, but the ordering is useful for some
applications.
}
\usage{
combos(n,p)
}
\arguments{
  \item{n}{The n in n choose p}
  \item{p}{The p in n choose p}
}
\value{
  a \code{matrix} of dimension p by \code{choose(n,p)}
} 
\note{
Implementation based on a Pascal algorithm of Limin Xiang 
and Kazuo Ushijima (2001) translated to ratfor for R.
If you have \pkg{rgl} installed you might try \code{demo("combos")}
for a visual impression of how this works.
}
\references{
Limin Xiang and Kazuo Ushijima (2001) 
"On O(1) Time Algorithms for Combinatorial Generation," 
\emph{Computer Journal}, 44(4), 292-302. 
}
\examples{
H <- combos(20,3)
}
\keyword{utilities}

\name{Mammals}
\alias{Mammals}
\docType{data}
\title{Garland(1983) Data on Running Speed of Mammals}
\description{
Observations on the maximal running speed of mammal species
and their body mass.
}
\usage{data(Mammals)}
\format{
  A data frame with 107 observations on the following 4 variables.
  \describe{
    \item{weight}{Body mass in Kg for "typical adult sizes"}
    \item{speed}{Maximal running speed (fastest sprint velocity on record)}
    \item{hoppers}{logical variable indicating animals that ambulate
	by hopping, e.g. kangaroos}
    \item{specials}{logical variable indicating special animals with
	"lifestyles in which speed does not figure as an important
	factor":  Hippopotamus, raccoon (Procyon), badger (Meles),
	coati (Nasua), skunk (Mephitis), man (Homo), porcupine
	(Erithizon), oppossum (didelphis), and sloth (Bradypus)
	}
  }
}
\examples{
data(Mammals)
attach(Mammals)
x <- log(weight)
y <- log(speed)
plot(x,y, xlab="Weight in log(Kg)", ylab="Speed in log(Km/hour)",type="n")
points(x[hoppers],y[hoppers],pch = "h", col="red")
points(x[specials],y[specials],pch = "s", col="blue")
others <- (!hoppers & !specials)
points(x[others],y[others], col="black",cex = .75)
fit <- rqss(y ~ qss(x, lambda = 1),tau = .9)
plot(fit)
}

\details{
Used by Chappell (1989) and Koenker, Ng and Portnoy (1994) to
illustrate the fitting of piecewise linear curves.
}
\source{
Garland, T. (1983) The relation between maximal running speed and body 
mass in terrestrial mammals, \emph{J. Zoology}, 199, 1557-1570.
}
\references{
Koenker, R., P. Ng and S. Portnoy, (1994)  Quantile Smoothing Splines'' 
\emph{Biometrika}, 81, 673-680.

Chappell, R. (1989) Fitting Bent Lines to Data, with Applications ot
Allometry,  \emph{J. Theo. Biology}, 138, 235-256.

}
\seealso{\code{\link{rqss}}}
\keyword{datasets}
